@@ -38,18 +38,16 @@ public struct PathMapping {
   }
 }
 
-public enum SymbolProviderKind {
+public enum SymbolProviderKind: Sendable {
   case clang
   case swift
 
-  init?(_ cKind: indexstoredb_symbol_provider_kind_t) {
+  init(_ cKind: indexstoredb_symbol_provider_kind_t) {
     switch cKind {
     case INDEXSTOREDB_SYMBOL_PROVIDER_KIND_SWIFT:
       self = .swift
     case INDEXSTOREDB_SYMBOL_PROVIDER_KIND_CLANG:
       self = .clang
-    case INDEXSTOREDB_SYMBOL_PROVIDER_KIND_UNKNOWN:
-      return nil
     default:
       preconditionFailure("Unknown enum case in indexstoredb_symbol_provider_kind_t")
     }
@@ -174,9 +172,11 @@ public final class IndexStoreDB {
   /// Stop iteration if `body` returns `false`.
   /// - Returns: `false` if iteration was terminated by `body` returning `true` or `true` if iteration finished.
   @discardableResult
-  public func forEachSymbolOccurrence(byUSR usr: String, roles: SymbolRole, _ body: @escaping (SymbolOccurrence) -> Bool) -> Bool {
-    return indexstoredb_index_symbol_occurrences_by_usr(impl, usr, roles.rawValue) { occur in
-      return body(SymbolOccurrence(occur))
+  public func forEachSymbolOccurrence(byUSR usr: String, roles: SymbolRole, _ body: (SymbolOccurrence) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_symbol_occurrences_by_usr(impl, usr, roles.rawValue) { occur in
+        return body(SymbolOccurrence(occur))
+      }
     }
   }
 
@@ -191,9 +191,12 @@ public final class IndexStoreDB {
   }
 
   @discardableResult
-  public func forEachRelatedSymbolOccurrence(byUSR usr: String, roles: SymbolRole, _ body: @escaping (SymbolOccurrence) -> Bool) -> Bool {
-    return indexstoredb_index_related_symbol_occurrences_by_usr(impl, usr, roles.rawValue) { occur in
-      return body(SymbolOccurrence(occur))
+  public func forEachRelatedSymbolOccurrence(byUSR usr: String, roles: SymbolRole, _ body: (SymbolOccurrence) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_related_symbol_occurrences_by_usr(impl, usr, roles.rawValue) {
+        occur in
+        return body(SymbolOccurrence(occur))
+      }
     }
   }
 
@@ -206,9 +209,11 @@ public final class IndexStoreDB {
     return result
   }
 
-  @discardableResult public func forEachCanonicalSymbolOccurrence(byName: String, body: @escaping (SymbolOccurrence) -> Bool) -> Bool {
-    return indexstoredb_index_canonical_symbol_occurences_by_name(impl, byName) { occur in
-      return body(SymbolOccurrence(occur))
+  @discardableResult public func forEachCanonicalSymbolOccurrence(byName: String, body: (SymbolOccurrence) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_canonical_symbol_occurences_by_name(impl, byName) { occur in
+        return body(SymbolOccurrence(occur))
+      }
     }
   }
 
@@ -227,17 +232,19 @@ public final class IndexStoreDB {
     anchorEnd: Bool,
     subsequence: Bool,
     ignoreCase: Bool,
-    body: @escaping (SymbolOccurrence) -> Bool
+    body: (SymbolOccurrence) -> Bool
   ) -> Bool {
-    return indexstoredb_index_canonical_symbol_occurences_containing_pattern(
-      impl,
-      pattern,
-      anchorStart,
-      anchorEnd,
-      subsequence,
-      ignoreCase
-    ) { occur in
-      body(SymbolOccurrence(occur))
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_canonical_symbol_occurences_containing_pattern(
+        impl,
+        pattern,
+        anchorStart,
+        anchorEnd,
+        subsequence,
+        ignoreCase
+      ) { occur in
+        body(SymbolOccurrence(occur))
+      }
     }
   }
 
@@ -263,15 +270,19 @@ public final class IndexStoreDB {
   }
 
   @discardableResult
-  public func forEachMainFileContainingFile(path: String, crossLanguage: Bool, body: @escaping (String) -> Bool) -> Bool {
+  public func forEachMainFileContainingFile(
+    path: String, crossLanguage: Bool, body: (String) -> Bool
+  ) -> Bool {
     let fromSwift = path.hasSuffix(".swift")
-    return indexstoredb_index_units_containing_file(impl, path) { unit in
-      let mainFileStr = String(cString: indexstoredb_unit_info_main_file_path(unit))
-      let toSwift = mainFileStr.hasSuffix(".swift")
-      if !crossLanguage && fromSwift != toSwift {
-        return true // continue
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_units_containing_file(impl, path) { unit in
+        let mainFileStr = String(cString: indexstoredb_unit_info_main_file_path(unit))
+        let toSwift = mainFileStr.hasSuffix(".swift")
+        if !crossLanguage && fromSwift != toSwift {
+          return true  // continue
+        }
+        return body(mainFileStr)
       }
-      return body(mainFileStr)
     }
   }
 
@@ -285,10 +296,12 @@ public final class IndexStoreDB {
   }
 
   @discardableResult
-  public func forEachUnitNameContainingFile(path: String, body: @escaping (String) -> Bool) -> Bool {
-    return indexstoredb_index_units_containing_file(impl, path) { unit in
-      let unitName = String(cString: indexstoredb_unit_info_unit_name(unit))
-      return body(unitName)
+  public func forEachUnitNameContainingFile(path: String, body: (String) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_units_containing_file(impl, path) { unit in
+        let unitName = String(cString: indexstoredb_unit_info_unit_name(unit))
+        return body(unitName)
+      }
     }
   }
 
@@ -300,41 +313,17 @@ public final class IndexStoreDB {
     }
     return result
   }
-    
-  public func symbolProvider(for sourceFilePath: String) -> SymbolProviderKind? {
-    var result: SymbolProviderKind? = nil
-    indexstoredb_index_units_containing_file(impl, sourceFilePath) { unit in
-      let providerKind = SymbolProviderKind(indexstoredb_unit_info_symbol_provider_kind(unit))
-
-      let mainFilePath = String(cString: indexstoredb_unit_info_main_file_path(unit))
-      if providerKind == .swift && mainFilePath != sourceFilePath {
-        // We have a unit that is "included" from Swift. This happens for header
-        // files that Swift files depend on. But Swift doesn't have includes and
-        // we shouldn't infer the header file's language to Swift based on this unit.
-        // Ignore it.
-        return true
-      }
-
-      if result == nil {
-        result = providerKind
-      } else if result != providerKind {
-        // Found two conflicting provider kinds. Return nil as we don't know the provider in this case.
-        result = nil
-        return false
-      }
-      return true
-    }
-    return result
-  }
 
   @discardableResult
-  public func foreachFileIncludedByFile(path: String, body: @escaping (String) -> Bool) -> Bool {
-    return indexstoredb_index_files_included_by_file(impl, path) { targetPath, line in
+  public func foreachFileIncludedByFile(path: String, body: (String) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_files_included_by_file(impl, path) { targetPath, line in
         let targetPathStr = String(cString: targetPath)
         return body(targetPathStr)
+      }
     }
   }
-    
+
   public func filesIncludedByFile(path: String) -> [String] {
     var result: [String] = []
     foreachFileIncludedByFile(path: path) { targetPath in
@@ -343,15 +332,17 @@ public final class IndexStoreDB {
     }
     return result
   }
-    
+
   @discardableResult
-  public func foreachFileIncludingFile(path: String, body: @escaping (String) -> Bool) -> Bool {
-    return indexstoredb_index_files_including_file(impl, path) { sourcePath, line in
+  public func foreachFileIncludingFile(path: String, body: (String) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_files_including_file(impl, path) { sourcePath, line in
         let sourcePathStr = String(cString: sourcePath)
         return body(sourcePathStr)
+      }
     }
   }
-      
+
   public func filesIncludingFile(path: String) -> [String] {
     var result: [String] = []
     foreachFileIncludingFile(path: path) { targetPath in
@@ -379,11 +370,14 @@ public final class IndexStoreDB {
 
   /// Iterates over recorded `#include`s of a unit.
   @discardableResult
-  public func forEachIncludeOfUnit(unitName: String, body: @escaping (UnitIncludeEntry) -> Bool) -> Bool {
-    return indexstoredb_index_includes_of_unit(impl, unitName) { sourcePath, targetPath, line in
-      let sourcePathStr = String(cString: sourcePath)
-      let targetPathStr = String(cString: targetPath)
-      return body(UnitIncludeEntry(sourcePath: sourcePathStr, targetPath: targetPathStr, line: line))
+  public func forEachIncludeOfUnit(unitName: String, body: (UnitIncludeEntry) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_includes_of_unit(impl, unitName) { sourcePath, targetPath, line in
+        let sourcePathStr = String(cString: sourcePath)
+        let targetPathStr = String(cString: targetPath)
+        return body(
+          UnitIncludeEntry(sourcePath: sourcePathStr, targetPath: targetPathStr, line: line))
+      }
     }
   }
 
@@ -402,9 +396,11 @@ public final class IndexStoreDB {
   /// - Parameter body: A closure to be called for each symbol. The closure should return true to
   /// continue iterating.
   @discardableResult
-  public func forEachSymbolName(body: @escaping (String) -> Bool) -> Bool {
-    return indexstoredb_index_symbol_names(impl) { name in
-      body(String(cString: name))
+  public func forEachSymbolName(body: (String) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_symbol_names(impl) { name in
+        body(String(cString: name))
+      }
     }
   }
 
@@ -427,10 +423,30 @@ public final class IndexStoreDB {
     return result
   }
 
+  public func symbolOccurrences(inFilePath path: String) -> [SymbolOccurrence] {
+    var result: [SymbolOccurrence] = []
+    forEachSymbolOccurrence(inFilePath: path) { occur in
+      result.append(occur)
+      return true
+    }
+    return result
+  }
+
   @discardableResult
-  func forEachSymbol(inFilePath filePath: String, body: @escaping (Symbol) -> Bool) -> Bool {
-    return indexstoredb_index_symbols_contained_in_file_path(impl, filePath) { symbol in
-      return body(Symbol(symbol))
+  func forEachSymbol(inFilePath filePath: String, body: (Symbol) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_symbols_contained_in_file_path(impl, filePath) { symbol in
+        return body(Symbol(symbol))
+      }
+    }
+  }
+
+  @discardableResult
+  func forEachSymbolOccurrence(inFilePath filePath: String, body: (SymbolOccurrence) -> Bool) -> Bool {
+    return withoutActuallyEscaping(body) { body in
+      return indexstoredb_index_symbol_occurrences_in_file_path(impl, filePath) { occur in
+        return body(SymbolOccurrence(occur))
+      }
     }
   }
 
