@@ -41,19 +41,21 @@ final class IndexTests: XCTestCase {
 
     try ws.buildAndIndex()
 
-    let csym = Symbol(usr: usr, name: "c()", kind: .function)
-    let asym = Symbol(usr: "s:4main1ayyF", name: "a()", kind: .function)
+    let csym = Symbol(usr: usr, name: "c()", kind: .function, language: .swift)
+    let asym = Symbol(usr: "s:4main1ayyF", name: "a()", kind: .function, language: .swift)
 
     let ccanon = SymbolOccurrence(
       symbol: csym,
       location: SymbolLocation(ws.testLoc("c"), moduleName: "main"),
       roles: [.definition, .canonical],
+      symbolProvider: .clang,
       relations: [])
 
     let ccall = SymbolOccurrence(
       symbol: csym,
       location: SymbolLocation(ws.testLoc("c:call")),
       roles: [.reference, .call, .calledBy, .containedBy],
+      symbolProvider: .clang,
       relations: [
         .init(symbol: asym, roles: [.calledBy, .containedBy])
     ])
@@ -92,9 +94,10 @@ final class IndexTests: XCTestCase {
     [
       ccall,
       SymbolOccurrence(
-        symbol: Symbol(usr: "s:4main1byyF", name: "b()", kind: .function),
+        symbol: Symbol(usr: "s:4main1byyF", name: "b()", kind: .function, language: .swift),
         location: SymbolLocation(ws.testLoc("b:call")),
         roles: [.reference, .call, .calledBy, .containedBy],
+        symbolProvider: .swift,
         relations: [
           .init(symbol: asym, roles: [.calledBy, .containedBy])
       ])
@@ -108,38 +111,38 @@ final class IndexTests: XCTestCase {
 
   #if os(macOS)
 
-    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class)
+    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class, language: .objc)
     let cdeclOccs = index.occurrences(ofUSR: cdecl.usr, roles: .all)
     checkOccurrences(cdeclOccs, expected: [
-      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical]),
-      cdecl.at(ws.testLoc("C:def"), roles: .definition),
-      cdecl.at(ws.testLoc("C:ref:swift"), roles: .reference),
-      cdecl.at(ws.testLoc("C:ref:e.mm"), roles: .reference),
+      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical], symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:def"), roles: .definition, symbolProvider: .clang),
+      cdecl.with(language: .swift).at(ws.testLoc("C:ref:swift"), roles: .reference, symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:ref:e.mm"), roles: .reference, symbolProvider: .clang),
     ])
 
-    let cmethod = Symbol(usr: "c:objc(cs)C(im)method", name: "method", kind: .instanceMethod)
+    let cmethod = Symbol(usr: "c:objc(cs)C(im)method", name: "method", kind: .instanceMethod, language: .objc)
     let cmethodOccs = index.occurrences(ofUSR: cmethod.usr, roles: .all)
     checkOccurrences(cmethodOccs, expected: [
-      cmethod.with(name: "method()").at(ws.testLoc("C.method:call:swift"), roles: [.call, .dynamic]),
-      cmethod.at(ws.testLoc("C.method:decl"), roles: .declaration),
-      cmethod.at(ws.testLoc("C.method:def"), roles: .definition),
-      cmethod.at(ws.testLoc("C.method:call:e.mm"), roles: [.call, .dynamic]),
+      cmethod.with(name: "method()", language: .swift).at(ws.testLoc("C.method:call:swift"), roles: [.call, .dynamic], symbolProvider: .swift),
+      cmethod.at(ws.testLoc("C.method:decl"), roles: .declaration, symbolProvider: .clang),
+      cmethod.at(ws.testLoc("C.method:def"), roles: .definition, symbolProvider: .clang),
+      cmethod.at(ws.testLoc("C.method:call:e.mm"), roles: [.call, .dynamic], symbolProvider: .clang),
     ])
   #endif
 
-    let ddecl = Symbol(usr: "c:@S@D", name: "D", kind: .class)
+    let ddecl = Symbol(usr: "c:@S@D", name: "D", kind: .class, language: .cxx)
     let dOccs = index.occurrences(ofUSR: ddecl.usr, roles: .all)
     checkOccurrences(dOccs, expected: [
-      ddecl.at(ws.testLoc("D:def"), roles: .definition),
-      ddecl.at(ws.testLoc("D:ref"), roles: .reference),
-      ddecl.at(ws.testLoc("D:ref:e.mm"), roles: .reference),
+      ddecl.at(ws.testLoc("D:def"), roles: .definition, symbolProvider: .clang),
+      ddecl.at(ws.testLoc("D:ref"), roles: .reference, symbolProvider: .clang),
+      ddecl.at(ws.testLoc("D:ref:e.mm"), roles: .reference, symbolProvider: .clang),
     ])
 
-    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function)
+    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function, language: .c)
     let bridgingHeaderOccs = index.occurrences(ofUSR: bhdecl.usr, roles: .all)
     checkOccurrences(bridgingHeaderOccs, expected: [
-      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration),
-      bhdecl.with(name: "bridgingHeader()").at(ws.testLoc("bridgingHeader:call"), roles: .call),
+      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration, symbolProvider: .clang),
+      bhdecl.with(name: "bridgingHeader()", language: .swift).at(ws.testLoc("bridgingHeader:call"), roles: .call, symbolProvider: .clang),
     ])
   }
 
@@ -154,28 +157,28 @@ final class IndexTests: XCTestCase {
     let index = ws.index
 
   #if os(macOS)
-    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class)
+    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class, language: .objc)
     let cdeclOccs = index.occurrences(ofUSR: cdecl.usr, roles: .all)
     checkOccurrences(cdeclOccs, expected: [
-      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical]),
-      cdecl.at(ws.testLoc("C:def"), roles: .definition),
-      cdecl.at(ws.testLoc("C:ref:swift"), roles: .reference),
+      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical], symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:def"), roles: .definition, symbolProvider: .clang),
+      cdecl.with(language: .swift).at(ws.testLoc("C:ref:swift"), roles: .reference, symbolProvider: .swift),
     ])
 
-    let cmethod = Symbol(usr: "c:objc(cs)C(im)method", name: "method", kind: .instanceMethod)
+    let cmethod = Symbol(usr: "c:objc(cs)C(im)method", name: "method", kind: .instanceMethod, language: .objc)
     let cmethodOccs = index.occurrences(ofUSR: cmethod.usr, roles: .all)
     checkOccurrences(cmethodOccs, expected: [
-      cmethod.with(name: "method()").at(ws.testLoc("C.method:call:swift"), roles: [.call, .dynamic]),
-      cmethod.at(ws.testLoc("C.method:decl"), roles: .declaration),
-      cmethod.at(ws.testLoc("C.method:def"), roles: .definition),
+      cmethod.with(name: "method()", language: .swift).at(ws.testLoc("C.method:call:swift"), roles: [.call, .dynamic], symbolProvider: .clang),
+      cmethod.at(ws.testLoc("C.method:decl"), roles: .declaration, symbolProvider: .clang),
+      cmethod.at(ws.testLoc("C.method:def"), roles: .definition, symbolProvider: .clang),
     ])
   #endif
 
-    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function)
+    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function, language: .c)
     let bridgingHeaderOccs = index.occurrences(ofUSR: bhdecl.usr, roles: .all)
     checkOccurrences(bridgingHeaderOccs, expected: [
-      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration),
-      bhdecl.with(name: "bridgingHeader()").at(ws.testLoc("bridgingHeader:call"), roles: .call),
+      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration, symbolProvider: .clang),
+      bhdecl.with(name: "bridgingHeader()", language: .swift).at(ws.testLoc("bridgingHeader:call"), roles: .call, symbolProvider: .swift),
     ])
   }
 
@@ -191,7 +194,7 @@ final class IndexTests: XCTestCase {
     let index = ws.index
 
   #if os(macOS)
-    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class)
+    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class, language: .objc)
     let getOccs = { index.occurrences(ofUSR: cdecl.usr, roles: .all) }
 
     // Output units are not set yet.
@@ -205,24 +208,24 @@ final class IndexTests: XCTestCase {
     index.addUnitOutFilePaths(indexOutputPaths, waitForProcessing: true)
 
     // The bridging header is referenced as a PCH unit dependency, make sure we can see the data.
-    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function)
+    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function, language: .c)
     let bridgingHeaderOccs = index.occurrences(ofUSR: bhdecl.usr, roles: .all)
     checkOccurrences(bridgingHeaderOccs, expected: [
-      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration),
-      bhdecl.with(name: "bridgingHeader()").at(ws.testLoc("bridgingHeader:call"), roles: .call),
+      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration, symbolProvider: .clang),
+      bhdecl.with(name: "bridgingHeader()", language: .swift).at(ws.testLoc("bridgingHeader:call"), roles: .call, symbolProvider: .swift),
     ])
   #if os(macOS)
     checkOccurrences(getOccs(), expected: [
-      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical]),
-      cdecl.at(ws.testLoc("C:def"), roles: .definition),
-      cdecl.at(ws.testLoc("C:ref:swift"), roles: .reference),
+      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical], symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:def"), roles: .definition, symbolProvider: .clang),
+      cdecl.with(language: .swift).at(ws.testLoc("C:ref:swift"), roles: .reference, symbolProvider: .swift),
     ])
 
     let outUnitASwift = try XCTUnwrap(indexOutputPaths.first{ $0.hasSuffix("-a.swift.o") })
     index.removeUnitOutFilePaths([outUnitASwift], waitForProcessing: true)
     checkOccurrences(getOccs(), expected: [
-      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical]),
-      cdecl.at(ws.testLoc("C:def"), roles: .definition),
+      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical], symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:def"), roles: .definition, symbolProvider: .clang),
     ])
   #endif
   }
@@ -239,7 +242,7 @@ final class IndexTests: XCTestCase {
     let index = ws.index
 
   #if os(macOS)
-    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class)
+    let cdecl = Symbol(usr: "c:objc(cs)C", name: "C", kind: .class, language: .objc)
     let getOccs = { index.occurrences(ofUSR: cdecl.usr, roles: .all) }
 
     // Output units are not set yet.
@@ -253,24 +256,24 @@ final class IndexTests: XCTestCase {
     index.addUnitOutFilePaths(indexOutputPaths, waitForProcessing: true)
 
     // The bridging header is referenced as a PCH unit dependency, make sure we can see the data.
-    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function)
+    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function, language: .c)
     let bridgingHeaderOccs = index.occurrences(ofUSR: bhdecl.usr, roles: .all)
     checkOccurrences(bridgingHeaderOccs, expected: [
-      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration),
-      bhdecl.with(name: "bridgingHeader()").at(ws.testLoc("bridgingHeader:call"), roles: .call),
+      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration, symbolProvider: .clang),
+      bhdecl.with(name: "bridgingHeader()", language: .swift).at(ws.testLoc("bridgingHeader:call"), roles: .call, symbolProvider: .clang),
     ])
   #if os(macOS)
     checkOccurrences(getOccs(), expected: [
-      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical]),
-      cdecl.at(ws.testLoc("C:def"), roles: .definition),
-      cdecl.at(ws.testLoc("C:ref:swift"), roles: .reference),
+      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical], symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:def"), roles: .definition, symbolProvider: .clang),
+      cdecl.with(language: .swift).at(ws.testLoc("C:ref:swift"), roles: .reference, symbolProvider: .swift),
     ])
 
     let outUnitASwift = try XCTUnwrap(indexOutputPaths.first{ $0.hasSuffix("-a.swift.o") })
     index.removeUnitOutFilePaths([outUnitASwift], waitForProcessing: true)
     checkOccurrences(getOccs(), expected: [
-      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical]),
-      cdecl.at(ws.testLoc("C:def"), roles: .definition),
+      cdecl.at(ws.testLoc("C:decl"), roles: [.declaration, .canonical], symbolProvider: .clang),
+      cdecl.at(ws.testLoc("C:def"), roles: .definition, symbolProvider: .clang),
     ])
   #endif
   }
@@ -279,11 +282,11 @@ final class IndexTests: XCTestCase {
     guard let ws = try staticTibsTestWorkspace(name: "SwiftModules") else { return }
     try ws.buildAndIndex()
 
-    let aaa = Symbol(usr: "s:1A3aaayyF", name: "aaa()", kind: .function)
+    let aaa = Symbol(usr: "s:1A3aaayyF", name: "aaa()", kind: .function, language: .swift)
     checkOccurrences(ws.index.occurrences(ofUSR: aaa.usr, roles: .all), expected: [
-      aaa.at(ws.testLoc("aaa:def"), moduleName: "A", roles: .definition),
-      aaa.at(ws.testLoc("aaa:call"), moduleName: "B", roles: .call),
-      aaa.at(ws.testLoc("aaa:call:c"), moduleName: "C", roles: .call),
+      aaa.at(ws.testLoc("aaa:def"), moduleName: "A", roles: .definition, symbolProvider: .swift),
+      aaa.at(ws.testLoc("aaa:call"), moduleName: "B", roles: .call, symbolProvider: .swift),
+      aaa.at(ws.testLoc("aaa:call:c"), moduleName: "C", roles: .call, symbolProvider: .swift),
     ])
   }
 
@@ -291,12 +294,12 @@ final class IndexTests: XCTestCase {
     guard let ws = try mutableTibsTestWorkspace(name: "proj1") else { return }
     try ws.buildAndIndex()
 
-    let cdecl = Symbol(usr: "s:4main1cyyF", name: "c()", kind: .function)
+    let cdecl = Symbol(usr: "s:4main1cyyF", name: "c()", kind: .function, language: .swift)
     let roles: SymbolRole = [.reference, .definition, .declaration]
 
     checkOccurrences(ws.index.occurrences(ofUSR: cdecl.usr, roles: .all), expected: [
-      cdecl.at(ws.testLoc("c"), roles: .definition),
-      cdecl.at(ws.testLoc("c:call"), roles: .call),
+      cdecl.at(ws.testLoc("c"), roles: .definition, symbolProvider: .clang),
+      cdecl.at(ws.testLoc("c:call"), roles: .call, symbolProvider: .clang),
     ])
 
     try ws.edit(rebuild: true) { editor, files in
@@ -312,9 +315,9 @@ final class IndexTests: XCTestCase {
     }
 
     checkOccurrences(ws.index.occurrences(ofUSR: cdecl.usr, roles: .all), expected: [
-      cdecl.at(ws.testLoc("c"), roles: .definition),
-      cdecl.at(ws.testLoc("c:call"), roles: .call),
-      cdecl.at(ws.testLoc("c:anotherOne"), roles: .call),
+      cdecl.at(ws.testLoc("c"), roles: .definition, symbolProvider: .clang),
+      cdecl.at(ws.testLoc("c:call"), roles: .call, symbolProvider: .clang),
+      cdecl.at(ws.testLoc("c:anotherOne"), roles: .call, symbolProvider: .clang),
     ])
 
     XCTAssertNotEqual(ws.testLoc("c").url, ws.testLoc("a:def").url)
@@ -331,9 +334,9 @@ final class IndexTests: XCTestCase {
 
     let newDecl = cdecl.with(usr: "s:4main1cSiyF")
     checkOccurrences(ws.index.occurrences(ofUSR: newDecl.usr, roles: .all), expected: [
-      newDecl.at(ws.testLoc("c"), roles: .definition),
-      newDecl.at(ws.testLoc("c:call"), roles: .call),
-      newDecl.at(ws.testLoc("c:anotherOne"), roles: .call),
+      newDecl.at(ws.testLoc("c"), roles: .definition, symbolProvider: .clang),
+      newDecl.at(ws.testLoc("c:call"), roles: .call, symbolProvider: .clang),
+      newDecl.at(ws.testLoc("c:anotherOne"), roles: .call, symbolProvider: .clang),
     ])
   }
 
@@ -348,18 +351,18 @@ final class IndexTests: XCTestCase {
       waitUntilDoneInitializing: true,
       listenToUnitEvents: true)
 
-    let csym = Symbol(usr: "s:4main1cyyF", name: "c()", kind: .function)
+    let csym = Symbol(usr: "s:4main1cyyF", name: "c()", kind: .function, language: .swift)
     let waitOccs = indexWait.occurrences(ofUSR: csym.usr, roles: [.reference, .definition])
 
     checkOccurrences(waitOccs, expected: [
       SymbolOccurrence(
        symbol: csym,
         location: SymbolLocation(ws.testLoc("c")),
-       roles: [.definition, .canonical]),
+       roles: [.definition, .canonical], symbolProvider: .clang),
       SymbolOccurrence(
         symbol: csym,
         location: SymbolLocation(ws.testLoc("c:call")),
-        roles: [.reference, .call, .calledBy, .containedBy]),
+        roles: [.reference, .call, .calledBy, .containedBy], symbolProvider: .clang),
     ])
   }
 
@@ -514,20 +517,20 @@ final class IndexTests: XCTestCase {
       IndexStoreDB.UnitIncludeEntry(sourcePath: main1, targetPath: uniq1, line: ws.testLoc("include_main1_uniq1").line),
     ])
   }
-    
+
   func testFilesIncludes() throws {
     guard let ws = try staticTibsTestWorkspace(name: "MainFiles") else { return }
     try ws.buildAndIndex()
     let index = ws.index
-        
+
     let main1 = ws.testLoc("main1").url.path
     let main2 = ws.testLoc("main2").url.path
     let uniq1 = ws.testLoc("uniq1").url.path
     let shared = ws.testLoc("shared").url.path
-        
+
     let includedFiles = index.filesIncludedByFile(path: main1).sorted()
     XCTAssertEqual(includedFiles, [shared, uniq1])
-    
+
     let includingFiles = index.filesIncludingFile(path: shared).sorted()
     XCTAssertEqual(includingFiles, [main1, main2])
   }
@@ -547,7 +550,7 @@ final class IndexTests: XCTestCase {
     try ws.buildAndIndex()
     let index = ws.index
 
-    let ddecl = Symbol(usr: "c:@S@D", name: "D", kind: .class)
+    let ddecl = Symbol(usr: "c:@S@D", name: "D", kind: .class, language: .cxx)
     let getOccs = { index.occurrences(ofUSR: ddecl.usr, roles: .all) }
 
     // Output units are not set yet.
@@ -556,24 +559,24 @@ final class IndexTests: XCTestCase {
     let indexOutputPaths = ws.builder.indexOutputPaths.map{$0.path}
     index.addUnitOutFilePaths(indexOutputPaths, waitForProcessing: true)
     checkOccurrences(getOccs(), expected: [
-      ddecl.at(ws.testLoc("D:def"), roles: .definition),
-      ddecl.at(ws.testLoc("D:ref"), roles: .reference),
-      ddecl.at(ws.testLoc("D:ref:e.mm"), roles: .reference),
+      ddecl.at(ws.testLoc("D:def"), roles: .definition, symbolProvider: .clang),
+      ddecl.at(ws.testLoc("D:ref"), roles: .reference, symbolProvider: .clang),
+      ddecl.at(ws.testLoc("D:ref:e.mm"), roles: .reference, symbolProvider: .clang),
     ])
 
     let outUnitEMM = try XCTUnwrap(indexOutputPaths.first{ $0.hasSuffix("-e.mm.o") })
     index.removeUnitOutFilePaths([outUnitEMM], waitForProcessing: true)
     checkOccurrences(getOccs(), expected: [
-      ddecl.at(ws.testLoc("D:def"), roles: .definition),
-      ddecl.at(ws.testLoc("D:ref"), roles: .reference),
+      ddecl.at(ws.testLoc("D:def"), roles: .definition, symbolProvider: .clang),
+      ddecl.at(ws.testLoc("D:ref"), roles: .reference, symbolProvider: .clang),
     ])
 
     // The bridging header is referenced as a PCH unit dependency, make sure we can see the data.
-    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function)
+    let bhdecl = Symbol(usr: "c:@F@bridgingHeader", name: "bridgingHeader", kind: .function, language: .c)
     let bridgingHeaderOccs = index.occurrences(ofUSR: bhdecl.usr, roles: .all)
     checkOccurrences(bridgingHeaderOccs, expected: [
-      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration),
-      bhdecl.with(name: "bridgingHeader()").at(ws.testLoc("bridgingHeader:call"), roles: .call),
+      bhdecl.at(ws.testLoc("bridgingHeader:decl"), roles: .declaration, symbolProvider: .clang),
+      bhdecl.with(name: "bridgingHeader()", language: .swift).at(ws.testLoc("bridgingHeader:call"), roles: .call, symbolProvider: .swift),
     ])
   }
 
@@ -634,28 +637,28 @@ final class IndexTests: XCTestCase {
 
     try ws.buildAndIndex()
 
-    let asyncFuncSym = Symbol(usr: "s:4main9asyncFuncyyYaF", name: "asyncFunc()", kind: .function, properties: .swiftAsync)
+    let asyncFuncSym = Symbol(usr: "s:4main9asyncFuncyyYaF", name: "asyncFunc()", kind: .function, properties: .swiftAsync, language: .swift)
     let asyncFuncOccs = index.occurrences(ofUSR: asyncFuncSym.usr, roles: .definition)
     checkOccurrences(asyncFuncOccs, expected: [
-      asyncFuncSym.at(ws.testLoc("asyncFunc:def"), roles: .definition)
+      asyncFuncSym.at(ws.testLoc("asyncFunc:def"), roles: .definition, symbolProvider: .swift)
     ])
 
-    let asyncMethSym = Symbol(usr: "s:4main8MyStructV11asyncMethodyyYaF", name: "asyncMethod()", kind: .instanceMethod, properties: .swiftAsync)
+    let asyncMethSym = Symbol(usr: "s:4main8MyStructV11asyncMethodyyYaF", name: "asyncMethod()", kind: .instanceMethod, properties: .swiftAsync, language: .swift)
     let asyncMethOccs = index.occurrences(ofUSR: asyncMethSym.usr, roles: .definition)
     checkOccurrences(asyncMethOccs, expected: [
-      asyncMethSym.at(ws.testLoc("asyncMethod:def"), roles: .definition)
+      asyncMethSym.at(ws.testLoc("asyncMethod:def"), roles: .definition, symbolProvider: .swift)
     ])
 
-    let testMeSym = Symbol(usr: "s:4main10MyTestCaseC6testMeyyF", name: "testMe()", kind: .instanceMethod, properties: .unitTest)
+    let testMeSym = Symbol(usr: "s:4main10MyTestCaseC6testMeyyF", name: "testMe()", kind: .instanceMethod, properties: .unitTest, language: .swift)
     let testMeOccs = index.occurrences(ofUSR: testMeSym.usr, roles: .definition)
     checkOccurrences(testMeOccs, expected: [
-      testMeSym.at(ws.testLoc("testMe:def"), roles: .definition)
+      testMeSym.at(ws.testLoc("testMe:def"), roles: .definition, symbolProvider: .swift)
     ])
 
-    let testMeAsyncSym = Symbol(usr: "s:4main10MyTestCaseC11testMeAsyncyyYaF", name: "testMeAsync()", kind: .instanceMethod, properties: [.unitTest, .swiftAsync])
+    let testMeAsyncSym = Symbol(usr: "s:4main10MyTestCaseC11testMeAsyncyyYaF", name: "testMeAsync()", kind: .instanceMethod, properties: [.unitTest, .swiftAsync], language: .swift)
     let testMeAsyncOccs = index.occurrences(ofUSR: testMeAsyncSym.usr, roles: .definition)
     checkOccurrences(testMeAsyncOccs, expected: [
-      testMeAsyncSym.at(ws.testLoc("testMeAsync:def"), roles: .definition)
+      testMeAsyncSym.at(ws.testLoc("testMeAsync:def"), roles: .definition, symbolProvider: .swift)
     ])
   }
 
@@ -665,11 +668,11 @@ final class IndexTests: XCTestCase {
 
     try ws.buildAndIndex()
 
-    let largeType = Symbol(usr: "c:@CT@LargeType", name: "LargeType", kind: .concept)
+    let largeType = Symbol(usr: "c:@CT@LargeType", name: "LargeType", kind: .concept, language: .c)
     let largeTypeOccs = ws.index.occurrences(ofUSR: largeType.usr, roles: .all)
     checkOccurrences(largeTypeOccs, expected: [
-      largeType.at(ws.testLoc("LargeType:def"), roles: .definition),
-      largeType.at(ws.testLoc("LargeType:ref"), roles: .reference),
+      largeType.at(ws.testLoc("LargeType:def"), roles: .definition, symbolProvider: .swift),
+      largeType.at(ws.testLoc("LargeType:ref"), roles: .reference, symbolProvider: .swift),
     ])
   }
 }
