@@ -41,6 +41,8 @@ def check_call(cmd: List[str], additional_env: Dict[str, str] = {}, verbose: boo
     if verbose:
         print_cmd(cmd=cmd, additional_env=additional_env)
 
+    sys.stdout.flush()
+    sys.stderr.flush()
     subprocess.check_call(cmd, env=env_with_additional_env(additional_env), stderr=subprocess.STDOUT)
 
 
@@ -51,6 +53,8 @@ def check_output(cmd: List[str], additional_env: Dict[str, str] = {}, capture_st
         stderr = subprocess.STDOUT
     else:
         stderr = subprocess.DEVNULL
+    sys.stdout.flush()
+    sys.stderr.flush()
     return subprocess.check_output(cmd, env=env_with_additional_env(additional_env), stderr=stderr, encoding='utf-8')
 
 # -----------------------------------------------------------------------------
@@ -150,8 +154,14 @@ def run_tests(swift_exec: str, args: argparse.Namespace) -> None:
     print('Cleaning ' + tests)
     shutil.rmtree(tests, ignore_errors=True)
 
-    cmd = [swift_exec, 'test', '--parallel', '--test-product', 'IndexStoreDBPackageTests'] + swiftpm_args
-    check_call(cmd, additional_env=additional_env, verbose=args.verbose)
+    cmd = [swift_exec, 'test', '--test-product', 'IndexStoreDBPackageTests'] + swiftpm_args
+    try:
+        check_call(cmd + ['--parallel'], additional_env=additional_env, verbose=args.verbose)
+    except:
+        print('--- Running tests in parallel failed. Re-running tests serially to capture more actionable output.')
+        check_call(cmd, additional_env=additional_env, verbose=args.verbose)
+        # Return with non-zero exit code even if serial test execution succeeds.
+        raise SystemExit(1)
 
 
 def handle_invocation(swift_exec: str, args: argparse.Namespace) -> None:
@@ -230,7 +240,13 @@ def main() -> None:
         handle_invocation(swift_exec, args)
 
         # Linux ubsan disabled: https://bugs.swift.org/browse/SR-12550
-        if platform.system() != 'Linux':
+        #
+        # ubsan disabled on macOS because building indexstore-db using a new, just-built clang with ubsan enabled but
+        # with an Xcode 15.1 SDK causes C++ exception to not be catchable with `try {} catch {}` blocks and
+        # indexstore-db relies on exception catching, possibly related to
+        # https://stackoverflow.com/questions/51673343/clang-with-libc-exceptions.
+        # Re-enable when Xcode on CI nodes has been updated to Xcode 16 (rdar://139647872)
+        if platform.system() != 'Linux' and False:
             print('=== %s indexstore-db with ubsan ===' % args.action)
             args.sanitize = ['undefined']
             args.build_path = os.path.join(base, 'test-ubsan')
