@@ -59,7 +59,7 @@ typedef void *indexstoredb_symbol_location_t;
 typedef void *indexstoredb_symbol_relation_t;
 typedef void *indexstoredb_unit_info_t;
 
-typedef enum {
+typedef enum : uint64_t {
   INDEXSTOREDB_SYMBOL_ROLE_DECLARATION = 1 << 0,
   INDEXSTOREDB_SYMBOL_ROLE_DEFINITION  = 1 << 1,
   INDEXSTOREDB_SYMBOL_ROLE_REFERENCE   = 1 << 2,
@@ -82,7 +82,7 @@ typedef enum {
   INDEXSTOREDB_SYMBOL_ROLE_REL_IBTYPEOF    = 1 << 17,
   INDEXSTOREDB_SYMBOL_ROLE_REL_SPECIALIZATIONOF = 1 << 18,
 
-  INDEXSTOREDB_SYMBOL_ROLE_CANONICAL = 1 << 63,
+  INDEXSTOREDB_SYMBOL_ROLE_CANONICAL = (uint64_t)1 << 63,
 } indexstoredb_symbol_role_t;
 
 typedef enum {
@@ -328,6 +328,15 @@ indexstoredb_index_symbols_contained_in_file_path(_Nonnull indexstoredb_index_t
                                                   const char *_Nonnull path,
                                                   _Nonnull indexstoredb_symbol_receiver_t);
 
+/// Iterates over all the symbol occurrences contained in the source file at \p path
+///
+/// The occurrence passed to the receiver is only valid for the duration of the
+/// receiver call.
+INDEXSTOREDB_PUBLIC bool
+indexstoredb_index_symbol_occurrences_in_file_path(_Nonnull indexstoredb_index_t index,
+                                                   const char *_Nonnull path,
+                                                   _Nonnull indexstoredb_symbol_occurrence_receiver_t);
+
 /// Returns the USR of the given symbol.
 ///
 /// The string has the same lifetime as the \c indexstoredb_symbol_t.
@@ -498,9 +507,6 @@ indexstoredb_unit_info_main_file_path(_Nonnull indexstoredb_unit_info_t);
 INDEXSTOREDB_PUBLIC const char *_Nonnull
 indexstoredb_unit_info_unit_name(_Nonnull indexstoredb_unit_info_t);
 
-INDEXSTOREDB_PUBLIC indexstoredb_symbol_provider_kind_t
-indexstoredb_unit_info_symbol_provider_kind(_Nonnull indexstoredb_unit_info_t info);
-
 /// Iterates over the compilation units that contain \p path and return their units.
 ///
 /// This can be used to find information for units that include a given header.
