@@ -314,6 +314,16 @@ indexstoredb_index_symbols_contained_in_file_path(_Nonnull indexstoredb_index_t
   });
 }
 
+bool
+indexstoredb_index_symbol_occurrences_in_file_path(_Nonnull indexstoredb_index_t index,
+                                                    const char *_Nonnull path,
+                                                    _Nonnull indexstoredb_symbol_occurrence_receiver_t receiver) {
+  auto obj = (Object<std::shared_ptr<IndexSystem>> *)index;
+  return obj->value->foreachSymbolOccurrenceInFilePath(path, [&](SymbolOccurrenceRef Occur) -> bool {
+    return receiver((indexstoredb_symbol_occurrence_t)Occur.get());
+  });
+}
+
 const char *
 indexstoredb_symbol_usr(indexstoredb_symbol_t symbol) {
   auto value = (Symbol *)symbol;
@@ -589,20 +599,6 @@ indexstoredb_unit_info_unit_name(indexstoredb_unit_info_t info) {
   return obj->UnitName.c_str();
 }
 
-indexstoredb_symbol_provider_kind_t
-indexstoredb_unit_info_symbol_provider_kind(_Nonnull indexstoredb_unit_info_t info) {
-  auto obj = (const StoreUnitInfo *)info;
-  if (!obj->SymProviderKind) {
-    return INDEXSTOREDB_SYMBOL_PROVIDER_KIND_UNKNOWN;
-  }
-  switch (*obj->SymProviderKind) {
-  case IndexStoreDB::SymbolProviderKind::Clang:
-    return INDEXSTOREDB_SYMBOL_PROVIDER_KIND_CLANG;
-  case IndexStoreDB::SymbolProviderKind::Swift:
-    return INDEXSTOREDB_SYMBOL_PROVIDER_KIND_SWIFT;
-  }
-}
-
 bool
 indexstoredb_index_units_containing_file(
   indexstoredb_index_t index,
