@@ -474,7 +474,6 @@ void StoreUnitRepo::registerUnit(StringRef unitName, bool isInitialScan, std::sh
   bool needDatabaseUpdate;
   Optional<bool> optIsSystem;
   Optional<bool> PrevHasTestSymbols;
-  Optional<SymbolProviderKind> PrevSymProviderKind;
   IDCode PrevMainFileCode;
   IDCode PrevOutFileCode;
   Optional<StoreUnitInfo> StoreUnitInfoOpt;
@@ -494,7 +493,6 @@ void StoreUnitRepo::registerUnit(StringRef unitName, bool isInitialScan, std::sh
       PrevMainFileCode = unitImport.getPrevMainFileCode();
       PrevOutFileCode = unitImport.getPrevOutFileCode();
       PrevHasTestSymbols = unitImport.getHasTestSymbols();
-      PrevSymProviderKind = unitImport.getSymbolProviderKind();
       return false;
     }
 
@@ -592,14 +590,7 @@ void StoreUnitRepo::registerUnit(StringRef unitName, bool isInitialScan, std::sh
     }
 
     unitImport.commit();
-    StoreUnitInfoOpt = StoreUnitInfo{
-      unitName,
-      CanonMainFile,
-      OutFileIdentifier,
-      unitImport.getHasTestSymbols().getValue(),
-      unitModTime,
-      unitImport.getSymbolProviderKind()
-    };
+    StoreUnitInfoOpt = StoreUnitInfo{unitName, CanonMainFile, OutFileIdentifier, unitImport.getHasTestSymbols().getValue(), unitModTime};
     import.commit();
     return false;
   };
@@ -612,14 +603,7 @@ void StoreUnitRepo::registerUnit(StringRef unitName, bool isInitialScan, std::sh
       ReadTransaction reader(SymIndex->getDBase());
       CanonicalFilePath mainFile = reader.getFullFilePathFromCode(PrevMainFileCode);
       std::string outFileIdentifier = reader.getUnitFileIdentifierFromCode(PrevOutFileCode);
-      StoreUnitInfoOpt = StoreUnitInfo{
-        unitName,
-        mainFile,
-        outFileIdentifier,
-        PrevHasTestSymbols.getValue(),
-        unitModTime,
-        PrevSymProviderKind.getValue()
-      };
+      StoreUnitInfoOpt = StoreUnitInfo{unitName, mainFile, outFileIdentifier, PrevHasTestSymbols.getValue(), unitModTime};
     }
     Delegate->processedStoreUnit(StoreUnitInfoOpt.getValue());
   }
@@ -826,7 +810,6 @@ void StoreUnitRepo::onUnitOutOfDate(IDCode unitCode, StringRef unitName,
   CanonicalFilePath MainFilePath;
   std::string OutFileIdentifier;
   bool hasTestSymbols = false;
-  Optional<SymbolProviderKind> SymProviderKind;
   llvm::sys::TimePoint<> CurrModTime;
   SmallVector<IDCode, 8> dependentUnits;
   {
@@ -838,7 +821,6 @@ void StoreUnitRepo::onUnitOutOfDate(IDCode unitCode, StringRef unitName,
       }
       OutFileIdentifier = reader.getUnitFileIdentifierFromCode(unitInfo.OutFileCode);
       hasTestSymbols = unitInfo.HasTestSymbols;
-      SymProviderKind = unitInfo.SymProviderKind;
       CurrModTime = unitInfo.ModTime;
     }
     reader.getDirectDependentUnits(unitCode, dependentUnits);
@@ -850,8 +832,7 @@ void StoreUnitRepo::onUnitOutOfDate(IDCode unitCode, StringRef unitName,
       MainFilePath,
       OutFileIdentifier,
       hasTestSymbols,
-      CurrModTime,
-      SymProviderKind
+      CurrModTime
     };
     Delegate->unitIsOutOfDate(unitInfo, trigger, synchronous);
   }
