@@ -76,6 +76,9 @@ public:
   bool foreachSymbolInFilePath(CanonicalFilePathRef filePath,
                                function_ref<bool(SymbolRef Symbol)> Receiver);
 
+  bool foreachSymbolOccurrenceInFilePath(CanonicalFilePathRef filePath,
+                                         function_ref<bool(SymbolOccurrenceRef Occur)> Receiver);
+
   bool foreachSymbolName(function_ref<bool(StringRef name)> receiver);
 
   bool foreachCanonicalSymbolOccurrenceByUSR(StringRef USR,
@@ -100,7 +103,7 @@ public:
   bool foreachUnitTestSymbol(function_ref<bool(SymbolOccurrenceRef Occur)> receiver);
 
   /// Returns the latest modification date of a unit that contains the given source file.
-  /// 
+  ///
   /// If no unit containing the given source file exists, returns `None`.
   llvm::Optional<sys::TimePoint<>> timestampOfLatestUnitForFile(CanonicalFilePathRef filePath);
 
@@ -149,7 +152,7 @@ void SymbolIndexImpl::importSymbols(ImportTransaction &import, SymbolDataProvide
     std::uninitialized_copy(Name.begin(), Name.end(), copiedStr);
     StringRef copiedName = StringRef(copiedStr, Name.size());
     // FIXME: Make this part of the compiler indexing output. E.g. a C++-like 'struct' should be a 'class' kind.
-    if (Info.isClassLike())
+    if (Info.Kind == SymbolKind::Struct && Info.Lang == SymbolLanguage::CXX)
       Info.Kind = SymbolKind::Class;
     auto pair = CoreSymbols.insert(std::make_pair(USR, CoreSymbolData{copiedName, Info, Roles, RelatedRoles}));
     bool wasInserted = pair.second;
@@ -481,7 +484,7 @@ bool SymbolIndexImpl::foreachSymbolInFilePath(CanonicalFilePathRef filePath,
                                                                   SymbolInfo info,
                                                                   SymbolRoleSet roles,
                                                                   SymbolRoleSet relatedRoles) -> bool {
-                      
+
                       if (roles.containsAny(SymbolRoleSet(SymbolRole::Definition) | SymbolRole::Declaration)) {
                         return Receiver(std::make_shared<Symbol>(info, name, usr));
                       } else {
@@ -500,6 +503,33 @@ bool SymbolIndexImpl::foreachSymbolInFilePath(CanonicalFilePathRef filePath,
     return didFinish;
 }
 
+bool SymbolIndexImpl::foreachSymbolOccurrenceInFilePath(CanonicalFilePathRef filePath,
+                                                        function_ref<bool(SymbolOccurrenceRef Occur)> Receiver) {
+  bool didFinish = true;
+  ReadTransaction reader(DBase);
+
+  IDCode filePathCode = reader.getFilePathCode(filePath);
+  reader.foreachUnitContainingFile(filePathCode, [&](ArrayRef<IDCode> idCodes) -> bool {
+    for (IDCode idCode : idCodes) {
+      UnitInfo unitInfo = reader.getUnitInfo(idCode);
+
+      for (UnitInfo::Provider provider : unitInfo.ProviderDepends) {
+        IDCode providerCode = provider.ProviderCode;
+        if (provider.FileCode == filePathCode) {
+          auto record = createVisibleProviderForCode(providerCode, reader);
+          didFinish = record->foreachSymbolOccurrence(Receiver);
+
+          return false;
+        }
+      }
+    }
+
+    return true;
+  });
+
+  return didFinish;
+}
+
 bool SymbolIndexImpl::foreachCanonicalSymbolOccurrenceByKind(SymbolKind symKind, bool workspaceOnly,
                                                              function_ref<bool(SymbolOccurrenceRef Occur)> Receiver) {
   return foreachCanonicalSymbolOccurrenceImpl(workspaceOnly,
@@ -706,6 +736,11 @@ bool SymbolIndex::foreachSymbolInFilePath(CanonicalFilePathRef filePath,
   return IMPL->foreachSymbolInFilePath(filePath, std::move(Receiver));
 }
 
+bool SymbolIndex::foreachSymbolOccurrenceInFilePath(CanonicalFilePathRef filePath,
+                                                    function_ref<bool(SymbolOccurrenceRef Occur)> Receiver) {
+  return IMPL->foreachSymbolOccurrenceInFilePath(filePath, std::move(Receiver));
+}
+
 bool SymbolIndex::foreachUnitTestSymbolReferencedByOutputPaths(ArrayRef<CanonicalFilePathRef> FilePaths,
     function_ref<bool(SymbolOccurrenceRef Occur)> Receiver) {
   return IMPL->foreachUnitTestSymbolReferencedByOutputPaths(FilePaths, std::move(Receiver));
