@@ -9,11 +9,12 @@
 #include "src/sched/sched_getcpucount.h"
 
 #include "src/__support/common.h"
+#include "src/__support/macros/config.h"
 
 #include <sched.h>
 #include <stddef.h>
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE_DECL {
 
 LLVM_LIBC_FUNCTION(int, __sched_getcpucount,
                    (size_t cpuset_size, const cpu_set_t *mask)) {
@@ -24,4 +25,4 @@ LLVM_LIBC_FUNCTION(int, __sched_getcpucount,
   return result;
 }
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE_DECL
