@@ -10,21 +10,22 @@
 
 #include "src/__support/OSUtil/syscall.h" // For internal syscall function.
 #include "src/__support/common.h"
+#include "src/__support/macros/config.h"
 #include "src/errno/libc_errno.h"
 
 #include <sys/syscall.h> // For syscall numbers.
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE_DECL {
 
 LLVM_LIBC_FUNCTION(int, sched_setscheduler,
                    (pid_t tid, int policy, const struct sched_param *param)) {
-  long ret =
-      __llvm_libc::syscall_impl(SYS_sched_setscheduler, tid, policy, param);
+  int ret = LIBC_NAMESPACE::syscall_impl<int>(SYS_sched_setscheduler, tid,
+                                              policy, param);
   if (ret < 0) {
     libc_errno = -ret;
     return -1;
   }
-  return 0;
+  return ret;
 }
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE_DECL
