@@ -9,13 +9,14 @@
 #include "posix_spawn_file_actions_addopen.h"
 
 #include "file_actions.h"
+#include "src/__support/CPP/new.h"
 #include "src/__support/common.h"
+#include "src/__support/macros/config.h"
 
 #include <errno.h>
 #include <spawn.h>
-#include <stdlib.h> // For malloc
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE_DECL {
 
 LLVM_LIBC_FUNCTION(int, posix_spawn_file_actions_addopen,
                    (posix_spawn_file_actions_t *__restrict actions, int fd,
@@ -25,19 +26,13 @@ LLVM_LIBC_FUNCTION(int, posix_spawn_file_actions_addopen,
   if (fd < 0)
     return EBADF;
 
-  auto *act = reinterpret_cast<SpawnFileOpenAction *>(
-      malloc(sizeof(SpawnFileOpenAction)));
+  AllocChecker ac;
+  auto *act = new (ac) SpawnFileOpenAction(path, fd, oflag, mode);
   if (act == nullptr)
     return ENOMEM;
+  BaseSpawnFileAction::add_action(actions, act);
 
-  act->type = BaseSpawnFileAction::OPEN;
-  act->fd = fd;
-  act->path = path;
-  act->oflag = oflag;
-  act->mode = mode;
-  act->next = nullptr;
-  enque_spawn_action(actions, act);
   return 0;
 }
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE_DECL
