@@ -22,18 +22,23 @@
 // defines convert_int
 #include "src/stdio/printf_core/int_converter.h"
 
-#ifndef LLVM_LIBC_PRINTF_DISABLE_FLOAT
+#ifndef LIBC_COPT_PRINTF_DISABLE_FLOAT
 // defines convert_float_decimal
-#include "src/stdio/printf_core/float_dec_converter.h"
 // defines convert_float_dec_exp
+// defines convert_float_dec_auto
+#include "src/stdio/printf_core/float_dec_converter.h"
 // defines convert_float_hex_exp
 #include "src/stdio/printf_core/float_hex_converter.h"
-// defines convert_float_mixed
-#endif // LLVM_LIBC_PRINTF_DISABLE_FLOAT
+#endif // LIBC_COPT_PRINTF_DISABLE_FLOAT
 
-#ifndef LLVM_LIBC_PRINTF_DISABLE_WRITE_INT
+#ifdef LIBC_INTERNAL_PRINTF_HAS_FIXED_POINT
+// defines convert_fixed
+#include "src/stdio/printf_core/fixed_converter.h"
+#endif // LIBC_INTERNAL_PRINTF_HAS_FIXED_POINT
+
+#ifndef LIBC_COPT_PRINTF_DISABLE_WRITE_INT
 #include "src/stdio/printf_core/write_int_converter.h"
-#endif // LLVM_LIBC_PRINTF_DISABLE_WRITE_INT
+#endif // LIBC_COPT_PRINTF_DISABLE_WRITE_INT
 
 // defines convert_pointer
 #include "src/stdio/printf_core/ptr_converter.h"
