@@ -8,17 +8,18 @@
 
 #include "src/string/memmem.h"
 #include "src/__support/common.h"
+#include "src/__support/macros/config.h"
 #include "src/string/memory_utils/inline_memmem.h"
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE_DECL {
 
 LLVM_LIBC_FUNCTION(void *, memmem,
                    (const void *haystack, size_t haystack_len,
                     const void *needle, size_t needle_len)) {
-  constexpr auto comp = [](unsigned char l, unsigned char r) -> int {
+  constexpr auto COMP = [](unsigned char l, unsigned char r) -> int {
     return l - r;
   };
-  return inline_memmem(haystack, haystack_len, needle, needle_len, comp);
+  return inline_memmem(haystack, haystack_len, needle, needle_len, COMP);
 }
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE_DECL
