@@ -11,20 +11,24 @@
 #include "src/__support/OSUtil/syscall.h" // For internal syscall function.
 #include "src/__support/common.h"
 
+#include "src/__support/macros/config.h"
 #include "src/errno/libc_errno.h"
 
 #include <linux/net.h>   // For SYS_SOCKET socketcall number.
 #include <sys/syscall.h> // For syscall numbers.
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE_DECL {
 
 LLVM_LIBC_FUNCTION(int, socket, (int domain, int type, int protocol)) {
 #ifdef SYS_socket
-  long ret = __llvm_libc::syscall_impl(SYS_socket, domain, type, protocol);
+  int ret =
+      LIBC_NAMESPACE::syscall_impl<int>(SYS_socket, domain, type, protocol);
 #elif defined(SYS_socketcall)
-  unsigned long sockcall_args[3] = {domain, type, protocol};
-  long ret =
-      __llvm_libc::syscall_impl(SYS_socketcall, SYS_SOCKET, sockcall_args);
+  unsigned long sockcall_args[3] = {static_cast<unsigned long>(domain),
+                                    static_cast<unsigned long>(type),
+                                    static_cast<unsigned long>(protocol)};
+  int ret = LIBC_NAMESPACE::syscall_impl<int>(SYS_socketcall, SYS_SOCKET,
+                                              sockcall_args);
 #else
 #error "socket and socketcall syscalls unavailable for this platform."
 #endif
@@ -35,4 +39,4 @@ LLVM_LIBC_FUNCTION(int, socket, (int domain, int type, int protocol)) {
   return ret;
 }
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE_DECL
