@@ -9,12 +9,13 @@
 #ifndef LLVM_LIBC_SRC_TIME_DIFFTIME_H
 #define LLVM_LIBC_SRC_TIME_DIFFTIME_H
 
+#include "src/__support/macros/config.h"
 #include <time.h>
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE_DECL {
 
 double difftime(time_t end, time_t beginning);
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE_DECL
 
 #endif // LLVM_LIBC_SRC_TIME_DIFFTIME_H
