@@ -13,9 +13,9 @@
 TEST(LlvmLibcStrlcatTest, TooBig) {
   const char *str = "cd";
   char buf[4]{"ab"};
-  EXPECT_EQ(__llvm_libc::strlcat(buf, str, 3), size_t(4));
+  EXPECT_EQ(LIBC_NAMESPACE::strlcat(buf, str, 3), size_t(4));
   EXPECT_STREQ(buf, "ab");
-  EXPECT_EQ(__llvm_libc::strlcat(buf, str, 4), size_t(4));
+  EXPECT_EQ(LIBC_NAMESPACE::strlcat(buf, str, 4), size_t(4));
   EXPECT_STREQ(buf, "abc");
 }
 
@@ -23,15 +23,15 @@ TEST(LlvmLibcStrlcatTest, Smaller) {
   const char *str = "cd";
   char buf[7]{"ab"};
 
-  EXPECT_EQ(__llvm_libc::strlcat(buf, str, 7), size_t(4));
+  EXPECT_EQ(LIBC_NAMESPACE::strlcat(buf, str, 7), size_t(4));
   EXPECT_STREQ(buf, "abcd");
 }
 
 TEST(LlvmLibcStrlcatTest, No0) {
   const char *str = "cd";
   char buf[7]{"ab"};
-  EXPECT_EQ(__llvm_libc::strlcat(buf, str, 1), size_t(3));
+  EXPECT_EQ(LIBC_NAMESPACE::strlcat(buf, str, 1), size_t(3));
   EXPECT_STREQ(buf, "ab");
-  EXPECT_EQ(__llvm_libc::strlcat(buf, str, 2), size_t(4));
+  EXPECT_EQ(LIBC_NAMESPACE::strlcat(buf, str, 2), size_t(4));
   EXPECT_STREQ(buf, "ab");
 }
