@@ -5,6 +5,7 @@ import lldb.formatters.Logger
 # implementation for your platform before relying on these formatters to do the right
 # thing for your setup
 
+
 def ForwardListSummaryProvider(valobj, dict):
     list_capping_size = valobj.GetTarget().GetMaximumNumberOfChildrenToDisplay()
     text = "size=" + str(valobj.GetNumChildren())
@@ -13,6 +14,7 @@ def ForwardListSummaryProvider(valobj, dict):
     else:
         return text
 
+
 def StdOptionalSummaryProvider(valobj, dict):
     has_value = valobj.GetNumChildren() > 0
     # We add wrapping spaces for consistency with the libcxx formatter
@@ -25,14 +27,16 @@ class StdOptionalSynthProvider:
 
     def update(self):
         try:
-            self.payload = self.valobj.GetChildMemberWithName('_M_payload')
-            self.value = self.payload.GetChildMemberWithName('_M_payload')
-            self.has_value = self.payload.GetChildMemberWithName('_M_engaged').GetValueAsUnsigned(0) != 0
+            self.payload = self.valobj.GetChildMemberWithName("_M_payload")
+            self.value = self.payload.GetChildMemberWithName("_M_payload")
+            self.has_value = (
+                self.payload.GetChildMemberWithName("_M_engaged").GetValueAsUnsigned(0)
+                != 0
+            )
         except:
             self.has_value = False
         return False
 
-
     def num_children(self):
         return 1 if self.has_value else 0
 
@@ -41,15 +45,18 @@ class StdOptionalSynthProvider:
 
     def get_child_at_index(self, index):
         # some versions of libstdcpp have an additional _M_value child with the actual value
-        possible_value = self.value.GetChildMemberWithName('_M_value')
+        possible_value = self.value.GetChildMemberWithName("_M_value")
         if possible_value.IsValid():
-            return possible_value.Clone('Value')
-        return self.value.Clone('Value')
+            return possible_value.Clone("Value")
+        return self.value.Clone("Value")
+
 
 """
  This formatter can be applied to all
  unordered map-like structures (unordered_map, unordered_multimap, unordered_set, unordered_multiset)
 """
+
+
 class StdUnorderedMapSynthProvider:
     def __init__(self, valobj, dict):
         self.valobj = valobj
@@ -76,9 +83,9 @@ class StdUnorderedMapSynthProvider:
         # later
         self.count = None
         try:
-            self.head = self.valobj.GetChildMemberWithName('_M_h')
-            self.before_begin = self.head.GetChildMemberWithName('_M_before_begin')
-            self.next = self.before_begin.GetChildMemberWithName('_M_nxt')
+            self.head = self.valobj.GetChildMemberWithName("_M_h")
+            self.before_begin = self.head.GetChildMemberWithName("_M_before_begin")
+            self.next = self.before_begin.GetChildMemberWithName("_M_nxt")
             self.data_type = self.extract_type()
             self.skip_size = self.next.GetType().GetByteSize()
             self.data_size = self.data_type.GetByteSize()
@@ -90,7 +97,7 @@ class StdUnorderedMapSynthProvider:
 
     def get_child_index(self, name):
         try:
-            return int(name.lstrip('[').rstrip(']'))
+            return int(name.lstrip("[").rstrip("]"))
         except:
             return -1
 
@@ -105,9 +112,11 @@ class StdUnorderedMapSynthProvider:
             offset = index
             current = self.next
             while offset > 0:
-                current = current.GetChildMemberWithName('_M_nxt')
+                current = current.GetChildMemberWithName("_M_nxt")
                 offset = offset - 1
-            return current.CreateChildAtOffset( '[' + str(index) + ']', self.skip_size, self.data_type)
+            return current.CreateChildAtOffset(
+                "[" + str(index) + "]", self.skip_size, self.data_type
+            )
 
         except:
             logger >> "Cannot get child"
@@ -121,7 +130,9 @@ class StdUnorderedMapSynthProvider:
     def num_children_impl(self):
         logger = lldb.formatters.Logger.Logger()
         try:
-            count = self.head.GetChildMemberWithName('_M_element_count').GetValueAsUnsigned(0)
+            count = self.head.GetChildMemberWithName(
+                "_M_element_count"
+            ).GetValueAsUnsigned(0)
             return count
         except:
             logger >> "Could not determine the size"
@@ -130,22 +141,25 @@ class StdUnorderedMapSynthProvider:
 
 class AbstractListSynthProvider:
     def __init__(self, valobj, dict, has_prev):
-        '''
+        """
         :param valobj: The value object of the list
         :param dict: A dict with metadata provided by LLDB
         :param has_prev: Whether the list supports a 'prev' pointer besides a 'next' one
-        '''
+        """
         logger = lldb.formatters.Logger.Logger()
         self.valobj = valobj
         self.count = None
         self.has_prev = has_prev
-        self.list_capping_size = self.valobj.GetTarget().GetMaximumNumberOfChildrenToDisplay()
-        logger >> "Providing synthetic children for a list named " + \
-            str(valobj.GetName())
+        self.list_capping_size = (
+            self.valobj.GetTarget().GetMaximumNumberOfChildrenToDisplay()
+        )
+        logger >> "Providing synthetic children for a list named " + str(
+            valobj.GetName()
+        )
 
     def next_node(self, node):
         logger = lldb.formatters.Logger.Logger()
-        return node.GetChildMemberWithName('_M_next')
+        return node.GetChildMemberWithName("_M_next")
 
     def is_valid(self, node):
         logger = lldb.formatters.Logger.Logger()
@@ -159,8 +173,7 @@ class AbstractListSynthProvider:
     def value(self, node):
         logger = lldb.formatters.Logger.Logger()
         value = node.GetValueAsUnsigned()
-        logger >> "synthetic value for {}: {}".format(
-            str(self.valobj.GetName()), value)
+        logger >> "synthetic value for {}: {}".format(str(self.valobj.GetName()), value)
         return value
 
     # Floyd's cycle-finding algorithm
@@ -178,8 +191,7 @@ class AbstractListSynthProvider:
             slow_value = self.value(slow)
             fast1 = self.next_node(fast2)
             fast2 = self.next_node(fast1)
-            if self.value(fast1) == slow_value or self.value(
-                    fast2) == slow_value:
+            if self.value(fast1) == slow_value or self.value(fast2) == slow_value:
                 return True
             slow = self.next_node(slow)
         return False
@@ -188,7 +200,7 @@ class AbstractListSynthProvider:
         logger = lldb.formatters.Logger.Logger()
         if self.count is None:
             # libstdc++ 6.0.21 added dedicated count field.
-            count_child = self.node.GetChildMemberWithName('_M_data')
+            count_child = self.node.GetChildMemberWithName("_M_data")
             if count_child and count_child.IsValid():
                 self.count = count_child.GetValueAsUnsigned(0)
             if self.count is None:
@@ -215,9 +227,11 @@ class AbstractListSynthProvider:
                     return 1
             size = 1
             current = self.next
-            while current.GetChildMemberWithName(
-                    '_M_next').GetValueAsUnsigned(0) != self.get_end_of_list_address():
-                current = current.GetChildMemberWithName('_M_next')
+            while (
+                current.GetChildMemberWithName("_M_next").GetValueAsUnsigned(0)
+                != self.get_end_of_list_address()
+            ):
+                current = current.GetChildMemberWithName("_M_next")
                 if not current.IsValid():
                     break
                 size = size + 1
@@ -232,7 +246,7 @@ class AbstractListSynthProvider:
     def get_child_index(self, name):
         logger = lldb.formatters.Logger.Logger()
         try:
-            return int(name.lstrip('[').rstrip(']'))
+            return int(name.lstrip("[").rstrip("]"))
         except:
             return -1
 
@@ -247,14 +261,15 @@ class AbstractListSynthProvider:
             offset = index
             current = self.next
             while offset > 0:
-                current = current.GetChildMemberWithName('_M_next')
+                current = current.GetChildMemberWithName("_M_next")
                 offset = offset - 1
             # C++ lists store the data of a node after its pointers. In the case of a forward list, there's just one pointer (next), and
             # in the case of a double-linked list, there's an additional pointer (prev).
             return current.CreateChildAtOffset(
-                '[' + str(index) + ']',
-               (2 if self.has_prev else 1) * current.GetType().GetByteSize(),
-                self.data_type)
+                "[" + str(index) + "]",
+                (2 if self.has_prev else 1) * current.GetType().GetByteSize(),
+                self.data_type,
+            )
         except:
             return None
 
@@ -273,7 +288,7 @@ class AbstractListSynthProvider:
         # later
         self.count = None
         try:
-            self.impl = self.valobj.GetChildMemberWithName('_M_impl')
+            self.impl = self.valobj.GetChildMemberWithName("_M_impl")
             self.data_type = self.extract_type()
             if (not self.data_type.IsValid()) or (not self.impl.IsValid()):
                 self.count = 0
@@ -285,34 +300,35 @@ class AbstractListSynthProvider:
             self.count = 0
         return False
 
-    '''
+    """
     Method is used to extract the list pointers into the variables (e.g self.node, self.next, and optionally to self.prev)
     and is mandatory to be overriden in each AbstractListSynthProvider subclass.
     This should return True or False depending on wheter it found valid data.
-    '''
+    """
+
     def updateNodes(self):
         raise NotImplementedError
 
     def has_children(self):
         return True
 
-    '''
+    """
      Method is used to identify if a node traversal has reached its end
      and is mandatory to be overriden in each AbstractListSynthProvider subclass
-    '''
+    """
+
     def get_end_of_list_address(self):
         raise NotImplementedError
 
 
 class StdForwardListSynthProvider(AbstractListSynthProvider):
-
     def __init__(self, valobj, dict):
         has_prev = False
         super().__init__(valobj, dict, has_prev)
 
     def updateNodes(self):
-        self.node = self.impl.GetChildMemberWithName('_M_head')
-        self.next = self.node.GetChildMemberWithName('_M_next')
+        self.node = self.impl.GetChildMemberWithName("_M_head")
+        self.next = self.node.GetChildMemberWithName("_M_next")
         if (not self.node.IsValid()) or (not self.next.IsValid()):
             return False
         return True
@@ -322,17 +338,21 @@ class StdForwardListSynthProvider(AbstractListSynthProvider):
 
 
 class StdListSynthProvider(AbstractListSynthProvider):
-
     def __init__(self, valobj, dict):
         has_prev = True
         super().__init__(valobj, dict, has_prev)
 
     def updateNodes(self):
         self.node_address = self.valobj.AddressOf().GetValueAsUnsigned(0)
-        self.node = self.impl.GetChildMemberWithName('_M_node')
-        self.prev = self.node.GetChildMemberWithName('_M_prev')
-        self.next = self.node.GetChildMemberWithName('_M_next')
-        if self.node_address == 0 or (not self.node.IsValid()) or (not self.next.IsValid()) or (not self.prev.IsValid()):
+        self.node = self.impl.GetChildMemberWithName("_M_node")
+        self.prev = self.node.GetChildMemberWithName("_M_prev")
+        self.next = self.node.GetChildMemberWithName("_M_next")
+        if (
+            self.node_address == 0
+            or (not self.node.IsValid())
+            or (not self.next.IsValid())
+            or (not self.prev.IsValid())
+        ):
             return False
         return True
 
@@ -341,9 +361,7 @@ class StdListSynthProvider(AbstractListSynthProvider):
 
 
 class StdVectorSynthProvider:
-
     class StdVectorImplementation(object):
-
         def __init__(self, valobj):
             self.valobj = valobj
             self.count = None
@@ -379,7 +397,7 @@ class StdVectorSynthProvider:
                 # this check might fail, unless the sizeof() we get is itself incremented to take the
                 # padding bytes into account - on current clang it looks like
                 # this is the case
-                num_children = (finish_val - start_val)
+                num_children = finish_val - start_val
                 if (num_children % self.data_size) != 0:
                     return 0
                 else:
@@ -398,7 +416,8 @@ class StdVectorSynthProvider:
             try:
                 offset = index * self.data_size
                 return self.start.CreateChildAtOffset(
-                    '[' + str(index) + ']', offset, self.data_type)
+                    "[" + str(index) + "]", offset, self.data_type
+                )
             except:
                 return None
 
@@ -407,16 +426,20 @@ class StdVectorSynthProvider:
             # mind later
             self.count = None
             try:
-                impl = self.valobj.GetChildMemberWithName('_M_impl')
-                self.start = impl.GetChildMemberWithName('_M_start')
-                self.finish = impl.GetChildMemberWithName('_M_finish')
-                self.end = impl.GetChildMemberWithName('_M_end_of_storage')
+                impl = self.valobj.GetChildMemberWithName("_M_impl")
+                self.start = impl.GetChildMemberWithName("_M_start")
+                self.finish = impl.GetChildMemberWithName("_M_finish")
+                self.end = impl.GetChildMemberWithName("_M_end_of_storage")
                 self.data_type = self.start.GetType().GetPointeeType()
                 self.data_size = self.data_type.GetByteSize()
                 # if any of these objects is invalid, it means there is no
                 # point in trying to fetch anything
-                if self.start.IsValid() and self.finish.IsValid(
-                ) and self.end.IsValid() and self.data_type.IsValid():
+                if (
+                    self.start.IsValid()
+                    and self.finish.IsValid()
+                    and self.end.IsValid()
+                    and self.data_type.IsValid()
+                ):
                     self.count = None
                 else:
                     self.count = 0
@@ -425,7 +448,6 @@ class StdVectorSynthProvider:
             return False
 
     class StdVBoolImplementation(object):
-
         def __init__(self, valobj, bool_type):
             self.valobj = valobj
             self.bool_type = bool_type
@@ -445,28 +467,31 @@ class StdVectorSynthProvider:
                 return None
             element_type = self.start_p.GetType().GetPointeeType()
             element_bits = 8 * element_type.GetByteSize()
-            element_offset = (index // element_bits) * \
-                element_type.GetByteSize()
+            element_offset = (index // element_bits) * element_type.GetByteSize()
             bit_offset = index % element_bits
             element = self.start_p.CreateChildAtOffset(
-                '[' + str(index) + ']', element_offset, element_type)
+                "[" + str(index) + "]", element_offset, element_type
+            )
             bit = element.GetValueAsUnsigned(0) & (1 << bit_offset)
             if bit != 0:
                 value_expr = "(bool)true"
             else:
                 value_expr = "(bool)false"
-            return self.valobj.CreateValueFromExpression(
-                "[%d]" % index, value_expr)
+            return self.valobj.CreateValueFromExpression("[%d]" % index, value_expr)
 
         def update(self):
             try:
-                m_impl = self.valobj.GetChildMemberWithName('_M_impl')
-                self.m_start = m_impl.GetChildMemberWithName('_M_start')
-                self.m_finish = m_impl.GetChildMemberWithName('_M_finish')
-                self.start_p = self.m_start.GetChildMemberWithName('_M_p')
-                self.finish_p = self.m_finish.GetChildMemberWithName('_M_p')
-                self.offset = self.m_finish.GetChildMemberWithName('_M_offset')
-                if self.offset.IsValid() and self.start_p.IsValid() and self.finish_p.IsValid():
+                m_impl = self.valobj.GetChildMemberWithName("_M_impl")
+                self.m_start = m_impl.GetChildMemberWithName("_M_start")
+                self.m_finish = m_impl.GetChildMemberWithName("_M_finish")
+                self.start_p = self.m_start.GetChildMemberWithName("_M_p")
+                self.finish_p = self.m_finish.GetChildMemberWithName("_M_p")
+                self.offset = self.m_finish.GetChildMemberWithName("_M_offset")
+                if (
+                    self.offset.IsValid()
+                    and self.start_p.IsValid()
+                    and self.finish_p.IsValid()
+                ):
                     self.valid = True
                 else:
                     self.valid = False
@@ -478,19 +503,19 @@ class StdVectorSynthProvider:
         logger = lldb.formatters.Logger.Logger()
         first_template_arg_type = valobj.GetType().GetTemplateArgumentType(0)
         if str(first_template_arg_type.GetName()) == "bool":
-            self.impl = self.StdVBoolImplementation(
-                valobj, first_template_arg_type)
+            self.impl = self.StdVBoolImplementation(valobj, first_template_arg_type)
         else:
             self.impl = self.StdVectorImplementation(valobj)
-        logger >> "Providing synthetic children for a vector named " + \
-            str(valobj.GetName())
+        logger >> "Providing synthetic children for a vector named " + str(
+            valobj.GetName()
+        )
 
     def num_children(self):
         return self.impl.num_children()
 
     def get_child_index(self, name):
         try:
-            return int(name.lstrip('[').rstrip(']'))
+            return int(name.lstrip("[").rstrip("]"))
         except:
             return -1
 
@@ -507,21 +532,27 @@ class StdVectorSynthProvider:
      This formatter can be applied to all
      map-like structures (map, multimap, set, multiset)
     """
-class StdMapLikeSynthProvider:
 
+
+class StdMapLikeSynthProvider:
     def __init__(self, valobj, dict):
         logger = lldb.formatters.Logger.Logger()
         self.valobj = valobj
         self.count = None
         self.kind = self.get_object_kind(valobj)
-        logger >> "Providing synthetic children for a " + self.kind + " named " + \
-            str(valobj.GetName())
+        (
+            logger
+            >> "Providing synthetic children for a "
+            + self.kind
+            + " named "
+            + str(valobj.GetName())
+        )
 
     def get_object_kind(self, valobj):
         type_name = valobj.GetTypeName()
         for kind in ["multiset", "multimap", "set", "map"]:
-           if kind in type_name:
-              return kind
+            if kind in type_name:
+                return kind
         return type_name
 
     # we need this function as a temporary workaround for rdar://problem/10801549
@@ -533,14 +564,26 @@ class StdMapLikeSynthProvider:
     # to find the type name
     def fixup_class_name(self, class_name):
         logger = lldb.formatters.Logger.Logger()
-        if class_name == 'std::basic_string<char, std::char_traits<char>, std::allocator<char> >':
-            return 'std::basic_string<char>', True
-        if class_name == 'basic_string<char, std::char_traits<char>, std::allocator<char> >':
-            return 'std::basic_string<char>', True
-        if class_name == 'std::basic_string<char, std::char_traits<char>, std::allocator<char> >':
-            return 'std::basic_string<char>', True
-        if class_name == 'basic_string<char, std::char_traits<char>, std::allocator<char> >':
-            return 'std::basic_string<char>', True
+        if (
+            class_name
+            == "std::basic_string<char, std::char_traits<char>, std::allocator<char> >"
+        ):
+            return "std::basic_string<char>", True
+        if (
+            class_name
+            == "basic_string<char, std::char_traits<char>, std::allocator<char> >"
+        ):
+            return "std::basic_string<char>", True
+        if (
+            class_name
+            == "std::basic_string<char, std::char_traits<char>, std::allocator<char> >"
+        ):
+            return "std::basic_string<char>", True
+        if (
+            class_name
+            == "basic_string<char, std::char_traits<char>, std::allocator<char> >"
+        ):
+            return "std::basic_string<char>", True
         return class_name, False
 
     def update(self):
@@ -553,9 +596,9 @@ class StdMapLikeSynthProvider:
             # if this gets set to True, then we will merrily return None for
             # any child from that moment on
             self.garbage = False
-            self.Mt = self.valobj.GetChildMemberWithName('_M_t')
-            self.Mimpl = self.Mt.GetChildMemberWithName('_M_impl')
-            self.Mheader = self.Mimpl.GetChildMemberWithName('_M_header')
+            self.Mt = self.valobj.GetChildMemberWithName("_M_t")
+            self.Mimpl = self.Mt.GetChildMemberWithName("_M_impl")
+            self.Mheader = self.Mimpl.GetChildMemberWithName("_M_header")
             if not self.Mheader.IsValid():
                 self.count = 0
             else:
@@ -572,11 +615,13 @@ class StdMapLikeSynthProvider:
                     # GCC does not emit DW_TAG_template_type_parameter for
                     # std::allocator<...>. For such a case, get the type of
                     # std::pair from a member of std::map.
-                    rep_type = self.valobj.GetChildMemberWithName('_M_t').GetType()
-                    self.data_type = rep_type.GetTypedefedType().GetTemplateArgumentType(1)
+                    rep_type = self.valobj.GetChildMemberWithName("_M_t").GetType()
+                    self.data_type = (
+                        rep_type.GetTypedefedType().GetTemplateArgumentType(1)
+                    )
 
                 # from libstdc++ implementation of _M_root for rbtree
-                self.Mroot = self.Mheader.GetChildMemberWithName('_M_parent')
+                self.Mroot = self.Mheader.GetChildMemberWithName("_M_parent")
                 self.data_size = self.data_type.GetByteSize()
                 self.skip_size = self.Mheader.GetType().GetByteSize()
         except:
@@ -596,7 +641,8 @@ class StdMapLikeSynthProvider:
             if root_ptr_val == 0:
                 return 0
             count = self.Mimpl.GetChildMemberWithName(
-                '_M_node_count').GetValueAsUnsigned(0)
+                "_M_node_count"
+            ).GetValueAsUnsigned(0)
             logger >> "I have " + str(count) + " children available"
             return count
         except:
@@ -605,7 +651,7 @@ class StdMapLikeSynthProvider:
     def get_child_index(self, name):
         logger = lldb.formatters.Logger.Logger()
         try:
-            return int(name.lstrip('[').rstrip(']'))
+            return int(name.lstrip("[").rstrip("]"))
         except:
             return -1
 
@@ -627,7 +673,8 @@ class StdMapLikeSynthProvider:
                 offset = offset - 1
             # skip all the base stuff and get at the data
             return current.CreateChildAtOffset(
-                '[' + str(index) + ']', self.skip_size, self.data_type)
+                "[" + str(index) + "]", self.skip_size, self.data_type
+            )
         except:
             return None
 
@@ -667,7 +714,7 @@ class StdMapLikeSynthProvider:
             x = node
             y = self.parent(x)
             max_steps -= 1
-            while(self.node_ptr_value(x) == self.node_ptr_value(self.right(y))):
+            while self.node_ptr_value(x) == self.node_ptr_value(self.right(y)):
                 x = y
                 y = self.parent(y)
                 max_steps -= 1
@@ -682,8 +729,10 @@ class StdMapLikeSynthProvider:
     def has_children(self):
         return True
 
+
 _list_uses_loop_detector = True
 
+
 class StdDequeSynthProvider:
     def __init__(self, valobj, d):
         self.valobj = valobj
@@ -693,7 +742,6 @@ class StdDequeSynthProvider:
         self.element_size = -1
         self.find_block_size()
 
-
     def find_block_size(self):
         # in order to use the deque we must have the block size, or else
         # it's impossible to know what memory addresses are valid
@@ -707,7 +755,7 @@ class StdDequeSynthProvider:
         # #define _GLIBCXX_DEQUE_BUF_SIZE 512
         #
         # return (__size < _GLIBCXX_DEQUE_BUF_SIZE
-	    #   ? size_t(_GLIBCXX_DEQUE_BUF_SIZE / __size) : size_t(1));
+        #   ? size_t(_GLIBCXX_DEQUE_BUF_SIZE / __size) : size_t(1));
         if self.element_size < 512:
             self.block_size = 512 // self.element_size
         else:
@@ -723,7 +771,7 @@ class StdDequeSynthProvider:
 
     def get_child_index(self, name):
         try:
-            return int(name.lstrip('[').rstrip(']'))
+            return int(name.lstrip("[").rstrip("]"))
         except:
             return -1
 
@@ -733,13 +781,15 @@ class StdDequeSynthProvider:
         if index >= self.num_children():
             return None
         try:
-            name = '[' + str(index) + ']'
+            name = "[" + str(index) + "]"
             # We first look for the element in the first subarray,
             # which might be incomplete.
             if index < self.first_node_size:
                 # The following statement is valid because self.first_elem is the pointer
                 # to the first element
-                return self.first_elem.CreateChildAtOffset(name, index * self.element_size, self.element_type)
+                return self.first_elem.CreateChildAtOffset(
+                    name, index * self.element_size, self.element_type
+                )
 
             # Now the rest of the subarrays except for maybe the last one
             # are going to be complete, so the final expression is simpler
@@ -747,10 +797,13 @@ class StdDequeSynthProvider:
 
             # We first move to the beginning of the node/subarray were our element is
             node = self.start_node.CreateChildAtOffset(
-                '',
+                "",
                 (1 + i) * self.valobj.GetProcess().GetAddressByteSize(),
-                self.element_type.GetPointerType())
-            return node.CreateChildAtOffset(name, j * self.element_size, self.element_type)
+                self.element_type.GetPointerType(),
+            )
+            return node.CreateChildAtOffset(
+                name, j * self.element_size, self.element_type
+            )
 
         except:
             return None
@@ -774,44 +827,160 @@ class StdDequeSynthProvider:
 
             count = 0
 
-            impl = self.valobj.GetChildMemberWithName('_M_impl')
+            impl = self.valobj.GetChildMemberWithName("_M_impl")
 
             # we calculate the size of the first node (i.e. first internal array)
-            self.start = impl.GetChildMemberWithName('_M_start')
-            self.start_node = self.start.GetChildMemberWithName('_M_node')
+            self.start = impl.GetChildMemberWithName("_M_start")
+            self.start_node = self.start.GetChildMemberWithName("_M_node")
             first_node_address = self.start_node.GetValueAsUnsigned(0)
-            first_node_last_elem = self.start.GetChildMemberWithName('_M_last').GetValueAsUnsigned(0)
-            self.first_elem = self.start.GetChildMemberWithName('_M_cur')
+            first_node_last_elem = self.start.GetChildMemberWithName(
+                "_M_last"
+            ).GetValueAsUnsigned(0)
+            self.first_elem = self.start.GetChildMemberWithName("_M_cur")
             first_node_first_elem = self.first_elem.GetValueAsUnsigned(0)
 
+            finish = impl.GetChildMemberWithName("_M_finish")
+            last_node_address = finish.GetChildMemberWithName(
+                "_M_node"
+            ).GetValueAsUnsigned(0)
+            last_node_first_elem = finish.GetChildMemberWithName(
+                "_M_first"
+            ).GetValueAsUnsigned(0)
+            last_node_last_elem = finish.GetChildMemberWithName(
+                "_M_cur"
+            ).GetValueAsUnsigned(0)
 
-            finish = impl.GetChildMemberWithName('_M_finish')
-            last_node_address = finish.GetChildMemberWithName('_M_node').GetValueAsUnsigned(0)
-            last_node_first_elem = finish.GetChildMemberWithName('_M_first').GetValueAsUnsigned(0)
-            last_node_last_elem = finish.GetChildMemberWithName('_M_cur').GetValueAsUnsigned(0)
-
-            if first_node_first_elem == 0 or first_node_last_elem == 0 or first_node_first_elem > first_node_last_elem:
+            if (
+                first_node_first_elem == 0
+                or first_node_last_elem == 0
+                or first_node_first_elem > first_node_last_elem
+            ):
                 return False
-            if last_node_first_elem == 0 or last_node_last_elem == 0 or last_node_first_elem > last_node_last_elem:
+            if (
+                last_node_first_elem == 0
+                or last_node_last_elem == 0
+                or last_node_first_elem > last_node_last_elem
+            ):
                 return False
 
-
             if last_node_address == first_node_address:
-                self.first_node_size = (last_node_last_elem - first_node_first_elem) // self.element_size
+                self.first_node_size = (
+                    last_node_last_elem - first_node_first_elem
+                ) // self.element_size
                 count += self.first_node_size
             else:
-                self.first_node_size = (first_node_last_elem - first_node_first_elem) // self.element_size
+                self.first_node_size = (
+                    first_node_last_elem - first_node_first_elem
+                ) // self.element_size
                 count += self.first_node_size
 
                 # we calculate the size of the last node
-                finish = impl.GetChildMemberWithName('_M_finish')
-                last_node_address = finish.GetChildMemberWithName('_M_node').GetValueAsUnsigned(0)
-                count += (last_node_last_elem - last_node_first_elem) // self.element_size
+                finish = impl.GetChildMemberWithName("_M_finish")
+                last_node_address = finish.GetChildMemberWithName(
+                    "_M_node"
+                ).GetValueAsUnsigned(0)
+                count += (
+                    last_node_last_elem - last_node_first_elem
+                ) // self.element_size
 
                 # we calculate the size of the intermediate nodes
-                num_intermediate_nodes = (last_node_address - first_node_address - 1) // self.valobj.GetProcess().GetAddressByteSize()
+                num_intermediate_nodes = (
+                    last_node_address - first_node_address - 1
+                ) // self.valobj.GetProcess().GetAddressByteSize()
                 count += self.block_size * num_intermediate_nodes
             self.count = count
         except:
             pass
         return False
+
+
+def VariantSummaryProvider(valobj, dict):
+    raw_obj = valobj.GetNonSyntheticValue()
+    index_obj = raw_obj.GetChildMemberWithName("_M_index")
+    data_obj = raw_obj.GetChildMemberWithName("_M_u")
+    if not (index_obj and index_obj.IsValid() and data_obj and data_obj.IsValid()):
+        return "<Can't find _M_index or _M_u>"
+
+    def get_variant_npos_value(index_byte_size):
+        if index_byte_size == 1:
+            return 0xFF
+        elif index_byte_size == 2:
+            return 0xFFFF
+        else:
+            return 0xFFFFFFFF
+
+    npos_value = get_variant_npos_value(index_obj.GetByteSize())
+    index = index_obj.GetValueAsUnsigned(0)
+    if index == npos_value:
+        return " No Value"
+
+    # Strip references and typedefs.
+    variant_type = raw_obj.GetType().GetCanonicalType().GetDereferencedType()
+    template_arg_count = variant_type.GetNumberOfTemplateArguments()
+
+    # Invalid index can happen when the variant is not initialized yet.
+    if index >= template_arg_count:
+        return " <Invalid>"
+
+    active_type = variant_type.GetTemplateArgumentType(index)
+    return f" Active Type = {active_type.GetDisplayTypeName()} "
+
+
+class VariantSynthProvider:
+    def __init__(self, valobj, dict):
+        self.raw_obj = valobj.GetNonSyntheticValue()
+        self.is_valid = False
+        self.index = None
+        self.data_obj = None
+
+    def update(self):
+        try:
+            self.index = self.raw_obj.GetChildMemberWithName(
+                "_M_index"
+            ).GetValueAsSigned(-1)
+            self.is_valid = self.index != -1
+            self.data_obj = self.raw_obj.GetChildMemberWithName("_M_u")
+        except:
+            self.is_valid = False
+        return False
+
+    def has_children(self):
+        return True
+
+    def num_children(self):
+        return 1 if self.is_valid else 0
+
+    def get_child_index(self, name):
+        return 0
+
+    def get_child_at_index(self, index):
+        if not self.is_valid:
+            return None
+        cur = 0
+        node = self.data_obj
+        while cur < self.index:
+            node = node.GetChildMemberWithName("_M_rest")
+            cur += 1
+
+        # _M_storage's type depends on variant field's type "_Type".
+        #  1. if '_Type' is literal type: _Type _M_storage.
+        #  2. otherwise, __gnu_cxx::__aligned_membuf<_Type> _M_storage.
+        #
+        # For 2. we have to cast it to underlying template _Type.
+
+        value = node.GetChildMemberWithName("_M_first").GetChildMemberWithName(
+            "_M_storage"
+        )
+        template_type = value.GetType().GetTemplateArgumentType(0)
+
+        # Literal type will return None for GetTemplateArgumentType(0)
+        if (
+            template_type
+            and "__gnu_cxx::__aligned_membuf" in value.GetType().GetDisplayTypeName()
+            and template_type.IsValid()
+        ):
+            value = value.Cast(template_type)
+
+        if value.IsValid():
+            return value.Clone("Value")
+        return None
