@@ -91,7 +91,7 @@ public:
   /// \param[in] user_data
   ///     User data where the SymbolFile parser can store data.
   ///
-  /// \param[in] file_spec
+  /// \param[in] support_file_sp
   ///     The file specification for the source file of this compile
   ///     unit.
   ///
@@ -116,7 +116,7 @@ public:
   ///     An rvalue list of already parsed support files.
   /// \see lldb::LanguageType
   CompileUnit(const lldb::ModuleSP &module_sp, void *user_data,
-              const FileSpec &file_spec, lldb::user_id_t uid,
+              lldb::SupportFileSP support_file_sp, lldb::user_id_t uid,
               lldb::LanguageType language, lldb_private::LazyBool is_optimized,
               SupportFileList &&support_files = {});
 
@@ -226,11 +226,15 @@ public:
                          const FileSpec *file_spec_ptr, bool exact,
                          LineEntry *line_entry);
 
-  /// Return the primary source file associated with this compile unit.
-  const FileSpec &GetPrimaryFile() const { return m_file_spec; }
+  /// Return the primary source spec associated with this compile unit.
+  const FileSpec &GetPrimaryFile() const {
+    return m_primary_support_file_sp->GetSpecOnly();
+  }
 
   /// Return the primary source file associated with this compile unit.
-  void SetPrimaryFile(const FileSpec &fs) { m_file_spec = fs; }
+  lldb::SupportFileSP GetPrimarySupportFile() const {
+    return m_primary_support_file_sp;
+  }
 
   /// Get the line table for the compile unit.
   ///
@@ -419,7 +423,7 @@ protected:
   /// compile unit.
   std::vector<SourceModule> m_imported_modules;
   /// The primary file associated with this compile unit.
-  FileSpec m_file_spec;
+  lldb::SupportFileSP m_primary_support_file_sp;
   /// Files associated with this compile unit's line table and declarations.
   SupportFileList m_support_files;
   /// Line table that will get parsed on demand.
