@@ -23,9 +23,7 @@
 #include "lldb/API/SBValue.h"
 #include "lldb/Breakpoint/BreakpointLocation.h"
 #include "lldb/Core/Debugger.h"
-#include "lldb/Core/StreamFile.h"
 #include "lldb/Core/StructuredDataImpl.h"
-#include "lldb/Core/ValueObject.h"
 #include "lldb/Interpreter/CommandInterpreter.h"
 #include "lldb/Symbol/CompileUnit.h"
 #include "lldb/Symbol/SymbolContext.h"
@@ -44,6 +42,7 @@
 #include "lldb/Utility/State.h"
 #include "lldb/Utility/Stream.h"
 #include "lldb/Utility/StructuredData.h"
+#include "lldb/ValueObject/ValueObject.h"
 #include "lldb/lldb-enumerations.h"
 
 #include <memory>
@@ -54,7 +53,7 @@ using namespace lldb_private;
 const char *SBThread::GetBroadcasterClassName() {
   LLDB_INSTRUMENT();
 
-  return Thread::GetStaticBroadcasterClass().AsCString();
+  return ConstString(Thread::GetStaticBroadcasterClass()).AsCString();
 }
 
 // Constructors
@@ -521,13 +520,13 @@ SBError SBThread::ResumeNewPlan(ExecutionContext &exe_ctx,
 
   Process *process = exe_ctx.GetProcessPtr();
   if (!process) {
-    sb_error.SetErrorString("No process in SBThread::ResumeNewPlan");
+    sb_error = Status::FromErrorString("No process in SBThread::ResumeNewPlan");
     return sb_error;
   }
 
   Thread *thread = exe_ctx.GetThreadPtr();
   if (!thread) {
-    sb_error.SetErrorString("No thread in SBThread::ResumeNewPlan");
+    sb_error = Status::FromErrorString("No thread in SBThread::ResumeNewPlan");
     return sb_error;
   }
 
@@ -563,7 +562,7 @@ void SBThread::StepOver(lldb::RunMode stop_other_threads, SBError &error) {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return;
   }
 
@@ -610,7 +609,7 @@ void SBThread::StepInto(const char *target_name, uint32_t end_line,
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return;
   }
 
@@ -647,7 +646,7 @@ void SBThread::StepInto(const char *target_name, uint32_t end_line,
   if (new_plan_status.Success())
     error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
   else
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
 }
 
 void SBThread::StepOut() {
@@ -664,7 +663,7 @@ void SBThread::StepOut(SBError &error) {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return;
   }
 
@@ -682,7 +681,7 @@ void SBThread::StepOut(SBError &error) {
   if (new_plan_status.Success())
     error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
   else
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
 }
 
 void SBThread::StepOutOfFrame(SBFrame &sb_frame) {
@@ -699,14 +698,14 @@ void SBThread::StepOutOfFrame(SBFrame &sb_frame, SBError &error) {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!sb_frame.IsValid()) {
-    error.SetErrorString("passed invalid SBFrame object");
+    error = Status::FromErrorString("passed invalid SBFrame object");
     return;
   }
 
   StackFrameSP frame_sp(sb_frame.GetFrameSP());
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return;
   }
 
@@ -714,7 +713,7 @@ void SBThread::StepOutOfFrame(SBFrame &sb_frame, SBError &error) {
   bool stop_other_threads = false;
   Thread *thread = exe_ctx.GetThreadPtr();
   if (sb_frame.GetThread().GetThreadID() != thread->GetID()) {
-    error.SetErrorString("passed a frame from another thread");
+    error = Status::FromErrorString("passed a frame from another thread");
     return;
   }
 
@@ -726,7 +725,7 @@ void SBThread::StepOutOfFrame(SBFrame &sb_frame, SBError &error) {
   if (new_plan_status.Success())
     error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
   else
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
 }
 
 void SBThread::StepInstruction(bool step_over) {
@@ -743,19 +742,19 @@ void SBThread::StepInstruction(bool step_over, SBError &error) {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return;
   }
 
   Thread *thread = exe_ctx.GetThreadPtr();
   Status new_plan_status;
   ThreadPlanSP new_plan_sp(thread->QueueThreadPlanForStepSingleInstruction(
-      step_over, true, true, new_plan_status));
+      step_over, false, true, new_plan_status));
 
   if (new_plan_status.Success())
     error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
   else
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
 }
 
 void SBThread::RunToAddress(lldb::addr_t addr) {
@@ -772,7 +771,7 @@ void SBThread::RunToAddress(lldb::addr_t addr, SBError &error) {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return;
   }
 
@@ -790,7 +789,7 @@ void SBThread::RunToAddress(lldb::addr_t addr, SBError &error) {
   if (new_plan_status.Success())
     error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
   else
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
 }
 
 SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
@@ -810,7 +809,7 @@ SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
     Thread *thread = exe_ctx.GetThreadPtr();
 
     if (line == 0) {
-      sb_error.SetErrorString("invalid line argument");
+      sb_error = Status::FromErrorString("invalid line argument");
       return sb_error;
     }
 
@@ -826,7 +825,7 @@ SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
 
     SymbolContext frame_sc;
     if (!frame_sp) {
-      sb_error.SetErrorString("no valid frames in thread to step");
+      sb_error = Status::FromErrorString("no valid frames in thread to step");
       return sb_error;
     }
 
@@ -836,7 +835,7 @@ SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
         eSymbolContextLineEntry | eSymbolContextSymbol);
 
     if (frame_sc.comp_unit == nullptr) {
-      sb_error.SetErrorStringWithFormat(
+      sb_error = Status::FromErrorStringWithFormat(
           "frame %u doesn't have debug information", frame_sp->GetFrameIndex());
       return sb_error;
     }
@@ -847,9 +846,10 @@ SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
       step_file_spec = sb_file_spec.ref();
     } else {
       if (frame_sc.line_entry.IsValid())
-        step_file_spec = frame_sc.line_entry.file;
+        step_file_spec = frame_sc.line_entry.GetFile();
       else {
-        sb_error.SetErrorString("invalid file argument or no file for frame");
+        sb_error = Status::FromErrorString(
+            "invalid file argument or no file for frame");
         return sb_error;
       }
     }
@@ -887,10 +887,11 @@ SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
     if (step_over_until_addrs.empty()) {
       if (all_in_function) {
         step_file_spec.GetPath(path, sizeof(path));
-        sb_error.SetErrorStringWithFormat("No line entries for %s:%u", path,
-                                          line);
+        sb_error = Status::FromErrorStringWithFormat(
+            "No line entries for %s:%u", path, line);
       } else
-        sb_error.SetErrorString("step until target not in current function");
+        sb_error = Status::FromErrorString(
+            "step until target not in current function");
     } else {
       Status new_plan_status;
       ThreadPlanSP new_plan_sp(thread->QueueThreadPlanForStepUntil(
@@ -901,10 +902,10 @@ SBError SBThread::StepOverUntil(lldb::SBFrame &sb_frame,
       if (new_plan_status.Success())
         sb_error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
       else
-        sb_error.SetErrorString(new_plan_status.AsCString());
+        sb_error = Status::FromErrorString(new_plan_status.AsCString());
     }
   } else {
-    sb_error.SetErrorString("this SBThread object is invalid");
+    sb_error = Status::FromErrorString("this SBThread object is invalid");
   }
   return sb_error;
 }
@@ -935,7 +936,7 @@ SBError SBThread::StepUsingScriptedThreadPlan(const char *script_class_name,
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
     return error;
   }
 
@@ -947,7 +948,7 @@ SBError SBThread::StepUsingScriptedThreadPlan(const char *script_class_name,
       false, script_class_name, obj_sp, false, new_plan_status);
 
   if (new_plan_status.Fail()) {
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
     return error;
   }
 
@@ -957,7 +958,7 @@ SBError SBThread::StepUsingScriptedThreadPlan(const char *script_class_name,
   if (new_plan_status.Success())
     error = ResumeNewPlan(exe_ctx, new_plan_sp.get());
   else
-    error.SetErrorString(new_plan_status.AsCString());
+    error = Status::FromErrorString(new_plan_status.AsCString());
 
   return error;
 }
@@ -971,14 +972,14 @@ SBError SBThread::JumpToLine(lldb::SBFileSpec &file_spec, uint32_t line) {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (!exe_ctx.HasThreadScope()) {
-    sb_error.SetErrorString("this SBThread object is invalid");
+    sb_error = Status::FromErrorString("this SBThread object is invalid");
     return sb_error;
   }
 
   Thread *thread = exe_ctx.GetThreadPtr();
 
   Status err = thread->JumpToLine(file_spec.ref(), line, true);
-  sb_error.SetError(err);
+  sb_error.SetError(std::move(err));
   return sb_error;
 }
 
@@ -1037,10 +1038,10 @@ bool SBThread::Suspend(SBError &error) {
       exe_ctx.GetThreadPtr()->SetResumeState(eStateSuspended);
       result = true;
     } else {
-      error.SetErrorString("process is running");
+      error = Status::FromErrorString("process is running");
     }
   } else
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
   return result;
 }
 
@@ -1065,10 +1066,10 @@ bool SBThread::Resume(SBError &error) {
       exe_ctx.GetThreadPtr()->SetResumeState(eStateRunning, override_suspend);
       result = true;
     } else {
-      error.SetErrorString("process is running");
+      error = Status::FromErrorString("process is running");
     }
   } else
-    error.SetErrorString("this SBThread object is invalid");
+    error = Status::FromErrorString("this SBThread object is invalid");
   return result;
 }
 
@@ -1230,7 +1231,8 @@ bool SBThread::GetStatus(SBStream &status) const {
   ExecutionContext exe_ctx(m_opaque_sp.get(), lock);
 
   if (exe_ctx.HasThreadScope()) {
-    exe_ctx.GetThreadPtr()->GetStatus(strm, 0, 1, 1, true);
+    exe_ctx.GetThreadPtr()->GetStatus(strm, 0, 1, 1, true,
+                                      /*show_hidden=*/true);
   } else
     strm.PutCString("No status");
 
@@ -1266,7 +1268,7 @@ SBError SBThread::GetDescriptionWithFormat(const SBFormat &format,
 
   SBError error;
   if (!format) {
-    error.SetErrorString("The provided SBFormat object is invalid");
+    error = Status::FromErrorString("The provided SBFormat object is invalid");
     return error;
   }
 
@@ -1280,7 +1282,7 @@ SBError SBThread::GetDescriptionWithFormat(const SBFormat &format,
     }
   }
 
-  error.SetErrorStringWithFormat(
+  error = Status::FromErrorStringWithFormat(
       "It was not possible to generate a thread description with the given "
       "format string '%s'",
       format.GetFormatEntrySP()->string.c_str());
@@ -1359,6 +1361,8 @@ bool SBThread::SafeToCallFunctions() {
   return true;
 }
 
+lldb::ThreadSP SBThread::GetSP() const { return m_opaque_sp->GetThreadSP(); }
+
 lldb_private::Thread *SBThread::operator->() {
   return get();
 }
