@@ -20,7 +20,7 @@ public:
   ~CommandObjectHealthcheck() override = default;
 
 protected:
-  bool DoExecute(Args &args, CommandReturnObject &result) override;
+  void DoExecute(Args &args, CommandReturnObject &result) override;
 };
 
 } // namespace lldb_private
