@@ -474,7 +474,7 @@ bool SourceManager::GetDefaultFileAndLine(FileSpec &file_spec, uint32_t &line) {
             if (sc.function->GetAddressRange()
                     .GetBaseAddress()
                     .CalculateSymbolContextLineEntry(line_entry)) {
-              SetDefaultFileAndLine(line_entry.file, line_entry.line);
+              SetDefaultFileAndLine(line_entry.GetFile(), line_entry.line);
               file_spec = m_last_file_spec;
               line = m_last_line;
               return true;
@@ -484,7 +484,7 @@ bool SourceManager::GetDefaultFileAndLine(FileSpec &file_spec, uint32_t &line) {
 #else // FIXME: upstream this!
         lldb_private::LineEntry line_entry(FindEntryPoint(executable_ptr));
         if (line_entry.IsValid()) {
-          SetDefaultFileAndLine(line_entry.file, line_entry.line);
+          SetDefaultFileAndLine(line_entry.GetFile(), line_entry.line);
           file_spec = m_last_file_spec;
           line = m_last_line;
           return true;
