@@ -23,6 +23,7 @@
 #include <chrono>
 #include <cstring>
 #include <memory>
+#include <shared_mutex>
 
 #include <cerrno>
 #include <cinttypes>
@@ -31,8 +32,8 @@
 using namespace lldb;
 using namespace lldb_private;
 
-ConstString &ThreadedCommunication::GetStaticBroadcasterClass() {
-  static ConstString class_name("lldb.communication");
+llvm::StringRef ThreadedCommunication::GetStaticBroadcasterClass() {
+  static constexpr llvm::StringLiteral class_name("lldb.communication");
   return class_name;
 }
 
@@ -91,14 +92,14 @@ size_t ThreadedCommunication::Read(void *dst, size_t dst_len,
     }
     if (timeout && timeout->count() == 0) {
       if (error_ptr)
-        error_ptr->SetErrorString("Timed out.");
+        *error_ptr = Status::FromErrorString("Timed out.");
       status = eConnectionStatusTimedOut;
       return 0;
     }
 
     if (!m_connection_sp) {
       if (error_ptr)
-        error_ptr->SetErrorString("Invalid connection.");
+        *error_ptr = Status::FromErrorString("Invalid connection.");
       status = eConnectionStatusNoConnection;
       return 0;
     }
@@ -125,7 +126,7 @@ size_t ThreadedCommunication::Read(void *dst, size_t dst_len,
     } else {
       if (!listener_sp->GetEvent(event_sp, timeout)) {
         if (error_ptr)
-          error_ptr->SetErrorString("Timed out.");
+          *error_ptr = Status::FromErrorString("Timed out.");
         status = eConnectionStatusTimedOut;
         return 0;
       }
@@ -155,6 +156,8 @@ size_t ThreadedCommunication::Read(void *dst, size_t dst_len,
 }
 
 bool ThreadedCommunication::StartReadThread(Status *error_ptr) {
+  std::lock_guard<std::mutex> lock(m_read_thread_mutex);
+
   if (error_ptr)
     error_ptr->Clear();
 
@@ -175,7 +178,7 @@ bool ThreadedCommunication::StartReadThread(Status *error_ptr) {
     m_read_thread = *maybe_thread;
   } else {
     if (error_ptr)
-      *error_ptr = Status(maybe_thread.takeError());
+      *error_ptr = Status::FromError(maybe_thread.takeError());
     else {
       LLDB_LOG_ERROR(GetLog(LLDBLog::Host), maybe_thread.takeError(),
                      "failed to launch host thread: {0}");
@@ -189,6 +192,8 @@ bool ThreadedCommunication::StartReadThread(Status *error_ptr) {
 }
 
 bool ThreadedCommunication::StopReadThread(Status *error_ptr) {
+  std::lock_guard<std::mutex> lock(m_read_thread_mutex);
+
   if (!m_read_thread.IsJoinable())
     return true;
 
@@ -199,13 +204,13 @@ bool ThreadedCommunication::StopReadThread(Status *error_ptr) {
 
   BroadcastEvent(eBroadcastBitReadThreadShouldExit, nullptr);
 
-  // error = m_read_thread.Cancel();
-
   Status error = m_read_thread.Join(nullptr);
   return error.Success();
 }
 
 bool ThreadedCommunication::JoinReadThread(Status *error_ptr) {
+  std::lock_guard<std::mutex> lock(m_read_thread_mutex);
+
   if (!m_read_thread.IsJoinable())
     return true;
 
