@@ -7,9 +7,9 @@
 //===----------------------------------------------------------------------===//
 
 #include "lldb/Expression/IRInterpreter.h"
+#include "lldb/Core/Debugger.h"
 #include "lldb/Core/Module.h"
 #include "lldb/Core/ModuleSpec.h"
-#include "lldb/Core/ValueObject.h"
 #include "lldb/Expression/DiagnosticManager.h"
 #include "lldb/Expression/IRExecutionUnit.h"
 #include "lldb/Expression/IRMemoryMap.h"
@@ -21,6 +21,7 @@
 #include "lldb/Utility/Scalar.h"
 #include "lldb/Utility/Status.h"
 #include "lldb/Utility/StreamString.h"
+#include "lldb/ValueObject/ValueObject.h"
 
 #include "lldb/Target/ABI.h"
 #include "lldb/Target/ExecutionContext.h"
@@ -95,7 +96,7 @@ public:
   typedef std::map<const Value *, lldb::addr_t> ValueMap;
 
   ValueMap m_values;
-  DataLayout &m_target_data;
+  const DataLayout &m_target_data;
   lldb_private::IRExecutionUnit &m_execution_unit;
   const BasicBlock *m_bb = nullptr;
   const BasicBlock *m_prev_bb = nullptr;
@@ -109,7 +110,7 @@ public:
   lldb::ByteOrder m_byte_order;
   size_t m_addr_byte_size;
 
-  InterpreterStackFrame(DataLayout &target_data,
+  InterpreterStackFrame(const DataLayout &target_data,
                         lldb_private::IRExecutionUnit &execution_unit,
                         lldb::addr_t stack_frame_bottom,
                         lldb::addr_t stack_frame_top)
@@ -263,7 +264,7 @@ public:
         lldb_private::ConstString name(constant_func->getName());
         bool missing_weak = false;
         lldb::addr_t addr = m_execution_unit.FindSymbol(name, missing_weak);
-        if (addr == LLDB_INVALID_ADDRESS || missing_weak)
+        if (addr == LLDB_INVALID_ADDRESS)
           return false;
         value = APInt(m_target_data.getPointerSizeInBits(), addr);
         return true;
@@ -408,7 +409,7 @@ public:
     lldb_private::Status alloc_error;
 
     return Malloc(m_target_data.getTypeAllocSize(type),
-                  m_target_data.getPrefTypeAlignment(type));
+                  m_target_data.getPrefTypeAlign(type).value());
   }
 
   std::string PrintData(lldb::addr_t addr, llvm::Type *type) {
@@ -464,13 +465,16 @@ static const char *unsupported_operand_error =
     "Interpreter doesn't handle one of the expression's operands";
 static const char *interpreter_internal_error =
     "Interpreter encountered an internal error";
+static const char *interrupt_error =
+    "Interrupted while interpreting expression";
 static const char *bad_value_error =
     "Interpreter couldn't resolve a value during execution";
 static const char *memory_allocation_error =
     "Interpreter couldn't allocate memory";
 static const char *memory_write_error = "Interpreter couldn't write to memory";
 static const char *memory_read_error = "Interpreter couldn't read from memory";
-static const char *infinite_loop_error = "Interpreter ran for too many cycles";
+static const char *timeout_error =
+    "Reached timeout while interpreting expression";
 static const char *too_many_functions_error =
     "Interpreter doesn't handle modules with multiple function bodies.";
 
@@ -526,11 +530,11 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
     if (f.begin() != f.end()) {
       if (saw_function_with_body) {
         LLDB_LOGF(log, "More than one function in the module has a body");
-        error.SetErrorToGenericError();
-        error.SetErrorString(too_many_functions_error);
+        error = lldb_private::Status::FromErrorString(too_many_functions_error);
         return false;
       }
       saw_function_with_body = true;
+      LLDB_LOGF(log, "Saw function with body: %s", f.getName().str().c_str());
     }
   }
 
@@ -539,8 +543,7 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
       switch (ii.getOpcode()) {
       default: {
         LLDB_LOGF(log, "Unsupported instruction: %s", PrintValue(&ii).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(unsupported_opcode_error);
+        error = lldb_private::Status::FromErrorString(unsupported_opcode_error);
         return false;
       }
       case Instruction::Add:
@@ -553,16 +556,16 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
         CallInst *call_inst = dyn_cast<CallInst>(&ii);
 
         if (!call_inst) {
-          error.SetErrorToGenericError();
-          error.SetErrorString(interpreter_internal_error);
+          error =
+              lldb_private::Status::FromErrorString(interpreter_internal_error);
           return false;
         }
 
         if (!CanIgnoreCall(call_inst) && !support_function_calls) {
           LLDB_LOGF(log, "Unsupported instruction: %s",
                     PrintValue(&ii).c_str());
-          error.SetErrorToGenericError();
-          error.SetErrorString(unsupported_opcode_error);
+          error =
+              lldb_private::Status::FromErrorString(unsupported_opcode_error);
           return false;
         }
       } break;
@@ -573,8 +576,8 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
         CmpInst *cmp_inst = dyn_cast<CmpInst>(&ii);
 
         if (!cmp_inst) {
-          error.SetErrorToGenericError();
-          error.SetErrorString(interpreter_internal_error);
+          error =
+              lldb_private::Status::FromErrorString(interpreter_internal_error);
           return false;
         }
 
@@ -583,8 +586,8 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
           LLDB_LOGF(log, "Unsupported ICmp predicate: %s",
                     PrintValue(&ii).c_str());
 
-          error.SetErrorToGenericError();
-          error.SetErrorString(unsupported_opcode_error);
+          error =
+              lldb_private::Status::FromErrorString(unsupported_opcode_error);
           return false;
         }
         case CmpInst::FCMP_OEQ:
@@ -645,7 +648,8 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
         case Type::ScalableVectorTyID: {
           LLDB_LOGF(log, "Unsupported operand type: %s",
                     PrintType(operand_type).c_str());
-          error.SetErrorString(unsupported_operand_error);
+          error =
+              lldb_private::Status::FromErrorString(unsupported_operand_error);
           return false;
         }
         }
@@ -657,7 +661,8 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
         if (operand_type->getPrimitiveSizeInBits() > 64) {
           LLDB_LOGF(log, "Unsupported operand type: %s",
                     PrintType(operand_type).c_str());
-          error.SetErrorString(unsupported_operand_error);
+          error =
+              lldb_private::Status::FromErrorString(unsupported_operand_error);
           return false;
         }
 
@@ -665,7 +670,8 @@ bool IRInterpreter::CanInterpret(llvm::Module &module, llvm::Function &function,
           if (!CanResolveConstant(constant)) {
             LLDB_LOGF(log, "Unsupported constant: %s",
                       PrintValue(constant).c_str());
-            error.SetErrorString(unsupported_operand_error);
+            error = lldb_private::Status::FromErrorString(
+                unsupported_operand_error);
             return false;
           }
         }
@@ -682,7 +688,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
                               lldb_private::Status &error,
                               lldb::addr_t stack_frame_bottom,
                               lldb::addr_t stack_frame_top,
-                              lldb_private::ExecutionContext &exe_ctx) {
+                              lldb_private::ExecutionContext &exe_ctx,
+                              lldb_private::Timeout<std::micro> timeout) {
   lldb_private::Log *log(GetLog(LLDBLog::Expressions));
 
   if (log) {
@@ -697,13 +704,14 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
               s.c_str());
   }
 
-  DataLayout data_layout(&module);
+  const DataLayout &data_layout = module.getDataLayout();
 
   InterpreterStackFrame frame(data_layout, execution_unit, stack_frame_bottom,
                               stack_frame_top);
 
   if (frame.m_frame_process_address == LLDB_INVALID_ADDRESS) {
-    error.SetErrorString("Couldn't allocate stack frame");
+    error =
+        lldb_private::Status::FromErrorString("Couldn't allocate stack frame");
   }
 
   int arg_index = 0;
@@ -712,7 +720,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
                                     ae = function.arg_end();
        ai != ae; ++ai, ++arg_index) {
     if (args.size() <= static_cast<size_t>(arg_index)) {
-      error.SetErrorString("Not enough arguments passed in to function");
+      error = lldb_private::Status::FromErrorString(
+          "Not enough arguments passed in to function");
       return false;
     }
 
@@ -721,11 +730,34 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
     frame.MakeArgument(&*ai, ptr);
   }
 
-  uint32_t num_insts = 0;
-
   frame.Jump(&function.front());
 
-  while (frame.m_ii != frame.m_ie && (++num_insts < 4096)) {
+  lldb_private::Process *process = exe_ctx.GetProcessPtr();
+  lldb_private::Target *target = exe_ctx.GetTargetPtr();
+
+  using clock = std::chrono::steady_clock;
+
+  // Compute the time at which the timeout has been exceeded.
+  std::optional<clock::time_point> end_time;
+  if (timeout && timeout->count() > 0)
+    end_time = clock::now() + *timeout;
+
+  while (frame.m_ii != frame.m_ie) {
+    // Timeout reached: stop interpreting.
+    if (end_time && clock::now() >= *end_time) {
+      error = lldb_private::Status::FromErrorString(timeout_error);
+      return false;
+    }
+
+    // If we have access to the debugger we can honor an interrupt request.
+    if (target) {
+      if (INTERRUPT_REQUESTED(target->GetDebugger(),
+                              "Interrupted in IR interpreting.")) {
+        error = lldb_private::Status::FromErrorString(interrupt_error);
+        return false;
+      }
+    }
+
     const Instruction *inst = &*frame.m_ii;
 
     LLDB_LOGF(log, "Interpreting %s", PrintValue(inst).c_str());
@@ -758,8 +790,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
             log,
             "getOpcode() returns %s, but instruction is not a BinaryOperator",
             inst->getOpcodeName());
-        error.SetErrorToGenericError();
-        error.SetErrorString(interpreter_internal_error);
+        error =
+            lldb_private::Status::FromErrorString(interpreter_internal_error);
         return false;
       }
 
@@ -771,15 +803,13 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(L, lhs, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(lhs).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
       if (!frame.EvaluateValue(R, rhs, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(rhs).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -859,8 +889,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       if (alloca_inst->isArrayAllocation()) {
         LLDB_LOGF(log,
                   "AllocaInsts are not handled if isArrayAllocation() is true");
-        error.SetErrorToGenericError();
-        error.SetErrorString(unsupported_opcode_error);
+        error = lldb_private::Status::FromErrorString(unsupported_opcode_error);
         return false;
       }
 
@@ -878,8 +907,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (R == LLDB_INVALID_ADDRESS) {
         LLDB_LOGF(log, "Couldn't allocate memory for an AllocaInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_allocation_error);
+        error = lldb_private::Status::FromErrorString(memory_allocation_error);
         return false;
       }
 
@@ -888,8 +916,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       if (P == LLDB_INVALID_ADDRESS) {
         LLDB_LOGF(log,
                   "Couldn't allocate the result pointer for an AllocaInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_allocation_error);
+        error = lldb_private::Status::FromErrorString(memory_allocation_error);
         return false;
       }
 
@@ -899,8 +926,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!write_error.Success()) {
         LLDB_LOGF(log, "Couldn't write the result pointer for an AllocaInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_write_error);
+        error = lldb_private::Status::FromErrorString(memory_write_error);
         lldb_private::Status free_error;
         execution_unit.Free(P, free_error);
         execution_unit.Free(R, free_error);
@@ -925,8 +951,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(S, source, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(source).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -941,8 +966,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(S, source, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(source).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -962,8 +986,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
         if (!frame.EvaluateValue(C, condition, module)) {
           LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(condition).c_str());
-          error.SetErrorToGenericError();
-          error.SetErrorString(bad_value_error);
+          error = lldb_private::Status::FromErrorString(bad_value_error);
           return false;
         }
 
@@ -992,8 +1015,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
         LLDB_LOGF(log,
                   "Encountered PHI node without having jumped from another "
                   "basic block");
-        error.SetErrorToGenericError();
-        error.SetErrorString(interpreter_internal_error);
+        error =
+            lldb_private::Status::FromErrorString(interpreter_internal_error);
         return false;
       }
 
@@ -1001,8 +1024,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       lldb_private::Scalar result;
       if (!frame.EvaluateValue(result, value, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(value).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
       frame.AssignValue(inst, result, module);
@@ -1024,8 +1046,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       if (!frame.EvaluateValue(P, pointer_operand, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s",
                   PrintValue(pointer_operand).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1046,8 +1067,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
           if (!frame.EvaluateValue(I, *ii, module)) {
             LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(*ii).c_str());
-            error.SetErrorToGenericError();
-            error.SetErrorString(bad_value_error);
+            error = lldb_private::Status::FromErrorString(bad_value_error);
             return false;
           }
 
@@ -1089,15 +1109,13 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(L, lhs, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(lhs).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
       if (!frame.EvaluateValue(R, rhs, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(rhs).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1186,8 +1204,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(I, src_operand, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(src_operand).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1208,8 +1225,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(I, src_operand, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(src_operand).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1230,8 +1246,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!frame.EvaluateValue(I, src_operand, module)) {
         LLDB_LOGF(log, "Couldn't evaluate %s", PrintValue(src_operand).c_str());
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1259,15 +1274,13 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (D == LLDB_INVALID_ADDRESS) {
         LLDB_LOGF(log, "LoadInst's value doesn't resolve to anything");
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
       if (P == LLDB_INVALID_ADDRESS) {
         LLDB_LOGF(log, "LoadInst's pointer doesn't resolve to anything");
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1277,8 +1290,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!read_error.Success()) {
         LLDB_LOGF(log, "Couldn't read the address to be loaded for a LoadInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_read_error);
+        error = lldb_private::Status::FromErrorString(memory_read_error);
         return false;
       }
 
@@ -1291,8 +1303,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
                                 read_error);
       if (!read_error.Success()) {
         LLDB_LOGF(log, "Couldn't read from a region on behalf of a LoadInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_read_error);
+        error = lldb_private::Status::FromErrorString(memory_read_error);
         return false;
       }
 
@@ -1301,8 +1312,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
                                  write_error);
       if (!write_error.Success()) {
         LLDB_LOGF(log, "Couldn't write to a region on behalf of a LoadInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_write_error);
+        error = lldb_private::Status::FromErrorString(memory_write_error);
         return false;
       }
 
@@ -1333,15 +1343,13 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (D == LLDB_INVALID_ADDRESS) {
         LLDB_LOGF(log, "StoreInst's value doesn't resolve to anything");
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
       if (P == LLDB_INVALID_ADDRESS) {
         LLDB_LOGF(log, "StoreInst's pointer doesn't resolve to anything");
-        error.SetErrorToGenericError();
-        error.SetErrorString(bad_value_error);
+        error = lldb_private::Status::FromErrorString(bad_value_error);
         return false;
       }
 
@@ -1351,8 +1359,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
       if (!read_error.Success()) {
         LLDB_LOGF(log, "Couldn't read the address to be loaded for a LoadInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_read_error);
+        error = lldb_private::Status::FromErrorString(memory_read_error);
         return false;
       }
 
@@ -1365,8 +1372,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
                                 read_error);
       if (!read_error.Success()) {
         LLDB_LOGF(log, "Couldn't read from a region on behalf of a StoreInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_read_error);
+        error = lldb_private::Status::FromErrorString(memory_read_error);
         return false;
       }
 
@@ -1375,8 +1381,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
                                  write_error);
       if (!write_error.Success()) {
         LLDB_LOGF(log, "Couldn't write to a region on behalf of a StoreInst");
-        error.SetErrorToGenericError();
-        error.SetErrorString(memory_write_error);
+        error = lldb_private::Status::FromErrorString(memory_write_error);
         return false;
       }
 
@@ -1396,30 +1401,30 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       // Get the return type
       llvm::Type *returnType = call_inst->getType();
       if (returnType == nullptr) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("unable to access return type");
+        error = lldb_private::Status::FromErrorString(
+            "unable to access return type");
         return false;
       }
 
       // Work with void, integer and pointer return types
       if (!returnType->isVoidTy() && !returnType->isIntegerTy() &&
           !returnType->isPointerTy()) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("return type is not supported");
+        error = lldb_private::Status::FromErrorString(
+            "return type is not supported");
         return false;
       }
 
       // Check we can actually get a thread
       if (exe_ctx.GetThreadPtr() == nullptr) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("unable to acquire thread");
+        error =
+            lldb_private::Status::FromErrorString("unable to acquire thread");
         return false;
       }
 
       // Make sure we have a valid process
-      if (!exe_ctx.GetProcessPtr()) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("unable to get the process");
+      if (!process) {
+        error =
+            lldb_private::Status::FromErrorString("unable to get the process");
         return false;
       }
 
@@ -1428,8 +1433,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       const llvm::Value *val = call_inst->getCalledOperand();
 
       if (!frame.EvaluateValue(I, val, module)) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("unable to get address of function");
+        error = lldb_private::Status::FromErrorString(
+            "unable to get address of function");
         return false;
       }
       lldb_private::Address funcAddr(I.ULongLong(LLDB_INVALID_ADDRESS));
@@ -1445,8 +1450,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       // We work with a fixed array of 16 arguments which is our upper limit
       static lldb_private::ABI::CallArgument rawArgs[16];
       if (numArgs >= 16) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("function takes too many arguments");
+        error = lldb_private::Status::FromErrorString(
+            "function takes too many arguments");
         return false;
       }
 
@@ -1459,16 +1464,16 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
         // Ensure that this argument is an supported type
         if (!arg_ty->isIntegerTy() && !arg_ty->isPointerTy()) {
-          error.SetErrorToGenericError();
-          error.SetErrorStringWithFormat("argument %d must be integer type", i);
+          error = lldb_private::Status::FromErrorStringWithFormat(
+              "argument %d must be integer type", i);
           return false;
         }
 
         // Extract the arguments value
         lldb_private::Scalar tmp_op = 0;
         if (!frame.EvaluateValue(tmp_op, arg_op, module)) {
-          error.SetErrorToGenericError();
-          error.SetErrorStringWithFormat("unable to evaluate argument %d", i);
+          error = lldb_private::Status::FromErrorStringWithFormat(
+              "unable to evaluate argument %d", i);
           return false;
         }
 
@@ -1478,7 +1483,7 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
           size_t dataSize = 0;
 
           bool Success = execution_unit.GetAllocSize(addr, dataSize);
-          (void)Success;
+          UNUSED_IF_ASSERT_DISABLED(Success);
           assert(Success &&
                  "unable to locate host data for transfer to device");
           // Create the required buffer
@@ -1516,27 +1521,26 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
       // Check if the plan is valid
       lldb_private::StreamString ss;
       if (!call_plan_sp || !call_plan_sp->ValidatePlan(&ss)) {
-        error.SetErrorToGenericError();
-        error.SetErrorStringWithFormat(
+        error = lldb_private::Status::FromErrorStringWithFormat(
             "unable to make ThreadPlanCallFunctionUsingABI for 0x%llx",
             I.ULongLong());
         return false;
       }
 
-      exe_ctx.GetProcessPtr()->SetRunningUserExpression(true);
+      process->SetRunningUserExpression(true);
 
       // Execute the actual function call thread plan
-      lldb::ExpressionResults res = exe_ctx.GetProcessRef().RunThreadPlan(
-          exe_ctx, call_plan_sp, options, diagnostics);
+      lldb::ExpressionResults res =
+          process->RunThreadPlan(exe_ctx, call_plan_sp, options, diagnostics);
 
       // Check that the thread plan completed successfully
       if (res != lldb::ExpressionResults::eExpressionCompleted) {
-        error.SetErrorToGenericError();
-        error.SetErrorString("ThreadPlanCallFunctionUsingABI failed");
+        error = lldb_private::Status::FromErrorString(
+            "ThreadPlanCallFunctionUsingABI failed");
         return false;
       }
 
-      exe_ctx.GetProcessPtr()->SetRunningUserExpression(false);
+      process->SetRunningUserExpression(false);
 
       // Void return type
       if (returnType->isVoidTy()) {
@@ -1552,8 +1556,8 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
 
         // Check if the return value is valid
         if (vobj == nullptr || !retVal) {
-          error.SetErrorToGenericError();
-          error.SetErrorString("unable to get the return value");
+          error = lldb_private::Status::FromErrorString(
+              "unable to get the return value");
           return false;
         }
 
@@ -1570,11 +1574,5 @@ bool IRInterpreter::Interpret(llvm::Module &module, llvm::Function &function,
     ++frame.m_ii;
   }
 
-  if (num_insts >= 4096) {
-    error.SetErrorToGenericError();
-    error.SetErrorString(infinite_loop_error);
-    return false;
-  }
-
   return false;
 }
