@@ -9,10 +9,10 @@
 #include "lldb/Expression/REPL.h"
 #include "lldb/Core/Debugger.h"
 #include "lldb/Core/PluginManager.h"
-#include "lldb/Core/StreamFile.h"
 #include "lldb/Expression/ExpressionVariable.h"
 #include "lldb/Expression/UserExpression.h"
 #include "lldb/Host/HostInfo.h"
+#include "lldb/Host/StreamFile.h"
 #include "lldb/Interpreter/CommandInterpreter.h"
 #include "lldb/Interpreter/CommandReturnObject.h"
 #include "lldb/Target/Thread.h"
@@ -359,12 +359,9 @@ void REPL::IOHandlerInputComplete(IOHandler &io_handler, std::string &code) {
 
       const char *expr_prefix = nullptr;
       lldb::ValueObjectSP result_valobj_sp;
+      lldb::ExpressionResults execution_results = UserExpression::Evaluate(
+          exe_ctx, expr_options, code.c_str(), expr_prefix, result_valobj_sp);
       Status error;
-      lldb::ExpressionResults execution_results =
-          UserExpression::Evaluate(exe_ctx, expr_options, code.c_str(),
-                                   expr_prefix, result_valobj_sp, error,
-                                   nullptr); // fixed expression
-
       if (llvm::Error err = OnExpressionEvaluated(exe_ctx, code, expr_options,
                                                   execution_results,
                                                   result_valobj_sp, error)) {
@@ -406,7 +403,26 @@ void REPL::IOHandlerInputComplete(IOHandler &io_handler, std::string &code) {
             add_to_code = false;
             [[fallthrough]];
           case lldb::eExpressionDiscarded:
-            error_sp->Printf("%s\n", error.AsCString());
+            // FIXME: BEGIN SWIFT
+            if (error.Success() && result_valobj_sp) {
+              // The color detection in RenderDiagnosticDetails doesn't work
+              // with error_sp.
+              StreamString diag_stream(useColors);
+              std::vector<DiagnosticDetail> diags;
+              llvm::Error error = result_valobj_sp->GetError().ToError();
+              error = llvm::handleErrors(
+                  std::move(error),
+                  [&](DiagnosticError &error) { diags = error.GetDetails(); });
+              // FIXME: Only correct for the first 999 lines.
+              unsigned prompt_len =
+                  llvm::StringRef(io_handler.GetPrompt()).size() + 3;
+              RenderDiagnosticDetails(diag_stream, prompt_len, true, diags);
+              *error_sp << diag_stream.GetString();
+              if (error)
+                *error_sp << toString(std::move(error));
+            } else
+              // END SWIFT
+              error_sp->Printf("%s\n", error.AsCString());
             break;
 
           case lldb::eExpressionCompleted:
@@ -517,7 +533,7 @@ void REPL::IOHandlerInputComplete(IOHandler &io_handler, std::string &code) {
 void REPL::IOHandlerComplete(IOHandler &io_handler,
                              CompletionRequest &request) {
   // Complete an LLDB command if the first character is a colon...
-  if (request.GetRawLine().startswith(":")) {
+  if (request.GetRawLine().starts_with(":")) {
     Debugger &debugger = m_target.GetDebugger();
 
     // auto complete LLDB commands
@@ -548,17 +564,15 @@ void REPL::IOHandlerComplete(IOHandler &io_handler,
   current_code.append(m_code.CopyList());
 
   IOHandlerEditline &editline = static_cast<IOHandlerEditline &>(io_handler);
-  const StringList *current_lines = editline.GetCurrentLines();
-  if (current_lines) {
-    const uint32_t current_line_idx = editline.GetCurrentLineIndex();
+  StringList current_lines = editline.GetCurrentLines();
+  const uint32_t current_line_idx = editline.GetCurrentLineIndex();
 
-    if (current_line_idx < current_lines->GetSize()) {
-      for (uint32_t i = 0; i < current_line_idx; ++i) {
-        const char *line_cstr = current_lines->GetStringAtIndex(i);
-        if (line_cstr) {
-          current_code.append("\n");
-          current_code.append(line_cstr);
-        }
+  if (current_line_idx < current_lines.GetSize()) {
+    for (uint32_t i = 0; i < current_line_idx; ++i) {
+      const char *line_cstr = current_lines.GetStringAtIndex(i);
+      if (line_cstr) {
+        current_code.append("\n");
+        current_code.append(line_cstr);
       }
     }
   }
