@@ -208,21 +208,23 @@ LaunchInNewTerminalWithAppleScript(const char *exe_path,
   Status error;
   char unix_socket_name[PATH_MAX] = "/tmp/XXXXXX";
   if (::mktemp(unix_socket_name) == NULL) {
-    error.SetErrorString("failed to make temporary path for a unix socket");
+    error = Status::FromErrorString(
+        "failed to make temporary path for a unix socket");
     return error;
   }
 
   StreamString command;
   FileSpec darwin_debug_file_spec = HostInfo::GetSupportExeDir();
   if (!darwin_debug_file_spec) {
-    error.SetErrorString("can't locate the 'darwin-debug' executable");
+    error =
+        Status::FromErrorString("can't locate the 'darwin-debug' executable");
     return error;
   }
 
   darwin_debug_file_spec.SetFilename("darwin-debug");
 
   if (!FileSystem::Instance().Exists(darwin_debug_file_spec)) {
-    error.SetErrorStringWithFormat(
+    error = Status::FromErrorStringWithFormat(
         "the 'darwin-debug' executable doesn't exists at '%s'",
         darwin_debug_file_spec.GetPath().c_str());
     return error;
@@ -314,7 +316,7 @@ LaunchInNewTerminalWithAppleScript(const char *exe_path,
       unix_socket_name, [&] { return AcceptPIDFromInferior(connect_url); });
 
   if (!accept_thread)
-    return Status(accept_thread.takeError());
+    return Status::FromError(accept_thread.takeError());
 
   [applescript executeAndReturnError:nil];
 
@@ -450,6 +452,38 @@ llvm::Error Host::OpenFileInExternalEditor(llvm::StringRef editor,
 #endif // TARGET_OS_OSX
 }
 
+llvm::Error Host::OpenURL(llvm::StringRef url) {
+#if !TARGET_OS_OSX
+  return llvm::errorCodeToError(
+      std::error_code(ENOTSUP, std::system_category()));
+#else  // !TARGET_OS_OSX
+  if (url.empty())
+    return llvm::createStringError("Cannot open empty URL.");
+
+  LLDB_LOG(GetLog(LLDBLog::Host), "Opening URL: {0}", url);
+
+  CFCString url_cfstr(url.data(), kCFStringEncodingUTF8);
+  CFCReleaser<CFURLRef> cfurl = ::CFURLCreateWithString(
+      /*allocator=*/NULL,
+      /*URLString*/ url_cfstr.get(),
+      /*baseURL=*/NULL);
+
+  if (!cfurl.get())
+    return llvm::createStringError(
+        llvm::formatv("could not create CFURL from URL \"{0}\"", url));
+
+  OSStatus error = ::LSOpenCFURLRef(
+      /*inURL=*/cfurl.get(),
+      /*outLaunchedURL=*/NULL);
+
+  if (error != noErr)
+    return llvm::createStringError(
+        llvm::formatv("LSOpenCFURLRef failed: error {0:x}", error));
+
+  return llvm::Error::success();
+#endif // TARGET_OS_OSX
+}
+
 bool Host::IsInteractiveGraphicSession() {
 #if !TARGET_OS_OSX
   return false;
@@ -783,8 +817,8 @@ static Status getXPCAuthorization(ProcessLaunchInfo &launch_info) {
         AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment,
                             kAuthorizationFlagDefaults, &authorizationRef);
     if (createStatus != errAuthorizationSuccess) {
-      error.SetError(1, eErrorTypeGeneric);
-      error.SetErrorString("Can't create authorizationRef.");
+      error = Status(1, eErrorTypeGeneric);
+      error = Status::FromErrorString("Can't create authorizationRef.");
       LLDB_LOG(log, "error: {0}", error);
       return error;
     }
@@ -832,11 +866,11 @@ static Status getXPCAuthorization(ProcessLaunchInfo &launch_info) {
     if (copyRightStatus != errAuthorizationSuccess) {
       // Eventually when the commandline supports running as root and the user
       // is not
-      // logged in in the current audit session, we will need the trick in gdb
+      // logged in to the current audit session, we will need the trick in gdb
       // where
       // we ask the user to type in the root passwd in the terminal.
-      error.SetError(2, eErrorTypeGeneric);
-      error.SetErrorStringWithFormat(
+      error = Status(2, eErrorTypeGeneric);
+      error = Status::FromErrorStringWithFormat(
           "Launching as root needs root authorization.");
       LLDB_LOG(log, "error: {0}", error);
 
@@ -908,16 +942,17 @@ static Status LaunchProcessXPC(const char *exe_path,
         errAuthorizationSuccess) {
       send_auth = true;
     } else {
-      error.SetError(3, eErrorTypeGeneric);
-      error.SetErrorStringWithFormat("Launching root via XPC needs to "
-                                     "externalize authorization reference.");
+      error = Status(3, eErrorTypeGeneric);
+      error = Status::FromErrorStringWithFormat(
+          "Launching root via XPC needs to "
+          "externalize authorization reference.");
       LLDB_LOG(log, "error: {0}", error);
       return error;
     }
     xpc_service = LaunchUsingXPCRightName;
   } else {
-    error.SetError(4, eErrorTypeGeneric);
-    error.SetErrorStringWithFormat(
+    error = Status(4, eErrorTypeGeneric);
+    error = Status::FromErrorStringWithFormat(
         "Launching via XPC is only currently available for root.");
     LLDB_LOG(log, "error: {0}", error);
     return error;
@@ -999,8 +1034,8 @@ static Status LaunchProcessXPC(const char *exe_path,
       int errorCode =
           xpc_dictionary_get_int64(reply, LauncherXPCServiceCodeTypeKey);
 
-      error.SetError(errorCode, eErrorTypeGeneric);
-      error.SetErrorStringWithFormat(
+      error = Status(errorCode, eErrorTypeGeneric);
+      error = Status::FromErrorStringWithFormat(
           "Problems with launching via XPC. Error type : %i, code : %i",
           errorType, errorCode);
       LLDB_LOG(log, "error: {0}", error);
@@ -1011,8 +1046,8 @@ static Status LaunchProcessXPC(const char *exe_path,
       }
     }
   } else if (returnType == XPC_TYPE_ERROR) {
-    error.SetError(5, eErrorTypeGeneric);
-    error.SetErrorStringWithFormat(
+    error = Status(5, eErrorTypeGeneric);
+    error = Status::FromErrorStringWithFormat(
         "Problems with launching via XPC. XPC error : %s",
         xpc_dictionary_get_string(reply, XPC_ERROR_KEY_DESCRIPTION));
     LLDB_LOG(log, "error: {0}", error);
@@ -1040,10 +1075,10 @@ static bool AddPosixSpawnFileAction(void *_file_actions, const FileAction *info,
 
   case FileAction::eFileActionClose:
     if (info->GetFD() == -1)
-      error.SetErrorString(
+      error = Status::FromErrorString(
           "invalid fd for posix_spawn_file_actions_addclose(...)");
     else {
-      error.SetError(
+      error = Status(
           ::posix_spawn_file_actions_addclose(file_actions, info->GetFD()),
           eErrorTypePOSIX);
       if (error.Fail())
@@ -1056,16 +1091,16 @@ static bool AddPosixSpawnFileAction(void *_file_actions, const FileAction *info,
 
   case FileAction::eFileActionDuplicate:
     if (info->GetFD() == -1)
-      error.SetErrorString(
+      error = Status::FromErrorString(
           "invalid fd for posix_spawn_file_actions_adddup2(...)");
     else if (info->GetActionArgument() == -1)
-      error.SetErrorString(
+      error = Status::FromErrorString(
           "invalid duplicate fd for posix_spawn_file_actions_adddup2(...)");
     else {
-      error.SetError(
-          ::posix_spawn_file_actions_adddup2(file_actions, info->GetFD(),
-                                             info->GetActionArgument()),
-          eErrorTypePOSIX);
+      error =
+          Status(::posix_spawn_file_actions_adddup2(file_actions, info->GetFD(),
+                                                    info->GetActionArgument()),
+                 eErrorTypePOSIX);
       if (error.Fail())
         LLDB_LOG(log,
                  "error: {0}, posix_spawn_file_actions_adddup2 "
@@ -1076,7 +1111,7 @@ static bool AddPosixSpawnFileAction(void *_file_actions, const FileAction *info,
 
   case FileAction::eFileActionOpen:
     if (info->GetFD() == -1)
-      error.SetErrorString(
+      error = Status::FromErrorString(
           "invalid fd in posix_spawn_file_actions_addopen(...)");
     else {
       int oflag = info->GetActionArgument();
@@ -1086,7 +1121,7 @@ static bool AddPosixSpawnFileAction(void *_file_actions, const FileAction *info,
       if (oflag & O_CREAT)
         mode = 0640;
 
-      error.SetError(::posix_spawn_file_actions_addopen(
+      error = Status(::posix_spawn_file_actions_addopen(
                          file_actions, info->GetFD(),
                          info->GetPath().str().c_str(), oflag, mode),
                      eErrorTypePOSIX);
@@ -1109,7 +1144,7 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
   Log *log(GetLog(LLDBLog::Host | LLDBLog::Process));
 
   posix_spawnattr_t attr;
-  error.SetError(::posix_spawnattr_init(&attr), eErrorTypePOSIX);
+  error = Status(::posix_spawnattr_init(&attr), eErrorTypePOSIX);
 
   if (error.Fail()) {
     LLDB_LOG(log, "error: {0}, ::posix_spawnattr_init ( &attr )", error);
@@ -1129,7 +1164,7 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
 
   short flags = GetPosixspawnFlags(launch_info);
 
-  error.SetError(::posix_spawnattr_setflags(&attr, flags), eErrorTypePOSIX);
+  error = Status(::posix_spawnattr_setflags(&attr, flags), eErrorTypePOSIX);
   if (error.Fail()) {
     LLDB_LOG(log,
              "error: {0}, ::posix_spawnattr_setflags ( &attr, flags={1:x} )",
@@ -1153,7 +1188,7 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
   //  its parent.
   if (is_graphical && launch_info.GetFlags().Test(eLaunchFlagDebug) &&
       !launch_info.GetFlags().Test(eLaunchFlagInheritTCCFromParent)) {
-    error.SetError(setup_posix_spawn_responsible_flag(&attr), eErrorTypePOSIX);
+    error = Status(setup_posix_spawn_responsible_flag(&attr), eErrorTypePOSIX);
     if (error.Fail()) {
       LLDB_LOG(log, "error: {0}, setup_posix_spawn_responsible_flag(&attr)",
                error);
@@ -1184,7 +1219,7 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
           (posix_spawnattr_setarchpref_np_t)dlsym(
               RTLD_DEFAULT, "posix_spawnattr_setarchpref_np");
       if (set_cpu_subtype && posix_spawnattr_setarchpref_np_fn) {
-        error.SetError((*posix_spawnattr_setarchpref_np_fn)(
+        error = Status((*posix_spawnattr_setarchpref_np_fn)(
                            &attr, 1, &cpu_type, &cpu_subtype, &ocount),
                        eErrorTypePOSIX);
         if (error.Fail())
@@ -1196,7 +1231,7 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
         if (error.Fail() || ocount != 1)
           return error;
       } else {
-        error.SetError(
+        error = Status(
             ::posix_spawnattr_setbinpref_np(&attr, 1, &cpu_type, &ocount),
             eErrorTypePOSIX);
         if (error.Fail())
@@ -1230,15 +1265,16 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
     std::string working_dir_path = working_dir.GetPath();
     if (__pthread_chdir(working_dir_path.c_str()) < 0) {
       if (errno == ENOENT) {
-        error.SetErrorStringWithFormat("No such file or directory: %s",
-                                       working_dir_path.c_str());
+        error = Status::FromErrorStringWithFormat(
+            "No such file or directory: %s", working_dir_path.c_str());
       } else if (errno == ENOTDIR) {
-        error.SetErrorStringWithFormat("Path doesn't name a directory: %s",
-                                       working_dir_path.c_str());
+        error = Status::FromErrorStringWithFormat(
+            "Path doesn't name a directory: %s", working_dir_path.c_str());
       } else {
-        error.SetErrorStringWithFormat("An unknown error occurred when "
-                                       "changing directory for process "
-                                       "execution.");
+        error =
+            Status::FromErrorStringWithFormat("An unknown error occurred when "
+                                              "changing directory for process "
+                                              "execution.");
       }
       return error;
     }
@@ -1248,8 +1284,8 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
   const size_t num_file_actions = launch_info.GetNumFileActions();
   if (num_file_actions > 0) {
     posix_spawn_file_actions_t file_actions;
-    error.SetError(::posix_spawn_file_actions_init(&file_actions),
-                   eErrorTypePOSIX);
+    error =
+        Status(::posix_spawn_file_actions_init(&file_actions), eErrorTypePOSIX);
     if (error.Fail()) {
       LLDB_LOG(log,
                "error: {0}, ::posix_spawn_file_actions_init ( &file_actions )",
@@ -1271,7 +1307,7 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
       }
     }
 
-    error.SetError(
+    error = Status(
         ::posix_spawnp(&result_pid, exe_path, &file_actions, &attr, argv, envp),
         eErrorTypePOSIX);
 
@@ -1289,9 +1325,9 @@ static Status LaunchProcessPosixSpawn(const char *exe_path,
     }
 
   } else {
-    error.SetError(
-        ::posix_spawnp(&result_pid, exe_path, NULL, &attr, argv, envp),
-        eErrorTypePOSIX);
+    error =
+        Status(::posix_spawnp(&result_pid, exe_path, NULL, &attr, argv, envp),
+               eErrorTypePOSIX);
 
     if (error.Fail()) {
       LLDB_LOG(log,
@@ -1343,8 +1379,8 @@ Status Host::LaunchProcess(ProcessLaunchInfo &launch_info) {
     FileSystem::Instance().ResolveExecutableLocation(exe_spec);
 
   if (!fs.Exists(exe_spec)) {
-    error.SetErrorStringWithFormatv("executable doesn't exist: '{0}'",
-                                    exe_spec);
+    error = Status::FromErrorStringWithFormatv(
+        "executable doesn't exist: '{0}'", exe_spec);
     return error;
   }
 
@@ -1353,8 +1389,9 @@ Status Host::LaunchProcess(ProcessLaunchInfo &launch_info) {
     return LaunchInNewTerminalWithAppleScript(exe_spec.GetPath().c_str(),
                                               launch_info);
 #else
-    error.SetErrorString("launching a process in a new terminal is not "
-                         "supported on iOS devices");
+    error =
+        Status::FromErrorString("launching a process in a new terminal is not "
+                                "supported on iOS devices");
     return error;
 #endif
   }
@@ -1379,7 +1416,8 @@ Status Host::LaunchProcess(ProcessLaunchInfo &launch_info) {
   } else {
     // Invalid process ID, something didn't go well
     if (error.Success())
-      error.SetErrorString("process launch failed for unknown reasons");
+      error =
+          Status::FromErrorString("process launch failed for unknown reasons");
   }
   return error;
 }
@@ -1387,18 +1425,32 @@ Status Host::LaunchProcess(ProcessLaunchInfo &launch_info) {
 Status Host::ShellExpandArguments(ProcessLaunchInfo &launch_info) {
   Status error;
   if (launch_info.GetFlags().Test(eLaunchFlagShellExpandArguments)) {
-    FileSpec expand_tool_spec = HostInfo::GetSupportExeDir();
-    if (!expand_tool_spec) {
-      error.SetErrorString(
-          "could not get support executable directory for lldb-argdumper tool");
-      return error;
+    FileSpec expand_tool_spec;
+    Environment host_env = Host::GetEnvironment();
+    std::string env_argdumper_path = host_env.lookup("LLDB_ARGDUMPER_PATH");
+    if (!env_argdumper_path.empty()) {
+      expand_tool_spec.SetFile(env_argdumper_path, FileSpec::Style::native);
+      Log *log(GetLog(LLDBLog::Host | LLDBLog::Process));
+      LLDB_LOGF(log,
+                "lldb-argdumper exe path set from environment variable: %s",
+                env_argdumper_path.c_str());
     }
-    expand_tool_spec.AppendPathComponent("lldb-argdumper");
-    if (!FileSystem::Instance().Exists(expand_tool_spec)) {
-      error.SetErrorStringWithFormat(
-          "could not find the lldb-argdumper tool: %s",
-          expand_tool_spec.GetPath().c_str());
-      return error;
+    bool argdumper_exists = FileSystem::Instance().Exists(env_argdumper_path);
+    if (!argdumper_exists) {
+      expand_tool_spec = HostInfo::GetSupportExeDir();
+      if (!expand_tool_spec) {
+        error = Status::FromErrorString(
+            "could not get support executable directory for "
+            "lldb-argdumper tool");
+        return error;
+      }
+      expand_tool_spec.AppendPathComponent("lldb-argdumper");
+      if (!FileSystem::Instance().Exists(expand_tool_spec)) {
+        error = Status::FromErrorStringWithFormat(
+            "could not find the lldb-argdumper tool: %s",
+            expand_tool_spec.GetPath().c_str());
+        return error;
+      }
     }
 
     StreamString expand_tool_spec_stream;
@@ -1414,7 +1466,7 @@ Status Host::ShellExpandArguments(ProcessLaunchInfo &launch_info) {
     if (!FileSystem::Instance().Exists(cwd)) {
       char *wd = getcwd(nullptr, 0);
       if (wd == nullptr) {
-        error.SetErrorStringWithFormat(
+        error = Status::FromErrorStringWithFormat(
             "cwd does not exist; cannot launch with shell argument expansion");
         return error;
       } else {
@@ -1433,32 +1485,32 @@ Status Host::ShellExpandArguments(ProcessLaunchInfo &launch_info) {
       return e;
 
     if (status != 0) {
-      error.SetErrorStringWithFormat("lldb-argdumper exited with error %d",
-                                     status);
+      error = Status::FromErrorStringWithFormat(
+          "lldb-argdumper exited with error %d", status);
       return error;
     }
 
     auto data_sp = StructuredData::ParseJSON(output);
     if (!data_sp) {
-      error.SetErrorString("invalid JSON");
+      error = Status::FromErrorString("invalid JSON");
       return error;
     }
 
     auto dict_sp = data_sp->GetAsDictionary();
     if (!data_sp) {
-      error.SetErrorString("invalid JSON");
+      error = Status::FromErrorString("invalid JSON");
       return error;
     }
 
     auto args_sp = dict_sp->GetObjectForDotSeparatedPath("arguments");
     if (!args_sp) {
-      error.SetErrorString("invalid JSON");
+      error = Status::FromErrorString("invalid JSON");
       return error;
     }
 
     auto args_array_sp = args_sp->GetAsArray();
     if (!args_array_sp) {
-      error.SetErrorString("invalid JSON");
+      error = Status::FromErrorString("invalid JSON");
       return error;
     }
 
