@@ -103,8 +103,8 @@ Status OptionGroupValueObjectDisplay::SetOptionValue(
   case 'D':
     if (option_arg.getAsInteger(0, max_depth)) {
       max_depth = UINT32_MAX;
-      error.SetErrorStringWithFormat("invalid max depth '%s'",
-                                     option_arg.str().c_str());
+      error = Status::FromErrorStringWithFormat("invalid max depth '%s'",
+                                                option_arg.str().c_str());
     } else {
       max_depth_is_default = false;
     }
@@ -113,16 +113,16 @@ Status OptionGroupValueObjectDisplay::SetOptionValue(
   case 'Z':
     if (option_arg.getAsInteger(0, elem_count)) {
       elem_count = UINT32_MAX;
-      error.SetErrorStringWithFormat("invalid element count '%s'",
-                                     option_arg.str().c_str());
+      error = Status::FromErrorStringWithFormat("invalid element count '%s'",
+                                                option_arg.str().c_str());
     }
     break;
 
   case 'P':
     if (option_arg.getAsInteger(0, ptr_depth)) {
       ptr_depth = 0;
-      error.SetErrorStringWithFormat("invalid pointer depth '%s'",
-                                     option_arg.str().c_str());
+      error = Status::FromErrorStringWithFormat("invalid pointer depth '%s'",
+                                                option_arg.str().c_str());
     }
     break;
 
@@ -131,23 +131,23 @@ Status OptionGroupValueObjectDisplay::SetOptionValue(
       no_summary_depth = 1;
     else if (option_arg.getAsInteger(0, no_summary_depth)) {
       no_summary_depth = 0;
-      error.SetErrorStringWithFormat("invalid pointer depth '%s'",
-                                     option_arg.str().c_str());
+      error = Status::FromErrorStringWithFormat("invalid pointer depth '%s'",
+                                                option_arg.str().c_str());
     }
     break;
 
   case 'S':
     use_synth = OptionArgParser::ToBoolean(option_arg, true, &success);
     if (!success)
-      error.SetErrorStringWithFormat("invalid synthetic-type '%s'",
-                                     option_arg.str().c_str());
+      error = Status::FromErrorStringWithFormat("invalid synthetic-type '%s'",
+                                                option_arg.str().c_str());
     break;
 
   case 'V':
     run_validator = OptionArgParser::ToBoolean(option_arg, true, &success);
     if (!success)
-      error.SetErrorStringWithFormat("invalid validate '%s'",
-                                     option_arg.str().c_str());
+      error = Status::FromErrorStringWithFormat("invalid validate '%s'",
+                                                option_arg.str().c_str());
     break;
 
   default:
