@@ -13,8 +13,6 @@
 #include "Plugins/LanguageRuntime/ObjC/AppleObjCRuntime/AppleObjCRuntime.h"
 #include "Plugins/TypeSystem/Clang/TypeSystemClang.h"
 #include "lldb/Core/Mangled.h"
-#include "lldb/Core/ValueObject.h"
-#include "lldb/Core/ValueObjectConstResult.h"
 #include "lldb/DataFormatters/FormattersHelpers.h"
 #include "lldb/DataFormatters/StringPrinter.h"
 #include "lldb/DataFormatters/TypeSummary.h"
@@ -27,6 +25,8 @@
 #include "lldb/Utility/LLDBLog.h"
 #include "lldb/Utility/Status.h"
 #include "lldb/Utility/Stream.h"
+#include "lldb/ValueObject/ValueObject.h"
+#include "lldb/ValueObject/ValueObjectConstResult.h"
 
 #include "llvm/ADT/APInt.h"
 #include "llvm/ADT/bit.h"
@@ -152,7 +152,7 @@ bool lldb_private::formatters::NSTimeZoneSummaryProvider(
     }
 
     ValueObject &time_zone = dyn_valobj_sp ? *dyn_valobj_sp : valobj;
-    llvm::ArrayRef<llvm::StringRef> identifier_path = {
+    llvm::SmallVector<llvm::StringRef> identifier_path = {
         "some", "timeZone", "_timeZone", "some", "identifier"};
     if (auto identifier_sp = time_zone.GetChildAtNamePath(identifier_path)) {
       std::string desc;
@@ -832,7 +832,7 @@ bool lldb_private::formatters::NSURLSummaryProvider(
 
   llvm::StringRef class_name = descriptor->GetClassName().GetStringRef();
 
-  if (!class_name.equals("NSURL"))
+  if (class_name != "NSURL")
     return false;
 
   uint64_t offset_text = ptr_size + ptr_size +
