@@ -12,20 +12,26 @@
 
 #include "SwiftFormatters.h"
 #include "Plugins/Language/Swift/SwiftStringIndex.h"
+#include "Plugins/LanguageRuntime/Swift/ReflectionContextInterface.h"
 #include "Plugins/LanguageRuntime/Swift/SwiftLanguageRuntime.h"
 #include "Plugins/TypeSystem/Clang/TypeSystemClang.h"
-#include "lldb/Core/ValueObject.h"
 #include "lldb/DataFormatters/FormattersHelpers.h"
 #include "lldb/DataFormatters/StringPrinter.h"
 #include "lldb/Target/Process.h"
 #include "lldb/Utility/ConstString.h"
 #include "lldb/Utility/DataBufferHeap.h"
+#include "lldb/Utility/LLDBLog.h"
+#include "lldb/Utility/Log.h"
 #include "lldb/Utility/Status.h"
 #include "lldb/Utility/Timer.h"
+#include "lldb/ValueObject/ValueObject.h"
 #include "lldb/lldb-enumerations.h"
 #include "swift/AST/Types.h"
 #include "swift/Demangling/ManglingMacros.h"
+#include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/StringRef.h"
+#include "llvm/Support/FormatAdapters.h"
+#include "llvm/Support/raw_ostream.h"
 #include <optional>
 
 // FIXME: we should not need this
@@ -205,11 +211,11 @@ static bool makeStringGutsSummary(
     llvm::StringRef variantCase = variant_sp->GetValueAsCString();
 
     ValueObjectSP payload_sp;
-    if (variantCase.startswith("immortal")) {
+    if (variantCase.starts_with("immortal")) {
       payload_sp = variant_sp->GetChildAtNamePath({g_immortal, g__value});
-    } else if (variantCase.startswith("native")) {
+    } else if (variantCase.starts_with("native")) {
       payload_sp = variant_sp->GetChildAtNamePath({g_immortal, g__value});
-    } else if (variantCase.startswith("bridged")) {
+    } else if (variantCase.starts_with("bridged")) {
       static ConstString g_bridged("bridged");
       auto anyobject_sp = variant_sp->GetChildMemberWithName(g_bridged, true);
       if (!anyobject_sp)
@@ -251,6 +257,8 @@ static bool makeStringGutsSummary(
   │ Immortal, Small     ║  1  │ASCII│  1  │  0  │
   ├─────────────────────╫─────┼─────┼─────┼─────┤
   │ Immortal, Large     ║  1  │  0  │  0  │  0  │
+  ├─────────────────────╫─────┼─────┼─────┼─────┤
+  │ Immortal, Bridged   ║  1  │  1  │  0  │  0  │
   ╞═════════════════════╬═════╪═════╪═════╪═════╡
   │ Native              ║  0  │  0  │  0  │  0  │
   ├─────────────────────╫─────┼─────┼─────┼─────┤
@@ -367,7 +375,7 @@ static bool makeStringGutsSummary(
   if ((discriminator & 0b0111'0000) == 0)
     return error("unexpected discriminator");
 
-  if ((discriminator & 0b1110'0000) == 0b0100'0000) { // 010xxxxx: Bridged
+  if ((discriminator & 0b0110'0000) == 0b0100'0000) { // x10xxxxx: Bridged
     TypeSystemClangSP clang_ts_sp =
         ScratchTypeSystemClang::GetForTarget(process->GetTarget());
     if (!clang_ts_sp)
@@ -721,6 +729,7 @@ bool lldb_private::formatters::swift::BuiltinObjC_SummaryProvider(
 namespace lldb_private {
 namespace formatters {
 namespace swift {
+
 class EnumSyntheticFrontEnd : public SyntheticChildrenFrontEnd {
 public:
   EnumSyntheticFrontEnd(lldb::ValueObjectSP valobj_sp);
@@ -736,6 +745,114 @@ private:
   ConstString m_element_name;
   size_t m_child_index;
 };
+
+/// Synthetic provider for `Swift.Task`.
+///
+/// As seen by lldb, a `Task` instance is an opaque pointer, with neither type
+/// metadata nor an AST to describe it. To implement this synthetic provider, a
+/// `Task`'s state is retrieved from a `ReflectionContext`, and that data is
+/// used to manually construct `ValueObject` children.
+class TaskSyntheticFrontEnd : public SyntheticChildrenFrontEnd {
+public:
+  TaskSyntheticFrontEnd(lldb::ValueObjectSP valobj_sp)
+      : SyntheticChildrenFrontEnd(*valobj_sp.get()) {}
+
+  constexpr static StringLiteral TaskChildren[] = {
+      "isChildTask",    "isFuture",    "isGroupChildTask",
+      "isAsyncLetTask", "isCancelled", "isStatusRecordLocked",
+      "isEscalated",    "isEnqueued",  "isRunning",
+  };
+
+  llvm::Expected<uint32_t> CalculateNumChildren() override {
+    auto count = ArrayRef(TaskChildren).size();
+    return m_task_info.hasIsRunning ? count : count - 1;
+  }
+
+  lldb::ValueObjectSP GetChildAtIndex(uint32_t idx) override {
+    auto target_sp = m_backend.GetTargetSP();
+#define RETURN_CHILD(FIELD, NAME)                                              \
+  if (!FIELD)                                                                  \
+    FIELD = ValueObject::CreateValueObjectFromBool(target_sp,                  \
+                                                   m_task_info.NAME, #NAME);   \
+  return FIELD
+
+    switch (idx) {
+    case 0:
+      RETURN_CHILD(m_is_child_task_sp, isChildTask);
+    case 1:
+      RETURN_CHILD(m_is_future_sp, isFuture);
+    case 2:
+      RETURN_CHILD(m_is_group_child_task_sp, isGroupChildTask);
+    case 3:
+      RETURN_CHILD(m_is_async_let_task_sp, isAsyncLetTask);
+    case 4:
+      RETURN_CHILD(m_is_cancelled_sp, isCancelled);
+    case 5:
+      RETURN_CHILD(m_is_status_record_locked_sp, isStatusRecordLocked);
+    case 6:
+      RETURN_CHILD(m_is_escalated_sp, isEscalated);
+    case 7:
+      RETURN_CHILD(m_is_enqueued_sp, isEnqueued);
+    case 8:
+      RETURN_CHILD(m_is_running_sp, isRunning);
+    default:
+      return {};
+    }
+
+#undef RETURN_CHILD
+  }
+
+  lldb::ChildCacheState Update() override {
+    if (auto *runtime = SwiftLanguageRuntime::Get(m_backend.GetProcessSP())) {
+      ThreadSafeReflectionContext reflection_ctx =
+          runtime->GetReflectionContext();
+      ValueObjectSP task_obj_sp = m_backend.GetChildMemberWithName("_task");
+      if (!task_obj_sp)
+        return ChildCacheState::eRefetch;
+      uint64_t task_ptr = task_obj_sp->GetValueAsUnsigned(LLDB_INVALID_ADDRESS);
+      if (task_ptr != LLDB_INVALID_ADDRESS) {
+        llvm::Expected<ReflectionContextInterface::AsyncTaskInfo> task_info =
+            reflection_ctx->asyncTaskInfo(task_ptr);
+        if (auto err = task_info.takeError()) {
+          LLDB_LOG(GetLog(LLDBLog::DataFormatters | LLDBLog::Types),
+                   "could not get info for async task {0:x}: {1}", task_ptr,
+                   fmt_consume(std::move(err)));
+        } else {
+          m_task_info = *task_info;
+          for (auto child :
+               {m_is_child_task_sp, m_is_future_sp, m_is_group_child_task_sp,
+                m_is_async_let_task_sp, m_is_cancelled_sp,
+                m_is_status_record_locked_sp, m_is_escalated_sp,
+                m_is_enqueued_sp, m_is_running_sp})
+            child.reset();
+        }
+      }
+    }
+    return ChildCacheState::eRefetch;
+  }
+
+  bool MightHaveChildren() override { return true; }
+
+  size_t GetIndexOfChildWithName(ConstString name) override {
+    ArrayRef children = TaskChildren;
+    auto it = llvm::find(children, name);
+    if (it == children.end())
+      return UINT32_MAX;
+    return std::distance(children.begin(), it);
+  }
+
+private:
+  ReflectionContextInterface::AsyncTaskInfo m_task_info;
+  ValueObjectSP m_is_child_task_sp;
+  ValueObjectSP m_is_future_sp;
+  ValueObjectSP m_is_group_child_task_sp;
+  ValueObjectSP m_is_async_let_task_sp;
+  ValueObjectSP m_is_cancelled_sp;
+  ValueObjectSP m_is_status_record_locked_sp;
+  ValueObjectSP m_is_escalated_sp;
+  ValueObjectSP m_is_enqueued_sp;
+  ValueObjectSP m_is_running_sp;
+};
 }
 }
 }
@@ -794,6 +911,14 @@ lldb_private::formatters::swift::EnumSyntheticFrontEndCreator(
   return (new EnumSyntheticFrontEnd(valobj_sp));
 }
 
+SyntheticChildrenFrontEnd *
+lldb_private::formatters::swift::TaskSyntheticFrontEndCreator(
+    CXXSyntheticChildren *, lldb::ValueObjectSP valobj_sp) {
+  if (!valobj_sp)
+    return NULL;
+  return new TaskSyntheticFrontEnd(valobj_sp);
+}
+
 bool lldb_private::formatters::swift::ObjC_Selector_SummaryProvider(
     ValueObject &valobj, Stream &stream, const TypeSummaryOptions &options) {
   LLDB_SCOPED_TIMER();
@@ -1215,27 +1340,27 @@ bool lldb_private::formatters::swift::LegacySIMD_SummaryProvider(
   ConstString full_type_name = valobj.GetTypeName();
 
   llvm::StringRef type_name = full_type_name.GetStringRef();
-  if (type_name.startswith("simd."))
+  if (type_name.starts_with("simd."))
     type_name = type_name.drop_front(5);
-  if (type_name.startswith("simd_"))
+  if (type_name.starts_with("simd_"))
     type_name = type_name.drop_front(5);
 
   // Get the type of object this is.
-  bool is_quaternion = type_name.startswith("quat");
+  bool is_quaternion = type_name.starts_with("quat");
   bool is_matrix = type_name[type_name.size() - 2] == 'x';
   bool is_vector = !is_matrix && !is_quaternion;
 
   // Get the kind of SIMD element inside of this object.
   std::optional<SIMDElementKind> kind = std::nullopt;
-  if (type_name.startswith("int"))
+  if (type_name.starts_with("int"))
     kind = SIMDElementKind::Int32;
-  else if (type_name.startswith("uint"))
+  else if (type_name.starts_with("uint"))
     kind = SIMDElementKind::UInt32;
-  else if ((is_quaternion && type_name.endswith("f")) ||
-           type_name.startswith("float"))
+  else if ((is_quaternion && type_name.ends_with("f")) ||
+           type_name.starts_with("float"))
     kind = SIMDElementKind::Float32;
-  else if ((is_quaternion && type_name.endswith("d")) ||
-           type_name.startswith("double"))
+  else if ((is_quaternion && type_name.ends_with("d")) ||
+           type_name.starts_with("double"))
     kind = SIMDElementKind::Float64;
   if (!kind)
     return false;
@@ -1294,13 +1419,13 @@ bool lldb_private::formatters::swift::GLKit_SummaryProvider(
   // Get the type name without the "GLKit." prefix.
   ConstString full_type_name = valobj.GetTypeName();
   llvm::StringRef type_name = full_type_name.GetStringRef();
-  if (type_name.startswith("GLKit."))
+  if (type_name.starts_with("GLKit."))
     type_name = type_name.drop_front(6);
 
   // Get the type of object this is.
   bool is_quaternion = type_name == "GLKQuaternion";
-  bool is_matrix = type_name.startswith("GLKMatrix");
-  bool is_vector = type_name.startswith("GLKVector");
+  bool is_matrix = type_name.starts_with("GLKMatrix");
+  bool is_vector = type_name.starts_with("GLKVector");
 
   if (!(is_quaternion || is_matrix || is_vector))
     return false;
