@@ -70,11 +70,11 @@ public:
       lldb_private::CompilerDeclContext decl_context) override {}
 
   // FIXME: What should this do?
-  lldb_private::ConstString
+  std::string
   GetDIEClassTemplateParams(const DWARFDIE &die) override {
     assert(false && "DWARFASTParserSwift::GetDIEClassTemplateParams has not "
                     "yet been implemented");
-    return lldb_private::ConstString();
+    return {};
   }
 
   static bool classof(const DWARFASTParser *Parser) {
