@@ -30,77 +30,6 @@
 
 namespace lldb_private {
 
-class ANSIColorStringStream : public llvm::raw_string_ostream {
-public:
-  ANSIColorStringStream(bool colorize)
-      : llvm::raw_string_ostream(m_buffer), m_colorize(colorize) {}
-  /// Changes the foreground color of text that will be output from
-  /// this point forward.
-  ///
-  /// \param colors      ANSI color to use, the special SAVEDCOLOR can
-  ///                    be used to change only the bold attribute, and
-  ///                    keep colors untouched.
-  /// \param bold        bold/brighter text, default false
-  /// \param bg          if true change the background,
-  ///                    default: change foreground
-  /// \returns           itself so it can be used within << invocations.
-  raw_ostream &changeColor(enum Colors colors, bool bold = false,
-                           bool bg = false) override {
-    if (llvm::sys::Process::ColorNeedsFlush())
-      flush();
-    const char *colorcode;
-    if (colors == SAVEDCOLOR)
-      colorcode = llvm::sys::Process::OutputBold(bg);
-    else
-      colorcode =
-          llvm::sys::Process::OutputColor(static_cast<char>(colors), bold, bg);
-    if (colorcode) {
-      size_t len = strlen(colorcode);
-      write(colorcode, len);
-    }
-    return *this;
-  }
-
-  /// Resets the colors to terminal defaults. Call this when you are
-  /// done outputting colored text, or before program exit.
-  raw_ostream &resetColor() override {
-    if (llvm::sys::Process::ColorNeedsFlush())
-      flush();
-    const char *colorcode = llvm::sys::Process::ResetColor();
-    if (colorcode) {
-      size_t len = strlen(colorcode);
-      write(colorcode, len);
-    }
-    return *this;
-  }
-
-  /// Reverses the forground and background colors.
-  raw_ostream &reverseColor() override {
-    if (llvm::sys::Process::ColorNeedsFlush())
-      flush();
-    const char *colorcode = llvm::sys::Process::OutputReverse();
-    if (colorcode) {
-      size_t len = strlen(colorcode);
-      write(colorcode, len);
-    }
-    return *this;
-  }
-
-  /// This function determines if this stream is connected to a "tty"
-  /// or "console" window. That is, the output would be displayed to
-  /// the user rather than being put on a pipe or stored in a file.
-  bool is_displayed() const override { return m_colorize; }
-
-  /// This function determines if this stream is displayed and
-  /// supports colors.
-  bool has_colors() const override { return m_colorize; }
-
-protected:
-  std::string m_buffer;
-  bool m_colorize;
-};
-
-
 class StoringDiagnosticConsumer : public swift::DiagnosticConsumer {
 public:
   StoringDiagnosticConsumer(SwiftASTContext &ast_context)
@@ -116,8 +45,8 @@ public:
   /// Consume a Diagnostic from the Swift compiler.
   void handleDiagnostic(swift::SourceManager &source_mgr,
                         const swift::DiagnosticInfo &info) override {
-    llvm::StringRef bufferName = "<anonymous>";
-    unsigned bufferID = 0;
+    llvm::StringRef buffer_name = "<anonymous>";
+    unsigned buffer_id = 0;
     std::pair<unsigned, unsigned> line_col = {0, 0};
 
     llvm::SmallString<256> text;
@@ -129,37 +58,46 @@ public:
 
     swift::SourceLoc source_loc = info.Loc;
     if (source_loc.isValid()) {
-      bufferID = source_mgr.findBufferContainingLoc(source_loc);
-      bufferName = source_mgr.getDisplayNameForLoc(source_loc);
+      buffer_id = source_mgr.findBufferContainingLoc(source_loc);
+      buffer_name = source_mgr.getDisplayNameForLoc(source_loc);
       line_col = source_mgr.getPresumedLineAndColumnForLoc(source_loc);
     }
 
     bool use_fixits = false;
-    std::string formatted_text;
+    // Determine what kind of diagnostic we're emitting, and whether
+    // we want to use its fixits:
+    lldb::Severity severity;
+    llvm::SourceMgr::DiagKind source_mgr_kind;
+    std::string kind_str;
+    switch (info.Kind) {
+    case swift::DiagnosticKind::Error:
+      severity = lldb::eSeverityError;
+      source_mgr_kind = llvm::SourceMgr::DK_Error;
+      kind_str = "error: ";
+      use_fixits = true;
+      break;
+    case swift::DiagnosticKind::Warning:
+      severity = lldb::eSeverityWarning;
+      source_mgr_kind = llvm::SourceMgr::DK_Warning;
+      kind_str = "warning: ";
+      break;
+    case swift::DiagnosticKind::Note:
+      severity = lldb::eSeverityInfo;
+      source_mgr_kind = llvm::SourceMgr::DK_Note;
+      kind_str = "note: ";
+      break;
+    case swift::DiagnosticKind::Remark:
+      severity = lldb::eSeverityInfo;
+      source_mgr_kind = llvm::SourceMgr::DK_Remark;
+      kind_str = "remark: ";
+      break;
+    }
+
+    std::string formatted_text, raw_text;
     if (!line_col.first) {
-      formatted_text = text.str();
+      use_fixits = false;
+      raw_text = formatted_text = text.str();
     } else {
-      ANSIColorStringStream os(m_colorize);
-
-      // Determine what kind of diagnostic we're emitting, and whether
-      // we want to use its fixits:
-      llvm::SourceMgr::DiagKind source_mgr_kind;
-      switch (info.Kind) {
-      case swift::DiagnosticKind::Error:
-        source_mgr_kind = llvm::SourceMgr::DK_Error;
-        use_fixits = true;
-        break;
-      case swift::DiagnosticKind::Warning:
-        source_mgr_kind = llvm::SourceMgr::DK_Warning;
-        break;
-      case swift::DiagnosticKind::Note:
-        source_mgr_kind = llvm::SourceMgr::DK_Note;
-        break;
-      case swift::DiagnosticKind::Remark:
-        source_mgr_kind = llvm::SourceMgr::DK_Remark;
-        break;
-      }
-
       // Swift may insert note diagnostics after an error diagnostic with fixits
       // related to that error. Check if the latest inserted diagnostic is an
       // error one, and that the diagnostic being processed is a note one that
@@ -171,11 +109,12 @@ public:
       if (info.Kind == swift::DiagnosticKind::Note &&
           !m_raw_swift_diagnostics.empty()) {
         auto &last_diagnostic = m_raw_swift_diagnostics.back();
-        if (last_diagnostic.kind == swift::DiagnosticKind::Error &&
+        if (last_diagnostic.detail.severity == lldb::eSeverityError &&
             last_diagnostic.fixits.empty() &&
-            last_diagnostic.bufferID == bufferID &&
-            last_diagnostic.column == line_col.second &&
-            last_diagnostic.line == line_col.first)
+            last_diagnostic.buffer_id == buffer_id &&
+            last_diagnostic.detail.source_location &&
+            last_diagnostic.detail.source_location->column == line_col.second &&
+            last_diagnostic.detail.source_location->line == line_col.first)
           last_diagnostic.fixits.insert(last_diagnostic.fixits.end(),
                                         info.FixIts.begin(), info.FixIts.end());
       }
@@ -193,11 +132,14 @@ public:
       // Display the diagnostic.
       auto message = source_mgr.GetMessage(source_loc, source_mgr_kind, text,
                                            ranges, fix_its);
-      source_mgr.getLLVMSourceMgr().PrintMessage(os, message);
-
-      // str() implicitly flushes the stram.
-      std::string &s = os.str();
-      formatted_text = !s.empty() ? std::move(s) : std::string(text);
+      {
+        llvm::raw_string_ostream os(formatted_text);
+        source_mgr.getLLVMSourceMgr().PrintMessage(os, message);
+      }
+      size_t kind_pos = formatted_text.find(kind_str);
+      formatted_text = formatted_text.substr(0, kind_pos) +
+                       formatted_text.substr(kind_pos + kind_str.size());
+      raw_text = text.str();
     }
     if (info.Kind == swift::DiagnosticKind::Remark) {
       if (info.ID == swift::diag::module_loaded.ID) {
@@ -207,14 +149,29 @@ public:
       }
       return;
     }
+    // FXIME: This is a heuristic.
+    uint16_t len = info.Ranges.size() ? info.Ranges.front().getByteLength() : 0;
+    bool in_user_input = false;
+    // FIXME: improve this!
+    if (buffer_name == "<REPL>" || buffer_name == "<EXPR>" ||
+        buffer_name == "repl.swift" ||
+        buffer_name.starts_with("<user expression"))
+      in_user_input = true;
+    DiagnosticDetail::SourceLocation loc = {FileSpec{buffer_name.str()},
+                                            line_col.first,
+                                            (uint16_t)line_col.second,
+                                            len,
+                                            !in_user_input,
+                                            in_user_input};
+    DiagnosticDetail detail = {loc, severity, raw_text, formatted_text};
     RawDiagnostic diagnostic(
-        formatted_text, info.Kind, bufferName.str(), bufferID, line_col.first,
-        line_col.second,
+        detail, buffer_id,
         use_fixits ? info.FixIts : llvm::ArrayRef<swift::Diagnostic::FixIt>());
     if (info.ID == swift::diag::error_from_clang.ID ||
         info.ID == swift::diag::bridging_header_error.ID) {
       if (m_raw_clang_diagnostics.empty() ||
-          m_raw_clang_diagnostics.back() != diagnostic) {
+          m_raw_clang_diagnostics.back().detail.rendered !=
+              diagnostic.detail.rendered) {
         m_raw_clang_diagnostics.push_back(std::move(diagnostic));
         if (info.Kind == swift::DiagnosticKind::Error) {
           m_num_clang_errors++;
@@ -284,63 +241,44 @@ public:
 
     auto format_diagnostic = [&](const RawDiagnostic &diagnostic,
                                  const DiagnosticOrigin origin) {
-      const lldb::Severity severity = SeverityForKind(diagnostic.kind);
+      const lldb::Severity severity = diagnostic.detail.severity;
+      const auto &diag_loc = diagnostic.detail.source_location;
 
       // Make sure the error line is in range or in another file.
-      if (diagnostic.bufferID == bufferID && !diagnostic.bufferName.empty() &&
-          (diagnostic.line < first_line || diagnostic.line > last_line))
+      if (diagnostic.buffer_id == bufferID && diag_loc && diag_loc->file &&
+          (diag_loc->line < first_line || diag_loc->line > last_line))
         return;
 
       // Heuristic to skip expected expression warnings.
-      if (diagnostic.description.find("$__lldb") != std::string::npos)
+      if (diagnostic.detail.message.find("$__lldb") != std::string::npos)
         return;
 
       // Diagnose global errors.
-      if (severity == lldb::eSeverityError && diagnostic.line == 0) {
-        diagnostic_manager.AddDiagnostic(diagnostic.description.c_str(),
-                                         severity, origin);
+      if (severity == lldb::eSeverityError &&
+          (!diag_loc || diag_loc->line == 0)) {
+        diagnostic_manager.AddDiagnostic(std::make_unique<Diagnostic>(
+            origin, diagnostic.buffer_id, diagnostic.detail));
         added_one_diagnostic = true;
         return;
       }
 
-      // Need to remap the error/warning to a different line.
-      StreamString match;
-      match.Printf("%s:%u:", diagnostic.bufferName.c_str(), diagnostic.line);
-      const size_t match_len = match.GetString().size();
-      size_t match_pos = diagnostic.description.find(match.GetString().str());
-      if (match_pos == std::string::npos)
-        return;
+      DiagnosticDetail detail = diagnostic.detail;
+      if (diag_loc) {
+        // Need to remap the error/warning to a different line.
+        DiagnosticDetail::SourceLocation new_loc;
+        new_loc = *diag_loc;
+        if (new_loc.line)
+          new_loc.line -= first_line - 1;
+        detail.source_location = new_loc;
+      }
 
-      // We have some <file>:<line>:" instances that need to be updated.
-      StreamString fixed_description;
-      size_t start_pos = 0;
-      do {
-        if (match_pos > start_pos)
-          fixed_description.Printf(
-              "%s",
-              diagnostic.description.substr(start_pos, match_pos).c_str());
-        fixed_description.Printf("%s:%u:", diagnostic.bufferName.c_str(),
-                                 diagnostic.line - first_line + 1);
-        start_pos = match_pos + match_len;
-        match_pos =
-            diagnostic.description.find(match.GetString().str(), start_pos);
-      } while (match_pos != std::string::npos);
-
-      // Append any last remaining text.
-      if (start_pos < diagnostic.description.size())
-        fixed_description.Printf(
-            "%s",
-            diagnostic.description
-                .substr(start_pos, diagnostic.description.size() - start_pos)
-                .c_str());
-
-      auto new_diagnostic = std::make_unique<SwiftDiagnostic>(
-          fixed_description.GetData(), severity, origin, bufferID);
+      auto new_diagnostic =
+          std::make_unique<SwiftDiagnostic>(detail, 0, diagnostic.buffer_id);
       for (auto fixit : diagnostic.fixits)
         new_diagnostic->AddFixIt(fixit);
 
       diagnostic_manager.AddDiagnostic(std::move(new_diagnostic));
-      if (diagnostic.kind == swift::DiagnosticKind::Error)
+      if (diagnostic.detail.severity == lldb::eSeverityError)
         added_one_diagnostic = true;
     };
 
@@ -356,27 +294,16 @@ public:
     // We found no error that is newer than the cursor position.
     // ClangImporter errors often happen outside the current transaction.
     for (const RawDiagnostic &diagnostic : m_raw_clang_diagnostics)
-      diagnostic_manager.AddDiagnostic(diagnostic.description.c_str(),
-                                       SeverityForKind(diagnostic.kind),
-                                       eDiagnosticOriginClang);
+      diagnostic_manager.AddDiagnostic(std::make_unique<Diagnostic>(
+          eDiagnosticOriginClang, 0, diagnostic.detail));
 
     // If we printed a clang error, ignore Swift warnings, which are
     // expected: The default lldb_expr wrapper produces some warnings.
     if (!m_num_clang_errors || m_num_swift_errors)
       for (const RawDiagnostic &diagnostic : m_raw_swift_diagnostics)
-        diagnostic_manager.AddDiagnostic(diagnostic.description.c_str(),
-                                         SeverityForKind(diagnostic.kind),
-                                         eDiagnosticOriginSwift);
+        diagnostic_manager.AddDiagnostic(std::make_unique<Diagnostic>(
+            eDiagnosticOriginClang, 0, diagnostic.detail));
   }
-
-  bool GetColorize() const { return m_colorize; }
-
-  bool SetColorize(bool b) {
-    const bool old = m_colorize;
-    m_colorize = b;
-    return old;
-  }
-
   void AddDiagnostic(std::unique_ptr<Diagnostic> diagnostic) {
     if (diagnostic)
       m_diagnostics.push_back(std::move(diagnostic));
@@ -389,30 +316,15 @@ private:
   // transcode them to SwiftDiagnostics in PrintDiagnostic.
   struct RawDiagnostic {
     RawDiagnostic() = default;
-    RawDiagnostic(std::string in_desc, swift::DiagnosticKind in_kind,
-                  std::string in_bufferName, unsigned in_bufferID,
-                  uint32_t in_line, uint32_t in_column,
-                  llvm::ArrayRef<swift::Diagnostic::FixIt> in_fixits)
-        : description(in_desc), bufferName(in_bufferName), fixits(in_fixits),
-          kind(in_kind), bufferID(in_bufferID), line(in_line),
-          column(in_column) {}
+    RawDiagnostic(DiagnosticDetail detail, unsigned buffer_id,
+                  llvm::ArrayRef<swift::Diagnostic::FixIt> fixits)
+        : detail(detail), fixits(fixits), buffer_id(buffer_id) {}
     RawDiagnostic(const RawDiagnostic &other) = default;
-    bool operator==(const RawDiagnostic& other) {
-      return kind == other.kind && line == other.line &&
-             bufferID == other.bufferID && column == other.column &&
-             bufferName == other.bufferName && description == other.description;
-    }
-    bool operator!=(const RawDiagnostic &other) { return !(*this == other); }
-
-    std::string description;
-    std::string bufferName;
+    DiagnosticDetail detail;
     std::vector<swift::DiagnosticInfo::FixIt> fixits;
-    swift::DiagnosticKind kind = swift::DiagnosticKind::Error;
     /// Only stored for comparison in HandleDiagnostic. There is no
     /// guarantee that the SourceMgr is still alive in a stored diagnostic.
-    unsigned bufferID;
-    uint32_t line;
-    uint32_t column;
+    unsigned buffer_id;
   };
 
   SwiftASTContext &m_ast_context;
@@ -424,7 +336,6 @@ private:
 
   unsigned m_num_swift_errors = 0;
   unsigned m_num_clang_errors = 0;
-  bool m_colorize = false;
 
   friend class SwiftASTContext::ScopedDiagnostics;
 };
