@@ -102,6 +102,11 @@ CompilerType TypeSystem::AddConstModifier(lldb::opaque_compiler_type_t type) {
   return CompilerType();
 }
 
+CompilerType TypeSystem::AddPtrAuthModifier(lldb::opaque_compiler_type_t type,
+                                            uint32_t payload) {
+  return CompilerType();
+}
+
 CompilerType
 TypeSystem::AddVolatileModifier(lldb::opaque_compiler_type_t type) {
   return CompilerType();
@@ -218,6 +223,11 @@ std::optional<llvm::json::Value> TypeSystem::ReportStatistics() {
   return std::nullopt;
 }
 
+CompilerDeclContext
+TypeSystem::GetCompilerDeclContextForType(const CompilerType &type) {
+  return CompilerDeclContext();
+}
+
 #pragma mark TypeSystemMap
 
 TypeSystemMap::TypeSystemMap() : m_mutex(), m_map() {}
@@ -350,29 +360,13 @@ TypeSystemMap::GetTypeSystemForLanguage(lldb::LanguageType language,
   return GetTypeSystemForLanguage(language);
 }
 
-// BEGIN SWIFT
-llvm::Expected<TypeSystemSP>
-TypeSystemMap::GetTypeSystemForLanguage(lldb::LanguageType language,
-                                        Target *target, bool can_create,
-                                        const char *compiler_options) {
-  if (can_create) {
-    return GetTypeSystemForLanguage(
-        language,
-        std::optional<CreateCallback>([language, target, compiler_options]() {
-          return TypeSystem::CreateInstance(language, target, compiler_options);
-        }));
-  }
-  return GetTypeSystemForLanguage(language);
-}
+bool TypeSystem::SupportsLanguageStatic(lldb::LanguageType language) {
+  if (language == eLanguageTypeUnknown || language >= eNumLanguageTypes)
+    return false;
 
-void TypeSystemMap::RemoveTypeSystemsForLanguage(lldb::LanguageType language) {
-  std::lock_guard<std::mutex> guard(m_mutex);
-  collection::iterator pos = m_map.find(language);
-  // If we are clearing the map, we don't need to remove this individual item.
-  // It will go away soon enough.
-  if (!m_clear_in_progress) {
-    if (pos != m_map.end())
-      m_map.erase(pos);
-  }
+  LanguageSet languages =
+      PluginManager::GetAllTypeSystemSupportedLanguagesForTypes();
+  if (languages.Empty())
+    return false;
+  return languages[language];
 }
-// END SWIFT
