@@ -13,30 +13,17 @@ class TestMultipleTargets(TestBase):
 
     @skipIf(oslist=["linux"], archs=["arm", "aarch64"])
     @skipIfNoSBHeaders
-    @skipIfHostIncompatibleWithRemote
     @expectedFailureAll(
         oslist=["windows"], archs=["i[3-6]86", "x86_64"], bugnumber="llvm.org/pr20282"
     )
     @expectedFlakeyNetBSD
+    @skipIfHostIncompatibleWithTarget
     def test_multiple_targets(self):
-        env = {self.dylibPath: self.getLLDBLibraryEnvVal()}
-
         self.driver_exe = self.getBuildArtifact("multi-target")
         self.buildDriver("main.cpp", self.driver_exe)
         self.addTearDownHook(lambda: os.remove(self.driver_exe))
 
-        # check_call will raise a CalledProcessError if multi-process-driver doesn't return
-        # exit code 0 to indicate success.  We can let this exception go - the test harness
-        # will recognize it as a test failure.
-
-        if self.TraceOn():
-            print("Running test %s" % self.driver_exe)
-            check_call([self.driver_exe, self.driver_exe], env=env)
-        else:
-            with open(os.devnull, "w") as fnull:
-                check_call(
-                    [self.driver_exe, self.driver_exe],
-                    env=env,
-                    stdout=fnull,
-                    stderr=fnull,
-                )
+        # check_call will raise a CalledProcessError if the executable doesn't
+        # return exit code 0 to indicate success.  We can let this exception go
+        # - the test harness will recognize it as a test failure.
+        subprocess.check_call([self.driver_exe, self.driver_exe])
