@@ -1,16 +1,14 @@
 """Test the lldb public C++ api breakpoint callbacks."""
 
-# __package__ = "lldbsuite.test"
-
-
 import os
+import subprocess
+
 from lldbsuite.test.decorators import *
 from lldbsuite.test.lldbtest import *
 from lldbsuite.test import lldbutil
-import unittest2
 
 
-@unittest2.skip("skipping due to frequent timeouts: rdar://28183131")
+@unittest.skip("skipping due to frequent timeouts: rdar://28183131")
 @skipIfNoSBHeaders
 class SBBreakpointCallbackCase(TestBase):
     NO_DEBUG_INFO_TESTCASE = True
@@ -25,10 +23,12 @@ class SBBreakpointCallbackCase(TestBase):
         self.generateSource("test_listener_event_process_state.cpp")
         self.generateSource("test_listener_resume.cpp")
         self.generateSource("test_stop-hook.cpp")
+        self.generateSource("test_concurrent_unwind.cpp")
 
     @skipIfRemote
     # clang-cl does not support throw or catch (llvm.org/pr24538)
     @skipIfWindows
+    @skipIfHostIncompatibleWithTarget
     def test_python_stop_hook(self):
         """Test that you can run a python command in a stop-hook when stdin is File based."""
         self.build_and_test("driver.cpp test_stop-hook.cpp", "test_python_stop_hook")
@@ -36,6 +36,7 @@ class SBBreakpointCallbackCase(TestBase):
     @skipIfRemote
     # clang-cl does not support throw or catch (llvm.org/pr24538)
     @skipIfWindows
+    @skipIfHostIncompatibleWithTarget
     def test_breakpoint_callback(self):
         """Test the that SBBreakpoint callback is invoked when a breakpoint is hit."""
         self.build_and_test(
@@ -45,6 +46,7 @@ class SBBreakpointCallbackCase(TestBase):
     @skipIfRemote
     # clang-cl does not support throw or catch (llvm.org/pr24538)
     @skipIfWindows
+    @skipIfHostIncompatibleWithTarget
     def test_breakpoint_location_callback(self):
         """Test the that SBBreakpointLocation callback is invoked when a breakpoint is hit."""
         self.build_and_test(
@@ -56,6 +58,7 @@ class SBBreakpointCallbackCase(TestBase):
     # clang-cl does not support throw or catch (llvm.org/pr24538)
     @skipIfWindows
     @expectedFlakeyFreeBSD
+    @skipIfHostIncompatibleWithTarget
     def test_sb_api_listener_event_description(self):
         """Test the description of an SBListener breakpoint event is valid."""
         self.build_and_test(
@@ -67,6 +70,7 @@ class SBBreakpointCallbackCase(TestBase):
     # clang-cl does not support throw or catch (llvm.org/pr24538)
     @skipIfWindows
     @expectedFlakeyFreeBSD
+    @skipIfHostIncompatibleWithTarget
     def test_sb_api_listener_event_process_state(self):
         """Test that a registered SBListener receives events when a process
         changes state.
@@ -81,6 +85,7 @@ class SBBreakpointCallbackCase(TestBase):
     @skipIfWindows
     @expectedFlakeyFreeBSD
     @skipIf(oslist=["linux"])  # flakey
+    @skipIfHostIncompatibleWithTarget
     def test_sb_api_listener_resume(self):
         """Test that a process can be resumed from a non-main thread."""
         self.build_and_test(
@@ -88,7 +93,19 @@ class SBBreakpointCallbackCase(TestBase):
             "test_listener_resume",
         )
 
-    def build_and_test(self, sources, test_name, args=None):
+    @skipIfRemote
+    # clang-cl does not support throw or catch (llvm.org/pr24538)
+    @skipIfWindows
+    @skipIfHostIncompatibleWithTarget
+    def test_concurrent_unwind(self):
+        """Test that you can run a python command in a stop-hook when stdin is File based."""
+        self.build_and_test(
+            "driver.cpp test_concurrent_unwind.cpp",
+            "test_concurrent_unwind",
+            inferior_source="deep_stack.cpp",
+        )
+
+    def build_and_test(self, sources, test_name, inferior_source="inferior.cpp"):
         """Build LLDB test from sources, and run expecting 0 exit code"""
 
         # These tests link against host lldb API.
@@ -101,7 +118,7 @@ class SBBreakpointCallbackCase(TestBase):
             )
 
         self.inferior = "inferior_program"
-        self.buildProgram("inferior.cpp", self.inferior)
+        self.buildProgram(inferior_source, self.inferior)
         self.addTearDownHook(lambda: os.remove(self.getBuildArtifact(self.inferior)))
 
         self.buildDriver(sources, test_name)
@@ -110,18 +127,10 @@ class SBBreakpointCallbackCase(TestBase):
         test_exe = self.getBuildArtifact(test_name)
         exe = [test_exe, self.getBuildArtifact(self.inferior)]
 
-        env = {self.dylibPath: self.getLLDBLibraryEnvVal()}
-        if "LLDB_DEBUGSERVER_PATH" in os.environ:
-            env["LLDB_DEBUGSERVER_PATH"] = os.environ["LLDB_DEBUGSERVER_PATH"]
-        try:
-            if self.TraceOn():
-                print("Running test %s" % " ".join(exe))
-                check_call(exe, env=env)
-            else:
-                with open(os.devnull, "w") as fnull:
-                    check_call(exe, env=env, stdout=fnull, stderr=fnull)
-        except subprocess.CalledProcessError as e:
-            self.fail(e)
+        # check_call will raise a CalledProcessError if the executable doesn't
+        # return exit code 0 to indicate success.  We can let this exception go
+        # - the test harness will recognize it as a test failure.
+        subprocess.check_call(exe)
 
     def build_program(self, sources, program):
         return self.buildDriver(sources, program)
