@@ -7,6 +7,7 @@ from lldbsuite.test.lldbtest import *
 from lldbsuite.test import lldbutil
 
 
+@skipIfLinux # Can't build std module rdar://138651439
 class TestDbgInfoContentForwardList(TestBase):
     @add_test_categories(["libc++"])
     @skipIf(compiler=no_match("clang"))
@@ -19,7 +20,13 @@ class TestDbgInfoContentForwardList(TestBase):
 
         self.runCmd("settings set target.import-std-module true")
 
-        list_type = "std::forward_list<Foo>"
+        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
+            [">", "16.0"]
+        ):
+            list_type = "std::forward_list<Foo>"
+        else:
+            list_type = "std::forward_list<Foo, std::allocator<Foo> >"
+
         value_type = "value_type"
 
         # FIXME: This has three elements in it but the formatter seems to
