@@ -105,7 +105,7 @@ class HelpCommandTestCase(TestBase):
     def test_help_image_list_shows_positional_args(self):
         """Command 'help image list' should describe positional args."""
         # 'image' is an alias for 'target modules'.
-        self.expect("help image list", substrs=["<shlib-name> [...]"])
+        self.expect("help image list", substrs=["<module> [...]"])
 
     @no_debug_info_test
     def test_help_target_variable_syntax(self):
