@@ -2,12 +2,10 @@ import lldb
 from lldbsuite.test.lldbtest import *
 from lldbsuite.test.decorators import *
 import lldbsuite.test.lldbutil as lldbutil
-import unittest2
 
 
 class SwiftAddressExpressionTest(TestBase):
     @swiftTest
-    @skipIfLinux # rdar://125497260
     def test(self):
         """Test that you can use register names in image lookup in a swift frame."""
         self.build()
