@@ -78,12 +78,24 @@ class TestCoroutineHandle(TestBase):
                 ],
             )
 
-        # Run until after the `co_yield`
         process = self.process()
+
+        # Break at a coroutine body
+        lldbutil.continue_to_source_breakpoint(
+            self,
+            process,
+            "int_generator my_generator_func",
+            lldb.SBFileSpec("main.cpp", False),
+        )
+
+        # Expect artificial variables to be displayed
+        self.expect("frame variable", substrs=["__promise", "__coro_frame"])
+
+        # Run until after the `co_yield`
         lldbutil.continue_to_source_breakpoint(
             self, process, "// Break after co_yield", lldb.SBFileSpec("main.cpp", False)
         )
-        # We correctly show the updated value inside `prommise.current_value`.
+        # We correctly show the updated value inside `promise.current_value`.
         self.expect_expr(
             "gen.hdl",
             result_children=[
