@@ -1,4 +1,4 @@
-﻿"""Test that lldb picks the correct DWARF location list entry with a return-pc out of bounds."""
+"""Test that lldb picks the correct DWARF location list entry with a return-pc out of bounds."""
 
 import lldb
 from lldbsuite.test.decorators import *
@@ -37,14 +37,18 @@ class LocationListLookupTestCase(TestBase):
                     process.GetSelectedThread().SetSelectedFrame(f.idx)
                     self.expect_expr("this", result_type="Foo *")
 
+    @skipIf(bugnumber = "rdar://135577167")
     @skipIf(oslist=["linux"], archs=["arm"])
     @skipIfDarwin
     def test_loclist_frame_var(self):
         self.build()
         self.check_local_vars(self.launch(), check_expr=False)
 
-    @skipIf(archs=no_match(["aarch64", "arm"]))
+    @skipIf(bugnumber = "rdar://135577167")
+    @skipIf(dwarf_version=["<", "3"])
+    @skipIf(compiler="clang", compiler_version=["<", "12.0"])
     @skipUnlessDarwin
+    @expectedFailureAll(archs=["x86_64"])
     def test_loclist_expr(self):
         self.build()
         self.check_local_vars(self.launch(), check_expr=True)
