@@ -5,8 +5,10 @@ from lldbsuite.test import lldbutil
 
 
 class CPPAcceleratorTableTestCase(TestBase):
+    @expectedFailureAll(setting=('plugin.typesystem.clang.experimental-redecl-completion', 'true'))
     @skipUnlessDarwin
     @skipIf(debug_info=no_match(["dwarf"]))
+    @skipIf(dwarf_version=[">=", "5"])
     def test(self):
         """Test that type lookups fail early (performance)"""
         self.build()
