@@ -2,7 +2,6 @@ import lldb
 from lldbsuite.test.lldbtest import *
 from lldbsuite.test.decorators import *
 import lldbsuite.test.lldbutil as lldbutil
-import unittest2
 
 
 class TestSwiftCTypeIvar(TestBase):
@@ -18,7 +17,7 @@ class TestSwiftCTypeIvar(TestBase):
         lldbutil.run_to_source_breakpoint(
             self, "break here", lldb.SBFileSpec("main.swift")
         )
-        # self.expect('v a', substrs=['asdf'])
+        # self.expect('frame var a', substrs=['asdf'])
         a = self.frame().FindVariable("a")
         lldbutil.check_variable(
             self,
