@@ -15,8 +15,8 @@ class TestSwiftForwardInteropCxxClassAsExistential(TestBase):
         _, _, _, _= lldbutil.run_to_source_breakpoint(
             self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))
 
-        self.expect('v x', substrs=['CxxClass', 'a1 = 10', 'a2 = 20', 'a3 = 30'])
+        self.expect('frame var x', substrs=['CxxClass', 'a1 = 10', 'a2 = 20', 'a3 = 30'])
         self.expect('expr x', substrs=['CxxClass', 'a1 = 10', 'a2 = 20', 'a3 = 30'])
 
-        self.expect('v y', substrs=['InheritedCxxClass', 'a1 = 10', 'a2 = 20', 'a3 = 30', 'a4 = 40'])
+        self.expect('frame var y', substrs=['InheritedCxxClass', 'a1 = 10', 'a2 = 20', 'a3 = 30', 'a4 = 40'])
         self.expect('expr y', substrs=['InheritedCxxClass', 'a1 = 10', 'a2 = 20', 'a3 = 30', 'a4 = 40'])
