@@ -16,10 +16,10 @@ class TestCxxForwardInteropNestedClasses(TestBase):
         _, _, _, _= lldbutil.run_to_source_breakpoint(
             self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))
 
-        self.expect('v nested', substrs=['CxxClass::NestedClass', 'b = 20'])
+        self.expect('frame var nested', substrs=['CxxClass::NestedClass', 'b = 20'])
         self.expect('expr nested', substrs=['CxxClass::NestedClass', 'b = 20'])
 
-        self.expect('v nestedSubclass', substrs=['CxxClass::NestedSubclass', 
+        self.expect('frame var nestedSubclass', substrs=['CxxClass::NestedSubclass', 
             'SuperClass = (a = 10)', 'c = 30'])
         self.expect('expr nestedSubclass', substrs=['CxxClass::NestedSubclass', 
             'SuperClass = (a = 10)', 'c = 30'])
