@@ -15,12 +15,12 @@ class TestSwiftForwardInteropGenericWithCxxType(TestBase):
         _, _, _, _= lldbutil.run_to_source_breakpoint(
             self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))
 
-        self.expect('v classWrapper', substrs=['Wrapper<CxxClass>', 't', 'a1 = 10',
+        self.expect('frame var classWrapper', substrs=['Wrapper<CxxClass>', 't', 'a1 = 10',
             'a2 = 20', 'a3 = 30'])
         self.expect('expr classWrapper', substrs=['Wrapper<CxxClass>', 't', 'a1 = 10',
             'a2 = 20', 'a3 = 30'])
 
-        self.expect('v subclassWrapper', substrs=['Wrapper<CxxSubclass>', 't', 
+        self.expect('frame var subclassWrapper', substrs=['Wrapper<CxxSubclass>', 't', 
             'CxxClass = (a1 = 10, a2 = 20, a3 = 30)', 'a4 = 40'])
         self.expect('expr subclassWrapper', substrs=['Wrapper<CxxSubclass>', 't', 
             'CxxClass = (a1 = 10, a2 = 20, a3 = 30)', 'a4 = 40'])
