@@ -15,6 +15,6 @@ class TestExternalProviderExtraInhabitants(TestBase):
         lldbutil.run_to_source_breakpoint(
             self, 'Set breakpoint here.', lldb.SBFileSpec('main.swift'))
 
-        self.expect('v object.size.some.width', substrs=['10'])
-        self.expect('v object.size.some.height', substrs=['20'])
+        self.expect('frame var object.size.some.width', substrs=['10'])
+        self.expect('frame var object.size.some.height', substrs=['20'])
 
