@@ -12,6 +12,4 @@
 import lldbsuite.test.lldbinline as lldbinline
 from lldbsuite.test.decorators import *
 
-lldbinline.MakeInlineTest(
-    __file__, globals(), decorators=[swiftTest, skipUnlessFoundation]
-)
+lldbinline.MakeInlineTest(__file__, globals(), decorators=[swiftTest,skipUnlessFoundation])
