@@ -12,8 +12,5 @@
 import lldbsuite.test.lldbinline as lldbinline
 from lldbsuite.test.decorators import *
 
-lldbinline.MakeInlineTest(
-    __file__,
-    globals(),
-    decorators=[swiftTest, skipUnlessFoundation, skipIf(oslist=["windows"])],
-)
+lldbinline.MakeInlineTest(__file__, globals(),
+                          decorators=[swiftTest,skipUnlessFoundation,skipIf(oslist=['windows'])])
