@@ -12,6 +12,5 @@
 import lldbsuite.test.lldbinline as lldbinline
 from lldbsuite.test.decorators import *
 
-lldbinline.MakeInlineTest(
-    __file__, globals(), decorators=[swiftTest, skipIf(oslist=["windows", "linux"])]
-)
+lldbinline.MakeInlineTest(__file__, globals(),
+                          decorators=[swiftTest,skipIf(oslist=['windows', 'linux'])])
