@@ -16,8 +16,8 @@ from lldbsuite.test.decorators import *
 import lldbsuite.test.lldbtest as lldbtest
 import os
 
-
 class TestSwiftMeta(lldbtest.TestBase):
+
     mydir = lldbtest.TestBase.compute_mydir(__file__)
 
     @swiftTest
@@ -28,9 +28,11 @@ class TestSwiftMeta(lldbtest.TestBase):
     def test_swiftBuild(self):
         self.build()
         exe = self.getBuildArtifact()
-        dsym = exe + ".dSYM"
+        dsym = exe+'.dSYM'
         self.assertTrue(os.path.isfile(exe))
         if self.getDebugInfo() == "dwarf":
-            self.assertFalse(os.path.isdir(dsym), "testing DWARF, but .dSYM present")
+            self.assertFalse(os.path.isdir(dsym),
+                             'testing DWARF, but .dSYM present')
         if self.getDebugInfo() == "dsym":
-            self.assertTrue(os.path.isdir(dsym), ".dSYM is missing in dsym config")
+            self.assertTrue(os.path.isdir(dsym),
+                            '.dSYM is missing in dsym config')
