@@ -25,54 +25,54 @@ class TestSwiftPrivateGenericType(TestBase):
                 'break here for struct', lldb.SBFileSpec('Public.swift'),
                 extra_images=['Public'])
         # Make sure this fails without generic expression evaluation.
-        self.expect("e --bind-generic-types true -- self", 
+        self.expect("expr --bind-generic-types true -- self", 
                     substrs=["Couldn't realize Swift AST type of self."], 
                     error=True)
         # Test that not binding works.
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Public.StructWrapper<T>", 
                              'name = "The invisible struct."'])
         # Test that the "auto" behavior also works.
-        self.expect("e --bind-generic-types auto -- self", 
+        self.expect("expr --bind-generic-types auto -- self", 
                     substrs=["Public.StructWrapper<T>", 
                              'name = "The invisible struct."'])
         # Test that the default (should be the auto option) also works.
-        self.expect("e -- self", substrs=["Public.StructWrapper<T>", 
+        self.expect("expr -- self", substrs=["Public.StructWrapper<T>", 
                                           'name = "The invisible struct."'])
 
         breakpoint = target.BreakpointCreateBySourceRegex(
             'break here for class', lldb.SBFileSpec('Public.swift'), None)
         lldbutil.continue_to_breakpoint(process, breakpoint)
-        self.expect("e --bind-generic-types true -- self", 
+        self.expect("expr --bind-generic-types true -- self", 
                     substrs=["Couldn't realize Swift AST type of self."], 
                     error=True)
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Public.ClassWrapper<Private.InvisibleStruct>", 
                              'name = "The invisible struct."'])
-        self.expect("e --bind-generic-types auto -- self", 
+        self.expect("expr --bind-generic-types auto -- self", 
                     substrs=["Public.ClassWrapper<Private.InvisibleStruct>", 
                              'name = "The invisible struct."'])
-        self.expect("e -- self", 
+        self.expect("expr -- self", 
                     substrs=["Public.ClassWrapper<Private.InvisibleStruct>", 
                              'name = "The invisible struct."'])
 
         breakpoint = target.BreakpointCreateBySourceRegex(
             'break here for two generic parameters', lldb.SBFileSpec('Public.swift'), None)
         lldbutil.continue_to_breakpoint(process, breakpoint)
-        self.expect("e --bind-generic-types true -- self", 
+        self.expect("expr --bind-generic-types true -- self", 
                     substrs=["Couldn't realize Swift AST type of self."], 
                     error=True)
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Public.TwoGenericParameters",
                              "<Private.InvisibleClass, Private.InvisibleStruct>", 
                              'name = "The invisible class."',
                              "someNumber = 42"])
-        self.expect("e --bind-generic-types auto -- self", 
+        self.expect("expr --bind-generic-types auto -- self", 
                     substrs=["Public.TwoGenericParameters",
                              "<Private.InvisibleClass, Private.InvisibleStruct>", 
                              'name = "The invisible class."',
                              "someNumber = 42"])
-        self.expect("e -- self", 
+        self.expect("expr -- self", 
                     substrs=["Public.TwoGenericParameters",
                              "<Private.InvisibleClass, Private.InvisibleStruct>", 
                              'name = "The invisible class."',
@@ -81,24 +81,24 @@ class TestSwiftPrivateGenericType(TestBase):
         breakpoint = target.BreakpointCreateBySourceRegex(
             'break here for three generic parameters', lldb.SBFileSpec('Public.swift'), None)
         lldbutil.continue_to_breakpoint(process, breakpoint)
-        self.expect("e --bind-generic-types true -- self", 
+        self.expect("expr --bind-generic-types true -- self", 
                     substrs=["Couldn't realize Swift AST type of self."], 
                     error=True)
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Public.ThreeGenericParameters",
                              "<T, U, V>", 
                              'name = "The invisible class."',
                              "someNumber = 42",
                              'name = "The invisible struct."',
                              "v = true"])
-        self.expect("e --bind-generic-types auto -- self", 
+        self.expect("expr --bind-generic-types auto -- self", 
                     substrs=["Public.ThreeGenericParameters",
                              "<T, U, V>", 
                              'name = "The invisible class."',
                              "someNumber = 42",
                              'name = "The invisible struct."',
                              "v = true"])
-        self.expect("e -- self", 
+        self.expect("expr -- self", 
                     substrs=["Public.ThreeGenericParameters",
                              "<T, U, V>", 
                              'name = "The invisible class."',
@@ -109,10 +109,10 @@ class TestSwiftPrivateGenericType(TestBase):
         breakpoint = target.BreakpointCreateBySourceRegex(
             'break here for four generic parameters', lldb.SBFileSpec('Public.swift'), None)
         lldbutil.continue_to_breakpoint(process, breakpoint)
-        self.expect("e --bind-generic-types true -- self", 
+        self.expect("expr --bind-generic-types true -- self", 
                     substrs=["Couldn't realize Swift AST type of self."], 
                     error=True)
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Public.FourGenericParameters",
                              "<T, U, V, W>", 
                              'name = "The invisible struct."',
@@ -120,7 +120,7 @@ class TestSwiftPrivateGenericType(TestBase):
                              "someNumber = 42",
                              "v = 3 values", "One", "two", "three",
                              "w = 482"])
-        self.expect("e --bind-generic-types auto -- self", 
+        self.expect("expr --bind-generic-types auto -- self", 
                     substrs=["Public.FourGenericParameters",
                              "<T, U, V, W>", 
                              'name = "The invisible struct."',
@@ -128,7 +128,7 @@ class TestSwiftPrivateGenericType(TestBase):
                              "someNumber = 42",
                              "v = 3 values", "One", "two", "three",
                              "w = 482"])
-        self.expect("e -- self", 
+        self.expect("expr -- self", 
                     substrs=["Public.FourGenericParameters",
                              "<T, U, V, W>", 
                              'name = "The invisible struct."',
@@ -140,19 +140,19 @@ class TestSwiftPrivateGenericType(TestBase):
         breakpoint = target.BreakpointCreateBySourceRegex(
             'break here for non-generic', lldb.SBFileSpec('Public.swift'), None)
         lldbutil.continue_to_breakpoint(process, breakpoint)
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Could not evaluate the expression without binding generic types."], 
                     error=True)
 
         breakpoint = target.BreakpointCreateBySourceRegex(
             'break here for nested generic parameters', lldb.SBFileSpec('Public.swift'), None)
         lldbutil.continue_to_breakpoint(process, breakpoint)
-        self.expect("e --bind-generic-types false -- self", 
+        self.expect("expr --bind-generic-types false -- self", 
                     substrs=["Could not evaluate the expression without binding generic types."], 
                     error=True)
 
         # Check that if both binding and not binding the generic type parameters fail, we report 
         # the "bind generic params" error message, as that's the default case that runs first.
-        self.expect("e --bind-generic-types auto -- self", 
+        self.expect("expr --bind-generic-types auto -- self", 
                     substrs=["Couldn't realize Swift AST type of self."], 
                     error=True)
