@@ -5,4 +5,7 @@ are correctly printed.
 import lldbsuite.test.lldbinline as lldbinline
 from lldbsuite.test.decorators import *
 
-lldbinline.MakeInlineTest(__file__, globals(), decorators=[swiftTest, skipUnlessDarwin])
+lldbinline.MakeInlineTest(
+    __file__, globals(),
+            decorators=[
+            swiftTest,skipUnlessDarwin])
