@@ -2,7 +2,6 @@ import lldb
 from lldbsuite.test.decorators import *
 import lldbsuite.test.lldbtest as lldbtest
 import lldbsuite.test.lldbutil as lldbutil
-import unittest2
 
 
 class TestSwiftRuntimeInstrumentationRecognizer(lldbtest.TestBase):
@@ -13,11 +12,16 @@ class TestSwiftRuntimeInstrumentationRecognizer(lldbtest.TestBase):
         self.runCmd("file " + self.getBuildArtifact("a.out"))
         self.runCmd("process launch")
 
-        self.expect("frame recognizer list",
-                    substrs=['Swift Runtime Instrumentation StackFrame Recognizer, symbol _swift_runtime_on_report (regexp)'])
+        self.expect(
+            "frame recognizer list",
+            substrs=[
+                "Swift runtime instrumentation frame recognizer",
+                "_swift_runtime_on_report",
+            ],
+        )
 
         self.expect("frame recognizer info 0",
-                    substrs=['frame 0 is recognized by Swift Runtime Instrumentation StackFrame Recognizer'])
+                    substrs=['frame 0 is recognized by Swift runtime instrumentation frame recognizer'])
 
         self.expect("thread info",
                     substrs=['stop reason = Fatal error: Division by zero'])
