@@ -18,7 +18,6 @@ import lldbsuite.test.lldbtest as lldbtest
 import lldbsuite.test.lldbutil as lldbutil
 import os
 import platform
-import unittest2
 
 class TestSwiftStepping(lldbtest.TestBase):
 
@@ -220,36 +219,11 @@ class TestSwiftStepping(lldbtest.TestBase):
         thread.StepOut()
         self.hit_correct_line(thread, "Second case with a where statement")
 
-        thread.StepOver()
         thread.StepOver()
         self.hit_correct_line(
             thread, "print in second case with where statement.")
 
-        #
-        # For some reason, we don't step from the body of the case to
-        # the end of the switch, but back to the case: statement, and
-        # then directly out of the switch.
-        #
         thread.StepOver()
-        steps_back_to_case = self.hit_correct_line(
-            thread,
-            "Sometimes the line table steps to here "
-            "after the body of the case.", False)
-        if steps_back_to_case:
-            self.fail(
-                "Stepping past a taken body of a case statement should not "
-                "step back to the case statement.")
-
-        if self.hit_correct_line(
-                thread,
-                "This is the end of the switch statement", False):
-            thread.StepOver()
-        elif not self.hit_correct_line(
-                thread, "Make a version of P that conforms directly", False):
-            self.fail(
-                "Stepping past the body of the case didn't stop "
-                "where expected.")
-
         self.hit_correct_line(
             thread, "Make a version of P that conforms directly")
 
