@@ -2,7 +2,6 @@ import lldb
 from lldbsuite.test.lldbtest import *
 from lldbsuite.test.decorators import *
 import lldbsuite.test.lldbutil as lldbutil
-import unittest2
 
 
 class TestSwiftTripleDetection(TestBase):
@@ -26,18 +25,7 @@ class TestSwiftTripleDetection(TestBase):
         bkpt = target.BreakpointCreateByName("main")
         process = target.LaunchSimple(None, None, self.get_process_working_directory())
         self.expect("expression 1")
-        import io
-        types_logfile = io.open(types_log, "r", encoding='utf-8')
-
-        import re
-        availability_re = re.compile(r'@available\(macCatalyst 1.*, \*\)')
-        found = 0
-        for line in types_logfile:
-            print(line)
-            if re.search('SwiftASTContextForExpressions.*Underspecified target triple .*-apple-macos-unknown', line):
-                found += 1
-                continue
-            if found == 1 and re.search('SwiftASTContextForExpressions.*setting to ".*-apple-macos.[0-9.]+"', line):
-               found += 1
-               break
-        self.assertEqual(found, 2)
+        self.filecheck('platform shell cat "%s"' % types_log, __file__)
+        # CHECK: {{SwiftASTContextForExpressions.*Module triple: ".*-apple-macos.[0-9.]+"}}
+        # CHECK: {{SwiftASTContextForExpressions.*Target triple: ".*-apple-macos-unknown"}}
+        # CHECK: {{SwiftASTContextForExpressions.*setting to ".*-apple-macos.[0-9.]+"}}
