@@ -20,8 +20,6 @@ import os
 
 
 class TestSwiftMetadataSymbols(TestBase):
-    mydir = TestBase.compute_mydir(__file__)
-
     @swiftTest
     @expectedFailureAll(bugnumber="<rdar://problem/31066543>")
     def test_swift_metadata_symbols(self):
@@ -29,9 +27,6 @@ class TestSwiftMetadataSymbols(TestBase):
         self.build()
         self.do_test()
 
-    def setUp(self):
-        TestBase.setUp(self)
-
     def do_test(self):
         """Tests that we can break and display simple types"""
         exe_name = "a.out"
@@ -41,10 +36,10 @@ class TestSwiftMetadataSymbols(TestBase):
         target = self.dbg.CreateTarget(exe)
         self.assertTrue(target, VALID_TARGET)
 
-        self.expect(
-            "target modules dump symtab -m a.out", patterns=["Metadata.*_TMC1a3Foo"]
-        )
+        self.expect("target modules dump symtab -m a.out",
+                    patterns=['Metadata.*_TMC1a3Foo'])
+
+        self.expect("target modules dump symtab a.out",
+                    patterns=['Metadata.*type metadata for'])
+
 
-        self.expect(
-            "target modules dump symtab a.out", patterns=["Metadata.*type metadata for"]
-        )
