@@ -13,7 +13,6 @@ import os
 
 class TestDAP_disassemble(lldbdap_testcase.DAPTestCaseBase):
     @skipIfWindows
-    @skipIfRemote
     def test_disassemble(self):
         """
         Tests the 'disassemble' request.
@@ -31,11 +30,11 @@ class TestDAP_disassemble(lldbdap_testcase.DAPTestCaseBase):
         self.continue_to_next_stop()
 
         pc_assembly = self.disassemble(frameIndex=0)
-        self.assertTrue("location" in pc_assembly, "Source location missing.")
-        self.assertTrue("instruction" in pc_assembly, "Assembly instruction missing.")
-        
+        self.assertIn("location", pc_assembly, "Source location missing.")
+        self.assertIn("instruction", pc_assembly, "Assembly instruction missing.")
+
         # The calling frame (qsort) is coming from a system library, as a result
         # we should not have a source location.
         qsort_assembly = self.disassemble(frameIndex=1)
-        self.assertFalse("location" in qsort_assembly, "Source location not expected.")
-        self.assertTrue("instruction" in pc_assembly, "Assembly instruction missing.")
+        self.assertNotIn("location", qsort_assembly, "Source location not expected.")
+        self.assertIn("instruction", pc_assembly, "Assembly instruction missing.")
