@@ -18,6 +18,7 @@
 #include "gmock/gmock.h"
 #include <future>
 #include <limits>
+#include <optional>
 
 using namespace lldb_private::process_gdb_remote;
 using namespace lldb_private;
@@ -178,7 +179,7 @@ TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo) {
       // the FileSpecs they create.
       FileSpec("/foo/baw.so", FileSpec::Style::windows),
   };
-  std::future<llvm::Optional<std::vector<ModuleSpec>>> async_result =
+  std::future<std::optional<std::vector<ModuleSpec>>> async_result =
       std::async(std::launch::async,
                  [&] { return client.GetModulesInfo(file_specs, triple); });
   HandlePacket(
@@ -192,19 +193,18 @@ TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo) {
   auto result = async_result.get();
   ASSERT_TRUE(result.has_value());
   ASSERT_EQ(1u, result->size());
-  EXPECT_EQ("/foo/bar.so", result.value()[0].GetFileSpec().GetPath());
-  EXPECT_EQ(triple, result.value()[0].GetArchitecture().GetTriple());
-  EXPECT_EQ(UUID("@ABCDEFGHIJKLMNO", 16),
-            result.value()[0].GetUUID());
-  EXPECT_EQ(0u, result.value()[0].GetObjectOffset());
-  EXPECT_EQ(1234u, result.value()[0].GetObjectSize());
+  EXPECT_EQ("/foo/bar.so", (*result)[0].GetFileSpec().GetPath());
+  EXPECT_EQ(triple, (*result)[0].GetArchitecture().GetTriple());
+  EXPECT_EQ(UUID("@ABCDEFGHIJKLMNO", 16), (*result)[0].GetUUID());
+  EXPECT_EQ(0u, (*result)[0].GetObjectOffset());
+  EXPECT_EQ(1234u, (*result)[0].GetObjectSize());
 }
 
 TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo_UUID20) {
   llvm::Triple triple("i386-pc-linux");
 
   FileSpec file_spec("/foo/bar.so", FileSpec::Style::posix);
-  std::future<llvm::Optional<std::vector<ModuleSpec>>> async_result =
+  std::future<std::optional<std::vector<ModuleSpec>>> async_result =
       std::async(std::launch::async,
                  [&] { return client.GetModulesInfo(file_spec, triple); });
   HandlePacket(
@@ -217,12 +217,11 @@ TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo_UUID20) {
   auto result = async_result.get();
   ASSERT_TRUE(result.has_value());
   ASSERT_EQ(1u, result->size());
-  EXPECT_EQ("/foo/bar.so", result.value()[0].GetFileSpec().GetPath());
-  EXPECT_EQ(triple, result.value()[0].GetArchitecture().GetTriple());
-  EXPECT_EQ(UUID("@ABCDEFGHIJKLMNOPQRS", 20),
-            result.value()[0].GetUUID());
-  EXPECT_EQ(0u, result.value()[0].GetObjectOffset());
-  EXPECT_EQ(1234u, result.value()[0].GetObjectSize());
+  EXPECT_EQ("/foo/bar.so", (*result)[0].GetFileSpec().GetPath());
+  EXPECT_EQ(triple, (*result)[0].GetArchitecture().GetTriple());
+  EXPECT_EQ(UUID("@ABCDEFGHIJKLMNOPQRS", 20), (*result)[0].GetUUID());
+  EXPECT_EQ(0u, (*result)[0].GetObjectOffset());
+  EXPECT_EQ(1234u, (*result)[0].GetObjectSize());
 }
 
 TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfoInvalidResponse) {
@@ -251,7 +250,7 @@ TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfoInvalidResponse) {
   };
 
   for (const char *response : invalid_responses) {
-    std::future<llvm::Optional<std::vector<ModuleSpec>>> async_result =
+    std::future<std::optional<std::vector<ModuleSpec>>> async_result =
         std::async(std::launch::async,
                    [&] { return client.GetModulesInfo(file_spec, triple); });
     HandlePacket(
@@ -288,7 +287,7 @@ TEST_F(GDBRemoteCommunicationClientTest, TestPacketSpeedJSON) {
   server_thread.join();
 
   GTEST_LOG_(INFO) << "Formatted output: " << ss.GetData();
-  auto object_sp = StructuredData::ParseJSON(std::string(ss.GetString()));
+  auto object_sp = StructuredData::ParseJSON(ss.GetString());
   ASSERT_TRUE(bool(object_sp));
   auto dict_sp = object_sp->GetAsDictionary();
   ASSERT_TRUE(bool(dict_sp));
@@ -301,7 +300,7 @@ TEST_F(GDBRemoteCommunicationClientTest, TestPacketSpeedJSON) {
   size_t num_packets;
   ASSERT_TRUE(dict_sp->GetValueForKeyAsInteger("num_packets", num_packets))
       << ss.GetString();
-  ASSERT_EQ(10, num_packets);
+  ASSERT_EQ(10, (int)num_packets);
 }
 
 TEST_F(GDBRemoteCommunicationClientTest, SendSignalsToIgnore) {
@@ -344,24 +343,36 @@ TEST_F(GDBRemoteCommunicationClientTest, GetMemoryRegionInfo) {
   EXPECT_EQ(MemoryRegionInfo::eYes, region_info.GetExecutable());
   EXPECT_EQ("/foo/bar.so", region_info.GetName().GetStringRef());
   EXPECT_EQ(MemoryRegionInfo::eDontKnow, region_info.GetMemoryTagged());
+  EXPECT_EQ(MemoryRegionInfo::eDontKnow, region_info.IsStackMemory());
 
   result = std::async(std::launch::async, [&] {
     return client.GetMemoryRegionInfo(addr, region_info);
   });
 
   HandlePacket(server, "qMemoryRegionInfo:a000",
-               "start:a000;size:2000;flags:;");
+               "start:a000;size:2000;flags:;type:stack;");
   EXPECT_TRUE(result.get().Success());
   EXPECT_EQ(MemoryRegionInfo::eNo, region_info.GetMemoryTagged());
+  EXPECT_EQ(MemoryRegionInfo::eYes, region_info.IsStackMemory());
 
   result = std::async(std::launch::async, [&] {
     return client.GetMemoryRegionInfo(addr, region_info);
   });
 
   HandlePacket(server, "qMemoryRegionInfo:a000",
-               "start:a000;size:2000;flags: mt  zz mt  ;");
+               "start:a000;size:2000;flags: mt  zz mt  ;type:ha,ha,stack;");
   EXPECT_TRUE(result.get().Success());
   EXPECT_EQ(MemoryRegionInfo::eYes, region_info.GetMemoryTagged());
+  EXPECT_EQ(MemoryRegionInfo::eYes, region_info.IsStackMemory());
+
+  result = std::async(std::launch::async, [&] {
+    return client.GetMemoryRegionInfo(addr, region_info);
+  });
+
+  HandlePacket(server, "qMemoryRegionInfo:a000",
+               "start:a000;size:2000;type:heap;");
+  EXPECT_TRUE(result.get().Success());
+  EXPECT_EQ(MemoryRegionInfo::eNo, region_info.IsStackMemory());
 }
 
 TEST_F(GDBRemoteCommunicationClientTest, GetMemoryRegionInfoInvalidResponse) {
@@ -444,8 +455,8 @@ TEST_F(GDBRemoteCommunicationClientTest, SendTraceSupportedPacket) {
 
 TEST_F(GDBRemoteCommunicationClientTest, GetQOffsets) {
   const auto &GetQOffsets = [&](llvm::StringRef response) {
-    std::future<Optional<QOffsets>> result = std::async(
-        std::launch::async, [&] { return client.GetQOffsets(); });
+    std::future<std::optional<QOffsets>> result =
+        std::async(std::launch::async, [&] { return client.GetQOffsets(); });
 
     HandlePacket(server, "qOffsets", response);
     return result.get();
@@ -458,20 +469,20 @@ TEST_F(GDBRemoteCommunicationClientTest, GetQOffsets) {
   EXPECT_EQ((QOffsets{true, {0x1234, 0x2345}}),
             GetQOffsets("TextSeg=1234;DataSeg=2345"));
 
-  EXPECT_EQ(llvm::None, GetQOffsets("E05"));
-  EXPECT_EQ(llvm::None, GetQOffsets("Text=bogus"));
-  EXPECT_EQ(llvm::None, GetQOffsets("Text=1234"));
-  EXPECT_EQ(llvm::None, GetQOffsets("Text=1234;Data=1234;"));
-  EXPECT_EQ(llvm::None, GetQOffsets("Text=1234;Data=1234;Bss=1234;"));
-  EXPECT_EQ(llvm::None, GetQOffsets("TEXTSEG=1234"));
-  EXPECT_EQ(llvm::None, GetQOffsets("TextSeg=0x1234"));
-  EXPECT_EQ(llvm::None, GetQOffsets("TextSeg=12345678123456789"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("E05"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("Text=bogus"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("Text=1234"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("Text=1234;Data=1234;"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("Text=1234;Data=1234;Bss=1234;"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("TEXTSEG=1234"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("TextSeg=0x1234"));
+  EXPECT_EQ(std::nullopt, GetQOffsets("TextSeg=12345678123456789"));
 }
 
 static void
 check_qmemtags(TestClient &client, MockServer &server, size_t read_len,
                int32_t type, const char *packet, llvm::StringRef response,
-               llvm::Optional<std::vector<uint8_t>> expected_tag_data) {
+               std::optional<std::vector<uint8_t>> expected_tag_data) {
   const auto &ReadMemoryTags = [&]() {
     std::future<DataBufferSP> result = std::async(std::launch::async, [&] {
       return client.ReadMemoryTags(0xDEF0, read_len, type);
@@ -522,26 +533,28 @@ TEST_F(GDBRemoteCommunicationClientTest, ReadMemoryTags) {
                  std::vector<uint8_t>{0x1, 0x2});
 
   // Empty response is an error
-  check_qmemtags(client, server, 17, 1, "qMemTags:def0,11:1", "", llvm::None);
+  check_qmemtags(client, server, 17, 1, "qMemTags:def0,11:1", "", std::nullopt);
   // Usual error response
   check_qmemtags(client, server, 17, 1, "qMemTags:def0,11:1", "E01",
-                 llvm::None);
+                 std::nullopt);
   // Leading m missing
-  check_qmemtags(client, server, 17, 1, "qMemTags:def0,11:1", "01", llvm::None);
+  check_qmemtags(client, server, 17, 1, "qMemTags:def0,11:1", "01",
+                 std::nullopt);
   // Anything other than m is an error
   check_qmemtags(client, server, 17, 1, "qMemTags:def0,11:1", "z01",
-                 llvm::None);
+                 std::nullopt);
   // Decoding tag data doesn't use all the chars in the packet
   check_qmemtags(client, server, 32, 1, "qMemTags:def0,20:1", "m09zz",
-                 llvm::None);
+                 std::nullopt);
   // Data that is not hex bytes
   check_qmemtags(client, server, 32, 1, "qMemTags:def0,20:1", "mhello",
-                 llvm::None);
+                 std::nullopt);
   // Data is not a complete hex char
-  check_qmemtags(client, server, 32, 1, "qMemTags:def0,20:1", "m9", llvm::None);
+  check_qmemtags(client, server, 32, 1, "qMemTags:def0,20:1", "m9",
+                 std::nullopt);
   // Data has a trailing hex char
   check_qmemtags(client, server, 32, 1, "qMemTags:def0,20:1", "m01020",
-                 llvm::None);
+                 std::nullopt);
 }
 
 static void check_Qmemtags(TestClient &client, MockServer &server,
@@ -591,3 +604,30 @@ TEST_F(GDBRemoteCommunicationClientTest, WriteMemoryTags) {
                  std::vector<uint8_t>{0x99}, "QMemTags:456789,0:80000000:99",
                  "E03", false);
 }
+
+// Prior to this verison, constructing a std::future for a type without a
+// default constructor is not possible.
+// https://developercommunity.visualstudio.com/t/c-shared-state-futuresstate-default-constructs-the/60897
+#if !defined(_MSC_VER) || _MSC_VER >= 1932
+TEST_F(GDBRemoteCommunicationClientTest, CalculateMD5) {
+  FileSpec file_spec("/foo/bar", FileSpec::Style::posix);
+  std::future<ErrorOr<MD5::MD5Result>> async_result = std::async(
+      std::launch::async, [&] { return client.CalculateMD5(file_spec); });
+
+  lldb_private::StreamString stream;
+  stream.PutCString("vFile:MD5:");
+  stream.PutStringAsRawHex8(file_spec.GetPath(false));
+  HandlePacket(server, stream.GetString().str(),
+               "F,"
+               "deadbeef01020304"
+               "05060708deadbeef");
+  auto result = async_result.get();
+
+  // Server and client puts/parses low, and then high
+  const uint64_t expected_low = 0xdeadbeef01020304;
+  const uint64_t expected_high = 0x05060708deadbeef;
+  ASSERT_TRUE(result);
+  EXPECT_EQ(expected_low, result->low());
+  EXPECT_EQ(expected_high, result->high());
+}
+#endif
