@@ -44,14 +44,15 @@ public:
 
 struct SwiftASTContextTester : public SwiftASTContext {
   #ifndef NDEBUG
-    SwiftASTContextTester() : SwiftASTContext() {}
-  #endif
+  SwiftASTContextTester()
+      : SwiftASTContext(), m_typeref_typesystem(new TypeSystemSwiftTypeRef()) {}
+#endif
 
-  TypeSystemSwiftTypeRef &GetTypeSystemSwiftTypeRef() override {
+  TypeSystemSwiftTypeRefSP GetTypeSystemSwiftTypeRef() override {
     return m_typeref_typesystem;
   }
 
-  TypeSystemSwiftTypeRef m_typeref_typesystem;
+  TypeSystemSwiftTypeRefSP m_typeref_typesystem;
 };
 
 TEST_F(TestSwiftASTContext, IsNonTriviallyManagedReferenceType) {
@@ -234,7 +235,7 @@ const std::vector<std::string> uniqued_flags = {
 TEST_F(ClangArgs, UniquingCollisionWithExistingFlags) {
   const std::vector<std::string> source = duplicated_flags;
   std::vector<std::string> dest = uniqued_flags;
-  SwiftASTContext::AddExtraClangArgs(source, dest, false);
+  SwiftASTContext::AddExtraClangArgs(source, dest);
 
   EXPECT_EQ(dest, uniqued_flags);
 }
@@ -242,7 +243,7 @@ TEST_F(ClangArgs, UniquingCollisionWithExistingFlags) {
 TEST_F(ClangArgs, UniquingCollisionWithAddedFlags) {
   const std::vector<std::string> source = duplicated_flags;
   std::vector<std::string> dest;
-  SwiftASTContext::AddExtraClangArgs(source, dest, false);
+  SwiftASTContext::AddExtraClangArgs(source, dest);
 
   EXPECT_EQ(dest, uniqued_flags);
 }
@@ -251,7 +252,7 @@ TEST_F(ClangArgs, DoubleDash) {
   // -v with all currently ignored arguments following.
   const std::vector<std::string> source{"-v", "--", "-Werror", ""};
   std::vector<std::string> dest;
-  SwiftASTContext::AddExtraClangArgs(source, dest, false);
+  SwiftASTContext::AddExtraClangArgs(source, dest);
 
   // Check that all ignored arguments got removed.
   EXPECT_EQ(dest, std::vector<std::string>({"-v"}));
