@@ -9,16 +9,15 @@
 #include "Standalone-c/Dialects.h"
 #include "mlir/Bindings/Python/PybindAdaptors.h"
 
-namespace py = pybind11;
 using namespace mlir::python::adaptors;
 
 PYBIND11_MODULE(_standaloneDialects, m) {
   //===--------------------------------------------------------------------===//
   // standalone dialect
   //===--------------------------------------------------------------------===//
-  auto standalone_m = m.def_submodule("standalone");
+  auto standaloneM = m.def_submodule("standalone");
 
-  standalone_m.def(
+  standaloneM.def(
       "register_dialect",
       [](MlirContext context, bool load) {
         MlirDialectHandle handle = mlirGetDialectHandle__standalone__();
