@@ -20,18 +20,10 @@ namespace arith {
 
 #define GEN_PASS_DECL
 #include "mlir/Dialect/Arith/Transforms/Passes.h.inc"
-#define GEN_PASS_DECL_ARITHINTRANGEOPTS
-#include "mlir/Dialect/Arith/Transforms/Passes.h.inc"
 
 class WideIntEmulationConverter;
 class NarrowTypeEmulationConverter;
 
-/// Create a pass to bufferize Arith ops.
-std::unique_ptr<Pass> createArithBufferizePass();
-
-/// Create a pass to bufferize arith.constant ops.
-std::unique_ptr<Pass> createConstantBufferizePass(uint64_t alignment = 0);
-
 /// Adds patterns to emulate wide Arith and Function ops over integer
 /// types into supported ones. This is done by splitting original power-of-two
 /// i2N integer types into two iN halves.
@@ -68,13 +60,6 @@ void populateExpandBFloat16Patterns(RewritePatternSet &patterns);
 /// Add patterns to expand Arith ops.
 void populateArithExpandOpsPatterns(RewritePatternSet &patterns);
 
-/// Create a pass to legalize Arith ops.
-std::unique_ptr<Pass> createArithExpandOpsPass();
-
-/// Create a pass to legalize Arith ops with specified configuration.
-std::unique_ptr<Pass>
-createArithExpandOpsPass(const ArithExpandOpsOptions &options);
-
 /// Create a pass to replace signed ops with unsigned ones where they are proven
 /// equivalent.
 std::unique_ptr<Pass> createArithUnsignedWhenEquivalentPass();
