@@ -14,12 +14,14 @@
 #ifndef MLIR_IR_ASMSTATE_H_
 #define MLIR_IR_ASMSTATE_H_
 
+#include "mlir/Bytecode/BytecodeReaderConfig.h"
 #include "mlir/IR/OperationSupport.h"
 #include "mlir/Support/LLVM.h"
 #include "llvm/ADT/MapVector.h"
 #include "llvm/ADT/StringMap.h"
 
 #include <memory>
+#include <variant>
 
 namespace mlir {
 class AsmResourcePrinter;
@@ -144,8 +146,7 @@ public:
   /// known to be of the correct type.
   template <typename T>
   ArrayRef<T> getDataAs() const {
-    return llvm::makeArrayRef<T>((const T *)data.data(),
-                                 data.size() / sizeof(T));
+    return llvm::ArrayRef<T>((const T *)data.data(), data.size() / sizeof(T));
   }
 
   /// Return a mutable reference to the raw underlying data of this blob.
@@ -192,18 +193,19 @@ public:
         llvm::deallocate_buffer, dataIsMutable);
   }
   /// Create a new heap allocated blob and copy the provided data into it.
-  static AsmResourceBlob allocateAndCopy(ArrayRef<char> data, size_t align,
-                                         bool dataIsMutable = true) {
+  static AsmResourceBlob allocateAndCopyWithAlign(ArrayRef<char> data,
+                                                  size_t align,
+                                                  bool dataIsMutable = true) {
     AsmResourceBlob blob = allocate(data.size(), align, dataIsMutable);
     std::memcpy(blob.getMutableData().data(), data.data(), data.size());
     return blob;
   }
   template <typename T>
-  static std::enable_if_t<!std::is_same<T, char>::value, AsmResourceBlob>
-  allocateAndCopy(ArrayRef<T> data, bool dataIsMutable = true) {
-    return allocateAndCopy(
+  static AsmResourceBlob allocateAndCopyInferAlign(ArrayRef<T> data,
+                                                   bool dataIsMutable = true) {
+    return allocateAndCopyWithAlign(
         ArrayRef<char>((const char *)data.data(), data.size() * sizeof(T)),
-        alignof(T));
+        alignof(T), dataIsMutable);
   }
 };
 /// This class provides a simple utility wrapper for creating "unmanaged"
@@ -214,17 +216,19 @@ public:
   /// Create a new unmanaged resource directly referencing the provided data.
   /// `dataIsMutable` indicates if the allocated data can be mutated. By
   /// default, we treat unmanaged blobs as immutable.
-  static AsmResourceBlob allocateWithAlign(ArrayRef<char> data, size_t align,
-                                           bool dataIsMutable = false) {
-    return AsmResourceBlob(data, align, /*deleter=*/{},
-                           /*dataIsMutable=*/false);
+  static AsmResourceBlob
+  allocateWithAlign(ArrayRef<char> data, size_t align,
+                    AsmResourceBlob::DeleterFn deleter = {},
+                    bool dataIsMutable = false) {
+    return AsmResourceBlob(data, align, std::move(deleter), dataIsMutable);
   }
   template <typename T>
-  static AsmResourceBlob allocateInferAlign(ArrayRef<T> data,
-                                            bool dataIsMutable = false) {
+  static AsmResourceBlob
+  allocateInferAlign(ArrayRef<T> data, AsmResourceBlob::DeleterFn deleter = {},
+                     bool dataIsMutable = false) {
     return allocateWithAlign(
         ArrayRef<char>((const char *)data.data(), data.size() * sizeof(T)),
-        alignof(T));
+        alignof(T), std::move(deleter), dataIsMutable);
   }
 };
 
@@ -472,6 +476,11 @@ public:
   /// Returns if the parser should verify the IR after parsing.
   bool shouldVerifyAfterParse() const { return verifyAfterParse; }
 
+  /// Returns the parsing configurations associated to the bytecode read.
+  BytecodeReaderConfig &getBytecodeReaderConfig() const {
+    return const_cast<BytecodeReaderConfig &>(bytecodeReaderConfig);
+  }
+
   /// Return the resource parser registered to the given name, or nullptr if no
   /// parser with `name` is registered.
   AsmResourceParser *getResourceParser(StringRef name) const {
@@ -506,6 +515,7 @@ private:
   bool verifyAfterParse;
   DenseMap<StringRef, std::unique_ptr<AsmResourceParser>> resourceParsers;
   FallbackAsmResourceMap *fallbackResourceMap;
+  BytecodeReaderConfig bytecodeReaderConfig;
 };
 
 //===----------------------------------------------------------------------===//
