@@ -21,7 +21,10 @@
 // We include these two headers because they cannot be practically forward
 // declared, and are effectively language features.
 #include "llvm/Support/Casting.h"
-#include <vector>
+
+// We include this header because large portions of mlir would have to include
+// it anyway.
+#include "llvm/Support/LogicalResult.h"
 
 // Workaround for clang-5 (PR41549)
 #if defined(__clang_major__)
@@ -80,6 +83,7 @@ class TypeSwitch;
 
 // Other common classes.
 class APInt;
+class DynamicAPInt;
 class APSInt;
 class APFloat;
 template <typename Fn>
@@ -122,7 +126,7 @@ template <typename KeyT, typename ValueT,
 using DenseMap = llvm::DenseMap<KeyT, ValueT, KeyInfoT, BucketT>;
 template <typename ValueT, typename ValueInfoT = DenseMapInfo<ValueT>>
 using DenseSet = llvm::DenseSet<ValueT, ValueInfoT>;
-template <typename T, typename Vector = std::vector<T>,
+template <typename T, typename Vector = llvm::SmallVector<T, 0>,
           typename Set = DenseSet<T>, unsigned N = 0>
 using SetVector = llvm::SetVector<T, Vector, Set, N>;
 template <typename AllocatorTy = llvm::MallocAllocator>
@@ -143,12 +147,22 @@ using TypeSwitch = llvm::TypeSwitch<T, ResultT>;
 using llvm::APFloat;
 using llvm::APInt;
 using llvm::APSInt;
+using llvm::DynamicAPInt;
 template <typename Fn>
 using function_ref = llvm::function_ref<Fn>;
 using llvm::iterator_range;
 using llvm::raw_ostream;
 using llvm::SMLoc;
 using llvm::SMRange;
+
+// LogicalResult.
+using llvm::failed;
+using llvm::failure;
+using llvm::FailureOr;
+using llvm::LogicalResult;
+using llvm::ParseResult;
+using llvm::succeeded;
+using llvm::success;
 } // namespace mlir
 
 #endif // MLIR_SUPPORT_LLVM_H
