@@ -14,6 +14,7 @@
 #include "mlir/Dialect/Tensor/IR/Tensor.h"
 #include "mlir/Dialect/Tosa/IR/TosaOps.h"
 #include "mlir/Dialect/Tosa/Transforms/Passes.h"
+#include "mlir/Dialect/Tosa/Utils/ConversionUtils.h"
 #include "mlir/Dialect/Tosa/Utils/QuantUtils.h"
 #include "mlir/Pass/Pass.h"
 #include "mlir/Transforms/GreedyPatternRewriteDriver.h"
@@ -28,122 +29,53 @@ namespace tosa {
 using namespace mlir;
 using namespace mlir::tosa;
 
-/// There are two potential ways implementing broadcast:
-/// a. https://www.tensorflow.org/xla/broadcasting#formal_definition
-/// b. https://numpy.org/doc/stable/user/basics.broadcasting.html
-/// This pass implements b (numpy style) now.
-
-/// In this pass, we insert RESHAPE operators to increase the rank of the
-/// lower rank operand as a first step in the broadcasting process. The TOSA
-/// operators that support broadcast require that the rank of the operands
-/// are equal.
-
-// Examples:
-// If lower=[c], higher=[a, b, c], [c] reshaped into [1, 1, c].
-// If lower=[b, c], higher=[a, b, c], [b, c] reshaped into [1, b, c].
-// If lower=[a], higher=[a, a], [a] reshaped into [1, a].
-// If lower=[a], target=[a, b, a], [a] reshaped into [1, 1, a].
-// If lower=[], target=[a, b, c], [] reshaped into [1, 1, 1].
-
-static LogicalResult
-computeReshapeOutput(ArrayRef<int64_t> higherRankShape,
-                     ArrayRef<int64_t> lowerRankShape,
-                     SmallVectorImpl<int64_t> &reshapeOutputShape) {
-  // Initialize new shapes with [1] * higherRank.
-  int64_t higherRank = higherRankShape.size();
-  int64_t lowerRank = lowerRankShape.size();
-
-  reshapeOutputShape.assign(higherRank, 1);
-
-  int64_t higherRankDim;
-  int64_t lowerRankDim;
-
-  for (int64_t i = higherRank - 1, j = lowerRank - 1; i >= 0 && j >= 0;
-       i--, j--) {
-    higherRankDim = higherRankShape[i];
-    lowerRankDim = lowerRankShape[j];
-
-    if (lowerRankDim == 1 && higherRankDim > 1)
-      reshapeOutputShape[i] = 1;
-    else if ((lowerRankDim > 1 && higherRankDim == 1) ||
-             (lowerRankDim == higherRankDim))
-      reshapeOutputShape[i] = lowerRankDim;
-    else if (higherRankDim != lowerRankDim)
-      return failure();
-  }
-  return success();
-}
+namespace {
 
 /// Common code to create the reshape op where necessary to make the rank of the
-/// operations equal. Returns the updated input1 and input2 for the original
-/// input. The caller is expected to use these to rewrite the original operator
-/// with the RESHAPE now in the graph.
-static LogicalResult reshapeLowerToHigher(PatternRewriter &rewriter,
-                                          Location loc,
-                                          RankedTensorType outputType,
-                                          Value input1, Value input2,
-                                          Value &outInput1, Value &outInput2) {
-  auto input1Ty = input1.getType().dyn_cast<RankedTensorType>();
-  auto input2Ty = input2.getType().dyn_cast<RankedTensorType>();
+/// operations equal. input1 and input2 will be updated when the rank has
+/// changed. The caller is expected to use these to rewrite the original
+/// operator with the RESHAPE now in the graph.
+/// return failure when (1) no reshape needed, or (2) output_type is specified
+/// and it has different rank
+LogicalResult reshapeLowerToHigher(PatternRewriter &rewriter, Location loc,
+                                   RankedTensorType outputType, Value &input1,
+                                   Value &input2) {
+  auto input1Ty = dyn_cast<RankedTensorType>(input1.getType());
+  auto input2Ty = dyn_cast<RankedTensorType>(input2.getType());
 
-  if (!input1Ty || !input2Ty)
-    return failure();
+  if (!input1Ty || !input2Ty) {
+    return rewriter.notifyMatchFailure(loc, "input not a ranked tensor");
+  }
 
   int64_t input1Rank = input1Ty.getRank();
   int64_t input2Rank = input2Ty.getRank();
 
-  Value higherTensorValue, lowerTensorValue;
-  // Cannot rewrite as its already correct.
   if (input1Rank == input2Rank)
-    return failure();
+    return rewriter.notifyMatchFailure(loc,
+                                       "cannot rewrite as its already correct");
 
-  if (input1Rank > input2Rank) {
-    higherTensorValue = input1;
-    lowerTensorValue = input2;
-  } else {
-    higherTensorValue = input2;
-    lowerTensorValue = input1;
+  Value input1Copy = input1;
+  Value input2Copy = input2;
+  if (EqualizeRanks(rewriter, loc, input1Copy, input2Copy).failed()) {
+    return rewriter.notifyMatchFailure(loc, "failed to reshape inputs");
   }
 
-  ArrayRef<int64_t> higherRankShape =
-      higherTensorValue.getType().cast<RankedTensorType>().getShape();
-  (void)higherRankShape;
-  ArrayRef<int64_t> lowerRankShape =
-      lowerTensorValue.getType().cast<RankedTensorType>().getShape();
-
-  SmallVector<int64_t, 4> reshapeOutputShape;
-
-  if (computeReshapeOutput(higherRankShape, lowerRankShape, reshapeOutputShape)
-          .failed())
-    return failure();
-
-  auto reshapeInputType = lowerTensorValue.getType().cast<RankedTensorType>();
-  auto reshapeOutputType = RankedTensorType::get(
-      ArrayRef<int64_t>(reshapeOutputShape), reshapeInputType.getElementType());
-
   // Verify the rank agrees with the output type if the output type is ranked.
   if (outputType) {
-    if (outputType.getShape().size() != reshapeOutputShape.size() ||
-        outputType.getShape().size() != higherRankShape.size())
-      return failure();
+    if (outputType.getRank() !=
+            llvm::cast<RankedTensorType>(input1Copy.getType()).getRank() ||
+        outputType.getRank() !=
+            llvm::cast<RankedTensorType>(input2Copy.getType()).getRank())
+      return rewriter.notifyMatchFailure(
+          loc, "the reshaped type doesn't agrees with the ranked output type");
   }
 
-  auto reshapeLower = rewriter.create<tosa::ReshapeOp>(
-      loc, reshapeOutputType, lowerTensorValue,
-      rewriter.getI64ArrayAttr(reshapeOutputShape));
-
-  if (input1Rank > input2Rank) {
-    outInput1 = higherTensorValue;
-    outInput2 = reshapeLower.getResult();
-  } else {
-    outInput1 = reshapeLower.getResult();
-    outInput2 = higherTensorValue;
-  }
+  input1 = input1Copy;
+  input2 = input2Copy;
 
   return success();
 }
 
-namespace {
 template <typename OpTy>
 struct ConvertTosaOp : public OpRewritePattern<OpTy> {
   using OpRewritePattern<OpTy>::OpRewritePattern;
@@ -155,18 +87,16 @@ struct ConvertTosaOp : public OpRewritePattern<OpTy> {
     Value input2 = tosaBinaryOp.getInput2();
     Value output = tosaBinaryOp.getResult();
 
-    auto outputType = output.getType().dyn_cast<RankedTensorType>();
+    auto outputType = dyn_cast<RankedTensorType>(output.getType());
     if (!outputType)
       return failure();
 
-    Value outInput1, outInput2;
     if (reshapeLowerToHigher(rewriter, tosaBinaryOp.getLoc(), outputType,
-                             input1, input2, outInput1, outInput2)
+                             input1, input2)
             .failed())
       return failure();
 
-    rewriter.replaceOpWithNewOp<OpTy>(tosaBinaryOp, outputType, outInput1,
-                                      outInput2);
+    rewriter.replaceOpWithNewOp<OpTy>(tosaBinaryOp, outputType, input1, input2);
 
     return success();
   }
@@ -185,18 +115,17 @@ struct ConvertTosaOp<tosa::MulOp> : public OpRewritePattern<tosa::MulOp> {
     Value input2 = tosaBinaryOp.getInput2();
     int32_t shift = tosaBinaryOp.getShift();
     Value output = tosaBinaryOp.getResult();
-    auto outputType = output.getType().dyn_cast<RankedTensorType>();
+    auto outputType = dyn_cast<RankedTensorType>(output.getType());
     if (!outputType)
       return failure();
 
-    Value outInput1, outInput2;
     if (reshapeLowerToHigher(rewriter, tosaBinaryOp.getLoc(), outputType,
-                             input1, input2, outInput1, outInput2)
+                             input1, input2)
             .failed())
       return failure();
 
-    rewriter.replaceOpWithNewOp<tosa::MulOp>(tosaBinaryOp, outputType,
-                                             outInput1, outInput2, shift);
+    rewriter.replaceOpWithNewOp<tosa::MulOp>(tosaBinaryOp, outputType, input1,
+                                             input2, shift);
 
     return success();
   }
@@ -217,18 +146,69 @@ struct ConvertTosaOp<tosa::ArithmeticRightShiftOp>
     Value input2 = tosaBinaryOp.getInput2();
     int32_t round = tosaBinaryOp.getRound();
     Value output = tosaBinaryOp.getResult();
-    auto outputType = output.getType().dyn_cast<RankedTensorType>();
+    auto outputType = dyn_cast<RankedTensorType>(output.getType());
     if (!outputType)
       return failure();
 
-    Value outInput1, outInput2;
     if (reshapeLowerToHigher(rewriter, tosaBinaryOp.getLoc(), outputType,
-                             input1, input2, outInput1, outInput2)
+                             input1, input2)
             .failed())
       return failure();
 
     rewriter.replaceOpWithNewOp<tosa::ArithmeticRightShiftOp>(
-        tosaBinaryOp, outputType, outInput1, outInput2, round);
+        tosaBinaryOp, outputType, input1, input2, round);
+
+    return success();
+  }
+};
+
+template <>
+struct ConvertTosaOp<tosa::SelectOp> : public OpRewritePattern<tosa::SelectOp> {
+  using OpRewritePattern<tosa::SelectOp>::OpRewritePattern;
+
+  LogicalResult matchAndRewrite(tosa::SelectOp tosaOp,
+                                PatternRewriter &rewriter) const override {
+
+    Value input1 = tosaOp.getPred();
+    Value input2 = tosaOp.getOnTrue();
+    Value input3 = tosaOp.getOnFalse();
+    Value output = tosaOp.getResult();
+
+    auto outputType = dyn_cast<RankedTensorType>(output.getType());
+    if (!outputType)
+      return rewriter.notifyMatchFailure(tosaOp, "output not a ranked tensor");
+
+    // Apply broadcasting to each pair of inputs separately, and chain them as
+    // compound as below so that the broadcasting happens all at once.
+    bool reshaped1 = reshapeLowerToHigher(rewriter, tosaOp.getLoc(), outputType,
+                                          input1, input2)
+                         .succeeded();
+
+    bool reshaped2 = reshapeLowerToHigher(rewriter, tosaOp.getLoc(), outputType,
+                                          input1, input3)
+                         .succeeded();
+
+    bool reshaped3 = reshapeLowerToHigher(rewriter, tosaOp.getLoc(), outputType,
+                                          input2, input3)
+                         .succeeded();
+
+    if (!reshaped1 && !reshaped2 && !reshaped3)
+      return rewriter.notifyMatchFailure(
+          tosaOp,
+          "cannot rewrite as the rank of all operands is already aligned");
+
+    int32_t result1Rank = cast<RankedTensorType>(input1.getType()).getRank();
+    int32_t result2Rank = cast<RankedTensorType>(input2.getType()).getRank();
+    int32_t result3Rank = cast<RankedTensorType>(input3.getType()).getRank();
+    int32_t outputRank = outputType.getRank();
+
+    if ((result1Rank != result2Rank) || (result2Rank != result3Rank) ||
+        (result1Rank != outputRank))
+      return rewriter.notifyMatchFailure(
+          tosaOp, "not all ranks are aligned with each other");
+
+    rewriter.replaceOpWithNewOp<tosa::SelectOp>(tosaOp, outputType, input1,
+                                                input2, input3);
 
     return success();
   }
@@ -252,7 +232,7 @@ public:
     patterns.add<ConvertTosaOp<tosa::AddOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::SubOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::MulOp>>(ctx);
-    patterns.add<ConvertTosaOp<tosa::DivOp>>(ctx);
+    patterns.add<ConvertTosaOp<tosa::IntDivOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::MaximumOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::MinimumOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::EqualOp>>(ctx);
@@ -264,6 +244,7 @@ public:
     patterns.add<ConvertTosaOp<tosa::LogicalAndOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::LogicalOrOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::LogicalXorOp>>(ctx);
+    patterns.add<ConvertTosaOp<tosa::SelectOp>>(ctx);
     patterns.add<ConvertTosaOp<tosa::PowOp>>(ctx);
     (void)applyPatternsAndFoldGreedily(func, std::move(patterns));
   }
