@@ -18,6 +18,7 @@
 #include "mlir/IR/Threading.h"
 #include "mlir/IR/Verifier.h"
 #include "mlir/Support/FileUtilities.h"
+#include "llvm/ADT/Hashing.h"
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/ScopeExit.h"
 #include "llvm/Support/CommandLine.h"
@@ -26,10 +27,30 @@
 #include "llvm/Support/Signals.h"
 #include "llvm/Support/Threading.h"
 #include "llvm/Support/ToolOutputFile.h"
+#include <optional>
 
 using namespace mlir;
 using namespace mlir::detail;
 
+//===----------------------------------------------------------------------===//
+// PassExecutionAction
+//===----------------------------------------------------------------------===//
+
+PassExecutionAction::PassExecutionAction(ArrayRef<IRUnit> irUnits,
+                                         const Pass &pass)
+    : Base(irUnits), pass(pass) {}
+
+void PassExecutionAction::print(raw_ostream &os) const {
+  os << llvm::formatv("`{0}` running `{1}` on Operation `{2}`", tag,
+                      pass.getName(), getOp()->getName());
+}
+
+Operation *PassExecutionAction::getOp() const {
+  ArrayRef<IRUnit> irUnits = getContextIRUnits();
+  return irUnits.empty() ? nullptr
+                         : llvm::dyn_cast_if_present<Operation *>(irUnits[0]);
+}
+
 //===----------------------------------------------------------------------===//
 // Pass
 //===----------------------------------------------------------------------===//
@@ -39,8 +60,16 @@ using namespace mlir::detail;
 void Pass::anchor() {}
 
 /// Attempt to initialize the options of this pass from the given string.
-LogicalResult Pass::initializeOptions(StringRef options) {
-  return passOptions.parseFromString(options);
+LogicalResult Pass::initializeOptions(
+    StringRef options,
+    function_ref<LogicalResult(const Twine &)> errorHandler) {
+  std::string errStr;
+  llvm::raw_string_ostream os(errStr);
+  if (failed(passOptions.parseFromString(options, os))) {
+    os.flush();
+    return errorHandler(errStr);
+  }
+  return success();
 }
 
 /// Copy the option values from 'other', which is another instance of this
@@ -50,17 +79,13 @@ void Pass::copyOptionValuesFrom(const Pass *other) {
 }
 
 /// Prints out the pass in the textual representation of pipelines. If this is
-/// an adaptor pass, print with the op_name(sub_pass,...) format.
+/// an adaptor pass, print its pass managers.
 void Pass::printAsTextualPipeline(raw_ostream &os) {
-  // Special case for adaptors to use the 'op_name(sub_passes)' format.
+  // Special case for adaptors to print its pass managers.
   if (auto *adaptor = dyn_cast<OpToOpPassAdaptor>(this)) {
     llvm::interleave(
         adaptor->getPassManagers(),
-        [&](OpPassManager &pm) {
-          os << pm.getOpAnchorName() << "(";
-          pm.printAsTextualPipeline(os);
-          os << ")";
-        },
+        [&](OpPassManager &pm) { pm.printAsTextualPipeline(os); },
         [&] { os << ","; });
     return;
   }
@@ -131,13 +156,14 @@ struct OpPassManagerImpl {
   LogicalResult finalizePassList(MLIRContext *ctx);
 
   /// Return the operation name of this pass manager.
-  Optional<OperationName> getOpName(MLIRContext &context) {
+  std::optional<OperationName> getOpName(MLIRContext &context) {
     if (!name.empty() && !opName)
       opName = OperationName(name, &context);
     return opName;
   }
-  Optional<StringRef> getOpName() const {
-    return name.empty() ? Optional<StringRef>() : Optional<StringRef>(name);
+  std::optional<StringRef> getOpName() const {
+    return name.empty() ? std::optional<StringRef>()
+                        : std::optional<StringRef>(name);
   }
 
   /// Return the name used to anchor this pass manager. This is either the name
@@ -156,7 +182,7 @@ struct OpPassManagerImpl {
 
   /// The cached OperationName (internalized in the context) for the name of the
   /// operation that passes of this pass manager operate on.
-  Optional<OperationName> opName;
+  std::optional<OperationName> opName;
 
   /// The set of passes to run as part of this pass manager.
   std::vector<std::unique_ptr<Pass>> passes;
@@ -188,8 +214,8 @@ OpPassManager &OpPassManagerImpl::nest(OpPassManager &&nested) {
 void OpPassManagerImpl::addPass(std::unique_ptr<Pass> pass) {
   // If this pass runs on a different operation than this pass manager, then
   // implicitly nest a pass manager for this operation if enabled.
-  Optional<StringRef> pmOpName = getOpName();
-  Optional<StringRef> passOpName = pass->getOpName();
+  std::optional<StringRef> pmOpName = getOpName();
+  std::optional<StringRef> passOpName = pass->getOpName();
   if (pmOpName && passOpName && *pmOpName != *passOpName) {
     if (nesting == OpPassManager::Nesting::Implicit)
       return nest(*passOpName).addPass(std::move(pass));
@@ -247,13 +273,14 @@ LogicalResult OpPassManagerImpl::finalizePassList(MLIRContext *ctx) {
   llvm::erase_if(passes, std::logical_not<std::unique_ptr<Pass>>());
 
   // If this is a op-agnostic pass manager, there is nothing left to do.
-  Optional<OperationName> rawOpName = getOpName(*ctx);
+  std::optional<OperationName> rawOpName = getOpName(*ctx);
   if (!rawOpName)
     return success();
 
   // Otherwise, verify that all of the passes are valid for the current
   // operation anchor.
-  Optional<RegisteredOperationName> opName = rawOpName->getRegisteredInfo();
+  std::optional<RegisteredOperationName> opName =
+      rawOpName->getRegisteredInfo();
   for (std::unique_ptr<Pass> &pass : passes) {
     if (opName && !pass->canScheduleOn(*opName)) {
       return emitError(UnknownLoc::get(ctx))
@@ -269,13 +296,14 @@ bool OpPassManagerImpl::canScheduleOn(MLIRContext &context,
                                       OperationName opName) {
   // If this pass manager is op-specific, we simply check if the provided
   // operation name is the same as this one.
-  Optional<OperationName> pmOpName = getOpName(context);
+  std::optional<OperationName> pmOpName = getOpName(context);
   if (pmOpName)
     return pmOpName == opName;
 
   // Otherwise, this is an op-agnostic pass manager. Check that the operation
   // can be scheduled on all passes within the manager.
-  Optional<RegisteredOperationName> registeredInfo = opName.getRegisteredInfo();
+  std::optional<RegisteredOperationName> registeredInfo =
+      opName.getRegisteredInfo();
   if (!registeredInfo ||
       !registeredInfo->hasTrait<OpTrait::IsIsolatedFromAbove>())
     return false;
@@ -294,12 +322,16 @@ OpPassManager::OpPassManager(StringRef name, Nesting nesting)
     : impl(new OpPassManagerImpl(name, nesting)) {}
 OpPassManager::OpPassManager(OperationName name, Nesting nesting)
     : impl(new OpPassManagerImpl(name, nesting)) {}
-OpPassManager::OpPassManager(OpPassManager &&rhs) : impl(std::move(rhs.impl)) {}
+OpPassManager::OpPassManager(OpPassManager &&rhs) { *this = std::move(rhs); }
 OpPassManager::OpPassManager(const OpPassManager &rhs) { *this = rhs; }
 OpPassManager &OpPassManager::operator=(const OpPassManager &rhs) {
   impl = std::make_unique<OpPassManagerImpl>(*rhs.impl);
   return *this;
 }
+OpPassManager &OpPassManager::operator=(OpPassManager &&rhs) {
+  impl = std::move(rhs.impl);
+  return *this;
+}
 
 OpPassManager::~OpPassManager() = default;
 
@@ -342,12 +374,13 @@ size_t OpPassManager::size() const { return impl->passes.size(); }
 OpPassManagerImpl &OpPassManager::getImpl() { return *impl; }
 
 /// Return the operation name that this pass manager operates on.
-Optional<StringRef> OpPassManager::getOpName() const {
+std::optional<StringRef> OpPassManager::getOpName() const {
   return impl->getOpName();
 }
 
 /// Return the operation name that this pass manager operates on.
-Optional<OperationName> OpPassManager::getOpName(MLIRContext &context) const {
+std::optional<OperationName>
+OpPassManager::getOpName(MLIRContext &context) const {
   return impl->getOpName(context);
 }
 
@@ -355,26 +388,28 @@ StringRef OpPassManager::getOpAnchorName() const {
   return impl->getOpAnchorName();
 }
 
-/// Prints out the given passes as the textual representation of a pipeline.
-static void printAsTextualPipeline(ArrayRef<std::unique_ptr<Pass>> passes,
-                                   raw_ostream &os) {
-  llvm::interleave(
-      passes,
-      [&](const std::unique_ptr<Pass> &pass) {
-        pass->printAsTextualPipeline(os);
-      },
-      [&] { os << ","; });
-}
-
 /// Prints out the passes of the pass manager as the textual representation
 /// of pipelines.
+void printAsTextualPipeline(
+    raw_ostream &os, StringRef anchorName,
+    const llvm::iterator_range<OpPassManager::pass_iterator> &passes) {
+  os << anchorName << "(";
+  llvm::interleave(
+      passes, [&](mlir::Pass &pass) { pass.printAsTextualPipeline(os); },
+      [&]() { os << ","; });
+  os << ")";
+}
 void OpPassManager::printAsTextualPipeline(raw_ostream &os) const {
-  ::printAsTextualPipeline(impl->passes, os);
+  StringRef anchorName = getOpAnchorName();
+  ::printAsTextualPipeline(
+      os, anchorName,
+      {MutableArrayRef<std::unique_ptr<Pass>>{impl->passes}.begin(),
+       MutableArrayRef<std::unique_ptr<Pass>>{impl->passes}.end()});
 }
 
 void OpPassManager::dump() {
-  llvm::errs() << "Pass Manager with " << impl->passes.size() << " passes: ";
-  ::printAsTextualPipeline(impl->passes, llvm::errs());
+  llvm::errs() << "Pass Manager with " << impl->passes.size() << " passes:\n";
+  printAsTextualPipeline(llvm::errs());
   llvm::errs() << "\n";
 }
 
@@ -414,6 +449,23 @@ LogicalResult OpPassManager::initialize(MLIRContext *context,
   return success();
 }
 
+llvm::hash_code OpPassManager::hash() {
+  llvm::hash_code hashCode{};
+  for (Pass &pass : getPasses()) {
+    // If this pass isn't an adaptor, directly hash it.
+    auto *adaptor = dyn_cast<OpToOpPassAdaptor>(&pass);
+    if (!adaptor) {
+      hashCode = llvm::hash_combine(hashCode, &pass);
+      continue;
+    }
+    // Otherwise, hash recursively each of the adaptors pass managers.
+    for (OpPassManager &adaptorPM : adaptor->getPassManagers())
+      llvm::hash_combine(hashCode, adaptorPM.hash());
+  }
+  return hashCode;
+}
+
+
 //===----------------------------------------------------------------------===//
 // OpToOpPassAdaptor
 //===----------------------------------------------------------------------===//
@@ -421,13 +473,16 @@ LogicalResult OpPassManager::initialize(MLIRContext *context,
 LogicalResult OpToOpPassAdaptor::run(Pass *pass, Operation *op,
                                      AnalysisManager am, bool verifyPasses,
                                      unsigned parentInitGeneration) {
-  Optional<RegisteredOperationName> opInfo = op->getRegisteredInfo();
+  std::optional<RegisteredOperationName> opInfo = op->getRegisteredInfo();
   if (!opInfo)
     return op->emitOpError()
            << "trying to schedule a pass on an unregistered operation";
   if (!opInfo->hasTrait<OpTrait::IsIsolatedFromAbove>())
     return op->emitOpError() << "trying to schedule a pass on an operation not "
                                 "marked as 'IsolatedFromAbove'";
+  if (!pass->canScheduleOn(*op->getName().getRegisteredInfo()))
+    return op->emitOpError()
+           << "trying to schedule a pass on an unsupported operation";
 
   // Initialize the pass state with a callback for the pass to dynamically
   // execute a pipeline on the currently visited operation.
@@ -462,12 +517,17 @@ LogicalResult OpToOpPassAdaptor::run(Pass *pass, Operation *op,
   if (pi)
     pi->runBeforePass(pass, op);
 
-  // Invoke the virtual runOnOperation method.
-  if (auto *adaptor = dyn_cast<OpToOpPassAdaptor>(pass))
-    adaptor->runOnOperation(verifyPasses);
-  else
-    pass->runOnOperation();
-  bool passFailed = pass->passState->irAndPassFailed.getInt();
+  bool passFailed = false;
+  op->getContext()->executeAction<PassExecutionAction>(
+      [&]() {
+        // Invoke the virtual runOnOperation method.
+        if (auto *adaptor = dyn_cast<OpToOpPassAdaptor>(pass))
+          adaptor->runOnOperation(verifyPasses);
+        else
+          pass->runOnOperation();
+        passFailed = pass->passState->irAndPassFailed.getInt();
+      },
+      {op}, *pass);
 
   // Invalidate any non preserved analyses.
   am.invalidate(pass->passState->preservedAnalyses);
@@ -582,7 +642,7 @@ LogicalResult OpToOpPassAdaptor::tryMergeInto(MLIRContext *ctx,
       // If this is a non-generic pass manager, a conflict will arise if a
       // non-generic pass manager's operation name can be scheduled on the
       // generic passmanager.
-      if (Optional<OperationName> pmOpName = pm.getOpName(*ctx))
+      if (std::optional<OperationName> pmOpName = pm.getOpName(*ctx))
         return genericPM.getImpl().canScheduleOn(*ctx, *pmOpName);
       // Otherwise, this is a generic pass manager. We current can't determine
       // when generic pass managers can be merged, so conservatively assume they
@@ -621,8 +681,8 @@ LogicalResult OpToOpPassAdaptor::tryMergeInto(MLIRContext *ctx,
   // After coalescing, sort the pass managers within rhs by name.
   auto compareFn = [](const OpPassManager *lhs, const OpPassManager *rhs) {
     // Order op-specific pass managers first and op-agnostic pass managers last.
-    if (Optional<StringRef> lhsName = lhs->getOpName()) {
-      if (Optional<StringRef> rhsName = rhs->getOpName())
+    if (std::optional<StringRef> lhsName = lhs->getOpName()) {
+      if (std::optional<StringRef> rhsName = rhs->getOpName())
         return lhsName->compare(*rhsName);
       return -1; // lhs(op-specific) < rhs(op-agnostic)
     }
@@ -696,7 +756,7 @@ void OpToOpPassAdaptor::runOnOperationAsyncImpl(bool verifyPasses) {
   // Create the async executors if they haven't been created, or if the main
   // pipeline has changed.
   if (asyncExecutors.empty() || hasSizeMismatch(asyncExecutors.front(), mgrs))
-    asyncExecutors.assign(context->getThreadPool().getThreadCount(), mgrs);
+    asyncExecutors.assign(context->getThreadPool().getMaxConcurrency(), mgrs);
 
   // This struct represents the information for a single operation to be
   // scheduled on a pass manager.
@@ -716,11 +776,11 @@ void OpToOpPassAdaptor::runOnOperationAsyncImpl(bool verifyPasses) {
   // execute over. This ensures that an analysis manager exists for each
   // operation, as well as providing a queue of operations to execute over.
   std::vector<OpPMInfo> opInfos;
-  DenseMap<OperationName, Optional<unsigned>> knownOpPMIdx;
+  DenseMap<OperationName, std::optional<unsigned>> knownOpPMIdx;
   for (auto &region : getOperation()->getRegions()) {
     for (Operation &op : region.getOps()) {
       // Get the pass manager index for this operation type.
-      auto pmIdxIt = knownOpPMIdx.try_emplace(op.getName(), llvm::None);
+      auto pmIdxIt = knownOpPMIdx.try_emplace(op.getName(), std::nullopt);
       if (pmIdxIt.second) {
         if (auto *mgr = findPassManagerFor(mgrs, op.getName(), *context))
           pmIdxIt.first->second = std::distance(mgrs.begin(), mgr);
@@ -768,11 +828,15 @@ void OpToOpPassAdaptor::runOnOperationAsyncImpl(bool verifyPasses) {
 // PassManager
 //===----------------------------------------------------------------------===//
 
-PassManager::PassManager(MLIRContext *ctx, Nesting nesting,
-                         StringRef operationName)
-    : OpPassManager(OperationName(operationName, ctx), nesting), context(ctx),
-      initializationKey(DenseMapInfo<llvm::hash_code>::getTombstoneKey()),
-      passTiming(false), verifyPasses(true) {}
+PassManager::PassManager(MLIRContext *ctx, StringRef operationName,
+                         Nesting nesting)
+    : OpPassManager(operationName, nesting), context(ctx), passTiming(false),
+      verifyPasses(true) {}
+
+PassManager::PassManager(OperationName operationName, Nesting nesting)
+    : OpPassManager(operationName, nesting),
+      context(operationName.getContext()), passTiming(false),
+      verifyPasses(true) {}
 
 PassManager::~PassManager() = default;
 
@@ -781,9 +845,11 @@ void PassManager::enableVerifier(bool enabled) { verifyPasses = enabled; }
 /// Run the passes within this manager on the provided operation.
 LogicalResult PassManager::run(Operation *op) {
   MLIRContext *context = getContext();
-  assert(op->getName() == getOpName(*context) &&
-         "operation has a different name than the PassManager or is from a "
-         "different context");
+  std::optional<OperationName> anchorOp = getOpName(*context);
+  if (anchorOp && anchorOp != op->getName())
+    return emitError(op->getLoc())
+           << "can't run '" << getOpAnchorName() << "' pass manager on '"
+           << op->getName() << "' op";
 
   // Register all dialects for the current pipeline.
   DialectRegistry dependentDialects;
@@ -796,20 +862,22 @@ LogicalResult PassManager::run(Operation *op) {
   if (failed(getImpl().finalizePassList(context)))
     return failure();
 
+  // Notify the context that we start running a pipeline for bookkeeping.
+  context->enterMultiThreadedExecution();
+
   // Initialize all of the passes within the pass manager with a new generation.
   llvm::hash_code newInitKey = context->getRegistryHash();
-  if (newInitKey != initializationKey) {
+  llvm::hash_code pipelineKey = hash();
+  if (newInitKey != initializationKey || pipelineKey != pipelineInitializationKey) {
     if (failed(initialize(context, impl->initializationGeneration + 1)))
       return failure();
     initializationKey = newInitKey;
+    pipelineKey = pipelineInitializationKey;
   }
 
   // Construct a top level analysis manager for the pipeline.
   ModuleAnalysisManager am(op, instrumentor.get());
 
-  // Notify the context that we start running a pipeline for book keeping.
-  context->enterMultiThreadedExecution();
-
   // If reproducer generation is enabled, run the pass manager with crash
   // handling enabled.
   LogicalResult result =
@@ -914,10 +982,10 @@ void detail::NestedAnalysisMap::invalidate(
 PassInstrumentation::~PassInstrumentation() = default;
 
 void PassInstrumentation::runBeforePipeline(
-    Optional<OperationName> name, const PipelineParentInfo &parentInfo) {}
+    std::optional<OperationName> name, const PipelineParentInfo &parentInfo) {}
 
 void PassInstrumentation::runAfterPipeline(
-    Optional<OperationName> name, const PipelineParentInfo &parentInfo) {}
+    std::optional<OperationName> name, const PipelineParentInfo &parentInfo) {}
 
 //===----------------------------------------------------------------------===//
 // PassInstrumentor
@@ -940,7 +1008,7 @@ PassInstrumentor::~PassInstrumentor() = default;
 
 /// See PassInstrumentation::runBeforePipeline for details.
 void PassInstrumentor::runBeforePipeline(
-    Optional<OperationName> name,
+    std::optional<OperationName> name,
     const PassInstrumentation::PipelineParentInfo &parentInfo) {
   llvm::sys::SmartScopedLock<true> instrumentationLock(impl->mutex);
   for (auto &instr : impl->instrumentations)
@@ -949,7 +1017,7 @@ void PassInstrumentor::runBeforePipeline(
 
 /// See PassInstrumentation::runAfterPipeline for details.
 void PassInstrumentor::runAfterPipeline(
-    Optional<OperationName> name,
+    std::optional<OperationName> name,
     const PassInstrumentation::PipelineParentInfo &parentInfo) {
   llvm::sys::SmartScopedLock<true> instrumentationLock(impl->mutex);
   for (auto &instr : llvm::reverse(impl->instrumentations))
