@@ -8,8 +8,10 @@
 
 #include "mlir/TableGen/AttrOrTypeDef.h"
 #include "mlir/TableGen/Dialect.h"
+#include "llvm/ADT/FunctionExtras.h"
 #include "llvm/ADT/SmallPtrSet.h"
 #include "llvm/ADT/StringExtras.h"
+#include "llvm/Support/ErrorHandling.h"
 #include "llvm/TableGen/Error.h"
 #include "llvm/TableGen/Record.h"
 
@@ -20,9 +22,9 @@ using namespace mlir::tblgen;
 // AttrOrTypeBuilder
 //===----------------------------------------------------------------------===//
 
-Optional<StringRef> AttrOrTypeBuilder::getReturnType() const {
-  Optional<StringRef> type = def->getValueAsOptionalString("returnType");
-  return type && !type->empty() ? type : llvm::None;
+std::optional<StringRef> AttrOrTypeBuilder::getReturnType() const {
+  std::optional<StringRef> type = def->getValueAsOptionalString("returnType");
+  return type && !type->empty() ? type : std::nullopt;
 }
 
 bool AttrOrTypeBuilder::hasInferredContextParameter() const {
@@ -56,9 +58,23 @@ AttrOrTypeDef::AttrOrTypeDef(const llvm::Record *def) : def(def) {
   if (auto *traitList = def->getValueAsListInit("traits")) {
     SmallPtrSet<const llvm::Init *, 32> traitSet;
     traits.reserve(traitSet.size());
-    for (auto *traitInit : *traitList)
-      if (traitSet.insert(traitInit).second)
-        traits.push_back(Trait::create(traitInit));
+    llvm::unique_function<void(llvm::ListInit *)> processTraitList =
+        [&](llvm::ListInit *traitList) {
+          for (auto *traitInit : *traitList) {
+            if (!traitSet.insert(traitInit).second)
+              continue;
+
+            // If this is an interface, add any bases to the trait list.
+            auto *traitDef = cast<llvm::DefInit>(traitInit)->getDef();
+            if (traitDef->isSubClassOf("Interface")) {
+              if (auto *bases = traitDef->getValueAsListInit("baseInterfaces"))
+                processTraitList(bases);
+            }
+
+            traits.push_back(Trait::create(traitInit));
+          }
+        };
+    processTraitList(traitList);
   }
 
   // Populate the parameters.
@@ -148,7 +164,7 @@ unsigned AttrOrTypeDef::getNumParameters() const {
   return parametersDag ? parametersDag->getNumArgs() : 0;
 }
 
-Optional<StringRef> AttrOrTypeDef::getMnemonic() const {
+std::optional<StringRef> AttrOrTypeDef::getMnemonic() const {
   return def->getValueAsOptionalString("mnemonic");
 }
 
@@ -156,7 +172,7 @@ bool AttrOrTypeDef::hasCustomAssemblyFormat() const {
   return def->getValueAsBit("hasCustomAssemblyFormat");
 }
 
-Optional<StringRef> AttrOrTypeDef::getAssemblyFormat() const {
+std::optional<StringRef> AttrOrTypeDef::getAssemblyFormat() const {
   return def->getValueAsOptionalString("assemblyFormat");
 }
 
@@ -168,14 +184,14 @@ bool AttrOrTypeDef::genVerifyDecl() const {
   return def->getValueAsBit("genVerifyDecl");
 }
 
-Optional<StringRef> AttrOrTypeDef::getExtraDecls() const {
+std::optional<StringRef> AttrOrTypeDef::getExtraDecls() const {
   auto value = def->getValueAsString("extraClassDeclaration");
-  return value.empty() ? Optional<StringRef>() : value;
+  return value.empty() ? std::optional<StringRef>() : value;
 }
 
-Optional<StringRef> AttrOrTypeDef::getExtraDefs() const {
+std::optional<StringRef> AttrOrTypeDef::getExtraDefs() const {
   auto value = def->getValueAsString("extraClassDefinition");
-  return value.empty() ? Optional<StringRef>() : value;
+  return value.empty() ? std::optional<StringRef>() : value;
 }
 
 ArrayRef<SMLoc> AttrOrTypeDef::getLoc() const { return def->getLoc(); }
@@ -196,7 +212,7 @@ bool AttrOrTypeDef::operator<(const AttrOrTypeDef &other) const {
 // AttrDef
 //===----------------------------------------------------------------------===//
 
-Optional<StringRef> AttrDef::getTypeBuilder() const {
+std::optional<StringRef> AttrDef::getTypeBuilder() const {
   return def->getValueAsOptionalString("typeBuilder");
 }
 
@@ -204,13 +220,29 @@ bool AttrDef::classof(const AttrOrTypeDef *def) {
   return def->getDef()->isSubClassOf("AttrDef");
 }
 
+StringRef AttrDef::getAttrName() const {
+  return def->getValueAsString("attrName");
+}
+
+//===----------------------------------------------------------------------===//
+// TypeDef
+//===----------------------------------------------------------------------===//
+
+bool TypeDef::classof(const AttrOrTypeDef *def) {
+  return def->getDef()->isSubClassOf("TypeDef");
+}
+
+StringRef TypeDef::getTypeName() const {
+  return def->getValueAsString("typeName");
+}
+
 //===----------------------------------------------------------------------===//
 // AttrOrTypeParameter
 //===----------------------------------------------------------------------===//
 
 template <typename InitT>
 auto AttrOrTypeParameter::getDefValue(StringRef name) const {
-  Optional<decltype(std::declval<InitT>().getValue())> result;
+  std::optional<decltype(std::declval<InitT>().getValue())> result;
   if (auto *param = dyn_cast<llvm::DefInit>(getDef()))
     if (auto *init = param->getDef()->getValue(name))
       if (auto *value = dyn_cast_or_null<InitT>(init->getValue()))
@@ -231,7 +263,7 @@ std::string AttrOrTypeParameter::getAccessorName() const {
          llvm::convertToCamelFromSnakeCase(getName(), /*capitalizeFirst=*/true);
 }
 
-Optional<StringRef> AttrOrTypeParameter::getAllocator() const {
+std::optional<StringRef> AttrOrTypeParameter::getAllocator() const {
   return getDefValue<llvm::StringInit>("allocator");
 }
 
@@ -242,7 +274,18 @@ StringRef AttrOrTypeParameter::getComparator() const {
 StringRef AttrOrTypeParameter::getCppType() const {
   if (auto *stringType = dyn_cast<llvm::StringInit>(getDef()))
     return stringType->getValue();
-  return getDefValue<llvm::StringInit>("cppType").value();
+  auto cppType = getDefValue<llvm::StringInit>("cppType");
+  if (cppType)
+    return *cppType;
+  if (auto *init = dyn_cast<llvm::DefInit>(getDef()))
+    llvm::PrintFatalError(
+        init->getDef()->getLoc(),
+        Twine("Missing `cppType` field in Attribute/Type parameter: ") +
+            init->getAsString());
+  llvm::report_fatal_error(
+      Twine("Missing `cppType` field in Attribute/Type parameter: ") +
+          getDef()->getAsString(),
+      /*gen_crash_diag=*/false);
 }
 
 StringRef AttrOrTypeParameter::getCppAccessorType() const {
@@ -258,15 +301,15 @@ StringRef AttrOrTypeParameter::getConvertFromStorage() const {
   return getDefValue<llvm::StringInit>("convertFromStorage").value_or("$_self");
 }
 
-Optional<StringRef> AttrOrTypeParameter::getParser() const {
+std::optional<StringRef> AttrOrTypeParameter::getParser() const {
   return getDefValue<llvm::StringInit>("parser");
 }
 
-Optional<StringRef> AttrOrTypeParameter::getPrinter() const {
+std::optional<StringRef> AttrOrTypeParameter::getPrinter() const {
   return getDefValue<llvm::StringInit>("printer");
 }
 
-Optional<StringRef> AttrOrTypeParameter::getSummary() const {
+std::optional<StringRef> AttrOrTypeParameter::getSummary() const {
   return getDefValue<llvm::StringInit>("summary");
 }
 
@@ -280,9 +323,10 @@ bool AttrOrTypeParameter::isOptional() const {
   return getDefaultValue().has_value();
 }
 
-Optional<StringRef> AttrOrTypeParameter::getDefaultValue() const {
-  Optional<StringRef> result = getDefValue<llvm::StringInit>("defaultValue");
-  return result && !result->empty() ? result : llvm::None;
+std::optional<StringRef> AttrOrTypeParameter::getDefaultValue() const {
+  std::optional<StringRef> result =
+      getDefValue<llvm::StringInit>("defaultValue");
+  return result && !result->empty() ? result : std::nullopt;
 }
 
 llvm::Init *AttrOrTypeParameter::getDef() const { return def->getArg(index); }
