@@ -20,6 +20,7 @@
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/Support/Debug.h"
+#include <optional>
 
 using namespace mlir;
 
@@ -73,10 +74,9 @@ Value spirv::Deserializer::getValue(uint32_t id) {
   return valueMap.lookup(id);
 }
 
-LogicalResult
-spirv::Deserializer::sliceInstruction(spirv::Opcode &opcode,
-                                      ArrayRef<uint32_t> &operands,
-                                      Optional<spirv::Opcode> expectedOpcode) {
+LogicalResult spirv::Deserializer::sliceInstruction(
+    spirv::Opcode &opcode, ArrayRef<uint32_t> &operands,
+    std::optional<spirv::Opcode> expectedOpcode) {
   auto binarySize = binary.size();
   if (curOffset >= binarySize) {
     return emitError(unknownLoc, "expected ")
@@ -164,7 +164,7 @@ LogicalResult spirv::Deserializer::processInstruction(
   case spirv::Opcode::OpTypeRuntimeArray:
   case spirv::Opcode::OpTypeStruct:
   case spirv::Opcode::OpTypePointer:
-  case spirv::Opcode::OpTypeCooperativeMatrixNV:
+  case spirv::Opcode::OpTypeCooperativeMatrixKHR:
     return processType(opcode, operands);
   case spirv::Opcode::OpTypeForwardPointer:
     return processTypeForwardPointer(operands);
@@ -353,7 +353,7 @@ Deserializer::processOp<spirv::EntryPointOp>(ArrayRef<uint32_t> words) {
     // The deserializer uses "spirv_fn_<id>" as the function name if the input
     // SPIR-V blob does not contain a name for it. We should use a more clear
     // indication for such case rather than relying on naming details.
-    if (!parsedFunc.getName().startswith("spirv_fn_"))
+    if (!parsedFunc.getName().starts_with("spirv_fn_"))
       return emitError(unknownLoc,
                        "function name mismatch between OpEntryPoint "
                        "and OpFunction with <id> ")
@@ -489,7 +489,8 @@ Deserializer::processOp<spirv::CopyMemoryOp>(ArrayRef<uint32_t> words) {
     auto attrValue = words[wordIndex++];
     auto attr = opBuilder.getAttr<spirv::MemoryAccessAttr>(
         static_cast<spirv::MemoryAccess>(attrValue));
-    attributes.push_back(opBuilder.getNamedAttr("memory_access", attr));
+    attributes.push_back(
+        opBuilder.getNamedAttr(attributeName<MemoryAccess>(), attr));
     isAlignedAttr = (attrValue == 2);
   }
 
