@@ -20,7 +20,7 @@ extension SymbolGraph.Symbol {
      For example,
      a class introduced in iOS 11 would have:
 
-     - a availability domain of `"iOS"` and
+     - an availability domain of `"iOS"` and
      - an `introduced` version of `11.0.0`.
 
      As another example,
@@ -59,3 +59,11 @@ extension SymbolGraph.Symbol {
         (mixins[Availability.mixinKey] as? Availability)?.availability
     }
 }
+
+extension UnifiedSymbolGraph.Symbol {
+    public var availability: [UnifiedSymbolGraph.Selector: [SymbolGraph.Symbol.Availability.AvailabilityItem]] {
+        mixins.compactMapValues({ mixins in
+            (mixins[SymbolGraph.Symbol.Availability.mixinKey] as? SymbolGraph.Symbol.Availability)?.availability
+        })
+    }
+}
