@@ -1,7 +1,7 @@
 /*
  This source file is part of the Swift.org open source project
 
- Copyright (c) 2021-2022 Apple Inc. and the Swift project authors
+ Copyright (c) 2021-2024 Apple Inc. and the Swift project authors
  Licensed under Apache License v2.0 with Runtime Library Exception
 
  See https://swift.org/LICENSE.txt for license information
@@ -19,7 +19,7 @@ extension SymbolGraph.Symbol {
         
         /// Create a new ``SymbolGraph/Symbol/KindIdentifier``.
         ///
-        /// - Note: Only use this initilaizer for defining a new kind. For initializing instances manually,
+        /// - Note: Only use this initializer for defining a new kind. For initializing instances manually,
         /// copy the initial initializer. For extracting identifiers form raw strings, use ``init(identifier:)``.
         public init(rawValue: String) {
             self.rawValue = rawValue
@@ -46,7 +46,9 @@ extension SymbolGraph.Symbol {
         public static let macro = KindIdentifier(rawValue: "macro")
         
         public static let method = KindIdentifier(rawValue: "method")
-        
+
+        public static let namespace = KindIdentifier(rawValue: "namespace")
+
         public static let property = KindIdentifier(rawValue: "property")
         
         public static let `protocol` = KindIdentifier(rawValue: "protocol")
@@ -66,7 +68,9 @@ extension SymbolGraph.Symbol {
         public static let typeSubscript = KindIdentifier(rawValue: "type.subscript")
         
         public static let `typealias` = KindIdentifier(rawValue: "typealias")
-        
+
+        public static let union = KindIdentifier(rawValue: "union")
+
         public static let `var` = KindIdentifier(rawValue: "var")
         
         public static let module = KindIdentifier(rawValue: "module")
@@ -108,6 +112,7 @@ extension SymbolGraph.Symbol {
             Self.ivar.rawValue: .ivar,
             Self.macro.rawValue: .macro,
             Self.method.rawValue: .method,
+            Self.namespace.rawValue: .namespace,
             Self.property.rawValue: .property,
             Self.protocol.rawValue: .protocol,
             Self.snippet.rawValue: .snippet,
@@ -118,6 +123,7 @@ extension SymbolGraph.Symbol {
             Self.typeProperty.rawValue: .typeProperty,
             Self.typeSubscript.rawValue: .typeSubscript,
             Self.typealias.rawValue: .typealias,
+            Self.union.rawValue: .union,
             Self.var.rawValue: .var,
             Self.module.rawValue: .module,
             Self.extension.rawValue: .extension,
@@ -231,6 +237,23 @@ extension SymbolGraph.Symbol {
             }
             return identifier
         }
+
+        /// Whether the kind supports grouping as overloads.
+        public var isOverloadableKind: Bool {
+            switch self {
+            case .method,
+                 .typeMethod,
+                 .`func`,
+                 .`init`,
+                 .macro,
+                 .subscript,
+                 .typeSubscript,
+                 .`operator`:
+                return true
+            default:
+                return false
+            }
+        }
     }
 }
 
