@@ -69,7 +69,7 @@ let package = Package(
     /// Driver tests.
     .testTarget(
       name: "SwiftDriverTests",
-      dependencies: ["SwiftDriver", "SwiftDriverExecution", "TestUtilities"]),
+      dependencies: ["SwiftDriver", "SwiftDriverExecution", "TestUtilities", "ToolingTestShim"]),
 
     /// IncrementalImport tests
     .testTarget(
@@ -93,6 +93,11 @@ let package = Package(
       dependencies: ["SwiftDriver", "SwiftDriverExecution"],
       path: "Tests/TestUtilities"),
 
+    .target(
+      name: "ToolingTestShim",
+      dependencies: ["SwiftDriver"],
+      path: "Tests/ToolingTestShim"),
+
     /// The options library.
     .target(
       name: "SwiftOptions",
@@ -120,7 +125,7 @@ let package = Package(
       ],
       exclude: ["CMakeLists.txt"]),
 
-    /// The help executable.
+    /// Build SDK Interfaces tool executable.
     .executableTarget(
       name: "swift-build-sdk-interfaces",
       dependencies: ["SwiftDriver", "SwiftDriverExecution"],
@@ -141,7 +146,7 @@ let package = Package(
 if ProcessInfo.processInfo.environment["SWIFT_DRIVER_LLBUILD_FWK"] == nil {
     if ProcessInfo.processInfo.environment["SWIFTCI_USE_LOCAL_DEPS"] == nil {
         package.dependencies += [
-            .package(url: "https://github.com/apple/swift-llbuild.git", branch: "release/6.0"),
+            .package(url: "https://github.com/apple/swift-llbuild.git", branch: "release/6.1"),
         ]
         package.targets.first(where: { $0.name == "SwiftDriverExecution" })!.dependencies += [
             .product(name: "llbuildSwift", package: "swift-llbuild"),
@@ -160,13 +165,13 @@ if ProcessInfo.processInfo.environment["SWIFT_DRIVER_LLBUILD_FWK"] == nil {
 
 if ProcessInfo.processInfo.environment["SWIFTCI_USE_LOCAL_DEPS"] == nil {
   package.dependencies += [
-    .package(url: "https://github.com/apple/swift-tools-support-core.git", branch: "release/6.0"),
+    .package(url: "https://github.com/apple/swift-tools-support-core.git", branch: "release/6.1"),
     .package(url: "https://github.com/jpsim/Yams.git", .upToNextMinor(from: "5.0.0")),
     // The 'swift-argument-parser' version declared here must match that
     // used by 'swift-package-manager' and 'sourcekit-lsp'. Please coordinate
     // dependency version changes here with those projects.
-    .package(url: "https://github.com/apple/swift-argument-parser.git", .upToNextMinor(from: "1.2.2")),
-    ]
+    .package(url: "https://github.com/apple/swift-argument-parser.git", from: "1.4.0"),
+  ]
 } else {
     package.dependencies += [
         .package(path: "../swift-tools-support-core"),
