@@ -19,11 +19,12 @@ import WinSDK
 import Glibc
 #elseif canImport(Musl)
 import Musl
+#elseif canImport(Bionic)
+import Bionic
 #else
 #error("Missing libc or equivalent")
 #endif
 
-import TSCBasic // <<<
 import class TSCBasic.DiagnosticsEngine
 import struct TSCBasic.Diagnostic
 import struct TSCBasic.ProcessResult
@@ -79,7 +80,7 @@ import var TSCBasic.stdoutStream
     case .regular, .silent:
       break
     case .verbose:
-      stdoutStream <<< arguments.map { $0.spm_shellEscaped() }.joined(separator: " ") <<< "\n"
+      stdoutStream.send("\(arguments.map { $0.spm_shellEscaped() }.joined(separator: " "))\n")
       stdoutStream.flush()
     case .parsableOutput:
       let messages = constructJobBeganMessages(job: job, arguments: arguments, pid: pid)
@@ -114,7 +115,7 @@ import var TSCBasic.stdoutStream
       let output = (try? result.utf8Output() + result.utf8stderrOutput()) ?? ""
       if !output.isEmpty {
         Driver.stdErrQueue.sync {
-          stderrStream <<< output
+          stderrStream.send(output)
           stderrStream.flush()
         }
       }
@@ -137,7 +138,11 @@ import var TSCBasic.stdoutStream
         }
 #else
       case .signalled(let signal):
+#if canImport(Bionic)
+        let errorMessage = String(cString: strsignal(signal))
+#else
         let errorMessage = strsignal(signal).map { String(cString: $0) } ?? ""
+#endif
         messages = constructJobSignalledMessages(job: job, error: errorMessage, output: output,
                                                  signal: signal, pid: pid).map {
           ParsableMessage(name: job.kind.rawValue, kind: .signalled($0))
@@ -169,8 +174,13 @@ import var TSCBasic.stdoutStream
     // FIXME: Do we need to do error handling here? Can this even fail?
     guard let json = try? message.toJSON() else { return }
     Driver.stdErrQueue.sync {
-      stderrStream <<< json.count <<< "\n"
-      stderrStream <<< String(data: json, encoding: .utf8)! <<< "\n"
+      stderrStream.send(
+        """
+        \(json.count)
+        \(String(data: json, encoding: .utf8)!)
+
+        """
+      )
       stderrStream.flush()
     }
   }
