@@ -34,7 +34,7 @@ public typealias ExternalTargetModuleDetailsMap = [ModuleDependencyId: ExternalT
 /// that specify said explicit module dependencies.
 @_spi(Testing) public struct ExplicitDependencyBuildPlanner {
   /// The module dependency graph.
-  private let dependencyGraph: InterModuleDependencyGraph
+  @_spi(Testing) public let dependencyGraph: InterModuleDependencyGraph
 
   /// A set of direct and transitive dependencies for every module in the dependency graph
   private let reachabilityMap: [ModuleDependencyId : Set<ModuleDependencyId>]
@@ -264,11 +264,6 @@ public typealias ExternalTargetModuleDetailsMap = [ModuleDependencyId: ExternalT
     for dependencyModule in swiftDependencyArtifacts {
       inputs.append(TypedVirtualPath(file: dependencyModule.modulePath.path,
                                      type: .swiftModule))
-
-      let prebuiltHeaderDependencyPaths = dependencyModule.prebuiltHeaderDependencyPaths ?? []
-      if cas != nil && !prebuiltHeaderDependencyPaths.isEmpty {
-        throw DependencyScanningError.unsupportedConfigurationForCaching("module \(dependencyModule.moduleName) has bridging header dependency")
-      }
     }
     for moduleArtifactInfo in clangDependencyArtifacts {
       let clangModulePath =
@@ -415,6 +410,29 @@ public typealias ExternalTargetModuleDetailsMap = [ModuleDependencyId: ExternalT
     }
   }
 
+  public func getLinkLibraryLoadCommandFlags(_ commandLine: inout [Job.ArgTemplate]) throws {
+    var allLinkLibraries: [LinkLibraryInfo] = []
+    for (_, moduleInfo) in dependencyGraph.modules {
+      guard let moduleLinkLibraries = moduleInfo.linkLibraries else {
+        continue
+      }
+      for linkLibrary in moduleLinkLibraries {
+        allLinkLibraries.append(linkLibrary)
+      }
+    }
+
+    for linkLibrary in allLinkLibraries {
+      if !linkLibrary.isFramework {
+        commandLine.appendFlag("-l\(linkLibrary.linkName)")
+      } else {
+        commandLine.appendFlag(.Xlinker)
+        commandLine.appendFlag("-framework")
+        commandLine.appendFlag(.Xlinker)
+        commandLine.appendFlag(linkLibrary.linkName)
+      }
+    }
+  }
+
   /// Resolve all module dependencies of the main module and add them to the lists of
   /// inputs and command line flags.
   public mutating func resolveMainModuleDependencies(inputs: inout [TypedVirtualPath],
@@ -595,8 +613,19 @@ public typealias ExternalTargetModuleDetailsMap = [ModuleDependencyId: ExternalT
 }
 
 internal extension ExplicitDependencyBuildPlanner {
-  func explainDependency(_ dependencyModuleName: String) throws -> [[ModuleDependencyId]]? {
-    return try dependencyGraph.explainDependency(dependencyModuleName: dependencyModuleName)
+  func explainDependency(_ dependencyModuleName: String, allPaths: Bool) throws -> [[ModuleDependencyId]]? {
+    return try dependencyGraph.explainDependency(dependencyModuleName: dependencyModuleName, allPaths: allPaths)
+  }
+
+  func findPath(from source: ModuleDependencyId, to destination: ModuleDependencyId) throws -> [ModuleDependencyId]? {
+    guard dependencyGraph.modules.contains(where: { $0.key == destination }) else { return nil }
+    var result: [ModuleDependencyId]? = nil
+    var visited: Set<ModuleDependencyId> = []
+    try dependencyGraph.findAPath(source: source,
+                                  pathSoFar: [source],
+                                  visited: &visited,
+                                  result: &result) { $0 == destination }
+    return result
   }
 }
 
