@@ -28,24 +28,26 @@ import protocol TSCBasic.FileSystem
       if let currentInfo = modules[swiftModuleId],
          externalModuleId.moduleName != mainModuleName {
         let newExternalModuleDetails =
-        try SwiftPrebuiltExternalModuleDetails(compiledModulePath:
-                                                TextualVirtualPath(path: VirtualPath.absolute(externalModulePath).intern()),
-                                               isFramework: externalModuleDetails.isFramework)
+        SwiftPrebuiltExternalModuleDetails(compiledModulePath:
+                                            TextualVirtualPath(path: VirtualPath.absolute(externalModulePath).intern()),
+                                           isFramework: externalModuleDetails.isFramework)
         let newInfo = ModuleInfo(modulePath: TextualVirtualPath(path: VirtualPath.absolute(externalModulePath).intern()),
                                  sourceFiles: [],
                                  directDependencies: currentInfo.directDependencies,
+                                 linkLibraries: currentInfo.linkLibraries,
                                  details: .swiftPrebuiltExternal(newExternalModuleDetails))
         Self.replaceModule(originalId: swiftModuleId, replacementId: prebuiltModuleId,
                            replacementInfo: newInfo, in: &modules)
       } else if let currentPrebuiltInfo = modules[prebuiltModuleId] {
         // Just update the isFramework bit on this prebuilt module dependency
         let newExternalModuleDetails =
-        try SwiftPrebuiltExternalModuleDetails(compiledModulePath:
-                                                TextualVirtualPath(path: VirtualPath.absolute(externalModulePath).intern()),
-                                               isFramework: externalModuleDetails.isFramework)
+        SwiftPrebuiltExternalModuleDetails(compiledModulePath:
+                                            TextualVirtualPath(path: VirtualPath.absolute(externalModulePath).intern()),
+                                           isFramework: externalModuleDetails.isFramework)
         let newInfo = ModuleInfo(modulePath: TextualVirtualPath(path: VirtualPath.absolute(externalModulePath).intern()),
                                  sourceFiles: [],
                                  directDependencies: currentPrebuiltInfo.directDependencies,
+                                 linkLibraries: currentPrebuiltInfo.linkLibraries,
                                  details: .swiftPrebuiltExternal(newExternalModuleDetails))
         Self.replaceModule(originalId: prebuiltModuleId, replacementId: prebuiltModuleId,
                            replacementInfo: newInfo, in: &modules)
@@ -232,6 +234,9 @@ extension InterModuleDependencyGraph {
     let firstModuleDependencies = firstInfo.directDependencies ?? []
     let secondModuleDependencies = secondInfo.directDependencies ?? []
     let combinedDependencies = Array(Set(firstModuleDependencies + secondModuleDependencies))
+    let firstLinkLibraries = firstInfo.linkLibraries ?? []
+    let secondLinkLibraries = secondInfo.linkLibraries ?? []
+    let combinedLinkLibraries = Array(Set(firstLinkLibraries + secondLinkLibraries))
 
     let firstModuleCapturedPCMArgs = firstDetails.capturedPCMArgs ?? Set<[String]>()
     let secondModuleCapturedPCMArgs = secondDetails.capturedPCMArgs ?? Set<[String]>()
@@ -246,6 +251,7 @@ extension InterModuleDependencyGraph {
     return ModuleInfo(modulePath: firstInfo.modulePath,
                       sourceFiles: combinedSourceFiles,
                       directDependencies: combinedDependencies,
+                      linkLibraries: combinedLinkLibraries,
                       details: .clang(combinedModuleDetails))
   }
 }
@@ -361,7 +367,7 @@ internal extension InterModuleDependencyGraph {
       }
     }
 
-    // Check if any of the textual sources of this module are newer than this module
+    // Check if any of the input sources of this module are newer than this module
     switch checkedModuleInfo.details {
     case .swift(let swiftDetails):
       if let moduleInterfacePath = swiftDetails.moduleInterfacePath {
@@ -394,11 +400,12 @@ internal extension InterModuleDependencyGraph {
         }
       }
     case .swiftPrebuiltExternal(_):
-      // TODO: We have to give-up here until we have a way to verify the timestamp of the binary module.
-      // We can do better here by knowing if this module hasn't changed - which would allows us to not
-      // invalidate any of the dependencies that depend on it.
-      reporter?.report("Unable to verify binary module dependency up-to-date: \(moduleID.moduleNameForDiagnostic)")
-      return false;
+      // We do not verify the binary module itself being out-of-date if we do not have a textual
+      // interface it was built from, but we can safely treat it as up-to-date, particularly
+      // because if it is newer than any of the modules they depend on it, they will
+      // still get invalidated in the check above for whether a module has
+      // any dependencies newer than it.
+      return true;
     case .swiftPlaceholder(_):
       // TODO: This should never ever happen. Hard error?
       return false;
@@ -409,33 +416,49 @@ internal extension InterModuleDependencyGraph {
 }
 
 internal extension InterModuleDependencyGraph {
-  func explainDependency(dependencyModuleName: String) throws -> [[ModuleDependencyId]]? {
+  func explainDependency(dependencyModuleName: String, allPaths: Bool) throws -> [[ModuleDependencyId]]? {
     guard modules.contains(where: { $0.key.moduleName == dependencyModuleName }) else { return nil }
-    var results = Set<[ModuleDependencyId]>()
-    try findAllPaths(source: .swift(mainModuleName),
-                     to: dependencyModuleName,
-                     pathSoFar: [.swift(mainModuleName)],
-                     results: &results)
-    return results.sorted(by: { $0.count < $1.count })
+    var result: Set<[ModuleDependencyId]> = []
+    if allPaths {
+      try findAllPaths(source: .swift(mainModuleName),
+                       pathSoFar: [.swift(mainModuleName)],
+                       results: &result,
+                       destinationMatch: { $0.moduleName == dependencyModuleName })
+    } else {
+      var visited: Set<ModuleDependencyId> = []
+      var singlePathResult: [ModuleDependencyId]? = nil
+      if try findAPath(source: .swift(mainModuleName),
+                       pathSoFar: [.swift(mainModuleName)],
+                       visited: &visited,
+                       result: &singlePathResult,
+                       destinationMatch: { $0.moduleName == dependencyModuleName }),
+         let resultingPath = singlePathResult {
+        result = [resultingPath]
+      }
+    }
+    return Array(result)
   }
 
-
-  private func findAllPaths(source: ModuleDependencyId,
-                            to moduleName: String,
-                            pathSoFar: [ModuleDependencyId],
-                            results: inout Set<[ModuleDependencyId]>) throws {
+  @discardableResult
+  func findAPath(source: ModuleDependencyId,
+                 pathSoFar: [ModuleDependencyId],
+                 visited: inout Set<ModuleDependencyId>,
+                 result: inout [ModuleDependencyId]?,
+                 destinationMatch: (ModuleDependencyId) -> Bool) throws -> Bool {
+    // Mark this node as visited
+    visited.insert(source)
     let sourceInfo = try moduleInfo(of: source)
-    // If the source is our target, we are done
-    if source.moduleName == moduleName {
+    if destinationMatch(source) {
       // If the source is a target Swift module, also check if it
       // depends on a corresponding Clang module with the same name.
       // If it does, add it to the path as well.
       var completePath = pathSoFar
       if let dependencies = sourceInfo.directDependencies,
-         dependencies.contains(.clang(moduleName)) {
-        completePath.append(.clang(moduleName))
+         dependencies.contains(.clang(source.moduleName)) {
+        completePath.append(.clang(source.moduleName))
       }
-      results.insert(completePath)
+      result = completePath
+      return true
     }
 
     var allDependencies = sourceInfo.directDependencies ?? []
@@ -445,10 +468,45 @@ internal extension InterModuleDependencyGraph {
     }
 
     for dependency in allDependencies {
-      try findAllPaths(source: dependency,
-                       to: moduleName,
+      if try findAPath(source: dependency,
                        pathSoFar: pathSoFar + [dependency],
-                       results: &results)
+                       visited: &visited,
+                       result: &result,
+                       destinationMatch: destinationMatch) {
+        return true
+      }
+    }
+    return false
+  }
+
+  private func findAllPaths(source: ModuleDependencyId,
+                            pathSoFar: [ModuleDependencyId],
+                            results: inout Set<[ModuleDependencyId]>,
+                            destinationMatch: (ModuleDependencyId) -> Bool) throws {
+    let sourceInfo = try moduleInfo(of: source)
+    if destinationMatch(source) {
+      // If the source is a target Swift module, also check if it
+      // depends on a corresponding Clang module with the same name.
+      // If it does, add it to the path as well.
+      var completePath = pathSoFar
+      if let dependencies = sourceInfo.directDependencies,
+         dependencies.contains(.clang(source.moduleName)) {
+        completePath.append(.clang(source.moduleName))
+      }
+      results.insert(completePath)
+      return
+    }
+
+    var allDependencies = sourceInfo.directDependencies ?? []
+    if case .swift(let swiftModuleDetails) = sourceInfo.details,
+          let overlayDependencies = swiftModuleDetails.swiftOverlayDependencies {
+      allDependencies.append(contentsOf: overlayDependencies)
+    }
+    for dependency in allDependencies {
+      try findAllPaths(source: dependency,
+                       pathSoFar: pathSoFar + [dependency],
+                       results: &results,
+                       destinationMatch: destinationMatch)
     }
   }
 }
