@@ -82,17 +82,24 @@ public class InterModuleDependencyOracle {
                                                     diagnostics: &diagnostics)
   }
 
-  /// Given a specified toolchain path, locate and instantiate an instance of the SwiftScan library
+  @available(*, deprecated, message: "use verifyOrCreateScannerInstance(swiftScanLibPath:)")
   public func verifyOrCreateScannerInstance(fileSystem: FileSystem,
                                             swiftScanLibPath: AbsolutePath) throws {
+    return try verifyOrCreateScannerInstance(swiftScanLibPath: swiftScanLibPath)
+  }
+
+  /// Given a specified toolchain path, locate and instantiate an instance of the SwiftScan library
+  public func verifyOrCreateScannerInstance(swiftScanLibPath: AbsolutePath?) throws {
     return try queue.sync {
-      if swiftScanLibInstance == nil {
+      guard let scanInstance = swiftScanLibInstance else {
         swiftScanLibInstance = try SwiftScan(dylib: swiftScanLibPath)
-      } else {
-        guard swiftScanLibInstance!.path == swiftScanLibPath else {
-          throw DependencyScanningError
-          .scanningLibraryInvocationMismatch(swiftScanLibInstance!.path, swiftScanLibPath)
-        }
+        return
+      }
+
+      guard scanInstance.path?.description == swiftScanLibPath?.description else {
+        throw DependencyScanningError
+          .scanningLibraryInvocationMismatch(scanInstance.path?.description ?? "built-in",
+                                             swiftScanLibPath?.description ?? "built-in")
       }
     }
   }
@@ -174,6 +181,13 @@ public class InterModuleDependencyOracle {
     return swiftScan.supportsDiagnosticSourceLocations
   }
 
+  @_spi(Testing) public func supportsLinkLibraries() throws -> Bool {
+    guard let swiftScan = swiftScanLibInstance else {
+      fatalError("Attempting to query supported scanner API with no scanner instance.")
+    }
+    return swiftScan.supportsLinkLibraries
+  }
+
   @_spi(Testing) public func getScannerDiagnostics() throws -> [ScannerDiagnosticPayload]? {
     guard let swiftScan = swiftScanLibInstance else {
       fatalError("Attempting to reset scanner cache with no scanner instance.")
@@ -202,10 +216,20 @@ public class InterModuleDependencyOracle {
     }
   }
 
+  // Note: this is `true` even in the `compilerIntegratedTooling` mode
+  // where the `SwiftScan` instance refers to the own image the driver is
+  // running in, since there is still technically a `SwiftScan` handle
+  // capable of handling API requests expected of it.
   private var hasScannerInstance: Bool { self.swiftScanLibInstance != nil }
+  func getScannerInstance() -> SwiftScan? {
+    self.swiftScanLibInstance
+  }
+  func setScannerInstance(_ instance: SwiftScan?) {
+    self.swiftScanLibInstance = instance
+  }
 
   /// Queue to sunchronize accesses to the scanner
-  internal let queue = DispatchQueue(label: "org.swift.swift-driver.swift-scan")
+  let queue = DispatchQueue(label: "org.swift.swift-driver.swift-scan")
 
   /// A reference to an instance of the compiler's libSwiftScan shared library
   private var swiftScanLibInstance: SwiftScan? = nil
