@@ -59,21 +59,20 @@ extension Toolchain {
 extension Driver {
 
   static func computeSupportedCompilerArgs(of toolchain: Toolchain,
+                                           libSwiftScan: SwiftScan?,
                                            parsedOptions: inout ParsedOptions,
                                            diagnosticsEngine: DiagnosticsEngine,
                                            fileSystem: FileSystem,
                                            executor: DriverExecutor)
   throws -> Set<String> {
-    // Disable in-process supported features query due to a race condition in the compiler's current
-    // build system where libSwiftScan may not be ready when building the Swift standard library.
-//    do {
-//      if let supportedArgs =
-//          try querySupportedCompilerArgsInProcess(of: toolchain, fileSystem: fileSystem) {
-//        return supportedArgs
-//      }
-//    } catch {
-//      diagnosticsEngine.emit(.remark_inprocess_supported_features_query_failed(error.localizedDescription))
-//    }
+    if let libSwiftScanInstance = libSwiftScan,
+       libSwiftScanInstance.canQuerySupportedArguments() {
+      do {
+        return try libSwiftScanInstance.querySupportedArguments()
+      } catch {
+        diagnosticsEngine.emit(.remark_inprocess_supported_features_query_failed(error.localizedDescription))
+      }
+    }
 
     // Fallback: Invoke `swift-frontend -emit-supported-features` and decode the output
     let frontendOverride = try FrontendOverride(&parsedOptions, diagnosticsEngine)
@@ -90,20 +89,6 @@ extension Driver {
     return Set(decodedSupportedFlagList)
   }
 
-  static func querySupportedCompilerArgsInProcess(of toolchain: Toolchain,
-                                                  fileSystem: FileSystem)
-  throws -> Set<String>? {
-    let optionalSwiftScanLibPath = try toolchain.lookupSwiftScanLib()
-    if let swiftScanLibPath = optionalSwiftScanLibPath,
-       fileSystem.exists(swiftScanLibPath) {
-      let libSwiftScanInstance = try SwiftScan(dylib: swiftScanLibPath)
-      if libSwiftScanInstance.canQuerySupportedArguments() {
-        return try libSwiftScanInstance.querySupportedArguments()
-      }
-    }
-    return nil
-  }
-
   static func computeSupportedCompilerFeatures(of toolchain: Toolchain,
                                                env: [String: String]) throws -> Set<String> {
     struct FeatureInfo: Codable {
