@@ -51,6 +51,15 @@ extension Driver {
     /// If the driver is in Explicit Module Build mode, the dependency graph has been computed
     case computed
   }
+
+  /// If the given option is specified but the frontend doesn't support it, throw an error.
+  func verifyFrontendSupportsOptionIfNecessary(_ option: Option) throws {
+    if parsedOptions.hasArgument(option) && !isFrontendArgSupported(option) {
+      diagnosticEngine.emit(.error_unsupported_opt_for_frontend(option: option))
+      throw ErrorDiagnostics.emitted
+    }
+  }
+
   /// Add frontend options that are common to different frontend invocations.
   mutating func addCommonFrontendOptions(
     commandLine: inout [Job.ArgTemplate],
@@ -135,9 +144,6 @@ extension Driver {
     // Add flags for C++ interop
     try commandLine.appendLast(.enableExperimentalCxxInterop, from: &parsedOptions)
     try commandLine.appendLast(.cxxInteroperabilityMode, from: &parsedOptions)
-    if let stdlibVariant = parsedOptions.getLastArgument(.experimentalCxxStdlib)?.asSingle {
-      appendXccFlag("-stdlib=\(stdlibVariant)")
-    }
 
     if isEmbeddedEnabled && parsedOptions.hasArgument(.enableLibraryEvolution) {
       diagnosticEngine.emit(.error_no_library_evolution_embedded)
@@ -156,6 +162,9 @@ extension Driver {
       throw ErrorDiagnostics.emitted
     }
 
+    try verifyFrontendSupportsOptionIfNecessary(.disableUpcomingFeature)
+    try verifyFrontendSupportsOptionIfNecessary(.disableExperimentalFeature)
+
     // Handle the CPU and its preferences.
     try commandLine.appendLast(.targetCpu, from: &parsedOptions)
 
@@ -198,6 +207,7 @@ extension Driver {
     try commandLine.appendLast(.fixitAll, from: &parsedOptions)
     try commandLine.appendLast(.warnSwift3ObjcInferenceMinimal, .warnSwift3ObjcInferenceComplete, from: &parsedOptions)
     try commandLine.appendLast(.warnImplicitOverrides, from: &parsedOptions)
+    try commandLine.appendLast(.warnSoftDeprecated, from: &parsedOptions)
     try commandLine.appendLast(.typoCorrectionLimit, from: &parsedOptions)
     try commandLine.appendLast(.enableAppExtension, from: &parsedOptions)
     try commandLine.appendLast(.enableLibraryEvolution, from: &parsedOptions)
@@ -213,6 +223,7 @@ extension Driver {
     try commandLine.appendLast(.importUnderlyingModule, from: &parsedOptions)
     try commandLine.appendLast(.moduleCachePath, from: &parsedOptions)
     try commandLine.appendLast(.moduleLinkName, from: &parsedOptions)
+    try commandLine.appendLast(.moduleAbiName, from: &parsedOptions)
     try commandLine.appendLast(.nostdimport, from: &parsedOptions)
     try commandLine.appendLast(.parseStdlib, from: &parsedOptions)
     try commandLine.appendLast(.solverMemoryThreshold, from: &parsedOptions)
@@ -225,7 +236,11 @@ extension Driver {
     try commandLine.appendLast(.profileGenerate, from: &parsedOptions)
     try commandLine.appendLast(.profileUse, from: &parsedOptions)
     try commandLine.appendLast(.profileCoverageMapping, from: &parsedOptions)
-    try commandLine.appendLast(.warningsAsErrors, .noWarningsAsErrors, from: &parsedOptions)
+    try commandLine.appendAllExcept(
+      includeList: [.warningTreating], 
+      excludeList: [], 
+      from: &parsedOptions
+    )
     try commandLine.appendLast(.sanitizeEQ, from: &parsedOptions)
     try commandLine.appendLast(.sanitizeRecoverEQ, from: &parsedOptions)
     try commandLine.appendLast(.sanitizeAddressUseOdrIndicator, from: &parsedOptions)
@@ -247,6 +262,7 @@ extension Driver {
     try commandLine.appendLast(.packageDescriptionVersion, from: &parsedOptions)
     try commandLine.appendLast(.serializeDiagnosticsPath, from: &parsedOptions)
     try commandLine.appendLast(.debugDiagnosticNames, from: &parsedOptions)
+    try commandLine.appendLast(.printDiagnosticGroups, from: &parsedOptions)
     try commandLine.appendLast(.scanDependencies, from: &parsedOptions)
     try commandLine.appendLast(.enableExperimentalConcisePoundFile, from: &parsedOptions)
     try commandLine.appendLast(.experimentalPackageInterfaceLoad, from: &parsedOptions)
@@ -260,6 +276,9 @@ extension Driver {
     try commandLine.appendLast(.lto, from: &parsedOptions)
     try commandLine.appendLast(.accessNotesPath, from: &parsedOptions)
     try commandLine.appendLast(.enableActorDataRaceChecks, .disableActorDataRaceChecks, from: &parsedOptions)
+    if isFrontendArgSupported(.dumpAstFormat) {
+      try commandLine.appendLast(.dumpAstFormat, from: &parsedOptions)
+    }
     try commandLine.appendAll(.D, from: &parsedOptions)
     try commandLine.appendAll(.debugPrefixMap, .coveragePrefixMap, .filePrefixMap, from: &parsedOptions)
     try commandLine.appendAllArguments(.Xfrontend, from: &parsedOptions)
@@ -267,14 +286,14 @@ extension Driver {
     if isFrontendArgSupported(.noAllocations) {
       try commandLine.appendLast(.noAllocations, from: &parsedOptions)
     }
-    if isFrontendArgSupported(.enableExperimentalFeature) {
-      try commandLine.appendAll(
-        .enableExperimentalFeature, from: &parsedOptions)
-    }
-    if isFrontendArgSupported(.enableUpcomingFeature) {
-      try commandLine.appendAll(
-        .enableUpcomingFeature, from: &parsedOptions)
+    if isFrontendArgSupported(.compilerAssertions) {
+      try commandLine.appendLast(.compilerAssertions, from: &parsedOptions)
     }
+    try commandLine.appendAll(.enableExperimentalFeature,
+                              .disableExperimentalFeature,
+                              .enableUpcomingFeature,
+                              .disableUpcomingFeature,
+                              from: &parsedOptions)
     try commandLine.appendAll(.moduleAlias, from: &parsedOptions)
     if isFrontendArgSupported(.enableBareSlashRegex) {
       try commandLine.appendLast(.enableBareSlashRegex, from: &parsedOptions)
@@ -331,6 +350,10 @@ extension Driver {
       commandLine.appendFlag(ver)
     }
 
+    if isFrontendArgSupported(.publicModuleName) {
+      try commandLine.appendLast(.publicModuleName, from: &parsedOptions)
+    }
+
     // Pass down -validate-clang-modules-once if we are working with a compiler that
     // supports it.
     if isFrontendArgSupported(.validateClangModulesOnce),
@@ -347,6 +370,10 @@ extension Driver {
       try commandLine.appendLast(.disableSandbox, from: &parsedOptions)
     }
 
+    if isFrontendArgSupported(.disableDynamicActorIsolation) {
+      try commandLine.appendLast(.disableDynamicActorIsolation, from: &parsedOptions)
+    }
+
     if !directModuleCC1Mode, let workingDirectory = workingDirectory {
       // Add -Xcc -working-directory before any other -Xcc options to ensure it is
       // overridden by an explicit -Xcc -working-directory, although having a
@@ -398,11 +425,13 @@ extension Driver {
     // CAS related options.
     if isCachingEnabled {
       commandLine.appendFlag(.cacheCompileJob)
-      if let casPath = try getOnDiskCASPath() {
+      if let casPath = try Self.getOnDiskCASPath(parsedOptions: &parsedOptions,
+                                                 toolchain: toolchain) {
         commandLine.appendFlag(.casPath)
         commandLine.appendFlag(casPath.pathString)
       }
-      if let pluginPath = try getCASPluginPath() {
+      if let pluginPath = try Self.getCASPluginPath(parsedOptions: &parsedOptions,
+                                                    toolchain: toolchain) {
         commandLine.appendFlag(.casPluginPath)
         commandLine.appendFlag(pluginPath.pathString)
       }
@@ -470,6 +499,12 @@ extension Driver {
       commandLine.appendFlag("-frontend-parseable-output")
     }
 
+    // If explicit auto-linking is enabled, ensure that compiler tasks do not produce
+    // auto-link load commands in resulting object files.
+    if parsedOptions.hasArgument(.explicitAutoLinking) {
+      commandLine.appendFlag(.disableAllAutolinking)
+    }
+
     savedUnknownDriverFlagsForSwiftFrontend.forEach {
       commandLine.appendFlag($0)
     }
@@ -782,10 +817,19 @@ extension Driver {
     guard isFrontendArgSupported(.pluginPath) else {
       return
     }
+    let pluginPathRoot = VirtualPath.absolute(try toolchain.executableDir.parentDirectory)
+
+    if isFrontendArgSupported(.inProcessPluginServerPath) {
+      commandLine.appendFlag(.inProcessPluginServerPath)
+#if os(Windows)
+      commandLine.appendPath(pluginPathRoot.appending(components: "bin", sharedLibraryName("SwiftInProcPluginServer")))
+#else
+      commandLine.appendPath(pluginPathRoot.appending(components: "lib", "swift", "host", sharedLibraryName("libSwiftInProcPluginServer")))
+#endif
+    }
 
     // Default paths for compiler plugins found within the toolchain
     // (loaded as shared libraries).
-    let pluginPathRoot = VirtualPath.absolute(try toolchain.executableDir.parentDirectory)
     commandLine.appendFlag(.pluginPath)
     commandLine.appendPath(pluginPathRoot.pluginPath)
 
