@@ -106,6 +106,19 @@ extension GenericUnixToolchain {
         commandLine.appendFlag("-pie")
       }
 
+      // On some platforms we want to enable --build-id
+      if targetTriple.os == .linux
+           || targetTriple.os == .freeBSD
+           || targetTriple.os == .openbsd
+           || parsedOptions.hasArgument(.buildId) {
+        commandLine.appendFlag("-Xlinker")
+        if let buildId = parsedOptions.getLastArgument(.buildId)?.asSingle {
+          commandLine.appendFlag("--build-id=\(buildId)")
+        } else {
+          commandLine.appendFlag("--build-id")
+        }
+      }
+
       let staticStdlib = parsedOptions.hasFlag(positive: .staticStdlib,
                                                negative: .noStaticStdlib,
                                                    default: false)
@@ -127,8 +140,16 @@ extension GenericUnixToolchain {
         isShared: hasRuntimeArgs
       )
 
-      if hasRuntimeArgs && targetTriple.environment != .android &&
-          toolchainStdlibRpath {
+      // An exception is made for native Android environments like the Termux
+      // app as they build and run natively like a Unix environment on Android,
+      // so add the stdlib RPATH by default there.
+      #if os(Android)
+      let addRpath = true
+      #else
+      let addRpath = targetTriple.environment != .android
+      #endif
+
+      if hasRuntimeArgs && addRpath && toolchainStdlibRpath {
         // FIXME: We probably shouldn't be adding an rpath here unless we know
         //        ahead of time the standard library won't be copied.
         for path in runtimePaths {
@@ -139,14 +160,30 @@ extension GenericUnixToolchain {
         }
       }
 
+      if targetInfo.sdkPath != nil {
+        for libpath in targetInfo.runtimeLibraryImportPaths {
+          commandLine.appendFlag(.L)
+          commandLine.appendPath(VirtualPath.lookup(libpath.path))
+        }
+      }
+
       if !isEmbeddedEnabled && !parsedOptions.hasArgument(.nostartfiles) {
-        let swiftrtPath = VirtualPath.lookup(targetInfo.runtimeResourcePath.path)
-          .appending(
-            components: targetTriple.platformName() ?? "",
-            String(majorArchitectureName(for: targetTriple)),
-            "swiftrt.o"
-          )
-        commandLine.appendPath(swiftrtPath)
+        let rsrc: VirtualPath
+        // Prefer the swiftrt.o runtime file from the SDK if it's specified.
+        if let sdk = targetInfo.sdkPath {
+          let swiftDir: String
+          if staticStdlib || staticExecutable {
+            swiftDir = "swift_static"
+          } else {
+            swiftDir = "swift"
+          }
+          rsrc = VirtualPath.lookup(sdk.path).appending(components: "usr", "lib", swiftDir)
+        } else {
+          rsrc = VirtualPath.lookup(targetInfo.runtimeResourcePath.path)
+        }
+        let platform: String = targetTriple.platformName() ?? ""
+        let architecture: String = majorArchitectureName(for: targetTriple)
+        commandLine.appendPath(rsrc.appending(components: platform, architecture, "swiftrt.o"))
       }
 
       // If we are linking statically, we need to add all
@@ -186,7 +223,15 @@ extension GenericUnixToolchain {
         commandLine.appendPath(try VirtualPath(path: opt.argument.asSingle))
       }
 
-      if let path = targetInfo.sdkPath?.path {
+      if let sysroot = parsedOptions.getLastArgument(.sysroot)?.asSingle {
+        commandLine.appendFlag("--sysroot")
+        try commandLine.appendPath(VirtualPath(path: sysroot))
+      } else if targetTriple.environment == .android,
+        let sysroot = AndroidNDK.getDefaultSysrootPath(in: self.env)
+      {
+        commandLine.appendFlag("--sysroot")
+        try commandLine.appendPath(VirtualPath(path: sysroot.pathString))
+      } else if let path = targetInfo.sdkPath?.path {
         commandLine.appendFlag("--sysroot")
         commandLine.appendPath(VirtualPath.lookup(path))
       }
@@ -246,9 +291,10 @@ extension GenericUnixToolchain {
       }
 
       if parsedOptions.hasArgument(.profileGenerate) {
+        let environment = (targetTriple.environment == .android) ? "-android" : ""
         let libProfile = VirtualPath.lookup(targetInfo.runtimeResourcePath.path)
           .appending(components: "clang", "lib", targetTriple.osName,
-                                 "libclang_rt.profile-\(targetTriple.archName).a")
+                                 "libclang_rt.profile-\(targetTriple.archName)\(environment).a")
         commandLine.appendPath(libProfile)
 
         // HACK: Hard-coded from llvm::getInstrProfRuntimeHookVarName()
