@@ -106,6 +106,8 @@ extension WebAssemblyToolchain {
           return .responseFilePath(input.file)
         } else if input.type == .object {
           return .path(input.file)
+        } else if lto != nil && input.type == .llvmBitcode {
+          return .path(input.file)
         } else {
           return nil
         }
@@ -158,8 +160,20 @@ extension WebAssemblyToolchain {
         throw Error.sanitizersUnsupportedForTarget(targetTriple.triple)
       }
 
-      guard !parsedOptions.hasArgument(.profileGenerate) else {
-        throw Error.profilingUnsupportedForTarget(targetTriple.triple)
+      if parsedOptions.hasArgument(.profileGenerate) {
+        let libProfile = VirtualPath.lookup(targetInfo.runtimeResourcePath.path)
+          .appending(components: "clang", "lib", targetTriple.osName,
+                                 "libclang_rt.profile-\(targetTriple.archName).a")
+        commandLine.appendPath(libProfile)
+      }
+
+      if let lto = lto {
+        switch lto {
+        case .llvmFull:
+          commandLine.appendFlag("-flto=full")
+        case .llvmThin:
+          commandLine.appendFlag("-flto=thin")
+        }
       }
 
       // Run clang++ in verbose mode if "-v" is set
