@@ -165,7 +165,6 @@ public final class DarwinToolchain: Toolchain {
     case argumentNotSupported(String)
     case invalidDeploymentTargetForIR(platform: DarwinPlatform, version: Triple.Version, archName: String)
     case unsupportedTargetVariant(variant: Triple)
-    case darwinOnlySupportsLibCxx
 
     public var description: String {
       switch self {
@@ -177,8 +176,6 @@ public final class DarwinToolchain: Toolchain {
         return "unsupported '\(variant.isiOS ? "-target-variant" : "-target")' value '\(variant.triple)'; use 'ios-macabi' instead"
       case .argumentNotSupported(let argument):
         return "\(argument) is no longer supported for Apple platforms"
-      case .darwinOnlySupportsLibCxx:
-        return "The only C++ standard library supported on Apple platforms is libc++"
       }
     }
   }
@@ -211,12 +208,6 @@ public final class DarwinToolchain: Toolchain {
     if parsedOptions.hasArgument(.staticExecutable) {
       throw ToolchainValidationError.argumentNotSupported("-static-executable")
     }
-    // If a C++ standard library is specified, it has to be libc++.
-    if let cxxLib = parsedOptions.getLastArgument(.experimentalCxxStdlib) {
-        if cxxLib.asSingle != "libc++" {
-            throw ToolchainValidationError.darwinOnlySupportsLibCxx
-        }
-    }
   }
 
   public func getDefaultDwarfVersion(targetTriple: Triple) -> UInt8 {
@@ -465,30 +456,54 @@ public final class DarwinToolchain: Toolchain {
     }
 
     if driver.isFrontendArgSupported(.externalPluginPath) {
-      // Determine the platform path. For simulator platforms, look into the
-      // corresponding device platform instance.
-      let origPlatformPath = VirtualPath.lookup(sdkPath)
-        .parentDirectory
-        .parentDirectory
-        .parentDirectory
-      let platformPath: VirtualPath
-      if let simulatorRange = origPlatformPath.basename.range(of: "Simulator.platform") {
-        let devicePlatform = origPlatformPath.basename.replacingCharacters(in: simulatorRange, with: "OS.platform")
-        platformPath = origPlatformPath.parentDirectory.appending(component: devicePlatform)
-      } else {
-        platformPath = origPlatformPath
+      // If the PLATFORM_DIR environment variable is set, also add plugin
+      // paths into it. Since this is a user override, it comes beore the
+      // default platform path that's based on the SDK.
+      if let platformDir = env["PLATFORM_DIR"],
+         let platformPath = try? VirtualPath(path: platformDir) {
+        addPluginPaths(
+          forPlatform: platformPath,
+          commandLine: &commandLine
+        )
       }
 
-      // Default paths for compiler plugins within the platform (accessed via that
-      // platform's plugin server).
-      let platformPathRoot = platformPath.appending(components: "Developer", "usr")
-      commandLine.appendFlag(.externalPluginPath)
-      commandLine.appendFlag("\(platformPathRoot.pluginPath.name)#\(platformPathRoot.pluginServerPath.name)")
-
-      commandLine.appendFlag(.externalPluginPath)
-      commandLine.appendFlag("\(platformPathRoot.localPluginPath.name)#\(platformPathRoot.pluginServerPath.name)")
+      // Determine the platform path based on the SDK path.
+      addPluginPaths(
+        forPlatform: VirtualPath.lookup(sdkPath)
+          .parentDirectory
+          .parentDirectory
+          .parentDirectory,
+        commandLine: &commandLine
+      )
     }
   }
+
+  /// Given the platform path (e.g., a path into something like iPhoneOS.platform),
+  /// add external plugin path arguments for compiler plugins that are distributed
+  /// within that path.
+  func addPluginPaths(
+    forPlatform origPlatformPath: VirtualPath,
+    commandLine: inout [Job.ArgTemplate]
+  ) {
+    // For simulator platforms, look into the corresponding device platform instance,
+    // because they share compiler plugins.
+    let platformPath: VirtualPath
+    if let simulatorRange = origPlatformPath.basename.range(of: "Simulator.platform") {
+      let devicePlatform = origPlatformPath.basename.replacingCharacters(in: simulatorRange, with: "OS.platform")
+      platformPath = origPlatformPath.parentDirectory.appending(component: devicePlatform)
+    } else {
+      platformPath = origPlatformPath
+    }
+
+    // Default paths for compiler plugins within the platform (accessed via that
+    // platform's plugin server).
+    let platformPathRoot = platformPath.appending(components: "Developer", "usr")
+    commandLine.appendFlag(.externalPluginPath)
+    commandLine.appendFlag("\(platformPathRoot.pluginPath.name)#\(platformPathRoot.pluginServerPath.name)")
+
+    commandLine.appendFlag(.externalPluginPath)
+    commandLine.appendFlag("\(platformPathRoot.localPluginPath.name)#\(platformPathRoot.pluginServerPath.name)")
+  }
 }
 
 extension Diagnostic.Message {
