@@ -38,8 +38,9 @@ enum class OptionKind {
 };
 
 #define LLVM_MAKE_OPT_ID_WITH_ID_PREFIX(ID_PREFIX, PREFIX, NAME, ID, KIND,     \
-                                        GROUP, ALIAS, ALIASARGS, FLAGS, PARAM, \
-                                        HELPTEXT, METAVAR, VALUES)             \
+                                        GROUP, ALIAS, ALIASARGS, FLAGS,        \
+                                        VISIBILITY, PARAM, HELPTEXT,           \
+                                        HELPTEXTFORVARIANTS, METAVAR, VALUES)  \
   ID_PREFIX##ID
 
 //. The IDs of each option
@@ -60,7 +61,7 @@ enum SwiftFlags {
   DoesNotAffectIncrementalBuild = (1 << 8),
   AutolinkExtractOption = (1 << 9),
   ModuleWrapOption = (1 << 10),
-  SwiftIndentOption = (1 << 11),
+  SwiftSynthesizeInterfaceOption = (1 << 11),
   ArgumentIsPath = (1 << 12),
   ModuleInterfaceOption = (1 << 13),
   SupplementaryOutput = (1 << 14),
@@ -134,9 +135,10 @@ struct RawOption {
 #undef PREFIX
 
 static const RawOption rawOptions[] = {
-#define OPTION(PREFIX, NAME, ID, KIND, GROUP, ALIAS, ALIASARGS, FLAGS, PARAM,  \
-               HELPTEXT, METAVAR, VALUES)                                      \
- { OptionID::Opt_##ID, PREFIX, NAME, swiftify(#ID), OptionKind::KIND, \
+#define OPTION(PREFIX, NAME, ID, KIND, GROUP, ALIAS, ALIASARGS, FLAGS,         \
+               VISIBILITY, PARAM, HELPTEXT, HELPTEXTFORVARIANTS, METAVAR,      \
+               VALUES)                                                         \
+ { OptionID::Opt_##ID, PREFIX, NAME, swiftify(#ID), OptionKind::KIND,          \
    OptionID::Opt_##GROUP, OptionID::Opt_##ALIAS, FLAGS, HELPTEXT, METAVAR, PARAM },
 #include "swift/Option/Options.inc"
 #undef OPTION
@@ -335,7 +337,7 @@ int makeOptions_main() {
         emitFlagIf(DoesNotAffectIncrementalBuild, ".doesNotAffectIncrementalBuild");
         emitFlagIf(AutolinkExtractOption, ".autolinkExtract");
         emitFlagIf(ModuleWrapOption, ".moduleWrap");
-        emitFlagIf(SwiftIndentOption, ".indent");
+        emitFlagIf(SwiftSynthesizeInterfaceOption, ".synthesizeInterface");
         if (option.kind == OptionKind::Input)
           emitFlag(".argumentIsPath");
         else
