@@ -1,5 +1,5 @@
 // Created by David Ungar on 7/28/21.
-// 
+//
 
 import XCTest
 @_spi(Testing) import SwiftDriver
@@ -65,10 +65,10 @@ class IncrementalBuildPerformanceTests: XCTestCase {
   ///    - limit: the maximum number of swiftdeps files to process.
   func test(swiftDepsDirectory: String, atMost limit: Int = .max, _ whatToMeasure: WhatToMeasure) throws {
     let (outputFileMap, inputs) = try createOFMAndInputs(swiftDepsDirectory, atMost: limit)
-    
+
     let info = IncrementalCompilationState.IncrementalDependencyAndInputSetup
       .mock(options: [], outputFileMap: outputFileMap)
-    
+
     let g = ModuleDependencyGraph.createForSimulatingCleanBuild(info.buildRecordInfo.buildRecord([], []), info)
     g.blockingConcurrentAccessOrMutation {
       switch whatToMeasure {
@@ -101,7 +101,7 @@ class IncrementalBuildPerformanceTests: XCTestCase {
   ) throws -> (OutputFileMap, [SwiftSourceFile]) {
     let workingDirectory = localFileSystem.currentWorkingDirectory!
     let swiftDepsDirPath = try VirtualPath.init(path: swiftDepsDirectory).resolvedRelativePath(base: workingDirectory).absolutePath!
-    let withoutExtensions: ArraySlice<Substring> = try! localFileSystem.getDirectoryContents(swiftDepsDirPath)
+    let withoutExtensions: ArraySlice<Substring> = try localFileSystem.getDirectoryContents(swiftDepsDirPath)
       .compactMap {
         fileName -> Substring? in
         guard let suffixRange = fileName.range(of: ".swiftdeps"),
@@ -138,7 +138,7 @@ class IncrementalBuildPerformanceTests: XCTestCase {
       invalidatedInputs.formUnion(g.collectInputsRequiringCompilation(byCompiling: primaryInput)!)
     }
     .subtracting(inputs) // have already compiled these
-    
+
     XCTAssertEqual(result.count, 0, "Should be no invalid inputs left")
   }
 }
