@@ -250,7 +250,6 @@ def install_swiftdriver(args, build_dir, prefix, targets) :
   # swift-driver: SwiftDriver.swiftmodule, SwiftOptions.swiftmodule
   # swift-tools-support-core: TSCUtility.swiftmodule, TSCBasic.swiftmodule
   # swift-argument-parser: ArgumentParser.swiftmodule (disabled until needed)
-  # swift-system: SystemPackage.swiftmodule
   install_binary_swift_modules(args, build_dir, install_lib, targets)
 
   # Modulemaps for C Modules:
@@ -327,11 +326,6 @@ def install_libraries(args, build_dir, universal_lib_dir, toolchain_lib_dir, tar
     install_library(args, build_dir, package_subpath, lib, shared_lib_ext,
                     universal_lib_dir, toolchain_lib_dir, 'swift-tools-support-core', targets)
 
-  # Install the swift-system shared library into the toolchain lib
-  package_subpath = os.path.join(args.configuration, 'dependencies', 'swift-system')
-  install_library(args, build_dir, package_subpath, 'libSystemPackage', shared_lib_ext,
-                  universal_lib_dir, toolchain_lib_dir, 'swift-system', targets)
-
   # Install the swift-argument-parser shared libraries into the toolchain lib
   package_subpath = os.path.join(args.configuration, 'dependencies', 'swift-argument-parser')
   for (lib, ext) in [('libArgumentParser', shared_lib_ext), ('libArgumentParserToolInfo', static_lib_ext)]:
@@ -379,11 +373,6 @@ def install_binary_swift_modules(args, build_dir, toolchain_lib_dir, targets):
                                  product_subpath)
   install_module(args, build_dir, package_subpath, toolchain_lib_dir, 'ArgumentParser', targets)
 
-  # swift-system
-  package_subpath = os.path.join(args.configuration, 'dependencies', 'swift-system',
-                                 product_subpath)
-  install_module(args, build_dir, package_subpath, toolchain_lib_dir, 'SystemPackage', targets)
-
 
 # Install the modulemaps and headers of the driver's C module dependencies into the toolchain
 # include directory
@@ -420,6 +409,10 @@ def build_using_cmake(args, toolchain_bin, build_dir, targets):
 
   if args.enable_asan:
     base_swift_flags.append('-sanitize=address')
+    # This is currently needed to work around a swift-driver
+    # bug when building with a 5.8 host toolchain.
+    base_swift_flags.append('-Xclang-linker')
+    base_swift_flags.append('-fsanitize=address')
 
   # Ensure we are not sharing the module cache with concurrent builds in CI
   base_swift_flags.append('-module-cache-path "{}"'.format(os.path.join(build_dir, 'module-cache')))
@@ -445,9 +438,6 @@ def build_using_cmake(args, toolchain_bin, build_dir, targets):
     # LLBuild
     build_llbuild_using_cmake(args, target, swiftc_exec, dependencies_dir,
                               base_cmake_flags, swift_flags)
-    # SwiftSystem
-    build_system_using_cmake(args, target, swiftc_exec, dependencies_dir,
-                             base_cmake_flags, swift_flags)
 
     # TSC
     build_tsc_using_cmake(args, target, swiftc_exec, dependencies_dir,
@@ -488,24 +478,11 @@ def build_llbuild_using_cmake(args, target, swiftc_exec, build_dir, base_cmake_f
   cmake_build(args, swiftc_exec, llbuild_cmake_flags, llbuild_swift_flags,
               llbuild_source_dir, llbuild_build_dir, 'products/all')
 
-def build_system_using_cmake(args, target, swiftc_exec, build_dir, base_cmake_flags, swift_flags):
-  print('Building Swift Driver dependency: Swift System')
-  system_source_dir = os.path.join(os.path.dirname(args.package_path), 'swift-system')
-  system_build_dir = os.path.join(build_dir, 'swift-system')
-  flags = [
-      # required due to swift-autolink-extract bug ("The file was not recognized as a valid object file")
-      "-DBUILD_SHARED_LIBS=YES"]
-  system_cmake_flags = base_cmake_flags + flags
-  system_swift_flags = swift_flags[:]
-  cmake_build(args, swiftc_exec, system_cmake_flags, system_swift_flags,
-              system_source_dir, system_build_dir)
-
 def build_tsc_using_cmake(args, target, swiftc_exec, build_dir, base_cmake_flags, swift_flags):
   print('Building Swift Driver dependency: TSC')
   tsc_source_dir = os.path.join(os.path.dirname(args.package_path), 'swift-tools-support-core')
   tsc_build_dir = os.path.join(build_dir, 'swift-tools-support-core')
-  flags = [
-      '-DSwiftSystem_DIR=' + os.path.join(os.path.join(build_dir, 'swift-system'), 'cmake/modules')]
+  flags = []
   tsc_cmake_flags = base_cmake_flags + flags
 
   tsc_swift_flags = swift_flags[:]
@@ -553,8 +530,7 @@ def build_swift_driver_using_cmake(args, target, swiftc_exec, build_dir, base_cm
         '-DLLBuild_DIR=' + os.path.join(os.path.join(dependencies_dir, 'llbuild'), 'cmake/modules'),
         '-DTSC_DIR=' + os.path.join(os.path.join(dependencies_dir, 'swift-tools-support-core'), 'cmake/modules'),
         '-DYams_DIR=' + os.path.join(os.path.join(dependencies_dir, 'yams'), 'cmake/modules'),
-        '-DArgumentParser_DIR=' + os.path.join(os.path.join(dependencies_dir, 'swift-argument-parser'), 'cmake/modules'),
-        '-DSwiftSystem_DIR=' + os.path.join(os.path.join(dependencies_dir, 'swift-system'), 'cmake/modules')]
+        '-DArgumentParser_DIR=' + os.path.join(os.path.join(dependencies_dir, 'swift-argument-parser'), 'cmake/modules')]
   driver_cmake_flags = base_cmake_flags + flags
   cmake_build(args, swiftc_exec, driver_cmake_flags, driver_swift_flags,
               driver_source_dir, driver_build_dir)
@@ -585,19 +561,24 @@ def cmake_build(args, swiftc_exec, cmake_args, swift_flags, source_path,
 
   if args.verbose:
     print(' '.join(ninja_cmd))
+  # Note: encoding is explicitly set to None to indicate that the output must
+  # be bytes, not strings. This is to work around per-system differences in
+  # default encoding. Some systems have a default encoding of 'ascii', but that
+  # conflicts with this output, which can contain UTF encoded characters. The
+  # bytes are then written, instead of printed, to bypass issues with encoding.
   ninjaProcess = subprocess.Popen(ninja_cmd, cwd=build_dir,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
-                                  env = os.environ,
-                                  encoding='utf-8')
+                                  env=os.environ,
+                                  encoding=None)
   stdout, stderr = ninjaProcess.communicate()
   if ninjaProcess.returncode != 0:
-    print(stdout)
+    sys.stdout.buffer.write(stdout)
     print('Ninja invocation failed: ')
-    print(stderr)
+    sys.stderr.buffer.write(stderr)
     sys.exit(ninjaProcess.returncode)
   if args.verbose:
-    print(stdout)
+    sys.stdout.buffer.write(stdout)
 
 def get_build_target(swiftc_path, args, cross_compile=False):
     """Returns the target-triple of the current machine."""
