@@ -17,12 +17,3 @@ struct _MatchResult<S: MatchingCollectionSearcher> {
     match.startIndex..<match.endIndex
   }
 }
-
-struct _BackwardMatchResult<S: BackwardMatchingCollectionSearcher> {
-  let match: S.BackwardSearched.SubSequence
-  let result: S.Match
-  
-  var range: Range<S.BackwardSearched.Index> {
-    match.startIndex..<match.endIndex
-  }
-}
