@@ -9,7 +9,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-@_implementationOnly import _RegexParser
+internal import _RegexParser
 
 /// A type that represents a regular expression.
 ///
@@ -76,6 +76,16 @@ public protocol RegexComponent<RegexOutput> {
 /// instances using a clear and flexible declarative syntax. Using this
 /// style, you can combine, capture, and transform regexes, `RegexBuilder`
 /// types, and custom parsers.
+///
+/// > Note:
+/// > Prior to Swift 6,
+/// > you might need to write `#/myregex/#` instead of `/myregex/`
+/// > when you make a regular expression using a literal.
+/// > For more information,
+/// > see [Regular Expression Literals][regex-literal] in  *[The Swift Programming Language][tspl]*.
+///
+/// [regex-literal]: https://docs.swift.org/swift-book/documentation/the-swift-programming-language/lexicalstructure/#Regular-Expression-Literals
+/// [tspl]: https://docs.swift.org/swift-book/
 @available(SwiftStdlib 5.7, *)
 public struct Regex<Output>: RegexComponent {
   let program: Program
