@@ -1831,7 +1831,6 @@ fileprivate let regexWithNonCapture = #/:(?:\d+):/#
 @available(SwiftStdlib 5.7, *)
 extension RegexDSLTests {
   func testLabeledCaptures_regularCapture() throws {
-    return
     // The output type of a regex with unlabeled captures is concatenated.
     let dslWithCapture = Regex {
       OneOrMore(.word)
@@ -1846,9 +1845,8 @@ extension RegexDSLTests {
   }
   
   func testLabeledCaptures_labeledCapture() throws {
-    return
-    guard #available(macOS 13, *) else {
-      throw XCTSkip("Fix only exists on macOS 13")
+    guard #available(macOS 14.0, *) else {
+      throw XCTSkip("Fix only exists on macOS 14")
     }
     // The output type of a regex with a labeled capture is dropped.
     let dslWithLabeledCapture = Regex {
@@ -1870,7 +1868,6 @@ extension RegexDSLTests {
   }
   
   func testLabeledCaptures_coalescingWithCapture() throws {
-    return
     let coalescingWithCapture = Regex {
       "e" as Character
       #/\u{301}(\d*)/#
@@ -1887,9 +1884,8 @@ extension RegexDSLTests {
   }
   
   func testLabeledCaptures_bothCapture() throws {
-    return
-    guard #available(macOS 13, *) else {
-      throw XCTSkip("Fix only exists on macOS 13")
+    guard #available(macOS 14.0, *) else {
+      throw XCTSkip("Fix only exists on macOS 14")
     }
     // Only the output type of a regex with a labeled capture is dropped,
     // outputs of other regexes in the same DSL are concatenated.
@@ -1914,9 +1910,8 @@ extension RegexDSLTests {
   }
   
   func testLabeledCaptures_tooManyCapture() throws {
-    return
-    guard #available(macOS 13, *) else {
-      throw XCTSkip("Fix only exists on macOS 13")
+    guard #available(macOS 14.0, *) else {
+      throw XCTSkip("Fix only exists on macOS 14")
     }
     // The output type of a regex with too many captures is dropped.
     // "Too many" means the left and right output types would add up to >= 10.
