@@ -2193,6 +2193,41 @@ extension RegexTests {
       ("cafe", true),
       ("CaFe", true),
       ("EfAc", true))
+    
+    matchTest(
+      #"(?i)a+b"#,
+      ("ab", true),
+      ("Ab", true),
+      ("aB", true),
+      ("AB", true),
+      ("AaAab", true),
+      ("aaaAB", true))
+    matchTest(
+      #"^(?i)a?b$"#,
+      ("ab", true),
+      ("Ab", true),
+      ("aB", true),
+      ("AB", true),
+      ("aaB", false),
+      ("b", true),
+      ("B", true))
+    matchTest(
+      #"^(?i)[a]?b$"#,
+      ("ab", true),
+      ("Ab", true),
+      ("aB", true),
+      ("AB", true),
+      ("b", true),
+      ("B", true))
+    matchTest(
+      #"^(?i)a{2,4}b$"#,
+      ("ab", false),
+      ("Ab", false),
+      ("AaB", true),
+      ("aAB", true),
+      ("aAaB", true),
+      ("aAaAB", true),
+      ("AaAaAB", false))
   }
 
   func testNonSemanticWhitespace() {
@@ -2381,7 +2416,18 @@ extension RegexTests {
     XCTAssertTrue("cafe".contains(caseInsensitiveRegex))
     XCTAssertTrue("CaFe".contains(caseInsensitiveRegex))
   }
-  
+
+  // https://github.com/swiftlang/swift-experimental-string-processing/issues/768
+  func testWordBoundaryCaching() throws {
+    // This will first find word boundaries up til the middle before failing,
+    // then it will find word boundaries til late in the string, then fail,
+    // and finally should succeed on a word boundary cached from the first
+    // attempt.
+    let input = "first second third fourth"
+    let regex = try Regex(#".*second\bX|.*third\bX|.*first\b"#)
+    XCTAssertTrue(input.contains(regex))
+  }
+
   // MARK: Character Semantics
   
   var eComposed: String { "é" }
