@@ -1,28 +1,27 @@
-import PackagePlugin
 import Foundation
+import PackagePlugin
 
 @main
 struct LintPlugin {
   func lint(tool: PluginContext.Tool, targetDirectories: [String], configurationFilePath: String?) throws {
     let swiftFormatExec = URL(fileURLWithPath: tool.path.string)
-    
+
     var arguments: [String] = ["lint"]
-    
+
     arguments.append(contentsOf: targetDirectories)
-    
+
     arguments.append(contentsOf: ["--recursive", "--parallel", "--strict"])
-    
+
     if let configurationFilePath = configurationFilePath {
       arguments.append(contentsOf: ["--configuration", configurationFilePath])
     }
-    
+
     let process = try Process.run(swiftFormatExec, arguments: arguments)
     process.waitUntilExit()
-    
+
     if process.terminationReason == .exit && process.terminationStatus == 0 {
       print("Linted the source code.")
-    }
-    else {
+    } else {
       let problem = "\(process.terminationReason):\(process.terminationStatus)"
       Diagnostics.error("swift-format invocation failed: \(problem)")
     }
@@ -35,16 +34,17 @@ extension LintPlugin: CommandPlugin {
     arguments: [String]
   ) async throws {
     let swiftFormatTool = try context.tool(named: "swift-format")
-    
+
     // Extract the arguments that specify what targets to format.
     var argExtractor = ArgumentExtractor(arguments)
     let targetNames = argExtractor.extractOption(named: "target")
-    
-    let targetsToFormat = targetNames.isEmpty ? context.package.targets : try context.package.targets(named: targetNames)
+
+    let targetsToFormat =
+      targetNames.isEmpty ? context.package.targets : try context.package.targets(named: targetNames)
     let configurationFilePath = argExtractor.extractOption(named: "swift-format-configuration").first
-    
+
     let sourceCodeTargets = targetsToFormat.compactMap { $0 as? SourceModuleTarget }
-    
+
     try lint(
       tool: swiftFormatTool,
       targetDirectories: sourceCodeTargets.map(\.directory.string),
@@ -61,7 +61,7 @@ extension LintPlugin: XcodeCommandPlugin {
     let swiftFormatTool = try context.tool(named: "swift-format")
     var argExtractor = ArgumentExtractor(arguments)
     let configurationFilePath = argExtractor.extractOption(named: "swift-format-configuration").first
-    
+
     try lint(
       tool: swiftFormatTool,
       targetDirectories: [context.xcodeProject.directory.string],
