@@ -45,7 +45,7 @@ public final class SwiftFormatter {
   /// This form of the `format` function automatically folds expressions using the default operator
   /// set defined in Swift. If you need more control over this—for example, to provide the correct
   /// precedence relationships for custom operators—you must parse and fold the syntax tree
-  /// manually and then call ``format(syntax:assumingFileURL:to:)``.
+  /// manually and then call ``format(syntax:source:operatorTable:assumingFileURL:selection:to:)``.
   ///
   /// - Parameters:
   ///   - url: The URL of the file containing the code to format.
@@ -72,7 +72,8 @@ public final class SwiftFormatter {
       assumingFileURL: url,
       selection: .infinite,
       to: &outputStream,
-      parsingDiagnosticHandler: parsingDiagnosticHandler)
+      parsingDiagnosticHandler: parsingDiagnosticHandler
+    )
   }
 
   /// Formats the given Swift source code and writes the result to an output stream.
@@ -80,7 +81,7 @@ public final class SwiftFormatter {
   /// This form of the `format` function automatically folds expressions using the default operator
   /// set defined in Swift. If you need more control over this—for example, to provide the correct
   /// precedence relationships for custom operators—you must parse and fold the syntax tree
-  /// manually and then call ``format(syntax:assumingFileURL:to:)``.
+  /// manually and then call ``format(syntax:source:operatorTable:assumingFileURL:selection:to:)``.
   ///
   /// - Parameters:
   ///   - source: The Swift source code to be formatted.
@@ -88,6 +89,10 @@ public final class SwiftFormatter {
   ///     which is associated with any diagnostics emitted during formatting. If this is nil, a
   ///     dummy value will be used.
   ///   - selection: The ranges to format
+  ///   - experimentalFeatures: The set of experimental features that should be enabled in the
+  ///     parser. These names must be from the set of parser-recognized experimental language
+  ///     features in `SwiftParser`'s `Parser.ExperimentalFeatures` enum, which match the spelling
+  ///     defined in the compiler's `Features.def` file.
   ///   - outputStream: A value conforming to `TextOutputStream` to which the formatted output will
   ///     be written.
   ///   - parsingDiagnosticHandler: An optional callback that will be notified if there are any
@@ -97,6 +102,7 @@ public final class SwiftFormatter {
     source: String,
     assumingFileURL url: URL?,
     selection: Selection,
+    experimentalFeatures: Set<String> = [],
     to outputStream: inout Output,
     parsingDiagnosticHandler: ((Diagnostic, SourceLocation) -> Void)? = nil
   ) throws {
@@ -109,10 +115,17 @@ public final class SwiftFormatter {
       source: source,
       operatorTable: .standardOperators,
       assumingFileURL: url,
-      parsingDiagnosticHandler: parsingDiagnosticHandler)
+      experimentalFeatures: experimentalFeatures,
+      parsingDiagnosticHandler: parsingDiagnosticHandler
+    )
     try format(
-      syntax: sourceFile, source: source, operatorTable: .standardOperators, assumingFileURL: url,
-      selection: selection, to: &outputStream)
+      syntax: sourceFile,
+      source: source,
+      operatorTable: .standardOperators,
+      assumingFileURL: url,
+      selection: selection,
+      to: &outputStream
+    )
   }
 
   /// Formats the given Swift syntax tree and writes the result to an output stream.
@@ -137,14 +150,24 @@ public final class SwiftFormatter {
   ///     be written.
   /// - Throws: If an unrecoverable error occurs when formatting the code.
   public func format<Output: TextOutputStream>(
-    syntax: SourceFileSyntax, source: String, operatorTable: OperatorTable,
-    assumingFileURL url: URL?, selection: Selection, to outputStream: inout Output
+    syntax: SourceFileSyntax,
+    source: String,
+    operatorTable: OperatorTable,
+    assumingFileURL url: URL?,
+    selection: Selection,
+    to outputStream: inout Output
   ) throws {
     let assumedURL = url ?? URL(fileURLWithPath: "source")
     let context = Context(
-      configuration: configuration, operatorTable: operatorTable, findingConsumer: findingConsumer,
-      fileURL: assumedURL, selection: selection, sourceFileSyntax: syntax, source: source,
-      ruleNameCache: ruleNameCache)
+      configuration: configuration,
+      operatorTable: operatorTable,
+      findingConsumer: findingConsumer,
+      fileURL: assumedURL,
+      selection: selection,
+      sourceFileSyntax: syntax,
+      source: source,
+      ruleNameCache: ruleNameCache
+    )
     let pipeline = FormatPipeline(context: context)
     let transformedSyntax = pipeline.rewrite(Syntax(syntax))
 
@@ -158,7 +181,8 @@ public final class SwiftFormatter {
       source: source,
       node: transformedSyntax,
       printTokenStream: debugOptions.contains(.dumpTokenStream),
-      whitespaceOnly: false)
+      whitespaceOnly: false
+    )
     outputStream.write(printer.prettyPrint())
   }
 }
