@@ -26,7 +26,8 @@ extension LintPipeline {
   ///   - node: The syntax node on which the rule will be applied. This lets us check whether the
   ///     rule is enabled for the particular source range where the node occurs.
   func visitIfEnabled<Rule: SyntaxLintRule, Node: SyntaxProtocol>(
-    _ visitor: (Rule) -> (Node) -> SyntaxVisitorContinueKind, for node: Node
+    _ visitor: (Rule) -> (Node) -> SyntaxVisitorContinueKind,
+    for node: Node
   ) {
     guard context.shouldFormat(Rule.self, node: Syntax(node)) else { return }
     let ruleId = ObjectIdentifier(Rule.self)
@@ -48,7 +49,8 @@ extension LintPipeline {
   ///   - node: The syntax node on which the rule will be applied. This lets us check whether the
   ///     rule is enabled for the particular source range where the node occurs.
   func visitIfEnabled<Rule: SyntaxFormatRule, Node: SyntaxProtocol>(
-    _ visitor: (Rule) -> (Node) -> Any, for node: Node
+    _ visitor: (Rule) -> (Node) -> Any,
+    for node: Node
   ) {
     // Note that visitor function type is expressed as `Any` because we ignore the return value, but
     // more importantly because the `visit` methods return protocol refinements of `Syntax` that
@@ -66,7 +68,8 @@ extension LintPipeline {
   ///   - rule: The type of the syntax rule we're cleaning up.
   ///   - node: The syntax node htat our traversal has left.
   func onVisitPost<R: Rule, Node: SyntaxProtocol>(
-    rule: R.Type, for node: Node
+    rule: R.Type,
+    for node: Node
   ) {
     let rule = ObjectIdentifier(rule)
     if case .some(let skipNode) = self.shouldSkipChildren[rule] {
