@@ -10,8 +10,8 @@
 //
 //===----------------------------------------------------------------------===//
 
-import SwiftSyntax
 import Foundation
+import SwiftSyntax
 
 /// PrettyPrinter takes a Syntax node and outputs a well-formatted, re-indented reproduction of the
 /// code as a String.
@@ -76,12 +76,13 @@ public class PrettyPrinter {
   /// original source. When enabling formatting, we copy the text between `disabledPosition` and the
   /// current position to `outputBuffer`. From then on, we continue to format until the next
   /// `disableFormatting` token.
-  private var disabledPosition: AbsolutePosition? = nil
+  private var disabledPosition: AbsolutePosition? = nil {
+    didSet {
+      outputBuffer.isEnabled = disabledPosition == nil
+    }
+  }
 
-  private var outputBuffer: String = ""
-
-  /// The number of spaces remaining on the current line.
-  private var spaceRemaining: Int
+  private var outputBuffer: PrettyPrintBuffer
 
   /// Keep track of the token lengths.
   private var lengths = [Int]()
@@ -103,7 +104,11 @@ public class PrettyPrinter {
 
   /// Keeps track of the line numbers and indentation states of the open (and unclosed) breaks seen
   /// so far.
-  private var activeOpenBreaks: [ActiveOpenBreak] = []
+  private var activeOpenBreaks: [ActiveOpenBreak] = [] {
+    didSet {
+      outputBuffer.currentIndentation = currentIndentation
+    }
+  }
 
   /// Stack of the active breaking contexts.
   private var activeBreakingContexts: [ActiveBreakingContext] = []
@@ -111,11 +116,14 @@ public class PrettyPrinter {
   /// The most recently ended breaking context, used to force certain following `contextual` breaks.
   private var lastEndedBreakingContext: ActiveBreakingContext? = nil
 
-  /// Keeps track of the current line number being printed.
-  private var lineNumber: Int = 1
-
   /// Indicates whether or not the current line being printed is a continuation line.
-  private var currentLineIsContinuation = false
+  private var currentLineIsContinuation = false {
+    didSet {
+      if oldValue != currentLineIsContinuation {
+        outputBuffer.currentIndentation = currentIndentation
+      }
+    }
+  }
 
   /// Keeps track of the continuation line state as you go into and out of open-close break groups.
   private var continuationStack: [Bool] = []
@@ -124,18 +132,6 @@ public class PrettyPrinter {
   /// corresponding end token are encountered.
   private var commaDelimitedRegionStack: [Int] = []
 
-  /// Keeps track of the most recent number of consecutive newlines that have been printed.
-  ///
-  /// This value is reset to zero whenever non-newline content is printed.
-  private var consecutiveNewlineCount = 0
-
-  /// Keeps track of the most recent number of spaces that should be printed before the next text
-  /// token.
-  private var pendingSpaces = 0
-
-  /// Indicates whether or not the printer is currently at the beginning of a line.
-  private var isAtStartOfLine = true
-
   /// Tracks how many printer control tokens to suppress firing breaks are active.
   private var activeBreakSuppressionCount = 0
 
@@ -155,7 +151,8 @@ public class PrettyPrinter {
   private var currentIndentation: [Indent] {
     let indentation = configuration.indentation
     var totalIndentation: [Indent] = activeOpenBreaks.flatMap { (open) -> [Indent] in
-      let count = (open.contributesBlockIndent ? 1 : 0)
+      let count =
+        (open.contributesBlockIndent ? 1 : 0)
         + (open.contributesContinuationIndent ? 1 : 0)
       return Array(repeating: indentation, count: count)
     }
@@ -173,7 +170,7 @@ public class PrettyPrinter {
   /// line number to increase by one by the time we reach the break, when we really wish to consider
   /// the break as being located at the end of the previous line.
   private var openCloseBreakCompensatingLineNumber: Int {
-    return isAtStartOfLine ? lineNumber - 1 : lineNumber
+    return outputBuffer.lineNumber - (outputBuffer.isAtStartOfLine ? 1 : 0)
   }
 
   /// Creates a new PrettyPrinter with the provided formatting configuration.
@@ -191,79 +188,15 @@ public class PrettyPrinter {
     self.tokens = node.makeTokenStream(
       configuration: configuration,
       selection: context.selection,
-      operatorTable: context.operatorTable)
+      operatorTable: context.operatorTable
+    )
     self.maxLineLength = configuration.lineLength
-    self.spaceRemaining = self.maxLineLength
     self.printTokenStream = printTokenStream
     self.whitespaceOnly = whitespaceOnly
-  }
-
-  /// Append the given string to the output buffer.
-  ///
-  /// No further processing is performed on the string.
-  private func writeRaw<S: StringProtocol>(_ str: S) {
-    if disabledPosition == nil {
-      outputBuffer.append(String(str))
-    }
-  }
-
-  /// Writes newlines into the output stream, taking into account any preexisting consecutive
-  /// newlines and the maximum allowed number of blank lines.
-  ///
-  /// This function does some implicit collapsing of consecutive newlines to ensure that the
-  /// results are consistent when breaks and explicit newlines coincide. For example, imagine a
-  /// break token that fires (thus creating a single non-discretionary newline) because it is
-  /// followed by a group that contains 2 discretionary newlines that were found in the user's
-  /// source code at that location. In that case, the break "overlaps" with the discretionary
-  /// newlines and it will write a newline before we get to the discretionaries. Thus, we have to
-  /// subtract the previously written newlines during the second call so that we end up with the
-  /// correct number overall.
-  ///
-  /// - Parameter newlines: The number and type of newlines to write.
-  private func writeNewlines(_ newlines: NewlineBehavior) {
-    let numberToPrint: Int
-    switch newlines {
-    case .elective:
-      numberToPrint = consecutiveNewlineCount == 0 ? 1 : 0
-    case .soft(let count, _):
-      // We add 1 to the max blank lines because it takes 2 newlines to create the first blank line.
-      numberToPrint = min(count, configuration.maximumBlankLines + 1) - consecutiveNewlineCount
-    case .hard(let count):
-      numberToPrint = count
-    }
-
-    guard numberToPrint > 0 else { return }
-    writeRaw(String(repeating: "\n", count: numberToPrint))
-    lineNumber += numberToPrint
-    isAtStartOfLine = true
-    consecutiveNewlineCount += numberToPrint
-    pendingSpaces = 0
-  }
-
-  /// Request that the given number of spaces be printed out before the next text token.
-  ///
-  /// Spaces are printed only when the next text token is printed in order to prevent us from
-  /// printing lines that are only whitespace or have trailing whitespace.
-  private func enqueueSpaces(_ count: Int) {
-    pendingSpaces += count
-    spaceRemaining -= count
-  }
-
-  /// Writes the given text to the output stream.
-  ///
-  /// Before printing the text, this function will print any line-leading indentation or interior
-  /// leading spaces that are required before the text itself.
-  private func write(_ text: String) {
-    if isAtStartOfLine {
-      writeRaw(currentIndentation.indentation())
-      spaceRemaining = maxLineLength - currentIndentation.length(in: configuration)
-      isAtStartOfLine = false
-    } else if pendingSpaces > 0  {
-      writeRaw(String(repeating: " ", count: pendingSpaces))
-    }
-    writeRaw(text)
-    consecutiveNewlineCount = 0
-    pendingSpaces = 0
+    self.outputBuffer = PrettyPrintBuffer(
+      maximumBlankLines: configuration.maximumBlankLines,
+      tabWidth: configuration.tabWidth
+    )
   }
 
   /// Print out the provided token, and apply line-wrapping and indentation as needed.
@@ -285,8 +218,7 @@ public class PrettyPrinter {
 
     switch token {
     case .contextualBreakingStart:
-      activeBreakingContexts.append(ActiveBreakingContext(lineNumber: lineNumber))
-
+      activeBreakingContexts.append(ActiveBreakingContext(lineNumber: outputBuffer.lineNumber))
       // Discard the last finished breaking context to keep it from effecting breaks inside of the
       // new context. The discarded context has already either had an impact on the contextual break
       // after it or there was no relevant contextual break, so it's safe to discard.
@@ -306,7 +238,7 @@ public class PrettyPrinter {
     // the group.
     case .open(let breaktype):
       // Determine if the break tokens in this group need to be forced.
-      if (length > spaceRemaining || lastBreak), case .consistent = breaktype {
+      if (!canFit(length) || lastBreak), case .consistent = breaktype {
         forceBreakStack.append(true)
       } else {
         forceBreakStack.append(false)
@@ -347,8 +279,9 @@ public class PrettyPrinter {
         // lines within it (unless they are themselves continuations within that particular
         // scope), so we need the continuation indentation to persist across all the lines in that
         // scope. Additionally, continuation open breaks must indent when the break fires.
-        let continuationBreakWillFire = openKind == .continuation
-          && (isAtStartOfLine || length > spaceRemaining || mustBreak)
+        let continuationBreakWillFire =
+          openKind == .continuation
+          && (outputBuffer.isAtStartOfLine || !canFit(length) || mustBreak)
         let contributesContinuationIndent = currentLineIsContinuation || continuationBreakWillFire
 
         activeOpenBreaks.append(
@@ -357,7 +290,9 @@ public class PrettyPrinter {
             kind: openKind,
             lineNumber: currentLineNumber,
             contributesContinuationIndent: contributesContinuationIndent,
-            contributesBlockIndent: openKind == .block))
+            contributesBlockIndent: openKind == .block
+          )
+        )
 
         continuationStack.append(currentLineIsContinuation)
 
@@ -371,13 +306,12 @@ public class PrettyPrinter {
           fatalError("Unmatched closing break")
         }
 
-        let openedOnDifferentLine
-          = openCloseBreakCompensatingLineNumber != matchingOpenBreak.lineNumber
+        let openedOnDifferentLine = openCloseBreakCompensatingLineNumber != matchingOpenBreak.lineNumber
 
         if matchingOpenBreak.contributesBlockIndent {
           // The actual line number is used, instead of the compensating line number. When the close
           // break is at the start of a new line, the block indentation isn't carried to the new line.
-          let currentLine = lineNumber
+          let currentLine = outputBuffer.lineNumber
           // When two or more open breaks are encountered on the same line, only the final open
           // break is allowed to increase the block indent, avoiding multiple block indents. As the
           // open breaks on that line are closed, the new final open break must be enabled again to
@@ -395,7 +329,7 @@ public class PrettyPrinter {
           // If it's a mandatory breaking close, then we must break (regardless of line length) if
           // the break is on a different line than its corresponding open break.
           mustBreak = openedOnDifferentLine
-        } else if spaceRemaining == 0 {
+        } else if !canFit() {
           // If there is no room left on the line, then we must force this break to fire so that the
           // next token that comes along (typically a closing bracket of some kind) ends up on the
           // next line.
@@ -423,12 +357,14 @@ public class PrettyPrinter {
           //
           // Likewise, we need to do this if we popped an old continuation state off the stack,
           // even if the break *doesn't* fire.
-          let matchingOpenBreakIndented = matchingOpenBreak.contributesContinuationIndent
+          let matchingOpenBreakIndented =
+            matchingOpenBreak.contributesContinuationIndent
             || matchingOpenBreak.contributesBlockIndent
           currentLineIsContinuation = matchingOpenBreakIndented && openedOnDifferentLine
         }
 
-        let wasContinuationWhenOpened = (continuationStack.popLast() ?? false)
+        let wasContinuationWhenOpened =
+          (continuationStack.popLast() ?? false)
           || matchingOpenBreak.contributesContinuationIndent
           // This ensures a continuation indent is propagated to following scope when an initial
           // scope would've indented if the leading break wasn't at the start of a line.
@@ -453,13 +389,13 @@ public class PrettyPrinter {
         // context includes a multiline trailing closure or multiline function argument list.
         if let lastBreakingContext = lastEndedBreakingContext {
           if configuration.lineBreakAroundMultilineExpressionChainComponents {
-            mustBreak = lastBreakingContext.lineNumber != lineNumber
+            mustBreak = lastBreakingContext.lineNumber != outputBuffer.lineNumber
           }
         }
 
         // Wait for a contextual break to fire and then update the breaking behavior for the rest of
         // the contextual breaks in this scope to match the behavior of the one that fired.
-        let willFire = (!isAtStartOfLine && length > spaceRemaining) || mustBreak
+        let willFire = !canFit(length) || mustBreak
         if willFire {
           // Update the active breaking context according to the most recently finished breaking
           // context so all following contextual breaks in this scope to have matching behavior.
@@ -468,7 +404,7 @@ public class PrettyPrinter {
             case .unset = activeContext.contextualBreakingBehavior
           {
             activeBreakingContexts[activeBreakingContexts.count - 1].contextualBreakingBehavior =
-              (closedContext.lineNumber == lineNumber) ? .continuation : .maintain
+              (closedContext.lineNumber == outputBuffer.lineNumber) ? .continuation : .maintain
           }
         }
 
@@ -486,7 +422,7 @@ public class PrettyPrinter {
 
       var overrideBreakingSuppressed = false
       switch newline {
-      case .elective: break
+      case .elective, .escaped: break
       case .soft(_, let discretionary):
         // A discretionary newline (i.e. from the source) should create a line break even if the
         // rules for breaking are disabled.
@@ -499,12 +435,16 @@ public class PrettyPrinter {
       }
 
       let suppressBreaking = isBreakingSuppressed && !overrideBreakingSuppressed
-      if !suppressBreaking && ((!isAtStartOfLine && length > spaceRemaining) || mustBreak) {
+      if !suppressBreaking && (!canFit(length) || mustBreak) {
         currentLineIsContinuation = isContinuationIfBreakFires
-        writeNewlines(newline)
+        if case .escaped = newline {
+          outputBuffer.enqueueSpaces(size)
+          outputBuffer.write("\\")
+        }
+        outputBuffer.writeNewlines(newline, shouldIndentBlankLines: configuration.indentBlankLines)
         lastBreak = true
       } else {
-        if isAtStartOfLine {
+        if outputBuffer.isAtStartOfLine {
           // Make sure that the continuation status is correct even at the beginning of a line
           // (for example, after a newline token). This is necessary because a discretionary newline
           // might be inserted into the token stream before a continuation break, and the length of
@@ -512,39 +452,37 @@ public class PrettyPrinter {
           // treat the line as a continuation.
           currentLineIsContinuation = isContinuationIfBreakFires
         }
-        enqueueSpaces(size)
+        outputBuffer.enqueueSpaces(size)
         lastBreak = false
       }
 
     // Print out the number of spaces according to the size, and adjust spaceRemaining.
     case .space(let size, _):
-      enqueueSpaces(size)
+      if configuration.indentBlankLines, outputBuffer.isAtStartOfLine {
+        // An empty string write is needed to add line-leading indentation that matches the current indentation on a line that contains only whitespaces.
+        outputBuffer.write("")
+      }
+      outputBuffer.enqueueSpaces(size)
 
     // Print any indentation required, followed by the text content of the syntax token.
     case .syntax(let text):
       guard !text.isEmpty else { break }
       lastBreak = false
-      write(text)
-      spaceRemaining -= text.count
+      outputBuffer.write(text)
 
     case .comment(let comment, let wasEndOfLine):
       lastBreak = false
 
-      write(comment.print(indent: currentIndentation))
       if wasEndOfLine {
-        if comment.length > spaceRemaining && !isBreakingSuppressed {
+        if !(canFit(comment.length) || isBreakingSuppressed) {
           diagnose(.moveEndOfLineComment, category: .endOfLineComment)
         }
-      } else {
-        spaceRemaining -= comment.length
       }
+      outputBuffer.write(comment.print(indent: currentIndentation))
 
     case .verbatim(let verbatim):
-      writeRaw(verbatim.print(indent: currentIndentation))
-      consecutiveNewlineCount = 0
-      pendingSpaces = 0
+      outputBuffer.writeVerbatim(verbatim.print(indent: currentIndentation), length)
       lastBreak = false
-      spaceRemaining -= length
 
     case .printerControl(let kind):
       switch kind {
@@ -574,7 +512,8 @@ public class PrettyPrinter {
       // We never want to add a trailing comma in an initializer so we disable trailing commas on
       // single element collections.
       let shouldHaveTrailingComma =
-        startLineNumber != openCloseBreakCompensatingLineNumber && !isSingleElement && configuration.multiElementCollectionTrailingCommas
+        startLineNumber != openCloseBreakCompensatingLineNumber && !isSingleElement
+        && configuration.multiElementCollectionTrailingCommas
       if shouldHaveTrailingComma && !hasTrailingComma {
         diagnose(.addTrailingComma, category: .trailingComma)
       } else if !shouldHaveTrailingComma && hasTrailingComma {
@@ -583,8 +522,7 @@ public class PrettyPrinter {
 
       let shouldWriteComma = whitespaceOnly ? hasTrailingComma : shouldHaveTrailingComma
       if shouldWriteComma {
-        write(",")
-        spaceRemaining -= 1
+        outputBuffer.write(",")
       }
 
     case .enableFormatting(let enabledPosition):
@@ -602,19 +540,14 @@ public class PrettyPrinter {
       var text = String(source[start..<end])
       // strip trailing whitespace so that the next formatting can add the right amount
       if let nonWhitespace = text.rangeOfCharacter(
-        from: CharacterSet.whitespaces.inverted, options: .backwards) {
+        from: CharacterSet.whitespaces.inverted,
+        options: .backwards
+      ) {
         text = String(text[..<nonWhitespace.upperBound])
       }
 
       self.disabledPosition = nil
-      writeRaw(text)
-      if text.hasSuffix("\n") {
-        isAtStartOfLine = true
-        consecutiveNewlineCount = 1
-      } else {
-        isAtStartOfLine = false
-        consecutiveNewlineCount = 0
-      }
+      outputBuffer.writeVerbatimAfterEnablingFormatting(text)
 
     case .disableFormatting(let newPosition):
       assert(disabledPosition == nil)
@@ -622,6 +555,13 @@ public class PrettyPrinter {
     }
   }
 
+  /// Indicates whether the current line can fit a string of the given length. If no length
+  /// is given, it indicates whether the current line can accomodate *any* text.
+  private func canFit(_ length: Int = 1) -> Bool {
+    let spaceRemaining = configuration.lineLength - outputBuffer.column
+    return outputBuffer.isAtStartOfLine || length <= spaceRemaining
+  }
+
   /// Scan over the array of Tokens and calculate their lengths.
   ///
   /// This method is based on the `scan` function described in Derek Oppen's "Pretty Printing" paper
@@ -673,16 +613,48 @@ public class PrettyPrinter {
       // Break lengths are equal to its size plus the token or group following it. Calculate the
       // length of any prior break tokens.
       case .break(_, let size, let newline):
-        if let index = delimIndexStack.last, case .break = tokens[index] {
-          lengths[index] += total
+        if let index = delimIndexStack.last, case .break(_, _, let lastNewline) = tokens[index] {
+          /// If the last break and this break are both `.escaped` we add an extra 1 to the total for the last `.escaped` break.
+          /// This is to handle situations where adding the `\` for an escaped line break would put us over the line length.
+          /// For example, consider the token sequence:
+          /// `[.syntax("this fits"), .break(.escaped), .syntax("this fits in line length"), .break(.escaped)]`
+          /// The naive layout of these tokens will incorrectly print as:
+          ///  """
+          ///  this fits this fits in line length \
+          ///  """
+          ///  which will be too long because of the '\' character. Instead we have to print it as:
+          ///  """
+          ///  this fits \
+          ///  this fits in line length
+          ///  """
+          ///
+          /// While not prematurely inserting a line in situations where a hard line break is occurring, such as:
+          ///
+          /// `[.syntax("some text"), .break(.escaped), .syntax("this is exactly the right length"), .break(.hard)]`
+          ///
+          /// We want this to print as:
+          /// """
+          /// some text this is exactly the right length
+          /// """
+          /// and not:
+          /// """
+          /// some text \
+          /// this is exactly the right length
+          /// """
+          if case .escaped = newline, case .escaped = lastNewline {
+            lengths[index] += total + 1
+          } else {
+            lengths[index] += total
+          }
           delimIndexStack.removeLast()
         }
         lengths.append(-total)
         delimIndexStack.append(i)
 
-        if case .elective = newline {
+        switch newline {
+        case .elective, .escaped:
           total += size
-        } else {
+        default:
           // `size` is never used in this case, because the break always fires. Use `maxLineLength`
           // to ensure enclosing groups are large enough to force preceding breaks to fire.
           total += maxLineLength
@@ -748,7 +720,7 @@ public class PrettyPrinter {
       fatalError("At least one .break(.open) was not matched by a .break(.close)")
     }
 
-    return outputBuffer
+    return outputBuffer.output
   }
 
   /// Used to track the indentation level for the debug token stream output.
@@ -843,11 +815,12 @@ public class PrettyPrinter {
   /// Emits a finding with the given message and category at the current location in `outputBuffer`.
   private func diagnose(_ message: Finding.Message, category: PrettyPrintFindingCategory) {
     // Add 1 since columns uses 1-based indices.
-    let column = maxLineLength - spaceRemaining + 1
+    let column = outputBuffer.column + 1
     context.findingEmitter.emit(
       message,
       category: category,
-      location: Finding.Location(file: context.fileURL.path, line: lineNumber, column: column))
+      location: Finding.Location(file: context.fileURL.path, line: outputBuffer.lineNumber, column: column)
+    )
   }
 }
 
