@@ -59,7 +59,8 @@ public class WhitespaceLinter {
       assert(
         safeCodeUnit(at: userWhitespace.endIndex, in: userText)
           == safeCodeUnit(at: formattedWhitespace.endIndex, in: formattedText),
-        "Non-whitespace characters do not match")
+        "Non-whitespace characters do not match"
+      )
 
       compareWhitespace(userWhitespace: userWhitespace, formattedWhitespace: formattedWhitespace)
 
@@ -82,7 +83,8 @@ public class WhitespaceLinter {
   ///   - formattedWhitespace: A slice of formatted text representing the current span of contiguous
   ///     whitespace that will be compared to the user whitespace.
   private func compareWhitespace(
-    userWhitespace: ArraySlice<UTF8.CodeUnit>, formattedWhitespace: ArraySlice<UTF8.CodeUnit>
+    userWhitespace: ArraySlice<UTF8.CodeUnit>,
+    formattedWhitespace: ArraySlice<UTF8.CodeUnit>
   ) {
     // We use a custom-crafted lazy-splitting iterator here instead of the standard
     // `Collection.split` function because Time Profiler indicated that a very large proportion of
@@ -97,7 +99,8 @@ public class WhitespaceLinter {
       userIndex: userWhitespace.startIndex,
       formattedIndex: formattedWhitespace.startIndex,
       userRuns: userRuns,
-      formattedRuns: formattedRuns)
+      formattedRuns: formattedRuns
+    )
 
     // No need to perform any further checks if the whitespace is identical.
     guard userWhitespace != formattedWhitespace else { return }
@@ -135,7 +138,10 @@ public class WhitespaceLinter {
             // If this isn't the last whitespace run, then it must precede a newline, so we check
             // for trailing whitespace violations.
             checkForTrailingWhitespaceErrors(
-              userIndex: userIndex, userRun: userRun, formattedRun: formattedRun)
+              userIndex: userIndex,
+              userRun: userRun,
+              formattedRun: formattedRun
+            )
           }
           userIndex += userRun.count + 1
         }
@@ -152,7 +158,8 @@ public class WhitespaceLinter {
         checkForIndentationErrors(
           userIndex: userIndex,
           userRun: userRunsIterator.latestElement!,
-          formattedRun: lastFormattedRun)
+          formattedRun: lastFormattedRun
+        )
       }
     }
 
@@ -163,7 +170,8 @@ public class WhitespaceLinter {
       diagnose(
         .addLinesError(excessFormattedLines),
         category: .addLines,
-        utf8Offset: userWhitespace.startIndex)
+        utf8Offset: userWhitespace.startIndex
+      )
     }
   }
 
@@ -257,7 +265,9 @@ public class WhitespaceLinter {
   ///   - userRun: A run of whitespace from the user text.
   ///   - formattedRun: A run of whitespace from the formatted text.
   private func checkForIndentationErrors(
-    userIndex: Int, userRun: ArraySlice<UTF8.CodeUnit>, formattedRun: ArraySlice<UTF8.CodeUnit>
+    userIndex: Int,
+    userRun: ArraySlice<UTF8.CodeUnit>,
+    formattedRun: ArraySlice<UTF8.CodeUnit>
   ) {
     guard userRun != formattedRun else { return }
 
@@ -266,7 +276,8 @@ public class WhitespaceLinter {
     diagnose(
       .indentationError(expected: expected, actual: actual),
       category: .indentation,
-      utf8Offset: userIndex)
+      utf8Offset: userIndex
+    )
   }
 
   /// Compare user and formatted whitespace buffers, and check for trailing whitespace.
@@ -276,7 +287,9 @@ public class WhitespaceLinter {
   ///   - userRun: The tokenized user whitespace buffer.
   ///   - formattedRun: The tokenized formatted whitespace buffer.
   private func checkForTrailingWhitespaceErrors(
-    userIndex: Int, userRun: ArraySlice<UTF8.CodeUnit>, formattedRun: ArraySlice<UTF8.CodeUnit>
+    userIndex: Int,
+    userRun: ArraySlice<UTF8.CodeUnit>,
+    formattedRun: ArraySlice<UTF8.CodeUnit>
   ) {
     if userRun != formattedRun {
       diagnose(.trailingWhitespaceError, category: .trailingWhitespace, utf8Offset: userIndex)
@@ -294,7 +307,9 @@ public class WhitespaceLinter {
   ///   - userRun: The tokenized user whitespace buffer.
   ///   - formattedRun: The tokenized formatted whitespace buffer.
   private func checkForSpacingErrors(
-    userIndex: Int, userRun: ArraySlice<UTF8.CodeUnit>, formattedRun: ArraySlice<UTF8.CodeUnit>
+    userIndex: Int,
+    userRun: ArraySlice<UTF8.CodeUnit>,
+    formattedRun: ArraySlice<UTF8.CodeUnit>
   ) {
     guard userRun != formattedRun else { return }
 
@@ -320,11 +335,20 @@ public class WhitespaceLinter {
   ///   - data: The input string.
   /// - Returns: A slice of `data` that covers the contiguous whitespace starting at the given
   ///   index.
-  private func contiguousWhitespace(startingAt offset: Int, in data: [UTF8.CodeUnit])
-    -> ArraySlice<UTF8.CodeUnit>
-  {
-    guard let whitespaceEnd =
-      data[offset...].firstIndex(where: { !UnicodeScalar($0).properties.isWhitespace })
+  private func contiguousWhitespace(
+    startingAt offset: Int,
+    in data: [UTF8.CodeUnit]
+  ) -> ArraySlice<UTF8.CodeUnit> {
+    func isWhitespace(_ char: UTF8.CodeUnit) -> Bool {
+      switch char {
+      case UInt8(ascii: " "), UInt8(ascii: "\n"), UInt8(ascii: "\t"), UInt8(ascii: "\r"), /*VT*/ 0x0B, /*FF*/ 0x0C:
+        return true
+      default:
+        return false
+      }
+    }
+    guard
+      let whitespaceEnd = data[offset...].firstIndex(where: { !isWhitespace($0) })
     else {
       return data[offset..<data.endIndex]
     }
@@ -354,7 +378,10 @@ public class WhitespaceLinter {
     let absolutePosition = AbsolutePosition(utf8Offset: utf8Offset)
     let sourceLocation = context.sourceLocationConverter.location(for: absolutePosition)
     context.findingEmitter.emit(
-      message, category: category, location: Finding.Location(sourceLocation))
+      message,
+      category: category,
+      location: Finding.Location(sourceLocation)
+    )
   }
 
   /// Returns the indentation that represents the indentation of the given whitespace, which is the
