@@ -17,6 +17,7 @@ import SwiftSyntax
 ///
 /// This rule does not apply to test code, defined as code which:
 ///   * Contains the line `import XCTest`
+///   * The function is marked with `@Test` attribute
 ///
 /// Lint: If an identifier contains underscores or begins with a capital letter, a lint error is
 ///       raised.
@@ -55,7 +56,10 @@ public final class AlwaysUseLowerCamelCase: SyntaxLintRule {
         continue
       }
       diagnoseLowerCamelCaseViolations(
-        pat.identifier, allowUnderscores: false, description: identifierDescription(for: node))
+        pat.identifier,
+        allowUnderscores: false,
+        description: identifierDescription(for: node)
+      )
     }
     return .visitChildren
   }
@@ -65,7 +69,10 @@ public final class AlwaysUseLowerCamelCase: SyntaxLintRule {
       return .visitChildren
     }
     diagnoseLowerCamelCaseViolations(
-      pattern.identifier, allowUnderscores: false, description: identifierDescription(for: node))
+      pattern.identifier,
+      allowUnderscores: false,
+      description: identifierDescription(for: node)
+    )
     return .visitChildren
   }
 
@@ -74,15 +81,24 @@ public final class AlwaysUseLowerCamelCase: SyntaxLintRule {
       if let closureParamList = input.as(ClosureShorthandParameterListSyntax.self) {
         for param in closureParamList {
           diagnoseLowerCamelCaseViolations(
-            param.name, allowUnderscores: false, description: identifierDescription(for: node))
+            param.name,
+            allowUnderscores: false,
+            description: identifierDescription(for: node)
+          )
         }
       } else if let parameterClause = input.as(ClosureParameterClauseSyntax.self) {
         for param in parameterClause.parameters {
           diagnoseLowerCamelCaseViolations(
-            param.firstName, allowUnderscores: false, description: identifierDescription(for: node))
+            param.firstName,
+            allowUnderscores: false,
+            description: identifierDescription(for: node)
+          )
           if let secondName = param.secondName {
             diagnoseLowerCamelCaseViolations(
-              secondName, allowUnderscores: false, description: identifierDescription(for: node))
+              secondName,
+              allowUnderscores: false,
+              description: identifierDescription(for: node)
+            )
           }
         }
       }
@@ -100,18 +116,27 @@ public final class AlwaysUseLowerCamelCase: SyntaxLintRule {
 
     // We allow underscores in test names, because there's an existing convention of using
     // underscores to separate phrases in very detailed test names.
-    let allowUnderscores = testCaseFuncs.contains(node)
+    let allowUnderscores = testCaseFuncs.contains(node) || node.hasAttribute("Test", inModule: "Testing")
+
     diagnoseLowerCamelCaseViolations(
-      node.name, allowUnderscores: allowUnderscores,
-      description: identifierDescription(for: node))
+      node.name,
+      allowUnderscores: allowUnderscores,
+      description: identifierDescription(for: node)
+    )
     for param in node.signature.parameterClause.parameters {
       // These identifiers aren't described using `identifierDescription(for:)` because no single
       // node can disambiguate the argument label from the parameter name.
       diagnoseLowerCamelCaseViolations(
-        param.firstName, allowUnderscores: false, description: "argument label")
+        param.firstName,
+        allowUnderscores: false,
+        description: "argument label"
+      )
       if let paramName = param.secondName {
         diagnoseLowerCamelCaseViolations(
-          paramName, allowUnderscores: false, description: "function parameter")
+          paramName,
+          allowUnderscores: false,
+          description: "function parameter"
+        )
       }
     }
     return .visitChildren
@@ -119,7 +144,10 @@ public final class AlwaysUseLowerCamelCase: SyntaxLintRule {
 
   public override func visit(_ node: EnumCaseElementSyntax) -> SyntaxVisitorContinueKind {
     diagnoseLowerCamelCaseViolations(
-      node.name, allowUnderscores: false, description: identifierDescription(for: node))
+      node.name,
+      allowUnderscores: false,
+      description: identifierDescription(for: node)
+    )
     return .skipChildren
   }
 
@@ -151,7 +179,9 @@ public final class AlwaysUseLowerCamelCase: SyntaxLintRule {
   }
 
   private func diagnoseLowerCamelCaseViolations(
-    _ identifier: TokenSyntax, allowUnderscores: Bool, description: String
+    _ identifier: TokenSyntax,
+    allowUnderscores: Bool,
+    description: String
   ) {
     guard case .identifier(let text) = identifier.tokenKind else { return }
     if text.isEmpty { return }
@@ -195,7 +225,8 @@ extension ReturnClauseSyntax {
 
 extension Finding.Message {
   fileprivate static func nameMustBeLowerCamelCase(
-    _ name: String, description: String
+    _ name: String,
+    description: String
   ) -> Finding.Message {
     "rename the \(description) '\(name)' using lowerCamelCase"
   }
