@@ -16,12 +16,11 @@ import SwiftSyntax
 ///
 /// Lint:  invalid use of `forEach` yield will yield a lint error.
 @_spi(Rules)
-public final class ReplaceForEachWithForLoop : SyntaxLintRule {
+public final class ReplaceForEachWithForLoop: SyntaxLintRule {
   public override func visit(_ node: FunctionCallExprSyntax) -> SyntaxVisitorContinueKind {
     // We are only interested in calls with a single trailing closure
     // argument.
-    if !node.arguments.isEmpty || node.trailingClosure == nil ||
-       !node.additionalTrailingClosures.isEmpty {
+    if !node.arguments.isEmpty || node.trailingClosure == nil || !node.additionalTrailingClosures.isEmpty {
       return .visitChildren
     }
 
