@@ -19,41 +19,40 @@ import SwiftSyntax
 class LintFrontend: Frontend {
   override func processFile(_ fileToProcess: FileToProcess) {
     let linter = SwiftLinter(
-      configuration: fileToProcess.configuration, findingConsumer: diagnosticsEngine.consumeFinding)
+      configuration: fileToProcess.configuration,
+      findingConsumer: diagnosticsEngine.consumeFinding
+    )
     linter.debugOptions = debugOptions
 
     let url = fileToProcess.url
     guard let source = fileToProcess.sourceText else {
       diagnosticsEngine.emitError(
-        "Unable to lint \(url.relativePath): file is not readable or does not exist.")
+        "Unable to lint \(url.relativePath): file is not readable or does not exist."
+      )
       return
     }
 
     do {
       try linter.lint(
         source: source,
-        assumingFileURL: url) { (diagnostic, location) in
-          guard !self.lintFormatOptions.ignoreUnparsableFiles else {
-            // No diagnostics should be emitted in this mode.
-            return
-          }
-          self.diagnosticsEngine.consumeParserDiagnostic(diagnostic, location)
+        assumingFileURL: url,
+        experimentalFeatures: Set(lintFormatOptions.experimentalFeatures)
+      ) { (diagnostic, location) in
+        guard !self.lintFormatOptions.ignoreUnparsableFiles else {
+          // No diagnostics should be emitted in this mode.
+          return
+        }
+        self.diagnosticsEngine.consumeParserDiagnostic(diagnostic, location)
       }
-
-    } catch SwiftFormatError.fileNotReadable {
-      diagnosticsEngine.emitError(
-        "Unable to lint \(url.relativePath): file is not readable or does not exist.")
-      return
     } catch SwiftFormatError.fileContainsInvalidSyntax {
       guard !lintFormatOptions.ignoreUnparsableFiles else {
         // The caller wants to silently ignore this error.
         return
       }
-      // Otherwise, relevant diagnostics about the problematic nodes have been emitted.
-      return
+      // Otherwise, relevant diagnostics about the problematic nodes have already been emitted; we
+      // don't need to print anything else.
     } catch {
-      diagnosticsEngine.emitError("Unable to lint \(url.relativePath): \(error)")
-      return
+      diagnosticsEngine.emitError("Unable to lint \(url.relativePath): \(error.localizedDescription).")
     }
   }
 }
