@@ -23,7 +23,8 @@ struct LintFormatOptions: ParsableArguments {
     help: """
       The path to a JSON file containing the configuration of the linter/formatter or a JSON \
       string containing the configuration directly.
-      """)
+      """
+  )
   var configuration: String?
 
   /// A list of comma-separated "start:end" pairs specifying UTF-8 offsets of the ranges to format.
@@ -34,7 +35,8 @@ struct LintFormatOptions: ParsableArguments {
     help: """
       A "start:end" pair specifying UTF-8 offsets of the range to format. Multiple ranges can be
       formatted by specifying several --offsets arguments.
-      """)
+      """
+  )
   var offsets: [Range<Int>] = []
 
   /// The filename for the source code when reading from standard input, to include in diagnostic
@@ -50,23 +52,27 @@ struct LintFormatOptions: ParsableArguments {
   /// If set, we recursively run on all ".swift" files in any provided directories.
   @Flag(
     name: .shortAndLong,
-    help: "Recursively run on '.swift' files in any provided directories.")
+    help: "Recursively run on '.swift' files in any provided directories."
+  )
   var recursive: Bool = false
 
   /// Whether unparsable files, due to syntax errors or unrecognized syntax, should be ignored or
   /// treated as containing an error. When ignored, unparsable files are output verbatim in format
   /// mode and no diagnostics are raised in lint mode. When not ignored, unparsable files raise a
   /// diagnostic in both format and lint mode.
-  @Flag(help: """
-    Ignores unparsable files, disabling all diagnostics and formatting for files that contain \
-    invalid syntax.
-    """)
+  @Flag(
+    help: """
+      Ignores unparsable files, disabling all diagnostics and formatting for files that contain \
+      invalid syntax.
+      """
+  )
   var ignoreUnparsableFiles: Bool = false
 
   /// Whether or not to run the formatter/linter in parallel.
   @Flag(
     name: .shortAndLong,
-    help: "Process files in parallel, simultaneously across multiple cores.")
+    help: "Process files in parallel, simultaneously across multiple cores."
+  )
   var parallel: Bool = false
 
   /// Whether colors should be used in diagnostics printed to standard error.
@@ -79,18 +85,30 @@ struct LintFormatOptions: ParsableArguments {
       Enables or disables color diagnostics when printing to standard error. The default behavior \
       if this flag is omitted is to use colors if standard error is connected to a terminal, and \
       to not use colors otherwise.
-      """)
+      """
+  )
   var colorDiagnostics: Bool?
 
   /// Whether symlinks should be followed.
-  @Flag(help: """
-    Follow symbolic links passed on the command line, or found during directory traversal when \
-    using `-r/--recursive`.
-    """)
+  @Flag(
+    help: """
+      Follow symbolic links passed on the command line, or found during directory traversal when \
+      using `-r/--recursive`.
+      """
+  )
   var followSymlinks: Bool = false
 
+  @Option(
+    name: .customLong("enable-experimental-feature"),
+    help: """
+      The name of an experimental swift-syntax parser feature that should be enabled by \
+      swift-format. Multiple features can be enabled by specifying this flag multiple times.
+      """
+  )
+  var experimentalFeatures: [String] = []
+
   /// The list of paths to Swift source files that should be formatted or linted.
-  @Argument(help: "Zero or more input filenames.")
+  @Argument(help: "Zero or more input filenames. Use `-` for stdin.")
   var paths: [String] = []
 
   @Flag(help: .hidden) var debugDisablePrettyPrint: Bool = false
@@ -129,7 +147,7 @@ extension Range<Int> {
   public init?(argument: String) {
     let pair = argument.components(separatedBy: ":")
     if pair.count == 2, let start = Int(pair[0]), let end = Int(pair[1]), start <= end {
-      self = start ..< end
+      self = start..<end
     } else {
       return nil
     }
@@ -137,7 +155,7 @@ extension Range<Int> {
 }
 
 #if compiler(>=6)
-extension Range<Int> : @retroactive ExpressibleByArgument {}
+extension Range<Int>: @retroactive ExpressibleByArgument {}
 #else
-extension Range<Int> : ExpressibleByArgument {}
+extension Range<Int>: ExpressibleByArgument {}
 #endif
