@@ -9,9 +9,9 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -19,14 +19,14 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3,
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = true
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testArrayCommasAbsentDisabled() {
     let input =
       """
@@ -35,9 +35,9 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -45,14 +45,14 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = false
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testArrayCommasPresentEnabled() {
     let input =
       """
@@ -61,9 +61,9 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3,
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -71,14 +71,14 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3,
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = true
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testArrayCommasPresentDisabled() {
     let input =
       """
@@ -87,9 +87,9 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3,
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -97,52 +97,148 @@ final class CommaTests: PrettyPrintTestCase {
         2,
         3
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = false
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testArraySingleLineCommasPresentEnabled() {
     let input =
       """
       let MyCollection = [1, 2, 3,]
-      
+
       """
-    
+
     // no effect expected
     let expected =
       """
       let MyCollection = [1, 2, 3]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = true
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
   }
-  
+
   func testArraySingleLineCommasPresentDisabled() {
     let input =
       """
       let MyCollection = [1, 2, 3,]
-      
+
       """
-    
+
     // no effect expected
     let expected =
       """
       let MyCollection = [1, 2, 3]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = false
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
   }
-  
+
+  func testArrayWithCommentCommasPresentEnabled() {
+    let input =
+      """
+      let MyCollection = [
+        1,
+        2 // some comment
+      ]
+
+      """
+
+    let expected =
+      """
+      let MyCollection = [
+        1,
+        2,  // some comment
+      ]
+
+      """
+
+    var configuration = Configuration.forTesting
+    configuration.multiElementCollectionTrailingCommas = true
+    assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
+  }
+
+  func testArrayWithCommentCommasPresentDisabled() {
+    let input =
+      """
+      let MyCollection = [
+        1,
+        2 // some comment
+      ]
+
+      """
+
+    let expected =
+      """
+      let MyCollection = [
+        1,
+        2  // some comment
+      ]
+
+      """
+
+    var configuration = Configuration.forTesting
+    configuration.multiElementCollectionTrailingCommas = false
+    assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
+  }
+
+  func testArrayWithTernaryOperatorAndCommentCommasPresentEnabled() {
+    let input =
+      """
+      let MyCollection = [
+        1,
+        true ? 1 : 2 // some comment
+      ]
+
+      """
+
+    let expected =
+      """
+      let MyCollection = [
+        1,
+        true ? 1 : 2,  // some comment
+      ]
+
+      """
+
+    var configuration = Configuration.forTesting
+    configuration.multiElementCollectionTrailingCommas = true
+    assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
+  }
+
+  func testArrayWithTernaryOperatorAndCommentCommasPresentDisabled() {
+    let input =
+      """
+      let MyCollection = [
+        1,
+        true ? 1 : 2 // some comment
+      ]
+
+      """
+
+    let expected =
+      """
+      let MyCollection = [
+        1,
+        true ? 1 : 2  // some comment
+      ]
+
+      """
+
+    var configuration = Configuration.forTesting
+    configuration.multiElementCollectionTrailingCommas = false
+    assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
+  }
+
   func testDictionaryCommasAbsentEnabled() {
     let input =
       """
@@ -151,9 +247,9 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -161,14 +257,14 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3,
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = true
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testDictionaryCommasAbsentDisabled() {
     let input =
       """
@@ -177,9 +273,9 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -187,14 +283,14 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = false
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testDictionaryCommasPresentEnabled() {
     let input =
       """
@@ -203,9 +299,9 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3,
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -213,14 +309,14 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3,
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = true
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testDictionaryCommasPresentDisabled() {
     let input =
       """
@@ -229,9 +325,9 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3,
       ]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
@@ -239,49 +335,49 @@ final class CommaTests: PrettyPrintTestCase {
         "b": 2,
         "c": 3
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = false
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 20, configuration: configuration)
   }
-  
+
   func testDictionarySingleLineCommasPresentDisabled() {
     let input =
       """
       let MyCollection = ["a": 1, "b": 2, "c": 3,]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
         "a": 1, "b": 2, "c": 3,
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = true
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
   }
-  
+
   func testDictionarySingleLineCommasPresentEnabled() {
     let input =
       """
       let MyCollection = ["a": 1, "b": 2, "c": 3,]
-      
+
       """
-    
+
     let expected =
       """
       let MyCollection = [
         "a": 1, "b": 2, "c": 3
       ]
-      
+
       """
-    
+
     var configuration = Configuration.forTesting
     configuration.multiElementCollectionTrailingCommas = false
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 40, configuration: configuration)
