@@ -1,3 +1,4 @@
+import SwiftFormat
 import _SwiftFormatTestSupport
 
 final class CommentTests: PrettyPrintTestCase {
@@ -199,6 +200,152 @@ final class CommentTests: PrettyPrintTestCase {
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 45)
   }
 
+  func testLineCommentsWithCustomLeadingSpaces() {
+    let pairs: [(String, String)] = [
+      (
+        """
+        // Line Comment0
+
+        // Line Comment1
+        // Line Comment2
+        let a = 123
+        let b = "456"  // End of line comment
+        let c = "More content"
+
+        """,
+        """
+        // Line Comment0
+
+        // Line Comment1
+        // Line Comment2
+        let a = 123
+        let b = "456"   // End of line comment
+        let c = "More content"
+
+        """
+      ),
+      (
+        """
+        // Comment 3
+        // Comment 4
+
+        let reallyLongVariableName = 123 // This comment should not wrap
+        // and should not combine with this comment
+
+        func MyFun() {
+          // just a comment
+        }
+        """,
+        """
+        // Comment 3
+        // Comment 4
+
+        let reallyLongVariableName = 123   // This comment should not wrap
+        // and should not combine with this comment
+
+        func MyFun() {
+          // just a comment
+        }
+
+        """
+      ),
+      (
+        """
+        func MyFun() {
+          // Comment 1
+          // Comment 2
+          let a = 123
+
+          let b = 456  // Comment 3
+        }
+
+        func MyFun() {
+          let c = 789 // Comment 4
+          // Comment 5
+        }
+        """,
+        """
+        func MyFun() {
+          // Comment 1
+          // Comment 2
+          let a = 123
+
+          let b = 456   // Comment 3
+        }
+
+        func MyFun() {
+          let c = 789   // Comment 4
+          // Comment 5
+        }
+
+        """
+      ),
+      (
+        """
+        let a = myfun(123 // Cmt 7
+        )
+        let a = myfun(var1: 123 // Cmt 7
+        )
+
+        guard condition else { return // Cmt 6
+        }
+
+        switch myvar {
+        case .one, .two, // three
+             .four:
+          dostuff()
+        default: ()
+        }
+
+        """,
+        """
+        let a = myfun(
+          123   // Cmt 7
+        )
+        let a = myfun(
+          var1: 123   // Cmt 7
+        )
+
+        guard condition else {
+          return   // Cmt 6
+        }
+
+        switch myvar {
+        case .one, .two,   // three
+          .four:
+          dostuff()
+        default: ()
+        }
+
+        """
+      ),
+      (
+        """
+        let a = 123 +  // comment
+          b + c
+
+        let d = 123
+        // Trailing Comment
+        """,
+        """
+        let a =
+          123   // comment
+          + b + c
+
+        let d = 123
+        // Trailing Comment
+
+        """
+      ),
+    ]
+
+    var config = Configuration.forTesting
+    config.spacesBeforeEndOfLineComments = 3
+    for (input, expected) in pairs {
+      assertPrettyPrintEqual(input: input, expected: expected, linelength: 45, configuration: config)
+    }
+  }
+
   func testContainerLineComments() {
     let input =
       """
@@ -274,6 +421,82 @@ final class CommentTests: PrettyPrintTestCase {
     assertPrettyPrintEqual(input: input, expected: expected, linelength: 80)
   }
 
+  func testContainerLineCommentsWithCustomLeadingSpaces() {
+    let input =
+      """
+      // Array comment
+      let a = [456, // small comment
+        789]
+
+      // Dictionary comment
+      let b = ["abc": 456, // small comment
+        "def": 789]
+
+      // Trailing comment
+      let c = [123, 456  // small comment
+      ]
+
+      // Multiline comment
+      let d = [123,
+        // comment line 1
+        // comment line 2
+        456
+      ]
+
+      /* Array comment */
+      let a = [456, /* small comment */
+        789]
+
+      /* Dictionary comment */
+      let b = ["abc": 456, /* small comment */
+        "def": 789]
+      """
+
+    let expected =
+      """
+      // Array comment
+      let a = [
+        456, // small comment
+        789,
+      ]
+
+      // Dictionary comment
+      let b = [
+        "abc": 456, // small comment
+        "def": 789,
+      ]
+
+      // Trailing comment
+      let c = [
+        123, 456, // small comment
+      ]
+
+      // Multiline comment
+      let d = [
+        123,
+        // comment line 1
+        // comment line 2
+        456,
+      ]
+
+      /* Array comment */
+      let a = [
+        456, /* small comment */
+        789,
+      ]
+
+      /* Dictionary comment */
+      let b = [
+        "abc": 456, /* small comment */
+        "def": 789,
+      ]
+
+      """
+    var config = Configuration.forTesting
+    config.spacesBeforeEndOfLineComments = 1
+    assertPrettyPrintEqual(input: input, expected: expected, linelength: 80, configuration: config)
+  }
+
   func testDocumentationBlockComments() {
     let input =
       """
@@ -360,14 +583,14 @@ final class CommentTests: PrettyPrintTestCase {
   func testBlockComments() {
     let input =
       """
-      /* Line Comment1 */
+              /* Line Comment1 */
       /* Line Comment2 */
       let a = 123
       let b = "456"  /* End of line comment */
       let c = "More content"
 
-      /* Comment 3
-         Comment 4 */
+            /* Comment 3
+               Comment 4 */
 
       let reallyLongVariableName = 123  /* This comment should wrap */
 
@@ -380,7 +603,9 @@ final class CommentTests: PrettyPrintTestCase {
       }
 
       let d = 123
-      /* Trailing Comment */
+        /* Trailing Comment */
+        /* Trailing
+           Block Comment */
       """
 
     let expected =
@@ -410,6 +635,8 @@ final class CommentTests: PrettyPrintTestCase {
 
       let d = 123
       /* Trailing Comment */
+      /* Trailing
+         Block Comment */
 
       """
 
@@ -777,7 +1004,30 @@ final class CommentTests: PrettyPrintTestCase {
       linelength: 45,
       whitespaceOnly: true,
       findings: [
-        FindingSpec("1️⃣", message: "move end-of-line comment that exceeds the line length"),
+        FindingSpec("1️⃣", message: "move end-of-line comment that exceeds the line length")
+      ]
+    )
+  }
+
+  // Tests that "end of line" comments are flagged only when they exceed the configured line length.
+  func testDiagnoseMoveEndOfLineCommentAroundBoundary() {
+    assertPrettyPrintEqual(
+      input: """
+        x  // 789
+        x  // 7890
+        x  1️⃣// 78901
+
+        """,
+      expected: """
+        x  // 789
+        x  // 7890
+        x  // 78901
+
+        """,
+      linelength: 10,
+      whitespaceOnly: true,
+      findings: [
+        FindingSpec("1️⃣", message: "move end-of-line comment that exceeds the line length")
       ]
     )
   }
