@@ -66,21 +66,28 @@ final class DoStmtTests: PrettyPrintTestCase {
       }
       """
 
-    assertPrettyPrintEqual(input: input, expected:
-      """
-      do
-      throws(FooError) {
-        foo()
-      }
+    assertPrettyPrintEqual(
+      input: input,
+      expected:
+        """
+        do
+        throws(FooError) {
+          foo()
+        }
 
-      """, linelength: 18)
-    assertPrettyPrintEqual(input: input, expected:
-      """
-      do throws(FooError) {
-        foo()
-      }
+        """,
+      linelength: 18
+    )
+    assertPrettyPrintEqual(
+      input: input,
+      expected:
+        """
+        do throws(FooError) {
+          foo()
+        }
 
-      """, linelength: 25)
+        """,
+      linelength: 25
+    )
   }
 }
-
