@@ -1,9 +1,8 @@
 import SwiftFormat
-import SwiftSyntax
-import SwiftParser
-import XCTest
-
 @_spi(Testing) import SwiftFormat
+import SwiftParser
+import SwiftSyntax
+import XCTest
 @_spi(Testing) import _SwiftFormatTestSupport
 
 class WhitespaceTestCase: DiagnosingTestCase {
@@ -40,9 +39,13 @@ class WhitespaceTestCase: DiagnosingTestCase {
       sourceFileSyntax: sourceFileSyntax,
       configuration: configuration,
       selection: .infinite,
-      findingConsumer: { emittedFindings.append($0) })
+      findingConsumer: { emittedFindings.append($0) }
+    )
     let linter = WhitespaceLinter(
-      user: markedText.textWithoutMarkers, formatted: expected, context: context)
+      user: markedText.textWithoutMarkers,
+      formatted: expected,
+      context: context
+    )
     linter.lint()
 
     assertFindings(
@@ -51,6 +54,7 @@ class WhitespaceTestCase: DiagnosingTestCase {
       emittedFindings: emittedFindings,
       context: context,
       file: file,
-      line: line)
+      line: line
+    )
   }
 }
