@@ -1,10 +1,9 @@
 import SwiftFormat
+@_spi(Rules) @_spi(Testing) import SwiftFormat
 import SwiftOperators
-import SwiftParser
+@_spi(ExperimentalLanguageFeatures) import SwiftParser
 import SwiftSyntax
 import XCTest
-
-@_spi(Rules) @_spi(Testing) import SwiftFormat
 @_spi(Testing) import _SwiftFormatTestSupport
 
 class LintOrFormatRuleTestCase: DiagnosingTestCase {
@@ -17,18 +16,21 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
   ///     where findings are expected to be emitted.
   ///   - findings: A list of `FindingSpec` values that describe the findings that are expected to
   ///     be emitted.
+  ///   - experimentalFeatures: The set of experimental features that should be enabled in the
+  ///     parser.
   ///   - file: The file the test resides in (defaults to the current caller's file).
   ///   - line: The line the test resides in (defaults to the current caller's line).
   final func assertLint<LintRule: SyntaxLintRule>(
     _ type: LintRule.Type,
     _ markedSource: String,
     findings: [FindingSpec] = [],
+    experimentalFeatures: Parser.ExperimentalFeatures = [],
     file: StaticString = #file,
     line: UInt = #line
   ) {
     let markedText = MarkedText(textWithMarkers: markedSource)
     let unmarkedSource = markedText.textWithoutMarkers
-    let tree = Parser.parse(source: unmarkedSource)
+    let tree = Parser.parse(source: unmarkedSource, experimentalFeatures: experimentalFeatures)
     let sourceFileSyntax =
       try! OperatorTable.standardOperators.foldAll(tree).as(SourceFileSyntax.self)!
 
@@ -41,39 +43,33 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
       sourceFileSyntax: sourceFileSyntax,
       configuration: configuration,
       selection: .infinite,
-      findingConsumer: { emittedFindings.append($0) })
-    let linter = type.init(context: context)
-    linter.walk(sourceFileSyntax)
-
-    assertFindings(
-      expected: findings,
-      markerLocations: markedText.markers,
-      emittedFindings: emittedFindings,
-      context: context,
-      file: file,
-      line: line)
+      findingConsumer: { emittedFindings.append($0) }
+    )
 
     var emittedPipelineFindings = [Finding]()
     // Disable default rules, so only select rule runs in pipeline
     configuration.rules = [type.ruleName: true]
     let pipeline = SwiftLinter(
       configuration: configuration,
-      findingConsumer: { emittedPipelineFindings.append($0) })
+      findingConsumer: { emittedPipelineFindings.append($0) }
+    )
     pipeline.debugOptions.insert(.disablePrettyPrint)
     try! pipeline.lint(
       syntax: sourceFileSyntax,
       source: unmarkedSource,
       operatorTable: OperatorTable.standardOperators,
-      assumingFileURL: URL(string: file.description)!)
+      assumingFileURL: URL(fileURLWithPath: file.description)
+    )
 
-    // Check that pipeline produces the same findings as the isolated linter rule
+    // Check that pipeline produces the expected findings
     assertFindings(
       expected: findings,
       markerLocations: markedText.markers,
       emittedFindings: emittedPipelineFindings,
       context: context,
       file: file,
-      line: line)
+      line: line
+    )
   }
 
   /// Asserts that the result of applying a formatter to the provided input code yields the output.
@@ -87,6 +83,8 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
   ///   - findings: A list of `FindingSpec` values that describe the findings that are expected to
   ///     be emitted.
   ///   - configuration: The configuration to use when formatting (or nil to use the default).
+  ///   - experimentalFeatures: The set of experimental features that should be enabled in the
+  ///     parser.
   ///   - file: The file the test resides in (defaults to the current caller's file)
   ///   - line:  The line the test resides in (defaults to the current caller's line)
   final func assertFormatting(
@@ -95,12 +93,13 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
     expected: String,
     findings: [FindingSpec] = [],
     configuration: Configuration? = nil,
+    experimentalFeatures: Parser.ExperimentalFeatures = [],
     file: StaticString = #file,
     line: UInt = #line
   ) {
     let markedInput = MarkedText(textWithMarkers: input)
     let originalSource: String = markedInput.textWithoutMarkers
-    let tree = Parser.parse(source: originalSource)
+    let tree = Parser.parse(source: originalSource, experimentalFeatures: experimentalFeatures)
     let sourceFileSyntax =
       try! OperatorTable.standardOperators.foldAll(tree).as(SourceFileSyntax.self)!
 
@@ -113,7 +112,8 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
       sourceFileSyntax: sourceFileSyntax,
       configuration: configuration,
       selection: .infinite,
-      findingConsumer: { emittedFindings.append($0) })
+      findingConsumer: { emittedFindings.append($0) }
+    )
 
     let formatter = formatType.init(context: context)
     let actual = formatter.visit(sourceFileSyntax)
@@ -125,7 +125,8 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
       emittedFindings: emittedFindings,
       context: context,
       file: file,
-      line: line)
+      line: line
+    )
 
     // Verify that the pretty printer can consume the transformed tree (e.g., it does not contain
     // any unfolded `SequenceExpr`s). Then do a whitespace-insensitive comparison of the two trees
@@ -139,27 +140,41 @@ class LintOrFormatRuleTestCase: DiagnosingTestCase {
       printTokenStream: false,
       whitespaceOnly: false
     ).prettyPrint()
-    let prettyPrintedTree = Parser.parse(source: prettyPrintedSource)
+    let prettyPrintedTree = Parser.parse(source: prettyPrintedSource, experimentalFeatures: experimentalFeatures)
     XCTAssertEqual(
       whitespaceInsensitiveText(of: actual),
       whitespaceInsensitiveText(of: prettyPrintedTree),
       "After pretty-printing and removing fluid whitespace, the files did not match",
-      file: file, line: line)
+      file: file,
+      line: line
+    )
 
     var emittedPipelineFindings = [Finding]()
     // Disable default rules, so only select rule runs in pipeline
     configuration.rules = [formatType.ruleName: true]
     let pipeline = SwiftFormatter(
-      configuration: configuration, findingConsumer: { emittedPipelineFindings.append($0) })
+      configuration: configuration,
+      findingConsumer: { emittedPipelineFindings.append($0) }
+    )
     pipeline.debugOptions.insert(.disablePrettyPrint)
     var pipelineActual = ""
     try! pipeline.format(
-      syntax: sourceFileSyntax, source: originalSource, operatorTable: OperatorTable.standardOperators,
-      assumingFileURL: nil, selection: .infinite, to: &pipelineActual)
+      syntax: sourceFileSyntax,
+      source: originalSource,
+      operatorTable: OperatorTable.standardOperators,
+      assumingFileURL: nil,
+      selection: .infinite,
+      to: &pipelineActual
+    )
     assertStringsEqualWithDiff(pipelineActual, expected)
     assertFindings(
-      expected: findings, markerLocations: markedInput.markers,
-      emittedFindings: emittedPipelineFindings, context: context, file: file, line: line)
+      expected: findings,
+      markerLocations: markedInput.markers,
+      emittedFindings: emittedPipelineFindings,
+      context: context,
+      file: file,
+      line: line
+    )
   }
 }
 
