@@ -140,18 +140,6 @@ extension Dictionary where Key == NSAttributedString.Key, Value == Any {
         try self.init(container, attributeTable: S.attributeKeyTypes())
     }
     
-    // These includingOnly SPI initializers were provided originally when conversion boxed attributes outside of the given scope as an AnyObject
-    // After rdar://80201634, these SPI initializers have the same behavior as the API initializers
-    @_spi(AttributedString)
-    public init<S: AttributeScope>(_ container: AttributeContainer, includingOnly scope: KeyPath<AttributeScopes, S.Type>) throws {
-        try self.init(container, including: S.self)
-    }
-    
-    @_spi(AttributedString)
-    public init<S: AttributeScope>(_ container: AttributeContainer, includingOnly scope: S.Type) throws {
-        try self.init(container, including: S.self)
-    }
-    
     fileprivate init(_ container: AttributeContainer, attributeTable: [String : any AttributedStringKey.Type], options: _AttributeConversionOptions = []) throws {
         self.init()
         for key in container.storage.keys {
@@ -186,16 +174,6 @@ extension NSAttributedString {
         try self.init(attrStr, attributeTable: scope.attributeKeyTypes())
     }
     
-    @_spi(AttributedString)
-    public convenience init<S: AttributeScope>(_ attrStr: AttributedString, includingOnly scope: KeyPath<AttributeScopes, S.Type>) throws {
-        try self.init(attrStr, including: S.self)
-    }
-    
-    @_spi(AttributedString)
-    public convenience init<S: AttributeScope>(_ attrStr: AttributedString, includingOnly scope: S.Type) throws {
-        try self.init(attrStr, including: scope)
-    }
-    
     internal convenience init(
         _ attrStr: AttributedString,
         attributeTable: [String : any AttributedStringKey.Type],
@@ -460,7 +438,8 @@ extension Range where Bound == AttributedString.Index {
         guard range.location != NSNotFound else { return nil }
         guard range.location >= 0, range.length >= 0 else { return nil }
         let endOffset = range.location + range.length
-        let bstr = string.__guts.string
+        let bstrBounds = Range<BigString.Index>(uncheckedBounds: (string.startIndex._value, string.endIndex._value))
+        let bstr = string.__guts.string[bstrBounds]
         guard endOffset <= bstr.utf16.count else { return nil }
 
         let start = bstr.utf16.index(bstr.startIndex, offsetBy: range.location)
