@@ -12,10 +12,10 @@
 
 #if canImport(Darwin)
 import Darwin
-#elseif os(Android)
-import Bionic
+#elseif canImport(Bionic)
+@preconcurrency import Bionic
 #elseif canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 #endif
 
 //===----------------------------------------------------------------------===//
