@@ -18,17 +18,17 @@ internal import _FoundationCShims
 
 #if canImport(Darwin)
 import Darwin
-#elseif os(Android)
-import Android
+#elseif canImport(Android)
+@preconcurrency import Android
 #elseif canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 #elseif canImport(Musl)
-import Musl
+@preconcurrency import Musl
 #elseif os(Windows)
 import CRT
 import WinSDK
 #elseif os(WASI)
-import WASILibc
+@preconcurrency import WASILibc
 #endif
 
 func _fgetxattr(_ fd: Int32, _ name: UnsafePointer<CChar>!, _ value: UnsafeMutableRawPointer!, _ size: Int, _ position: UInt32, _ options: Int32) -> Int {
@@ -266,8 +266,8 @@ internal func readBytesFromFile(path inPath: PathOrURL, reportProgress: Bool, ma
         let szMapSize: UInt64 = min(UInt64(maxLength ?? Int.max), szFileSize)
         let pData: UnsafeMutableRawPointer =
             MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, SIZE_T(szMapSize))
-        return ReadBytesResult(bytes: pData, length: Int(szMapSize), deallocator: .custom({ hMapping, _ in
-            guard UnmapViewOfFile(hMapping) else {
+        return ReadBytesResult(bytes: pData, length: Int(szMapSize), deallocator: .custom({ pData, _ in
+            guard UnmapViewOfFile(pData) else {
                 fatalError("UnmapViewOfFile")
             }
             guard CloseHandle(hMapping) else {
@@ -327,7 +327,7 @@ internal func readBytesFromFile(path inPath: PathOrURL, reportProgress: Bool, ma
     }
     
     let fileSize = min(Int(clamping: filestat.st_size), maxLength ?? Int.max)
-    let fileType = filestat.st_mode & S_IFMT
+    let fileType = mode_t(filestat.st_mode) & S_IFMT
 #if !NO_FILESYSTEM
     let shouldMap = shouldMapFileDescriptor(fd, path: inPath, options: options)
 #else
@@ -377,7 +377,7 @@ internal func readBytesFromFile(path inPath: PathOrURL, reportProgress: Bool, ma
         localProgress?.completedUnitCount = 1
     } else if shouldMap {
 #if !NO_FILESYSTEM
-#if os(Android)
+#if canImport(Android)
         let bytes = mmap(nil, Int(fileSize), PROT_READ, MAP_PRIVATE, fd, 0)
         if bytes == UnsafeMutableRawPointer(bitPattern: -1) {
             throw CocoaError.errorWithFilePath(inPath, errno: errno, reading: true)
