@@ -17,21 +17,21 @@ internal import DarwinPrivate.sys.content_protection
 
 #if canImport(Darwin)
 import Darwin
-#elseif os(Android)
-import Android
+#elseif canImport(Android)
+@preconcurrency import Android
 import posix_filesystem
 #elseif canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 internal import _FoundationCShims
 #elseif canImport(Musl)
-import Musl
+@preconcurrency import Musl
 internal import _FoundationCShims
 #elseif os(Windows)
 import CRT
 import WinSDK
 #elseif os(WASI)
 internal import _FoundationCShims
-import WASILibc
+@preconcurrency import WASILibc
 #endif
 
 extension Date {
@@ -162,7 +162,8 @@ extension stat {
     }
     
     fileprivate var fileAttributes: [FileAttributeKey : Any] {
-        let fileType = st_mode.fileType
+        // On 32 bit Android, st_mode is UInt32.
+        let fileType = mode_t(st_mode).fileType
         var result: [FileAttributeKey : Any] = [
             .size : _writeFileAttributePrimitive(st_size, as: UInt.self),
             .modificationDate : modificationDate,
@@ -387,7 +388,7 @@ extension _FileManagerImpl {
             guard stat(rep, &fileInfo) == 0 else {
                 return (false, false)
             }
-            let isDir = (fileInfo.st_mode & S_IFMT) == S_IFDIR
+            let isDir = (mode_t(fileInfo.st_mode) & S_IFMT) == S_IFDIR
             return (true, isDir)
         }
 #endif
@@ -466,7 +467,7 @@ extension _FileManagerImpl {
             return false
         }
         
-        if ((dirInfo.st_mode & S_ISVTX) != 0) && fileManager.fileExists(atPath: path) {
+        if ((mode_t(dirInfo.st_mode) & S_ISVTX) != 0) && fileManager.fileExists(atPath: path) {
             // its sticky so verify that we own the file
             // otherwise we answer YES on the principle that if
             // we create files we can delete them
