@@ -33,6 +33,11 @@ struct _Win32DirectoryContentsSequence: Sequence {
         init(path: String, appendSlashForDirectory: Bool, prefix: [String]) {
             self.slash = appendSlashForDirectory
 
+            guard !path.isEmpty else {
+                self.error = CocoaError.errorWithFilePath(.fileReadInvalidFileName, path)
+                return
+            }
+
             do {
                 hFind = try "\(path)\\*".withNTPathRepresentation {
                     // We use `FindFirstFileExW` to avoid the lookup of the short name of the file. We never consult the field and this can speed up the file enumeration.
@@ -109,19 +114,24 @@ struct _Win32DirectoryContentsSequence: Sequence {
 
 #if canImport(Darwin)
 import Darwin
-#elseif os(Android)
-import Android
+#elseif canImport(Android)
+@preconcurrency import Android
 import posix_filesystem.dirent
 #elseif canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 internal import _FoundationCShims
 #elseif canImport(Musl)
-import Musl
+@preconcurrency import Musl
+#elseif os(WASI)
+@preconcurrency import WASILibc
 internal import _FoundationCShims
 #endif
 
 // MARK: Directory Iteration
 
+// No FTS support in wasi-libc for now (https://github.com/WebAssembly/wasi-libc/issues/520)
+#if !os(WASI)
+
 struct _FTSSequence: Sequence {
     enum Element {
         struct SwiftFTSENT {
@@ -318,10 +328,12 @@ extension Sequence<_FTSSequence.Element> {
     }
 }
 
+#endif // !os(WASI)
+
 struct _POSIXDirectoryContentsSequence: Sequence {
     #if canImport(Darwin)
     typealias DirectoryEntryPtr = UnsafeMutablePointer<DIR>
-    #elseif os(Android) || canImport(Glibc) || canImport(Musl)
+    #elseif canImport(Android) || canImport(Glibc) || canImport(Musl) || os(WASI)
     typealias DirectoryEntryPtr = OpaquePointer
     #endif
     
@@ -346,10 +358,18 @@ struct _POSIXDirectoryContentsSequence: Sequence {
                     continue
                 }
                 // Use name
-                let fileName = withUnsafeBytes(of: &dent.pointee.d_name) { buf in
+                let fileName: String
+                #if os(WASI)
+                // Use shim on WASI because wasi-libc defines `d_name` as
+                // "flexible array member" which is not supported by
+                // ClangImporter yet.
+                fileName = String(cString: _platform_shims_dirent_d_name(dent))
+                #else
+                fileName = withUnsafeBytes(of: &dent.pointee.d_name) { buf in
                     let ptr = buf.baseAddress!.assumingMemoryBound(to: CChar.self)
                     return String(cString: ptr)
                 }
+                #endif
 
                 if fileName == "." || fileName == ".." || fileName == "._" {
                     continue
@@ -367,7 +387,7 @@ struct _POSIXDirectoryContentsSequence: Sequence {
                         let statDir = directoryPath + "/" + fileName
                         if stat(statDir, &statBuf) == 0 {
                             // #define S_ISDIR(m)      (((m) & S_IFMT) == S_IFDIR)
-                            if (statBuf.st_mode & S_IFMT) == S_IFDIR {
+                            if (mode_t(statBuf.st_mode) & S_IFMT) == S_IFDIR {
                                 isDirectory = true
                             }
                         }
