@@ -12,17 +12,17 @@
 
 #if canImport(Darwin)
 import Darwin
-#elseif os(Android)
-import Android
+#elseif canImport(Android)
+@preconcurrency import Android
 #elseif canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 #elseif canImport(Musl)
-import Musl
+@preconcurrency import Musl
 #elseif os(Windows)
 import CRT
 import WinSDK
 #elseif os(WASI)
-import WASILibc
+@preconcurrency import WASILibc
 #endif
 
 #if FOUNDATION_FRAMEWORK
@@ -37,14 +37,6 @@ internal import QuarantinePrivate
 internal import _FoundationCShims
 
 extension CocoaError {
-    fileprivate static func fileOperationError(_ code: CocoaError.Code,  _ sourcePath: String, _ destinationPath: String? = nil, variant: String? = nil) -> CocoaError {
-        var info: [String : AnyHashable] = [NSSourceFilePathErrorKey:sourcePath]
-        if let destinationPath {
-            info[NSDestinationFilePathErrorKey] = destinationPath
-        }
-        return CocoaError.errorWithFilePath(code, sourcePath, variant: variant, userInfo: info)
-    }
-
 #if os(Windows)
     private static func fileOperationError(_ dwError: DWORD, _ suspectedErroneousPath: String, sourcePath: String? = nil, destinationPath: String? = nil, variant: String? = nil) -> CocoaError {
         var path = suspectedErroneousPath
@@ -57,23 +49,16 @@ extension CocoaError {
             }
             path = lastLength > MAX_PATH || fullLength > MAX_PATH ? destinationPath : sourcePath
         }
-
-        var info: [String : AnyHashable] = [:]
-        if let sourcePath {
-            info[NSSourceFilePathErrorKey] = sourcePath
-        }
-        if let destinationPath {
-            info[NSDestinationFilePathErrorKey] = destinationPath
-        }
-        return CocoaError.errorWithFilePath(path, win32: dwError, reading: false, variant: variant, userInfo: info)
+        
+        return CocoaError.errorWithFilePath(path, win32: dwError, reading: false, variant: variant, source: sourcePath, destination: destinationPath)
     }
 
     fileprivate static func removeFileError(_ dwError: DWORD, _ path: String) -> CocoaError {
-        var err = CocoaError.fileOperationError(dwError, path, variant: "Remove")
         if dwError == ERROR_DIR_NOT_EMPTY {
-            err = CocoaError(.fileWriteNoPermission, userInfo: err.userInfo)
+            CocoaError.fileOperationError(ERROR_ACCESS_DENIED, path, variant: "Remove")
+        } else {
+            CocoaError.fileOperationError(dwError, path, variant: "Remove")
         }
-        return err
     }
 
     fileprivate static func moveFileError(_ error: DWORD, _ src: URL, _ dst: URL) -> CocoaError {
@@ -107,22 +92,15 @@ extension CocoaError {
             }
         }
         
-        var userInfo: [String : AnyHashable] = [:]
-        if let sourcePath {
-            userInfo[NSSourceFilePathErrorKey] = sourcePath
-        }
-        if let destinationPath {
-            userInfo[NSDestinationFilePathErrorKey] = destinationPath
-        }
-        return CocoaError.errorWithFilePath(erroneousPath, errno: errNum, reading: false, variant: variant, additionalUserInfo: userInfo)
+        return CocoaError.errorWithFilePath(erroneousPath, errno: errNum, reading: false, variant: variant, source: sourcePath, destination: destinationPath)
     }
     
     fileprivate static func removeFileError(_ errNum: Int32, _ path: String) -> CocoaError {
-        var err = CocoaError.fileOperationError(errNum, path, variant: "Remove")
         if errNum == ENOTEMPTY {
-            err = CocoaError(.fileWriteNoPermission, userInfo: err.userInfo)
+            CocoaError.fileOperationError(EPERM, path, variant: "Remove")
+        } else {
+            CocoaError.fileOperationError(errNum, path, variant: "Remove")
         }
-        return err
     }
     
     fileprivate static func moveFileError(_ errNum: Int32, _ src: URL, _ dst: URL) -> CocoaError {
@@ -514,7 +492,14 @@ enum _FileOperations {
             // We failed for a reason other than the directory not being empty, so throw
             throw CocoaError.removeFileError(errno, resolve(path: pathStr))
         }
-        
+
+        #if os(WASI)
+
+        // wasi-libc does not support FTS, so we don't support removing non-empty directories on WASI for now.
+        throw CocoaError.errorWithFilePath(.featureUnsupported, pathStr)
+
+        #else
+
         let seq = _FTSSequence(path, FTS_PHYSICAL | FTS_XDEV | FTS_NOCHDIR | FTS_NOSTAT)
         let iterator = seq.makeIterator()
         var isFirst = true
@@ -563,6 +548,7 @@ enum _FileOperations {
                 }
             }
         }
+        #endif
         
     }
     #endif
@@ -803,7 +789,7 @@ enum _FileOperations {
         try src.withNTPathRepresentation { pwszSource in
             var faAttributes: WIN32_FILE_ATTRIBUTE_DATA = .init()
             guard GetFileAttributesExW(pwszSource, GetFileExInfoStandard, &faAttributes) else {
-                throw CocoaError.fileOperationError(.fileReadNoSuchFile, src, dst, variant: bCopyFile ? "Copy" : "Link")
+                throw CocoaError.errorWithFilePath(.fileReadNoSuchFile, src, variant: bCopyFile ? "Copy" : "Link", source: src, destination: dst)
             }
 
             guard delegate.shouldPerformOnItemAtPath(src, to: dst) else { return }
@@ -870,7 +856,7 @@ enum _FileOperations {
 
         #if !os(WASI) // WASI doesn't have fchmod for now
         // Set the file permissions using fchmod() instead of when open()ing to avoid umask() issues
-        let permissions = fileInfo.st_mode & ~S_IFMT
+        let permissions = mode_t(fileInfo.st_mode) & ~S_IFMT
         guard fchmod(dstfd, permissions) == 0 else {
             try delegate.throwIfNecessary(errno, String(cString: srcPtr), String(cString: dstPtr))
             return
@@ -883,7 +869,16 @@ enum _FileOperations {
         }
         
         let total: Int = Int(fileInfo.st_size)
-        let chunkSize: Int = Int(fileInfo.st_blksize)
+        // Respect the optimal block size for the file system if available
+        // Some platforms including WASI don't provide this information, so we
+        // fall back to the default chunk size 4KB, which is a common page size.
+        let defaultChunkSize = 1024 * 4 // 4KB
+        let chunkSize: Int
+        if fileInfo.st_blksize > 0 {
+            chunkSize = Int(fileInfo.st_blksize)
+        } else {
+            chunkSize = defaultChunkSize
+        }
         var current: off_t = 0
         
         #if os(WASI)
@@ -913,7 +908,143 @@ enum _FileOperations {
         #endif
     }
     #endif
+    
+    #if !canImport(Darwin)
+    private static func _copyDirectoryMetadata(srcFD: CInt, srcPath: @autoclosure () -> String, dstFD: CInt, dstPath: @autoclosure () -> String, delegate: some LinkOrCopyDelegate) throws {
+        #if !os(WASI) && !os(Android)
+        // Copy extended attributes
+        var size = flistxattr(srcFD, nil, 0)
+        if size > 0 {
+            try withUnsafeTemporaryAllocation(of: CChar.self, capacity: size) { keyList in
+                size = flistxattr(srcFD, keyList.baseAddress!, size)
+                if size > 0 {
+                    var current = keyList.baseAddress!
+                    let end = keyList.baseAddress!.advanced(by: keyList.count)
+                    while current < end {
+                        var valueSize = fgetxattr(srcFD, current, nil, 0)
+                        if valueSize >= 0 {
+                            try withUnsafeTemporaryAllocation(of: UInt8.self, capacity: valueSize) { valueBuffer in
+                                valueSize = fgetxattr(srcFD, current, valueBuffer.baseAddress!, valueSize)
+                                if valueSize >= 0 {
+                                    if fsetxattr(dstFD, current, valueBuffer.baseAddress!, valueSize, 0) != 0 {
+                                        try delegate.throwIfNecessary(errno, srcPath(), dstPath())
+                                    }
+                                }
+                            }
+                        }
+                        current = current.advanced(by: strlen(current) + 1) /* pass null byte */
+                    }
+                }
+            }
+        }
+        #endif
+        var statInfo = stat()
+        if fstat(srcFD, &statInfo) == 0 {
+            #if !os(WASI) // WASI doesn't have fchown for now
+            // Copy owner/group
+            if fchown(dstFD, statInfo.st_uid, statInfo.st_gid) != 0 {
+                try delegate.throwIfNecessary(errno, srcPath(), dstPath())
+            }
+            #endif
+            
+            // Copy modification date
+            let value = statInfo.st_mtim
+            var tv = (value, value)
+            try withUnsafePointer(to: &tv) {
+                try $0.withMemoryRebound(to: timespec.self, capacity: 2) {
+                    if futimens(dstFD, $0) != 0 {
+                        try delegate.throwIfNecessary(errno, srcPath(), dstPath())
+                    }
+                }
+            }
+            
+            #if !os(WASI) // WASI doesn't have fchmod for now
+            // Copy permissions
+            if fchmod(dstFD, mode_t(statInfo.st_mode)) != 0 {
+                try delegate.throwIfNecessary(errno, srcPath(), dstPath())
+            }
+            #endif
+        } else {
+            try delegate.throwIfNecessary(errno, srcPath(), dstPath())
+        }
+    }
+    #endif
+    
+    private static func _openDirectoryFD(_ ptr: UnsafePointer<CChar>, srcPath: @autoclosure () -> String, dstPath: @autoclosure () -> String, delegate: some LinkOrCopyDelegate) throws -> CInt? {
+        let fd = open(ptr, O_RDONLY | O_NOFOLLOW | O_DIRECTORY)
+        guard fd >= 0 else {
+            try delegate.throwIfNecessary(errno, srcPath(), dstPath())
+            return nil
+        }
+        return fd
+    }
+    
+    // Safely copies metadata from one directory to another ensuring that both paths are directories and cannot be swapped for files before/while copying metadata
+    private static func _safeCopyDirectoryMetadata(src: UnsafePointer<CChar>, dst: UnsafePointer<CChar>, delegate: some LinkOrCopyDelegate, extraFlags: Int32 = 0) throws {
+        guard let srcFD = try _openDirectoryFD(src, srcPath: String(cString: src), dstPath: String(cString: dst), delegate: delegate) else {
+            return
+        }
+        defer { close(srcFD) }
+        
+        guard let dstFD = try _openDirectoryFD(dst, srcPath: String(cString: src), dstPath: String(cString: dst), delegate: delegate) else {
+            return
+        }
+        defer { close(dstFD) }
+        
+        #if canImport(Darwin)
+        if fcopyfile(srcFD, dstFD, nil, copyfile_flags_t(COPYFILE_METADATA | COPYFILE_NOFOLLOW | extraFlags)) != 0 {
+            try delegate.throwIfNecessary(errno, String(cString: src), String(cString: dst))
+        }
+        #else
+        try _copyDirectoryMetadata(srcFD: srcFD, srcPath: String(cString: src), dstFD: dstFD, dstPath: String(cString: dst), delegate: delegate)
+        #endif
+    }
 
+    #if os(WASI)
+    private static func _linkOrCopyFile(_ srcPtr: UnsafePointer<CChar>, _ dstPtr: UnsafePointer<CChar>, with fileManager: FileManager, delegate: some LinkOrCopyDelegate) throws {
+        let src = String(cString: srcPtr)
+        let dst = String(cString: dstPtr)
+        guard delegate.shouldPerformOnItemAtPath(src, to: dst) else { return }
+
+        var stat = stat()
+        guard lstat(srcPtr, &stat) == 0 else {
+            try delegate.throwIfNecessary(errno, src, dst)
+            return
+        }
+        let copyFile = delegate.copyData
+        guard !stat.isDirectory else {
+            // wasi-libc does not support FTS for now, so we don't support copying/linking
+            // directories on WASI for now.
+            let error = CocoaError.errorWithFilePath(.featureUnsupported, src, variant: copyFile ? "Copy" : "Link", source: src, destination: dst)
+            try delegate.throwIfNecessary(error, src, dst)
+            return
+        }
+
+        // For now, we support only copying regular files and symlinks.
+        // After we get FTS support (https://github.com/WebAssembly/wasi-libc/pull/522),
+        // we can remove this method and use the below FTS-based implementation.
+
+        if stat.isSymbolicLink {
+            try withUnsafeTemporaryAllocation(of: CChar.self, capacity: FileManager.MAX_PATH_SIZE) { tempBuff in
+                tempBuff.initialize(repeating: 0)
+                defer { tempBuff.deinitialize() }
+                let len = readlink(srcPtr, tempBuff.baseAddress!, FileManager.MAX_PATH_SIZE - 1)
+                if len >= 0, symlink(tempBuff.baseAddress!, dstPtr) != -1 {
+                    return
+                }
+                try delegate.throwIfNecessary(errno, src, dst)
+            }
+        } else {
+            if copyFile {
+                try _copyRegularFile(srcPtr, dstPtr, delegate: delegate)
+            } else {
+                if link(srcPtr, dstPtr) != 0 {
+                    try delegate.throwIfNecessary(errno, src, dst)
+                }
+            }
+        }
+    }
+    #else
     private static func _linkOrCopyFile(_ srcPtr: UnsafePointer<CChar>, _ dstPtr: UnsafePointer<CChar>, with fileManager: FileManager, delegate: some LinkOrCopyDelegate) throws {
         try withUnsafeTemporaryAllocation(of: CChar.self, capacity: FileManager.MAX_PATH_SIZE) { buffer in
             let dstLen = Platform.copyCString(dst: buffer.baseAddress!, src: dstPtr, size: FileManager.MAX_PATH_SIZE)
@@ -960,18 +1091,7 @@ enum _FileOperations {
                         
                     case FTS_DP:
                         // Directory being visited in post-order - copy the permissions over.
-                        #if canImport(Darwin)
-                        if copyfile(fts_path, buffer.baseAddress!, nil, copyfile_flags_t(COPYFILE_METADATA | COPYFILE_NOFOLLOW | extraFlags)) != 0 {
-                            try delegate.throwIfNecessary(errno, String(cString: fts_path), String(cString: buffer.baseAddress!))
-                        }
-                        #else
-                        do {
-                            let attributes = try fileManager.attributesOfItem(atPath: String(cString: fts_path))
-                            try fileManager.setAttributes(attributes, ofItemAtPath: String(cString: buffer.baseAddress!))
-                        } catch {
-                            try delegate.throwIfNecessary(error, String(cString: fts_path), String(cString: buffer.baseAddress!))
-                        }
-                        #endif
+                        try Self._safeCopyDirectoryMetadata(src: fts_path, dst: buffer.baseAddress!, delegate: delegate, extraFlags: extraFlags)
                         
                     case FTS_SL: fallthrough    // Symlink.
                     case FTS_SLNONE:            // Symlink with no target.
@@ -1026,6 +1146,7 @@ enum _FileOperations {
             }
         }
     }
+    #endif
     
     private static func linkOrCopyFile(_ src: String, dst: String, with fileManager: FileManager, delegate: some LinkOrCopyDelegate) throws {
         try src.withFileSystemRepresentation { srcPtr in
