@@ -13,7 +13,7 @@
 #if canImport(Darwin)
 import Darwin
 #elseif canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 #endif
 
 internal import _FoundationCShims
@@ -672,8 +672,21 @@ extension JSONDecoderImpl: Decoder {
             }
 
         case .base64:
-            let string = try self.unwrapString(from: mapValue, for: codingPathNode, additionalKey)
-            guard let data = Data(base64Encoded: string) else {
+            guard case .string(let region, let isSimple) = mapValue else {
+                throw self.createTypeMismatchError(type: String.self, for: codingPathNode.path(byAppending: additionalKey), value: mapValue)
+            }
+            var data: Data?
+            if isSimple {
+                data = withBuffer(for: region) { buffer, _ in
+                    Data(decodingBase64: buffer)
+                }
+            }
+            if data == nil {
+                // For compatibility, try decoding as a string and then base64 decoding it.
+                let string = try self.unwrapString(from: mapValue, for: codingPathNode, additionalKey)
+                data = Data(base64Encoded: string)
+            }
+            guard let data else {
                 throw DecodingError.dataCorrupted(DecodingError.Context(codingPath: codingPathNode.path(byAppending: additionalKey), debugDescription: "Encountered Data is not valid Base64."))
             }
 
@@ -773,8 +786,8 @@ extension JSONDecoderImpl: Decoder {
 
         var iter = jsonMap.makeObjectIterator(from: region.startOffset)
         while let (keyValue, value) = iter.next() {
-            // Failing to unwrap a string here is impossible, as scanning already guarantees that dictionary keys are strings.
-            let key = try! self.unwrapString(from: keyValue, for: dictCodingPathNode, _CodingKey?.none)
+            // We know these values are keys, but UTF-8 decoding could still fail.
+            let key = try self.unwrapString(from: keyValue, for: dictCodingPathNode, _CodingKey?.none)
             let value = try self.unwrap(value, as: dictType.elementType, for: dictCodingPathNode, _CodingKey(stringValue: key)!)
             result[key]._setIfNil(to: value)
         }
@@ -1189,14 +1202,14 @@ extension JSONDecoderImpl {
             switch keyDecodingStrategy {
             case .useDefaultKeys:
                 while let (keyValue, value) = iter.next() {
-                    // Failing to unwrap a string here is impossible, as scanning already guarantees that dictionary keys are strings.
-                    let key = try! impl.unwrapString(from: keyValue, for: codingPathNode, _CodingKey?.none)
+                    // We know these values are keys, but UTF-8 decoding could still fail.
+                    let key = try impl.unwrapString(from: keyValue, for: codingPathNode, _CodingKey?.none)
                     result[key]._setIfNil(to: value)
                 }
             case .convertFromSnakeCase:
                 while let (keyValue, value) = iter.next() {
-                    // Failing to unwrap a string here is impossible, as scanning already guarantees that dictionary keys are strings.
-                    let key = try! impl.unwrapString(from: keyValue, for: codingPathNode, _CodingKey?.none)
+                    // We know these values are keys, but UTF-8 decoding could still fail.
+                    let key = try impl.unwrapString(from: keyValue, for: codingPathNode, _CodingKey?.none)
 
                     // Convert the snake case keys in the container to camel case.
                     // If we hit a duplicate key after conversion, then we'll use the first one we saw.
@@ -1206,8 +1219,8 @@ extension JSONDecoderImpl {
             case .custom(let converter):
                 let codingPathForCustomConverter = codingPathNode.path
                 while let (keyValue, value) = iter.next() {
-                    // Failing to unwrap a string here is impossible, as scanning already guarantees that dictionary keys are strings.
-                    let key = try! impl.unwrapString(from: keyValue, for: codingPathNode, _CodingKey?.none)
+                    // We know these values are keys, but UTF-8 decoding could still fail. 
+                    let key = try impl.unwrapString(from: keyValue, for: codingPathNode, _CodingKey?.none)
 
                     var pathForKey = codingPathForCustomConverter
                     pathForKey.append(_CodingKey(stringValue: key)!)
@@ -1253,35 +1266,80 @@ extension JSONDecoderImpl {
             return bool
         }
 
+        func decodeIfPresent(_ type: Bool.Type, forKey key: K) throws -> Bool? {
+            guard let value = getValueIfPresent(forKey: key) else {
+                return nil
+            }
+            switch value {
+            case .null: return nil
+            case .bool(let result): return result
+            default: throw createTypeMismatchError(type: type, forKey: key, value: value)
+            }
+        }
+
         func decode(_ type: String.Type, forKey key: K) throws -> String {
             let value = try getValue(forKey: key)
             return try impl.unwrapString(from: value, for: self.codingPathNode, key)
         }
 
+        func decodeIfPresent(_ type: String.Type, forKey key: K) throws -> String? {
+            guard let value = getValueIfPresent(forKey: key) else {
+                return nil
+            }
+            switch value {
+            case .null: return nil
+            default: return try impl.unwrapString(from: value, for: self.codingPathNode, key)
+            }
+        }
+
         func decode(_: Double.Type, forKey key: K) throws -> Double {
             try decodeFloatingPoint(key: key)
         }
 
+        func decodeIfPresent(_: Double.Type, forKey key: K) throws -> Double? {
+            try decodeFloatingPointIfPresent(key: key)
+        }
+
         func decode(_: Float.Type, forKey key: K) throws -> Float {
             try decodeFloatingPoint(key: key)
         }
 
+        func decodeIfPresent(_: Float.Type, forKey key: K) throws -> Float? {
+            try decodeFloatingPointIfPresent(key: key)
+        }
+
         func decode(_: Int.Type, forKey key: K) throws -> Int {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: Int.Type, forKey key: K) throws -> Int? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: Int8.Type, forKey key: K) throws -> Int8 {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: Int8.Type, forKey key: K) throws -> Int8? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: Int16.Type, forKey key: K) throws -> Int16 {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: Int16.Type, forKey key: K) throws -> Int16? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: Int32.Type, forKey key: K) throws -> Int32 {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: Int32.Type, forKey key: K) throws -> Int32? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: Int64.Type, forKey key: K) throws -> Int64 {
             try decodeFixedWidthInteger(key: key)
         }
@@ -1291,22 +1349,42 @@ extension JSONDecoderImpl {
           try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: Int64.Type, forKey key: K) throws -> Int64? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: UInt.Type, forKey key: K) throws -> UInt {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: UInt.Type, forKey key: K) throws -> UInt? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: UInt8.Type, forKey key: K) throws -> UInt8 {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: UInt8.Type, forKey key: K) throws -> UInt8? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: UInt16.Type, forKey key: K) throws -> UInt16 {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: UInt16.Type, forKey key: K) throws -> UInt16? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: UInt32.Type, forKey key: K) throws -> UInt32 {
             try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: UInt32.Type, forKey key: K) throws -> UInt32? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode(_: UInt64.Type, forKey key: K) throws -> UInt64 {
             try decodeFixedWidthInteger(key: key)
         }
@@ -1316,10 +1394,24 @@ extension JSONDecoderImpl {
           try decodeFixedWidthInteger(key: key)
         }
 
+        func decodeIfPresent(_: UInt64.Type, forKey key: K) throws -> UInt64? {
+            try decodeFixedWidthIntegerIfPresent(key: key)
+        }
+
         func decode<T: Decodable>(_ type: T.Type, forKey key: K) throws -> T {
             try self.impl.unwrap(try getValue(forKey: key), as: type, for: codingPathNode, key)
         }
 
+        func decodeIfPresent<T: Decodable>(_ type: T.Type, forKey key: K) throws -> T? {
+            guard let value = getValueIfPresent(forKey: key) else {
+                return nil
+            }
+            switch value {
+            case .null: return nil
+            default: return try self.impl.unwrap(value, as: type, for: codingPathNode, key)
+            }
+        }
+
         func nestedContainer<NestedKey: CodingKey>(keyedBy type: NestedKey.Type, forKey key: K) throws -> KeyedDecodingContainer<NestedKey> {
             let value = try getValue(forKey: key)
             return try impl.with(value: value, path: codingPathNode.appending(key)) {
@@ -1365,6 +1457,10 @@ extension JSONDecoderImpl {
             return value
         }
 
+        @inline(__always) private func getValueIfPresent(forKey key: some CodingKey) -> JSONMap.Value? {
+            dictionary[key.stringValue]
+        }
+
         private func createTypeMismatchError(type: Any.Type, forKey key: K, value: JSONMap.Value) -> DecodingError {
             return DecodingError.typeMismatch(type, .init(
                 codingPath: self.codingPathNode.path(byAppending: key), debugDescription: "Expected to decode \(type) but found \(value.debugDataTypeDescription) instead."
@@ -1380,6 +1476,26 @@ extension JSONDecoderImpl {
             let value = try getValue(forKey: key)
             return try self.impl.unwrapFloatingPoint(from: value, as: T.self, for: codingPathNode, key)
         }
+
+        @inline(__always) private func decodeFixedWidthIntegerIfPresent<T: FixedWidthInteger>(key: Self.Key) throws -> T? {
+            guard let value = getValueIfPresent(forKey: key) else {
+                return nil
+            }
+            switch value {
+            case .null: return nil
+            default: return try self.impl.unwrapFixedWidthInteger(from: value, as: T.self, for: codingPathNode, key)
+            }
+        }
+
+        @inline(__always) private func decodeFloatingPointIfPresent<T: PrevalidatedJSONNumberBufferConvertible & BinaryFloatingPoint>(key: K) throws -> T? {
+            guard let value = getValueIfPresent(forKey: key) else {
+                return nil
+            }
+            switch value {
+            case .null: return nil
+            default: return try self.impl.unwrapFloatingPoint(from: value, as: T.self, for: codingPathNode, key)
+            }
+        }
     }
 }
 
@@ -1443,6 +1559,17 @@ extension JSONDecoderImpl {
             return bool
         }
 
+        mutating func decodeIfPresent(_ type: Bool.Type) throws -> Bool? {
+            let value = self.peekNextValueIfPresent(ofType: Bool.self)
+            let result: Bool? = switch value {
+            case nil, .null: nil
+            case .bool(let bool): bool
+            default: throw impl.createTypeMismatchError(type: type, for: self.currentCodingPath, value: value!)
+            }
+            advanceToNextValue()
+            return result
+        }
+
         mutating func decode(_ type: String.Type) throws -> String {
             let value = try self.peekNextValue(ofType: String.self)
             let string = try impl.unwrapString(from: value, for: codingPathNode, currentIndexKey)
@@ -1450,30 +1577,64 @@ extension JSONDecoderImpl {
             return string
         }
 
+        mutating func decodeIfPresent(_ type: String.Type) throws -> String? {
+            let value = self.peekNextValueIfPresent(ofType: String.self)
+            let result: String? = switch value {
+            case nil, .null: nil
+            default: try impl.unwrapString(from: value.unsafelyUnwrapped, for: codingPathNode, currentIndexKey)
+            }
+            advanceToNextValue()
+            return result
+        }
+
         mutating func decode(_: Double.Type) throws -> Double {
             try decodeFloatingPoint()
         }
 
+        mutating func decodeIfPresent(_ type: Double.Type) throws -> Double? {
+            try decodeFloatingPointIfPresent()
+        }
+
         mutating func decode(_: Float.Type) throws -> Float {
             try decodeFloatingPoint()
         }
 
+        mutating func decodeIfPresent(_ type: Float.Type) throws -> Float? {
+            try decodeFloatingPointIfPresent()
+        }
+
         mutating func decode(_: Int.Type) throws -> Int {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: Int.Type) throws -> Int? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: Int8.Type) throws -> Int8 {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: Int8.Type) throws -> Int8? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: Int16.Type) throws -> Int16 {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: Int16.Type) throws -> Int16? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: Int32.Type) throws -> Int32 {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: Int32.Type) throws -> Int32? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: Int64.Type) throws -> Int64 {
             try decodeFixedWidthInteger()
         }
@@ -1483,22 +1644,42 @@ extension JSONDecoderImpl {
           try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: Int64.Type) throws -> Int64? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: UInt.Type) throws -> UInt {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: UInt.Type) throws -> UInt? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: UInt8.Type) throws -> UInt8 {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: UInt8.Type) throws -> UInt8? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: UInt16.Type) throws -> UInt16 {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: UInt16.Type) throws -> UInt16? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: UInt32.Type) throws -> UInt32 {
             try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: UInt32.Type) throws -> UInt32? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode(_: UInt64.Type) throws -> UInt64 {
             try decodeFixedWidthInteger()
         }
@@ -1508,6 +1689,10 @@ extension JSONDecoderImpl {
           try decodeFixedWidthInteger()
         }
 
+        mutating func decodeIfPresent(_: UInt64.Type) throws -> UInt64? {
+            try decodeFixedWidthIntegerIfPresent()
+        }
+
         mutating func decode<T: Decodable>(_ type: T.Type) throws -> T {
             let value = try self.peekNextValue(ofType: type)
             let result = try impl.unwrap(value, as: type, for: codingPathNode, currentIndexKey)
@@ -1516,6 +1701,16 @@ extension JSONDecoderImpl {
             return result
         }
 
+        mutating func decodeIfPresent<T: Decodable>(_ type: T.Type) throws -> T? {
+            let value = self.peekNextValueIfPresent(ofType: type)
+            let result: T? = switch value {
+            case nil, .null: nil
+            default: try impl.unwrap(value.unsafelyUnwrapped, as: type, for: codingPathNode, currentIndexKey)
+            }
+            advanceToNextValue()
+            return result
+        }
+
         mutating func nestedContainer<NestedKey: CodingKey>(keyedBy type: NestedKey.Type) throws -> KeyedDecodingContainer<NestedKey> {
             let value = try self.peekNextValue(ofType: KeyedDecodingContainer<NestedKey>.self)
             let container = try impl.with(value: value, path: codingPathNode.appending(index: currentIndex)) {
@@ -1555,11 +1750,20 @@ extension JSONDecoderImpl {
         }
 
         @inline(__always)
-        private mutating func peekNextValue<T>(ofType type: T.Type) throws -> JSONMap.Value {
+        private mutating func peekNextValueIfPresent<T>(ofType type: T.Type) -> JSONMap.Value? {
             if let value = peekedValue {
                 return value
             }
             guard let nextValue = valueIterator.next() else {
+                return nil
+            }
+            peekedValue = nextValue
+            return nextValue
+        }
+
+        @inline(__always)
+        private mutating func peekNextValue<T>(ofType type: T.Type) throws -> JSONMap.Value {
+            guard let nextValue = peekNextValueIfPresent(ofType: type) else {
                 var message = "Unkeyed container is at end."
                 if T.self == UnkeyedContainer.self {
                     message = "Cannot get nested unkeyed container -- unkeyed container is at end."
@@ -1577,7 +1781,6 @@ extension JSONDecoderImpl {
                           debugDescription: message,
                           underlyingError: nil))
             }
-            peekedValue = nextValue
             return nextValue
         }
 
@@ -1596,6 +1799,26 @@ extension JSONDecoderImpl {
             advanceToNextValue()
             return result
         }
+
+        @inline(__always) private mutating func decodeFixedWidthIntegerIfPresent<T: FixedWidthInteger>() throws -> T? {
+            let value = self.peekNextValueIfPresent(ofType: T.self)
+            let result: T? = switch value {
+            case nil, .null: nil
+            default: try impl.unwrapFixedWidthInteger(from: value.unsafelyUnwrapped, as: T.self, for: codingPathNode, currentIndexKey)
+            }
+            advanceToNextValue()
+            return result
+        }
+
+        @inline(__always) private mutating func decodeFloatingPointIfPresent<T: PrevalidatedJSONNumberBufferConvertible & BinaryFloatingPoint>() throws -> T? {
+            let value = self.peekNextValueIfPresent(ofType: T.self)
+            let result: T? = switch value {
+            case nil, .null: nil
+            default: try impl.unwrapFloatingPoint(from: value.unsafelyUnwrapped, as: T.self, for: codingPathNode, currentIndexKey)
+            }
+            advanceToNextValue()
+            return result
+        }
     }
 }
 
