@@ -65,7 +65,7 @@ public struct Locale : Hashable, Equatable, Sendable {
     ///
     /// - note: The autoupdating Locale will only compare equal to another autoupdating Locale.
     public static var autoupdatingCurrent : Locale {
-        Locale(inner: LocaleCache.cache.autoupdatingCurrent)
+        Locale(inner: LocaleCache.autoupdatingCurrent)
     }
 
     /// Returns the user's current locale.
@@ -75,12 +75,12 @@ public struct Locale : Hashable, Equatable, Sendable {
 
     /// System locale.
     internal static var system : Locale {
-        Locale(inner: LocaleCache.cache.system)
+        Locale(inner: LocaleCache.system)
     }
     
     /// Unlocalized locale (`en_001`).
     internal static var unlocalized : Locale {
-        Locale(inner: LocaleCache.cache.unlocalized)
+        Locale(inner: LocaleCache.unlocalized)
     }
 
 #if FOUNDATION_FRAMEWORK && canImport(_FoundationICU)
@@ -232,7 +232,10 @@ public struct Locale : Hashable, Equatable, Sendable {
 
     /// Returns the identifier of the locale.
     public var identifier: String {
-        _locale.identifier
+        @_effects(releasenone)
+        get {
+            _locale.identifier
+        }
     }
 
     /// Returns the language code of the locale, or nil if has none.
@@ -742,18 +745,16 @@ public struct Locale : Hashable, Equatable, Sendable {
     /// - "nl": Dutch
     /// - "el": Greek
     /// For all other locales such as en_US, this is `false`.
-    internal var doesNotRequireSpecialCaseHandling: Bool {
-        // We call into the _LocaleBase because this value is cached in many cases. They will defer the initial calculation to the below helper function.
-        _locale.doesNotRequireSpecialCaseHandling
-    }
-    
     package static func identifierDoesNotRequireSpecialCaseHandling(_ identifier: String) -> Bool {
-        guard identifier.count >= 2 else { return true }
-
-        let first = identifier.prefix(2)
-        switch first {
-        case "az", "lt", "tr", "nl", "el":
-            return false // Does require special handling
+        var byteIterator = identifier.utf8.makeIterator()
+        switch (byteIterator.next(), byteIterator.next()) {
+        case
+            (UInt8(ascii: "a"), UInt8(ascii: "z")),
+            (UInt8(ascii: "l"), UInt8(ascii: "t")),
+            (UInt8(ascii: "t"), UInt8(ascii: "r")),
+            (UInt8(ascii: "n"), UInt8(ascii: "l")),
+            (UInt8(ascii: "e"), UInt8(ascii: "l")):
+            return false
         default:
             return true // Does not require special handling
         }
