@@ -18,33 +18,6 @@ internal import _FoundationCShims
 
 fileprivate let stringEncodingAttributeName = "com.apple.TextEncoding"
 
-private struct ExtendingToUTF16Sequence<Base: Sequence<UInt8>> : Sequence {
-    typealias Element = UInt16
-    
-    struct Iterator : IteratorProtocol {
-        private var base: Base.Iterator
-        
-        init(_ base: Base.Iterator) {
-            self.base = base
-        }
-        
-        mutating func next() -> Element? {
-            guard let value = base.next() else { return nil }
-            return UInt16(value)
-        }
-    }
-    
-    private let base: Base
-    
-    init(_ base: Base) {
-        self.base = base
-    }
-    
-    func makeIterator() -> Iterator {
-        Iterator(base.makeIterator())
-    }
-}
-
 
 @available(macOS 10.10, iOS 8.0, watchOS 2.0, tvOS 9.0, *)
 extension String {
@@ -66,7 +39,7 @@ extension String {
         where S.Iterator.Element == UInt8
     {
         switch encoding {
-        case .ascii:
+        case .ascii, .nonLossyASCII:
             func makeString(buffer: UnsafeBufferPointer<UInt8>) -> String? {
                 return String(_validating: buffer, as: Unicode.ASCII.self)
             }
@@ -112,7 +85,17 @@ extension String {
                 self = string
             } else {
                 // String is not valid UTF8
+#if FOUNDATION_FRAMEWORK && os(macOS)
+                // Allow for invalid UTF8 to be repaired in compatibility cases
+                if Self.compatibiltity2 {
+                    self = String(decoding: Array(bytes), as: UTF8.self)
+                } else {
+                    return nil
+                }
+
+#else
                 return nil
+#endif
             }
         case .utf16BigEndian, .utf16LittleEndian, .utf16:
             // See also the package extension String?(_utf16:), which does something similar to this without the swapping of big/little.
@@ -171,12 +154,9 @@ extension String {
             }
         #if !FOUNDATION_FRAMEWORK
         case .isoLatin1:
-            guard bytes.allSatisfy(\.isValidISOLatin1) else {
-                return nil
-            }
-            // isoLatin1 is an 8-bit encoding that represents a subset of UTF-16
-            // Map to 16-bit values and decode as UTF-16
-            self.init(_validating: ExtendingToUTF16Sequence(bytes), as: UTF16.self)
+            // ISO Latin 1 bytes are always valid since it's an 8-bit encoding that maps scalars 0x0 through 0xFF
+            // Simply extend each byte to 16 bits and decode as UTF-16
+            self.init(decoding: bytes.lazy.map { UInt16($0) }, as: UTF16.self)
         case .macOSRoman:
             func buildString(_ bytes: UnsafeBufferPointer<UInt8>) -> String {
                 String(unsafeUninitializedCapacity: bytes.count * 3) { buffer in
