@@ -75,11 +75,6 @@ extension NSCalendar {
         }
         return _NSSwiftCalendar(calendar: Calendar(identifier: id))
     }
-
-    @objc
-    class func _resetCurrent() {
-        CalendarCache.cache.reset()
-    }
 }
 
 // MARK: -
