@@ -33,7 +33,7 @@ internal class ICUNumberFormatterBase : @unchecked Sendable {
     /// Stored for testing purposes only
     internal let skeleton: String
 
-    init?(skeleton: String, localeIdentifier: String, preferences: LocalePreferences?) {
+    init?(skeleton: String, localeIdentifier: String) {
         self.skeleton = skeleton
         let ustr = Array(skeleton.utf16)
         var status = U_ZERO_ERROR
@@ -222,29 +222,28 @@ internal class ICUNumberFormatterBase : @unchecked Sendable {
 
 final class ICUNumberFormatter : ICUNumberFormatterBase, @unchecked Sendable {
     fileprivate struct Signature : Hashable {
-        let collection: NumberFormatStyleConfiguration.Collection
+        let skeleton: String
         let localeIdentifier: String
-        let localePreferences: LocalePreferences?
     }
 
     fileprivate static let cache = FormatterCache<Signature, ICUNumberFormatter?>()
 
     private static func _create(with signature: Signature) -> ICUNumberFormatter? {
         Self.cache.formatter(for: signature) {
-            .init(skeleton: signature.collection.skeleton, localeIdentifier: signature.localeIdentifier, preferences: signature.localePreferences)
+            .init(skeleton: signature.skeleton, localeIdentifier: signature.localeIdentifier)
         }
     }
 
     static func create<T: BinaryInteger>(for style: IntegerFormatStyle<T>) -> ICUNumberFormatter? {
-        _create(with: .init(collection: style.collection, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     static func create(for style: Decimal.FormatStyle) -> ICUNumberFormatter? {
-        _create(with: .init(collection: style.collection, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     static func create<T: BinaryFloatingPoint>(for style: FloatingPointFormatStyle<T>) -> ICUNumberFormatter? {
-        _create(with: .init(collection: style.collection, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     func attributedFormat(_ v: Value) -> AttributedString {
@@ -259,16 +258,15 @@ final class ICUNumberFormatter : ICUNumberFormatterBase, @unchecked Sendable {
 
 final class ICUCurrencyNumberFormatter : ICUNumberFormatterBase, @unchecked Sendable {
     fileprivate struct Signature : Hashable {
-        let collection: CurrencyFormatStyleConfiguration.Collection
+        let skeleton: String
         let currencyCode: String
         let localeIdentifier: String
-        let localePreferences: LocalePreferences?
     }
 
     private static func skeleton(for signature: Signature) -> String {
         var s = "currency/\(signature.currencyCode)"
 
-        let stem = signature.collection.skeleton
+        let stem = signature.skeleton
         if stem.count > 0 {
             s += " " + stem
         }
@@ -280,20 +278,20 @@ final class ICUCurrencyNumberFormatter : ICUNumberFormatterBase, @unchecked Send
 
     static private func _create(with signature: Signature) -> ICUCurrencyNumberFormatter? {
         return Self.cache.formatter(for: signature) {
-            .init(skeleton: Self.skeleton(for: signature), localeIdentifier: signature.localeIdentifier, preferences: signature.localePreferences)
+            .init(skeleton: Self.skeleton(for: signature), localeIdentifier: signature.localeIdentifier)
         }
     }
 
     static func create<T: BinaryInteger>(for style: IntegerFormatStyle<T>.Currency) -> ICUCurrencyNumberFormatter? {
-        _create(with: .init(collection: style.collection, currencyCode: style.currencyCode, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, currencyCode: style.currencyCode, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     static func create(for style: Decimal.FormatStyle.Currency) -> ICUCurrencyNumberFormatter? {
-        _create(with: .init(collection: style.collection, currencyCode: style.currencyCode, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, currencyCode: style.currencyCode, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     static func create<T: BinaryFloatingPoint>(for style: FloatingPointFormatStyle<T>.Currency) -> ICUCurrencyNumberFormatter? {
-        _create(with: .init(collection: style.collection, currencyCode: style.currencyCode, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, currencyCode: style.currencyCode, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     func attributedFormat(_ v: Value) -> AttributedString {
@@ -308,14 +306,13 @@ final class ICUCurrencyNumberFormatter : ICUNumberFormatterBase, @unchecked Send
 
 final class ICUPercentNumberFormatter : ICUNumberFormatterBase, @unchecked Sendable {
     fileprivate struct Signature : Hashable {
-        let collection: NumberFormatStyleConfiguration.Collection
+        let skeleton: String
         let localeIdentifier: String
-        let localePreferences: LocalePreferences?
     }
 
     private static func skeleton(for signature: Signature) -> String {
         var s = "percent"
-        let stem = signature.collection.skeleton
+        let stem = signature.skeleton
         if stem.count > 0 {
             s += " " + stem
         }
@@ -326,20 +323,20 @@ final class ICUPercentNumberFormatter : ICUNumberFormatterBase, @unchecked Senda
 
     private static func _create(with signature: Signature) -> ICUPercentNumberFormatter? {
         return Self.cache.formatter(for: signature) {
-            .init(skeleton: Self.skeleton(for: signature), localeIdentifier: signature.localeIdentifier, preferences: signature.localePreferences)
+            .init(skeleton: Self.skeleton(for: signature), localeIdentifier: signature.localeIdentifier)
         }
     }
 
     static func create<T: BinaryInteger>(for style: IntegerFormatStyle<T>.Percent) -> ICUPercentNumberFormatter? {
-        _create(with: .init(collection: style.collection, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     static func create(for style: Decimal.FormatStyle.Percent) -> ICUPercentNumberFormatter? {
-        _create(with: .init(collection: style.collection, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     static func create<T: BinaryFloatingPoint>(for style: FloatingPointFormatStyle<T>.Percent) -> ICUPercentNumberFormatter? {
-        _create(with: .init(collection: style.collection, localeIdentifier: style.locale.identifierCapturingPreferences, localePreferences: style.locale.prefs))
+        _create(with: .init(skeleton: style.collection.skeleton, localeIdentifier: style.locale.identifierCapturingPreferences))
     }
 
     func attributedFormat(_ v: Value) -> AttributedString {
@@ -356,15 +353,14 @@ final class ICUByteCountNumberFormatter : ICUNumberFormatterBase, @unchecked Sen
     fileprivate struct Signature : Hashable {
         let skeleton: String
         let localeIdentifier: String
-        let localePreferences: LocalePreferences?
     }
 
     fileprivate static let cache = FormatterCache<Signature, ICUByteCountNumberFormatter?>()
 
     static func create(for skeleton: String, locale: Locale) -> ICUByteCountNumberFormatter? {
-        let signature = Signature(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences, localePreferences: locale.prefs)
+        let signature = Signature(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences)
         return Self.cache.formatter(for: signature) {
-            .init(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences, preferences: locale.prefs)
+            .init(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences)
         }
     }
 
@@ -415,15 +411,14 @@ final class ICUMeasurementNumberFormatter : ICUNumberFormatterBase, @unchecked S
     fileprivate struct Signature : Hashable {
         let skeleton: String
         let localeIdentifier: String
-        let localePreferences: LocalePreferences?
     }
 
     fileprivate static let cache = FormatterCache<Signature, ICUMeasurementNumberFormatter?>()
 
     static func create(for skeleton: String, locale: Locale) -> ICUMeasurementNumberFormatter? {
-        let signature = Signature(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences, localePreferences: locale.prefs)
+        let signature = Signature(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences)
         return Self.cache.formatter(for: signature) {
-            .init(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences, preferences: locale.prefs)
+            .init(skeleton: skeleton, localeIdentifier: locale.identifierCapturingPreferences)
         }
     }
 
