@@ -27,7 +27,7 @@ internal import Foundation_Private.NSLocale
 extension NSLocale {
     @objc
     static var _autoupdatingCurrent: NSLocale {
-        LocaleCache.cache.autoupdatingCurrentNSLocale()
+        LocaleCache.autoupdatingCurrentNSLocale
     }
 
     @objc
@@ -37,7 +37,7 @@ extension NSLocale {
 
     @objc
     static var _system: NSLocale {
-        LocaleCache.cache.systemNSLocale()
+        LocaleCache.systemNSLocale
     }
 
     @objc
@@ -65,7 +65,7 @@ extension NSLocale {
 
     @objc
     private class func _resetCurrent() {
-        LocaleCache.cache.reset()
+        LocaleNotifications.cache.reset()
     }
 
     @objc
@@ -218,8 +218,7 @@ extension NSLocale {
 
     @objc(_doesNotRequireSpecialCaseHandling)
     func _doesNotRequireSpecialCaseHandling() -> Bool {
-        // Unable to use cached locale; create a new one. Subclass `_NSSwiftLocale` implements a better version
-        return Locale(identifier: localeIdentifier).doesNotRequireSpecialCaseHandling
+        Locale.identifierDoesNotRequireSpecialCaseHandling(localeIdentifier)
     }
 }
 
@@ -231,6 +230,7 @@ internal class _NSSwiftLocale: _NSLocaleBridge, @unchecked Sendable {
 
     internal init(_ locale: Locale) {
         self.locale = locale
+
         // The superclass does not care at all what the identifier is. Avoid a potentially recursive call into the Locale cache here by just using an empty string.
         super.init(localeIdentifier: "")
     }
@@ -244,7 +244,7 @@ internal class _NSSwiftLocale: _NSLocaleBridge, @unchecked Sendable {
             return NSLocale.self
         }
     }
-    
+
     override init(localeIdentifier string: String) {
         self.locale = Locale(identifier: string)
         super.init(localeIdentifier: "")
@@ -360,7 +360,10 @@ internal class _NSSwiftLocale: _NSLocaleBridge, @unchecked Sendable {
     // MARK: -
 
     override var localeIdentifier: String {
-        locale.identifier
+        @_effects(releasenone)
+        get {
+            return locale.identifier
+        }
     }
 
     @available(macOS, deprecated: 13) @available(iOS, deprecated: 16) @available(tvOS, deprecated: 16) @available(watchOS, deprecated: 9)
@@ -520,8 +523,9 @@ internal class _NSSwiftLocale: _NSLocaleBridge, @unchecked Sendable {
         return _NSSwiftLocale(copy)
     }
     
+    @_effects(releasenone)
     override func _doesNotRequireSpecialCaseHandling() -> Bool {
-        return locale.doesNotRequireSpecialCaseHandling
+        Locale.identifierDoesNotRequireSpecialCaseHandling(locale.identifier)
     }
 }
 
