@@ -15,7 +15,7 @@ import FoundationEssentials
 #endif
 
 #if canImport(Glibc)
-import Glibc
+@preconcurrency import Glibc
 #endif
 
 #if canImport(ucrt)
