@@ -282,6 +282,14 @@ final class FileManagerTests : XCTestCase {
             XCTAssertThrowsError(try $0.subpathsOfDirectory(atPath: "does_not_exist")) {
                 XCTAssertEqual(($0 as? CocoaError)?.code, .fileReadNoSuchFile)
             }
+
+            XCTAssertThrowsError(try $0.subpathsOfDirectory(atPath: "")) {
+                #if os(Windows)
+                XCTAssertEqual(($0 as? CocoaError)?.code, .fileReadInvalidFileName)
+                #else
+                XCTAssertEqual(($0 as? CocoaError)?.code, .fileReadNoSuchFile)
+                #endif
+            }
             
             let fullContents = ["dir1", "dir1/dir2", "dir1/dir2/Bar", "dir1/dir2/Foo", "dir1/dir3", "dir1/dir3/Baz", "symlinks", "symlinks/Bar", "symlinks/Foo", "symlinks/Parent"]
             let cwd = $0.currentDirectoryPath
@@ -487,7 +495,7 @@ final class FileManagerTests : XCTestCase {
     func testCopyItemAtPathToPath() throws {
         let data = randomData()
         try FileManagerPlayground {
-            Directory("dir") {
+            Directory("dir", attributes: [.posixPermissions : 0o777]) {
                 File("foo", contents: data)
                 "bar"
             }
@@ -502,8 +510,10 @@ final class FileManagerTests : XCTestCase {
             XCTAssertEqual($0.delegateCaptures.shouldCopy, [.init("dir", "dir2"), .init("dir/bar", "dir2/bar"), .init("dir/foo", "dir2/foo")])
 #else
             XCTAssertEqual($0.delegateCaptures.shouldCopy, [.init("dir", "dir2"), .init("dir/foo", "dir2/foo"), .init("dir/bar", "dir2/bar")])
+            
+            // Specifically for non-Windows (where copying directory metadata takes a special path) double check that the metadata was copied exactly
+            XCTAssertEqual(try $0.attributesOfItem(atPath: "dir2")[.posixPermissions] as? UInt, 0o777)
 #endif
-
             XCTAssertThrowsError(try $0.copyItem(atPath: "does_not_exist", toPath: "dir3")) {
                 XCTAssertEqual(($0 as? CocoaError)?.code, .fileReadNoSuchFile)
             }
@@ -832,6 +842,13 @@ final class FileManagerTests : XCTestCase {
             }
         }
     }
+    
+    func testSpecialTrashDirectoryDuplication() throws {
+        try FileManagerPlayground {}.test { fileManager in
+            let trashURLs = fileManager.urls(for: .trashDirectory, in: .userDomainMask)
+            XCTAssertEqual(trashURLs.count, 1, "There should only be one trash directory for the user domain, found \(trashURLs)")
+        }
+    }
     #endif
     
     func testSearchPaths() throws {
@@ -1012,14 +1029,22 @@ final class FileManagerTests : XCTestCase {
         #if canImport(Darwin) && !os(macOS)
         throw XCTSkip("This test is not applicable on this platform")
         #else
-        #if os(Windows)
-        let fallbackPath = URL(filePath: try XCTUnwrap(ProcessInfo.processInfo.environment["ALLUSERSPROFILE"]), directoryHint: .isDirectory)
-        #else
-        let fallbackPath = URL(filePath: "/var/empty", directoryHint: .isDirectory)
-        #endif
-        
-        XCTAssertEqual(FileManager.default.homeDirectory(forUser: ""), fallbackPath)
-        XCTAssertEqual(FileManager.default.homeDirectory(forUser: UUID().uuidString), fallbackPath)
+        XCTAssertNil(FileManager.default.homeDirectory(forUser: ""))
+        XCTAssertNil(FileManager.default.homeDirectory(forUser: UUID().uuidString))
         #endif
     }
+
+    func testWindowsDirectoryCreationCrash() throws {
+        try FileManagerPlayground {
+            Directory("a\u{301}") {
+
+            }
+        }.test {
+            XCTAssertTrue($0.fileExists(atPath: "a\u{301}"))
+            let data = randomData()
+            XCTAssertTrue($0.createFile(atPath: "a\u{301}/test", contents: data))
+            XCTAssertTrue($0.fileExists(atPath: "a\u{301}/test"))
+            XCTAssertEqual($0.contents(atPath: "a\u{301}/test"), data)
+        }
+    }
 }
