@@ -516,11 +516,38 @@ final class GregorianCalendarRecurrenceRuleTests: XCTestCase {
     func testYearlyRecurrenceWithWeekNumberExpansion() {
         var calendar = Calendar(identifier: .gregorian)
         calendar.firstWeekday = 2 // Week starts on Monday
+        calendar.minimumDaysInFirstWeek = 4
+        
+        let start = Date(timeIntervalSince1970: 1357048800.0) // 2013-01-01T14:00:00-0000
+        let end   = Date(timeIntervalSince1970: 1483279200.0) // 2017-01-01T14:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: calendar, frequency: .yearly)
+        rule.weeks = [1, 2]
+        
+        let results = Array(rule.recurrences(of: start, in: start..<end))
+        
+        let expectedResults: [Date] = [
+            Date(timeIntervalSince1970: 1357048800.0), // 2013-01-01T14:00:00-0000
+            Date(timeIntervalSince1970: 1357653600.0), // 2013-01-08T14:00:00-0000
+            Date(timeIntervalSince1970: 1388584800.0), // 2014-01-01T14:00:00-0000
+            Date(timeIntervalSince1970: 1389189600.0), // 2014-01-08T14:00:00-0000
+            Date(timeIntervalSince1970: 1420120800.0), // 2015-01-01T14:00:00-0000
+            Date(timeIntervalSince1970: 1420725600.0), // 2015-01-08T14:00:00-0000
+            Date(timeIntervalSince1970: 1452261600.0), // 2016-01-08T14:00:00-0000
+            Date(timeIntervalSince1970: 1452866400.0), // 2016-01-15T14:00:00-0000
+        ]
+        
+        XCTAssertEqual(results, expectedResults)
+    }
+    
+    func testYearlyRecurrenceWithWeekNumberExpansionCountingBack() {
+        var calendar = Calendar(identifier: .gregorian)
+        calendar.firstWeekday = 2 // Week starts on Monday
         
         let start = Date(timeIntervalSince1970: 1704117600.0) // 2024-01-01T14:00:00-0000
         let end   = Date(timeIntervalSince1970: 1767225600.0) // 2026-01-01T00:00:00-0000
         
-        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .yearly)
+        var rule = Calendar.RecurrenceRule(calendar: calendar, frequency: .yearly)
         rule.weeks = [1, -1]
         
         let results = Array(rule.recurrences(of: start, in: start..<end))
@@ -636,4 +663,107 @@ final class GregorianCalendarRecurrenceRuleTests: XCTestCase {
         }
         // If we get here, there isn't an infinite loop
    }
+    
+   func testOutOfRangeComponents() {
+        let start = Date(timeIntervalSince1970: 1695304800.0) // 2023-09-21T14:00:00-0000
+        let end   = Date(timeIntervalSince1970: 1729519200.0) // 2024-10-21T14:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .monthly)
+        rule.daysOfTheMonth = [32, -32]
+        
+        let eventStart = Date(timeIntervalSince1970: 1285077600.0) // 2010-09-21T14:00:00-0000
+        let results = Array(rule.recurrences(of: eventStart, in: start..<end))
+        
+        XCTAssertEqual(results, [])
+    }
+    
+    func testFirstMondaysStrictMatching() {
+        let startDate = Date(timeIntervalSince1970: 1706659200.0) // 2024-01-31T00:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .monthly, matchingPolicy: .strict)
+        rule.weekdays = [.nth(1, .monday)]
+        
+        var dates = rule.recurrences(of: startDate).makeIterator()
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1707091200.0)) // 2024-02-05T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1709510400.0)) // 2024-03-04T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1711929600.0)) // 2024-04-01T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1714953600.0)) // 2024-05-06T00:00:00-0000
+    }
+    
+    func testFifthFridaysStrictMatching() {
+        let startDate = Date(timeIntervalSince1970: 1706659200.0) // 2024-01-31T00:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .monthly, matchingPolicy: .strict)
+        rule.weekdays = [.nth(5, .friday)]
+        
+        var dates = rule.recurrences(of: startDate).makeIterator()
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1711670400.0)) // 2024-03-29T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1717113600.0)) // 2024-05-31T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1724976000.0)) // 2024-08-30T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1732838400.0)) // 2024-11-29T00:00:00-0000
+    }
+
+    func testYearlyRecurrenceWeekdayExpansionStrictMatching() {
+        let startDate = Date(timeIntervalSince1970: 1709164800.0) // 2024-02-29T00:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .yearly, matchingPolicy: .strict)
+        rule.weekdays = [.nth(5, .friday)]
+        
+        var dates = rule.recurrences(of: startDate).makeIterator()
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1738281600.0)) // 2025-01-31T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1769731200.0)) // 2026-01-30T00:00:00-0000
+    }
+
+    func testYearlyRecurrenceDayOfYearExpansionStrictMatching() {
+        let startDate = Date(timeIntervalSince1970: 1709164800.0) // 2024-02-29T00:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .yearly, matchingPolicy: .strict)
+        rule.daysOfTheYear = [61]
+        
+        var dates = rule.recurrences(of: startDate).makeIterator()
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1709251200.0)) // 2024-03-01T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1740873600.0)) // 2025-03-02T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1772409600.0)) // 2026-03-02T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1803945600.0)) // 2027-03-02T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1835481600.0)) // 2028-03-01T00:00:00-0000
+    }
+
+    func testYearlyRecurrenceWeekExpansionStrictMatching() {
+        let startDate = Date(timeIntervalSince1970: 1709164800.0) // 2024-02-29T00:00:00-0000
+        
+        var rule = Calendar.RecurrenceRule(calendar: gregorian, frequency: .yearly, matchingPolicy: .strict)
+        rule.weeks = [2]
+        
+        var dates = rule.recurrences(of: startDate).makeIterator()
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1736553600.0)) // 2025-01-11T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1767484800.0)) // 2026-01-04T00:00:00-0000
+        XCTAssertEqual(dates.next(), Date(timeIntervalSince1970: 1799020800.0)) // 2027-01-04T00:00:00-0000
+    }
+
+    func testWeekdayFilter() {
+        var alarms = Calendar.RecurrenceRule(calendar: gregorian, frequency: .daily)
+        alarms.weekdays = [.every(.monday), .every(.tuesday), .every(.wednesday), .every(.thursday), .every(.friday)]
+        alarms.hours = [7, 8]
+        alarms.minutes = [0, 15, 30, 45]
+
+        let start = Date(timeIntervalSince1970: 1695304800.0) // 2023-09-21T14:00:00-0000
+        var results = alarms.recurrences(of: start).makeIterator()
+
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695366000.0)) // 2023-09-22T07:00:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695366900.0)) // 2023-09-22T07:15:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695367800.0)) // 2023-09-22T07:30:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695368700.0)) // 2023-09-22T07:45:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695369600.0)) // 2023-09-22T08:00:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695370500.0)) // 2023-09-22T08:15:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695371400.0)) // 2023-09-22T08:30:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695372300.0)) // 2023-09-22T08:45:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695625200.0)) // 2023-09-25T07:00:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695626100.0)) // 2023-09-25T07:15:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695627000.0)) // 2023-09-25T07:30:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695627900.0)) // 2023-09-25T07:45:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695628800.0)) // 2023-09-25T08:00:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695629700.0)) // 2023-09-25T08:15:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695630600.0)) // 2023-09-25T08:30:00-0000
+        XCTAssertEqual(results.next(), Date(timeIntervalSince1970: 1695631500.0)) // 2023-09-25T08:45:00-0000
+    }
 }
