@@ -502,6 +502,11 @@ class TestPropertyListEncoder : XCTestCase {
         }
     }
 
+    private func _forEachEncodingFormat(_ body: (PropertyListDecoder.PropertyListFormat) throws -> Void) rethrows {
+        try body(.xml)
+        try body(.binary)
+    }
+
     // MARK: - Other tests
     func testUnkeyedContainerContainingNulls() throws {
         struct UnkeyedContainerContainingNullTestType : Codable, Equatable {
@@ -858,9 +863,6 @@ class TestPropertyListEncoder : XCTestCase {
         }
     }
 
-#if FOUNDATION_FRAMEWORK
-    // Requires old style plist support
-    // Requires "NEXTStep" decoding in String(bytes:encoding:) for decoding the octal characters
 
     func test_oldStylePlist_invalid() {
         let data = "goodbye cruel world".data(using: String._Encoding.utf16)!
@@ -968,7 +970,6 @@ string3 = "Good evening"; // Test
             XCTFail("Unexpected error: \(error)")
         }
     }
-#endif
     
 #if FOUNDATION_FRAMEWORK
     // Requires __PlistDictionaryDecoder
@@ -1503,6 +1504,47 @@ data1 = <7465
             _testRoundTrip(of: input, in: .xml)
         }
     }
+    
+    func testCustomSubclass() throws {
+        // verify we consult the subclass for the output format
+        let encodeMe = ["hello":"world"]
+        let encoder = XMLOnlyEncoder()
+        let data = try encoder.encode(encodeMe)
+        let dataAsStr = String(data: data, encoding: .utf8)!
+        XCTAssertTrue(dataAsStr.hasPrefix("<?xml"))
+    }
+
+    func test_decodeIfPresent() throws {
+        try _forEachEncodingFormat { format in
+            let encoder = PropertyListEncoder()
+            encoder.outputFormat = format
+
+            let emptyDictEncoding = try encoder.encode(DecodeIfPresentAllTypes<KeyedEncodeWithoutNulls>.allNils)
+            let testEmptyDict = try PropertyListDecoder().decode(DecodeIfPresentAllTypes<UseKeyed>.self, from: emptyDictEncoding)
+            XCTAssertEqual(testEmptyDict, .allNils)
+
+            let allNullDictEncoding = try encoder.encode(DecodeIfPresentAllTypes<KeyedEncodeWithNulls>.allNils)
+            let testAllNullDict = try PropertyListDecoder().decode(DecodeIfPresentAllTypes<UseKeyed>.self, from: allNullDictEncoding)
+            XCTAssertEqual(testAllNullDict, .allNils)
+
+            let allOnesDictEncoding = try encoder.encode(DecodeIfPresentAllTypes<UseKeyed>.allOnes)
+            let testAllOnesDict = try PropertyListDecoder().decode(DecodeIfPresentAllTypes<UseKeyed>.self, from: allOnesDictEncoding)
+            XCTAssertEqual(testAllOnesDict, .allOnes)
+
+            let emptyArrayEncoding = try encoder.encode(DecodeIfPresentAllTypes<UnkeyedEncodeWithoutNulls>.allNils)
+            let testEmptyArray = try PropertyListDecoder().decode(DecodeIfPresentAllTypes<UseUnkeyed>.self, from: emptyArrayEncoding)
+            XCTAssertEqual(testEmptyArray, .allNils)
+
+            let allNullArrayEncoding = try encoder.encode(DecodeIfPresentAllTypes<UnkeyedEncodeWithNulls>.allNils)
+            let testAllNullArray = try PropertyListDecoder().decode(DecodeIfPresentAllTypes<UseUnkeyed>.self, from: allNullArrayEncoding)
+            XCTAssertEqual(testAllNullArray, .allNils)
+
+            let allOnesArrayEncoding = try encoder.encode(DecodeIfPresentAllTypes<UseUnkeyed>.allOnes)
+            let testAllOnesArray = try PropertyListDecoder().decode(DecodeIfPresentAllTypes<UseUnkeyed>.self, from: allOnesArrayEncoding)
+            XCTAssertEqual(testAllOnesArray, .allOnes)
+        }
+
+    }
 }
             
 
@@ -2036,3 +2078,11 @@ private struct MultipleDecodeOptionsTestType : Codable, Equatable {
     }
 }
 
+// MARK: - Helper Class
+
+class XMLOnlyEncoder : PropertyListEncoder, @unchecked Sendable {
+    override var outputFormat: PropertyListDecoder.PropertyListFormat {
+        get { return .xml }
+        set { }
+    }
+}
