@@ -179,7 +179,7 @@ final class URLTests : XCTestCase {
             }
             #endif
 
-            let url = URL(string: test.key, relativeTo: base)
+            let url = URL(stringOrEmpty: test.key, relativeTo: base)
             XCTAssertNotNil(url, "Got nil url for string: \(test.key)")
             XCTAssertEqual(url?.absoluteString, test.value, "Failed test for string: \(test.key)")
         }
@@ -242,7 +242,7 @@ final class URLTests : XCTestCase {
             "http:g"        :  "g", // For strict parsers
         ]
         for test in tests {
-            let url = URL(string: test.key, relativeTo: base)!
+            let url = URL(stringOrEmpty: test.key, relativeTo: base)!
             XCTAssertEqual(url.path(), test.value)
             if (url.hasDirectoryPath && url.path().count > 1) {
                 // The trailing slash is stripped in .path for file system compatibility
@@ -340,13 +340,50 @@ final class URLTests : XCTestCase {
 
     #if os(Windows)
     func testURLWindowsDriveLetterPath() throws {
-        let url = URL(filePath: "C:\\test\\path", directoryHint: .notDirectory)
+        var url = URL(filePath: #"C:\test\path"#, directoryHint: .notDirectory)
         // .absoluteString and .path() use the RFC 8089 URL path
         XCTAssertEqual(url.absoluteString, "file:///C:/test/path")
         XCTAssertEqual(url.path(), "/C:/test/path")
         // .path and .fileSystemPath strip the leading slash
         XCTAssertEqual(url.path, "C:/test/path")
         XCTAssertEqual(url.fileSystemPath, "C:/test/path")
+
+        url = URL(filePath: #"C:\"#, directoryHint: .isDirectory)
+        XCTAssertEqual(url.absoluteString, "file:///C:/")
+        XCTAssertEqual(url.path(), "/C:/")
+        XCTAssertEqual(url.path, "C:/")
+        XCTAssertEqual(url.fileSystemPath, "C:/")
+
+        url = URL(filePath: #"C:\\\"#, directoryHint: .isDirectory)
+        XCTAssertEqual(url.absoluteString, "file:///C:///")
+        XCTAssertEqual(url.path(), "/C:///")
+        XCTAssertEqual(url.path, "C:/")
+        XCTAssertEqual(url.fileSystemPath, "C:/")
+
+        url = URL(filePath: #"\C:\"#, directoryHint: .isDirectory)
+        XCTAssertEqual(url.absoluteString, "file:///C:/")
+        XCTAssertEqual(url.path(), "/C:/")
+        XCTAssertEqual(url.path, "C:/")
+        XCTAssertEqual(url.fileSystemPath, "C:/")
+
+        let base = URL(filePath: #"\d:\path\"#, directoryHint: .isDirectory)
+        url = URL(filePath: #"%43:\fake\letter"#, directoryHint: .notDirectory, relativeTo: base)
+        // ":" is encoded to "%3A" in the first path segment so it's not mistaken as the scheme separator
+        XCTAssertEqual(url.relativeString, "%2543%3A/fake/letter")
+        XCTAssertEqual(url.path(), "/d:/path/%2543%3A/fake/letter")
+        XCTAssertEqual(url.path, "d:/path/%43:/fake/letter")
+        XCTAssertEqual(url.fileSystemPath, "d:/path/%43:/fake/letter")
+
+        let cwd = URL.currentDirectory()
+        var iter = cwd.path().utf8.makeIterator()
+        if iter.next() == ._slash,
+           let driveLetter = iter.next(), driveLetter.isLetter!,
+           iter.next() == ._colon {
+            let path = #"\\?\"# + "\(Unicode.Scalar(driveLetter))" + #":\"#
+            url = URL(filePath: path, directoryHint: .isDirectory)
+            XCTAssertEqual(url.path.last, "/")
+            XCTAssertEqual(url.fileSystemPath.last, "/")
+        }
     }
     #endif
 
@@ -570,11 +607,13 @@ final class URLTests : XCTestCase {
         XCTAssertEqual(appended.absoluteString, "file:///var/mobile/relative/no:slash")
         XCTAssertEqual(appended.relativePath, "relative/no:slash")
 
-        // `appending(component:)` should explicitly treat `component` as a single
-        // path component, meaning "/" should be encoded to "%2F" before appending
+        // .appending(component:) should explicitly treat slashComponent as a single
+        // path component, meaning "/" should be encoded to "%2F" before appending.
+        // However, the old behavior didn't do this for file URLs, so we maintain the
+        // old behavior to prevent breakage.
         appended = url.appending(component: slashComponent, directoryHint: .notDirectory)
-        checkBehavior(appended.absoluteString, new: "file:///var/mobile/relative/%2Fwith:slash", old: "file:///var/mobile/relative/with:slash")
-        checkBehavior(appended.relativePath, new: "relative/%2Fwith:slash", old: "relative/with:slash")
+        XCTAssertEqual(appended.absoluteString, "file:///var/mobile/relative/with:slash")
+        XCTAssertEqual(appended.relativePath, "relative/with:slash")
 
         appended = url.appendingPathComponent(component, isDirectory: false)
         XCTAssertEqual(appended.absoluteString, "file:///var/mobile/relative/no:slash")
@@ -650,7 +689,7 @@ final class URLTests : XCTestCase {
         checkBehavior(relative.path, new: "/", old: "/..")
 
         relative = URL(filePath: "", relativeTo: absolute)
-        checkBehavior(relative.relativePath, new: "", old: ".")
+        XCTAssertEqual(relative.relativePath, ".")
         XCTAssertTrue(relative.hasDirectoryPath)
         XCTAssertEqual(relative.path, "/absolute")
 
@@ -829,22 +868,30 @@ final class URLTests : XCTestCase {
     }
 
     func testURLTildeFilePath() throws {
-        var url = URL(filePath: "~")
+        func urlIsAbsolute(_ url: URL) -> Bool {
+            if url.relativePath.utf8.first == ._slash {
+                return true
+            }
+            guard url.baseURL != nil else {
+                return false
+            }
+            #if !FOUNDATION_FRAMEWORK_NSURL
+            return url.path().utf8.first == ._slash
+            #else
+            return url.path.utf8.first == ._slash
+            #endif
+        }
+
         // "~" must either be expanded to an absolute path or resolved against a base URL
-        XCTAssertTrue(
-            url.relativePath.utf8.first == ._slash || (url.baseURL != nil && url.path().utf8.first == ._slash)
-        )
+        var url = URL(filePath: "~")
+        XCTAssertTrue(urlIsAbsolute(url))
 
         url = URL(filePath: "~", directoryHint: .isDirectory)
-        XCTAssertTrue(
-            url.relativePath.utf8.first == ._slash || (url.baseURL != nil && url.path().utf8.first == ._slash)
-        )
+        XCTAssertTrue(urlIsAbsolute(url))
         XCTAssertEqual(url.path().utf8.last, ._slash)
 
         url = URL(filePath: "~/")
-        XCTAssertTrue(
-            url.relativePath.utf8.first == ._slash || (url.baseURL != nil && url.path().utf8.first == ._slash)
-        )
+        XCTAssertTrue(urlIsAbsolute(url))
         XCTAssertEqual(url.path().utf8.last, ._slash)
     }
 
@@ -868,6 +915,15 @@ final class URLTests : XCTestCase {
         url.deletePathExtension()
         // Old behavior only searches the last empty component, so the extension isn't actually removed
         checkBehavior(url.path(), new: "/path/", old: "/path.foo///")
+
+        url = URL(filePath: "/tmp/x")
+        url.appendPathExtension("")
+        XCTAssertEqual(url.path(), "/tmp/x")
+        XCTAssertEqual(url, url.deletingPathExtension().appendingPathExtension(url.pathExtension))
+
+        url = URL(filePath: "/tmp/x.")
+        url.deletePathExtension()
+        XCTAssertEqual(url.path(), "/tmp/x.")
     }
 
     func testURLAppendingToEmptyPath() throws {
@@ -910,6 +966,21 @@ final class URLTests : XCTestCase {
         XCTAssertEqual(schemeOnly.absoluteString, "scheme:foo")
     }
 
+    func testURLEmptySchemeCompatibility() throws {
+        var url = try XCTUnwrap(URL(string: ":memory:"))
+        XCTAssertEqual(url.scheme, "")
+
+        let base = try XCTUnwrap(URL(string: "://home"))
+        XCTAssertEqual(base.host(), "home")
+
+        url = try XCTUnwrap(URL(string: "/path", relativeTo: base))
+        XCTAssertEqual(url.scheme, "")
+        XCTAssertEqual(url.host(), "home")
+        XCTAssertEqual(url.path, "/path")
+        XCTAssertEqual(url.absoluteString, "://home/path")
+        XCTAssertEqual(url.absoluteURL.scheme, "")
+    }
+
     func testURLComponentsPercentEncodedUnencodedProperties() throws {
         var comp = URLComponents()
 
@@ -1291,6 +1362,29 @@ final class URLTests : XCTestCase {
         comp = try XCTUnwrap(URLComponents(string: legalURLString))
         XCTAssertEqual(comp.string, legalURLString)
         XCTAssertEqual(comp.percentEncodedPath, colonFirstPath)
+
+        // Colons should be percent-encoded by URLComponents.string if
+        // they could be misinterpreted as a scheme separator.
+
+        comp = URLComponents()
+        comp.percentEncodedPath = "not%20a%20scheme:"
+        XCTAssertEqual(comp.string, "not%20a%20scheme%3A")
+
+        // These would fail if we did not percent-encode the colon.
+        // .string should always produce a valid URL string, or nil.
+
+        XCTAssertNotNil(URL(string: comp.string!))
+        XCTAssertNotNil(URLComponents(string: comp.string!))
+
+        // In rare cases, an app might rely on URL allowing an empty scheme,
+        // but then take that string and pass it to URLComponents to modify
+        // other components of the URL. We shouldn't percent-encode the colon
+        // in these cases.
+
+        let url = try XCTUnwrap(URL(string: "://host/path"))
+        comp = try XCTUnwrap(URLComponents(string: url.absoluteString))
+        comp.query = "key=value"
+        XCTAssertEqual(comp.string, "://host/path?key=value")
     }
 
     func testURLComponentsInvalidPaths() {
@@ -1371,6 +1465,28 @@ final class URLTests : XCTestCase {
         XCTAssertEqual(comp.path, "/my\u{0}path")
     }
 
+    func testURLStandardizedEmptyString() {
+        let url = URL(string: "../../../")!
+        let standardized = url.standardized
+        XCTAssertTrue(standardized.path().isEmpty)
+    }
+
+#if FOUNDATION_FRAMEWORK
+    func testURLComponentsBridging() {
+        var nsURLComponents = NSURLComponents(
+            string: "https://example.com?url=https%3A%2F%2Fapple.com"
+        )!
+        var urlComponents = nsURLComponents as URLComponents
+        XCTAssertEqual(urlComponents.string, nsURLComponents.string)
+
+        urlComponents = URLComponents(
+            string: "https://example.com?url=https%3A%2F%2Fapple.com"
+        )!
+        nsURLComponents = urlComponents as NSURLComponents
+        XCTAssertEqual(urlComponents.string, nsURLComponents.string)
+    }
+#endif
+
     func testURLComponentsUnixDomainSocketOverHTTPScheme() {
         var comp = URLComponents()
         comp.scheme = "http+unix"
@@ -1404,4 +1520,38 @@ final class URLTests : XCTestCase {
         XCTAssertEqual(comp.host, "/path/to/socket")
         XCTAssertEqual(comp.path, "/info")
     }
+
+    func testURLComponentsUnixDomainSocketOverWebSocketScheme() {
+        var comp = URLComponents()
+        comp.scheme = "ws+unix"
+        comp.host = "/path/to/socket"
+        comp.path = "/info"
+        XCTAssertEqual(comp.string, "ws+unix://%2Fpath%2Fto%2Fsocket/info")
+
+        comp.scheme = "wss+unix"
+        XCTAssertEqual(comp.string, "wss+unix://%2Fpath%2Fto%2Fsocket/info")
+
+        comp.encodedHost = "%2Fpath%2Fto%2Fsocket"
+        XCTAssertEqual(comp.string, "wss+unix://%2Fpath%2Fto%2Fsocket/info")
+        XCTAssertEqual(comp.encodedHost, "%2Fpath%2Fto%2Fsocket")
+        XCTAssertEqual(comp.host, "/path/to/socket")
+        XCTAssertEqual(comp.path, "/info")
+
+        // "/path/to/socket" is not a valid host for schemes
+        // that IDNA-encode hosts instead of percent-encoding
+        comp.scheme = "ws"
+        XCTAssertNil(comp.string)
+
+        comp.scheme = "wss"
+        XCTAssertNil(comp.string)
+
+        comp.scheme = "wss+unix"
+        XCTAssertEqual(comp.string, "wss+unix://%2Fpath%2Fto%2Fsocket/info")
+
+        // Check that we can parse a percent-encoded ws+unix URL string
+        comp = URLComponents(string: "ws+unix://%2Fpath%2Fto%2Fsocket/info")!
+        XCTAssertEqual(comp.encodedHost, "%2Fpath%2Fto%2Fsocket")
+        XCTAssertEqual(comp.host, "/path/to/socket")
+        XCTAssertEqual(comp.path, "/info")
+    }
 }
