@@ -12,7 +12,8 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+import NIOCore
+import NIOPosix
 
 func run(identifier: String) {
     MultiThreadedEventLoopGroup.withCurrentThreadAsEventLoop { loop in
