@@ -17,7 +17,7 @@ set -eu
 
 function make_package() {
     cat > "$tmpdir/syscallwrapper/Package.swift" <<"EOF"
-// swift-tools-version:5.0
+// swift-tools-version:5.7
 // The swift-tools-version declares the minimum version of Swift required to build this package.
 
 import PackageDescription
@@ -28,20 +28,29 @@ let package = Package(
     targets: [
         .target(
             name: "syscallwrapper",
-            dependencies: ["CNIOLinux", "CNIODarwin"]),
+            dependencies: ["CNIOLinux", "CNIODarwin", "NIOCore"]),
         .target(
             name: "CNIOLinux",
-            dependencies: []),
+            dependencies: [],
+            cSettings: [
+                .define("_GNU_SOURCE")
+            ]),
         .target(
             name: "CNIODarwin",
             dependencies: []),
+        // This target does nothing, it just makes imports work.
+        .target(
+            name: "NIOCore",
+            dependencies: []),
     ]
 )
 EOF
     cp "$here/../../Tests/NIOTests/SystemCallWrapperHelpers.swift" \
-        "$here/../../Sources/NIO/System.swift" \
-        "$here/../../Sources/NIO/IO.swift" \
+        "$here/../../Sources/NIOPosix/System.swift" \
+        "$here/../../Sources/NIOPosix/IO.swift" \
         "$tmpdir/syscallwrapper/Sources/syscallwrapper"
     ln -s "$here/../../Sources/CNIOLinux" "$tmpdir/syscallwrapper/Sources"
     ln -s "$here/../../Sources/CNIODarwin" "$tmpdir/syscallwrapper/Sources"
+    mkdir "$tmpdir/syscallwrapper/Sources/NIOCore"
+    touch "$tmpdir/syscallwrapper/Sources/NIOCore/empty.swift"
 }
