@@ -12,12 +12,10 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+import NIOCore
 import NIOHTTP1
 
-#if compiler(>=5.5) // we cannot write this on one line with `&&` because Swift 5.0 doesn't like it...
-#if compiler(>=5.5) && $AsyncAwait
-@available(macOS 9999, iOS 9999, watchOS 9999, tvOS 9999, *)
+@available(macOS 10.15, iOS 13, tvOS 13, watchOS 6, *)
 struct AsyncChannelIO<Request, Response> {
     let channel: Channel
 
@@ -40,5 +38,3 @@ struct AsyncChannelIO<Request, Response> {
         try await self.channel.close()
     }
 }
-#endif
-#endif
