@@ -14,7 +14,7 @@
 
 import CNIOAtomics
 
-#if os(macOS) || os(iOS) || os(watchOS) || os(tvOS)
+#if canImport(Darwin)
 import Darwin
 fileprivate func sys_sched_yield() {
     pthread_yield_np()
@@ -26,7 +26,14 @@ fileprivate func sys_sched_yield() {
   Sleep(0)
 }
 #else
+#if canImport(Glibc)
 import Glibc
+#elseif canImport(Musl)
+import Musl
+#else
+#error("The concurrency atomics module was unable to identify your C library.")
+#endif
+
 fileprivate func sys_sched_yield() {
     _ = sched_yield()
 }
@@ -51,6 +58,7 @@ fileprivate func sys_sched_yield() {
 /// Atomic primitives are useful when building constructs that need to
 /// communicate or cooperate across multiple threads. In the case of
 /// SwiftNIO this usually involves communicating across multiple event loops.
+@available(*, deprecated, message: "please use UnsafeAtomic from https://github.com/apple/swift-atomics instead")
 public struct UnsafeEmbeddedAtomic<T: AtomicPrimitive> {
     @usableFromInline
     internal let value: OpaquePointer
@@ -89,6 +97,7 @@ public struct UnsafeEmbeddedAtomic<T: AtomicPrimitive> {
     ///
     /// - Parameter rhs: The value to add to this object.
     /// - Returns: The previous value of this object, before the addition occurred.
+    @discardableResult
     @inlinable
     public func add(_ rhs: T) -> T {
         return T.atomic_add(self.value, rhs)
@@ -102,6 +111,7 @@ public struct UnsafeEmbeddedAtomic<T: AtomicPrimitive> {
     ///
     /// - Parameter rhs: The value to subtract from this object.
     /// - Returns: The previous value of this object, before the subtraction occurred.
+    @discardableResult
     @inlinable
     public func sub(_ rhs: T) -> T {
         return T.atomic_sub(self.value, rhs)
@@ -171,7 +181,7 @@ public struct UnsafeEmbeddedAtomic<T: AtomicPrimitive> {
 /// By necessity, all atomic values are references: after all, it makes no
 /// sense to talk about managing an atomic value when each time it's modified
 /// the thread that modified it gets a local copy!
-@available(*, deprecated, message:"please use NIOAtomic instead")
+@available(*, deprecated, message:"please use ManagedAtomic from https://github.com/apple/swift-atomics instead")
 public final class Atomic<T: AtomicPrimitive> {
     @usableFromInline
     internal let embedded: UnsafeEmbeddedAtomic<T>
@@ -210,6 +220,7 @@ public final class Atomic<T: AtomicPrimitive> {
     ///
     /// - Parameter rhs: The value to add to this object.
     /// - Returns: The previous value of this object, before the addition occurred.
+    @discardableResult
     @inlinable
     public func add(_ rhs: T) -> T {
         return self.embedded.add(rhs)
@@ -223,6 +234,7 @@ public final class Atomic<T: AtomicPrimitive> {
     ///
     /// - Parameter rhs: The value to subtract from this object.
     /// - Returns: The previous value of this object, before the subtraction occurred.
+    @discardableResult
     @inlinable
     public func sub(_ rhs: T) -> T {
         return self.embedded.sub(rhs)
@@ -525,7 +537,7 @@ public final class AtomicBox<T: AnyObject> {
             }
         }
 
-        // step 2: After having gained 'ownership' of the old value, we can release the Unmanged.
+        // step 2: After having gained 'ownership' of the old value, we can release the Unmanaged.
         let oldPtr = Unmanaged<T>.fromOpaque(UnsafeRawPointer(bitPattern: oldPtrBits)!)
         return oldPtr.takeRetainedValue()
     }
