@@ -12,9 +12,13 @@
 //
 //===----------------------------------------------------------------------===//
 
+#if !canImport(Darwin) || os(macOS)
 let crashTestSuites: [String: Any] = [
     "EventLoopCrashTests": EventLoopCrashTests(),
     "ByteBufferCrashTests": ByteBufferCrashTests(),
     "SystemCrashTests": SystemCrashTests(),
     "HTTPCrashTests": HTTPCrashTests(),
+    "StrictCrashTests": StrictCrashTests(),
+    "LoopBoundTests": LoopBoundTests(),
 ]
+#endif
