@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2020 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2020-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -12,9 +12,10 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+import NIOCore
 
 struct ByteBufferCrashTests {
+    #if !canImport(Darwin) || os(macOS)
     let testMovingReaderIndexPastWriterIndex = CrashTest(
         regex: #"Precondition failed: new readerIndex: 1, expected: range\(0, 0\)"#
     ) {
@@ -27,4 +28,5 @@ struct ByteBufferCrashTests {
     ) {
         _ = ByteBufferAllocator().buffer(capacity: -1)
     }
+    #endif
 }
