@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2020 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2020-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -12,7 +12,8 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+#if !canImport(Darwin) || os(macOS)
+import NIOPosix
 import Foundation
 
 fileprivate let group = MultiThreadedEventLoopGroup(numberOfThreads: 1)
@@ -20,6 +21,7 @@ fileprivate let group = MultiThreadedEventLoopGroup(numberOfThreads: 1)
 struct SystemCrashTests {
     let testEBADFIsUnacceptable = CrashTest(
         regex: "Precondition failed: unacceptable errno \(EBADF) Bad file descriptor in", {
-            _ = try? NIOPipeBootstrap(group: group).withPipes(inputDescriptor: .max, outputDescriptor: .max - 1).wait()
+            _ = try? NIOPipeBootstrap(group: group).takingOwnershipOfDescriptors(input: .max, output: .max - 1).wait()
         })
 }
+#endif
