@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2019 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -12,29 +12,27 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+import NIOCore
 
 private func writeChunk(wrapOutboundOut: (IOData) -> NIOAny, context: ChannelHandlerContext, isChunked: Bool, chunk: IOData, promise: EventLoopPromise<Void>?) {
-    let (mW1, mW2, mW3): (EventLoopPromise<Void>?, EventLoopPromise<Void>?, EventLoopPromise<Void>?)
-
-    switch (isChunked, promise) {
-    case (true, .some(let p)):
-        /* chunked encoding and the user's interested: we need three promises and need to cascade into the users promise */
-        let (w1, w2, w3) = (context.eventLoop.makePromise() as EventLoopPromise<Void>, context.eventLoop.makePromise() as EventLoopPromise<Void>, context.eventLoop.makePromise() as EventLoopPromise<Void>)
-        w1.futureResult.and(w2.futureResult).and(w3.futureResult).map { (_: ((((), ()), ()))) in }.cascade(to: p)
-        (mW1, mW2, mW3) = (w1, w2, w3)
-    case (false, .some(let p)):
-        /* not chunked, so just use the user's promise for the actual data */
-        (mW1, mW2, mW3) = (nil, p, nil)
-    case (_, .none):
-        /* user isn't interested, let's not bother even allocating promises */
-        (mW1, mW2, mW3) = (nil, nil, nil)
-    }
-
     let readableBytes = chunk.readableBytes
 
-    /* we don't want to copy the chunk unnecessarily and therefore call write an annoyingly large number of times */
-    if isChunked {
+    // we don't want to copy the chunk unnecessarily and therefore call write an annoyingly large number of times
+    // we also don't frame empty chunks as they would otherwise end the response stream
+    // we still need to write the empty IODate to complete the promise in the right order but is otherwise a no-op.
+    if isChunked && readableBytes > 0 {
+        let (mW1, mW2, mW3): (EventLoopPromise<Void>?, EventLoopPromise<Void>?, EventLoopPromise<Void>?)
+
+        if let p = promise {
+            /* chunked encoding and the user's interested: we need three promises and need to cascade into the users promise */
+            let (w1, w2, w3) = (context.eventLoop.makePromise() as EventLoopPromise<Void>, context.eventLoop.makePromise() as EventLoopPromise<Void>, context.eventLoop.makePromise() as EventLoopPromise<Void>)
+            w1.futureResult.and(w2.futureResult).and(w3.futureResult).map { (_: ((((), ()), ()))) in }.cascade(to: p)
+            (mW1, mW2, mW3) = (w1, w2, w3)
+        } else {
+            /* user isn't interested, let's not bother even allocating promises */
+            (mW1, mW2, mW3) = (nil, nil, nil)
+        }
+        
         var buffer = context.channel.allocator.buffer(capacity: 32)
         let len = String(readableBytes, radix: 16)
         buffer.writeString(len)
@@ -47,7 +45,7 @@ private func writeChunk(wrapOutboundOut: (IOData) -> NIOAny, context: ChannelHan
         buffer.moveReaderIndex(forwardBy: buffer.readableBytes - 2)
         context.write(wrapOutboundOut(.byteBuffer(buffer)), promise: mW3)
     } else {
-        context.write(wrapOutboundOut(chunk), promise: mW2)
+        context.write(wrapOutboundOut(chunk), promise: promise)
     }
 }
 
@@ -128,6 +126,16 @@ private func correctlyFrameTransportHeaders(hasBody: HTTPMethod.HasBody, headers
     }
 }
 
+/// Validates the response/request headers to ensure that we correctly send the body with chunked transfer
+/// encoding, when needed.
+private func messageIsChunked(headers: HTTPHeaders, version: HTTPVersion) -> Bool {
+    if version.major == 1 && version.minor >= 1 {
+        return headers.first(name: "transfer-encoding") == "chunked" ? true : false
+    } else {
+        return false
+    }
+}
+
 /// A `ChannelOutboundHandler` that can serialize HTTP requests.
 ///
 /// This channel handler is used to translate messages from a series of
@@ -136,9 +144,37 @@ public final class HTTPRequestEncoder: ChannelOutboundHandler, RemovableChannelH
     public typealias OutboundIn = HTTPClientRequestPart
     public typealias OutboundOut = IOData
 
+    /// Configuration for the ``HTTPRequestEncoder``.
+    ///
+    /// This object controls the behaviour of the ``HTTPRequestEncoder``. It enables users to
+    /// change the default behaviour of the type to better handle a wide range of use-cases.
+    public struct Configuration: Sendable, Hashable {
+        /// Whether the ``HTTPRequestEncoder`` should automatically add `Content-Length` or
+        /// `Transfer-Encoding` headers when appropriate.
+        ///
+        /// Defaults to `true`.
+        ///
+        /// Set to `false` if you are confident you are appropriately setting these framing
+        /// headers yourself, to skip NIO's transformation.
+        public var automaticallySetFramingHeaders: Bool
+
+        public init() {
+            self.automaticallySetFramingHeaders = true
+        }
+    }
+
     private var isChunked = false
 
-    public init () { }
+    private var configuration: Configuration
+
+    public convenience init() {
+        self.init(configuration: Configuration())
+    }
+
+    public init(configuration: Configuration) {
+        self.configuration = configuration
+    }
+
 
     public func write(context: ChannelHandlerContext, data: NIOAny, promise: EventLoopPromise<Void>?) {
         switch self.unwrapOutboundIn(data) {
@@ -146,19 +182,17 @@ public final class HTTPRequestEncoder: ChannelOutboundHandler, RemovableChannelH
             assert(!(request.headers.contains(name: "content-length") &&
                         request.headers[canonicalForm: "transfer-encoding"].contains("chunked"[...])),
                      "illegal HTTP sent: \(request) contains both a content-length and transfer-encoding:chunked")
-            self.isChunked = correctlyFrameTransportHeaders(hasBody: request.method.hasRequestBody,
-                                                            headers: &request.headers, version: request.version) == .chunked
+            if self.configuration.automaticallySetFramingHeaders {
+                self.isChunked = correctlyFrameTransportHeaders(hasBody: request.method.hasRequestBody,
+                                                                headers: &request.headers, version: request.version) == .chunked
+            } else {
+                self.isChunked = messageIsChunked(headers: request.headers, version: request.version)
+            }
 
             writeHead(wrapOutboundOut: self.wrapOutboundOut, writeStartLine: { buffer in
                 buffer.write(request: request)
             }, context: context, headers: request.headers, promise: promise)
         case .body(let bodyPart):
-            guard bodyPart.readableBytes > 0 else {
-                // Empty writes shouldn't send any bytes in chunked or identity encoding.
-                context.write(self.wrapOutboundOut(bodyPart), promise: promise)
-                return
-            }
-
             writeChunk(wrapOutboundOut: self.wrapOutboundOut, context: context, isChunked: self.isChunked, chunk: bodyPart, promise: promise)
         case .end(let trailers):
             writeTrailers(wrapOutboundOut: self.wrapOutboundOut, context: context, isChunked: self.isChunked, trailers: trailers, promise: promise)
@@ -166,6 +200,9 @@ public final class HTTPRequestEncoder: ChannelOutboundHandler, RemovableChannelH
     }
 }
 
+@available(*, unavailable)
+extension HTTPRequestEncoder: Sendable {}
+
 /// A `ChannelOutboundHandler` that can serialize HTTP responses.
 ///
 /// This channel handler is used to translate messages from a series of
@@ -174,9 +211,36 @@ public final class HTTPResponseEncoder: ChannelOutboundHandler, RemovableChannel
     public typealias OutboundIn = HTTPServerResponsePart
     public typealias OutboundOut = IOData
 
+    /// Configuration for the ``HTTPResponseEncoder``.
+    ///
+    /// This object controls the behaviour of the ``HTTPResponseEncoder``. It enables users to
+    /// change the default behaviour of the type to better handle a wide range of use-cases.
+    public struct Configuration: Sendable, Hashable {
+        /// Whether the ``HTTPResponseEncoder`` should automatically add `Content-Length` or
+        /// `Transfer-Encoding` headers when appropriate.
+        ///
+        /// Defaults to `true`.
+        ///
+        /// Set to `false` if you are confident you are appropriately setting these framing
+        /// headers yourself, to skip NIO's transformation.
+        public var automaticallySetFramingHeaders: Bool
+
+        public init() {
+            self.automaticallySetFramingHeaders = true
+        }
+    }
+
     private var isChunked = false
 
-    public init () { }
+    private var configuration: Configuration
+
+    public convenience init() {
+        self.init(configuration: Configuration())
+    }
+
+    public init(configuration: Configuration) {
+        self.configuration = configuration
+    }
 
     public func write(context: ChannelHandlerContext, data: NIOAny, promise: EventLoopPromise<Void>?) {
         switch self.unwrapOutboundIn(data) {
@@ -184,8 +248,13 @@ public final class HTTPResponseEncoder: ChannelOutboundHandler, RemovableChannel
             assert(!(response.headers.contains(name: "content-length") &&
                         response.headers[canonicalForm: "transfer-encoding"].contains("chunked"[...])),
                      "illegal HTTP sent: \(response) contains both a content-length and transfer-encoding:chunked")
-            self.isChunked = correctlyFrameTransportHeaders(hasBody: response.status.mayHaveResponseBody ? .yes : .no,
-                                                            headers: &response.headers, version: response.version) == .chunked
+
+            if self.configuration.automaticallySetFramingHeaders {
+                self.isChunked = correctlyFrameTransportHeaders(hasBody: response.status.mayHaveResponseBody ? .yes : .no,
+                                                                headers: &response.headers, version: response.version) == .chunked
+            } else {
+                self.isChunked = messageIsChunked(headers: response.headers, version: response.version)
+            }
 
             writeHead(wrapOutboundOut: self.wrapOutboundOut, writeStartLine: { buffer in
                 buffer.write(response: response)
@@ -198,6 +267,9 @@ public final class HTTPResponseEncoder: ChannelOutboundHandler, RemovableChannel
     }
 }
 
+@available(*, unavailable)
+extension HTTPResponseEncoder: Sendable {}
+
 private extension ByteBuffer {
     private mutating func write(status: HTTPResponseStatus) {
         self.writeString(String(status.code))
