@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2019 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -11,8 +11,7 @@
 // SPDX-License-Identifier: Apache-2.0
 //
 //===----------------------------------------------------------------------===//
-
-import NIO
+import NIOCore
 
 /// Errors that may be raised by the `HTTPServerProtocolUpgrader`.
 public enum HTTPServerUpgradeErrors: Error {
@@ -20,7 +19,7 @@ public enum HTTPServerUpgradeErrors: Error {
 }
 
 /// User events that may be fired by the `HTTPServerProtocolUpgrader`.
-public enum HTTPServerUpgradeEvents {
+public enum HTTPServerUpgradeEvents: Sendable {
     /// Fired when HTTP upgrade has completed and the
     /// `HTTPServerProtocolUpgrader` is about to remove itself from the
     /// `ChannelPipeline`.
@@ -63,11 +62,11 @@ public final class HTTPServerUpgradeHandler: ChannelInboundHandler, RemovableCha
     public typealias InboundIn = HTTPServerRequestPart
     public typealias InboundOut = HTTPServerRequestPart
     public typealias OutboundOut = HTTPServerResponsePart
-
+    
     private let upgraders: [String: HTTPServerProtocolUpgrader]
     private let upgradeCompletionHandler: (ChannelHandlerContext) -> Void
 
-    private let httpEncoder: HTTPResponseEncoder?
+    private let httpEncoder: HTTPResponseEncoder
     private let extraHTTPHandlers: [RemovableChannelHandler]
 
     /// Whether we've already seen the first request.
@@ -88,7 +87,12 @@ public final class HTTPServerUpgradeHandler: ChannelInboundHandler, RemovableCha
     ///     this should include the `HTTPDecoder`, but should also include any other handler that cannot tolerate
     ///     receiving non-HTTP data.
     /// - Parameter upgradeCompletionHandler: A block that will be fired when HTTP upgrade is complete.
-    public init(upgraders: [HTTPServerProtocolUpgrader], httpEncoder: HTTPResponseEncoder, extraHTTPHandlers: [RemovableChannelHandler], upgradeCompletionHandler: @escaping (ChannelHandlerContext) -> Void) {
+    public init(
+        upgraders: [HTTPServerProtocolUpgrader],
+        httpEncoder: HTTPResponseEncoder,
+        extraHTTPHandlers: [RemovableChannelHandler],
+        upgradeCompletionHandler: @escaping (ChannelHandlerContext) -> Void
+    ) {
         var upgraderMap = [String: HTTPServerProtocolUpgrader]()
         for upgrader in upgraders {
             upgraderMap[upgrader.supportedProtocol.lowercased()] = upgrader
@@ -172,7 +176,7 @@ public final class HTTPServerUpgradeHandler: ChannelInboundHandler, RemovableCha
         self.handleUpgrade(context: context, request: request, requestedProtocols: requestedProtocols)
             .hop(to: context.eventLoop) // the user might return a future from another EventLoop.
             .whenSuccess { callback in
-                assert(context.eventLoop.inEventLoop)
+                context.eventLoop.assertInEventLoop()
                 if let callback = callback {
                     self.gotUpgrader(upgrader: callback)
                 } else {
@@ -231,17 +235,22 @@ public final class HTTPServerUpgradeHandler: ChannelInboundHandler, RemovableCha
                 self.removeExtraHandlers(context: context).flatMap {
                     self.sendUpgradeResponse(context: context, upgradeRequest: request, responseHeaders: finalResponseHeaders)
                 }.flatMap {
-                    self.removeHandler(context: context, handler: self.httpEncoder)
-                }.map {
+                    context.pipeline.syncOperations.removeHandler(self.httpEncoder)
+                }.flatMap { () -> EventLoopFuture<Void> in
                     self.upgradeCompletionHandler(context)
-                }.flatMap {
-                    upgrader.upgrade(context: context, upgradeRequest: request)
-                }.map {
-                    context.fireUserInboundEventTriggered(HTTPServerUpgradeEvents.upgradeComplete(toProtocol: proto, upgradeRequest: request))
-                    self.upgradeState = .upgradeComplete
-                }.whenComplete { (_: Result<Void, Error>) in
-                    // When we remove ourselves we'll be delivering any buffered data.
-                    context.pipeline.removeHandler(context: context, promise: nil)
+                    return upgrader.upgrade(context: context, upgradeRequest: request)
+                }.whenComplete { result in
+                    switch result {
+                    case .success:
+                        context.fireUserInboundEventTriggered(HTTPServerUpgradeEvents.upgradeComplete(toProtocol: proto, upgradeRequest: request))
+                        self.upgradeState = .upgradeComplete
+                        // When we remove ourselves we'll be delivering any buffered data.
+                        context.pipeline.removeHandler(context: context, promise: nil)
+
+                    case .failure(let error):
+                        // Remain in the '.upgrading' state.
+                        context.fireErrorCaught(error)
+                    }
                 }
             }
         }.flatMapError { error in
@@ -296,20 +305,11 @@ public final class HTTPServerUpgradeHandler: ChannelInboundHandler, RemovableCha
         context.pipeline.removeHandler(context: context, promise: nil)
     }
 
-    /// Builds the initial mandatory HTTP headers for HTTP ugprade responses.
+    /// Builds the initial mandatory HTTP headers for HTTP upgrade responses.
     private func buildUpgradeHeaders(`protocol`: String) -> HTTPHeaders {
         return HTTPHeaders([("connection", "upgrade"), ("upgrade", `protocol`)])
     }
 
-    /// Removes the given channel handler from the channel pipeline.
-    private func removeHandler(context: ChannelHandlerContext, handler: RemovableChannelHandler?) -> EventLoopFuture<Void> {
-        if let handler = handler {
-            return context.pipeline.removeHandler(handler)
-        } else {
-            return context.eventLoop.makeSucceededFuture(())
-        }
-    }
-
     /// Removes any extra HTTP-related handlers from the channel pipeline.
     private func removeExtraHandlers(context: ChannelHandlerContext) -> EventLoopFuture<Void> {
         guard self.extraHTTPHandlers.count > 0 else {
@@ -321,6 +321,8 @@ public final class HTTPServerUpgradeHandler: ChannelInboundHandler, RemovableCha
     }
 }
 
+extension HTTPServerUpgradeHandler: @unchecked Sendable {}
+
 extension HTTPServerUpgradeHandler {
     /// The state of the upgrade handler.
     private enum UpgradeState {
