@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2018 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -11,8 +11,7 @@
 // SPDX-License-Identifier: Apache-2.0
 //
 //===----------------------------------------------------------------------===//
-
-import NIO
+import NIOCore
 
 let crlf: StaticString = "\r\n"
 let headerSeparator: StaticString = ": "
@@ -116,7 +115,7 @@ public struct HTTPRequestHead: Equatable {
     public static func ==(lhs: HTTPRequestHead, rhs: HTTPRequestHead) -> Bool {
         return lhs.method == rhs.method && lhs.uri == rhs.uri && lhs.version == rhs.version && lhs.headers == rhs.headers
     }
-    
+
     private mutating func copyStorageIfNotUniquelyReferenced () {
         if !isKnownUniquelyReferenced(&self._storage) {
             self._storage = self._storage.copy()
@@ -124,18 +123,22 @@ public struct HTTPRequestHead: Equatable {
     }
 }
 
+extension HTTPRequestHead: @unchecked Sendable {}
+
 /// The parts of a complete HTTP message, either request or response.
 ///
 /// A HTTP message is made up of a request or status line with several headers,
 /// encoded by `.head`, zero or more body parts, and optionally some trailers. To
 /// indicate that a complete HTTP message has been sent or received, we use `.end`,
-/// which may also contain any trailers that make up the mssage.
+/// which may also contain any trailers that make up the message.
 public enum HTTPPart<HeadT: Equatable, BodyT: Equatable> {
     case head(HeadT)
     case body(BodyT)
     case end(HTTPHeaders?)
 }
 
+extension HTTPPart: Sendable where HeadT: Sendable, BodyT: Sendable {}
+
 extension HTTPPart: Equatable {}
 
 /// The components of a HTTP request from the view of a HTTP client.
@@ -221,7 +224,7 @@ public struct HTTPResponseHead: Equatable {
     public static func ==(lhs: HTTPResponseHead, rhs: HTTPResponseHead) -> Bool {
         return lhs.status == rhs.status && lhs.version == rhs.version && lhs.headers == rhs.headers
     }
-    
+
     private mutating func copyStorageIfNotUniquelyReferenced () {
         if !isKnownUniquelyReferenced(&self._storage) {
             self._storage = self._storage.copy()
@@ -229,6 +232,16 @@ public struct HTTPResponseHead: Equatable {
     }
 }
 
+extension HTTPResponseHead: @unchecked Sendable {}
+
+extension HTTPResponseHead {
+    /// Determines if the head is purely informational. If a head is informational another head will follow this
+    /// head eventually.
+    var isInformational: Bool {
+        100 <= self.status.code && self.status.code < 200 && self.status.code != 101
+    }
+}
+
 private extension UInt8 {
     var isASCII: Bool {
         return self <= 127
@@ -311,7 +324,7 @@ public struct HTTPHeaders: CustomStringConvertible, ExpressibleByDictionaryLiter
         // Otherwise we'd only have the one below with a default argument for `allocator`.
         self.init(headers, keepAliveState: .unknown)
     }
-    
+
     /// Construct a `HTTPHeaders` structure.
     ///
     /// - parameters
@@ -478,6 +491,8 @@ public struct HTTPHeaders: CustomStringConvertible, ExpressibleByDictionaryLiter
     }
 }
 
+extension HTTPHeaders: Sendable {}
+
 extension HTTPHeaders {
 
     /// The total number of headers that can be contained without allocating new storage.
@@ -502,9 +517,9 @@ extension ByteBuffer {
     mutating func write(headers: HTTPHeaders) {
         for header in headers.headers {
             self.writeString(header.0)
-            self.writeStaticString(": ")
+            self.writeStaticString(headerSeparator)
             self.writeString(header.1)
-            self.writeStaticString("\r\n")
+            self.writeStaticString(crlf)
         }
         self.writeStaticString(crlf)
     }
@@ -512,7 +527,7 @@ extension ByteBuffer {
 
 extension HTTPHeaders: RandomAccessCollection {
     public typealias Element = (name: String, value: String)
-    
+
     public struct Index: Comparable {
         fileprivate let base: Array<(String, String)>.Index
         public static func < (lhs: Index, rhs: Index) -> Bool {
@@ -541,28 +556,35 @@ extension HTTPHeaders: RandomAccessCollection {
     }
 }
 
-extension Character {
+extension UTF8.CodeUnit {
     var isASCIIWhitespace: Bool {
-        return self == " " || self == "\t" || self == "\r" || self == "\n" || self == "\r\n"
+        switch self {
+        case UInt8(ascii: " "),
+             UInt8(ascii: "\t"):
+          return true
+
+        default:
+          return false
+        }
     }
 }
 
 extension String {
     func trimASCIIWhitespace() -> Substring {
-        return self.dropFirst(0).trimWhitespace()
+        return Substring(self).trimWhitespace()
     }
 }
 
-private extension Substring {
-    func trimWhitespace() -> Substring {
-        var me = self
-        while me.first?.isASCIIWhitespace == .some(true) {
-            me = me.dropFirst()
+extension Substring {
+    fileprivate func trimWhitespace() -> Substring {
+        guard let firstNonWhitespace = self.utf8.firstIndex(where: { !$0.isASCIIWhitespace }) else {
+          // The whole substring is ASCII whitespace.
+          return Substring()
         }
-        while me.last?.isASCIIWhitespace == .some(true) {
-            me = me.dropLast()
-        }
-        return me
+
+        // There must be at least one non-ascii whitespace character, so banging here is safe.
+        let lastNonWhitespace = self.utf8.lastIndex(where: { !$0.isASCIIWhitespace })!
+        return Substring(self.utf8[firstNonWhitespace...lastNonWhitespace])
     }
 }
 
@@ -587,7 +609,7 @@ extension HTTPHeaders: Equatable {
     }
 }
 
-public enum HTTPMethod: Equatable {
+public enum HTTPMethod: Equatable, Sendable {
     internal enum HasBody {
         case yes
         case no
@@ -646,7 +668,7 @@ public enum HTTPMethod: Equatable {
 }
 
 /// A structure representing a HTTP version.
-public struct HTTPVersion: Equatable {
+public struct HTTPVersion: Equatable, Sendable {
     /// Create a HTTP version.
     ///
     /// - Parameter major: The major version number.
@@ -762,10 +784,20 @@ public enum HTTPParserError: Error {
     case invalidStatus
     case invalidMethod
     case invalidURL
+
+    @available(*, deprecated, message: "Cannot be thrown")
     case invalidHost
+
+    @available(*, deprecated, message: "Cannot be thrown")
     case invalidPort
+
+    @available(*, deprecated, message: "Cannot be thrown")
     case invalidPath
+
+    @available(*, deprecated, message: "Cannot be thrown")
     case invalidQueryString
+
+    @available(*, deprecated, message: "Cannot be thrown")
     case invalidFragment
     case lfExpected
     case invalidHeaderToken
@@ -1042,7 +1074,7 @@ extension HTTPResponseStatus {
 }
 
 /// A HTTP response status code.
-public enum HTTPResponseStatus {
+public enum HTTPResponseStatus: Sendable {
     /* use custom if you want to use a non-standard response code or
      have it available in a (UInt, String) pair from a higher-level web framework. */
     case custom(code: UInt, reasonPhrase: String)
@@ -1267,18 +1299,9 @@ public enum HTTPResponseStatus {
     }
 }
 
-extension HTTPResponseStatus: Equatable {
-    public static func ==(lhs: HTTPResponseStatus, rhs: HTTPResponseStatus) -> Bool {
-        switch (lhs, rhs) {
-        case (.custom(let lcode, let lreason), .custom(let rcode, let rreason)):
-            return lcode == rcode && lreason == rreason
-        case (.custom, _), (_, .custom):
-            return false
-        default:
-            return lhs.code == rhs.code
-        }
-    }
-}
+extension HTTPResponseStatus: Equatable {}
+
+extension HTTPResponseStatus: Hashable {}
 
 extension HTTPRequestHead: CustomStringConvertible {
     public var description: String {
@@ -1286,6 +1309,12 @@ extension HTTPRequestHead: CustomStringConvertible {
     }
 }
 
+extension HTTPResponseStatus: CustomStringConvertible {
+    public var description: String {
+        return "\(self.code) \(self.reasonPhrase)"
+    }
+}
+
 extension HTTPResponseHead: CustomStringConvertible {
     public var description: String {
         return "HTTPResponseHead { version: \(self.version), status: \(self.status), headers: \(self.headers) }"
@@ -1373,7 +1402,7 @@ extension HTTPMethod: RawRepresentable {
                 return value
         }
     }
-        
+
     public init(rawValue: String) {
         switch rawValue {
             case "GET":
