@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2019 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2019-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -11,8 +11,7 @@
 // SPDX-License-Identifier: Apache-2.0
 //
 //===----------------------------------------------------------------------===//
-
-import NIO
+import NIOCore
 
 /// Errors that may be raised by the `HTTPClientProtocolUpgrader`.
 public struct NIOHTTPClientUpgradeError: Hashable, Error {
@@ -115,10 +114,19 @@ public final class NIOHTTPClientUpgradeHandler: ChannelDuplexHandler, RemovableC
     ///     At the very least this should include the `HTTPEncoder` and `HTTPDecoder`, but should also include
     ///     any other handler that cannot tolerate receiving non-HTTP data.
     /// - Parameter upgradeCompletionHandler: A closure that will be fired when HTTP upgrade is complete.
-    public init(upgraders: [NIOHTTPClientProtocolUpgrader],
-                httpHandlers: [RemovableChannelHandler],
-                upgradeCompletionHandler: @escaping (ChannelHandlerContext) -> Void) {
-
+    public convenience init(
+        upgraders: [NIOHTTPClientProtocolUpgrader],
+        httpHandlers: [RemovableChannelHandler],
+        upgradeCompletionHandler: @escaping (ChannelHandlerContext) -> Void
+    ) {
+        self.init(_upgraders: upgraders, httpHandlers: httpHandlers, upgradeCompletionHandler: upgradeCompletionHandler)
+    }
+    
+    private init(
+        _upgraders upgraders: [NIOHTTPClientProtocolUpgrader],
+        httpHandlers: [RemovableChannelHandler],
+        upgradeCompletionHandler: @escaping (ChannelHandlerContext) -> Void
+    ) {
         precondition(upgraders.count > 0, "A minimum of one protocol upgrader must be specified.")
 
         self.upgraders = upgraders
@@ -370,6 +378,8 @@ public final class NIOHTTPClientUpgradeHandler: ChannelDuplexHandler, RemovableC
     }
 }
 
+extension NIOHTTPClientUpgradeHandler: @unchecked Sendable {}
+
 extension NIOHTTPClientUpgradeHandler {
     /// The state of the upgrade handler.
     fileprivate enum UpgradeState {
