@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2018 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -12,7 +12,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+import NIOCore
 
 /// The length of the TLS record header in bytes.
 private let tlsRecordHeaderLength = 5
@@ -33,7 +33,7 @@ private let sniHostNameType: UInt8 = 0
 /// contains the hostname received in the SNI extension. If `fallback`,
 /// then either we could not parse the SNI extension or it was not there
 /// at all.
-public enum SNIResult: Equatable {
+public enum SNIResult: Equatable, Sendable {
     case fallback
     case hostname(String)
 }
@@ -101,7 +101,7 @@ public final class SNIHandler: ByteToMessageDecoder {
 
     private let completionHandler: (SNIResult) -> EventLoopFuture<Void>
     private var waitingForUser: Bool
-
+    
     public init(sniCompleteHandler: @escaping (SNIResult) -> EventLoopFuture<Void>) {
         self.cumulationBuffer = nil
         self.completionHandler = sniCompleteHandler
@@ -426,3 +426,6 @@ public final class SNIHandler: ByteToMessageDecoder {
         }
     }
 }
+
+@available(*, unavailable)
+extension SNIHandler: Sendable {}
