@@ -11,24 +11,37 @@
 // SPDX-License-Identifier: Apache-2.0
 //
 //===----------------------------------------------------------------------===//
-#if os(macOS) || os(iOS) || os(watchOS) || os(tvOS)
+#if canImport(Darwin)
 import Darwin
-#else
+#elseif canImport(Glibc)
 import Glibc
+#else
+#error("The Concurrency helpers test module was unable to identify your C library.")
 #endif
 import Dispatch
 import XCTest
-import NIO
+import NIOCore
 @testable import NIOConcurrencyHelpers
 
 class NIOConcurrencyHelpersTests: XCTestCase {
     private func sumOfIntegers(until n: UInt64) -> UInt64 {
         return n*(n+1)/2
     }
+    
+    #if canImport(Darwin)
+    let noAsyncs: UInt64 = 50
+    #else
+    /// `swift-corelibs-libdispatch` implementation of concurrent queues only initially spawn up to `System.coreCount` threads.
+    /// Afterwards they will create one thread per second.
+    /// Therefore this test takes `noAsyncs - System.coreCount` seconds to execute.
+    /// For example if `noAsyncs == 50` and `System.coreCount == 8` this test takes ~42 seconds to execute.
+    /// On non Darwin system we therefore limit the number of async operations to `System.coreCount`.
+    let noAsyncs: UInt64 = UInt64(System.coreCount)
+    #endif
 
     @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testLargeContendedAtomicSum() {
-        let noAsyncs: UInt64 = 50
+        
         let noCounts: UInt64 = 2_000
 
         let q = DispatchQueue(label: "q", attributes: .concurrent)
@@ -36,23 +49,23 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         let ai = NIOConcurrencyHelpers.Atomic<UInt64>(value: 0)
         let everybodyHere = DispatchSemaphore(value: 0)
         let go = DispatchSemaphore(value: 0)
-        for thread in 1...noAsyncs {
+        for thread in 1...self.noAsyncs {
             q.async(group: g) {
                 everybodyHere.signal()
                 go.wait()
                 for _ in 0..<noCounts {
-                    _ = ai.add(thread)
+                    ai.add(thread)
                 }
             }
         }
-        for _ in 0..<noAsyncs {
+        for _ in 0..<self.noAsyncs {
             everybodyHere.wait()
         }
-        for _ in 0..<noAsyncs {
+        for _ in 0..<self.noAsyncs {
             go.signal()
         }
         g.wait()
-        XCTAssertEqual(sumOfIntegers(until: noAsyncs) * noCounts, ai.load())
+        XCTAssertEqual(sumOfIntegers(until: self.noAsyncs) * noCounts, ai.load())
     }
 
     @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
@@ -239,8 +252,8 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         testFor(UInt.self)
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testLargeContendedNIOAtomicSum() {
-        let noAsyncs: UInt64 = 50
         let noCounts: UInt64 = 2_000
 
         let q = DispatchQueue(label: "q", attributes: .concurrent)
@@ -248,25 +261,26 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         let ai = NIOConcurrencyHelpers.NIOAtomic<UInt64>.makeAtomic(value: 0)
         let everybodyHere = DispatchSemaphore(value: 0)
         let go = DispatchSemaphore(value: 0)
-        for thread in 1...noAsyncs {
+        for thread in 1...self.noAsyncs {
             q.async(group: g) {
                 everybodyHere.signal()
                 go.wait()
                 for _ in 0..<noCounts {
-                    _ = ai.add(thread)
+                    ai.add(thread)
                 }
             }
         }
-        for _ in 0..<noAsyncs {
+        for _ in 0..<self.noAsyncs {
             everybodyHere.wait()
         }
-        for _ in 0..<noAsyncs {
+        for _ in 0..<self.noAsyncs {
             go.signal()
         }
         g.wait()
-        XCTAssertEqual(sumOfIntegers(until: noAsyncs) * noCounts, ai.load())
+        XCTAssertEqual(sumOfIntegers(until: self.noAsyncs) * noCounts, ai.load())
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testCompareAndExchangeBoolNIOAtomic() {
         let ab = NIOAtomic<Bool>.makeAtomic(value: true)
 
@@ -280,6 +294,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         XCTAssertTrue(ab.compareAndExchange(expected: false, desired: true))
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testAllOperationsBoolNIOAtomic() {
         let ab = NIOAtomic<Bool>.makeAtomic(value: false)
         XCTAssertEqual(false, ab.load())
@@ -296,6 +311,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         XCTAssertFalse(ab.compareAndExchange(expected: false, desired: true))
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testCompareAndExchangeUIntsNIOAtomic() {
         func testFor<T: NIOAtomicPrimitive & FixedWidthInteger & UnsignedInteger>(_ value: T.Type) {
             let zero: T = 0
@@ -326,6 +342,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         testFor(UInt.self)
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testCompareAndExchangeIntsNIOAtomic() {
         func testFor<T: NIOAtomicPrimitive & FixedWidthInteger & SignedInteger>(_ value: T.Type) {
             let zero: T = 0
@@ -357,6 +374,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         testFor(Int.self)
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testAddSubNIOAtomic() {
         func testFor<T: NIOAtomicPrimitive & FixedWidthInteger>(_ value: T.Type) {
             let zero: T = 0
@@ -395,6 +413,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         testFor(UInt.self)
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testExchangeNIOAtomic() {
         func testFor<T: NIOAtomicPrimitive & FixedWidthInteger>(_ value: T.Type) {
             let zero: T = 0
@@ -426,6 +445,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         testFor(UInt.self)
     }
 
+    @available(*, deprecated, message: "deprecated because it tests deprecated functionality")
     func testLoadStoreNIOAtomic() {
         func testFor<T: NIOAtomicPrimitive & FixedWidthInteger>(_ value: T.Type) {
             let zero: T = 0
@@ -453,7 +473,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
 
 
     func testLockMutualExclusion() {
-        let l = Lock()
+        let l = NIOLock()
 
         var x = 1
         let q = DispatchQueue(label: "q")
@@ -485,7 +505,7 @@ class NIOConcurrencyHelpersTests: XCTestCase {
     }
 
     func testWithLockMutualExclusion() {
-        let l = Lock()
+        let l = NIOLock()
 
         var x = 1
         let q = DispatchQueue(label: "q")
@@ -999,6 +1019,40 @@ class NIOConcurrencyHelpersTests: XCTestCase {
         doIt()
         XCTAssertEqual(iterations + 1, allDeallocations.load())
     }
+
+    func testNIOLockedValueBox() {
+        struct State {
+            var count: Int = 0
+        }
+
+        let lv = NIOLockedValueBox<State>(State())
+        spawnAndJoinRacingThreads(count: 50) { _ in
+            for _ in 0..<1000 {
+                lv.withLockedValue { state in
+                    state.count += 1
+                }
+            }
+        }
+
+        XCTAssertEqual(50_000, lv.withLockedValue { $0.count })
+    }
+
+    func testNIOLockedValueBoxHandlesThingsWithTransitiveClassesProperly() {
+        struct State {
+            var counts: [Int] = []
+        }
+
+        let lv = NIOLockedValueBox<State>(State())
+        spawnAndJoinRacingThreads(count: 50) { _ in
+            for i in 0..<1000 {
+                lv.withLockedValue { state in
+                    state.counts.append(i)
+                }
+            }
+        }
+
+        XCTAssertEqual(50_000, lv.withLockedValue { $0.counts.count })
+    }
 }
 
 func spawnAndJoinRacingThreads(count: Int, _ body: @escaping (Int) -> Void) {
@@ -1025,7 +1079,7 @@ func spawnAndJoinRacingThreads(count: Int, _ body: @escaping (Int) -> Void) {
     group.wait()
 }
 
-func assert(_ condition: @autoclosure () -> Bool, within time: TimeAmount, testInterval: TimeAmount? = nil, _ message: String = "condition not satisfied in time", file: StaticString = #file, line: UInt = #line) {
+func assert(_ condition: @autoclosure () -> Bool, within time: TimeAmount, testInterval: TimeAmount? = nil, _ message: String = "condition not satisfied in time", file: StaticString = #filePath, line: UInt = #line) {
     let testInterval = testInterval ?? TimeAmount.nanoseconds(time.nanoseconds / 5)
     let endTime = NIODeadline.now() + time
 
@@ -1039,6 +1093,7 @@ func assert(_ condition: @autoclosure () -> Bool, within time: TimeAmount, testI
     }
 }
 
+@available(*, deprecated, message: "deprecated because it is used to test deprecated functionality")
 fileprivate class IntHolderWithDeallocationTracking {
     private(set) var value: Int
     let allDeallocations: NIOAtomic<Int>
@@ -1050,6 +1105,6 @@ fileprivate class IntHolderWithDeallocationTracking {
 
     deinit {
         self.value = -1
-        _ = self.allDeallocations.add(1)
+        self.allDeallocations.add(1)
     }
 }
