@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2018 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -11,15 +11,10 @@
 // SPDX-License-Identifier: Apache-2.0
 //
 //===----------------------------------------------------------------------===//
-//
-//  HTTPServerPipelineHandlerTest.swift
-//  NIOHTTP1Tests
-//
-//  Created by Cory Benfield on 02/03/2018.
-//
 
 import XCTest
-import NIO
+import NIOCore
+import NIOEmbedded
 @testable import NIOHTTP1
 
 private final class ReadRecorder: ChannelInboundHandler {
@@ -97,6 +92,21 @@ private final class QuiesceEventRecorder: ChannelInboundHandler {
     }
 }
 
+// This handler drops close mode output. This is because EmbeddedChannel doesn't support it,
+// and tests here don't require that it does anything sensible.
+private final class CloseOutputSuppressor: ChannelOutboundHandler {
+    typealias OutboundIn = Any
+    typealias OutboundOut = Any
+
+    func close(context: ChannelHandlerContext, mode: CloseMode, promise: EventLoopPromise<Void>?) {
+        if mode == .output {
+            promise?.succeed()
+        } else {
+            context.close(mode: mode, promise: promise)
+        }
+    }
+}
+
 
 class HTTPServerPipelineHandlerTest: XCTestCase {
     var channel: EmbeddedChannel! = nil
@@ -115,12 +125,13 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         self.writeRecorder = WriteRecorder()
         self.pipelineHandler = HTTPServerPipelineHandler()
         self.quiesceEventRecorder = QuiesceEventRecorder()
-        XCTAssertNoThrow(try channel.pipeline.addHandler(self.readCounter).wait())
-        XCTAssertNoThrow(try channel.pipeline.addHandler(HTTPResponseEncoder()).wait())
-        XCTAssertNoThrow(try channel.pipeline.addHandler(self.writeRecorder).wait())
-        XCTAssertNoThrow(try channel.pipeline.addHandler(self.pipelineHandler).wait())
-        XCTAssertNoThrow(try channel.pipeline.addHandler(self.readRecorder).wait())
-        XCTAssertNoThrow(try channel.pipeline.addHandler(self.quiesceEventRecorder).wait())
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(CloseOutputSuppressor()))
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(self.readCounter))
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(HTTPResponseEncoder()))
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(self.writeRecorder))
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(self.pipelineHandler))
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(self.readRecorder))
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.addHandler(self.quiesceEventRecorder))
 
         self.requestHead = HTTPRequestHead(version: .http1_1, method: .GET, uri: "/path")
         self.requestHead.headers.add(name: "Host", value: "example.com")
@@ -742,7 +753,7 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
             var state: State = .errorExpected
 
             func errorCaught(context: ChannelHandlerContext, error: Error) {
-                XCTAssertEqual(HTTPParserError.headerOverflow, error as? HTTPParserError)
+                XCTAssertEqual(HTTPParserError.unknown, error as? HTTPParserError)
                 XCTAssertEqual(.errorExpected, self.state)
                 self.state = .done
             }
@@ -753,10 +764,10 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         }
 
         let handler = VerifyOrderHandler()
-        XCTAssertNoThrow(try self.channel.pipeline.addHandler(HTTPServerProtocolErrorHandler()).wait())
-        XCTAssertNoThrow(try self.channel.pipeline.addHandler(handler).wait())
+        XCTAssertNoThrow(try self.channel.pipeline.syncOperations.addHandler(HTTPServerProtocolErrorHandler()))
+        XCTAssertNoThrow(try self.channel.pipeline.syncOperations.addHandler(handler))
 
-        self.channel.pipeline.fireErrorCaught(HTTPParserError.headerOverflow)
+        self.channel.pipeline.fireErrorCaught(HTTPParserError.unknown)
 
         XCTAssertEqual(.done, handler.state)
     }
@@ -813,8 +824,8 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         }
 
         let handler = VerifyOrderHandler()
-        XCTAssertNoThrow(try self.channel.pipeline.addHandler(HTTPServerProtocolErrorHandler()).wait())
-        XCTAssertNoThrow(try self.channel.pipeline.addHandler(handler).wait())
+        XCTAssertNoThrow(try self.channel.pipeline.syncOperations.addHandler(HTTPServerProtocolErrorHandler()))
+        XCTAssertNoThrow(try self.channel.pipeline.syncOperations.addHandler(handler))
 
         XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(makeRequestHead(uri: "/one"))))
         XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
@@ -841,7 +852,7 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         XCTAssertEqual(self.readCounter.readCount, 1)
 
         // Remove the handler.
-        XCTAssertNoThrow(try channel.pipeline.removeHandler(self.pipelineHandler).wait())
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.removeHandler(self.pipelineHandler).wait())
 
         // This should have automatically triggered a call to read(), but only one.
         XCTAssertEqual(self.readCounter.readCount, 2)
@@ -870,7 +881,7 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         XCTAssertEqual(self.readCounter.readCount, 1)
 
         // Remove the handler.
-        XCTAssertNoThrow(try channel.pipeline.removeHandler(self.pipelineHandler).wait())
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.removeHandler(self.pipelineHandler).wait())
 
         // This should have automatically triggered a call to read(), but only one.
         XCTAssertEqual(self.readCounter.readCount, 2)
@@ -891,7 +902,7 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
                         .channelRead(HTTPServerRequestPart.end(nil))])
 
         // Remove the handler.
-        XCTAssertNoThrow(try channel.pipeline.removeHandler(self.pipelineHandler).wait())
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.removeHandler(self.pipelineHandler).wait())
 
         // The extra data should have been forwarded.
         XCTAssertEqual(self.readRecorder.reads,
@@ -934,7 +945,7 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         XCTAssertEqual(self.readCounter.readCount, 1)
 
         // Now remove the handler.
-        XCTAssertNoThrow(try channel.pipeline.removeHandler(self.pipelineHandler).wait())
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.removeHandler(self.pipelineHandler).wait())
 
         // Channel should be open, but the quiesce event should have fired, and read
         // shouldn't have been called as we aren't expecting more data.
@@ -970,7 +981,7 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         XCTAssertEqual(self.readCounter.readCount, 3)
 
         // Now remove the handler.
-        XCTAssertNoThrow(try channel.pipeline.removeHandler(self.pipelineHandler).wait())
+        XCTAssertNoThrow(try channel.pipeline.syncOperations.removeHandler(self.pipelineHandler).wait())
 
         // Channel should be open, but the quiesce event should have fired, and read
         // shouldn't be (as it passed through.
@@ -978,4 +989,244 @@ class HTTPServerPipelineHandlerTest: XCTestCase {
         XCTAssertEqual(self.quiesceEventRecorder.quiesceCount, 1)
         XCTAssertEqual(self.readCounter.readCount, 3)
     }
+    
+    func testServerCanRespondContinue() throws {
+        // Send in the first part of a request.
+        var expect100ContinueHead = self.requestHead!
+        expect100ContinueHead.headers.replaceOrAdd(name: "expect", value: "100-continue")
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(expect100ContinueHead)))
+
+        var continueResponse = self.responseHead
+        continueResponse!.status = .continue
+
+        // Now the server sends a continue response.
+        XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.head(continueResponse!)).wait())
+
+        // The client response completes.
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Now the server sends the final response.
+        XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.head(self.responseHead)).wait())
+        XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.end(nil)).wait())
+     }
+
+    func testServerCanRespondProcessingMultipleTimes() throws {
+         // Send in a request.
+         XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+         XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+         // We haven't completed our response, so no more reading
+         XCTAssertEqual(self.readCounter.readCount, 0)
+         self.channel.read()
+         XCTAssertEqual(self.readCounter.readCount, 0)
+
+         var processResponse: HTTPResponseHead = self.responseHead!
+         processResponse.status = .processing
+
+         // Now the server sends multiple processing responses.
+         XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.head(processResponse)).wait())
+
+         // We are processing... Reading not allowed
+         XCTAssertEqual(self.readCounter.readCount, 0)
+         self.channel.read()
+         XCTAssertEqual(self.readCounter.readCount, 0)
+
+         // Continue processing...
+         XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.head(processResponse)).wait())
+
+         // We are processing... Reading not allowed
+         XCTAssertEqual(self.readCounter.readCount, 0)
+         self.channel.read()
+         XCTAssertEqual(self.readCounter.readCount, 0)
+
+         // Continue processing...
+         XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.head(processResponse)).wait())
+
+         // We are processing... Reading not allowed
+         XCTAssertEqual(self.readCounter.readCount, 0)
+         self.channel.read()
+         XCTAssertEqual(self.readCounter.readCount, 0)
+
+         // Now send the actual response!
+         XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.head(self.responseHead)).wait())
+         XCTAssertNoThrow(try channel.writeAndFlush(HTTPServerResponsePart.end(nil)).wait())
+
+         // This should have triggered a read
+         XCTAssertEqual(self.readCounter.readCount, 1)
+    }
+
+    func testServerCloseOutputForcesReadsBackOn() throws {
+        // Send in a request
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Reads are blocked.
+        XCTAssertEqual(self.readCounter.readCount, 0)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 0)
+
+        XCTAssertEqual(self.readRecorder.reads,
+                       [.channelRead(HTTPServerRequestPart.head(self.requestHead)),
+                        .channelRead(HTTPServerRequestPart.end(nil))])
+
+        // Now the server sends close output
+        XCTAssertNoThrow(try channel.close(mode: .output).wait())
+
+        // This unblocked the read and further reads can continue.
+        XCTAssertEqual(self.readCounter.readCount, 1)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 2)
+    }
+
+    func testCloseOutputAlwaysAllowsReads() throws {
+        // Send in a request
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Reads are blocked.
+        XCTAssertEqual(self.readCounter.readCount, 0)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 0)
+
+        XCTAssertEqual(self.readRecorder.reads,
+                       [.channelRead(HTTPServerRequestPart.head(self.requestHead)),
+                        .channelRead(HTTPServerRequestPart.end(nil))])
+
+        // Now the server sends close output
+        XCTAssertNoThrow(try channel.close(mode: .output).wait())
+
+        // This unblocked the read and further reads can continue.
+        XCTAssertEqual(self.readCounter.readCount, 1)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 2)
+
+        // New requests can come in, but are dropped.
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Reads keep working.
+        XCTAssertEqual(self.readCounter.readCount, 2)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 3)
+
+        XCTAssertEqual(self.readRecorder.reads,
+                       [.channelRead(HTTPServerRequestPart.head(self.requestHead)),
+                        .channelRead(HTTPServerRequestPart.end(nil))])
+    }
+
+    func testCloseOutputFirstIsOkEvenIfItsABitWeird() throws {
+        // Server sends close output first
+        XCTAssertNoThrow(try channel.close(mode: .output).wait())
+
+        // Send in a request
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Reads are unblocked.
+        XCTAssertEqual(self.readCounter.readCount, 0)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 1)
+
+        // But the data is dropped.
+        XCTAssertEqual(self.readRecorder.reads, [])
+
+        // New requests can come in, and are dropped.
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Reads keep working.
+        XCTAssertEqual(self.readCounter.readCount, 1)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 2)
+
+        XCTAssertEqual(self.readRecorder.reads, [])
+    }
+
+    func testPipelinedRequestsAreDroppedWhenWeSendCloseOutput() throws {
+        // Send in three requests
+        for _ in 0..<3 {
+            XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+            XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+        }
+
+        // Reads are blocked and only one request was read.
+        XCTAssertEqual(self.readCounter.readCount, 0)
+        self.channel.read()
+        XCTAssertEqual(self.readCounter.readCount, 0)
+
+        XCTAssertEqual(self.readRecorder.reads,
+                       [.channelRead(HTTPServerRequestPart.head(self.requestHead)),
+                        .channelRead(HTTPServerRequestPart.end(nil))])
+
+        // Server sends close mode output. The buffered requests are dropped.
+        XCTAssertNoThrow(try channel.close(mode: .output).wait())
+
+        XCTAssertEqual(self.readRecorder.reads,
+                       [.channelRead(HTTPServerRequestPart.head(self.requestHead)),
+                        .channelRead(HTTPServerRequestPart.end(nil))])
+    }
+
+    func testWritesAfterCloseOutputAreDropped() throws {
+        // Send in a request
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.end(nil)))
+
+        // Server sends a head.
+        XCTAssertNoThrow(try self.channel.writeOutbound(HTTPServerResponsePart.head(self.responseHead)))
+
+        // Now the server sends close output
+        XCTAssertNoThrow(try channel.close(mode: .output).wait())
+
+        // Now, in error, the server sends .body and .end. Both pass unannounced.
+        XCTAssertNoThrow(try self.channel.writeOutbound(HTTPServerResponsePart.body(.byteBuffer(ByteBuffer()))))
+        XCTAssertNoThrow(try self.channel.writeOutbound(HTTPServerResponsePart.end(nil)))
+    }
+
+    func testSendingHeadTwiceGivesError() throws {
+        self.pipelineHandler.failOnPreconditions = false
+        // Sending a head once is normal
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+        // Sending a head twice is an error
+        XCTAssertThrowsError(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead))) { error in
+            XCTAssertEqual(error as? HTTPServerPipelineHandler.ConnectionStateError, .preconditionViolated(message: "received request head in state requestAndResponseEndPending"))
+        }
+    }
+
+    func testServerRespondToNothing() {
+        self.pipelineHandler.failOnPreconditions = false
+
+        // Writing an end whilst in state idle is an error
+        XCTAssertThrowsError(try self.channel.writeOutbound(HTTPServerResponsePart.end(nil))) { error in
+            XCTAssertEqual(error as? HTTPServerPipelineHandler.ConnectionStateError, .preconditionViolated(message: "Unexpectedly received a response in state idle"))
+        }
+
+        // Calling finish surfaces the error again
+        XCTAssertThrowsError(try self.channel.finish()) { error in
+            XCTAssertEqual(error as? HTTPServerPipelineHandler.ConnectionStateError, .preconditionViolated(message: "Unexpectedly received a response in state idle"))
+        }
+    }
+
+    func testServerRequestEndFirstIsError() {
+        self.pipelineHandler.failOnPreconditions = false
+        // End sending a request which was never started
+        XCTAssertThrowsError(try self.channel.writeInbound(HTTPServerRequestPart.end(nil))) { error in
+            XCTAssertEqual(error as? HTTPServerPipelineHandler.ConnectionStateError, .preconditionViolated(message: "Received second request"))
+        }
+    }
+
+    func testForcefulShutdownWhenViolatedPrecondition() {
+        self.pipelineHandler.failOnPreconditions = false
+
+        // End sending a request which was never started
+        XCTAssertThrowsError(try self.channel.writeInbound(HTTPServerRequestPart.end(nil))) { error in
+            XCTAssertEqual(error as? HTTPServerPipelineHandler.ConnectionStateError, .preconditionViolated(message: "Received second request"))
+        }
+        // The handler should now refuse further io, and forcefully shutdown
+        XCTAssertNoThrow(try self.channel.writeInbound(HTTPServerRequestPart.head(self.requestHead)))
+
+        self.channel.embeddedEventLoop.run()
+
+        // Ensure the channel is closed
+        XCTAssertNoThrow(try self.channel.closeFuture.wait())
+    }
 }
