@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2018 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -13,7 +13,8 @@
 //===----------------------------------------------------------------------===//
 
 import XCTest
-@testable import NIO
+@testable import NIOCore
+@testable import NIOPosix
 import NIOConcurrencyHelpers
 
 extension System {
@@ -25,9 +26,27 @@ extension System {
             return false
         }
     }
+
+    static var supportsVsock: Bool {
+#if canImport(Darwin) || os(Linux) || os(Android)
+        guard let socket = try? Socket(protocolFamily: .vsock, type: .stream) else { return false }
+        XCTAssertNoThrow(try socket.close())
+#if !canImport(Darwin)
+        do {
+            let fd = try Posix.open(file: "/dev/vsock", oFlag: O_RDONLY | O_CLOEXEC)
+            try Posix.close(descriptor: fd)
+        } catch {
+            return false
+        }
+#endif
+        return true
+#else
+        return false
+#endif
+    }
 }
 
-func withPipe(_ body: (NIO.NIOFileHandle, NIO.NIOFileHandle) throws -> [NIO.NIOFileHandle]) throws {
+func withPipe(_ body: (NIOCore.NIOFileHandle, NIOCore.NIOFileHandle) throws -> [NIOCore.NIOFileHandle]) throws {
     var fds: [Int32] = [-1, -1]
     fds.withUnsafeMutableBufferPointer { ptr in
         XCTAssertEqual(0, pipe(ptr.baseAddress!))
@@ -49,6 +68,29 @@ func withPipe(_ body: (NIO.NIOFileHandle, NIO.NIOFileHandle) throws -> [NIO.NIOF
     }
 }
 
+@available(macOS 10.15, iOS 13, tvOS 13, watchOS 6, *)
+func withPipe(_ body: (NIOCore.NIOFileHandle, NIOCore.NIOFileHandle) async throws -> [NIOCore.NIOFileHandle]) async throws {
+    var fds: [Int32] = [-1, -1]
+    fds.withUnsafeMutableBufferPointer { ptr in
+        XCTAssertEqual(0, pipe(ptr.baseAddress!))
+    }
+    let readFH = NIOFileHandle(descriptor: fds[0])
+    let writeFH = NIOFileHandle(descriptor: fds[1])
+    var toClose: [NIOFileHandle] = [readFH, writeFH]
+    var error: Error? = nil
+    do {
+        toClose = try await body(readFH, writeFH)
+    } catch let err {
+        error = err
+    }
+    try toClose.forEach { fh in
+        XCTAssertNoThrow(try fh.close())
+    }
+    if let error = error {
+        throw error
+    }
+}
+
 func withTemporaryDirectory<T>(_ body: (String) throws -> T) rethrows -> T {
     let dir = createTemporaryDirectory()
     defer {
@@ -57,6 +99,15 @@ func withTemporaryDirectory<T>(_ body: (String) throws -> T) rethrows -> T {
     return try body(dir)
 }
 
+@available(macOS 10.15, iOS 13, tvOS 13, watchOS 6, *)
+func withTemporaryDirectory<T>(_ body: (String) async throws -> T) async rethrows -> T {
+    let dir = createTemporaryDirectory()
+    defer {
+        try? FileManager.default.removeItem(atPath: dir)
+    }
+    return try await body(dir)
+}
+
 /// This function creates a filename that can be used for a temporary UNIX domain socket path.
 ///
 /// If the temporary directory is too long to store a UNIX domain socket path, it will `chdir` into the temporary
@@ -93,7 +144,7 @@ func withTemporaryUnixDomainSocketPathName<T>(directory: String = temporaryDirec
     return try body(shortEnoughPath)
 }
 
-func withTemporaryFile<T>(content: String? = nil, _ body: (NIO.NIOFileHandle, String) throws -> T) rethrows -> T {
+func withTemporaryFile<T>(content: String? = nil, _ body: (NIOCore.NIOFileHandle, String) throws -> T) rethrows -> T {
     let (fd, path) = openTemporaryFile()
     let fileHandle = NIOFileHandle(descriptor: fd)
     defer {
@@ -120,6 +171,35 @@ func withTemporaryFile<T>(content: String? = nil, _ body: (NIO.NIOFileHandle, St
     }
     return try body(fileHandle, path)
 }
+
+@available(macOS 10.15, iOS 13, tvOS 13, watchOS 6, *)
+func withTemporaryFile<T>(content: String? = nil, _ body: @escaping @Sendable (NIOCore.NIOFileHandle, String) async throws -> T) async rethrows -> T {
+    let (fd, path) = openTemporaryFile()
+    let fileHandle = NIOFileHandle(descriptor: fd)
+    defer {
+        XCTAssertNoThrow(try fileHandle.close())
+        XCTAssertEqual(0, unlink(path))
+    }
+    if let content = content {
+        try Array(content.utf8).withUnsafeBufferPointer { ptr in
+            var toWrite = ptr.count
+            var start = ptr.baseAddress!
+            while toWrite > 0 {
+                let res = try Posix.write(descriptor: fd, pointer: start, size: toWrite)
+                switch res {
+                case .processed(let written):
+                    toWrite -= written
+                    start = start + written
+                case .wouldBlock:
+                    XCTFail("unexpectedly got .wouldBlock from a file")
+                    continue
+                }
+            }
+            XCTAssertEqual(0, lseek(fd, 0, SEEK_SET))
+        }
+    }
+    return try await body(fileHandle, path)
+}
 var temporaryDirectory: String {
     get {
 #if targetEnvironment(simulator)
@@ -253,7 +333,7 @@ func assertSetGetOptionOnOpenAndClosed<Option: ChannelOption>(channel: Channel,
     }
 }
 
-func assertNoThrowWithValue<T>(_ body: @autoclosure () throws -> T, defaultValue: T? = nil, message: String? = nil, file: StaticString = #file, line: UInt = #line) throws -> T {
+func assertNoThrowWithValue<T>(_ body: @autoclosure () throws -> T, defaultValue: T? = nil, message: String? = nil, file: StaticString = #filePath, line: UInt = #line) throws -> T {
     do {
         return try body()
     } catch {
@@ -272,14 +352,12 @@ func resolverDebugInformation(eventLoop: EventLoop, host: String, previouslyRece
         case .unixDomainSocket(_):
             return "uds"
         case .v4(let sa):
-            var addr = sa.address
-            return addr.addressDescription()
+            return __testOnly_addressDescription(sa.address)
         case .v6(let sa):
-            var addr = sa.address
-            return addr.addressDescription()
+            return __testOnly_addressDescription(sa.address)
         }
     }
-    let res = GetaddrinfoResolver(loop: eventLoop, aiSocktype: .stream, aiProtocol: CInt(IPPROTO_TCP))
+    let res = GetaddrinfoResolver(loop: eventLoop, aiSocktype: .stream, aiProtocol: .tcp)
     let ipv6Results = try assertNoThrowWithValue(res.initiateAAAAQuery(host: host, port: 0).wait()).map(printSocketAddress)
     let ipv4Results = try assertNoThrowWithValue(res.initiateAQuery(host: host, port: 0).wait()).map(printSocketAddress)
 
@@ -292,7 +370,7 @@ func resolverDebugInformation(eventLoop: EventLoop, host: String, previouslyRece
     """
 }
 
-func assert(_ condition: @autoclosure () -> Bool, within time: TimeAmount, testInterval: TimeAmount? = nil, _ message: String = "condition not satisfied in time", file: StaticString = #file, line: UInt = #line) {
+func assert(_ condition: @autoclosure () -> Bool, within time: TimeAmount, testInterval: TimeAmount? = nil, _ message: String = "condition not satisfied in time", file: StaticString = #filePath, line: UInt = #line) {
     let testInterval = testInterval ?? TimeAmount.nanoseconds(time.nanoseconds / 5)
     let endTime = NIODeadline.now() + time
 
@@ -307,15 +385,15 @@ func assert(_ condition: @autoclosure () -> Bool, within time: TimeAmount, testI
 }
 
 func getBoolSocketOption(channel: Channel, level: NIOBSDSocket.OptionLevel, name: NIOBSDSocket.Option,
-                         file: StaticString = #file, line: UInt = #line) throws -> Bool {
+                         file: StaticString = #filePath, line: UInt = #line) throws -> Bool {
     return try assertNoThrowWithValue(channel.getOption(ChannelOptions.Types.SocketOption(level: level, name: name)), file: (file), line: line).wait() != 0
 }
 
-func assertSuccess<Value>(_ result: Result<Value, Error>, file: StaticString = #file, line: UInt = #line) {
+func assertSuccess<Value>(_ result: Result<Value, Error>, file: StaticString = #filePath, line: UInt = #line) {
     guard case .success = result else { return XCTFail("Expected result to be successful", file: (file), line: line) }
 }
 
-func assertFailure<Value>(_ result: Result<Value, Error>, file: StaticString = #file, line: UInt = #line) {
+func assertFailure<Value>(_ result: Result<Value, Error>, file: StaticString = #filePath, line: UInt = #line) {
     guard case .failure = result else { return XCTFail("Expected result to be a failure", file: (file), line: line) }
 }
 
@@ -488,7 +566,7 @@ final class FulfillOnFirstEventHandler: ChannelDuplexHandler {
     }
 }
 
-func forEachActiveChannelType<T>(file: StaticString = #file,
+func forEachActiveChannelType<T>(file: StaticString = #filePath,
                                  line: UInt = #line,
                                  _ body: @escaping (Channel) throws -> T) throws -> [T] {
     let group = MultiThreadedEventLoopGroup(numberOfThreads: 1)
@@ -497,7 +575,7 @@ func forEachActiveChannelType<T>(file: StaticString = #file,
     }
     let channelEL = group.next()
 
-    let lock = Lock()
+    let lock = NIOLock()
     var ret: [T] = []
     _ = try forEachCrossConnectedStreamChannelPair(file: (file), line: line) { (chan1: Channel, chan2: Channel) throws -> Void in
         var innerRet: [T] = [try body(chan1)]
@@ -528,7 +606,7 @@ func forEachActiveChannelType<T>(file: StaticString = #file,
 
 func withTCPServerChannel<R>(bindTarget: SocketAddress? = nil,
                              group: EventLoopGroup,
-                             file: StaticString = #file,
+                             file: StaticString = #filePath,
                              line: UInt = #line,
                              _ body: (Channel) throws -> R) throws -> R {
     let server = try ServerBootstrap(group: group)
@@ -547,7 +625,7 @@ func withTCPServerChannel<R>(bindTarget: SocketAddress? = nil,
 
 func withCrossConnectedSockAddrChannels<R>(bindTarget: SocketAddress,
                                            forceSeparateEventLoops: Bool = false,
-                                           file: StaticString = #file,
+                                           file: StaticString = #filePath,
                                            line: UInt = #line,
                                            _ body: (Channel, Channel) throws -> R) throws -> R {
     let serverGroup = MultiThreadedEventLoopGroup(numberOfThreads: 1)
@@ -597,7 +675,7 @@ func withCrossConnectedSockAddrChannels<R>(bindTarget: SocketAddress,
 }
 
 func withCrossConnectedTCPChannels<R>(forceSeparateEventLoops: Bool = false,
-                                      file: StaticString = #file,
+                                      file: StaticString = #filePath,
                                       line: UInt = #line,
                                       _ body: (Channel, Channel) throws -> R) throws -> R {
     return try withCrossConnectedSockAddrChannels(bindTarget: .init(ipAddress: "127.0.0.1", port: 0),
@@ -606,7 +684,7 @@ func withCrossConnectedTCPChannels<R>(forceSeparateEventLoops: Bool = false,
 }
 
 func withCrossConnectedUnixDomainSocketChannels<R>(forceSeparateEventLoops: Bool = false,
-                                                   file: StaticString = #file,
+                                                   file: StaticString = #filePath,
                                                    line: UInt = #line,
                                                    _ body: (Channel, Channel) throws -> R) throws -> R {
     return try withTemporaryDirectory { tempDir in
@@ -618,7 +696,7 @@ func withCrossConnectedUnixDomainSocketChannels<R>(forceSeparateEventLoops: Bool
 }
 
 func withCrossConnectedPipeChannels<R>(forceSeparateEventLoops: Bool = false,
-                                       file: StaticString = #file,
+                                       file: StaticString = #filePath,
                                        line: UInt = #line,
                                        _ body: (Channel, Channel) throws -> R) throws -> R {
     let channel1Group = MultiThreadedEventLoopGroup(numberOfThreads: 1)
@@ -645,13 +723,13 @@ func withCrossConnectedPipeChannels<R>(forceSeparateEventLoops: Bool = false,
                     try pipe2Read.withUnsafeFileDescriptor { pipe2Read in
                         try pipe2Write.withUnsafeFileDescriptor { pipe2Write in
                             let channel1 = try NIOPipeBootstrap(group: channel1Group)
-                                .withPipes(inputDescriptor: pipe1Read, outputDescriptor: pipe2Write)
+                                .takingOwnershipOfDescriptors(input: pipe1Read, output: pipe2Write)
                                 .wait()
                             defer {
                                 XCTAssertNoThrow(try channel1.syncCloseAcceptingAlreadyClosed())
                             }
                             let channel2 = try NIOPipeBootstrap(group: channel2Group)
-                                .withPipes(inputDescriptor: pipe2Read, outputDescriptor: pipe1Write)
+                                .takingOwnershipOfDescriptors(input: pipe2Read, output: pipe1Write)
                                 .wait()
                             defer {
                                 XCTAssertNoThrow(try channel2.syncCloseAcceptingAlreadyClosed())
@@ -673,7 +751,7 @@ func withCrossConnectedPipeChannels<R>(forceSeparateEventLoops: Bool = false,
 }
 
 func forEachCrossConnectedStreamChannelPair<R>(forceSeparateEventLoops: Bool = false,
-                                               file: StaticString = #file,
+                                               file: StaticString = #filePath,
                                                line: UInt = #line,
                                                _ body: (Channel, Channel) throws -> R) throws -> [R] {
     let r1 = try withCrossConnectedTCPChannels(forceSeparateEventLoops: forceSeparateEventLoops, body)
@@ -693,7 +771,7 @@ extension EventLoopFuture {
             }
             return fulfilled
         } else {
-            let lock = Lock()
+            let lock = NIOLock()
             let group = DispatchGroup()
             var fulfilled = false // protected by lock
 
