@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2019 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2019-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -12,7 +12,8 @@
 //
 //===----------------------------------------------------------------------===//
 
-import NIO
+import NIOCore
+import NIOPosix
 import NIOHTTP1
 import NIOTestUtils
 import XCTest
@@ -312,6 +313,43 @@ class NIOHTTP1TestServerTest: XCTestCase {
         XCTAssertNotNil(channel)
         XCTAssertNoThrow(try channel.closeFuture.wait())
     }
+
+    func testReceiveBodyWithoutAggregation() {
+        let testServer = NIOHTTP1TestServer(group: self.group, aggregateBody: false)
+
+        let responsePromise = self.group.next().makePromise(of: String.self)
+        var channel: Channel!
+        XCTAssertNoThrow(channel = try self.connect(serverPort: testServer.serverPort,
+                                                    responsePromise: responsePromise).wait())
+
+        var headers = HTTPHeaders()
+        headers.add(name: "Content-Type", value: "text/plain; charset=utf-8")
+        let requestHead = HTTPRequestHead(version: .http1_1, method: .POST, uri: "/uri", headers: headers)
+        channel.writeAndFlush(NIOAny(HTTPClientRequestPart.head(requestHead)), promise: nil)
+        XCTAssertNoThrow(try testServer.receiveHeadAndVerify { head in
+            XCTAssertEqual(head.uri, "/uri")
+            XCTAssertEqual(head.headers["Content-Type"], ["text/plain; charset=utf-8"])
+        })
+        XCTAssertNoThrow(try testServer.writeOutbound(.head(.init(version: .http1_1, status: .ok))))
+
+        for _ in 0..<10 {
+            channel.writeAndFlush(NIOAny(HTTPClientRequestPart.body(.byteBuffer(ByteBuffer(string: "ping")))), promise: nil)
+            XCTAssertNoThrow(try testServer.receiveBodyAndVerify { buffer in
+                XCTAssertEqual(String(buffer: buffer), "ping")
+            })
+            XCTAssertNoThrow(try testServer.writeOutbound(.body(.byteBuffer(ByteBuffer(string: "pong")))))
+        }
+
+        channel.writeAndFlush(NIOAny(HTTPClientRequestPart.end(nil)), promise: nil)
+        XCTAssertNoThrow(try testServer.receiveEndAndVerify { trailers in
+            XCTAssertNil(trailers)
+        })
+        XCTAssertNoThrow(try testServer.writeOutbound(.end(nil)))
+
+        XCTAssertNoThrow(try testServer.stop())
+        XCTAssertNotNil(channel)
+        XCTAssertNoThrow(try channel.closeFuture.wait())
+    }
 }
 
 private final class TestHTTPHandler: ChannelInboundHandler {
@@ -352,7 +390,7 @@ private final class TestHTTPHandler: ChannelInboundHandler {
 }
 
 extension HTTPServerRequestPart {
-    func assertHead(expectedURI: String, file: StaticString = #file, line: UInt = #line) {
+    func assertHead(expectedURI: String, file: StaticString = #filePath, line: UInt = #line) {
         switch self {
         case .head(let head):
             XCTAssertEqual(.GET, head.method)
@@ -363,7 +401,7 @@ extension HTTPServerRequestPart {
         }
     }
 
-    func assertBody(expectedMessage: String, file: StaticString = #file, line: UInt = #line) {
+    func assertBody(expectedMessage: String, file: StaticString = #filePath, line: UInt = #line) {
         switch self {
         case .body(let buffer):
             // Note that the test server coalesces the body parts for us.
@@ -374,7 +412,7 @@ extension HTTPServerRequestPart {
         }
     }
 
-    func assertEnd(file: StaticString = #file, line: UInt = #line) {
+    func assertEnd(file: StaticString = #filePath, line: UInt = #line) {
         switch self {
         case .end(_):
             ()
@@ -414,7 +452,7 @@ func assert(_ condition: @autoclosure () -> Bool,
             within time: TimeAmount,
             testInterval: TimeAmount? = nil,
             _ message: String = "condition not satisfied in time",
-            file: StaticString = #file, line: UInt = #line) {
+            file: StaticString = #filePath, line: UInt = #line) {
     let testInterval = testInterval ?? TimeAmount.nanoseconds(time.nanoseconds / 5)
     let endTime = NIODeadline.now() + time
 
@@ -431,7 +469,7 @@ func assert(_ condition: @autoclosure () -> Bool,
 func assertNoThrowWithValue<T>(_ body: @autoclosure () throws -> T,
                                defaultValue: T? = nil,
                                message: String? = nil,
-                               file: StaticString = #file,
+                               file: StaticString = #filePath,
                                line: UInt = #line) throws -> T {
     do {
         return try body()
