@@ -2,7 +2,7 @@
 //
 // This source file is part of the SwiftNIO open source project
 //
-// Copyright (c) 2017-2019 Apple Inc. and the SwiftNIO project authors
+// Copyright (c) 2017-2021 Apple Inc. and the SwiftNIO project authors
 // Licensed under Apache License v2.0
 //
 // See LICENSE.txt for license information
@@ -13,7 +13,8 @@
 //===----------------------------------------------------------------------===//
 
 import XCTest
-@testable import NIO
+@testable import NIOCore
+import NIOEmbedded
 import NIOHTTP1
 @testable import NIOWebSocket
 
@@ -111,12 +112,41 @@ private class WebSocketRecorderHandler: ChannelInboundHandler {
     }
 }
 
+struct WebSocketServerUpgraderConfiguration {
+    let maxFrameSize: Int
+    let automaticErrorHandling: Bool
+    let shouldUpgrade: @Sendable (Channel, HTTPRequestHead) -> EventLoopFuture<HTTPHeaders?>
+    let upgradePipelineHandler: @Sendable (Channel, HTTPRequestHead) -> EventLoopFuture<Void>
+
+    @preconcurrency
+    init(
+        maxFrameSize: Int = 1 << 14,
+        automaticErrorHandling: Bool = true,
+        shouldUpgrade: @escaping @Sendable (Channel, HTTPRequestHead) -> EventLoopFuture<HTTPHeaders?>,
+        upgradePipelineHandler: @escaping @Sendable (Channel, HTTPRequestHead) -> EventLoopFuture<Void>
+    ) {
+        self.maxFrameSize = maxFrameSize
+        self.automaticErrorHandling = automaticErrorHandling
+        self.shouldUpgrade = shouldUpgrade
+        self.upgradePipelineHandler = upgradePipelineHandler
+    }
+}
+
 class WebSocketServerEndToEndTests: XCTestCase {
-    private func createTestFixtures(upgraders: [NIOWebSocketServerUpgrader]) -> (loop: EmbeddedEventLoop, serverChannel: EmbeddedChannel, clientChannel: EmbeddedChannel) {
+    func createTestFixtures(
+        upgraders: [WebSocketServerUpgraderConfiguration]
+    ) -> (loop: EmbeddedEventLoop, serverChannel: EmbeddedChannel, clientChannel: EmbeddedChannel) {
         let loop = EmbeddedEventLoop()
         let serverChannel = EmbeddedChannel(loop: loop)
-        let upgradeConfig = (upgraders: upgraders as [HTTPServerProtocolUpgrader], completionHandler: { (context: ChannelHandlerContext) in } )
-        XCTAssertNoThrow(try serverChannel.pipeline.configureHTTPServerPipeline(withServerUpgrade: upgradeConfig).wait())
+        let upgraders = upgraders.map { NIOWebSocketServerUpgrader(
+            maxFrameSize: $0.maxFrameSize,
+            automaticErrorHandling: $0.automaticErrorHandling,
+            shouldUpgrade: $0.shouldUpgrade,
+            upgradePipelineHandler: $0.upgradePipelineHandler
+        )}
+        XCTAssertNoThrow(try serverChannel.pipeline.configureHTTPServerPipeline(
+            withServerUpgrade: (upgraders: upgraders as [HTTPServerProtocolUpgrader], completionHandler: { (context: ChannelHandlerContext) in } )
+        ).wait())
         let clientChannel = EmbeddedChannel(loop: loop)
         return (loop: loop, serverChannel: serverChannel, clientChannel: clientChannel)
     }
@@ -127,7 +157,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testBasicUpgradeDance() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in channel.eventLoop.makeSucceededFuture(()) })
         let (loop, server, client) = self.createTestFixtures(upgraders: [basicUpgrader])
         defer {
@@ -149,7 +179,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testUpgradeWithProtocolName() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in channel.eventLoop.makeSucceededFuture(()) })
         let (loop, server, client) = self.createTestFixtures(upgraders: [basicUpgrader])
         defer {
@@ -168,7 +198,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testCanRejectUpgrade() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(nil) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(nil) },
                                               upgradePipelineHandler: { (channel, req) in
                                                   XCTFail("Should not have called")
                                                   return channel.eventLoop.makeSucceededFuture(())
@@ -199,7 +229,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
         var acceptPromise: EventLoopPromise<HTTPHeaders?>? = nil
         var upgradeComplete = false
 
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in
                                                   acceptPromise = channel.eventLoop.makePromise()
                                                   return acceptPromise!.futureResult
                                               },
@@ -238,7 +268,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testRequiresVersion13() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in channel.eventLoop.makeSucceededFuture(()) })
         let (loop, server, client) = self.createTestFixtures(upgraders: [basicUpgrader])
         defer {
@@ -260,7 +290,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testRequiresVersionHeader() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in channel.eventLoop.makeSucceededFuture(()) })
         let (loop, server, client) = self.createTestFixtures(upgraders: [basicUpgrader])
         defer {
@@ -282,7 +312,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testRequiresKeyHeader() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in channel.eventLoop.makeSucceededFuture(()) })
         let (loop, server, client) = self.createTestFixtures(upgraders: [basicUpgrader])
         defer {
@@ -304,7 +334,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testUpgradeMayAddCustomHeaders() throws {
-        let upgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in
+        let upgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in
                                             var hdrs = HTTPHeaders()
                                             hdrs.add(name: "TestHeader", value: "TestValue")
                                             return channel.eventLoop.makeSucceededFuture(hdrs)
@@ -327,8 +357,8 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testMayRegisterMultipleWebSocketEndpoints() throws {
-        func buildHandler(path: String) -> NIOWebSocketServerUpgrader {
-            return NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in
+        func buildHandler(path: String) -> WebSocketServerUpgraderConfiguration {
+            return WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in
                                          guard head.uri == "/\(path)" else { return channel.eventLoop.makeSucceededFuture(nil) }
                                          var hdrs = HTTPHeaders()
                                          hdrs.add(name: "Target", value: path)
@@ -358,7 +388,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
 
     func testSendAFewFrames() throws {
         let recorder = WebSocketRecorderHandler()
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in
                                                 channel.pipeline.addHandler(recorder)
 
@@ -379,7 +409,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
                                           expectedResponseHeaders: ["Upgrade: websocket", "Sec-WebSocket-Accept: OfS0wDaT5NoxF2gqm7Zj2YtetzM=", "Connection: upgrade"]))
 
         // Put a frame encoder in the client pipeline.
-        XCTAssertNoThrow(try client.pipeline.addHandler(WebSocketFrameEncoder()).wait())
+        XCTAssertNoThrow(try client.pipeline.syncOperations.addHandler(WebSocketFrameEncoder()))
 
         var data = client.allocator.buffer(capacity: 12)
         data.writeString("hello, world")
@@ -396,7 +426,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
     }
 
     func testMaxFrameSize() throws {
-        let basicUpgrader = NIOWebSocketServerUpgrader(maxFrameSize: 16, shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(maxFrameSize: 16, shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in
             return channel.eventLoop.makeSucceededFuture(())
         })
@@ -415,13 +445,13 @@ class WebSocketServerEndToEndTests: XCTestCase {
                                           expectedResponseLine: "HTTP/1.1 101 Switching Protocols",
                                           expectedResponseHeaders: ["Upgrade: websocket", "Sec-WebSocket-Accept: OfS0wDaT5NoxF2gqm7Zj2YtetzM=", "Connection: upgrade"]))
 
-        let decoder = ((try server.pipeline.context(handlerType: ByteToMessageHandler<WebSocketFrameDecoder>.self).wait()).handler as! ByteToMessageHandler<WebSocketFrameDecoder>).decoder
+        let decoder = ((try server.pipeline.syncOperations.context(handlerType: ByteToMessageHandler<WebSocketFrameDecoder>.self)).handler as! ByteToMessageHandler<WebSocketFrameDecoder>).decoder
         XCTAssertEqual(16, decoder?.maxFrameSize)
     }
 
     func testAutomaticErrorHandling() throws {
         let recorder = WebSocketRecorderHandler()
-        let basicUpgrader = NIOWebSocketServerUpgrader(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in
                                                 channel.pipeline.addHandler(recorder)
 
@@ -459,7 +489,7 @@ class WebSocketServerEndToEndTests: XCTestCase {
 
     func testNoAutomaticErrorHandling() throws {
         let recorder = WebSocketRecorderHandler()
-        let basicUpgrader = NIOWebSocketServerUpgrader(automaticErrorHandling: false,
+        let basicUpgrader = WebSocketServerUpgraderConfiguration(automaticErrorHandling: false,
                                               shouldUpgrade: { (channel, head) in channel.eventLoop.makeSucceededFuture(HTTPHeaders()) },
                                               upgradePipelineHandler: { (channel, req) in
                                                 channel.pipeline.addHandler(recorder)
@@ -496,3 +526,32 @@ class WebSocketServerEndToEndTests: XCTestCase {
         XCTAssertNoThrow(XCTAssertEqual([], try server.readAllOutboundBytes()))
     }
 }
+
+#if !canImport(Darwin) || swift(>=5.10)
+@available(macOS 13, iOS 16, tvOS 16, watchOS 9, *)
+final class TypedWebSocketServerEndToEndTests: WebSocketServerEndToEndTests {
+    override func createTestFixtures(
+        upgraders: [WebSocketServerUpgraderConfiguration]
+    ) -> (loop: EmbeddedEventLoop, serverChannel: EmbeddedChannel, clientChannel: EmbeddedChannel) {
+        let loop = EmbeddedEventLoop()
+        let serverChannel = EmbeddedChannel(loop: loop)
+        let upgraders = upgraders.map { NIOTypedWebSocketServerUpgrader(
+            maxFrameSize: $0.maxFrameSize,
+            enableAutomaticErrorHandling: $0.automaticErrorHandling,
+            shouldUpgrade: $0.shouldUpgrade,
+            upgradePipelineHandler: $0.upgradePipelineHandler
+        )}
+
+        XCTAssertNoThrow(try serverChannel.pipeline.syncOperations.configureUpgradableHTTPServerPipeline(
+            configuration: .init(
+                upgradeConfiguration: NIOTypedHTTPServerUpgradeConfiguration<Void>(
+                    upgraders: upgraders,
+                    notUpgradingCompletionHandler: { $0.eventLoop.makeSucceededVoidFuture() }
+                )
+            )
+        ))
+        let clientChannel = EmbeddedChannel(loop: loop)
+        return (loop: loop, serverChannel: serverChannel, clientChannel: clientChannel)
+    }
+}
+#endif
