@@ -894,7 +894,7 @@ extension UInt64 {
 
     init? (eightBytesStorage: BytesStorage) {
         if !eightBytesStorage.has(8) { return nil }
-		var up_byte = UnsafeMutablePointer<UInt8>.allocate(capacity: 8)
+		let up_byte = UnsafeMutablePointer<UInt8>.allocate(capacity: 8)
         defer { up_byte.deallocate() }
         for idx in 0..<8 {
             up_byte[idx] = eightBytesStorage.get()
@@ -958,7 +958,7 @@ extension Bool {
 
 extension Float {
     init? (storage: BytesStorage) {
-        var ubPtr = UnsafeMutablePointer<UInt8>.allocate(capacity: 4)
+        let ubPtr = UnsafeMutablePointer<UInt8>.allocate(capacity: 4)
         defer { ubPtr.deallocate() }
         for idx in 0..<4 {
             ubPtr[idx] = storage.get()
@@ -973,7 +973,7 @@ extension Float {
 
 extension Double {
     init? (storage: BytesStorage) {
-        var ubPtr = UnsafeMutablePointer<UInt8>.allocate(capacity: 8)
+        let ubPtr = UnsafeMutablePointer<UInt8>.allocate(capacity: 8)
         defer { ubPtr.deallocate() }
         for idx in 0..<8 {
             ubPtr[idx] = storage.get()
