@@ -7,5 +7,6 @@ let package = Package(
         .executable(name: "secho", targets: ["secho"])
     ],
     targets: [
-        .target(name: "secho", dependencies: [])
-    ])
\ No newline at end of file
+        .target(name: "secho", dependencies: []),
+        .testTarget(name: "TestSuite")
+    ])
