@@ -2,6 +2,8 @@
 	import Glibc
 #elseif canImport(Musl)
 	import Musl
+#elseif canImport(Android)
+	import Android
 #else
 	import Darwin.C
 #endif
