@@ -1,6 +1,12 @@
 import Foundation
 import PackagePlugin
 
+#if os(Android)
+let touchExe = "/system/bin/touch"
+#else
+let touchExe = "/usr/bin/touch"
+#endif
+
 @main
 struct MyPlugin: BuildToolPlugin {
     
@@ -16,7 +22,7 @@ struct MyPlugin: BuildToolPlugin {
                 displayName:
                     "Running prebuild command for target \(target.name)",
                 executable:
-                    URL(fileURLWithPath: "/usr/bin/touch"),
+                    URL(fileURLWithPath: touchExe),
                 arguments: 
                     outputPaths.map{ $0.path },
                 outputFilesDirectory:
