@@ -12,26 +12,34 @@
 //
 //===----------------------------------------------------------------------===//
 
-import PackageDescription
 import class Foundation.ProcessInfo
+import PackageDescription
 
 // When building the toolchain on the CI for ELF platforms, remove the CI's
 // stdlib absolute runpath and add ELF's $ORIGIN relative paths before installing.
-let swiftpmLinkSettings : [LinkerSetting]
-let packageLibraryLinkSettings : [LinkerSetting]
+let swiftpmLinkSettings: [LinkerSetting]
+let packageLibraryLinkSettings: [LinkerSetting]
 if let resourceDirPath = ProcessInfo.processInfo.environment["SWIFTCI_INSTALL_RPATH_OS"] {
-  swiftpmLinkSettings = [ .unsafeFlags(["-no-toolchain-stdlib-rpath", "-Xlinker", "-rpath", "-Xlinker", "$ORIGIN/../lib/swift/\(resourceDirPath)"]) ]
-  packageLibraryLinkSettings = [ .unsafeFlags(["-no-toolchain-stdlib-rpath", "-Xlinker", "-rpath", "-Xlinker", "$ORIGIN/../../\(resourceDirPath)"]) ]
+    swiftpmLinkSettings = [.unsafeFlags([
+        "-no-toolchain-stdlib-rpath",
+        "-Xlinker", "-rpath",
+        "-Xlinker", "$ORIGIN/../lib/swift/\(resourceDirPath)",
+    ])]
+    packageLibraryLinkSettings = [.unsafeFlags([
+        "-no-toolchain-stdlib-rpath",
+        "-Xlinker", "-rpath",
+        "-Xlinker", "$ORIGIN/../../\(resourceDirPath)",
+    ])]
 } else {
-  swiftpmLinkSettings = []
-  packageLibraryLinkSettings = []
+    swiftpmLinkSettings = []
+    packageLibraryLinkSettings = []
 }
 
 /** SwiftPMDataModel is the subset of SwiftPM product that includes just its data model.
-This allows some clients (such as IDEs) that use SwiftPM's data model but not its build system
-to not have to depend on SwiftDriver, SwiftLLBuild, etc. We should probably have better names here,
-though that could break some clients.
-*/
+ This allows some clients (such as IDEs) that use SwiftPM's data model but not its build system
+ to not have to depend on SwiftDriver, SwiftLLBuild, etc. We should probably have better names here,
+ though that could break some clients.
+ */
 let swiftPMDataModelProduct = (
     name: "SwiftPMDataModel",
     targets: [
@@ -52,7 +60,7 @@ let swiftPMDataModelProduct = (
  command line tools, while `libSwiftPMDataModel` includes only the data model.
 
  NOTE: This API is *unstable* and may change at any time.
-*/
+ */
 let swiftPMProduct = (
     name: "SwiftPM",
     targets: swiftPMDataModelProduct.targets + [
@@ -64,36 +72,44 @@ let swiftPMProduct = (
 )
 
 #if os(Windows)
+let includeDynamicLibrary: Bool = false
 let systemSQLitePkgConfig: String? = nil
 #else
-let systemSQLitePkgConfig: String? = "sqlite3"
+let includeDynamicLibrary: Bool = true
+var systemSQLitePkgConfig: String? = "sqlite3"
+if ProcessInfo.processInfo.environment["SWIFTCI_INSTALL_RPATH_OS"] == "android" {
+    systemSQLitePkgConfig = nil
+}
 #endif
 
 /** An array of products which have two versions listed: one dynamically linked, the other with the
-automatic linking type with `-auto` suffix appended to product's name.
-*/
+ automatic linking type with `-auto` suffix appended to product's name.
+ */
 let autoProducts = [swiftPMProduct, swiftPMDataModelProduct]
 
 let package = Package(
     name: "SwiftPM",
     platforms: [
         .macOS(.v13),
-        .iOS(.v16)
+        .iOS(.v16),
     ],
     products:
-        autoProducts.flatMap {
-          [
+    autoProducts.flatMap {
+        (includeDynamicLibrary ? [
             .library(
                 name: $0.name,
                 type: .dynamic,
                 targets: $0.targets
             ),
+        ] : [])
+        +
+        [
             .library(
                 name: "\($0.name)-auto",
                 targets: $0.targets
-            )
-          ]
-        } + [
+            ),
+        ]
+    } + [
         .library(
             name: "XCBuildSupport",
             targets: ["XCBuildSupport"]
@@ -153,12 +169,17 @@ let package = Package(
         .target(
             name: "SourceKitLSPAPI",
             dependencies: [
+                "Basics",
                 "Build",
-                "SPMBuildCore"
+                "PackageGraph",
+                "PackageLoading",
+                "PackageModel",
+                "SPMBuildCore",
             ],
             exclude: ["CMakeLists.txt"],
             swiftSettings: [
                 .enableExperimentalFeature("AccessLevelOnImport"),
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -166,10 +187,26 @@ let package = Package(
 
         .systemLibrary(name: "SPMSQLite3", pkgConfig: systemSQLitePkgConfig),
 
+        .target(
+            name: "_AsyncFileSystem",
+            dependencies: [
+                .product(name: "SystemPackage", package: "swift-system"),
+            ],
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .enableExperimentalFeature("StrictConcurrency"),
+                .enableExperimentalFeature("AccessLevelOnImport"),
+                .enableExperimentalFeature("InternalImportsByDefault"),
+                .unsafeFlags(["-static"]),
+            ]
+        ),
+
         .target(
             name: "Basics",
             dependencies: [
-                "SPMSQLite3",
+                "_AsyncFileSystem",
+                .target(name: "SPMSQLite3", condition: .when(platforms: [.macOS, .iOS, .tvOS, .watchOS, .visionOS, .macCatalyst, .linux])),
+                .product(name: "SwiftToolchainCSQLite", package: "swift-toolchain-sqlite", condition: .when(platforms: [.windows, .android])),
                 .product(name: "DequeModule", package: "swift-collections"),
                 .product(name: "OrderedCollections", package: "swift-collections"),
                 .product(name: "SwiftToolsSupport-auto", package: "swift-tools-support-core"),
@@ -179,6 +216,7 @@ let package = Package(
             swiftSettings: [
                 .enableExperimentalFeature("StrictConcurrency"),
                 .enableExperimentalFeature("AccessLevelOnImport"),
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -186,7 +224,10 @@ let package = Package(
             /** The llbuild manifest model */
             name: "LLBuildManifest",
             dependencies: ["Basics"],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -199,7 +240,10 @@ let package = Package(
                 "PackageModel",
                 "PackageSigning",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -207,16 +251,22 @@ let package = Package(
             name: "SourceControl",
             dependencies: [
                 "Basics",
-                "PackageModel"
+                "PackageModel",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
             /** Shim for llbuild library */
             name: "SPMLLBuild",
             dependencies: ["Basics"],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         // MARK: Project Model
@@ -225,7 +275,10 @@ let package = Package(
             /** Primitive Package model objects */
             name: "PackageModel",
             dependencies: ["Basics"],
-            exclude: ["CMakeLists.txt", "README.md"]
+            exclude: ["CMakeLists.txt", "README.md"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -235,14 +288,11 @@ let package = Package(
                 "Basics",
                 "PackageLoading",
                 "PackageModel",
-                .product(name: "SwiftBasicFormat", package: "swift-syntax"),
-                .product(name: "SwiftDiagnostics", package: "swift-syntax"),
-                .product(name: "SwiftIDEUtils", package: "swift-syntax"),
-                .product(name: "SwiftParser", package: "swift-syntax"),
-                .product(name: "SwiftSyntax", package: "swift-syntax"),
-                .product(name: "SwiftSyntaxBuilder", package: "swift-syntax"),
-            ],
-            exclude: ["CMakeLists.txt"]
+            ] + swiftSyntaxDependencies(["SwiftBasicFormat", "SwiftDiagnostics", "SwiftIDEUtils", "SwiftParser", "SwiftSyntax", "SwiftSyntaxBuilder"]),
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -253,7 +303,10 @@ let package = Package(
                 "PackageModel",
                 "SourceControl",
             ],
-            exclude: ["CMakeLists.txt", "README.md"]
+            exclude: ["CMakeLists.txt", "README.md"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         // MARK: Package Dependency Resolution
@@ -264,9 +317,12 @@ let package = Package(
             dependencies: [
                 "Basics",
                 "PackageLoading",
-                "PackageModel"
+                "PackageModel",
             ],
-            exclude: ["CMakeLists.txt", "README.md"]
+            exclude: ["CMakeLists.txt", "README.md"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         // MARK: Package Collections
@@ -276,7 +332,10 @@ let package = Package(
             name: "PackageCollectionsModel",
             dependencies: [],
             exclude: [
-                "Formats/v1.md"
+                "Formats/v1.md",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -289,6 +348,9 @@ let package = Package(
                 "PackageCollectionsSigning",
                 "PackageModel",
                 "SourceControl",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -299,6 +361,9 @@ let package = Package(
                 .product(name: "X509", package: "swift-certificates"),
                 "Basics",
                 "PackageCollectionsModel",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -308,9 +373,12 @@ let package = Package(
                 "Basics",
                 "PackageModel",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
-        
+
         .target(
             name: "PackageSigning",
             dependencies: [
@@ -319,7 +387,10 @@ let package = Package(
                 "Basics",
                 "PackageModel",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         // MARK: Package Manager Functionality
@@ -329,9 +400,12 @@ let package = Package(
             name: "SPMBuildCore",
             dependencies: [
                 "Basics",
-                "PackageGraph"
+                "PackageGraph",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
         .target(
             /** Builds Modules and Products */
@@ -345,7 +419,10 @@ let package = Package(
                 .product(name: "SwiftDriver", package: "swift-driver"),
                 "DriverSupport",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
         .target(
             name: "DriverSupport",
@@ -354,13 +431,19 @@ let package = Package(
                 "PackageModel",
                 .product(name: "SwiftDriver", package: "swift-driver"),
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
         .target(
             /** Support for building using Xcode's build system */
             name: "XCBuildSupport",
             dependencies: ["DriverSupport", "SPMBuildCore", "PackageGraph"],
-            exclude: ["CMakeLists.txt", "CODEOWNERS"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
         .target(
             /** High level functionality */
@@ -375,7 +458,10 @@ let package = Package(
                 "SourceControl",
                 "SPMBuildCore",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
         .target(
             // ** High level interface for package discovery */
@@ -386,6 +472,9 @@ let package = Package(
                 "PackageModel",
                 "PackageRegistry",
                 "PackageSigning",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -404,7 +493,10 @@ let package = Package(
                 "Workspace",
                 "XCBuildSupport",
             ],
-            exclude: ["CMakeLists.txt"]
+            exclude: ["CMakeLists.txt"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -413,7 +505,6 @@ let package = Package(
             dependencies: [
                 .product(name: "ArgumentParser", package: "swift-argument-parser"),
                 .product(name: "OrderedCollections", package: "swift-collections"),
-                .product(name: "SwiftIDEUtils", package: "swift-syntax"),
                 "Basics",
                 "Build",
                 "CoreCommands",
@@ -422,8 +513,11 @@ let package = Package(
                 "SourceControl",
                 "Workspace",
                 "XCBuildSupport",
-            ],
-            exclude: ["CMakeLists.txt", "README.md"]
+            ] + swiftSyntaxDependencies(["SwiftIDEUtils"]),
+            exclude: ["CMakeLists.txt", "README.md"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -436,7 +530,10 @@ let package = Package(
                 "SPMBuildCore",
                 "PackageModel",
             ],
-            exclude: ["CMakeLists.txt", "README.md"]
+            exclude: ["CMakeLists.txt", "README.md"],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
         ),
 
         .target(
@@ -449,6 +546,9 @@ let package = Package(
                 "CoreCommands",
                 "PackageCollections",
                 "PackageModel",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -468,6 +568,22 @@ let package = Package(
                 "SourceControl",
                 "SPMBuildCore",
                 "Workspace",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
+        ),
+
+        .target(
+            name: "QueryEngine",
+            dependencies: [
+                "_AsyncFileSystem",
+                "Basics",
+                .product(name: "Crypto", package: "swift-crypto"),
+            ],
+            swiftSettings: [
+                .enableExperimentalFeature("StrictConcurrency=complete"),
+                .unsafeFlags(["-static"]),
             ]
         ),
 
@@ -534,7 +650,7 @@ let package = Package(
                 "Commands",
                 "SwiftSDKCommand",
                 "PackageCollectionsCommand",
-                "PackageRegistryCommand"
+                "PackageRegistryCommand",
             ],
             linkerSettings: swiftpmLinkSettings
         ),
@@ -543,6 +659,15 @@ let package = Package(
             name: "swift-package-registry",
             dependencies: ["Commands", "PackageRegistryCommand"]
         ),
+        .executableTarget(
+            /** Utility to produce the artifacts for prebuilts */
+            name: "swift-build-prebuilts",
+            dependencies: [
+                .product(name: "ArgumentParser", package: "swift-argument-parser"),
+                "Basics",
+                "Workspace",
+            ]
+        ),
 
         // MARK: Support for Swift macros, should eventually move to a plugin-based solution
 
@@ -559,8 +684,8 @@ let package = Package(
         // MARK: Additional Test Dependencies
 
         .target(
-            /** SwiftPM test support library */
-            name: "SPMTestSupport",
+            /** SwiftPM internal test suite support library */
+            name: "_InternalTestSupport",
             dependencies: [
                 "Basics",
                 "Build",
@@ -573,45 +698,66 @@ let package = Package(
                 .product(name: "TSCTestSupport", package: "swift-tools-support-core"),
                 "Workspace",
                 "XCBuildSupport",
+            ],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
             ]
         ),
 
         .target(
-            /** Test for thread-santizer. */
+            /** Test for thread-sanitizer. */
             name: "tsan_utils",
-            dependencies: []),
+            dependencies: [],
+            swiftSettings: [
+                .unsafeFlags(["-static"]),
+            ]
+        ),
 
         // MARK: SwiftPM tests
 
+        .testTarget(
+            name: "_AsyncFileSystemTests",
+            dependencies: [
+                "_AsyncFileSystem",
+                "_InternalTestSupport",
+            ]
+        ),
+
         .testTarget(
             name: "SourceKitLSPAPITests",
             dependencies: [
                 "SourceKitLSPAPI",
-                "SPMTestSupport",
+                "_InternalTestSupport",
             ]
         ),
 
         .testTarget(
             name: "BasicsTests",
-            dependencies: ["Basics", "SPMTestSupport", "tsan_utils"],
+            dependencies: ["Basics", "_InternalTestSupport", "tsan_utils"],
             exclude: [
                 "Archiver/Inputs/archive.tar.gz",
                 "Archiver/Inputs/archive.zip",
                 "Archiver/Inputs/invalid_archive.tar.gz",
                 "Archiver/Inputs/invalid_archive.zip",
+                "processInputs/long-stdout-stderr",
+                "processInputs/exit4",
+                "processInputs/simple-stdout-stderr",
+                "processInputs/deadlock-if-blocking-io",
+                "processInputs/echo",
+                "processInputs/in-to-out",
             ]
         ),
         .testTarget(
             name: "BuildTests",
-            dependencies: ["Build", "PackageModel", "SPMTestSupport"]
+            dependencies: ["Build", "PackageModel", "Commands", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "LLBuildManifestTests",
-            dependencies: ["Basics", "LLBuildManifest", "SPMTestSupport"]
+            dependencies: ["Basics", "LLBuildManifest", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "WorkspaceTests",
-            dependencies: ["Workspace", "SPMTestSupport"]
+            dependencies: ["Workspace", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageDescriptionTests",
@@ -619,32 +765,31 @@ let package = Package(
         ),
         .testTarget(
             name: "SPMBuildCoreTests",
-            dependencies: ["SPMBuildCore", "SPMTestSupport"]
+            dependencies: ["SPMBuildCore", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageLoadingTests",
-            dependencies: ["PackageLoading", "SPMTestSupport"],
+            dependencies: ["PackageLoading", "_InternalTestSupport"],
             exclude: ["Inputs", "pkgconfigInputs"]
         ),
         .testTarget(
             name: "PackageModelTests",
-            dependencies: ["PackageModel", "SPMTestSupport"]
+            dependencies: ["PackageModel", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageModelSyntaxTests",
             dependencies: [
                 "PackageModelSyntax",
-                "SPMTestSupport",
-                .product(name: "SwiftIDEUtils", package: "swift-syntax"),
-            ]
+                "_InternalTestSupport",
+            ] + swiftSyntaxDependencies(["SwiftIDEUtils"])
         ),
         .testTarget(
             name: "PackageGraphTests",
-            dependencies: ["PackageGraph", "SPMTestSupport"]
+            dependencies: ["PackageGraph", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageGraphPerformanceTests",
-            dependencies: ["PackageGraph", "SPMTestSupport"],
+            dependencies: ["PackageGraph", "_InternalTestSupport"],
             exclude: [
                 "Inputs/PerfectHTTPServer.json",
                 "Inputs/ZewoHTTPServer.json",
@@ -654,40 +799,44 @@ let package = Package(
         ),
         .testTarget(
             name: "PackageCollectionsModelTests",
-            dependencies: ["PackageCollectionsModel", "SPMTestSupport"]
+            dependencies: ["PackageCollectionsModel", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageCollectionsSigningTests",
-            dependencies: ["PackageCollectionsSigning", "SPMTestSupport"]
+            dependencies: ["PackageCollectionsSigning", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageCollectionsTests",
-            dependencies: ["PackageCollections", "SPMTestSupport", "tsan_utils"]
+            dependencies: ["PackageCollections", "_InternalTestSupport", "tsan_utils"]
         ),
         .testTarget(
             name: "PackageFingerprintTests",
-            dependencies: ["PackageFingerprint", "SPMTestSupport"]
+            dependencies: ["PackageFingerprint", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackagePluginAPITests",
-            dependencies: ["PackagePlugin", "SPMTestSupport"]
+            dependencies: ["PackagePlugin", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "PackageRegistryTests",
-            dependencies: ["SPMTestSupport", "PackageRegistry"]
+            dependencies: ["_InternalTestSupport", "PackageRegistry"]
         ),
         .testTarget(
             name: "PackageSigningTests",
-            dependencies: ["SPMTestSupport", "PackageSigning"]
+            dependencies: ["_InternalTestSupport", "PackageSigning"]
+        ),
+        .testTarget(
+            name: "QueryEngineTests",
+            dependencies: ["QueryEngine", "_InternalTestSupport"]
         ),
         .testTarget(
             name: "SourceControlTests",
-            dependencies: ["SourceControl", "SPMTestSupport"],
+            dependencies: ["SourceControl", "_InternalTestSupport"],
             exclude: ["Inputs/TestRepo.tgz"]
         ),
         .testTarget(
             name: "XCBuildSupportTests",
-            dependencies: ["XCBuildSupport", "SPMTestSupport"],
+            dependencies: ["XCBuildSupport", "_InternalTestSupport"],
             exclude: ["Inputs/Foo.pc"]
         ),
         // Examples (These are built to ensure they stay up to date with the API.)
@@ -708,7 +857,7 @@ package.targets.append(contentsOf: [
 ])
 #endif
 
-// Workaround SPM's attempt to link in executables which does not work on all
+// Workaround SwiftPM's attempt to link in executables which does not work on all
 // platforms.
 #if !os(Windows)
 package.targets.append(contentsOf: [
@@ -716,12 +865,13 @@ package.targets.append(contentsOf: [
         name: "FunctionalPerformanceTests",
         dependencies: [
             "swift-package-manager",
-            "SPMTestSupport",
+            "_InternalTestSupport",
         ]
     ),
 ])
 
-// rdar://101868275 "error: cannot find 'XCTAssertEqual' in scope" can affect almost any functional test, so we flat out disable them all until we know what is going on
+// rdar://101868275 "error: cannot find 'XCTAssertEqual' in scope" can affect almost any functional test, so we flat out
+// disable them all until we know what is going on
 if ProcessInfo.processInfo.environment["SWIFTCI_DISABLE_SDK_DEPENDENT_TESTS"] == nil {
     package.targets.append(contentsOf: [
         .testTarget(
@@ -729,7 +879,7 @@ if ProcessInfo.processInfo.environment["SWIFTCI_DISABLE_SDK_DEPENDENT_TESTS"] ==
             dependencies: [
                 "swift-package-manager",
                 "PackageModel",
-                "SPMTestSupport"
+                "_InternalTestSupport",
             ]
         ),
 
@@ -739,7 +889,12 @@ if ProcessInfo.processInfo.environment["SWIFTCI_DISABLE_SDK_DEPENDENT_TESTS"] ==
                 "Basics",
             ]
         ),
-
+        .testTarget(
+            name: "_InternalTestSupportTests",
+            dependencies: [
+                "_InternalTestSupport"
+            ]
+        ),
         .testTarget(
             name: "CommandsTests",
             dependencies: [
@@ -751,7 +906,7 @@ if ProcessInfo.processInfo.environment["SWIFTCI_DISABLE_SDK_DEPENDENT_TESTS"] ==
                 "PackageModelSyntax",
                 "PackageRegistryCommand",
                 "SourceControl",
-                "SPMTestSupport",
+                "_InternalTestSupport",
                 "Workspace",
                 "dummy-swiftc",
             ]
@@ -760,6 +915,23 @@ if ProcessInfo.processInfo.environment["SWIFTCI_DISABLE_SDK_DEPENDENT_TESTS"] ==
 }
 #endif
 
+/// Whether swift-syntax is being built as a single dynamic library instead of as a separate library per module.
+///
+/// This means that the swift-syntax symbols don't need to be statically linked, which allows us to stay below the
+/// maximum number of exported symbols on Windows, in turn allowing us to build sourcekit-lsp using SwiftPM on Windows
+/// and run its tests.
+var buildDynamicSwiftSyntaxLibrary: Bool {
+  ProcessInfo.processInfo.environment["SWIFTSYNTAX_BUILD_DYNAMIC_LIBRARY"] != nil
+}
+
+func swiftSyntaxDependencies(_ names: [String]) -> [Target.Dependency] {
+  if buildDynamicSwiftSyntaxLibrary {
+    return [.product(name: "_SwiftSyntaxDynamic", package: "swift-syntax")]
+  } else {
+    return names.map { .product(name: $0, package: "swift-syntax") }
+  }
+}
+
 // Add package dependency on llbuild when not bootstrapping.
 //
 // When bootstrapping SwiftPM, we can't use llbuild as a package dependency it
@@ -770,7 +942,7 @@ if ProcessInfo.processInfo.environment["SWIFTCI_DISABLE_SDK_DEPENDENT_TESTS"] ==
 // this right now.
 
 /// When not using local dependencies, the branch to use for llbuild and TSC repositories.
-let relatedDependenciesBranch = "release/6.0"
+let relatedDependenciesBranch = "release/6.1"
 
 if ProcessInfo.processInfo.environment["SWIFTPM_LLBUILD_FWK"] == nil {
     if ProcessInfo.processInfo.environment["SWIFTCI_USE_LOCAL_DEPS"] == nil {
@@ -795,13 +967,14 @@ if ProcessInfo.processInfo.environment["SWIFTCI_USE_LOCAL_DEPS"] == nil {
         // The 'swift-argument-parser' version declared here must match that
         // used by 'swift-driver' and 'sourcekit-lsp'. Please coordinate
         // dependency version changes here with those projects.
-        .package(url: "https://github.com/apple/swift-argument-parser.git", .upToNextMinor(from: "1.2.2")),
+        .package(url: "https://github.com/apple/swift-argument-parser.git", .upToNextMinor(from: "1.4.0")),
         .package(url: "https://github.com/apple/swift-driver.git", branch: relatedDependenciesBranch),
         .package(url: "https://github.com/apple/swift-crypto.git", .upToNextMinor(from: "3.0.0")),
         .package(url: "https://github.com/swiftlang/swift-syntax.git", branch: relatedDependenciesBranch),
-        .package(url: "https://github.com/apple/swift-system.git", "1.1.1" ..< "1.4.0"),
+        .package(url: "https://github.com/apple/swift-system.git", from: "1.1.1"),
         .package(url: "https://github.com/apple/swift-collections.git", "1.0.1" ..< "1.2.0"),
-        .package(url: "https://github.com/apple/swift-certificates.git", "1.0.1" ..< "1.4.0"),
+        .package(url: "https://github.com/apple/swift-certificates.git", "1.0.1" ..< "1.6.0"),
+        .package(url: "https://github.com/swiftlang/swift-toolchain-sqlite.git", from: "1.0.0"),
     ]
 } else {
     package.dependencies += [
@@ -813,5 +986,6 @@ if ProcessInfo.processInfo.environment["SWIFTCI_USE_LOCAL_DEPS"] == nil {
         .package(path: "../swift-system"),
         .package(path: "../swift-collections"),
         .package(path: "../swift-certificates"),
+        .package(path: "../swift-toolchain-sqlite"),
     ]
 }
