@@ -22,41 +22,14 @@ public protocol AuthorizationProvider: Sendable {
 }
 
 public protocol AuthorizationWriter {
-    @available(*, noasync, message: "Use the async alternative")
     func addOrUpdate(
         for url: URL,
         user: String,
         password: String,
-        persist: Bool,
-        callback: @escaping (Result<Void, Error>) -> Void
-    )
+        persist: Bool
+    ) async throws
 
-    @available(*, noasync, message: "Use the async alternative")
-    func remove(for url: URL, callback: @escaping (Result<Void, Error>) -> Void)
-}
-
-public extension AuthorizationWriter {
-    func addOrUpdate(
-        for url: URL,
-        user: String,
-        password: String,
-        persist: Bool = true
-    ) async throws {
-        try await safe_async {
-            self.addOrUpdate(
-                for: url,
-                user: user,
-                password: password, 
-                persist: persist,
-                callback: $0)
-        }
-    }
-
-    func remove(for url: URL) async throws {
-        try await safe_async {
-            self.remove(for: url, callback: $0)
-        }
-    }
+    func remove(for url: URL) async throws
 }
 
 public enum AuthorizationProviderError: Error {
@@ -97,16 +70,15 @@ public final class NetrcAuthorizationProvider: AuthorizationProvider, Authorizat
         for url: URL,
         user: String,
         password: String,
-        persist: Bool = true,
-        callback: @escaping (Result<Void, Error>) -> Void
-    ) {
+        persist: Bool = true
+    ) async throws {
         guard let machine = Self.machine(for: url) else {
-            return callback(.failure(AuthorizationProviderError.invalidURLHost))
+            throw AuthorizationProviderError.invalidURLHost
         }
 
         if !persist {
             self.cache[machine] = (user, password)
-            return callback(.success(()))
+            return
         }
 
         // Same entry already exists, no need to add or update
@@ -114,7 +86,7 @@ public final class NetrcAuthorizationProvider: AuthorizationProvider, Authorizat
         guard netrc?.machines
             .first(where: { $0.name.lowercased() == machine && $0.login == user && $0.password == password }) == nil
         else {
-            return callback(.success(()))
+            return
         }
 
         do {
@@ -131,21 +103,15 @@ public final class NetrcAuthorizationProvider: AuthorizationProvider, Authorizat
                     stream.write("\n")
                 }
             }
-
-            callback(.success(()))
         } catch {
-            callback(.failure(
-                AuthorizationProviderError
-                    .other("Failed to update netrc file at \(self.path): \(error.interpolationDescription)")
-            ))
+            throw AuthorizationProviderError
+                .other("Failed to update netrc file at \(self.path): \(error.interpolationDescription)")
         }
     }
 
-    public func remove(for url: URL, callback: @escaping (Result<Void, Error>) -> Void) {
-        callback(.failure(
-            AuthorizationProviderError
-                .other("User must edit netrc file at \(self.path) manually to remove entries")
-        ))
+    public func remove(for url: URL) async throws {
+        throw AuthorizationProviderError
+            .other("User must edit netrc file at \(self.path) manually to remove entries")
     }
 
     public func authentication(for url: URL) -> (user: String, password: String)? {
@@ -214,11 +180,10 @@ public final class KeychainAuthorizationProvider: AuthorizationProvider, Authori
         for url: URL,
         user: String,
         password: String,
-        persist: Bool = true,
-        callback: @escaping (Result<Void, Error>) -> Void
-    ) {
+        persist: Bool = true
+    ) async throws {
         guard let protocolHostPort = ProtocolHostPort(from: url) else {
-            return callback(.failure(AuthorizationProviderError.invalidURLHost))
+            throw AuthorizationProviderError.invalidURLHost
         }
 
         self.observabilityScope
@@ -226,35 +191,25 @@ public final class KeychainAuthorizationProvider: AuthorizationProvider, Authori
 
         if !persist {
             self.cache[protocolHostPort.description] = (user, password)
-            return callback(.success(()))
+            return
         }
 
         let passwordData = Data(password.utf8)
 
-        do {
-            if !(try self.update(protocolHostPort: protocolHostPort, account: user, password: passwordData)) {
-                try self.create(protocolHostPort: protocolHostPort, account: user, password: passwordData)
-            }
-            callback(.success(()))
-        } catch {
-            callback(.failure(error))
+        if !(try self.update(protocolHostPort: protocolHostPort, account: user, password: passwordData)) {
+            try self.create(protocolHostPort: protocolHostPort, account: user, password: passwordData)
         }
     }
 
-    public func remove(for url: URL, callback: @escaping (Result<Void, Error>) -> Void) {
+    public func remove(for url: URL) async throws {
         guard let protocolHostPort = ProtocolHostPort(from: url) else {
-            return callback(.failure(AuthorizationProviderError.invalidURLHost))
+            throw AuthorizationProviderError.invalidURLHost
         }
 
         self.observabilityScope
             .emit(debug: "remove credentials for '\(protocolHostPort)' [\(url.absoluteString)] from keychain")
 
-        do {
-            try self.delete(protocolHostPort: protocolHostPort)
-            callback(.success(()))
-        } catch {
-            callback(.failure(error))
-        }
+        try self.delete(protocolHostPort: protocolHostPort)
     }
 
     public func authentication(for url: URL) -> (user: String, password: String)? {
