@@ -16,24 +16,24 @@ import struct Foundation.Date
 /// an `async` closure. This type serves as a replacement for `ThreadSafeBox`
 /// implemented with Swift Concurrency primitives.
 public actor SendableBox<Value: Sendable> {
-    init(_ value: Value? = nil) {
+    public init(_ value: Value) {
         self.value = value
     }
 
-    var value: Value?
+    public var value: Value
+
+    public func set(_ value: Value) {
+        self.value = value
+    }
 }
 
 extension SendableBox where Value == Int {
     func increment() {
-        if let value {
-            self.value = value + 1
-        }
+        self.value = value + 1
     }
 
     func decrement() {
-        if let value {
-            self.value = value - 1
-        }
+        self.value = value - 1
     }
 }
 
