@@ -56,6 +56,10 @@ public class ObservabilitySystem {
             self.underlying(scope, diagnostic)
         }
     }
+
+    public static var NOOP: ObservabilityScope {
+        ObservabilitySystem { _, _ in }.topScope
+    }
 }
 
 public protocol ObservabilityHandlerProvider {
@@ -246,6 +250,18 @@ extension DiagnosticsEmitterProtocol {
         }
     }
 
+    public func trap<T>(_ closure: () async throws -> T) async -> T? {
+        do {
+            return try await closure()
+        } catch Diagnostics.fatalError {
+            // FIXME: (diagnostics) deprecate this with Diagnostics.fatalError
+            return nil
+        } catch {
+            self.emit(error)
+            return nil
+        }
+    }
+
     /// trap a throwing closure, emitting diagnostics on error and returning boolean representing success
     @discardableResult
     public func trap(_ closure: () throws -> Void) -> Bool {
@@ -261,6 +277,20 @@ extension DiagnosticsEmitterProtocol {
         }
     }
 
+    @discardableResult
+    public func trap(_ closure: () async throws -> Void) async -> Bool {
+        do {
+            try await closure()
+            return true
+        } catch Diagnostics.fatalError {
+            // FIXME: (diagnostics) deprecate this with Diagnostics.fatalError
+            return false
+        } catch {
+            self.emit(error)
+            return false
+        }
+    }
+
     /// If `underlyingError` is not `nil`, its human-readable description is interpolated with `message`,
     /// otherwise `message` itself is returned.
     private func makeMessage(from message: String, underlyingError: Error?) -> String {
