@@ -32,30 +32,33 @@ extension LLBuildManifestBuilder {
             inputs.append(resourcesNode)
         }
 
-        func addStaticTargetInputs(_ target: ResolvedModule) {
-            if case .swift(let desc)? = self.plan.targetMap[target.id], target.type == .library {
+        func addStaticTargetInputs(_ description: ModuleBuildDescription?) {
+            if case .swift(let desc) = description, desc.target.type == .library {
                 inputs.append(file: desc.moduleOutputPath)
             }
         }
 
-        for dependency in target.target.dependencies(satisfying: target.buildEnvironment) {
+        for dependency in target.dependencies(using: self.plan) {
             switch dependency {
-            case .module(let target, _):
-                addStaticTargetInputs(target)
+            case .module(_, let description):
+                addStaticTargetInputs(description)
 
-            case .product(let product, _):
+            case .product(let product, let productDescription):
                 switch product.type {
                 case .executable, .snippet, .library(.dynamic), .macro:
-                    guard let planProduct = plan.productMap[product.id] else {
-                        throw InternalError("unknown product \(product)")
+                    guard let productDescription else {
+                        throw InternalError("No build description for product: \(product)")
                     }
                     // Establish a dependency on binary of the product.
-                    let binary = try planProduct.binaryPath
-                    inputs.append(file: binary)
+                    try inputs.append(file: productDescription.binaryPath)
 
                 case .library(.automatic), .library(.static), .plugin:
-                    for target in product.modules {
-                        addStaticTargetInputs(target)
+                    for module in product.modules {
+                        let dependencyDescription = self.plan.description(
+                            for: module,
+                            context: product.type == .plugin ? .host : target.destination
+                        )
+                        addStaticTargetInputs(dependencyDescription)
                     }
                 case .test:
                     break
