@@ -191,10 +191,8 @@ extension LLBuildManifestBuilder {
     public func addTargetsToExplicitBuildManifest() throws {
         // Sort the product targets in topological order in order to collect and "bubble up"
         // their respective dependency graphs to the depending targets.
-        let nodes = self.plan.targets.compactMap {
-            ResolvedModule.Dependency.module($0.target, conditions: [])
-        }
-        let allPackageDependencies = try topologicalSort(nodes, successors: { $0.dependencies })
+        let allPackageDependencies = self.plan.targets.flatMap { $0.recursiveDependencies(using: self.plan) }
+
         // Instantiate the inter-module dependency oracle which will cache commonly-scanned
         // modules across targets' Driver instances.
         let dependencyOracle = InterModuleDependencyOracle()
@@ -206,14 +204,15 @@ extension LLBuildManifestBuilder {
 
         // Create commands for all module descriptions in the plan.
         for dependency in allPackageDependencies.reversed() {
-            guard case .module(let target, _) = dependency else {
+            guard case .module(let module, let description) = dependency else {
                 // Product dependency build jobs are added after the fact.
                 // Targets that depend on product dependencies will expand the corresponding
                 // product into its constituent targets.
                 continue
             }
-            guard target.underlying.type != .systemModule,
-                  target.underlying.type != .binary
+
+            guard module.underlying.type != .systemModule,
+                  module.underlying.type != .binary
             else {
                 // Much like non-Swift targets, system modules will consist of a modulemap
                 // somewhere in the filesystem, with the path to that module being either
@@ -225,9 +224,11 @@ extension LLBuildManifestBuilder {
                 // be able to detect such targets' modules.
                 continue
             }
-            guard let description = plan.targetMap[target.id] else {
-                throw InternalError("Expected description for target \(target)")
+
+            guard let description else {
+                throw InternalError("Expected description for module \(module)")
             }
+
             switch description {
             case .swift(let desc):
                 try self.createExplicitSwiftTargetCompileCommand(
@@ -319,32 +320,29 @@ extension LLBuildManifestBuilder {
         for targetDescription: ModuleBuildDescription,
         dependencyModuleDetailsMap: inout SwiftDriver.ExternalTargetModuleDetailsMap
     ) throws {
-        for dependency in targetDescription.target.dependencies(satisfying: targetDescription.buildParameters.buildEnvironment) {
+        for dependency in targetDescription.dependencies(using: self.plan) {
             switch dependency {
-            case .product:
-                // Product dependencies are broken down into the targets that make them up.
-                guard let dependencyProduct = dependency.product else {
-                    throw InternalError("unknown dependency product for \(dependency)")
-                }
-                for dependencyProductTarget in dependencyProduct.modules {
-                    guard let dependencyTargetDescription = self.plan.targetMap[dependencyProductTarget.id] else {
-                        throw InternalError("unknown dependency target for \(dependencyProductTarget)")
+            case .product(let product, let productDescription):
+                for productDependency in product.modules {
+                    guard let dependencyModuleDescription = self.plan.description(
+                        for: productDependency,
+                        context: productDescription?.destination ?? targetDescription.destination
+                    ) else
+                    {
+                        throw InternalError("unknown dependency target for \(productDependency)")
                     }
                     try self.addTargetDependencyInfo(
-                        for: dependencyTargetDescription,
+                        for: dependencyModuleDescription,
                         dependencyModuleDetailsMap: &dependencyModuleDetailsMap
                     )
                 }
-            case .module:
-                // Product dependencies are broken down into the targets that make them up.
-                guard
-                    let dependencyTarget = dependency.module,
-                    let dependencyTargetDescription = self.plan.targetMap[dependencyTarget.id]
-                else {
-                    throw InternalError("unknown dependency target for \(dependency)")
+            case .module(let dependencyModule, let dependencyDescription):
+                guard let dependencyDescription else {
+                    throw InternalError("No build description for module: \(dependencyModule)")
                 }
+                // Product dependencies are broken down into the targets that make them up.
                 try self.addTargetDependencyInfo(
-                    for: dependencyTargetDescription,
+                    for: dependencyDescription,
                     dependencyModuleDetailsMap: &dependencyModuleDetailsMap
                 )
             }
@@ -358,7 +356,7 @@ extension LLBuildManifestBuilder {
         guard case .swift(let dependencySwiftTargetDescription) = targetDescription else {
             return
         }
-        dependencyModuleDetailsMap[ModuleDependencyId.swiftPlaceholder(targetDescription.target.c99name)] =
+        dependencyModuleDetailsMap[ModuleDependencyId.swiftPlaceholder(targetDescription.module.c99name)] =
             SwiftDriver.ExternalTargetModuleDetails(
                 path: TSCAbsolutePath(dependencySwiftTargetDescription.moduleOutputPath),
                 isFramework: false
@@ -378,6 +376,9 @@ extension LLBuildManifestBuilder {
         let cmdName = target.getCommandName()
 
         self.manifest.addWriteSourcesFileListCommand(sources: target.sources, sourcesFileListPath: target.sourcesFileListPath)
+        let outputFileMapPath = target.tempsPath.appending("output-file-map.json")
+        // FIXME: Eliminate side effect.
+        try target.writeOutputFileMap(to: outputFileMapPath)
         self.manifest.addSwiftCmd(
             name: cmdName,
             inputs: inputs + [Node.file(target.sourcesFileListPath)],
@@ -393,8 +394,8 @@ extension LLBuildManifestBuilder {
             sources: target.sources,
             fileList: target.sourcesFileListPath,
             isLibrary: isLibrary,
-            wholeModuleOptimization: target.buildParameters.configuration == .release,
-            outputFileMapPath: try target.writeOutputFileMap(), // FIXME: Eliminate side effect.
+            wholeModuleOptimization: target.useWholeModuleOptimization,
+            outputFileMapPath: outputFileMapPath,
             prepareForIndexing: target.buildParameters.prepareForIndexing != .off
         )
     }
@@ -422,63 +423,73 @@ extension LLBuildManifestBuilder {
 
         let prepareForIndexing = target.buildParameters.prepareForIndexing
 
-        func addStaticTargetInputs(_ target: ResolvedModule) throws {
+        func addStaticTargetInputs(_ module: ResolvedModule, _ description: ModuleBuildDescription?) throws {
             // Ignore C Modules.
-            if target.underlying is SystemLibraryModule { return }
+            if module.underlying is SystemLibraryModule { return }
             // Ignore Binary Modules.
-            if target.underlying is BinaryModule { return }
+            if module.underlying is BinaryModule { return }
             // Ignore Plugin Modules.
-            if target.underlying is PluginModule { return }
+            if module.underlying is PluginModule { return }
+
+            guard let description else {
+                throw InternalError("No build description for module: \(module)")
+            }
 
             // Depend on the binary for executable targets.
-            if target.type == .executable && prepareForIndexing == .off {
-                // FIXME: Optimize.
+            if module.type == .executable && prepareForIndexing == .off {
+                // FIXME: Optimize. Build plan could build a mapping between executable modules
+                // and their products to speed up search here, which is inefficient if the plan
+                // contains a lot of products.
                 if let productDescription = try plan.productMap.values.first(where: {
-                    try $0.product.type == .executable && $0.product.executableModule.id == target.id
+                    try $0.product.type == .executable &&
+                        $0.product.executableModule.id == module.id &&
+                        $0.destination == description.destination
                 }) {
                     try inputs.append(file: productDescription.binaryPath)
                 }
                 return
             }
 
-            switch self.plan.targetMap[target.id] {
-            case .swift(let target)?:
-                inputs.append(file: target.moduleOutputPath)
-            case .clang(let target)?:
+            switch description {
+            case .swift(let swiftDescription):
+                inputs.append(file: swiftDescription.moduleOutputPath)
+            case .clang(let clangDescription):
                 if prepareForIndexing != .off {
                     // In preparation, we're only building swiftmodules
                     // propagate the dependency to the header files in this target
-                    for header in target.clangTarget.headers {
+                    for header in clangDescription.clangTarget.headers {
                         inputs.append(file: header)
                     }
                 } else {
-                    for object in try target.objects {
+                    for object in try clangDescription.objects {
                         inputs.append(file: object)
                     }
                 }
-            case nil:
-                throw InternalError("unexpected: target \(target) not in target map \(self.plan.targetMap)")
             }
         }
 
-        for dependency in target.target.dependencies(satisfying: target.buildParameters.buildEnvironment) {
+        for dependency in target.dependencies(using: self.plan) {
             switch dependency {
-            case .module(let target, _):
-                try addStaticTargetInputs(target)
+            case .module(let module, let description):
+                try addStaticTargetInputs(module, description)
 
-            case .product(let product, _):
+            case .product(let product, let productDescription):
                 switch product.type {
                 case .executable, .snippet, .library(.dynamic), .macro:
-                    guard let planProduct = plan.productMap[product.id] else {
-                        throw InternalError("unknown product \(product)")
+                    guard let productDescription else {
+                        throw InternalError("No description for product: \(product)")
                     }
                     // Establish a dependency on binary of the product.
-                    try inputs.append(file: planProduct.binaryPath)
+                    try inputs.append(file: productDescription.binaryPath)
 
                 // For automatic and static libraries, and plugins, add their targets as static input.
                 case .library(.automatic), .library(.static), .plugin:
-                    for target in product.modules {
-                        try addStaticTargetInputs(target)
+                    for module in product.modules {
+                        let description = self.plan.description(
+                            for: module,
+                            context: product.type == .plugin ? .host : target.destination
+                        )
+                        try addStaticTargetInputs(module, description)
                     }
 
                 case .test:
@@ -498,9 +509,12 @@ extension LLBuildManifestBuilder {
 
         let additionalInputs = try self.addBuildToolPlugins(.swift(target))
 
-        // Depend on any required macro product's output.
-        try target.requiredMacroProducts.forEach { macro in
-            try inputs.append(.virtual(macro.llbuildTargetName))
+        // Depend on any required macro's output.
+        try target.requiredMacros.forEach { macro in
+            inputs.append(.virtual(getLLBuildTargetName(
+                macro: macro,
+                buildParameters: target.macroBuildParameters
+            )))
         }
 
         return inputs + additionalInputs
