@@ -19,12 +19,11 @@ import class PackageModel.SystemLibraryModule
 extension BuildPlan {
     func plan(swiftTarget: SwiftModuleBuildDescription) throws {
         // We need to iterate recursive dependencies because Swift compiler needs to see all the targets a target
-        // depends on.
-        let environment = swiftTarget.buildParameters.buildEnvironment
-        for case .module(let dependency, _) in try swiftTarget.target.recursiveDependencies(satisfying: environment) {
+        // builds against
+        for case .module(let dependency, let description) in swiftTarget.recursiveLinkDependencies(using: self) {
             switch dependency.underlying {
             case let underlyingTarget as ClangModule where underlyingTarget.type == .library:
-                guard case let .clang(target)? = targetMap[dependency.id] else {
+                guard case let .clang(target)? = description else {
                     throw InternalError("unexpected clang target \(underlyingTarget)")
                 }
                 // Add the path to modulemap of the dependency. Currently we require that all Clang targets have a
@@ -54,5 +53,4 @@ extension BuildPlan {
             }
         }
     }
-
 }
