@@ -34,14 +34,14 @@ public func generateTestObservationCode(buildParameters: BuildParameters) -> Str
                 return "\(buildParameters.testOutputPath)"
             }
 
-            private func write(record: Encodable) {
+            private func write(record: any Encodable) {
                 let lock = FileLock(at: URL(fileURLWithPath: self.testOutputPath + ".lock"))
                 _ = try? lock.withLock {
                     self._write(record: record)
                 }
             }
 
-            private func _write(record: Encodable) {
+            private func _write(record: any Encodable) {
                 if let data = try? JSONEncoder().encode(record) {
                     if let fileHandle = FileHandle(forWritingAtPath: self.testOutputPath) {
                         defer { fileHandle.closeFile() }
@@ -435,7 +435,7 @@ public func generateTestObservationCode(buildParameters: BuildParameters) -> Str
         }
 
         extension TestErrorInfo {
-            init(_ error: Swift.Error) {
+            init(_ error: any Swift.Error) {
                 self.init(description: "\\(error)", type: "\\(Swift.type(of: error))")
             }
         }
