@@ -15,6 +15,8 @@ import ArgumentParser
 @_spi(SwiftPMInternal)
 import Basics
 
+import _Concurrency
+
 @_spi(SwiftPMInternal)
 import CoreCommands
 
@@ -30,6 +32,7 @@ import SPMBuildCore
 import func TSCLibc.exit
 import Workspace
 
+import struct TSCBasic.FileSystemError
 import struct TSCBasic.ByteString
 import enum TSCBasic.JSON
 import class Basics.AsyncProcess
@@ -39,6 +42,8 @@ import class TSCBasic.Thread
 
 #if os(Windows)
 import WinSDK // for ERROR_NOT_FOUND
+#elseif canImport(Android)
+import Android
 #endif
 
 private enum TestError: Swift.Error {
@@ -114,6 +119,11 @@ struct TestEventStreamOptions: ParsableArguments {
     @Option(name: .customLong("event-stream-version"),
             help: .hidden)
     var eventStreamVersion: Int?
+
+  /// Experimental path for writing attachments (Swift Testing only.)
+  @Option(name: .customLong("experimental-attachments-path"),
+          help: .private)
+  var experimentalAttachmentsPath: AbsolutePath?
 }
 
 struct TestCommandOptions: ParsableArguments {
@@ -195,6 +205,9 @@ struct TestCommandOptions: ParsableArguments {
     var enableExperimentalTestOutput: Bool {
         return testOutput == .experimentalSummary
     }
+
+    @OptionGroup(visibility: .hidden)
+    package var traits: TraitOptions
 }
 
 /// Tests filtering specifier, which is used to filter tests to run.
@@ -225,7 +238,6 @@ public enum TestOutput: String, ExpressibleByArgument {
 }
 
 /// swift-test tool namespace
-
 public struct SwiftTestCommand: AsyncSwiftCommand {
     public static var configuration = CommandConfiguration(
         commandName: "test",
@@ -419,14 +431,14 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
         }
 
         if self.options.shouldPrintCodeCovPath {
-            try printCodeCovPath(swiftCommandState)
+            try await printCodeCovPath(swiftCommandState)
         } else if self.options._deprecated_shouldListTests {
             // backward compatibility 6/2022 for deprecation of flag into a subcommand
             let command = try List.parse()
-            try command.run(swiftCommandState)
+            try await command.run(swiftCommandState)
         } else {
             let (productsBuildParameters, _) = try swiftCommandState.buildParametersForTest(options: self.options)
-            let testProducts = try buildTestsIfNeeded(swiftCommandState: swiftCommandState)
+            let testProducts = try await buildTestsIfNeeded(swiftCommandState: swiftCommandState)
 
             // Clean out the code coverage directory that may contain stale
             // profraw files from a previous run of the code coverage tool.
@@ -436,7 +448,8 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
 
             try await run(swiftCommandState, buildParameters: productsBuildParameters, testProducts: testProducts)
 
-            // process code Coverage if request
+            // Process code coverage if requested. We do not process it if the test run failed.
+            // See https://github.com/swiftlang/swift-package-manager/pull/6894 for more info.
             if self.options.enableCodeCoverage, swiftCommandState.executionStatus != .failure {
                 try await processCodeCoverage(testProducts, swiftCommandState: swiftCommandState)
             }
@@ -449,7 +462,7 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
         additionalArguments: [String],
         productsBuildParameters: BuildParameters,
         swiftCommandState: SwiftCommandState,
-        library: BuildParameters.Testing.Library
+        library: TestingLibrary
     ) async throws -> TestRunner.Result {
         // Pass through all arguments from the command line to Swift Testing.
         var additionalArguments = additionalArguments
@@ -466,15 +479,17 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
             }
             additionalArguments += commandLineArguments
 
-            if var xunitPath = options.xUnitOutput, options.testLibraryOptions.isEnabled(.xctest, swiftCommandState: swiftCommandState) {
-                // We are running Swift Testing, XCTest is also running in this session, and an xUnit path
-                // was specified. Make sure we don't stomp on XCTest's XML output by having Swift Testing
-                // write to a different path.
-                var xunitFileName = "\(xunitPath.basenameWithoutExt)-swift-testing"
-                if let ext = xunitPath.extension {
-                    xunitFileName = "\(xunitFileName).\(ext)"
+            if var xunitPath = options.xUnitOutput {
+                if options.testLibraryOptions.isEnabled(.xctest, swiftCommandState: swiftCommandState) {
+                    // We are running Swift Testing, XCTest is also running in this session, and an xUnit path
+                    // was specified. Make sure we don't stomp on XCTest's XML output by having Swift Testing
+                    // write to a different path.
+                    var xunitFileName = "\(xunitPath.basenameWithoutExt)-swift-testing"
+                    if let ext = xunitPath.extension {
+                        xunitFileName = "\(xunitFileName).\(ext)"
+                    }
+                    xunitPath = xunitPath.parentDirectory.appending(xunitFileName)
                 }
-                xunitPath = xunitPath.parentDirectory.appending(xunitFileName)
                 additionalArguments += ["--xunit-output", xunitPath.pathString]
             }
         }
@@ -482,7 +497,7 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
         let toolchain = try swiftCommandState.getTargetToolchain()
         let testEnv = try TestingSupport.constructTestEnvironment(
             toolchain: toolchain,
-            buildParameters: productsBuildParameters,
+            destinationBuildParameters: productsBuildParameters,
             sanitizers: globalOptions.build.sanitizers,
             library: library
         )
@@ -559,18 +574,18 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
         }
 
         // Merge all the profraw files to produce a single profdata file.
-        try mergeCodeCovRawDataFiles(swiftCommandState: swiftCommandState)
+        try await mergeCodeCovRawDataFiles(swiftCommandState: swiftCommandState)
 
         let (productsBuildParameters, _) = try swiftCommandState.buildParametersForTest(options: self.options)
         for product in testProducts {
             // Export the codecov data as JSON.
             let jsonPath = productsBuildParameters.codeCovAsJSONPath(packageName: rootManifest.displayName)
-            try exportCodeCovAsJSON(to: jsonPath, testBinary: product.binaryPath, swiftCommandState: swiftCommandState)
+            try await exportCodeCovAsJSON(to: jsonPath, testBinary: product.binaryPath, swiftCommandState: swiftCommandState)
         }
     }
 
     /// Merges all profraw profiles in codecoverage directory into default.profdata file.
-    private func mergeCodeCovRawDataFiles(swiftCommandState: SwiftCommandState) throws {
+    private func mergeCodeCovRawDataFiles(swiftCommandState: SwiftCommandState) async throws {
         // Get the llvm-prof tool.
         let llvmProf = try swiftCommandState.getTargetToolchain().getLLVMProf()
 
@@ -588,7 +603,7 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
         }
         args += ["-o", productsBuildParameters.codeCovDataFile.pathString]
 
-        try AsyncProcess.checkNonZeroExit(arguments: args)
+        try await AsyncProcess.checkNonZeroExit(arguments: args)
     }
 
     /// Exports profdata as a JSON file.
@@ -596,7 +611,7 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
         to path: AbsolutePath,
         testBinary: AbsolutePath,
         swiftCommandState: SwiftCommandState
-    ) throws {
+    ) async throws {
         // Export using the llvm-cov tool.
         let llvmCov = try swiftCommandState.getTargetToolchain().getLLVMCov()
         let (productsBuildParameters, _) = try swiftCommandState.buildParametersForTest(options: self.options)
@@ -606,7 +621,7 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
             "-instr-profile=\(productsBuildParameters.codeCovDataFile)",
             testBinary.pathString
         ]
-        let result = try AsyncProcess.popen(arguments: args)
+        let result = try await AsyncProcess.popen(arguments: args)
 
         if result.exitStatus != .terminated(code: 0) {
             let output = try result.utf8Output() + result.utf8stderrOutput()
@@ -620,13 +635,14 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
     /// - Returns: The paths to the build test products.
     private func buildTestsIfNeeded(
         swiftCommandState: SwiftCommandState
-    ) throws -> [BuiltTestProduct] {
+    ) async throws -> [BuiltTestProduct] {
         let (productsBuildParameters, toolsBuildParameters) = try swiftCommandState.buildParametersForTest(options: self.options)
-        return try Commands.buildTestsIfNeeded(
+        return try await Commands.buildTestsIfNeeded(
             swiftCommandState: swiftCommandState,
             productsBuildParameters: productsBuildParameters,
             toolsBuildParameters: toolsBuildParameters,
-            testProduct: self.options.sharedOptions.testProduct
+            testProduct: self.options.sharedOptions.testProduct,
+            traitConfiguration: .init(traitOptions: self.options.traits)
         )
     }
 
@@ -662,16 +678,13 @@ public struct SwiftTestCommand: AsyncSwiftCommand {
 }
 
 extension SwiftTestCommand {
-    func printCodeCovPath(_ swiftCommandState: SwiftCommandState) throws {
+    func printCodeCovPath(_ swiftCommandState: SwiftCommandState) async throws {
         let workspace = try swiftCommandState.getActiveWorkspace()
         let root = try swiftCommandState.getWorkspaceRoot()
-        let rootManifests = try temp_await {
-            workspace.loadRootManifests(
-                packages: root.packages,
-                observabilityScope: swiftCommandState.observabilityScope,
-                completion: $0
-            )
-        }
+        let rootManifests = try await workspace.loadRootManifests(
+            packages: root.packages,
+            observabilityScope: swiftCommandState.observabilityScope
+        )
         guard let rootManifest = rootManifests.values.first else {
             throw StringError("invalid manifests at \(root.packages)")
         }
@@ -693,7 +706,7 @@ extension SwiftTestCommand {
         }
     }
 
-    struct List: SwiftCommand {
+    struct List: AsyncSwiftCommand {
         static let configuration = CommandConfiguration(
             abstract: "Lists test methods in specifier format"
         )
@@ -712,16 +725,36 @@ extension SwiftTestCommand {
         @OptionGroup()
         var testEventStreamOptions: TestEventStreamOptions
 
+        @OptionGroup(visibility: .hidden)
+        package var traits: TraitOptions
+
         // for deprecated passthrough from SwiftTestTool (parse will fail otherwise)
         @Flag(name: [.customLong("list-tests"), .customShort("l")], help: .hidden)
         var _deprecated_passthrough: Bool = false
 
-        func run(_ swiftCommandState: SwiftCommandState) throws {
+        func run(_ swiftCommandState: SwiftCommandState) async throws {
+            do {
+                try await self.runCommand(swiftCommandState)
+            } catch let error as FileSystemError {
+                if sharedOptions.shouldSkipBuilding {
+                    throw ErrorWithContext(error, """
+                        Test build artifacts were not found in the build folder.
+                        The `--skip-build` flag was provided; either build the tests first with \
+                        `swift build --build tests` or rerun the `swift test list` command without \
+                        `--skip-build`
+                        """
+                    )
+                }
+                throw error
+            }
+        }
+
+        func runCommand(_ swiftCommandState: SwiftCommandState) async throws {
             let (productsBuildParameters, toolsBuildParameters) = try swiftCommandState.buildParametersForTest(
                 enableCodeCoverage: false,
                 shouldSkipBuilding: sharedOptions.shouldSkipBuilding
             )
-            let testProducts = try buildTestsIfNeeded(
+            let testProducts = try await buildTestsIfNeeded(
                 swiftCommandState: swiftCommandState,
                 productsBuildParameters: productsBuildParameters,
                 toolsBuildParameters: toolsBuildParameters
@@ -730,7 +763,7 @@ extension SwiftTestCommand {
             let toolchain = try swiftCommandState.getTargetToolchain()
             let testEnv = try TestingSupport.constructTestEnvironment(
                 toolchain: toolchain,
-                buildParameters: productsBuildParameters,
+                destinationBuildParameters: productsBuildParameters,
                 sanitizers: globalOptions.build.sanitizers,
                 library: .swiftTesting
             )
@@ -773,6 +806,13 @@ extension SwiftTestCommand {
                     })
                     if result == .failure {
                         swiftCommandState.executionStatus = .failure
+                        // If the runner reports failure do a check to ensure
+                        // all the binaries are present on the file system.
+                        for path in testProducts.map(\.binaryPath) {
+                            if !swiftCommandState.fileSystem.exists(path) {
+                                throw FileSystemError(.noEntry, path)
+                            }
+                        }
                     }
                 } else if let testEntryPointPath {
                     // Cannot run Swift Testing because an entry point file was used and the developer
@@ -788,12 +828,13 @@ extension SwiftTestCommand {
             swiftCommandState: SwiftCommandState,
             productsBuildParameters: BuildParameters,
             toolsBuildParameters: BuildParameters
-        ) throws -> [BuiltTestProduct] {
-            return try Commands.buildTestsIfNeeded(
+        ) async throws -> [BuiltTestProduct] {
+            return try await Commands.buildTestsIfNeeded(
                 swiftCommandState: swiftCommandState,
                 productsBuildParameters: productsBuildParameters,
                 toolsBuildParameters: toolsBuildParameters,
-                testProduct: self.sharedOptions.testProduct
+                testProduct: self.sharedOptions.testProduct,
+                traitConfiguration: .init(traitOptions: self.traits)
             )
         }
     }
@@ -838,7 +879,7 @@ final class TestRunner {
     private let observabilityScope: ObservabilityScope
 
     /// Which testing library to use with this test run.
-    private let library: BuildParameters.Testing.Library
+    private let library: TestingLibrary
 
     /// Get the arguments used on this platform to pass test specifiers to XCTest.
     static func xctestArguments<S>(forTestSpecifiers testSpecifiers: S) -> [String] where S: Collection, S.Element == String {
@@ -868,7 +909,7 @@ final class TestRunner {
         toolchain: UserToolchain,
         testEnv: Environment,
         observabilityScope: ObservabilityScope,
-        library: BuildParameters.Testing.Library
+        library: TestingLibrary
     ) {
         self.bundlePaths = bundlePaths
         self.additionalArguments = additionalArguments
@@ -907,23 +948,31 @@ final class TestRunner {
     /// Constructs arguments to execute XCTest.
     private func args(forTestAt testPath: AbsolutePath) throws -> [String] {
         var args: [String] = []
+
+        if let runner = self.toolchain.swiftSDK.toolset.knownTools[.testRunner], let runnerPath = runner.path {
+            args.append(runnerPath.pathString)
+            args.append(contentsOf: runner.extraCLIOptions)
+            args.append(testPath.relative(to: localFileSystem.currentWorkingDirectory!).pathString)
+            args.append(contentsOf: self.additionalArguments)
+        } else {
 #if os(macOS)
-        switch library {
-        case .xctest:
-            guard let xctestPath = self.toolchain.xctestPath else {
-                throw TestError.xcodeNotInstalled
+            switch library {
+            case .xctest:
+                guard let xctestPath = self.toolchain.xctestPath else {
+                    throw TestError.xcodeNotInstalled
+                }
+                args += [xctestPath.pathString]
+            case .swiftTesting:
+                let helper = try self.toolchain.getSwiftTestingHelper()
+                args += [helper.pathString, "--test-bundle-path", testPath.pathString]
             }
-            args += [xctestPath.pathString]
-        case .swiftTesting:
-            let helper = try self.toolchain.getSwiftTestingHelper()
-            args += [helper.pathString, "--test-bundle-path", testPath.pathString]
+            args += self.additionalArguments
+            args += [testPath.pathString]
+    #else
+            args += [testPath.pathString]
+            args += self.additionalArguments
+    #endif
         }
-        args += additionalArguments
-        args += [testPath.pathString]
-#else
-        args += [testPath.pathString]
-        args += additionalArguments
-#endif
 
         if library == .swiftTesting {
             // HACK: tell the test bundle/executable that we want to run Swift Testing, not XCTest.
@@ -1097,7 +1146,7 @@ final class ParallelTestRunner {
 
         let testEnv = try TestingSupport.constructTestEnvironment(
             toolchain: self.toolchain,
-            buildParameters: self.productsBuildParameters,
+            destinationBuildParameters: self.productsBuildParameters,
             sanitizers: self.buildOptions.sanitizers,
             library: .xctest // swift-testing does not use ParallelTestRunner
         )
@@ -1450,7 +1499,7 @@ private extension Basics.Diagnostic {
 /// it duplicates the definition of this constant in its own source. Any changes
 /// to this constant in either package must be mirrored in the other.
 private var EXIT_NO_TESTS_FOUND: CInt {
-#if os(macOS) || os(Linux)
+#if os(macOS) || os(Linux) || canImport(Android)
     EX_UNAVAILABLE
 #elseif os(Windows)
     ERROR_NOT_FOUND
@@ -1467,9 +1516,11 @@ private func buildTestsIfNeeded(
     swiftCommandState: SwiftCommandState,
     productsBuildParameters: BuildParameters,
     toolsBuildParameters: BuildParameters,
-    testProduct: String?
-) throws -> [BuiltTestProduct] {
-    let buildSystem = try swiftCommandState.createBuildSystem(
+    testProduct: String?,
+    traitConfiguration: TraitConfiguration
+) async throws -> [BuiltTestProduct] {
+    let buildSystem = try await swiftCommandState.createBuildSystem(
+        traitConfiguration: traitConfiguration,
         productsBuildParameters: productsBuildParameters,
         toolsBuildParameters: toolsBuildParameters
     )
@@ -1480,10 +1531,10 @@ private func buildTestsIfNeeded(
         .allIncludingTests
     }
 
-    try buildSystem.build(subset: subset)
+    try await buildSystem.build(subset: subset)
 
     // Find the test product.
-    let testProducts = buildSystem.builtTestProducts
+    let testProducts = await buildSystem.builtTestProducts
     guard !testProducts.isEmpty else {
         if let testProduct {
             throw TestError.productIsNotTest(productName: testProduct)
