@@ -15,13 +15,13 @@ import LLBuildManifest
 import protocol TSCBasic.OutputByteStream
 
 /// Serializes an LLBuildManifest graph to a .dot file
-public struct DOTManifestSerializer {
+struct DOTManifestSerializer {
     var kindCounter = [String: Int]()
     var hasEmittedStyling = Set<String>()
     let manifest: LLBuildManifest
 
     /// Creates a serializer that will serialize the given manifest.
-    public init(manifest: LLBuildManifest) {
+    init(manifest: LLBuildManifest) {
         self.manifest = manifest
     }
 
@@ -38,11 +38,11 @@ public struct DOTManifestSerializer {
 
     /// Quote the name and escape the quotes and backslashes
     func quoteName(_ name: String) -> String {
-        return "\"" + name.replacingOccurrences(of: "\"", with: "\\\"")
-                          .replacingOccurrences(of: "\\", with: "\\\\") + "\""
+        "\"" + name.replacingOccurrences(of: "\"", with: "\\\"")
+            .replacingOccurrences(of: "\\", with: "\\\\") + "\""
     }
 
-    public mutating func writeDOT(to stream: OutputByteStream) {
+    mutating func writeDOT(to stream: OutputByteStream) {
         stream.write("digraph Jobs {\n")
         for (name, command) in manifest.commands {
             let jobName = quoteName(label(for: command))
