@@ -59,10 +59,13 @@ public struct BuildSettingCondition: Sendable {
     let platforms: [Platform]?
     /// The applicable build configuration for this build setting condition.
     let config: BuildConfiguration?
+    /// The applicable traits for this build setting condition.
+    let traits: Set<String>?
 
-    private init(platforms: [Platform]?, config: BuildConfiguration?) {
+    private init(platforms: [Platform]?, config: BuildConfiguration?, traits: Set<String>?) {
         self.platforms = platforms
         self.config = config
+        self.traits = traits
     }
 
     @available(_PackageDescription, deprecated: 5.7)
@@ -71,7 +74,23 @@ public struct BuildSettingCondition: Sendable {
         configuration: BuildConfiguration? = nil
     ) -> BuildSettingCondition {
         precondition(!(platforms == nil && configuration == nil))
-        return BuildSettingCondition(platforms: platforms, config: configuration)
+        return BuildSettingCondition(platforms: platforms, config: configuration, traits: nil)
+    }
+
+    /// Creates a build setting condition.
+    ///
+    /// - Parameters:
+    ///   - platforms: The applicable platforms for this build setting condition.
+    ///   - configuration: The applicable build configuration for this build setting condition.
+    ///   - traits: The applicable traits for this build setting condition.
+    @available(_PackageDescription, introduced: 6.1)
+    public static func when(
+        platforms: [Platform]? = nil,
+        configuration: BuildConfiguration? = nil,
+        traits: Set<String>? = nil
+    ) -> BuildSettingCondition {
+        precondition(!(platforms == nil && configuration == nil && traits == nil))
+        return BuildSettingCondition(platforms: platforms, config: configuration, traits: traits)
     }
 
     /// Creates a build setting condition.
@@ -81,7 +100,7 @@ public struct BuildSettingCondition: Sendable {
     ///   - configuration: The applicable build configuration for this build setting condition.
     @available(_PackageDescription, introduced: 5.7)
     public static func when(platforms: [Platform], configuration: BuildConfiguration) -> BuildSettingCondition {
-        BuildSettingCondition(platforms: platforms, config: configuration)
+        BuildSettingCondition(platforms: platforms, config: configuration, traits: nil)
     }
 
     /// Creates a build setting condition.
@@ -89,7 +108,7 @@ public struct BuildSettingCondition: Sendable {
     /// - Parameter platforms: The applicable platforms for this build setting condition.
     @available(_PackageDescription, introduced: 5.7)
     public static func when(platforms: [Platform]) -> BuildSettingCondition {
-        BuildSettingCondition(platforms: platforms, config: .none)
+        BuildSettingCondition(platforms: platforms, config: .none, traits: nil)
     }
 
     /// Creates a build setting condition.
@@ -97,7 +116,7 @@ public struct BuildSettingCondition: Sendable {
     /// - Parameter configuration: The applicable build configuration for this build setting condition.
     @available(_PackageDescription, introduced: 5.7)
     public static func when(configuration: BuildConfiguration) -> BuildSettingCondition {
-        BuildSettingCondition(platforms: .none, config: configuration)
+        BuildSettingCondition(platforms: .none, config: configuration, traits: nil)
     }
 }
 
@@ -367,8 +386,8 @@ public struct SwiftSetting: Sendable {
 
     /// Enable Swift interoperability with a given language.
     ///
-    /// This is useful for enabling interoperability with Swift and C++ for a given
-    /// target.
+    /// This is useful for enabling interoperability between Swift and C++ for
+    /// a given target.
     ///
     /// Enabling C++ interoperability mode might alter the way some existing
     /// C and Objective-C APIs are imported.
@@ -376,7 +395,7 @@ public struct SwiftSetting: Sendable {
     /// - Since: First available in PackageDescription 5.9.
     ///
     /// - Parameters:
-    ///   - mode: The language mode, either C or CXX.
+    ///   - mode: The language mode, either C or Cxx.
     ///   - condition: A condition that restricts the application of the build
     /// setting.
     @available(_PackageDescription, introduced: 5.9)
