@@ -42,7 +42,7 @@ public final class Target {
     }
 
     /// The different types of a target's dependency on another entity.
-    public enum Dependency {
+    public enum Dependency: Sendable {
         /// A dependency on a target.
         ///
         ///  - Parameters:
@@ -1357,11 +1357,13 @@ extension Target.Dependency {
 }
 
 /// A condition that limits the application of a target's dependency.
-public struct TargetDependencyCondition {
+public struct TargetDependencyCondition: Sendable {
     let platforms: [Platform]?
+    let traits: Set<String>?
 
-    private init(platforms: [Platform]?) {
+    private init(platforms: [Platform]?, traits: Set<String>?) {
         self.platforms = platforms
+        self.traits = traits
     }
 
     /// Creates a target dependency condition.
@@ -1374,7 +1376,7 @@ public struct TargetDependencyCondition {
     ) -> TargetDependencyCondition {
         // FIXME: This should be an error, not a precondition.
         precondition(!(platforms == nil))
-        return TargetDependencyCondition(platforms: platforms)
+        return TargetDependencyCondition(platforms: platforms, traits: nil)
     }
 
     /// Creates a target dependency condition.
@@ -1384,7 +1386,29 @@ public struct TargetDependencyCondition {
     public static func when(
         platforms: [Platform]
     ) -> TargetDependencyCondition? {
-        return !platforms.isEmpty ? TargetDependencyCondition(platforms: platforms) : .none
+        return !platforms.isEmpty ? TargetDependencyCondition(platforms: platforms, traits: nil) : .none
+    }
+
+    /// Creates a target dependency condition.
+    ///
+    /// - Parameter platforms: The applicable platforms for this target dependency condition.
+    /// - Parameter traits: The applicable traits for this target dependency condition.
+    @available(_PackageDescription, introduced: 6.1)
+    public static func when(
+        platforms: [Platform],
+        traits: Set<String>
+    ) -> TargetDependencyCondition? {
+        return TargetDependencyCondition(platforms: platforms, traits: traits)
+    }
+
+    /// Creates a target dependency condition.
+    ///
+    /// - Parameter traits: The applicable traits for this target dependency condition.
+    @available(_PackageDescription, introduced: 6.1)
+    public static func when(
+        traits: Set<String>
+    ) -> TargetDependencyCondition? {
+        return TargetDependencyCondition(platforms: nil, traits: traits)
     }
 }
 
