@@ -29,6 +29,7 @@ extension ModulesGraph {
         requiredDependencies: [PackageReference] = [],
         unsafeAllowedPackages: Set<PackageReference> = [],
         binaryArtifacts: [PackageIdentity: [String: BinaryArtifact]],
+        prebuilts: [PackageIdentity: [String: PrebuiltLibrary]],
         shouldCreateMultipleTestProducts: Bool = false,
         createREPLProduct: Bool = false,
         customPlatformsRegistry: PlatformRegistry? = .none,
@@ -39,7 +40,49 @@ extension ModulesGraph {
         productsFilter: ((Product) -> Bool)? = nil,
         modulesFilter: ((Module) -> Bool)? = nil
     ) throws -> ModulesGraph {
+        try Self.load(
+            root: root,
+            identityResolver: identityResolver,
+            additionalFileRules: additionalFileRules,
+            externalManifests: externalManifests,
+            requiredDependencies: requiredDependencies,
+            unsafeAllowedPackages: unsafeAllowedPackages,
+            binaryArtifacts: binaryArtifacts,
+            prebuilts: prebuilts,
+            shouldCreateMultipleTestProducts: shouldCreateMultipleTestProducts,
+            createREPLProduct: createREPLProduct,
+            traitConfiguration: nil,
+            customPlatformsRegistry: customPlatformsRegistry,
+            customXCTestMinimumDeploymentTargets: customXCTestMinimumDeploymentTargets,
+            testEntryPointPath: testEntryPointPath,
+            fileSystem: fileSystem,
+            observabilityScope: observabilityScope,
+            productsFilter: productsFilter,
+            modulesFilter: modulesFilter
+        )
+    }
 
+    /// Load the package graph for the given package path.
+    package static func load(
+        root: PackageGraphRoot,
+        identityResolver: IdentityResolver,
+        additionalFileRules: [FileRuleDescription] = [],
+        externalManifests: OrderedCollections.OrderedDictionary<PackageIdentity, (manifest: Manifest, fs: FileSystem)>,
+        requiredDependencies: [PackageReference] = [],
+        unsafeAllowedPackages: Set<PackageReference> = [],
+        binaryArtifacts: [PackageIdentity: [String: BinaryArtifact]],
+        prebuilts: [PackageIdentity: [String: PrebuiltLibrary]], // Product name to library mapping
+        shouldCreateMultipleTestProducts: Bool = false,
+        createREPLProduct: Bool = false,
+        traitConfiguration: TraitConfiguration? = nil,
+        customPlatformsRegistry: PlatformRegistry? = .none,
+        customXCTestMinimumDeploymentTargets: [PackageModel.Platform: PlatformVersion]? = .none,
+        testEntryPointPath: AbsolutePath? = nil,
+        fileSystem: FileSystem,
+        observabilityScope: ObservabilityScope,
+        productsFilter: ((Product) -> Bool)? = nil,
+        modulesFilter: ((Module) -> Bool)? = nil
+    ) throws -> ModulesGraph {
         let observabilityScope = observabilityScope.makeChildScope(description: "Loading Package Graph")
 
         // Create a map of the manifests, keyed by their identity.
@@ -53,43 +96,78 @@ extension ModulesGraph {
         let rootDependencies = Set(root.dependencies.compactMap{
             manifestMap[$0.identity]?.manifest
         })
-        let rootManifestNodes = root.packages.map { identity, package in
-            GraphLoadingNode(identity: identity, manifest: package.manifest, productFilter: .everything)
+
+        let rootManifestNodes = try root.packages.map { identity, package in
+            // If we have enabled traits passed then we start with those. If there are no enabled
+            // traits passed then the default traits will be used.
+            var enabledTraits = traitConfiguration?.enabledTraits
+
+            // If all traits should be enabled we just get the set of all traits of the package
+            if traitConfiguration?.enableAllTraits ?? false {
+                enabledTraits = Set(package.manifest.traits.map { $0.name })
+            }
+
+            return try GraphLoadingNode(
+                identity: identity,
+                manifest: package.manifest,
+                productFilter: .everything,
+                enabledTraits: calculateEnabledTraits(
+                    identity: identity,
+                    manifest: package.manifest,
+                    explictlyEnabledTraits: enabledTraits
+                )
+            )
         }
-        let rootDependencyNodes = root.dependencies.lazy.compactMap { dependency in
-            manifestMap[dependency.identity].map {
-                GraphLoadingNode(
+        let rootDependencyNodes = try root.dependencies.lazy.compactMap { dependency in
+            try manifestMap[dependency.identity].map {
+                try GraphLoadingNode(
                     identity: dependency.identity,
                     manifest: $0.manifest,
-                    productFilter: dependency.productFilter
+                    productFilter: dependency.productFilter,
+                    enabledTraits: []
                 )
             }
         }
         let inputManifests = (rootManifestNodes + rootDependencyNodes).map {
-            KeyedPair($0, key: $0.id)
+            KeyedPair($0, key: $0.identity)
         }
 
         // Collect the manifests for which we are going to build packages.
-        var allNodes = [GraphLoadingNode]()
+        var allNodes = OrderedDictionary<PackageIdentity, GraphLoadingNode>()
 
         let nodeSuccessorProvider = { (node: KeyedPair<GraphLoadingNode, PackageIdentity>) in
-            node.item.requiredDependencies.compactMap { dependency in
-                manifestMap[dependency.identity].map { manifest, _ in
-                    KeyedPair(
-                        GraphLoadingNode(
-                            identity: dependency.identity,
-                            manifest: manifest,
-                            productFilter: dependency.productFilter
-                        ),
-                        key: dependency.identity
-                    )
+            return try node.item.requiredDependencies.compactMap { dependency in
+                return try manifestMap[dependency.identity].map { manifest, _ in
+                    // We are going to check the conditionally enabled traits here and enable them if
+                    // required. This checks the current node and then enables the conditional
+                    // dependencies of the dependency node.
+                    let explictlyEnabledTraits = dependency.traits?.filter {
+                        guard let conditionTraits = $0.condition?.traits else {
+                            return true
+                        }
+                        return !conditionTraits.intersection(node.item.enabledTraits).isEmpty
+                    }.map { $0.name }
+
+                    return try KeyedPair(
+                            GraphLoadingNode(
+                                identity: dependency.identity,
+                                manifest: manifest,
+                                productFilter: dependency.productFilter,
+                                enabledTraits: calculateEnabledTraits(
+                                    identity: dependency.identity,
+                                    manifest: manifest,
+                                    explictlyEnabledTraits: explictlyEnabledTraits.flatMap { Set($0) }
+                                )
+                            ),
+                            key: dependency.identity
+                        )
                 }
             }
         }
 
         // Package dependency cycles feature is gated on tools version 6.0.
         if !root.manifests.allSatisfy({ $1.toolsVersion >= .v6_0 }) {
-            if let cycle = findCycle(inputManifests, successors: nodeSuccessorProvider) {
+            if let cycle = try findCycle(inputManifests, successors: nodeSuccessorProvider) {
                 let path = (cycle.path + cycle.cycle).map(\.item.manifest)
                 observabilityScope.emit(PackageGraphError.dependencyCycleDetected(
                     path: path, cycle: cycle.cycle[0].item.manifest
@@ -106,20 +184,19 @@ extension ModulesGraph {
         }
 
         // Cycles in dependencies don't matter as long as there are no module cycles between packages.
-        depthFirstSearch(
+        try depthFirstSearch(
             inputManifests,
             successors: nodeSuccessorProvider
         ) {
-            allNodes.append($0.item)
-        } onDuplicate: { _,_ in
-            // no de-duplication is required.
+            allNodes[$0.key] = $0.item
+        } onDuplicate: { first, second in
+            // We are unifying the enabled traits on duplicate
+            allNodes[first.key]?.enabledTraits.formUnion(second.item.enabledTraits)
         }
-        // sort by identity
-        allNodes = allNodes.sorted { $0.id < $1.id }
 
         // Create the packages.
         var manifestToPackage: [Manifest: Package] = [:]
-        for node in allNodes {
+        for node in allNodes.values {
             let nodeObservabilityScope = observabilityScope.makeChildScope(
                 description: "loading package \(node.identity)",
                 metadata: .packageMetadata(identity: node.identity, kind: node.manifest.packageKind)
@@ -138,12 +215,14 @@ extension ModulesGraph {
                     productFilter: node.productFilter,
                     path: packagePath,
                     additionalFileRules: additionalFileRules,
-                    binaryArtifacts: binaryArtifacts[node.identity] ?? [:],                
+                    binaryArtifacts: binaryArtifacts[node.identity] ?? [:],
+                    prebuilts: prebuilts,
                     shouldCreateMultipleTestProducts: shouldCreateMultipleTestProducts,
                     testEntryPointPath: testEntryPointPath,
                     createREPLProduct: manifest.packageKind.isRoot ? createREPLProduct : false,
                     fileSystem: fileSystem,
-                    observabilityScope: nodeObservabilityScope
+                    observabilityScope: nodeObservabilityScope,
+                    enabledTraits: node.enabledTraits
                 )
                 let package = try builder.construct()
                 manifestToPackage[manifest] = package
@@ -166,11 +245,12 @@ extension ModulesGraph {
 
         // Resolve dependencies and create resolved packages.
         let resolvedPackages = try createResolvedPackages(
-            nodes: allNodes,
+            nodes: Array(allNodes.values),
             identityResolver: identityResolver,
             manifestToPackage: manifestToPackage,
             rootManifests: root.manifests,
             unsafeAllowedPackages: unsafeAllowedPackages,
+            prebuilts: prebuilts,
             platformRegistry: customPlatformsRegistry ?? .default,
             platformVersionProvider: platformVersionProvider,
             fileSystem: fileSystem,
@@ -180,7 +260,12 @@ extension ModulesGraph {
         )
 
         let rootPackages = resolvedPackages.filter { root.manifests.values.contains($0.manifest) }
-        checkAllDependenciesAreUsed(packages: resolvedPackages, rootPackages, observabilityScope: observabilityScope)
+        checkAllDependenciesAreUsed(
+            packages: resolvedPackages,
+            rootPackages,
+            prebuilts: prebuilts,
+            observabilityScope: observabilityScope
+        )
 
         return try ModulesGraph(
             rootPackages: rootPackages,
@@ -195,6 +280,7 @@ extension ModulesGraph {
 private func checkAllDependenciesAreUsed(
     packages: IdentifiableSet<ResolvedPackage>,
     _ rootPackages: [ResolvedPackage],
+    prebuilts: [PackageIdentity: [String: PrebuiltLibrary]],
     observabilityScope: ObservabilityScope
 ) {
     for package in rootPackages {
@@ -210,6 +296,22 @@ private func checkAllDependenciesAreUsed(
             }
         })
 
+        // List all dependencies of modules that are guarded by a trait.
+        let traitGuardedProductDependencies = Set(package.underlying.modules.flatMap { module in
+            module.dependencies.compactMap { moduleDependency in
+                switch moduleDependency {
+                case .product(let product, let conditions):
+                    if conditions.contains(where: { $0.traitCondition != nil }) {
+                        // This is a product dependency that was enabled by a trait
+                        return product.name
+                    }
+                    return nil
+                case .module:
+                    return nil
+                }
+            }
+        })
+
         for dependencyId in package.dependencies {
             guard let dependency = packages[dependencyId] else {
                 observabilityScope.emit(.error("Unknown package: \(dependencyId)"))
@@ -233,7 +335,16 @@ private func checkAllDependenciesAreUsed(
             if dependency.products.contains(where: \.isCommandPlugin) {
                 continue
             }
-            
+
+            // Skip this check if traits are enabled since it is valid to add a dependency just
+            // to enable traits on it. This is useful if there is a transitive dependency in the graph
+            // that can be configured by enabling traits e.g. the depdency has a trait for its logging
+            // behaviour. This allows the root package to configure traits of transitive dependencies
+            // without emitting an unused dependency warning.
+            if !dependency.enabledTraits.isEmpty {
+                continue
+            }
+
             // Make sure that any diagnostics we emit below are associated with the package.
             let packageDiagnosticsScope = observabilityScope.makeChildScope(
                 description: "Package Dependency Validation",
@@ -244,7 +355,13 @@ private func checkAllDependenciesAreUsed(
             let dependencyIsUsed = dependency.products.contains { product in
                 // Don't compare by product ID, but by product name to make sure both build triples as properties of
                 // `ResolvedProduct.ID` are allowed.
-                productDependencies.contains { $0.name == product.name }
+                let usedByPackage = productDependencies.contains { $0.name == product.name }
+                // We check if any of the products of this dependency is guarded by a trait.
+                let traitGuarded = traitGuardedProductDependencies.contains(product.name)
+                // Consider prebuilts as used
+                let prebuilt = prebuilts[dependency.identity]?.keys.contains(product.name) ?? false
+
+                return usedByPackage || traitGuarded || prebuilt
             }
 
             if !dependencyIsUsed && !observabilityScope.errorsReportedInAnyScope {
@@ -272,6 +389,7 @@ private func createResolvedPackages(
     // FIXME: This shouldn't be needed once <rdar://problem/33693433> is fixed.
     rootManifests: [PackageIdentity: Manifest],
     unsafeAllowedPackages: Set<PackageReference>,
+    prebuilts: [PackageIdentity: [String: PrebuiltLibrary]],
     platformRegistry: PlatformRegistry,
     platformVersionProvider: PlatformVersionProvider,
     fileSystem: FileSystem,
@@ -281,7 +399,7 @@ private func createResolvedPackages(
 ) throws -> IdentifiableSet<ResolvedPackage> {
 
     // Create package builder objects from the input manifests.
-    let packageBuilders: [ResolvedPackageBuilder] = nodes.compactMap{ node in
+    let packageBuilders: [ResolvedPackageBuilder] = nodes.compactMap { node in
         guard let package = manifestToPackage[node.manifest] else {
             return nil
         }
@@ -291,6 +409,7 @@ private func createResolvedPackages(
         return ResolvedPackageBuilder(
             package,
             productFilter: node.productFilter,
+            enabledTraits: node.enabledTraits,
             isAllowedToVendUnsafeProducts: isAllowedToVendUnsafeProducts,
             allowedToOverride: allowedToOverride,
             platformVersionProvider: platformVersionProvider
@@ -323,8 +442,9 @@ private func createResolvedPackages(
         var dependenciesByNameForModuleDependencyResolution = [String: ResolvedPackageBuilder]()
         var dependencyNamesForModuleDependencyResolutionOnly = [PackageIdentity: String]()
 
-        // Establish the manifest-declared package dependencies.
-        package.manifest.dependenciesRequired(for: packageBuilder.productFilter).forEach { dependency in
+        package.manifest.dependenciesRequired(
+            for: packageBuilder.productFilter
+        ).forEach { dependency in
             let dependencyPackageRef = dependency.packageRef
 
             // Otherwise, look it up by its identity.
@@ -551,6 +671,31 @@ private func createResolvedPackages(
 
             // Establish product dependencies.
             for case .product(let productRef, let conditions) in moduleBuilder.module.dependencies {
+                if let traitCondition = conditions.compactMap({ $0.traitCondition }).first {
+                    if packageBuilder.enabledTraits.intersection(traitCondition.traits).isEmpty {
+                        ///  If we land here non of the traits required to enable this depenendcy has been enabled.
+                        continue
+                    }
+                }
+
+                if let package = productRef.package, prebuilts[.plain(package)]?[productRef.name] != nil {
+                    // See if we're using a prebuilt instead
+                    if moduleBuilder.module.type == .macro {
+                        continue
+                    } else if moduleBuilder.module.type == .test {
+                        // use prebuilt if this is a test that depends a macro target
+                        // these are guaranteed built for host
+                        if moduleBuilder.module.dependencies.contains(where: { dep in
+                            guard let module = dep.module else {
+                                return false
+                            }
+                            return module.type == .macro
+                        }) {
+                            continue
+                        }
+                    }
+                }
+
                 // Find the product in this package's dependency products.
                 // Look it up by ID if module aliasing is used, otherwise by name.
                 let product = lookupByProductIDs ? productDependencyMap[productRef.identity] : productDependencyMap[productRef.name]
@@ -583,7 +728,7 @@ private func createResolvedPackages(
                             dependencyProductName: productRef.name,
                             dependencyPackageName: productRef.package,
                             dependencyProductInDecl: !declProductsAsDependency.isEmpty,
-                            similarProductName: bestMatchedProductName,
+                            similarProductName: bestMatchedProductName, 
                             packageContainingSimilarProduct: packageContainingBestMatchedProduct
                         )
                         packageObservabilityScope.emit(error)
@@ -650,7 +795,7 @@ private func createResolvedPackages(
                 case (.some(let registryIdentity), .none):
                     observabilityScope.emit(
                         ModuleError.duplicateModulesScmAndRegistry(
-                            regsitryPackage: registryIdentity,
+                            registryPackage: registryIdentity,
                             scmPackage: potentiallyDuplicatePackage.key.package2.identity,
                             modules: potentiallyDuplicatePackage.value
                         )
@@ -658,7 +803,7 @@ private func createResolvedPackages(
                 case (.none, .some(let registryIdentity)):
                     observabilityScope.emit(
                         ModuleError.duplicateModulesScmAndRegistry(
-                            regsitryPackage: registryIdentity,
+                            registryPackage: registryIdentity,
                             scmPackage: potentiallyDuplicatePackage.key.package1.identity,
                             modules: potentiallyDuplicatePackage.value
                         )
@@ -731,6 +876,48 @@ private func emitDuplicateProductDiagnostic(
     )
 }
 
+private func calculateEnabledTraits(
+    identity: PackageIdentity,
+    manifest: Manifest,
+    explictlyEnabledTraits: Set<String>?
+) throws -> Set<String> {
+    // This the point where we flatten the enabled traits and resolve the recursive traits
+    var recursiveEnabledTraits = explictlyEnabledTraits ?? []
+    let areDefaultsEnabled = recursiveEnabledTraits.remove("default") != nil
+
+    // We are going to calculate which traits are actually enabled for a node here. To do this
+    // we have to check if default traits should be used and then flatten all the enabled traits.
+    for trait in recursiveEnabledTraits {
+        // Check if the enabled trait is a valid trait
+        if manifest.traits.first(where: { $0.name == trait }) == nil {
+            // The enabled trait is invalid
+            throw ModuleError.invalidTrait(package: identity, trait: trait)
+        }
+    }
+
+    // We have to enable all default traits if no traits are enabled or the defaults are explicitly enabled
+    if explictlyEnabledTraits == nil || areDefaultsEnabled {
+        recursiveEnabledTraits.formUnion(manifest.traits.first { $0.name == "default" }?.enabledTraits ?? [])
+    }
+
+    while true {
+        let flattendEnabledTraits = Set(manifest.traits
+            .lazy
+            .filter { recursiveEnabledTraits.contains($0.name) }
+            .map { $0.enabledTraits }
+            .joined()
+        )
+        let newRecursiveEnabledTraits = recursiveEnabledTraits.union(flattendEnabledTraits)
+        if newRecursiveEnabledTraits.count == recursiveEnabledTraits.count {
+            break
+        } else {
+            recursiveEnabledTraits = newRecursiveEnabledTraits
+        }
+    }
+
+    return recursiveEnabledTraits
+}
+
 fileprivate extension Package {
     var doesNotSupportProductAliases: Bool {
         // We can never use the identity based lookup for older packages because they lack the necessary information.
@@ -1075,6 +1262,9 @@ private final class ResolvedPackageBuilder: ResolvedBuilder<ResolvedPackage> {
     /// The products in this package.
     var products: [ResolvedProductBuilder] = []
 
+    /// The enabled traits of this package.
+    var enabledTraits: Set<String> = []
+
     /// The dependencies of this package.
     var dependencies: [ResolvedPackageBuilder] = []
 
@@ -1095,12 +1285,14 @@ private final class ResolvedPackageBuilder: ResolvedBuilder<ResolvedPackage> {
     init(
         _ package: Package,
         productFilter: ProductFilter,
+        enabledTraits: Set<String>,
         isAllowedToVendUnsafeProducts: Bool,
         allowedToOverride: Bool,
         platformVersionProvider: PlatformVersionProvider
     ) {
         self.package = package
         self.productFilter = productFilter
+        self.enabledTraits = enabledTraits
         self.isAllowedToVendUnsafeProducts = isAllowedToVendUnsafeProducts
         self.allowedToOverride = allowedToOverride
         self.platformVersionProvider = platformVersionProvider
@@ -1111,11 +1303,12 @@ private final class ResolvedPackageBuilder: ResolvedBuilder<ResolvedPackage> {
         var modules = products.reduce(into: IdentifiableSet()) { $0.formUnion($1.modules) }
         try modules.formUnion(self.modules.map { try $0.construct() })
 
-        return try ResolvedPackage(
+        return ResolvedPackage(
             underlying: self.package,
             defaultLocalization: self.defaultLocalization,
             supportedPlatforms: self.supportedPlatforms,
             dependencies: self.dependencies.map { $0.package.identity },
+            enabledTraits: self.enabledTraits,
             modules: modules,
             products: products,
             registryMetadata: self.registryMetadata,
