@@ -11,6 +11,7 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
+import _Concurrency
 import Dispatch
 import PackageModel
 
@@ -43,25 +44,25 @@ public protocol PackageContainer {
     var shouldInvalidatePinnedVersions: Bool { get }
 
     /// Returns true if the tools version is compatible at the given version.
-    func isToolsVersionCompatible(at version: Version) -> Bool
+    func isToolsVersionCompatible(at version: Version) async -> Bool
 
     /// Returns the tools version for the given version
-    func toolsVersion(for version: Version) throws -> ToolsVersion
+    func toolsVersion(for version: Version) async throws -> ToolsVersion
 
     /// Get the list of versions which are available for the package.
     ///
     /// The list will be returned in sorted order, with the latest version *first*.
     /// All versions will not be requested at once. Resolver will request the next one only
     /// if the previous one did not satisfy all constraints.
-    func toolsVersionsAppropriateVersionsDescending() throws -> [Version]
+    func toolsVersionsAppropriateVersionsDescending() async throws -> [Version]
 
     /// Get the list of versions in the repository sorted in the ascending order, that is the earliest
     /// version appears first.
-    func versionsAscending() throws -> [Version]
+    func versionsAscending() async throws -> [Version]
 
     /// Get the list of versions in the repository sorted in the descending order, that is the latest
     /// version appears first.
-    func versionsDescending() throws -> [Version]
+    func versionsDescending() async throws -> [Version]
 
     // FIXME: We should perhaps define some particularly useful error codes
     // here, so the resolver can handle errors more meaningfully.
@@ -74,7 +75,7 @@ public protocol PackageContainer {
     /// - Precondition: `versions.contains(version)`
     /// - Throws: If the version could not be resolved; this will abort
     ///   dependency resolution completely.
-    func getDependencies(at version: Version, productFilter: ProductFilter) throws -> [PackageContainerConstraint]
+    func getDependencies(at version: Version, productFilter: ProductFilter) async throws -> [PackageContainerConstraint]
 
     /// Fetch the declared dependencies for a particular revision.
     ///
@@ -83,28 +84,28 @@ public protocol PackageContainer {
     ///
     /// - Throws: If the revision could not be resolved; this will abort
     ///   dependency resolution completely.
-    func getDependencies(at revision: String, productFilter: ProductFilter) throws -> [PackageContainerConstraint]
+    func getDependencies(at revision: String, productFilter: ProductFilter) async throws -> [PackageContainerConstraint]
 
     /// Fetch the dependencies of an unversioned package container.
     ///
     /// NOTE: This method should not be called on a versioned container.
-    func getUnversionedDependencies(productFilter: ProductFilter) throws -> [PackageContainerConstraint]
+    func getUnversionedDependencies(productFilter: ProductFilter) async throws -> [PackageContainerConstraint]
 
     /// Get the updated identifier at a bound version.
     ///
     /// This can be used by the containers to fill in the missing information that is obtained
     /// after the container is available. The updated identifier is returned in result of the
     /// dependency resolution.
-    func loadPackageReference(at boundVersion: BoundVersion) throws -> PackageReference
+    func loadPackageReference(at boundVersion: BoundVersion) async throws -> PackageReference
 }
 
 extension PackageContainer {
-    public func reversedVersions() throws -> [Version] {
-        try self.versionsDescending()
+    public func reversedVersions() async throws -> [Version] {
+        try await self.versionsDescending()
     }
 
-    public func versionsDescending() throws -> [Version] {
-        try self.versionsAscending().reversed()
+    public func versionsDescending() async throws -> [Version] {
+        try await self.versionsAscending().reversed()
     }
 
     public var shouldInvalidatePinnedVersions: Bool {
@@ -188,13 +189,16 @@ public extension PackageContainerProvider {
         observabilityScope: ObservabilityScope,
         on queue: DispatchQueue
     ) async throws -> PackageContainer {
-        try await safe_async {
+        try await withCheckedThrowingContinuation { continuation in
             self.getContainer(
                 for: package,
                 updateStrategy: updateStrategy,
                 observabilityScope: observabilityScope,
                 on: queue,
-                completion: $0)
+                completion: {
+                    continuation.resume(with: $0)
+                }
+            )
         }
     }
 }
