@@ -14,7 +14,7 @@ import Basics
 import PackageModel
 
 extension Basics.Diagnostic {
-    static func targetHasNoSources(name: String, type: TargetDescription.TargetType, shouldSuggestRelaxedSourceDir: Bool) -> Self {
+    static func targetHasNoSources(name: String, type: TargetDescription.TargetKind, shouldSuggestRelaxedSourceDir: Bool) -> Self {
         let folderName = PackageBuilder.suggestedPredefinedSourceDirectory(type: type)
         var clauses = ["Source files for target \(name) should be located under '\(folderName)/\(name)'"]
         if shouldSuggestRelaxedSourceDir {
