@@ -87,10 +87,16 @@ public enum ModuleError: Swift.Error {
 
     /// Indicates several targets with the same name exist in a registry and scm package
     case duplicateModulesScmAndRegistry(
-        regsitryPackage: PackageIdentity.RegistryIdentity,
+        registryPackage: PackageIdentity.RegistryIdentity,
         scmPackage: PackageIdentity,
         modules: [String]
     )
+
+    /// Indicates that an invalid trait was enabled.
+    case invalidTrait(
+        package: PackageIdentity,
+        trait: String
+    )
 }
 
 extension ModuleError: CustomStringConvertible {
@@ -169,6 +175,10 @@ extension ModuleError: CustomStringConvertible {
             by activating the automatic source-control to registry replacement, or by using mirrors. \
             if they are not duplicate consider using the `moduleAliases` parameter in manifest to provide unique names
             """
+        case .invalidTrait(let package, let trait):
+            return """
+            Trait '"\(trait)"' is not declared by package '\(package)'.
+            """
         }
     }
 }
@@ -258,6 +268,32 @@ public struct BinaryArtifact {
     }
 }
 
+/// A structure representing a prebuilt library to be used instead of a source dependency
+public struct PrebuiltLibrary {
+    /// The package identity.
+    public let identity: PackageIdentity
+
+    /// The name of the binary target the artifact corresponds to.
+    public let libraryName: String
+
+    /// The path to the extracted prebuilt artifacts
+    public let path: AbsolutePath
+
+    /// The products in the library
+    public let products: [String]
+
+    /// The C modules that need their includes directory added to the include path
+    public let cModules: [String]
+
+    public init(identity: PackageIdentity, libraryName: String, path: AbsolutePath, products: [String], cModules: [String]) {
+        self.identity = identity
+        self.libraryName = libraryName
+        self.path = path
+        self.products = products
+        self.cModules = cModules
+    }
+}
+
 /// Helper for constructing a package following the convention system.
 ///
 /// The 'builder' here refers to the builder pattern and not any build system
@@ -285,6 +321,9 @@ public final class PackageBuilder {
     /// Information concerning the different downloaded or local (archived) binary target artifacts.
     private let binaryArtifacts: [String: BinaryArtifact]
 
+    /// Prebuilts that may referenced from this package's targets
+    private let prebuilts: [PackageIdentity: [Product.ID: PrebuiltLibrary]]
+
     /// Create multiple test products.
     ///
     /// If set to true, one test product will be created for each test target.
@@ -321,6 +360,9 @@ public final class PackageBuilder {
     // Caches the version we chose to build for.
     private var swiftVersionCache: SwiftLanguageVersion? = nil
 
+    /// The enabled traits of this package.
+    private let enabledTraits: Set<String>
+
     /// Create a builder for the given manifest and package `path`.
     ///
     /// - Parameters:
@@ -338,12 +380,14 @@ public final class PackageBuilder {
         path: AbsolutePath,
         additionalFileRules: [FileRuleDescription],
         binaryArtifacts: [String: BinaryArtifact],
+        prebuilts: [PackageIdentity: [String: PrebuiltLibrary]],
         shouldCreateMultipleTestProducts: Bool = false,
         testEntryPointPath: AbsolutePath? = nil,
         warnAboutImplicitExecutableTargets: Bool = true,
         createREPLProduct: Bool = false,
         fileSystem: FileSystem,
-        observabilityScope: ObservabilityScope
+        observabilityScope: ObservabilityScope,
+        enabledTraits: Set<String>
     ) {
         self.identity = identity
         self.manifest = manifest
@@ -351,6 +395,7 @@ public final class PackageBuilder {
         self.packagePath = path
         self.additionalFileRules = additionalFileRules
         self.binaryArtifacts = binaryArtifacts
+        self.prebuilts = prebuilts
         self.shouldCreateMultipleTestProducts = shouldCreateMultipleTestProducts
         self.testEntryPointPath = testEntryPointPath
         self.createREPLProduct = createREPLProduct
@@ -360,6 +405,7 @@ public final class PackageBuilder {
             metadata: .packageMetadata(identity: self.identity, kind: self.manifest.packageKind)
         )
         self.fileSystem = fileSystem
+        self.enabledTraits = enabledTraits
     }
 
     /// Build a new package following the conventions.
@@ -965,7 +1011,7 @@ public final class PackageBuilder {
             {
                 self.observabilityScope
                     .emit(
-                        warning: "'\(potentialModule.name)' was identified as an executable target given the presence of a 'main.swift' file. Starting with tools version \(ToolsVersion.v5_4) executable targets should be declared as 'executableTarget()'"
+                        warning: "'\(potentialModule.name)' was identified as an executable target given the presence of a 'main' file. Starting with tools version \(ToolsVersion.v5_4) executable targets should be declared as 'executableTarget()'"
                     )
             }
         }
@@ -1054,6 +1100,11 @@ public final class PackageBuilder {
 
         // Process each setting.
         for setting in target.settings {
+            if let traits = setting.condition?.traits, traits.intersection(self.enabledTraits).isEmpty {
+                // The setting is currently not enabled so we should skip it
+                continue
+            }
+        
             let decl: BuildSettings.Declaration
             let values: [String]
 
@@ -1183,6 +1234,44 @@ public final class PackageBuilder {
             table.add(assignment, for: decl)
         }
 
+        // For each trait we are now generating an additional define
+        for trait in self.enabledTraits {
+            var assignment = BuildSettings.Assignment()
+            assignment.values = ["\(trait)"]
+            assignment.conditions = []
+            table.add(assignment, for: .SWIFT_ACTIVE_COMPILATION_CONDITIONS)
+        }
+
+        // Add in flags for prebuilts if the target is a macro or a macro test.
+        // Currently we only support prebuilts for macros.
+        if target.type == .macro || target.isMacroTest(in: manifest) {
+            let prebuiltLibraries: [String: PrebuiltLibrary] = target.dependencies.reduce(into: .init()) {
+                guard case let .product(name: name, package: package, moduleAliases: _, condition: _) = $1,
+                      let package = package,
+                      let prebuilt = prebuilts[.plain(package)]?[name]
+                else {
+                    return
+                }
+
+                $0[prebuilt.libraryName] = prebuilt
+            }
+
+            for prebuilt in prebuiltLibraries.values {
+                let lib = prebuilt.path.appending(components: ["lib", "lib\(prebuilt.libraryName).a"]).pathString
+                var ldFlagsAssignment = BuildSettings.Assignment()
+                ldFlagsAssignment.values = [lib]
+                table.add(ldFlagsAssignment, for: .OTHER_LDFLAGS)
+
+                var includeDirs: [AbsolutePath] = [prebuilt.path.appending(component: "Modules")]
+                for cModule in prebuilt.cModules {
+                    includeDirs.append(prebuilt.path.appending(components: "include", cModule))
+                }
+                var includeAssignment = BuildSettings.Assignment()
+                includeAssignment.values = includeDirs.map({ "-I\($0.pathString)" })
+                table.add(includeAssignment, for: .OTHER_SWIFT_FLAGS)
+            }
+        }
+
         return table
     }
 
@@ -1203,6 +1292,10 @@ public final class PackageBuilder {
             conditions.append(.init(platforms: platforms))
         }
 
+        if let traits = condition?.traits {
+            conditions.append(.traits(.init(traits: traits)))
+        }
+
         return conditions
     }
 
@@ -1763,4 +1856,26 @@ extension TargetDescription {
     fileprivate var usesUnsafeFlags: Bool {
         settings.filter(\.kind.isUnsafeFlags).isEmpty == false
     }
+
+    fileprivate func isMacroTest(in manifest: Manifest) -> Bool {
+        guard self.type == .test else { return false }
+
+        return self.dependencies.contains(where: {
+            let name: String
+            switch $0 {
+            case .byName(name: let n, condition: _):
+                name = n
+            case .target(name: let n, condition: _):
+                name = n
+            default:
+                return false
+            }
+
+            guard let target = manifest.targetMap[name] else {
+                return false
+            }
+
+            return target.type == .macro
+        })
+    }
 }
