@@ -82,12 +82,12 @@ public struct Package {
         versions: [Version],
         licenseURL: URL? = nil,
         readmeURL: URL? = nil,
-        repositoryURLs: [SourceControlURL]?,
-        resources: [Resource],
-        author: Author?,
-        description: String?,
-        publishedAt: Date?,
-        signingEntity: SigningEntity?,
+        repositoryURLs: [SourceControlURL]? = nil,
+        resources: [Resource] = [],
+        author: Author? = nil,
+        description: String? = nil,
+        publishedAt: Date? = nil,
+        signingEntity: SigningEntity? = nil,
         latestVersion: Version? = nil,
         source: Source
     ) {
@@ -168,127 +168,111 @@ public struct PackageSearchClient {
 
     private func getVersionMetadata(
         package: PackageIdentity,
-        version: Version,
-        callback: @escaping (Result<Metadata, Error>) -> Void
-    ) {
-        self.registryClient.getPackageVersionMetadata(
+        version: Version
+    ) async throws -> Metadata {
+        let metadata = try await self.registryClient.getPackageVersionMetadata(
             package: package,
             version: version,
             fileSystem: self.fileSystem,
             observabilityScope: observabilityScope,
-            callbackQueue: DispatchQueue.sharedConcurrent
-        ) { result in
-            callback(result.tryMap { metadata in
-                Metadata(
-                    licenseURL: metadata.licenseURL,
-                    readmeURL: metadata.readmeURL,
-                    repositoryURLs: metadata.repositoryURLs,
-                    resources: metadata.resources.map { .init($0) },
-                    author: metadata.author.map { .init($0) },
-                    description: metadata.description,
-                    publishedAt: metadata.publishedAt,
-                    signingEntity: metadata.sourceArchive?.signingEntity
-                )
-            })
-        }
+            callbackQueue: DispatchQueue.sharedConcurrent)
+
+        return Metadata(
+            licenseURL: metadata.licenseURL,
+            readmeURL: metadata.readmeURL,
+            repositoryURLs: metadata.repositoryURLs,
+            resources: metadata.resources.map { .init($0) },
+            author: metadata.author.map { .init($0) },
+            description: metadata.description,
+            publishedAt: metadata.publishedAt,
+            signingEntity: metadata.sourceArchive?.signingEntity
+        )
     }
 
     public func findPackages(
-        _ query: String,
-        callback: @escaping (Result<[Package], Error>) -> Void
-    ) {
+        _ query: String
+    ) async throws -> [Package] {
         let identity = PackageIdentity.plain(query)
 
         // Search the package index and collections for a search term.
-        let search = { (error: Error?) in
-            self.indexAndCollections.findPackages(query) { result in
-                do {
-                    let packages = try result.get().items.map {
-                        let versions = $0.package.versions.sorted(by: >)
-                        let latestVersion = versions.first
-                        
-                        return Package(
-                            identity: $0.package.identity,
-                            location: $0.package.location,
-                            versions: $0.package.versions.map(\.version),
-                            licenseURL: $0.package.license?.url,
-                            readmeURL: $0.package.readmeURL,
-                            repositoryURLs: nil,
-                            resources: [],
-                            author: latestVersion?.author.map { .init($0) },
-                            description: latestVersion?.summary,
-                            publishedAt: latestVersion?.createdAt,
-                            signingEntity: latestVersion?.signer.map { SigningEntity(signer: $0) },
-                            latestVersion: latestVersion?.version,
-                            // this only makes sense in connection with providing versioned metadata
-                            source: .indexAndCollections(collections: $0.collections, indexes: $0.indexes)
-                        )
-                    }
-                    if packages.isEmpty, let error {
-                        // If the search result is empty and we had a previous error, emit it now.
-                        return callback(.failure(error))
-                    } else {
-                        return callback(.success(packages))
-                    }
-                } catch {
-                    return callback(.failure(error))
-                }
+        let search = { (error: Error?) async throws -> [Package] in
+            let result = try await self.indexAndCollections.findPackages(query)
+            let packages = result.items.map {
+                let versions = $0.package.versions.sorted(by: >)
+                let latestVersion = versions.first
+
+                return Package(
+                    identity: $0.package.identity,
+                    location: $0.package.location,
+                    versions: $0.package.versions.map(\.version),
+                    licenseURL: $0.package.license?.url,
+                    readmeURL: $0.package.readmeURL,
+                    repositoryURLs: nil,
+                    resources: [],
+                    author: latestVersion?.author.map { .init($0) },
+                    description: latestVersion?.summary,
+                    publishedAt: latestVersion?.createdAt,
+                    signingEntity: latestVersion?.signer.map { SigningEntity(signer: $0) },
+                    latestVersion: latestVersion?.version,
+                    // this only makes sense in connection with providing versioned metadata
+                    source: .indexAndCollections(collections: $0.collections, indexes: $0.indexes)
+                )
             }
+            if packages.isEmpty, let error {
+                // If the search result is empty and we had a previous error, emit it now.
+                throw error
+            }
+            return packages
         }
 
         // Interpret the given search term as a URL and fetch the corresponding Git repository to
         // determine the available version tags and branches. If the search term cannot be interpreted
         // as a URL or there are any errors during the process, we fall back to searching the configured
         // index or package collections.
-        let fetchStandalonePackageByURL = { (error: Error?) in
+        let fetchStandalonePackageByURL = { (error: Error?) async throws -> [Package] in
             let url = SourceControlURL(query)
-
             do {
-                try withTemporaryDirectory(removeTreeOnDeinit: true) { (tempDir: AbsolutePath) in
+                return try withTemporaryDirectory(removeTreeOnDeinit: true) { (tempDir: AbsolutePath) in
                     let tempPath = tempDir.appending(component: url.lastPathComponent)
-                    do {
-                        let repositorySpecifier = RepositorySpecifier(url: url)
-                        try self.repositoryProvider.fetch(
-                            repository: repositorySpecifier,
-                            to: tempPath,
-                            progressHandler: nil
-                        )
-                        if try self.repositoryProvider.isValidDirectory(tempPath),
-                           let repository = try self.repositoryProvider.open(
-                               repository: repositorySpecifier,
-                               at: tempPath
-                           ) as? GitRepository
-                        {
-                            let branches = try repository.getBranches()
-                            let versions = try repository.getTags().compactMap { Version($0) }
-                            let package = Package(
-                                identity: .init(url: url),
-                                location: url.absoluteString,
-                                branches: branches,
-                                versions: versions,
-                                licenseURL: nil,
-                                readmeURL: self.guessReadMeURL(
-                                    baseURL: url,
-                                    defaultBranch: try repository.getDefaultBranch()
-                                ),
-                                repositoryURLs: nil,
-                                resources: [],
-                                author: nil,
-                                description: nil,
-                                publishedAt: nil,
-                                signingEntity: nil,
-                                latestVersion: nil,
-                                // this only makes sense in connection with providing versioned metadata
-                                source: .sourceControl(url: url)
-                            )
-                            return callback(.success([package]))
-                        }
-                    } catch {
-                        return search(error)
+                    let repositorySpecifier = RepositorySpecifier(url: url)
+                    try self.repositoryProvider.fetch(
+                        repository: repositorySpecifier,
+                        to: tempPath,
+                        progressHandler: nil
+                    )
+                    guard try self.repositoryProvider.isValidDirectory(tempPath), let repository = try self.repositoryProvider.open(
+                        repository: repositorySpecifier,
+                        at: tempPath
+                    ) as? GitRepository else {
+                        return []
                     }
+
+                    let branches = try repository.getBranches()
+                    let versions = try repository.getTags().compactMap { Version($0) }
+                    let package = Package(
+                        identity: .init(url: url),
+                        location: url.absoluteString,
+                        branches: branches,
+                        versions: versions,
+                        licenseURL: nil,
+                        readmeURL: self.guessReadMeURL(
+                            baseURL: url,
+                            defaultBranch: try repository.getDefaultBranch()
+                        ),
+                        repositoryURLs: nil,
+                        resources: [],
+                        author: nil,
+                        description: nil,
+                        publishedAt: nil,
+                        signingEntity: nil,
+                        latestVersion: nil,
+                        // this only makes sense in connection with providing versioned metadata
+                        source: .sourceControl(url: url)
+                    )
+                    return [package]
                 }
             } catch {
-                return search(error)
+                return try await search(error)
             }
         }
 
@@ -296,87 +280,45 @@ public struct PackageSearchClient {
         // package metadata for it from the configured registry. If there are any errors
         // or the search term does not work as a registry identity, we will fall back on
         // `fetchStandalonePackageByURL`.
-        if identity.isRegistry {
-            return self.registryClient.getPackageMetadata(
-                package: identity,
-                observabilityScope: observabilityScope,
-                callbackQueue: DispatchQueue.sharedConcurrent
-            ) { result in
-                do {
-                    let metadata = try result.get()
-                    let versions = metadata.versions.sorted(by: >)
-
-                    // See if the latest package version has readmeURL set
-                    if let version = versions.first {
-                        self.getVersionMetadata(package: identity, version: version) { result in
-                            let licenseURL: URL?
-                            let readmeURL: URL?
-                            let repositoryURLs: [SourceControlURL]?
-                            let resources: [Package.Resource]
-                            let author: Package.Author?
-                            let description: String?
-                            let publishedAt: Date?
-                            let signingEntity: SigningEntity?
-                            if case .success(let metadata) = result {
-                                licenseURL = metadata.licenseURL
-                                readmeURL = metadata.readmeURL
-                                repositoryURLs = metadata.repositoryURLs
-                                resources = metadata.resources
-                                author = metadata.author
-                                description = metadata.description
-                                publishedAt = metadata.publishedAt
-                                signingEntity = metadata.signingEntity
-                            } else {
-                                licenseURL = nil
-                                readmeURL = self.guessReadMeURL(alternateLocations: metadata.alternateLocations)
-                                repositoryURLs = nil
-                                resources = []
-                                author = nil
-                                description = nil
-                                publishedAt = nil
-                                signingEntity = nil
-                            }
-
-                            return callback(.success([Package(
-                                identity: identity,
-                                versions: metadata.versions,
-                                licenseURL: licenseURL,
-                                readmeURL: readmeURL,
-                                repositoryURLs: repositoryURLs,
-                                resources: resources,
-                                author: author,
-                                description: description,
-                                publishedAt: publishedAt,
-                                signingEntity: signingEntity,
-                                latestVersion: version,
-                                source: .registry(url: metadata.registry.url)
-                            )]))
-                        }
-                    } else {
-                        let readmeURL: URL? = self.guessReadMeURL(alternateLocations: metadata.alternateLocations)
-                        return callback(.success([Package(
-                            identity: identity,
-                            versions: metadata.versions,
-                            licenseURL: nil,
-                            readmeURL: readmeURL,
-                            repositoryURLs: nil,
-                            resources: [],
-                            author: nil,
-                            description: nil,
-                            publishedAt: nil,
-                            signingEntity: nil,
-                            latestVersion: nil,
-                            // this only makes sense in connection with providing versioned metadata
-                            source: .registry(url: metadata.registry.url)
-                        )]))
-                    }
-                } catch {
-                    return fetchStandalonePackageByURL(error)
-                }
-            }
-        } else {
-            return fetchStandalonePackageByURL(nil)
+        guard identity.isRegistry else {
+            return try await fetchStandalonePackageByURL(nil)
         }
+        let metadata: RegistryClient.PackageMetadata
+        do {
+            metadata = try await self.registryClient.getPackageMetadata(package: identity, observabilityScope: observabilityScope, callbackQueue: DispatchQueue.sharedConcurrent)
+        } catch {
+            return try await fetchStandalonePackageByURL(error)
+        }
+
+        let versions = metadata.versions.sorted(by: >)
+
+        // See if the latest package version has readmeURL set
+        guard let version = versions.first else {
+            let readmeURL: URL? = self.guessReadMeURL(alternateLocations: metadata.alternateLocations)
+            return [Package(
+                identity: identity,
+                versions: versions,
+                readmeURL: readmeURL,
+                // this only makes sense in connection with providing versioned metadata
+                source: .registry(url: metadata.registry.url)
+            )]
+        }
+
+        let versionMetadata = try? await self.getVersionMetadata(package: identity, version: version)
+        return [Package(
+            identity: identity,
+            versions: versions,
+            licenseURL: versionMetadata?.licenseURL,
+            readmeURL: versionMetadata?.readmeURL,
+            repositoryURLs: versionMetadata?.repositoryURLs,
+            resources: versionMetadata?.resources ?? [],
+            author: versionMetadata?.author,
+            description: versionMetadata?.description,
+            publishedAt: versionMetadata?.publishedAt,
+            signingEntity: versionMetadata?.signingEntity,
+            latestVersion: version,
+            source: .registry(url: metadata.registry.url)
+        )]
     }
 
     public func lookupIdentities(
@@ -410,7 +352,7 @@ public struct PackageSearchClient {
         ) { result in
             do {
                 let metadata = try result.get()
-                let alternateLocations = metadata.alternateLocations ?? []
+                let alternateLocations = metadata.alternateLocations
                 return completion(.success(Set(alternateLocations)))
             } catch {
                 return completion(.failure(error))
