@@ -18,7 +18,7 @@ import Foundation
 /// This represents a reference to a package containing its identity and location.
 public struct PackageReference {
     /// The kind of package reference.
-    public enum Kind: Equatable, CustomStringConvertible, Sendable {
+    public enum Kind: Hashable, CustomStringConvertible, Sendable {
         /// A root package.
         case root(AbsolutePath)
 
