@@ -10,7 +10,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-import TSCBasic
+import Basics
 
 /// An individual resource file and its corresponding rule.
 public struct Resource: Codable, Equatable {
@@ -24,11 +24,13 @@ public struct Resource: Codable, Equatable {
 
     /// The relative location of the resource in the resource bundle.
     public var destination: RelativePath {
-        switch self.rule {
-        case .process(.some(let localization)):
-            return RelativePath("\(localization).\(Self.localizationDirectoryExtension)/\(path.basename)")
-        default:
-            return RelativePath(path.basename)
+        get throws {
+            switch self.rule {
+            case .process(.some(let localization)):
+                return try RelativePath(validating: "\(localization).\(Self.localizationDirectoryExtension)/\(path.basename)")
+            default:
+                return try RelativePath(validating: path.basename)
+            }
         }
     }
 
