@@ -38,7 +38,7 @@ public struct BuildParameters: Encodable {
     }
 
     /// The destination for which code should be compiled for.
-    public enum Destination: Encodable {
+    public enum Destination: Hashable, Encodable {
         /// The destination for which build tools are compiled.
         case host
 
@@ -163,7 +163,7 @@ public struct BuildParameters: Encodable {
         driverParameters: Driver = .init(),
         linkingParameters: Linking = .init(),
         outputParameters: Output = .init(),
-        testingParameters: Testing? = nil
+        testingParameters: Testing = .init()
     ) throws {
         let triple = try triple ?? .getHostTriple(usingSwiftCompiler: toolchain.swiftCompilerPath)
         self.debuggingParameters = debuggingParameters ?? .init(
@@ -218,7 +218,7 @@ public struct BuildParameters: Encodable {
         self.driverParameters = driverParameters
         self.linkingParameters = linkingParameters
         self.outputParameters = outputParameters
-        self.testingParameters = testingParameters ?? .init(configuration: configuration, targetTriple: triple)
+        self.testingParameters = testingParameters
     }
 
     /// The path to the build directory (inside the data directory).
@@ -271,28 +271,37 @@ public struct BuildParameters: Encodable {
         return try buildPath.appending(binaryRelativePath(for: product))
     }
 
+    public func macroBinaryPath(_ module: ResolvedModule) throws -> AbsolutePath {
+        assert(module.type == .macro)
+        #if BUILD_MACROS_AS_DYLIBS
+        return buildPath.appending(try dynamicLibraryPath(for: module.name))
+        #else
+        return buildPath.appending(try executablePath(for: module.name))
+        #endif
+    }
+
     /// Returns the path to the dynamic library of a product for the current build parameters.
-    func potentialDynamicLibraryPath(for product: ResolvedProduct) throws -> RelativePath {
-        try RelativePath(validating: "\(self.triple.dynamicLibraryPrefix)\(product.name)\(self.suffix)\(self.triple.dynamicLibraryExtension)")
+    private func dynamicLibraryPath(for name: String) throws -> RelativePath {
+        try RelativePath(validating: "\(self.triple.dynamicLibraryPrefix)\(name)\(self.suffix)\(self.triple.dynamicLibraryExtension)")
+    }
+
+    /// Returns the path to the executable of a product for the current build parameters.
+    package func executablePath(for name: String) throws -> RelativePath {
+        try RelativePath(validating: "\(name)\(self.suffix)\(self.triple.executableExtension)")
     }
 
     /// Returns the path to the binary of a product for the current build parameters, relative to the build directory.
     public func binaryRelativePath(for product: ResolvedProduct) throws -> RelativePath {
-        let potentialExecutablePath = try RelativePath(validating: "\(product.name)\(self.suffix)\(self.triple.executableExtension)")
-
         switch product.type {
         case .executable, .snippet:
-            return potentialExecutablePath
+            return try executablePath(for: product.name)
         case .library(.static):
             return try RelativePath(validating: "lib\(product.name)\(self.suffix)\(self.triple.staticLibraryExtension)")
         case .library(.dynamic):
-            return try potentialDynamicLibraryPath(for: product)
+            return try dynamicLibraryPath(for: product.name)
         case .library(.automatic), .plugin:
             fatalError()
         case .test:
-            guard !self.triple.isWasm else {
-                return try RelativePath(validating: "\(product.name).wasm")
-            }
             let base = "\(product.name).xctest"
             if self.triple.isDarwin() {
                 return try RelativePath(validating: "\(base)/Contents/MacOS/\(product.name)")
@@ -301,9 +310,9 @@ public struct BuildParameters: Encodable {
             }
         case .macro:
             #if BUILD_MACROS_AS_DYLIBS
-            return try potentialDynamicLibraryPath(for: product)
+            return try dynamicLibraryPath(for: product.name)
             #else
-            return potentialExecutablePath
+            return try executablePath(for: product.name)
             #endif
         }
     }
@@ -335,20 +344,6 @@ private struct _Toolchain: Encodable {
     }
 }
 
-extension BuildParameters {
-    /// Whether to build Swift code with whole module optimization (WMO)
-    /// enabled.
-    public var useWholeModuleOptimization: Bool {
-        switch configuration {
-        case .debug:
-            return false
-
-        case .release:
-            return true
-        }
-    }
-}
-
 extension Triple {
     public var supportsTestSummary: Bool {
         return !self.isWindows()
