@@ -27,7 +27,7 @@ public final class InitPackage {
         public var packageType: PackageType
 
         /// The set of supported testing libraries to include in the package.
-        public var supportedTestingLibraries: Set<BuildParameters.Testing.Library>
+        public var supportedTestingLibraries: Set<TestingLibrary>
 
         /// The list of platforms in the manifest.
         ///
@@ -36,7 +36,7 @@ public final class InitPackage {
 
         public init(
             packageType: PackageType,
-            supportedTestingLibraries: Set<BuildParameters.Testing.Library>,
+            supportedTestingLibraries: Set<TestingLibrary>,
             platforms: [SupportedPlatform] = []
         ) {
             self.packageType = packageType
@@ -93,7 +93,7 @@ public final class InitPackage {
     public convenience init(
         name: String,
         packageType: PackageType,
-        supportedTestingLibraries: Set<BuildParameters.Testing.Library>,
+        supportedTestingLibraries: Set<TestingLibrary>,
         destinationPath: AbsolutePath,
         installedSwiftPMConfiguration: InstalledSwiftPMConfiguration,
         fileSystem: FileSystem
@@ -887,7 +887,7 @@ public final class InitPackage {
 
 private enum InitError: Swift.Error {
     case manifestAlreadyExists
-    case unsupportedTestingLibraryForPackageType(_ testingLibrary: BuildParameters.Testing.Library, _ packageType: InitPackage.PackageType)
+    case unsupportedTestingLibraryForPackageType(_ testingLibrary: TestingLibrary, _ packageType: InitPackage.PackageType)
 }
 
 extension InitError: CustomStringConvertible {
