@@ -77,6 +77,11 @@ extension Workspace {
             self.scratchDirectory.appending("artifacts")
         }
 
+        /// Path to the downloaded prebuilts directory
+        public var prebuiltsDirectory: AbsolutePath {
+            self.scratchDirectory.appending("prebuilts")
+        }
+
         // Path to temporary files related to running plugins in the workspace
         public var pluginWorkingDirectory: AbsolutePath {
             self.scratchDirectory.appending("plugins")
@@ -149,6 +154,11 @@ extension Workspace {
             self.sharedCacheDirectory.map { $0.appending("artifacts") }
         }
 
+        /// Path to the shared prebuilts cache
+        public var sharedPrebuiltsCacheDirectory: AbsolutePath? {
+            self.sharedCacheDirectory.map { $0.appending("prebuilts")}
+        }
+
         /// Create a new workspace location.
         ///
         /// - Parameters:
@@ -779,6 +789,15 @@ public struct WorkspaceConfiguration {
     /// Whether or not there should be import restrictions applied when loading manifests
     public var manifestImportRestrictions: (startingToolsVersion: ToolsVersion, allowedImports: [String])?
 
+    /// Whether or not to use prebuilt swift-syntax for macros
+    public var usePrebuilts: Bool
+
+    /// String URL to allow override of the prebuilts download location
+    public var prebuiltsDownloadURL: String?
+
+    /// Path to root certificate used when validating the manifest signing during testing
+    public var prebuiltsRootCertPath: String?
+
     public init(
         skipDependenciesUpdates: Bool,
         prefetchBasedOnResolvedFile: Bool,
@@ -791,7 +810,10 @@ public struct WorkspaceConfiguration {
         skipSignatureValidation: Bool,
         sourceControlToRegistryDependencyTransformation: SourceControlToRegistryDependencyTransformation,
         defaultRegistry: Registry?,
-        manifestImportRestrictions: (startingToolsVersion: ToolsVersion, allowedImports: [String])?
+        manifestImportRestrictions: (startingToolsVersion: ToolsVersion, allowedImports: [String])?,
+        usePrebuilts: Bool,
+        prebuiltsDownloadURL: String?,
+        prebuiltsRootCertPath: String?
     ) {
         self.skipDependenciesUpdates = skipDependenciesUpdates
         self.prefetchBasedOnResolvedFile = prefetchBasedOnResolvedFile
@@ -805,6 +827,9 @@ public struct WorkspaceConfiguration {
         self.sourceControlToRegistryDependencyTransformation = sourceControlToRegistryDependencyTransformation
         self.defaultRegistry = defaultRegistry
         self.manifestImportRestrictions = manifestImportRestrictions
+        self.usePrebuilts = usePrebuilts
+        self.prebuiltsDownloadURL = prebuiltsDownloadURL
+        self.prebuiltsRootCertPath = prebuiltsRootCertPath
     }
 
     /// Default instance of WorkspaceConfiguration
@@ -821,7 +846,10 @@ public struct WorkspaceConfiguration {
             skipSignatureValidation: false,
             sourceControlToRegistryDependencyTransformation: .disabled,
             defaultRegistry: .none,
-            manifestImportRestrictions: .none
+            manifestImportRestrictions: .none,
+            usePrebuilts: false,
+            prebuiltsDownloadURL: nil,
+            prebuiltsRootCertPath: nil
         )
     }
 
