@@ -10,12 +10,13 @@
 //
 //===----------------------------------------------------------------------===//
 
+import _Concurrency
+
 import struct Basics.AbsolutePath
 import struct Basics.Diagnostic
 import struct Basics.InternalError
 import class Basics.ObservabilityScope
 import struct Basics.SwiftVersion
-import func Basics.temp_await
 import func Basics.depthFirstSearch
 import class Basics.ThreadSafeKeyValueStore
 import class Dispatch.DispatchGroup
@@ -136,12 +137,19 @@ extension Workspace {
                 let dependency = dependency.dependency
                 switch dependency.state {
                 case .sourceControlCheckout(let checkout):
-                    if checkout.isBranchOrRevisionBased {
-                        result.insert(dependency.packageRef)
+                    let packageRef = dependency.packageRef
+
+                    if checkout.isBranchOrRevisionBased
+                      // FIXME: Remove this once we have a general mechanism
+                      //        for passing "safe" flags.
+                      || packageRef.identity == .plain("swift-corelibs-foundation")
+                    {
+                      result.insert(packageRef)
                     }
-                case .registryDownload, .edited, .custom:
+
+                case .registryDownload, .custom:
                     continue
-                case .fileSystem:
+                case .fileSystem, .edited:
                     result.insert(dependency.packageRef)
                 }
             }
@@ -177,22 +185,26 @@ extension Workspace {
             )
 
             var inputIdentities: OrderedCollections.OrderedSet<PackageReference> = []
-            let inputNodes: [GraphLoadingNode] = root.packages.map { identity, package in
+            let inputNodes: [GraphLoadingNode] = try root.packages.map { identity, package in
                 inputIdentities.append(package.reference)
-                let node = GraphLoadingNode(
+                let node = try GraphLoadingNode(
                     identity: identity,
                     manifest: package.manifest,
-                    productFilter: .everything
+                    productFilter: .everything,
+                    // We are enabling all traits of the root packages in the workspace integration for now
+                    enabledTraits: Set(package.manifest.traits.map { $0.name })
                 )
                 return node
             } + root.dependencies.compactMap { dependency in
                 let package = dependency.packageRef
                 inputIdentities.append(package)
-                return manifestsMap[dependency.identity].map { manifest in
-                    GraphLoadingNode(
+                return try manifestsMap[dependency.identity].map { manifest in
+                    try GraphLoadingNode(
                         identity: dependency.identity,
                         manifest: manifest,
-                        productFilter: dependency.productFilter
+                        productFilter: dependency.productFilter,
+                        // We are enabling all traits of the root packages in the workspace integration for now
+                        enabledTraits: Set(manifest.traits.map { $0.name })
                     )
                 }
             }
@@ -200,8 +212,8 @@ extension Workspace {
             let topLevelDependencies = root.packages.flatMap { $1.manifest.dependencies.map(\.packageRef) }
 
             var requiredIdentities: OrderedCollections.OrderedSet<PackageReference> = []
-            _ = transitiveClosure(inputNodes) { node in
-                node.manifest.dependenciesRequired(for: node.productFilter).compactMap { dependency in
+            _ = try transitiveClosure(inputNodes) { node in
+                try node.manifest.dependenciesRequired(for: node.productFilter).compactMap { dependency in
                     let package = dependency.packageRef
                     let (inserted, index) = requiredIdentities.append(package)
                     if !inserted {
@@ -242,11 +254,13 @@ extension Workspace {
                             """)
                         }
                     }
-                    return manifestsMap[dependency.identity].map { manifest in
-                        GraphLoadingNode(
+                    return try manifestsMap[dependency.identity].map { manifest in
+                        try GraphLoadingNode(
                             identity: dependency.identity,
                             manifest: manifest,
-                            productFilter: dependency.productFilter
+                            productFilter: dependency.productFilter,
+                            // We are enabling all traits of the root packages in the workspace integration for now
+                            enabledTraits: Set(manifest.traits.map { $0.name })
                         )
                     }
                 }
@@ -370,33 +384,23 @@ extension Workspace {
     }
 
     /// Returns manifest interpreter flags for a package.
-    // TODO: should this be throwing instead?
-    public func interpreterFlags(for packagePath: AbsolutePath) -> [String] {
-        do {
-            guard let manifestLoader = self.manifestLoader as? ManifestLoader else {
-                throw StringError("unexpected manifest loader kind")
-            }
-
-            let manifestPath = try ManifestLoader.findManifest(
-                packagePath: packagePath,
-                fileSystem: self.fileSystem,
-                currentToolsVersion: self.currentToolsVersion
-            )
-            let manifestToolsVersion = try ToolsVersionParser.parse(
-                manifestPath: manifestPath,
-                fileSystem: self.fileSystem
-            )
-
-            guard self.currentToolsVersion >= manifestToolsVersion || SwiftVersion.current.isDevelopment,
-                  manifestToolsVersion >= ToolsVersion.minimumRequired
-            else {
-                throw StringError("invalid tools version")
-            }
-            return manifestLoader.interpreterFlags(for: manifestToolsVersion)
-        } catch {
-            // We ignore all failures here and return empty array.
-            return []
+    public func interpreterFlags(for manifestPath: AbsolutePath) throws -> [String] {
+        guard let manifestLoader = self.manifestLoader as? ManifestLoader else {
+            throw StringError("unexpected manifest loader kind")
         }
+
+        let manifestToolsVersion = (try? ToolsVersionParser.parse(
+            manifestPath: manifestPath,
+            fileSystem: self.fileSystem
+        )) ?? self.currentToolsVersion
+
+
+        guard self.currentToolsVersion >= manifestToolsVersion || SwiftVersion.current.isDevelopment,
+                manifestToolsVersion >= ToolsVersion.minimumRequired
+        else {
+            throw StringError("invalid tools version")
+        }
+        return manifestLoader.interpreterFlags(for: manifestToolsVersion)
     }
 
     /// Load the manifests for the current dependency tree.
@@ -407,7 +411,7 @@ extension Workspace {
         root: PackageGraphRoot,
         automaticallyAddManagedDependencies: Bool = false,
         observabilityScope: ObservabilityScope
-    ) throws -> DependencyManifests {
+    ) async throws -> DependencyManifests {
         let prepopulateManagedDependencies: ([PackageReference]) throws -> Void = { refs in
             // pre-populate managed dependencies if we are asked to do so (this happens when resolving to a resolved
             // file)
@@ -447,7 +451,9 @@ extension Workspace {
         }
 
         // Validates that all the managed dependencies are still present in the file system.
-        self.fixManagedDependencies(observabilityScope: observabilityScope)
+        await self.fixManagedDependencies(
+            observabilityScope: observabilityScope
+        )
         guard !observabilityScope.errorsReported else {
             // return partial results
             return DependencyManifests(
@@ -461,11 +467,10 @@ extension Workspace {
         // Load root dependencies manifests (in parallel)
         let rootDependencies = root.dependencies.map(\.packageRef)
         try prepopulateManagedDependencies(rootDependencies)
-        let rootDependenciesManifests = try temp_await { self.loadManagedManifests(
+        let rootDependenciesManifests = await self.loadManagedManifests(
             for: rootDependencies,
-            observabilityScope: observabilityScope,
-            completion: $0
-        ) }
+            observabilityScope: observabilityScope
+        )
 
         let topLevelManifests = root.manifests.merging(rootDependenciesManifests, uniquingKeysWith: { lhs, _ in
             lhs // prefer roots!
@@ -473,26 +478,24 @@ extension Workspace {
 
         // optimization: preload first level dependencies manifest (in parallel)
         let firstLevelDependencies = topLevelManifests.values.map { $0.dependencies.map(\.packageRef) }.flatMap { $0 }
-        let firstLevelManifests = try temp_await { self.loadManagedManifests(
+        let firstLevelManifests = await self.loadManagedManifests(
             for: firstLevelDependencies,
-            observabilityScope: observabilityScope,
-            completion: $0
-        ) } // FIXME: this should not block
+            observabilityScope: observabilityScope
+        )
 
         // Continue to load the rest of the manifest for this graph
         // Creates a map of loaded manifests. We do this to avoid reloading the shared nodes.
         var loadedManifests = firstLevelManifests
-        let successorManifests: (KeyedPair<Manifest, Key>) throws -> [KeyedPair<Manifest, Key>] = { pair in
+        let successorManifests: (KeyedPair<Manifest, Key>) async throws -> [KeyedPair<Manifest, Key>] = { pair in
             // optimization: preload manifest we know about in parallel
             let dependenciesRequired = pair.item.dependenciesRequired(for: pair.key.productFilter)
             let dependenciesToLoad = dependenciesRequired.map(\.packageRef)
                 .filter { !loadedManifests.keys.contains($0.identity) }
             try prepopulateManagedDependencies(dependenciesToLoad)
-            let dependenciesManifests = try temp_await { self.loadManagedManifests(
+            let dependenciesManifests = await self.loadManagedManifests(
                 for: dependenciesToLoad,
-                observabilityScope: observabilityScope,
-                completion: $0
-            ) }
+                observabilityScope: observabilityScope
+            )
             dependenciesManifests.forEach { loadedManifests[$0.key] = $0.value }
             return dependenciesRequired.compactMap { dependency in
                 loadedManifests[dependency.identity].flatMap {
@@ -520,7 +523,7 @@ extension Workspace {
             }
 
             var deduplication = [PackageIdentity: Int]()
-            try depthFirstSearch(
+            try await depthFirstSearch(
                 manifestGraphRoots,
                 successors: successorManifests
             ) { pair in
@@ -549,23 +552,19 @@ extension Workspace {
             }
         }
 
-        let dependencies = try dependencyManifests.map { identity, manifest, productFilter -> (
-            Manifest,
-            ManagedDependency,
-            ProductFilter,
-            FileSystem
-        ) in
+        var dependencies: [(Manifest, ManagedDependency, ProductFilter, FileSystem)] = []
+        for (identity, manifest, productFilter) in dependencyManifests {
             guard let dependency = self.state.dependencies[identity] else {
                 throw InternalError("dependency not found for \(identity) at \(manifest.packageLocation)")
             }
 
             let packageRef = PackageReference(identity: identity, kind: manifest.packageKind)
-            let fileSystem = try self.getFileSystem(
+            let fileSystem = try await self.getFileSystem(
                 package: packageRef,
                 state: dependency.state,
                 observabilityScope: observabilityScope
             )
-            return (manifest, dependency, productFilter, fileSystem ?? self.fileSystem)
+            dependencies.append((manifest, dependency, productFilter, fileSystem ?? self.fileSystem))
         }
 
         return DependencyManifests(
@@ -577,41 +576,41 @@ extension Workspace {
     }
 
     /// Loads the given manifests, if it is present in the managed dependencies.
+    ///
+
     private func loadManagedManifests(
         for packages: [PackageReference],
-        observabilityScope: ObservabilityScope,
-        completion: @escaping (Result<[PackageIdentity: Manifest], Error>) -> Void
-    ) {
-        let sync = DispatchGroup()
-        let manifests = ThreadSafeKeyValueStore<PackageIdentity, Manifest>()
-        Set(packages).forEach { package in
-            sync.enter()
-            self.loadManagedManifest(for: package, observabilityScope: observabilityScope) { manifest in
-                defer { sync.leave() }
-                if let manifest {
-                    manifests[package.identity] = manifest
+        observabilityScope: ObservabilityScope
+    ) async -> [PackageIdentity: Manifest] {
+        await withTaskGroup(of: (PackageIdentity, Manifest?).self) { group in
+            for package in Set(packages) {
+                group.addTask {
+                    await (
+                        package.identity,
+                        self.loadManagedManifest(for: package, observabilityScope: observabilityScope)
+                    )
                 }
             }
-        }
-
-        sync.notify(queue: .sharedConcurrent) {
-            completion(.success(manifests.get()))
+            return await group.compactMap {
+                $0 as? (PackageIdentity, Manifest)
+            }.reduce(into: [PackageIdentity:Manifest]()) { partialResult, loadedManifest in
+                partialResult[loadedManifest.0] = loadedManifest.1
+            }
         }
     }
 
     /// Loads the given manifest, if it is present in the managed dependencies.
     private func loadManagedManifest(
         for package: PackageReference,
-        observabilityScope: ObservabilityScope,
-        completion: @escaping (Manifest?) -> Void
-    ) {
+        observabilityScope: ObservabilityScope
+    ) async -> Manifest? {
         // Check if this dependency is available.
         // we also compare the location as this function may attempt to load
         // dependencies that have the same identity but from a different location
         // which is an error case we diagnose an report about in the GraphLoading part which
         // is prepared to handle the case where not all manifest are available
         guard let managedDependency = self.state.dependencies[comparingLocation: package] else {
-            return completion(.none)
+            return nil
         }
 
         // Get the path of the package.
@@ -642,7 +641,7 @@ extension Workspace {
 
         let fileSystem: FileSystem?
         do {
-            fileSystem = try self.getFileSystem(
+            fileSystem = try await self.getFileSystem(
                 package: package,
                 state: managedDependency.state,
                 observabilityScope: observabilityScope
@@ -657,17 +656,18 @@ extension Workspace {
         }
 
         // Load and return the manifest.
-        self.loadManifest(
-            packageIdentity: managedDependency.packageRef.identity,
-            packageKind: packageKind,
-            packagePath: packagePath,
-            packageLocation: managedDependency.packageRef.locationString,
-            packageVersion: packageVersion,
-            fileSystem: fileSystem,
-            observabilityScope: observabilityScope
-        ) { result in
-            // error is added to diagnostics in the function above
-            completion(try? result.get())
+        return await withCheckedContinuation { continuation in
+            self.loadManifest(
+                packageIdentity: managedDependency.packageRef.identity,
+                packageKind: packageKind,
+                packagePath: packagePath,
+                packageLocation: managedDependency.packageRef.locationString,
+                packageVersion: packageVersion,
+                fileSystem: fileSystem,
+                observabilityScope: observabilityScope
+            ) { result in
+                continuation.resume(returning: try? result.get())
+            }
         }
     }
 
@@ -763,7 +763,9 @@ extension Workspace {
     /// If some checkout dependency is removed form the file system, clone it again.
     /// If some edited dependency is removed from the file system, mark it as unedited and
     /// fallback on the original checkout.
-    private func fixManagedDependencies(observabilityScope: ObservabilityScope) {
+    private func fixManagedDependencies(
+        observabilityScope: ObservabilityScope
+    ) async {
         // Reset managed dependencies if the state file was removed during the lifetime of the Workspace object.
         if !self.state.dependencies.isEmpty && !self.state.stateFileExists() {
             try? self.state.reset()
@@ -772,7 +774,7 @@ extension Workspace {
         // Make a copy of dependencies as we might mutate them in the for loop.
         let allDependencies = Array(self.state.dependencies)
         for dependency in allDependencies {
-            observabilityScope.makeChildScope(
+            await observabilityScope.makeChildScope(
                 description: "copying managed dependencies",
                 metadata: dependency.packageRef.diagnosticsMetadata
             ).trap {
@@ -785,7 +787,7 @@ extension Workspace {
                 switch dependency.state {
                 case .sourceControlCheckout(let checkoutState):
                     // If some checkout dependency has been removed, retrieve it again.
-                    _ = try self.checkoutRepository(
+                    _ = try await self.checkoutRepository(
                         package: dependency.packageRef,
                         at: checkoutState,
                         observabilityScope: observabilityScope
@@ -795,7 +797,7 @@ extension Workspace {
 
                 case .registryDownload(let version):
                     // If some downloaded dependency has been removed, retrieve it again.
-                    _ = try self.downloadRegistryArchive(
+                    _ = try await self.downloadRegistryArchive(
                         package: dependency.packageRef,
                         at: version,
                         observabilityScope: observabilityScope
@@ -804,15 +806,12 @@ extension Workspace {
                         .emit(.registryDependencyMissing(packageName: dependency.packageRef.identity.description))
 
                 case .custom(let version, let path):
-                    let container = try temp_await {
-                        self.packageContainerProvider.getContainer(
-                            for: dependency.packageRef,
-                            updateStrategy: .never,
-                            observabilityScope: observabilityScope,
-                            on: .sharedConcurrent,
-                            completion: $0
-                        )
-                    }
+                    let container = try await self.packageContainerProvider.getContainer(
+                        for: dependency.packageRef,
+                        updateStrategy: .never,
+                        observabilityScope: observabilityScope,
+                        on: .sharedConcurrent
+                    )
                     if let customContainer = container as? CustomPackageContainer {
                         let newPath = try customContainer.retrieve(at: version, observabilityScope: observabilityScope)
                         observabilityScope
@@ -832,7 +831,11 @@ extension Workspace {
                     // Note: We don't resolve the dependencies when unediting
                     // here because we expect this method to be called as part
                     // of some other resolve operation (i.e. resolve, update, etc).
-                    try self.unedit(dependency: dependency, forceRemove: true, observabilityScope: observabilityScope)
+                    try await self.unedit(
+                        dependency: dependency,
+                        forceRemove: true,
+                        observabilityScope: observabilityScope
+                    )
 
                     observabilityScope
                         .emit(.editedDependencyMissing(packageName: dependency.packageRef.identity.description))
@@ -849,7 +852,7 @@ extension Workspace {
         package: PackageReference,
         state: Workspace.ManagedDependency.State,
         observabilityScope: ObservabilityScope
-    ) throws -> FileSystem? {
+    ) async throws -> FileSystem? {
         // Only custom containers may provide a file system.
         guard self.customPackageContainerProvider != nil else {
             return nil
@@ -860,13 +863,15 @@ extension Workspace {
         case .fileSystem:
             return nil
         case .custom:
-            let container = try temp_await {
+            let container = try await withCheckedThrowingContinuation { continuation in
                 self.packageContainerProvider.getContainer(
                     for: package,
                     updateStrategy: .never,
                     observabilityScope: observabilityScope,
                     on: .sharedConcurrent,
-                    completion: $0
+                    completion: {
+                        continuation.resume(with: $0)
+                    }
                 )
             }
             guard let customContainer = container as? CustomPackageContainer else {
