@@ -12,6 +12,7 @@
 
 import ArgumentParser
 import Basics
+import _Concurrency
 import Build
 import Dispatch
 
@@ -29,13 +30,16 @@ import XCBuildSupport
 import struct TSCBasic.KeyedPair
 import func TSCBasic.topologicalSort
 import var TSCBasic.stdoutStream
-
+import enum TSCBasic.GraphError
+import struct TSCBasic.OrderedSet
 import enum TSCUtility.Diagnostics
 import struct TSCUtility.Version
 
-SwiftBootstrapBuildTool.main()
+await { () async in
+    await SwiftBootstrapBuildTool.main()
+}()
 
-struct SwiftBootstrapBuildTool: ParsableCommand {
+struct SwiftBootstrapBuildTool: AsyncParsableCommand {
     static let configuration = CommandConfiguration(
         commandName: "swift-bootstrap",
         abstract: "Bootstrapping build tool, only use in the context of bootstrapping SwiftPM itself",
@@ -112,12 +116,12 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
     @Flag()
     public var useIntegratedSwiftDriver: Bool = false
 
-    /// A flag that indicates this build should check whether targets only import
+    /// An option that indicates this build should check whether targets only import
     /// their explicitly-declared dependencies
-    @Option()
+    @Option(help: "Check that targets only import their explicitly-declared dependencies")
     public var explicitTargetDependencyImportCheck: TargetDependencyImportCheckingMode = .none
 
-    enum TargetDependencyImportCheckingMode: String, Codable, ExpressibleByArgument {
+    enum TargetDependencyImportCheckingMode: String, Codable, ExpressibleByArgument, CaseIterable {
         case none
         case error
     }
@@ -158,7 +162,7 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
 
     public init() {}
 
-    public func run() throws {
+    public func run() async throws {
         do {
             let fileSystem = localFileSystem
 
@@ -187,7 +191,7 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
                 observabilityScope: observabilityScope,
                 logLevel: self.logLevel
             )
-            try builder.build(
+            try await builder.build(
                 packagePath: packagePath,
                 scratchDirectory: scratchDirectory,
                 buildSystem: self.buildSystem,
@@ -213,11 +217,6 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
         let observabilityScope: ObservabilityScope
         let logLevel: Basics.Diagnostic.Severity
 
-        static let additionalSwiftBuildFlags = [
-            "-Xfrontend", "-disable-implicit-concurrency-module-import",
-            "-Xfrontend", "-disable-implicit-string-processing-module-import"
-        ]
-
         init(fileSystem: FileSystem, observabilityScope: ObservabilityScope, logLevel: Basics.Diagnostic.Severity) throws {
             self.identityResolver = DefaultIdentityResolver()
             self.dependencyMapper = DefaultDependencyMapper(identityResolver: self.identityResolver)
@@ -246,7 +245,7 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
             useIntegratedSwiftDriver: Bool,
             explicitTargetDependencyImportCheck: TargetDependencyImportCheckingMode,
             shouldDisableLocalRpath: Bool
-        ) throws {
+        ) async throws {
             let buildSystem = try createBuildSystem(
                 packagePath: packagePath,
                 scratchDirectory: scratchDirectory,
@@ -260,7 +259,7 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
                 shouldDisableLocalRpath: shouldDisableLocalRpath,
                 logLevel: logLevel
             )
-            try buildSystem.build(subset: .allExcludingTests)
+            try await buildSystem.build(subset: .allExcludingTests)
         }
 
         func createBuildSystem(
@@ -277,7 +276,6 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
             logLevel: Basics.Diagnostic.Severity
         ) throws -> BuildSystem {
             var buildFlags = buildFlags
-            buildFlags.swiftCompilerFlags += Self.additionalSwiftBuildFlags
 
             let dataPath = scratchDirectory.appending(
                 component: self.targetToolchain.targetTriple.platformBuildPathComponent(buildSystem: buildSystem)
@@ -310,19 +308,34 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
 
             let manifestLoader = createManifestLoader(manifestBuildFlags: manifestBuildFlags)
 
-            let packageGraphLoader = {
-                try self.loadPackageGraph(packagePath: packagePath, manifestLoader: manifestLoader)
+            let asyncUnsafePackageGraphLoader = {
+                try await self.loadPackageGraph(packagePath: packagePath, manifestLoader: manifestLoader)
             }
 
             switch buildSystem {
             case .native:
+                let pluginScriptRunner = DefaultPluginScriptRunner(
+                    fileSystem: self.fileSystem,
+                    cacheDir: scratchDirectory.appending("plugin-cache"),
+                    toolchain: self.hostToolchain,
+                    extraPluginSwiftCFlags: [],
+                    enableSandbox: true,
+                    verboseOutput: self.logLevel <= .info
+                )
                 return BuildOperation(
                     // when building `swift-bootstrap`, host and target build parameters are the same
                     productsBuildParameters: buildParameters,
                     toolsBuildParameters: buildParameters,
                     cacheBuildManifest: false,
-                    packageGraphLoader: packageGraphLoader,
+                    packageGraphLoader: asyncUnsafePackageGraphLoader,
+                    pluginConfiguration: .init(
+                        scriptRunner: pluginScriptRunner,
+                        workDirectory: scratchDirectory.appending(component: "plugin-working-directory"),
+                        disableSandbox: false
+                    ),
                     scratchDirectory: scratchDirectory,
+                    // When bootrapping no special trait build configuration is used
+                    traitConfiguration: nil,
                     additionalFileRules: [],
                     pkgConfigDirectories: [],
                     outputStream: TSCBasic.stdoutStream,
@@ -333,7 +346,7 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
             case .xcode:
                 return try XcodeBuildSystem(
                     buildParameters: buildParameters,
-                    packageGraphLoader: packageGraphLoader,
+                    packageGraphLoader: asyncUnsafePackageGraphLoader,
                     outputStream: TSCBasic.stdoutStream,
                     logLevel: logLevel,
                     fileSystem: self.fileSystem,
@@ -343,7 +356,7 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
         }
 
         func createManifestLoader(manifestBuildFlags: [String]) -> ManifestLoader {
-            var extraManifestFlags = manifestBuildFlags + Self.additionalSwiftBuildFlags
+            var extraManifestFlags = manifestBuildFlags
             if self.logLevel <= .info {
                 extraManifestFlags.append("-v")
             }
@@ -355,19 +368,19 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
             )
         }
 
-        func loadPackageGraph(packagePath: AbsolutePath, manifestLoader: ManifestLoader) throws -> ModulesGraph {
+        func loadPackageGraph(packagePath: AbsolutePath, manifestLoader: ManifestLoader) async throws -> ModulesGraph {
             let rootPackageRef = PackageReference(identity: .init(path: packagePath), kind: .root(packagePath))
-            let rootPackageManifest =  try temp_await { self.loadManifest(manifestLoader: manifestLoader, package: rootPackageRef, completion: $0) }
+            let rootPackageManifest =  try await self.loadManifest(manifestLoader: manifestLoader, package: rootPackageRef)
 
             var loadedManifests = [PackageIdentity: Manifest]()
             loadedManifests[rootPackageRef.identity] = rootPackageManifest
 
             // Compute the transitive closure of available dependencies.
             let input = loadedManifests.map { identity, manifest in KeyedPair(manifest, key: identity) }
-            _ = try topologicalSort(input) { pair in
+            _ = try await topologicalSort(input) { pair in
                 let dependenciesRequired = pair.item.dependenciesRequired(for: .everything)
                 let dependenciesToLoad = dependenciesRequired.map{ $0.packageRef }.filter { !loadedManifests.keys.contains($0.identity) }
-                let dependenciesManifests = try temp_await { self.loadManifests(manifestLoader: manifestLoader, packages: dependenciesToLoad, completion: $0) }
+                let dependenciesManifests = try await self.loadManifests(manifestLoader: manifestLoader, packages: dependenciesToLoad)
                 dependenciesManifests.forEach { loadedManifests[$0.key] = $0.value }
                 return dependenciesRequired.compactMap { dependency in
                     loadedManifests[dependency.identity].flatMap {
@@ -389,74 +402,50 @@ struct SwiftBootstrapBuildTool: ParsableCommand {
                     partial[item.key] = (manifest: item.value, fs: self.fileSystem)
                 },
                 binaryArtifacts: [:],
+                prebuilts: [:],
                 fileSystem: fileSystem,
-                observabilityScope: observabilityScope,
-                // Plugins can't be used in bootstrap builds, exclude those.
-                productsFilter: {
-                    $0.type != .plugin
-                },
-                modulesFilter: {
-                    $0.type != .plugin
-                }
+                observabilityScope: observabilityScope
             )
         }
 
         func loadManifests(
             manifestLoader: ManifestLoader,
-            packages: [PackageReference],
-            completion: @escaping (Result<[PackageIdentity: Manifest], Error>) -> Void
-        ) {
-            let sync = DispatchGroup()
-            let manifestsLock = NSLock()
-            var manifests = [PackageIdentity: Manifest]()
-            Set(packages).forEach { package in
-                sync.enter()
-                self.loadManifest(manifestLoader: manifestLoader, package: package) { result in
-                    defer { sync.leave() }
-                    switch result {
-                    case .success(let manifest):
-                        manifestsLock.withLock {
-                            manifests[package.identity] = manifest
-                        }
-                    case .failure(let error):
-                        return completion(.failure(error))
+            packages: [PackageReference]
+        ) async throws -> [PackageIdentity: Manifest] {
+            return try await withThrowingTaskGroup(of: (package:PackageReference, manifest:Manifest).self) { group in
+                for package in packages {
+                    group.addTask {
+                        try await (package, self.loadManifest(manifestLoader: manifestLoader, package: package))
                     }
                 }
-            }
-
-            sync.notify(queue: .sharedConcurrent) {
-                completion(.success(manifestsLock.withLock { manifests }))
+                return try await group.reduce(into: [:]) { partialResult, packageManifest in
+                    partialResult[packageManifest.package.identity] = packageManifest.manifest
+                }
             }
         }
 
         func loadManifest(
             manifestLoader: ManifestLoader,
-            package: PackageReference,
-            completion: @escaping (Result<Manifest, Error>) -> Void
-        ) {
-            do {
-                let packagePath = try AbsolutePath(validating: package.locationString) // FIXME
-                let manifestPath = packagePath.appending(component: Manifest.filename)
-                let manifestToolsVersion = try ToolsVersionParser.parse(manifestPath: manifestPath, fileSystem: fileSystem)
-                manifestLoader.load(
-                    manifestPath: manifestPath,
-                    manifestToolsVersion: manifestToolsVersion,
-                    packageIdentity: package.identity,
-                    packageKind: package.kind,
-                    packageLocation: package.locationString,
-                    packageVersion: .none,
-                    identityResolver: identityResolver,
-                    dependencyMapper: dependencyMapper,
-                    fileSystem: fileSystem,
-                    observabilityScope: observabilityScope,
-                    delegateQueue: .sharedConcurrent,
-                    callbackQueue: .sharedConcurrent,
-                    completion: completion
-                )
-            } catch {
-                completion(.failure(error))
-            }
-        }        
+            package: PackageReference
+        ) async throws -> Manifest {
+            let packagePath = try AbsolutePath(validating: package.locationString) // FIXME
+            let manifestPath = packagePath.appending(component: Manifest.filename)
+            let manifestToolsVersion = try ToolsVersionParser.parse(manifestPath: manifestPath, fileSystem: fileSystem)
+            return try await manifestLoader.load(
+                manifestPath: manifestPath,
+                manifestToolsVersion: manifestToolsVersion,
+                packageIdentity: package.identity,
+                packageKind: package.kind,
+                packageLocation: package.locationString,
+                packageVersion: .none,
+                identityResolver: identityResolver,
+                dependencyMapper: dependencyMapper,
+                fileSystem: fileSystem,
+                observabilityScope: observabilityScope,
+                delegateQueue: .sharedConcurrent,
+                callbackQueue: .sharedConcurrent
+            )
+        }
     }
 }
 
@@ -491,8 +480,52 @@ extension BuildConfiguration {
 
 #if compiler(<6.0)
 extension AbsolutePath: ExpressibleByArgument {}
-extension BuildConfiguration: ExpressibleByArgument {}
+extension BuildConfiguration: ExpressibleByArgument, CaseIterable {}
 #else
 extension AbsolutePath: @retroactive ExpressibleByArgument {}
-extension BuildConfiguration: @retroactive ExpressibleByArgument {}
+extension BuildConfiguration: @retroactive ExpressibleByArgument, CaseIterable {}
 #endif
+
+public func topologicalSort<T: Hashable>(
+    _ nodes: [T], successors: (T) async throws -> [T]
+) async throws -> [T] {
+    // Implements a topological sort via recursion and reverse postorder DFS.
+    func visit(_ node: T,
+               _ stack: inout OrderedSet<T>, _ visited: inout Set<T>, _ result: inout [T],
+               _ successors: (T) async throws -> [T]) async throws {
+        // Mark this node as visited -- we are done if it already was.
+        if !visited.insert(node).inserted {
+            return
+        }
+
+        // Otherwise, visit each adjacent node.
+        for succ in try await successors(node) {
+            guard stack.append(succ) else {
+                // If the successor is already in this current stack, we have found a cycle.
+                //
+                // FIXME: We could easily include information on the cycle we found here.
+                throw TSCBasic.GraphError.unexpectedCycle
+            }
+            try await visit(succ, &stack, &visited, &result, successors)
+            let popped = stack.removeLast()
+            assert(popped == succ)
+        }
+
+        // Add to the result.
+        result.append(node)
+    }
+
+    // FIXME: This should use a stack not recursion.
+    var visited = Set<T>()
+    var result = [T]()
+    var stack = OrderedSet<T>()
+    for node in nodes {
+        precondition(stack.isEmpty)
+        stack.append(node)
+        try await visit(node, &stack, &visited, &result, successors)
+        let popped = stack.removeLast()
+        assert(popped == node)
+    }
+
+    return result.reversed()
+}
