@@ -11,9 +11,10 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
-import TSCBasic
 import XCTest
 
+import struct TSCBasic.ByteString
+
 final class ByteStringExtensionsTests: XCTestCase {
     func testSHA256Checksum() {
         let byteString = ByteString(encodingAsUTF8: "abc")
