@@ -431,5 +431,84 @@ class NetrcTests: XCTestCase {
         XCTAssertEqual(netrc.machines[1].login, "fred")
         XCTAssertEqual(netrc.machines[1].password, "sunshine4ever")
     }
-}
 
+    func testComments() throws {
+        let content = """
+            # A comment at the beginning of the line
+            machine example.com # Another comment
+            login anonymous  # Another comment
+            password qw#erty  # Another comment
+            """
+
+        let netrc = try NetrcParser.parse(content)
+
+        let machine = netrc.machines.first
+        XCTAssertEqual(machine?.name, "example.com")
+        XCTAssertEqual(machine?.login, "anonymous")
+        XCTAssertEqual(machine?.password, "qw#erty")
+    }
+
+    // TODO: These permutation tests would be excellent swift-testing parameterized tests.
+    func testAllHashQuotingPermutations() throws {
+        let cases = [
+            ("qwerty", "qwerty"),
+            ("qwe#rty", "qwe#rty"),
+            ("\"qwe#rty\"", "qwe#rty"),
+            ("\"qwe #rty\"", "qwe #rty"),
+            ("\"qwe# rty\"", "qwe# rty"),
+        ]
+
+        for (testCase, expected) in cases {
+            let content = """
+                machine example.com
+                login \(testCase)
+                password \(testCase)
+                """
+            let netrc = try NetrcParser.parse(content)
+
+            let machine = netrc.machines.first
+            XCTAssertEqual(machine?.name, "example.com")
+            XCTAssertEqual(machine?.login, expected, "Expected login \(testCase) to parse as \(expected)")
+            XCTAssertEqual(machine?.password, expected, "Expected \(testCase) to parse as \(expected)")
+        }
+    }
+
+    func testAllCommentPermutations() throws {
+        let cases = [
+            ("qwerty   # a comment", "qwerty"),
+            ("qwe#rty   # a comment", "qwe#rty"),
+            ("\"qwe#rty\"   # a comment", "qwe#rty"),
+            ("\"qwe #rty\"   # a comment", "qwe #rty"),
+            ("\"qwe# rty\"   # a comment", "qwe# rty"),
+        ]
+
+        for (testCase, expected) in cases {
+            let content = """
+                machine example.com
+                login \(testCase)
+                password \(testCase)
+                """
+            let netrc = try NetrcParser.parse(content)
+
+            let machine = netrc.machines.first
+            XCTAssertEqual(machine?.name, "example.com")
+            XCTAssertEqual(machine?.login, expected, "Expected login \(testCase) to parse as \(expected)")
+            XCTAssertEqual(machine?.password, expected, "Expected password \(testCase) to parse as \(expected)")
+        }
+    }
+
+    func testQuotedMachine() throws {
+        let content = """
+            machine "example.com"
+            login anonymous
+            password qwerty
+            """
+
+        let netrc = try NetrcParser.parse(content)
+
+        let machine = netrc.machines.first
+        XCTAssertEqual(machine?.name, "example.com")
+        XCTAssertEqual(machine?.login, "anonymous")
+        XCTAssertEqual(machine?.password, "qwerty")
+    }
+}
