@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2015-2023 Apple Inc. and the Swift project authors
+// Copyright (c) 2015-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -13,18 +13,20 @@
 import Basics
 @testable import Build
 import LLBuildManifest
+
+@_spi(DontAdoptOutsideOfSwiftPMExposedForBenchmarksAndTestsOnly)
 import PackageGraph
+
 import PackageModel
 import struct SPMBuildCore.BuildParameters
 
 @_spi(SwiftPMInternal)
-import SPMTestSupport
+import _InternalTestSupport
 
-import class TSCBasic.InMemoryFileSystem
 import XCTest
 
 final class LLBuildManifestBuilderTests: XCTestCase {
-    func testCreateProductCommand() throws {
+    func testCreateProductCommand() async throws {
         let pkg = AbsolutePath("/pkg")
         let fs = InMemoryFileSystem(
             emptyFiles:
@@ -48,7 +50,7 @@ final class LLBuildManifestBuilderTests: XCTestCase {
 
         // macOS, release build
 
-        var plan = try mockBuildPlan(
+        var plan = try await mockBuildPlan(
             environment: BuildEnvironment(
                 platform: .macOS,
                 configuration: .release
@@ -81,7 +83,7 @@ final class LLBuildManifestBuilderTests: XCTestCase {
 
         // macOS, debug build
 
-        plan = try mockBuildPlan(
+        plan = try await mockBuildPlan(
             environment: BuildEnvironment(
                 platform: .macOS,
                 configuration: .debug
@@ -133,7 +135,7 @@ final class LLBuildManifestBuilderTests: XCTestCase {
 
         // Linux, release build
 
-        plan = try mockBuildPlan(
+        plan = try await mockBuildPlan(
             environment: BuildEnvironment(
                 platform: .linux,
                 configuration: .release
@@ -162,7 +164,7 @@ final class LLBuildManifestBuilderTests: XCTestCase {
 
         // Linux, debug build
 
-        plan = try mockBuildPlan(
+        plan = try await mockBuildPlan(
             environment: BuildEnvironment(
                 platform: .linux,
                 configuration: .debug
@@ -191,12 +193,12 @@ final class LLBuildManifestBuilderTests: XCTestCase {
     }
     
     /// Verifies that two modules with the same name but different triples don't share same build manifest keys.
-    func testToolsBuildTriple() throws {
+    func testToolsBuildTriple() async throws {
         let (graph, fs, scope) = try macrosPackageGraph()
         let productsTriple = Triple.x86_64MacOS
         let toolsTriple = Triple.arm64Linux
 
-        let plan = try BuildPlan(
+        let plan = try await BuildPlan(
             destinationBuildParameters: mockBuildParameters(
                 destination: .target,
                 shouldLinkStaticSwiftStdlib: true,
