@@ -10,6 +10,7 @@
 //
 //===----------------------------------------------------------------------===//
 
+import Foundation
 import Basics
 @testable import Commands
 @testable import CoreCommands
@@ -17,7 +18,7 @@ import PackageGraph
 import PackageLoading
 import PackageModel
 import SPMBuildCore
-import SPMTestSupport
+import _InternalTestSupport
 import Workspace
 import XCTest
 
@@ -141,7 +142,7 @@ final class BuildCommandTests: CommandsTestCase {
                     stderr.contains(
                         "error: Target A imports another target (B) in the package without declaring it a dependency."
                     ),
-                    "got stdout: \(stdout), stderr: \(stderr)"
+                    "got stdout: \(String(describing: stdout)), stderr: \(String(describing: stderr))"
                 )
             }
         }
@@ -158,7 +159,7 @@ final class BuildCommandTests: CommandsTestCase {
                     stderr.contains(
                         "warning: Target A imports another target (B) in the package without declaring it a dependency."
                     ),
-                    "got stdout: \(stdout), stderr: \(stderr)"
+                    "got stdout: \(String(describing: stdout)), stderr: \(String(describing: stderr))"
                 )
             }
         }
@@ -678,4 +679,41 @@ final class BuildCommandTests: CommandsTestCase {
             XCTAssertGreaterThan(codeCovFiles.count, 0)
         }
     }
+
+    func testFatalErrorDisplayedCorrectNumberOfTimesWhenSingleXCTestHasFatalErrorInBuildCompilation() async throws {
+        // Test for GitHub Issue #6605
+        // GIVEN we have a Swift Package that has a fatalError building the tests
+        #if compiler(>=6)
+        let expected = 0
+        #else
+        let expected = 1
+        #endif
+        try await fixture(name: "Miscellaneous/Errors/FatalErrorInSingleXCTest/TypeLibrary") { fixturePath in
+            // WHEN swift-build --build-tests is executed"
+            await XCTAssertAsyncThrowsError(try await self.execute(["--build-tests"], packagePath: fixturePath)) { error in
+                // THEN I expect a failure
+                guard case SwiftPMError.executionFailure(_, let stdout, let stderr) = error else {
+                    XCTFail("Building the package was expected to fail, but it was successful")
+                    return
+                }
+
+                let matchString = "error: fatalError"
+                let stdoutMatches = getNumberOfMatches(of: matchString, in: stdout)
+                let stderrMatches = getNumberOfMatches(of: matchString, in: stderr)
+                let actualNumMatches = stdoutMatches + stderrMatches
+
+                // AND a fatal error message is printed \(expected) times
+                XCTAssertEqual(
+                    actualNumMatches,
+                    expected,
+                    [
+                        "Actual (\(actualNumMatches)) is not as expected (\(expected))",
+                        "stdout: \(stdout.debugDescription)",
+                        "stderr: \(stderr.debugDescription)"
+                    ].joined(separator: "\n")
+                )
+            }
+        }
+    }
+
 }
