@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2014-2019 Apple Inc. and the Swift project authors
+// Copyright (c) 2014-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -12,14 +12,11 @@
 
 import Basics
 import Commands
-import SPMTestSupport
+import _InternalTestSupport
 import Workspace
 import XCTest
 
-import class TSCBasic.InMemoryFileSystem
-
 final class MultiRootSupportTests: CommandsTestCase {
-
     func testWorkspaceLoader() throws {
         let fs = InMemoryFileSystem(emptyFiles: [
             "/tmp/test/dep/Package.swift",
