@@ -18,7 +18,7 @@ import PackageModel
 import PackageRegistry
 @testable import PackageRegistryCommand
 import PackageSigning
-import SPMTestSupport
+import _InternalTestSupport
 import TSCclibc // for SPM_posix_spawn_file_actions_addchdir_np_supported
 import Workspace
 import XCTest
@@ -358,7 +358,7 @@ final class PackageRegistryCommandTests: CommandsTestCase {
 
             let workingDirectory = temporaryDirectory.appending(component: UUID().uuidString)
 
-            let archivePath = try PackageArchiver.archive(
+            let archivePath = try await PackageArchiver.archive(
                 packageIdentity: packageIdentity,
                 packageVersion: "1.3.5",
                 packageDirectory: packageDirectory,
@@ -388,7 +388,7 @@ final class PackageRegistryCommandTests: CommandsTestCase {
 
             let workingDirectory = temporaryDirectory.appending(component: UUID().uuidString)
 
-            let archivePath = try PackageArchiver.archive(
+            let archivePath = try await PackageArchiver.archive(
                 packageIdentity: packageIdentity,
                 packageVersion: "1.5.4",
                 packageDirectory: packageDirectory,
@@ -423,7 +423,7 @@ final class PackageRegistryCommandTests: CommandsTestCase {
 
             let workingDirectory = temporaryDirectory.appending(component: UUID().uuidString)
 
-            let archivePath = try PackageArchiver.archive(
+            let archivePath = try await PackageArchiver.archive(
                 packageIdentity: packageIdentity,
                 packageVersion: "0.3.1",
                 packageDirectory: packageDirectory,
@@ -716,6 +716,7 @@ final class PackageRegistryCommandTests: CommandsTestCase {
             )
         }
 
+        @Sendable
         func validateManifest(
             manifestFile: String,
             in archivePath: AbsolutePath,
