@@ -11,7 +11,7 @@
 //===----------------------------------------------------------------------===//
 
 import DriverSupport
-import SPMTestSupport
+import _InternalTestSupport
 import PackageModel
 import XCTest
 
