@@ -13,7 +13,7 @@
 import Basics
 import PackageModel
 import SourceControl
-import SPMTestSupport
+import _InternalTestSupport
 import Workspace
 import XCTest
 
@@ -339,11 +339,11 @@ final class MiscellaneousTestCase: XCTestCase {
         // Currently swift-ci does not build/install swift-driver before running
         // swift-package-manager tests which results in this test failing.
         // See the following additional discussion:
-        // - https://github.com/apple/swift/pull/69696
-        // - https://github.com/apple/swift/pull/61766
+        // - https://github.com/swiftlang/swift/pull/69696
+        // - https://github.com/swiftlang/swift/pull/61766
         // - https://github.com/swiftlang/swift-package-manager/pull/5842#issuecomment-1301632685
         try await fixture(name: "Miscellaneous/LTO/SwiftAndCTargets") { fixturePath in
-            /*let output =*/ 
+            /*let output =*/
             try await executeSwiftBuild(
                 fixturePath,
                 extraArgs: ["--experimental-lto-mode=full", "--verbose"]
@@ -423,6 +423,7 @@ final class MiscellaneousTestCase: XCTestCase {
         }
     }
 
+    @available(macOS 15, *)
     func testTestsCanLinkAgainstAsyncExecutable() async throws {
         #if compiler(<5.10)
         try XCTSkipIf(true, "skipping because host compiler doesn't have a fix for symbol conflicts yet")
@@ -662,6 +663,7 @@ final class MiscellaneousTestCase: XCTestCase {
         try await fixture(name: "Miscellaneous/RootPackageWithConditionals") { path in
             let (_, stderr) = try await SwiftPM.Build.execute(packagePath: path, env: ["SWIFT_DRIVER_SWIFTSCAN_LIB" : "/this/is/a/bad/path"])
             let errors = stderr.components(separatedBy: .newlines).filter { !$0.contains("[logging] misuse") && !$0.isEmpty }
+                                                                  .filter { !$0.contains("Unable to locate libSwiftScan") }
             XCTAssertEqual(errors, [], "unexpected errors: \(errors)")
         }
     }
