@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2014-2023 Apple Inc. and the Swift project authors
+// Copyright (c) 2014-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -11,13 +11,12 @@
 //===----------------------------------------------------------------------===//
 
 import struct Basics.AbsolutePath
+import class Basics.InMemoryFileSystem
 import class Foundation.PropertyListDecoder
 @testable import LLBuildManifest
-import SPMTestSupport
-import class TSCBasic.InMemoryFileSystem
+import _InternalTestSupport
 import XCTest
 
-
 private let testEntitlement = "test-entitlement"
 
 final class LLBuildManifestTests: XCTestCase {
