@@ -11,8 +11,9 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
+import _Concurrency
 @testable import PackageCollectionsSigning
-import SPMTestSupport
+import _InternalTestSupport
 import SwiftASN1
 import X509
 import XCTest
