@@ -11,10 +11,11 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
+import _Concurrency
 import Foundation
 import PackageCollectionsModel
 @testable import PackageCollectionsSigning
-import SPMTestSupport
+import _InternalTestSupport
 import X509
 import XCTest
 
