@@ -12,7 +12,7 @@
 import Basics
 import PackageModel
 import PackageModelSyntax
-import SPMTestSupport
+import _InternalTestSupport
 @_spi(FixItApplier) import SwiftIDEUtils
 import SwiftParser
 import SwiftSyntax
@@ -72,7 +72,8 @@ class ManifestEditTests: XCTestCase {
             identity: PackageIdentity(url: swiftSystemURL),
             nameForTargetDependencyResolutionOnly: nil,
             url: swiftSystemURL,
-            requirement: .branch("main"), productFilter: .nothing
+            requirement: .branch("main"), productFilter: .nothing,
+            traits: []
         )
 
     func testAddPackageDependencyExistingComma() throws {
@@ -99,7 +100,8 @@ class ManifestEditTests: XCTestCase {
                         identity: PackageIdentity(url: Self.swiftSystemURL),
                         nameForTargetDependencyResolutionOnly: nil,
                         url: Self.swiftSystemURL,
-                        requirement: .branch("main"), productFilter: .nothing
+                        requirement: .branch("main"), productFilter: .nothing,
+                        traits:[]
                     ),
                     to: manifest
                 )
@@ -131,7 +133,8 @@ class ManifestEditTests: XCTestCase {
                         nameForTargetDependencyResolutionOnly: nil,
                         url: Self.swiftSystemURL,
                         requirement: .exact("510.0.0"),
-                        productFilter: .nothing
+                        productFilter: .nothing,
+                        traits: []
                     ),
                     to: manifest
                 )
@@ -165,7 +168,8 @@ class ManifestEditTests: XCTestCase {
                         nameForTargetDependencyResolutionOnly: nil,
                         url: Self.swiftSystemURL,
                         requirement: .range(versionRange),
-                        productFilter: .nothing
+                        productFilter: .nothing,
+                        traits: []
                     ),
                     to: manifest
                 )
@@ -194,7 +198,8 @@ class ManifestEditTests: XCTestCase {
                         nameForTargetDependencyResolutionOnly: nil,
                         url: Self.swiftSystemURL,
                         requirement: .range(versionRange),
-                        productFilter: .nothing
+                        productFilter: .nothing,
+                        traits: []
                     ),
                     to: manifest
                 )
@@ -223,7 +228,8 @@ class ManifestEditTests: XCTestCase {
                         nameForTargetDependencyResolutionOnly: nil,
                         url: Self.swiftSystemURL,
                         requirement: .range(Version(508,0,0)..<Version(510,0,0)),
-                        productFilter: .nothing
+                        productFilter: .nothing,
+                        traits: []
                     ),
                 to: manifest
             )
@@ -462,7 +468,7 @@ class ManifestEditTests: XCTestCase {
                     // These are the targets
                     .target(name: "MyLib"),
                     .executableTarget(
-                        name: "MyProgram",
+                        name: "MyProgram target-name",
                         dependencies: [
                             .product(name: "SwiftSyntax", package: "swift-syntax"),
                             .target(name: "TargetLib"),
@@ -473,13 +479,13 @@ class ManifestEditTests: XCTestCase {
             )
             """,
             expectedAuxiliarySources: [
-                RelativePath("Sources/MyProgram/MyProgram.swift") : """
+                RelativePath("Sources/MyProgram target-name/MyProgram target-name.swift") : """
                 import MyLib
                 import SwiftSyntax
                 import TargetLib
 
                 @main
-                struct MyProgramMain {
+                struct MyProgram_target_nameMain {
                     static func main() {
                         print("Hello, world")
                     }
@@ -488,7 +494,7 @@ class ManifestEditTests: XCTestCase {
             ]) { manifest in
             try AddTarget.addTarget(
                 TargetDescription(
-                    name: "MyProgram",
+                    name: "MyProgram target-name",
                     dependencies: [
                         .product(name: "SwiftSyntax", package: "swift-syntax"),
                         .target(name: "TargetLib", condition: nil),
@@ -518,11 +524,11 @@ class ManifestEditTests: XCTestCase {
             let package = Package(
                 name: "packages",
                 dependencies: [
-                    .package(url: "https://github.com/swiftlang/swift-syntax.git", from: "600.0.0-latest"),
+                    .package(url: "https://github.com/swiftlang/swift-syntax.git", from: "601.0.0-latest"),
                 ],
                 targets: [
                     .macro(
-                        name: "MyMacro",
+                        name: "MyMacro target-name",
                         dependencies: [
                             .product(name: "SwiftCompilerPlugin", package: "swift-syntax"),
                             .product(name: "SwiftSyntaxMacros", package: "swift-syntax")
@@ -532,33 +538,33 @@ class ManifestEditTests: XCTestCase {
             )
             """,
             expectedAuxiliarySources: [
-                RelativePath("Sources/MyMacro/MyMacro.swift") : """
+                RelativePath("Sources/MyMacro target-name/MyMacro target-name.swift") : """
                 import SwiftCompilerPlugin
                 import SwiftSyntaxMacros
 
-                struct MyMacro: Macro {
+                struct MyMacro_target_name: Macro {
                     /// TODO: Implement one or more of the protocols that inherit
                     /// from Macro. The appropriate macro protocol is determined
-                    /// by the "macro" declaration that MyMacro implements.
+                    /// by the "macro" declaration that MyMacro_target_name implements.
                     /// Examples include:
                     ///     @freestanding(expression) macro --> ExpressionMacro
                     ///     @attached(member) macro         --> MemberMacro
                 }
                 """,
-                RelativePath("Sources/MyMacro/ProvidedMacros.swift") : """
+                RelativePath("Sources/MyMacro target-name/ProvidedMacros.swift") : """
                 import SwiftCompilerPlugin
 
                 @main
-                struct MyMacroMacros: CompilerPlugin {
+                struct MyMacro_target_nameMacros: CompilerPlugin {
                     let providingMacros: [Macro.Type] = [
-                        MyMacro.self,
+                        MyMacro_target_name.self,
                     ]
                 }
                 """
                 ]
         ) { manifest in
             try AddTarget.addTarget(
-                TargetDescription(name: "MyMacro", type: .macro),
+                TargetDescription(name: "MyMacro target-name", type: .macro),
                 to: manifest
             )
         }
@@ -576,17 +582,17 @@ class ManifestEditTests: XCTestCase {
             let package = Package(
                 name: "packages",
                 targets: [
-                    .testTarget(name: "MyTest"),
+                    .testTarget(name: "MyTest target-name"),
                 ]
             )
             """,
             expectedAuxiliarySources: [
-                RelativePath("Tests/MyTest/MyTest.swift") : """
+                RelativePath("Tests/MyTest target-name/MyTest target-name.swift") : """
                 import Testing
 
                 @Suite
-                struct MyTestTests {
-                    @Test("MyTest tests")
+                struct MyTest_target_nameTests {
+                    @Test("MyTest_target_name tests")
                     func example() {
                         #expect(42 == 17 + 25)
                     }
@@ -595,7 +601,7 @@ class ManifestEditTests: XCTestCase {
             ]) { manifest in
             try AddTarget.addTarget(
                 TargetDescription(
-                    name: "MyTest",
+                    name: "MyTest target-name",
                     type: .test
                 ),
                 to: manifest,
