@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2023 Apple Inc. and the Swift project authors
+// Copyright (c) 2023-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -12,11 +12,9 @@
 
 @testable import Basics
 @testable import PackageModel
-import SPMTestSupport
+import _InternalTestSupport
 import XCTest
 
-import class TSCBasic.InMemoryFileSystem
-
 private let usrBinTools = Dictionary(uniqueKeysWithValues: Toolset.KnownTool.allCases.map {
     ($0, try! AbsolutePath(validating: "/usr/bin/\($0.rawValue)"))
 })
@@ -210,4 +208,88 @@ final class ToolsetTests: XCTestCase {
             )
         )
     }
+
+    func testToolsetTargetToolchain() throws {
+        let fileSystem = InMemoryFileSystem()
+
+        for testFile in [compilersNoRoot, noValidToolsNoRoot, unknownToolsNoRoot, otherToolsNoRoot, someToolsWithRoot, someToolsWithRelativeRoot] {
+            try fileSystem.writeFileContents(testFile.path, string: testFile.json.underlying)
+        }
+
+        let hostSwiftSDK = try SwiftSDK.hostSwiftSDK(environment: [:])
+        let hostTriple = try! Triple("arm64-apple-macosx14.0")
+        let observability = ObservabilitySystem.makeForTesting()
+
+        let store = SwiftSDKBundleStore(
+            swiftSDKsDirectory: "/",
+            fileSystem: fileSystem,
+            observabilityScope: observability.topScope,
+            outputHandler: { _ in }
+        )
+
+        do {
+            let targetSwiftSDK = try SwiftSDK.deriveTargetSwiftSDK(
+                hostSwiftSDK: hostSwiftSDK,
+                hostTriple: hostTriple,
+                customToolsets: [compilersNoRoot.path],
+                store: store,
+                observabilityScope: observability.topScope,
+                fileSystem: fileSystem
+            )
+
+            let targetToolset = try Toolset(from: compilersNoRoot.path, at: fileSystem, observability.topScope)
+
+            // By default, the target SDK paths configuration is the same as the host SDK.
+            XCTAssertEqual(targetSwiftSDK.pathsConfiguration, hostSwiftSDK.pathsConfiguration)
+
+            var expectedToolset = hostSwiftSDK.toolset
+            expectedToolset.merge(with: targetToolset)
+
+            XCTAssertEqual(targetSwiftSDK.toolset, expectedToolset)
+        }
+
+        do {
+            let targetSwiftSDK = try SwiftSDK.deriveTargetSwiftSDK(
+                hostSwiftSDK: hostSwiftSDK,
+                hostTriple: hostTriple,
+                customToolsets: [someToolsWithRoot.path],
+                store: store,
+                observabilityScope: observability.topScope,
+                fileSystem: fileSystem
+            )
+
+            let targetToolset = try Toolset(from: someToolsWithRoot.path, at: fileSystem, observability.topScope)
+
+            // By default, the target SDK paths configuration is the same as the host SDK.
+            XCTAssertEqual(targetSwiftSDK.pathsConfiguration, hostSwiftSDK.pathsConfiguration)
+
+            var expectedToolset = hostSwiftSDK.toolset
+            expectedToolset.merge(with: targetToolset)
+
+            XCTAssertEqual(targetSwiftSDK.toolset, expectedToolset)
+        }
+
+        do {
+            let targetSwiftSDK = try SwiftSDK.deriveTargetSwiftSDK(
+                hostSwiftSDK: hostSwiftSDK,
+                hostTriple: hostTriple,
+                customToolsets: [compilersNoRoot.path, someToolsWithRoot.path],
+                store: store,
+                observabilityScope: observability.topScope,
+                fileSystem: fileSystem
+            )
+
+            let toolset1 = try Toolset(from: compilersNoRoot.path, at: fileSystem, observability.topScope)
+            let toolset2 = try Toolset(from: someToolsWithRoot.path, at: fileSystem, observability.topScope)
+
+            // By default, the target SDK paths configuration is the same as the host SDK.
+            XCTAssertEqual(targetSwiftSDK.pathsConfiguration, hostSwiftSDK.pathsConfiguration)
+
+            var expectedToolset = hostSwiftSDK.toolset
+            expectedToolset.merge(with: toolset1)
+            expectedToolset.merge(with: toolset2)
+
+            XCTAssertEqual(targetSwiftSDK.toolset, expectedToolset)
+        }
+    }
 }
