@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2022-2023 Apple Inc. and the Swift project authors
+// Copyright (c) 2022-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -11,17 +11,16 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
+import _Concurrency
 import PackageModel
 import PackageLoading
 @testable import PackageRegistry
-import SPMTestSupport
+import _InternalTestSupport
 import XCTest
 
-import class TSCBasic.InMemoryFileSystem
-
 import struct TSCUtility.Version
 
-class RegistryDownloadsManagerTests: XCTestCase {
+final class RegistryDownloadsManagerTests: XCTestCase {
     func testNoCache() async throws {
         let observability = ObservabilitySystem.makeForTesting()
         let fs = InMemoryFileSystem()
