@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2023 Apple Inc. and the Swift project authors
+// Copyright (c) 2023-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -15,11 +15,9 @@ import Foundation
 import Basics
 import PackageModel
 @testable import PackageSigning
-import SPMTestSupport
+import _InternalTestSupport
 import XCTest
 
-import class TSCBasic.InMemoryFileSystem
-
 import struct TSCUtility.Version
 
 final class FilePackageSigningEntityStorageTests: XCTestCase {
@@ -42,19 +40,19 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organizationalUnit: "SwiftPM Test Unit 2",
             organization: "SwiftPM Test"
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("1.0.0"),
             signingEntity: davinci,
             origin: .registry(URL("http://foo.com"))
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("1.1.0"),
             signingEntity: davinci,
             origin: .registry(URL("http://bar.com"))
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("2.0.0"),
             signingEntity: appleseed,
@@ -62,7 +60,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         )
         // Record signing entity for another package
         let otherPackage = PackageIdentity.plain("other.LinkedList")
-        try await storage.put(
+        try storage.put(
             package: otherPackage,
             version: Version("1.0.0"),
             signingEntity: appleseed,
@@ -78,7 +76,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
 
         // Signed versions should be saved
         do {
-            let packageSigners = try await storage.get(package: package)
+            let packageSigners = try storage.get(package: package)
             XCTAssertNil(packageSigners.expectedSigner)
             XCTAssertEqual(packageSigners.signers.count, 2)
             XCTAssertEqual(packageSigners.signers[davinci]?.versions, [Version("1.0.0"), Version("1.1.0")])
@@ -91,7 +89,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         }
 
         do {
-            let packageSigners = try await storage.get(package: otherPackage)
+            let packageSigners = try storage.get(package: otherPackage)
             XCTAssertNil(packageSigners.expectedSigner)
             XCTAssertEqual(packageSigners.signers.count, 1)
             XCTAssertEqual(packageSigners.signers[appleseed]?.versions, [Version("1.0.0")])
@@ -118,7 +116,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organization: "SwiftPM Test"
         )
         let version = Version("1.0.0")
-        try await storage.put(
+        try storage.put(
             package: package,
             version: version,
             signingEntity: davinci,
@@ -126,7 +124,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         )
 
         // Writing different signing entities for the same version should fail
-        await XCTAssertAsyncThrowsError(try await storage.put(
+        await XCTAssertAsyncThrowsError(try storage.put(
             package: package,
             version: version,
             signingEntity: appleseed,
@@ -151,7 +149,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organization: "SwiftPM Test"
         )
         let version = Version("1.0.0")
-        try await storage.put(
+        try storage.put(
             package: package,
             version: version,
             signingEntity: appleseed,
@@ -159,14 +157,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         )
 
         // Writing same signing entity for version should be ok
-        try await storage.put(
+        try storage.put(
             package: package,
             version: version,
             signingEntity: appleseed,
             origin: .registry(URL("http://bar.com")) // origin is different and should be added
         )
 
-        let packageSigners = try await storage.get(package: package)
+        let packageSigners = try storage.get(package: package)
         XCTAssertNil(packageSigners.expectedSigner)
         XCTAssertEqual(packageSigners.signers.count, 1)
         XCTAssertEqual(packageSigners.signers[appleseed]?.versions, [Version("1.0.0")])
@@ -185,7 +183,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         let appleseed = SigningEntity.unrecognized(name: "J. Appleseed", organizationalUnit: nil, organization: nil)
         let version = Version("1.0.0")
 
-        await XCTAssertAsyncThrowsError(try await storage.put(
+        await XCTAssertAsyncThrowsError(try storage.put(
             package: package,
             version: version,
             signingEntity: appleseed,
@@ -216,7 +214,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organization: "SwiftPM Test"
         )
         let version = Version("1.0.0")
-        try await storage.put(
+        try storage.put(
             package: package,
             version: version,
             signingEntity: davinci,
@@ -224,14 +222,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         )
 
         // Adding different signing entity for the same version should not fail
-        try await storage.add(
+        try storage.add(
             package: package,
             version: version,
             signingEntity: appleseed,
             origin: .registry(URL("http://bar.com"))
         )
 
-        let packageSigners = try await storage.get(package: package)
+        let packageSigners = try storage.get(package: package)
         XCTAssertNil(packageSigners.expectedSigner)
         XCTAssertEqual(packageSigners.signers.count, 2)
         XCTAssertEqual(packageSigners.signers[appleseed]?.versions, [Version("1.0.0")])
@@ -255,14 +253,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organization: "SwiftPM Test"
         )
         let version = Version("1.0.0")
-        try await storage.put(
+        try storage.put(
             package: package,
             version: version,
             signingEntity: davinci,
             origin: .registry(URL("http://foo.com"))
         )
 
-        await XCTAssertAsyncThrowsError(try await storage.add(
+        await XCTAssertAsyncThrowsError(try storage.add(
             package: package,
             version: version,
             signingEntity: appleseed,
@@ -292,7 +290,7 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organizationalUnit: "SwiftPM Test Unit 2",
             organization: "SwiftPM Test"
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("1.0.0"),
             signingEntity: davinci,
@@ -300,14 +298,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         )
 
         // Sets package's expectedSigner and add package version signer
-        try await storage.changeSigningEntityFromVersion(
+        try storage.changeSigningEntityFromVersion(
             package: package,
             version: Version("1.5.0"),
             signingEntity: appleseed,
             origin: .registry(URL("http://bar.com"))
         )
 
-        let packageSigners = try await storage.get(package: package)
+        let packageSigners = try storage.get(package: package)
         XCTAssertEqual(packageSigners.expectedSigner?.signingEntity, appleseed)
         XCTAssertEqual(packageSigners.expectedSigner?.fromVersion, Version("1.5.0"))
         XCTAssertEqual(packageSigners.signers.count, 2)
@@ -330,14 +328,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organizationalUnit: "SwiftPM Test Unit",
             organization: "SwiftPM Test"
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("1.0.0"),
             signingEntity: davinci,
             origin: .registry(URL("http://foo.com"))
         )
 
-        await XCTAssertAsyncThrowsError(try await storage.changeSigningEntityFromVersion(
+        await XCTAssertAsyncThrowsError(try storage.changeSigningEntityFromVersion(
             package: package,
             version: Version("1.5.0"),
             signingEntity: appleseed,
@@ -367,13 +365,13 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organizationalUnit: "SwiftPM Test Unit 2",
             organization: "SwiftPM Test"
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("1.0.0"),
             signingEntity: davinci,
             origin: .registry(URL("http://foo.com"))
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("2.0.0"),
             signingEntity: appleseed,
@@ -381,14 +379,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
         )
 
         // Sets package's expectedSigner and remove all other signers
-        try await storage.changeSigningEntityForAllVersions(
+        try storage.changeSigningEntityForAllVersions(
             package: package,
             version: Version("1.5.0"),
             signingEntity: appleseed,
             origin: .registry(URL("http://bar.com"))
         )
 
-        let packageSigners = try await storage.get(package: package)
+        let packageSigners = try storage.get(package: package)
         XCTAssertEqual(packageSigners.expectedSigner?.signingEntity, appleseed)
         XCTAssertEqual(packageSigners.expectedSigner?.fromVersion, Version("1.5.0"))
         XCTAssertEqual(packageSigners.signers.count, 1)
@@ -409,14 +407,14 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
             organizationalUnit: "SwiftPM Test Unit",
             organization: "SwiftPM Test"
         )
-        try await storage.put(
+        try storage.put(
             package: package,
             version: Version("1.0.0"),
             signingEntity: davinci,
             origin: .registry(URL("http://foo.com"))
         )
 
-        await XCTAssertAsyncThrowsError(try await storage.changeSigningEntityForAllVersions(
+        await XCTAssertAsyncThrowsError(try storage.changeSigningEntityForAllVersions(
             package: package,
             version: Version("1.5.0"),
             signingEntity: appleseed,
@@ -430,11 +428,10 @@ final class FilePackageSigningEntityStorageTests: XCTestCase {
 }
 
 extension PackageSigningEntityStorage {
-    fileprivate func get(package: PackageIdentity) async throws -> PackageSigners {
-        try await self.get(
+    fileprivate func get(package: PackageIdentity) throws -> PackageSigners {
+        try self.get(
             package: package,
-            observabilityScope: ObservabilitySystem.NOOP,
-            callbackQueue: .sharedConcurrent
+            observabilityScope: ObservabilitySystem.NOOP
         )
     }
 
@@ -443,14 +440,13 @@ extension PackageSigningEntityStorage {
         version: Version,
         signingEntity: SigningEntity,
         origin: SigningEntity.Origin
-    ) async throws {
-        try await self.put(
-                package: package,
-                version: version,
-                signingEntity: signingEntity,
-                origin: origin,
-                observabilityScope: ObservabilitySystem.NOOP,
-                callbackQueue: .sharedConcurrent
+    ) throws {
+        try self.put(
+            package: package,
+            version: version,
+            signingEntity: signingEntity,
+            origin: origin,
+            observabilityScope: ObservabilitySystem.NOOP
         )
     }
 
@@ -459,14 +455,13 @@ extension PackageSigningEntityStorage {
         version: Version,
         signingEntity: SigningEntity,
         origin: SigningEntity.Origin
-    ) async throws {
-        try await self.add(
+    ) throws {
+        try self.add(
             package: package,
             version: version,
             signingEntity: signingEntity,
             origin: origin,
-            observabilityScope: ObservabilitySystem.NOOP,
-            callbackQueue: .sharedConcurrent
+            observabilityScope: ObservabilitySystem.NOOP
         )
     }
 
@@ -475,16 +470,14 @@ extension PackageSigningEntityStorage {
         version: Version,
         signingEntity: SigningEntity,
         origin: SigningEntity.Origin
-    ) async throws {
-        try await self.changeSigningEntityFromVersion(
+    ) throws {
+        try self.changeSigningEntityFromVersion(
             package: package,
             version: version,
             signingEntity: signingEntity,
             origin: origin,
-            observabilityScope: ObservabilitySystem.NOOP,
-            callbackQueue: .sharedConcurrent
+            observabilityScope: ObservabilitySystem.NOOP
         )
-
     }
 
     fileprivate func changeSigningEntityForAllVersions(
@@ -492,14 +485,13 @@ extension PackageSigningEntityStorage {
         version: Version,
         signingEntity: SigningEntity,
         origin: SigningEntity.Origin
-    ) async throws {
-        try await self.changeSigningEntityForAllVersions(
+    ) throws {
+        try self.changeSigningEntityForAllVersions(
             package: package,
             version: version,
             signingEntity: signingEntity,
             origin: origin,
-            observabilityScope: ObservabilitySystem.NOOP,
-            callbackQueue: .sharedConcurrent
+            observabilityScope: ObservabilitySystem.NOOP
         )
     }
 }
