@@ -12,23 +12,27 @@
 
 @_spi(SwiftPMInternal)
 import Basics
+
+@_spi(DontAdoptOutsideOfSwiftPMExposedForBenchmarksAndTestsOnly)
 @testable import PackageGraph
+
 import PackageLoading
 
 @_spi(SwiftPMInternal)
 import PackageModel
 
 @testable import SPMBuildCore
-import SPMTestSupport
+import _InternalTestSupport
 import Workspace
 import XCTest
 
-import class TSCBasic.InMemoryFileSystem
+@testable import class Build.BuildPlan
+import struct Build.PluginConfiguration
 
 import struct TSCUtility.SerializedDiagnostics
 
 final class PluginInvocationTests: XCTestCase {
-    func testBasics() throws {
+    func testBasics() async throws {
         // Construct a canned file system and package graph with a single package and a library that uses a build tool plugin that invokes a tool.
         let fileSystem = InMemoryFileSystem(emptyFiles:
             "/Foo/Plugins/FooPlugin/source.swift",
@@ -83,28 +87,45 @@ final class PluginInvocationTests: XCTestCase {
         XCTAssertNoDiagnostics(observability.diagnostics)
         PackageGraphTester(graph) { graph in
             graph.check(packages: "Foo")
-            // "FooTool{Lib}" duplicated as it's present for both build tools and end products triples.
-            graph.check(modules: "Foo", "FooPlugin", "FooTool", "FooTool", "FooToolLib", "FooToolLib")
+            graph.check(modules: "Foo", "FooPlugin", "FooTool", "FooToolLib")
             graph.checkTarget("Foo") { target in
                 target.check(dependencies: "FooPlugin")
             }
-            graph.checkTarget("FooPlugin", destination: .tools) { target in
+            graph.checkTarget("FooPlugin") { target in
                 target.check(type: .plugin)
                 target.check(dependencies: "FooTool")
             }
-            for destination: BuildTriple in [.tools, .destination] {
-                graph.checkTarget("FooTool", destination: destination) { target in
-                    target.check(type: .executable)
-                    target.check(buildTriple: destination)
-                    target.checkDependency("FooToolLib") { dependency in
-                        dependency.checkTarget {
-                            $0.check(buildTriple: destination)
-                        }
+            graph.checkTarget("FooTool") { target in
+                target.check(type: .executable)
+                target.checkDependency("FooToolLib") { dependency in
+                    dependency.checkTarget { _ in
                     }
                 }
             }
         }
 
+        // "FooTool{Lib}" duplicated as it's present for both build host and end target.
+        do {
+            let buildPlanResult = try await BuildPlanResult(plan: mockBuildPlan(
+                graph: graph,
+                linkingParameters: .init(
+                    shouldLinkStaticSwiftStdlib: true
+                ),
+                fileSystem: fileSystem,
+                observabilityScope: observability.topScope
+            ))
+            buildPlanResult.checkProductsCount(3)
+            buildPlanResult.checkTargetsCount(5) // Note: plugins are not included here.
+
+            buildPlanResult.check(destination: .target, for: "Foo")
+            
+            buildPlanResult.check(destination: .host, for: "FooTool")
+            buildPlanResult.check(destination: .target, for: "FooTool")
+            
+            buildPlanResult.check(destination: .host, for: "FooToolLib")
+            buildPlanResult.check(destination: .target, for: "FooToolLib")
+        }
+
         // A fake PluginScriptRunner that just checks the input conditions and returns canned output.
         struct MockPluginScriptRunner: PluginScriptRunner {
             var hostTriple: Triple {
@@ -172,11 +193,11 @@ final class PluginInvocationTests: XCTestCase {
                                 "configuration": {
                                     "version": 2,
                                     "displayName": "Do something",
-                                    "executable": "/bin/FooTool",
+                                    "executable": "file:///bin/FooTool",
                                     "arguments": [
                                         "-c", "/Foo/Sources/Foo/SomeFile.abc"
                                     ],
-                                    "workingDirectory": "/Foo/Sources/Foo",
+                                    "workingDirectory": "file:///Foo/Sources/Foo",
                                     "environment": {
                                         "X": "Y"
                                     },
@@ -195,6 +216,7 @@ final class PluginInvocationTests: XCTestCase {
                     callbackQueue.sync {
                         completion(.failure(error))
                     }
+                    return
                 }
 
                 // If we get this far we succeeded, so invoke the completion handler.
@@ -211,15 +233,14 @@ final class PluginInvocationTests: XCTestCase {
             destination: .host,
             environment: BuildEnvironment(platform: .macOS, configuration: .debug)
         )
-        let results = try graph.invokeBuildToolPlugins(
-            outputDir: outputDir,
+
+        let results = try await invokeBuildToolPlugins(
+            graph: graph,
             buildParameters: buildParameters,
-            additionalFileRules: [],
-            toolSearchDirectories: [UserToolchain.default.swiftCompilerPath.parentDirectory],
-            pkgConfigDirectories: [],
+            fileSystem: fileSystem,
+            outputDir: outputDir,
             pluginScriptRunner: pluginRunner,
-            observabilityScope: observability.topScope,
-            fileSystem: fileSystem
+            observabilityScope: observability.topScope
         )
         let builtToolsDir = AbsolutePath("/path/to/build/\(buildParameters.triple)/debug")
 
@@ -314,7 +335,10 @@ final class PluginInvocationTests: XCTestCase {
             XCTAssert(rootManifests.count == 1, "\(rootManifests)")
 
             // Load the package graph.
-            let packageGraph = try workspace.loadPackageGraph(rootInput: rootInput, observabilityScope: observability.topScope)
+            let packageGraph = try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )
             XCTAssertNoDiagnostics(observability.diagnostics)
             XCTAssert(packageGraph.packages.count == 1, "\(packageGraph.packages)")
             
@@ -509,8 +533,8 @@ final class PluginInvocationTests: XCTestCase {
                 }
                 """)
 
-            // NTFS does not have nanosecond granularity (nor is this is a guaranteed file 
-            // system feature on all file systems). Add a sleep before the execution to ensure that we have sufficient 
+            // NTFS does not have nanosecond granularity (nor is this is a guaranteed file
+            // system feature on all file systems). Add a sleep before the execution to ensure that we have sufficient
             // precision to read a difference.
             try await Task.sleep(nanoseconds: UInt64(SendableTimeInterval.seconds(1).nanoseconds()!))
 
@@ -691,7 +715,10 @@ final class PluginInvocationTests: XCTestCase {
             XCTAssert(rootManifests.count == 1, "\(rootManifests)")
 
             // Load the package graph.
-            XCTAssertThrowsError(try workspace.loadPackageGraph(rootInput: rootInput, observabilityScope: observability.topScope)) { error in
+            await XCTAssertAsyncThrowsError(try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )) { error in
                 var diagnosed = false
                 if let realError = error as? PackageGraphError,
                    realError.description == "plugin 'MyPlugin' cannot depend on 'FooLib' of type 'library' from package 'foopackage'; this dependency is unsupported" {
@@ -767,7 +794,10 @@ final class PluginInvocationTests: XCTestCase {
             XCTAssert(rootManifests.count == 1, "\(rootManifests)")
 
             // Load the package graph.
-            XCTAssertThrowsError(try workspace.loadPackageGraph(rootInput: rootInput, observabilityScope: observability.topScope)) { error in
+            await XCTAssertAsyncThrowsError(try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )) { error in
                 var diagnosed = false
                 if let realError = error as? PackageGraphError,
                    realError.description == "plugin 'MyPlugin' cannot depend on 'MyLibrary' of type 'library'; this dependency is unsupported" {
@@ -778,6 +808,177 @@ final class PluginInvocationTests: XCTestCase {
         }
     }
 
+    func testPrebuildPluginShouldUseBinaryTarget() async throws {
+        // Only run the test if the environment in which we're running actually supports Swift concurrency (which the plugin APIs require).
+        try XCTSkipIf(!UserToolchain.default.supportsSwiftConcurrency(), "skipping because test environment doesn't support concurrency")
+
+        try await testWithTemporaryDirectory { tmpPath in
+            // Create a sample package with a library target and a plugin.
+            let packageDir = tmpPath.appending(components: "mypkg")
+            try localFileSystem.createDirectory(packageDir, recursive: true)
+            try localFileSystem.writeFileContents(packageDir.appending("Package.swift"), string: """
+                // swift-tools-version:5.7
+
+                import PackageDescription
+
+                let package = Package(
+                    name: "mypkg",
+                    products: [
+                        .library(
+                            name: "MyLib",
+                            targets: ["MyLib"])
+                    ],
+                    targets: [
+                        .target(
+                            name: "MyLib",
+                            plugins: [
+                                .plugin(name: "X")
+                            ]),
+                        .plugin(
+                            name: "X",
+                            capability: .buildTool(),
+                            dependencies: [ "Y" ]
+                        ),
+                        .binaryTarget(
+                            name: "Y",
+                            path: "Binaries/Y.\(artifactBundleExtension)"
+                        ),
+                    ]
+                )
+                """)
+
+            let libTargetDir = packageDir.appending(components: "Sources", "MyLib")
+            try localFileSystem.createDirectory(libTargetDir, recursive: true)
+            try localFileSystem.writeFileContents(libTargetDir.appending("file.swift"), string: """
+                public struct MyUtilLib {
+                    public let strings: [String]
+                    public init(args: [String]) {
+                        self.strings = args
+                    }
+                }
+            """)
+
+            let depTargetDir = packageDir.appending(components: "Sources", "Y")
+            try localFileSystem.createDirectory(depTargetDir, recursive: true)
+            try localFileSystem.writeFileContents(depTargetDir.appending("main.swift"), string: """
+                struct Y {
+                    func run() {
+                        print("You passed us two arguments, argumentOne, and argumentTwo")
+                    }
+                }
+                Y.main()
+            """)
+
+            let pluginTargetDir = packageDir.appending(components: "Plugins", "X")
+            try localFileSystem.createDirectory(pluginTargetDir, recursive: true)
+            try localFileSystem.writeFileContents(pluginTargetDir.appending("plugin.swift"), string: """
+                  import PackagePlugin
+                  @main struct X: BuildToolPlugin {
+                      func createBuildCommands(context: PluginContext, target: Target) async throws -> [Command] {
+                          [
+                              Command.prebuildCommand(
+                                  displayName: "X: Running Y before the build...",
+                                  executable: try context.tool(named: "Y").path,
+                                  arguments: [ "ARGUMENT_ONE", "ARGUMENT_TWO" ],
+                                  outputFilesDirectory: context.pluginWorkDirectory.appending("OUTPUT_FILES_DIRECTORY")
+                              )
+                          ]
+                      }
+                  }
+                  """)
+            
+            let artifactVariants = [try UserToolchain.default.targetTriple].map {
+                """
+                { "path": "Y", "supportedTriples": ["\($0.tripleString)"] }
+                """
+            }
+
+            let bundlePath = packageDir.appending(components: "Binaries", "Y.\(artifactBundleExtension)")
+            let bundleMetadataPath = bundlePath.appending(component: "info.json")
+            try localFileSystem.createDirectory(bundleMetadataPath.parentDirectory, recursive: true)
+            try localFileSystem.writeFileContents(
+                bundleMetadataPath,
+                string: """
+                {   "schemaVersion": "1.0",
+                    "artifacts": {
+                        "Y": {
+                            "type": "executable",
+                            "version": "1.2.3",
+                            "variants": [
+                                \(artifactVariants.joined(separator: ","))
+                            ]
+                        }
+                    }
+                }
+                """
+            )
+            let binaryPath = bundlePath.appending(component: "Y")
+            try localFileSystem.writeFileContents(binaryPath, string: "")
+
+            // Load a workspace from the package.
+            let observability = ObservabilitySystem.makeForTesting()
+            let workspace = try Workspace(
+                fileSystem: localFileSystem,
+                forRootPackage: packageDir,
+                customManifestLoader: ManifestLoader(toolchain: UserToolchain.default),
+                delegate: MockWorkspaceDelegate()
+            )
+
+            // Load the root manifest.
+            let rootInput = PackageGraphRootInput(packages: [packageDir], dependencies: [])
+            let rootManifests = try await workspace.loadRootManifests(
+                packages: rootInput.packages,
+                observabilityScope: observability.topScope
+            )
+            XCTAssert(rootManifests.count == 1, "\(rootManifests)")
+
+            // Load the package graph.
+            let packageGraph = try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )
+            XCTAssertNoDiagnostics(observability.diagnostics)
+            XCTAssert(packageGraph.packages.count == 1, "\(packageGraph.packages)")
+
+            // Find the build tool plugin.
+            let buildToolPlugin = try XCTUnwrap(packageGraph.packages.first?.modules.map(\.underlying).filter{ $0.name == "X" }.first as? PluginModule)
+            XCTAssertEqual(buildToolPlugin.name, "X")
+            XCTAssertEqual(buildToolPlugin.capability, .buildTool)
+
+            // Create a plugin script runner for the duration of the test.
+            let pluginCacheDir = tmpPath.appending("plugin-cache")
+            let pluginScriptRunner = DefaultPluginScriptRunner(
+                fileSystem: localFileSystem,
+                cacheDir: pluginCacheDir,
+                toolchain: try UserToolchain.default
+            )
+
+            // Invoke build tool plugin
+            do {
+                let outputDir = packageDir.appending(".build")
+                let buildParameters = mockBuildParameters(
+                    destination: .host,
+                    environment: BuildEnvironment(platform: .macOS, configuration: .debug)
+                )
+
+                let result = try await invokeBuildToolPlugins(
+                    graph: packageGraph,
+                    buildParameters: buildParameters,
+                    fileSystem: localFileSystem,
+                    outputDir: outputDir,
+                    pluginScriptRunner: pluginScriptRunner,
+                    observabilityScope: observability.topScope
+                )
+
+                let diags = result.flatMap(\.value.results).flatMap(\.diagnostics)
+                testDiagnostics(diags) { result in
+                    result.checkIsEmpty()
+                }
+            }
+        }
+    }
+
+
     func testPrebuildPluginShouldNotUseExecTarget() async throws {
         // Only run the test if the environment in which we're running actually supports Swift concurrency (which the plugin APIs require).
         try XCTSkipIf(!UserToolchain.default.supportsSwiftConcurrency(), "skipping because test environment doesn't support concurrency")
@@ -874,7 +1075,10 @@ final class PluginInvocationTests: XCTestCase {
             XCTAssert(rootManifests.count == 1, "\(rootManifests)")
 
             // Load the package graph.
-            let packageGraph = try workspace.loadPackageGraph(rootInput: rootInput, observabilityScope: observability.topScope)
+            let packageGraph = try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )
             XCTAssertNoDiagnostics(observability.diagnostics)
             XCTAssert(packageGraph.packages.count == 1, "\(packageGraph.packages)")
 
@@ -894,18 +1098,18 @@ final class PluginInvocationTests: XCTestCase {
             // Invoke build tool plugin
             do {
                 let outputDir = packageDir.appending(".build")
-                let result = try packageGraph.invokeBuildToolPlugins(
+                let buildParameters = mockBuildParameters(
+                    destination: .host,
+                    environment: BuildEnvironment(platform: .macOS, configuration: .debug)
+                )
+
+                let result = try await invokeBuildToolPlugins(
+                    graph: packageGraph,
+                    buildParameters: buildParameters,
+                    fileSystem: localFileSystem,
                     outputDir: outputDir,
-                    buildParameters: mockBuildParameters(
-                        destination: .host,
-                        environment: BuildEnvironment(platform: .macOS, configuration: .debug)
-                    ),
-                    additionalFileRules: [],
-                    toolSearchDirectories: [UserToolchain.default.swiftCompilerPath.parentDirectory],
-                    pkgConfigDirectories: [],
                     pluginScriptRunner: pluginScriptRunner,
-                    observabilityScope: observability.topScope,
-                    fileSystem: localFileSystem
+                    observabilityScope: observability.topScope
                 )
 
                 let diags = result.flatMap(\.value.results).flatMap(\.diagnostics)
@@ -1065,7 +1269,10 @@ final class PluginInvocationTests: XCTestCase {
             )
             XCTAssert(rootManifests.count == 1, "\(rootManifests)")
 
-            let graph = try workspace.loadPackageGraph(rootInput: rootInput, observabilityScope: observability.topScope)
+            let graph = try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )
             let dict = try await workspace.loadPluginImports(packageGraph: graph)
 
             var count = 0
@@ -1165,7 +1372,7 @@ final class PluginInvocationTests: XCTestCase {
             try localFileSystem.writeFileContents(myPluginTargetDir.appending("plugin.swift"), string: content)
             let artifactVariants = artifactSupportedTriples.map {
                 """
-                { "path": "LocalBinaryTool\($0.tripleString).sh", "supportedTriples": ["\($0.tripleString)"] }
+                { "path": "\($0.tripleString)/LocalBinaryTool", "supportedTriples": ["\($0.tripleString)"] }
                 """
             }
 
@@ -1209,7 +1416,10 @@ final class PluginInvocationTests: XCTestCase {
             XCTAssert(rootManifests.count == 1, "\(rootManifests)")
 
             // Load the package graph.
-            let packageGraph = try workspace.loadPackageGraph(rootInput: rootInput, observabilityScope: observability.topScope)
+            let packageGraph = try await workspace.loadPackageGraph(
+                rootInput: rootInput,
+                observabilityScope: observability.topScope
+            )
             XCTAssertNoDiagnostics(observability.diagnostics)
 
             // Find the build tool plugin.
@@ -1242,18 +1452,18 @@ final class PluginInvocationTests: XCTestCase {
 
             // Invoke build tool plugin
             let outputDir = packageDir.appending(".build")
-            return try packageGraph.invokeBuildToolPlugins(
+            let buildParameters = mockBuildParameters(
+                destination: .host,
+                environment: BuildEnvironment(platform: .macOS, configuration: .debug)
+            )
+
+            return try await invokeBuildToolPlugins(
+                graph: packageGraph,
+                buildParameters: buildParameters,
+                fileSystem: localFileSystem,
                 outputDir: outputDir,
-                buildParameters: mockBuildParameters(
-                    destination: .host,
-                    environment: BuildEnvironment(platform: .macOS, configuration: .debug)
-                ),
-                additionalFileRules: [],
-                toolSearchDirectories: [UserToolchain.default.swiftCompilerPath.parentDirectory],
-                pkgConfigDirectories: [],
                 pluginScriptRunner: pluginScriptRunner,
-                observabilityScope: observability.topScope,
-                fileSystem: localFileSystem
+                observabilityScope: observability.topScope
             ).mapValues(\.results)
         }
     }
@@ -1298,8 +1508,76 @@ final class PluginInvocationTests: XCTestCase {
             $0.value.forEach {
                 XCTAssertTrue($0.succeeded, "plugin unexpectedly failed")
                 XCTAssertEqual($0.diagnostics.map { $0.message }, [], "plugin produced unexpected diagnostics")
-                XCTAssertEqual($0.buildCommands.first?.configuration.executable.basename, "LocalBinaryTool\(hostTriple.tripleString).sh")
+                XCTAssertEqual($0.buildCommands.first?.configuration.executable.basename, "LocalBinaryTool")
             }
         }
     }
+
+    private func invokeBuildToolPlugins(
+        graph: ModulesGraph,
+        buildParameters: BuildParameters,
+        fileSystem: any FileSystem,
+        outputDir: AbsolutePath,
+        pluginScriptRunner: PluginScriptRunner,
+        observabilityScope: ObservabilityScope
+    ) async throws -> [ResolvedModule.ID: (target: ResolvedModule, results: [BuildToolPluginInvocationResult])] {
+        let pluginsPerModule = graph.pluginsPerModule(
+            satisfying: buildParameters.buildEnvironment
+        )
+
+        let plugins = pluginsPerModule.values.reduce(into: IdentifiableSet<ResolvedModule>()) { result, plugins in
+            plugins.forEach { result.insert($0) }
+        }
+
+        var pluginInvocationResults: [ResolvedModule.ID: (
+            target: ResolvedModule,
+            results: [BuildToolPluginInvocationResult]
+        )] = [:]
+
+        let pluginConfiguration = PluginConfiguration(
+            scriptRunner: pluginScriptRunner,
+            workDirectory: outputDir.parentDirectory,
+            disableSandbox: false
+        )
+
+        for (moduleID, _) in pluginsPerModule {
+            let module = graph.allModules[moduleID]!
+
+            let results = try await BuildPlan.invokeBuildToolPlugins(
+                for: module,
+                destination: .target,
+                configuration: pluginConfiguration,
+                buildParameters: buildParameters,
+                modulesGraph: graph,
+                tools: mockPluginTools(
+                    plugins: plugins,
+                    fileSystem: fileSystem,
+                    buildParameters: buildParameters,
+                    hostTriple: hostTriple
+                ),
+                additionalFileRules: [],
+                pkgConfigDirectories: [],
+                fileSystem: fileSystem,
+                observabilityScope: observabilityScope
+            )
+
+            pluginInvocationResults[moduleID] = (target: module, results: results)
+        }
+
+        return pluginInvocationResults
+    }
+}
+
+extension BuildPlanResult {
+    func check(
+        destination: BuildParameters.Destination,
+        for target: String,
+        file: StaticString = #file,
+        line: UInt = #line
+    ) {
+        let targets = self.targetMap.filter {
+            $0.module.name == target && $0.destination == destination
+        }
+        XCTAssertEqual(targets.count, 1, file: file, line: line)
+    }    
 }
