@@ -11,12 +11,13 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
+import _Concurrency
 import Foundation
 import PackageGraph
 import PackageLoading
 import PackageModel
 import PackageRegistry
-import SPMTestSupport
+import _InternalTestSupport
 @testable import Workspace
 import XCTest
 
@@ -99,7 +100,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v4)
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, ["1.0.1"])
         }
 
@@ -107,7 +108,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v4_2)
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, ["1.0.2", "1.0.1"])
         }
 
@@ -115,7 +116,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_4)
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, ["1.0.3", "1.0.2", "1.0.1"])
         }
     }
@@ -167,8 +168,9 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_2) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            XCTAssertEqual(try container.toolsVersion(for: packageVersion), .v5_3)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let version = try await container.toolsVersion(for: packageVersion)
+            XCTAssertEqual(version, .v5_3)
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, [])
         }
 
@@ -176,8 +178,9 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_3) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            XCTAssertEqual(try container.toolsVersion(for: packageVersion), .v5_3)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let version = try await container.toolsVersion(for: packageVersion)
+            XCTAssertEqual(version, .v5_3)
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, [packageVersion])
         }
 
@@ -185,8 +188,9 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_4) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            XCTAssertEqual(try container.toolsVersion(for: packageVersion), .v5_4)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let version = try await container.toolsVersion(for: packageVersion)
+            XCTAssertEqual(version, .v5_4)
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, [packageVersion])
         }
 
@@ -194,8 +198,9 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_5) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            XCTAssertEqual(try container.toolsVersion(for: packageVersion), .v5_5)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let version = try await container.toolsVersion(for: packageVersion)
+            XCTAssertEqual(version, .v5_5)
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, [packageVersion])
         }
 
@@ -203,8 +208,9 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_6) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref)
-            XCTAssertEqual(try container.toolsVersion(for: packageVersion), .v5_5)
-            let versions = try container.toolsVersionsAppropriateVersionsDescending()
+            let version = try await container.toolsVersion(for: packageVersion)
+            XCTAssertEqual(version, .v5_5)
+            let versions = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(versions, [packageVersion])
         }
     }
@@ -220,7 +226,7 @@ final class RegistryPackageContainerTests: XCTestCase {
         let v5_3_3 = ToolsVersion(string: "5.3.3")!
 
         func createProvider(_ toolsVersion: ToolsVersion) throws -> PackageContainerProvider {
-            let supportedVersions = Set<ToolsVersion>([ToolsVersion.v5_3, v5_3_3, .v5_4, .v5_5])
+            let supportedVersions = Set<ToolsVersion>([ToolsVersion.v5, .v5_3, v5_3_3, .v5_4, .v5_5])
             let registryClient = try makeRegistryClient(
                 packageIdentity: packageIdentity,
                 packageVersion: packageVersion,
@@ -293,7 +299,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_3) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref) as! RegistryPackageContainer
-            let manifest = try container.loadManifest(version: packageVersion)
+            let manifest = try await container.loadManifest(version: packageVersion)
             XCTAssertEqual(manifest.toolsVersion, .v5_3)
         }
 
@@ -301,7 +307,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(v5_3_3) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref) as! RegistryPackageContainer
-            let manifest = try container.loadManifest(version: packageVersion)
+            let manifest = try await container.loadManifest(version: packageVersion)
             XCTAssertEqual(manifest.toolsVersion, v5_3_3)
         }
 
@@ -309,7 +315,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_4) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref) as! RegistryPackageContainer
-            let manifest = try container.loadManifest(version: packageVersion)
+            let manifest = try await container.loadManifest(version: packageVersion)
             XCTAssertEqual(manifest.toolsVersion, .v5_4)
         }
 
@@ -317,7 +323,7 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_5) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref) as! RegistryPackageContainer
-            let manifest = try container.loadManifest(version: packageVersion)
+            let manifest = try await container.loadManifest(version: packageVersion)
             XCTAssertEqual(manifest.toolsVersion, .v5_5)
         }
 
@@ -325,9 +331,17 @@ final class RegistryPackageContainerTests: XCTestCase {
             let provider = try createProvider(.v5_6) // the version of the alternate
             let ref = PackageReference.registry(identity: packageIdentity)
             let container = try await provider.getContainer(for: ref) as! RegistryPackageContainer
-            let manifest = try container.loadManifest(version: packageVersion)
+            let manifest = try await container.loadManifest(version: packageVersion)
             XCTAssertEqual(manifest.toolsVersion, .v5_5)
         }
+        
+        do {
+            let provider = try createProvider(.v5) // the version of the alternate
+            let ref = PackageReference.registry(identity: packageIdentity)
+            let container = try await provider.getContainer(for: ref) as! RegistryPackageContainer
+            let manifest = try await container.loadManifest(version: packageVersion)
+            XCTAssertEqual(manifest.toolsVersion, .v5)
+        }
     }
 
     func makeRegistryClient(
@@ -486,7 +500,13 @@ final class RegistryPackageContainerTests: XCTestCase {
         guard let registryIdentity = identity.registry else {
             preconditionFailure("invalid registry identity: '\(identity)'")
         }
-        let versionString = version.patch == 0 ? "\(version.major).\(version.minor)" : version.description
+        let versionString = if version.patch == 0 && version.minor == 0 {
+            "\(version.major)"
+        } else if version.patch == 0 {
+            "\(version.major).\(version.minor)"
+        } else {
+            version.description
+        }
         return "<http://localhost/\(registryIdentity.scope)/\(registryIdentity.name)/\(version)/\(Manifest.filename)?swift-version=\(version)>; rel=\"alternate\"; filename=\"\(Manifest.basename)@swift-\(versionString).swift\"; swift-tools-version=\"\(version)\""
     }
 }
