@@ -11,12 +11,13 @@
 //===----------------------------------------------------------------------===//
 
 import Basics
+import _Concurrency
 import Foundation
 import PackageGraph
 import PackageLoading
 import PackageModel
 import SourceControl
-import SPMTestSupport
+import _InternalTestSupport
 @testable import Workspace
 import XCTest
 
@@ -164,7 +165,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
 
         let ref = PackageReference.localSourceControl(identity: PackageIdentity(path: repoPath), path: repoPath)
         let container = try await provider.getContainer(for: ref)
-        let v = try container.toolsVersionsAppropriateVersionsDescending()
+        let v = try await container.toolsVersionsAppropriateVersionsDescending()
         XCTAssertEqual(v, ["2.0.3", "1.0.3", "1.0.2", "1.0.1", "1.0.0"])
     }
 
@@ -224,7 +225,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
             let provider = try createProvider(ToolsVersion(version: "4.0.0"))
             let ref = PackageReference.localSourceControl(identity: PackageIdentity(path: repoPath), path: repoPath)
             let container = try await provider.getContainer(for: ref)
-            let v = try container.toolsVersionsAppropriateVersionsDescending()
+            let v = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(v, ["1.0.1"])
         }
 
@@ -233,7 +234,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
             let ref = PackageReference.localSourceControl(identity: PackageIdentity(path: repoPath), path: repoPath)
             let container = try await provider.getContainer(for: ref) as! SourceControlPackageContainer
             XCTAssertTrue(container.validToolsVersionsCache.isEmpty)
-            let v = try container.toolsVersionsAppropriateVersionsDescending()
+            let v = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(container.validToolsVersionsCache["1.0.0"], false)
             XCTAssertEqual(container.validToolsVersionsCache["1.0.1"], true)
             XCTAssertEqual(container.validToolsVersionsCache["1.0.2"], true)
@@ -245,7 +246,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
             let provider = try createProvider(ToolsVersion(version: "3.0.0"))
             let ref = PackageReference.localSourceControl(identity: PackageIdentity(path: repoPath), path: repoPath)
             let container = try await provider.getContainer(for: ref)
-            let v = try container.toolsVersionsAppropriateVersionsDescending()
+            let v = try await container.toolsVersionsAppropriateVersionsDescending()
             XCTAssertEqual(v, [])
         }
 
@@ -256,7 +257,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
             let container = try await provider.getContainer(for: ref) as! SourceControlPackageContainer
             let revision = try container.getRevision(forTag: "1.0.0")
             do {
-                _ = try container.getDependencies(at: revision.identifier, productFilter: .nothing)
+                _ = try await container.getDependencies(at: revision.identifier, productFilter: .nothing)
             } catch let error as SourceControlPackageContainer.GetDependenciesError {
                 let error = error.underlyingError as! UnsupportedToolsVersion
                 XCTAssertMatch(error.description, .and(.prefix("package '\(PackageIdentity(path: repoPath))' @"), .suffix("is using Swift tools version 3.1.0 which is no longer supported; consider using '// swift-tools-version:4.0' to specify the current tools version")))
@@ -307,7 +308,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
 
         let ref = PackageReference.localSourceControl(identity: PackageIdentity(path: repoPath), path: repoPath)
         let container = try await provider.getContainer(for: ref)
-        let v = try container.toolsVersionsAppropriateVersionsDescending()
+        let v = try await container.toolsVersionsAppropriateVersionsDescending()
         XCTAssertEqual(v, ["1.0.4-alpha", "1.0.2-dev.2", "1.0.2-dev", "1.0.1", "1.0.0", "1.0.0-beta.1", "1.0.0-alpha.1"])
     }
 
@@ -358,7 +359,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
         )
         let ref = PackageReference.localSourceControl(identity: PackageIdentity(path: repoPath), path: repoPath)
         let container = try await provider.getContainer(for: ref)
-        let v = try container.toolsVersionsAppropriateVersionsDescending()
+        let v = try await container.toolsVersionsAppropriateVersionsDescending()
         XCTAssertEqual(v, ["2.0.1", "1.3.0", "1.2.0", "1.1.0", "1.0.4", "1.0.2", "1.0.1", "1.0.0"])
     }
 
@@ -547,7 +548,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
             let container = try await containerProvider.getContainer(for: packageRef) as! SourceControlPackageContainer
 
             // Simulate accessing a fictitious dependency on the `master` branch, and check that we get back the expected error.
-            do { _ = try container.getDependencies(at: "master", productFilter: .everything) }
+            do { _ = try await container.getDependencies(at: "master", productFilter: .everything) }
             catch let error as SourceControlPackageContainer.GetDependenciesError {
                 // We expect to get an error message that mentions main.
                 XCTAssertMatch(error.description, .and(.prefix("could not find a branch named ‘master’"), .suffix("(did you mean ‘main’?)")))
@@ -556,7 +557,7 @@ final class SourceControlPackageContainerTests: XCTestCase {
             }
 
             // Simulate accessing a fictitious dependency on some random commit that doesn't exist, and check that we get back the expected error.
-            do { _ = try container.getDependencies(at: "535f4cb5b4a0872fa691473e82d7b27b9894df00", productFilter: .everything) }
+            do { _ = try await container.getDependencies(at: "535f4cb5b4a0872fa691473e82d7b27b9894df00", productFilter: .everything) }
             catch let error as SourceControlPackageContainer.GetDependenciesError {
                 // We expect to get an error message about the specific commit.
                 XCTAssertMatch(error.description, .prefix("could not find the commit 535f4cb5b4a0872fa691473e82d7b27b9894df00"))
@@ -712,8 +713,8 @@ final class SourceControlPackageContainerTests: XCTestCase {
             let packageReference = PackageReference.localSourceControl(identity: PackageIdentity(path: packageDirectory), path: packageDirectory)
             let container = try await containerProvider.getContainer(for: packageReference)
 
-            let forNothing = try container.getDependencies(at: version, productFilter: .specific([]))
-            let forProduct = try container.getDependencies(at: version, productFilter: .specific(["Product"]))
+            let forNothing = try await container.getDependencies(at: version, productFilter: .specific([]))
+            let forProduct = try await container.getDependencies(at: version, productFilter: .specific(["Product"]))
             #if ENABLE_TARGET_BASED_DEPENDENCY_RESOLUTION
             // If the cache overlaps (incorrectly), these will be the same.
             XCTAssertNotEqual(forNothing, forProduct)
@@ -727,13 +728,15 @@ extension PackageContainerProvider {
         for package: PackageReference,
         updateStrategy: ContainerUpdateStrategy = .always
     ) async throws -> PackageContainer {
-        try await safe_async {
+        try await withCheckedThrowingContinuation { continuation in
             self.getContainer(
                 for: package,
                 updateStrategy: updateStrategy,
                 observabilityScope: ObservabilitySystem.NOOP,
                 on: .global(),
-                completion: $0
+                completion: {
+                  continuation.resume(with: $0)
+                }
             )
         }
     }
