@@ -2,7 +2,7 @@
 //
 // This source file is part of the Swift open source project
 //
-// Copyright (c) 2014-2020 Apple Inc. and the Swift project authors
+// Copyright (c) 2014-2024 Apple Inc. and the Swift project authors
 // Licensed under Apache License v2.0 with Runtime Library Exception
 //
 // See http://swift.org/LICENSE.txt for license information
@@ -19,10 +19,8 @@ import PackageModel
 @testable import Workspace
 import XCTest
 
-import class TSCBasic.InMemoryFileSystem
-
 /// Test cases for `rewriteToolsVersionSpecification(toDefaultManifestIn:specifying:fileSystem:)`
-class ToolsVersionSpecificationRewriterTests: XCTestCase {
+final class ToolsVersionSpecificationRewriterTests: XCTestCase {
     
     /// Tests `rewriteToolsVersionSpecification(toDefaultManifestIn:specifying:fileSystem:)`.
     func testNonVersionSpecificManifests() throws {
